#!/stand/sysco/tools/bin/perl
#
use lib "/stand/sysco/lib/sscope";

################################################################################
#
#  Modules required for SuperScope
#
################################################################################
use common_defs;
use jtag_subs;
use scancfg;
use shub_01_mmrs;
use reg_subs;
use mmr_subs;
use shub_subs;
use pktgen;
use error_regs;
use fmt_subs;
use ecc;
use mbist_subs;
use md_dir_state;

################################################################################
#
# Initialize variable values
#
################################################################################
$node            = 0;   # default node is NODE 0
$target_module   = "001c01";
$verbose         = 0;
$fail_check      = 0;
$starting_addr   = 0x0;
$max_cachelines  = 4; 
$dp              = 0; # Is data poisoned?
$data_poison_xor_value = 0x003c3fff003c3fff;
$ecc_poison_xor_value  = 0x03;
$l2_copy_num = $SYSCO_COPY_INVALID;

#
# Default JTAG configuration file name
#
$scancfg_file = "";
$scancfg_part_str = "";


################################################################################
#
#  Process the command-line parameters (simple for now)
#
################################################################################
mbist_parse_parameters();

################################################################################
#
#  Initialize the JTAG connection, load scan configuration and reset SICs/TAPs
#
################################################################################

jtag_lib_msg_level(0);      # JTAG library debug message control

#
#  Make the connection to the JTAG controller (system controllers)
#
if (( $l2_host eq "" ) || ($sim_mode == 1))       # No L2 hardware: emulation mode
{  jtag_connect($JTAG_ACCESS_NONE, "localhost", $l2_port);
   mmr_access($MMR_ACCESS_NONE);
   print "\n\n***** NO HARDWARE CONNECTION - SOFTWARE EMULATION *****\n";
   print "\nUse the \"--l2 <L2 address>\" command-line parameter to specify a real L2.\n\n";
}
else
{  jtag_connect($JTAG_ACCESS_SYSCO, $l2_host, $DEFAULT_JTAG_PORT, $l2_copy_num);
   mmr_access($MMR_ACCESS_JTAG, $TRUE);
}
jtag_set_data_just($JTAG_JUST_RIGHT);

#
#  Read the scan configuration from file or hardware (C,R,M,Px, and Ix bricks only)
#

 # print "\n\nEarly abort for testing.\n\n";
 # exit;

# scancfg_load($scancfg_file, $TRUE, "ciprsmt", $scancfg_part_str);     #  use this when providing brick types
scancfg_load($scancfg_file, $TRUE, $cfg_brick_type, $scancfg_part_str);                 #  use this with very latest release

#
#  Initialize the JTAG TAP(s) and SIC(s)
#
scancfg_hardware_init();


#
#  Build list of target chips: all from config or user specified target
#

$mref = $modules{ $target_module };
@chip_names = scan_module_chips($mref, 0xFFFF, 'logical');

# print "@chip_names" . "\n";

$current_node=0;
foreach $curr_name ( @chip_names ) {
   @target = split(":", $curr_name);

#  printf "target[0]=$target[0]  target[1]=$target[1]  target[2]=$target[2]\n";

   if ( $target[2] =~ /^SHUB/i ) {
#     printf("Node:%d, Current_Node:%d\n", $node, $current_node);
      if ( $node == $current_node ) {
         $target_board=$target[1];
         $target_chip= $target[2];
      }
      $current_node++;
   }
}

mmr_set_timeout(10);

################################################################################
#
# Start of BIST testing
#
################################################################################

print"\nRunning script on NODE $node\n\n";

mmr_target($SELECT, $target_module, $target_board, $target_chip);

############ Read Memory ########################################################

@main_array = read_mem($starting_addr, $max_cachelines); 

############ Print out the Array ################################################

$array_length = 0;
$array_length = @main_array;
$iter = 0;

printf"\n";
get_dir_state($main_array[$iter][0], $verbose, 0, 0);
printf"\n";
print"Memory Addr  D-Word Data         ECC\n";
print"***************************************************\n";

for($iter = 0; $iter < $array_length; $iter++) {
  if(($iter >= 16) && (($iter % 16) == 0)) {
    printf"\n";
    get_dir_state($main_array[$iter][0], $verbose, 0, 0);
    printf"\n";
  }
  if(($iter >= 32) && (($iter % 32) == 0)) {
    printf"\n";
    print"Memory Addr  D-Word Data         ECC\n";
    print"***************************************************\n";
  }

  printf"0x%09llX, 0x%016llX, 0x%02lX", $main_array[$iter][0], $main_array[$iter][1], $main_array[$iter][2];


  if ( ($ExpECC = Calculate_ECC($main_array[$iter][1])) != $main_array[$iter][2]) {
     printf(" <- Bad ECC (ExpECC:0x%02lX), SYN:0x%02lX", $ExpECC, ($ExpECC ^ $main_array[$iter][2]) );
     if ( $dp == 1 ) {
        printf"  (0x%016llX, 0x%02lX)", $main_array[$iter][1] ^ $data_poison_xor_value, $main_array[$iter][2] ^ $ecc_poison_xor_value;
     }
  }

  if ( $dp >= 2 ) {
     printf"  (0x%016llX, 0x%02lX)", $main_array[$iter][1] ^ $data_poison_xor_value, $main_array[$iter][2] ^ $ecc_poison_xor_value;
  }

  printf"\n";

}

if(check_summary_err_mmr()) {
    clear_summary_err_mmr();
    $fail_check++;
}

if($fail_check) {
    $emess = "\n\n ERROR:  BIST FAILURE!!!\n\n";
    dsp_error_and_exit($emess);
}

################################################################################
#
# End of BIST testing.
#
################################################################################

################################################################################
#
#  Close the JTAG connection
#
################################################################################
if ( defined($jtag_init) ) { # JTAG/scan module included?
   jtag_disconnect();
}  

exit 1;

################################################################################
#  USAGE: superscope [-h] [-f <file>] [--l2 <L2 name>] [-n <node #>] [-v]
#                    [-c <lines>] [-a <address>]
#
#     -h Prints out the command line options
#     -f <file> Define
#     --l2 <L2 name/address>
#     -v Verbose output
#     -n <node> Selects which node to run on
#     -c <cachelines> Selects the number of cachelines to read, must be an even number
#     -a <address> Selects the starting address of the memory dump
#     -module Selects the target brick
################################################################################

sub mbist_usage
{
   print "\n";
   print "USAGE: memdmp.pl  [-h] [--l2 <L2 name>] [--copy <copy#>] [-module <module>]\n";
   print "                  [-n <node#>] [-v] [-a <address>] [-c <lines>] [-dp <level#>]\n";
   print "-h                Display superscope usage and exit.\n";
   print "--l2 <host>       Name or IP address of L2 system controller\n";
   print "--copy <copy#>    To be used for multiple L2's on the same IP address.\n";
   print "-v                Selects verbose output.\n";
   print "-n <node#>        Selects which node to run on\n";
   print "-c <clines>       Selects the number of cachelines to read.\n";
   print "-a <hex_address>  Selects the starting address of the mem dump.\n";
   print "-module <module>  Selects the target module.\n";
   print "-dp <level#>      Calculate and display poisoned data/ecc.\n";
   print "                  The Intel CPU may have written poisoned data to memory.\n";
   print "                  Original data can be recovered by XOR'ing a fixed data\n"; 
   print "                  pattern to the memory data and ecc.\n"; 
   print "                     level 0 - Don't display (default)\n";
   print "                     level 1 - Display only if ecc is bad.\n";
   print "                     level 2 - Always Display.\n";
   print "\n";
}

sub mbist_parse_parameters
{
   $argc = $#ARGV + 1;
   $i = 0;
   while ( $i < $argc )
   {
      if ( $ARGV[$i] eq '-h' )
      {  mbist_usage();
         exit 1;
      }
      elsif ( $ARGV[$i] eq '-v' )
      {
         $verbose = 1;
      }
      elsif ( ($i + 1) == $argc )
      {  print "Missing parameter with command line option.\n";
         print "Option: $ARGV[$i]\n";
         mbist_usage();
         exit 1;
      }
      else 
      {
         if ( $ARGV[$i] eq '-f' )
         {  $i++;
            $scancfg_file = $ARGV[$i];
         }
         elsif ( $ARGV[$i] eq '--l2' )
         {  $i++;
            $l2_host = $ARGV[$i];
         }
         elsif ( $ARGV[$i] eq '-n' )
         {  $i++;
            $node = $ARGV[$i];
         }
         elsif ( $ARGV[$i] eq '-c' )
         {  $i++;
            if($ARGV[$i] <= 1) {
              print"Command Line Failure: ";
              print"Number of Cachelines must be greater then 1!\n";
              exit 1;
            }
            $max_cachelines = $ARGV[$i];
         }
         elsif ( $ARGV[$i] eq '-a' )
         {  $i++;
	       if($ARGV[$i] !~ /^0x/) {
              print"Command Line Failure: ";
	      print"Address must be preceeded with a \'0x\'!\n";
              exit 1;
	    }
            $starting_addr = oct($ARGV[$i]);
         }
         elsif ( $ARGV[$i] eq '-module' )
         {
            $i++;
            $target_module = $ARGV[$i];
         }
         elsif ( $ARGV[$i] eq '--copy' )
         {  $i++;
            $l2_copy_num = $ARGV[$i];
            $display_string .=  "\n L2 copy value of $l2_copy_num selected via command line arg.";
         }
         elsif ( $ARGV[$i] eq '-dp' )
         {  $i++;
            $dp = $ARGV[$i];
         }
         else
         {  print "Unknown command line option specified.\n";
            print "Option: $ARGV[$i]\n";
            mbist_usage();
            exit 1;
         }
      }

      $i++;
   }
}

sub Calculate_ECC
{
   my $ecc;
   my $data = $_[0];

   $ecc = 0;

   for ($bit=0; $bit<64; $bit++)
   {
      if ( (0x1<<$bit) & $data )
      {
         $ecc ^= $ECC[$bit];
      }
   }

   return($ecc);
}

