#!/stand/sysco/tools/bin/perl
#
#  Basic superscope script to allow user to read cacheline directory state.
#  use "-help" to see command line options.

################################################################################
#
#  Modules required for SuperScope
#
################################################################################

use lib "/stand/sysco/lib/sscope";

# use lib "/data/cf/sn2-diag-sw/sn2/meyer/sys_ctrl/sscope/perl/sscope";
# use lib "/data/cf/sn2-diag-sw/sn2/meyer/sys_ctrl/sscope/perl/lib";
# use lib "/data/cf/sn2-diag-sw/sn2/meyer/sys_ctrl/sscope/perl/lib.linux";
# use lib "/data/cf/sn2-diag-sw/sn2/meyer/sys_ctrl/sscope/perl/errdmp";
# use lib "/data/cf/sn2-diag-sw/sn2/meyer/sys_ctrl/sscope/perl/elogd";
# print "\n\n********** USING DEVELOPMENT LIBRARIES **********\n";


use common_defs;
use jtag_subs;
use scancfg;
use shub_01_mmrs;
use reg_subs;
use mmr_subs;
use fmt_subs;
use shub_subs;
use error_regs;
use md_dir_state;



################################################################################
#
#  Global variables for this script
#
################################################################################

$debug_level = 0;     # Debug message control

$num_base = 'hex';    # Default numeric base for access windows

$node_select = 0;
$user_target = 0;
$target_slot = "01";
$target_rack = "001";
$node_count = 0;

$loop_enable = 0;		# set to 1 to loop forever
$user_address = 0;		# set when a user has supplied an address value to loop on
$user_length = 1;		# set when a user has block length - default = 1

$verbose_mode = 0;

$l2_host = "";
$l2_port = 8000;
$l2_copy_num = $SYSCO_COPY_INVALID;


$num_base = 'hex';    # Default numeric base for access windows

#Default JTAG configuration file name
#$scancfg_file = "/stand/sysco/cfg/scan/ip41_001S11.cfg";
$scancfg_file = "";


################################################################################
#
#  Process the command-line parameters (support "-f" and "--l2")
#
################################################################################

example_parse_parameters();  # Simple parameter list


################################################################################
#
#  Read the JTAG configuration file and open system controller connection
#
################################################################################

jtag_lib_msg_level(0);      # JTAG library debug message control

#
#  Make the connection to the JTAG controller (system controllers)
#
if ( $l2_host eq "" )       # No L2 hardware: emulation mode
{  jtag_connect($JTAG_ACCESS_NONE, "localhost", $l2_port);
   mmr_access($MMR_ACCESS_NONE);
   print "\n\n***** NO HARDWARE CONNECTION - SOFTWARE EMULATION *****\n\n";
   print "\nUse the \"--l2 <L2 address>\" command-line parameter to specify a real L2.\n\n";
   exit(1);
}
else
{  jtag_connect($JTAG_ACCESS_SYSCO, $l2_host, $l2_port, $l2_copy_num);
   mmr_access($MMR_ACCESS_JTAG);
}
mmr_access($MMR_ACCESS_JTAG);  # Force to JTAG access?

#
#  Initialize the JTAG TAP(s) and SIC(s)
#
jtag_set_data_just($JTAG_JUST_RIGHT);

scancfg_load($scancfg_file, $TRUE);

if ( $verbose_mode == 1 )  {
    print "\n-------------------------------------------------------------------\n";
    print "\nCURRENT SCAN CHAIN CONFIGURATION:\n";
    scancfg_dump($FALSE);
    print "\n-------------------------------------------------------------------\n";
}

scancfg_hardware_init();

parse_scan_config();

################################################################################
#
#  START MMR ACCESS (REAL WORK OF SCRIPT GOES HERE)
#
################################################################################

#
#  Target ASIC for MMR operations: all MMR operations will be directed
#     to the ASIC (until another "mmr_target" is executed)
#

$target_module= "$target_rack" . "c" . "$target_slot, NODE$target_plane";   # Target module definition

if ($node_count > 0)  {
  mmr_target($SELECT, $target_list[0]->{MODULE}, $target_list[0]->{NODE}, $target_list[0]->{CHIP});
}
else {
  print "\n\n  ERROR!  Unable to locate specified brick $target_module in scan config.\n";
  print "  Aborting script.\n\n";
  exit(1);
}


$nasid_value = get_node_id();

if ($user_address == 0)  {
  @address_list = (  0xc0000 );
}
else  {
  for ($len = 0; $len < $user_length; $len++)  {
      $address_list[$len] = $user_entered_addr + ($len * 0x80);
  }
}
		   

print "\n-------------------------------------------------------\n";

foreach $addr_main (@address_list)  {
    if ($verbose_mode == 1)  {
        printf "\n Getting directory state for address 0x%09llX\n\n", $user_entered_addr;
    }
    get_dir_state($addr_main, $verbose_mode, 0, 0);
}

print "\n-------------------------------------------------------\n";



###############################################################################e
#
#  Close JTAG connection and cleanup
#
################################################################################

if ( defined($jtag_init) )   # JTAG/scan module included?
{  jtag_disconnect();
}  

print "\n\n the read_dir_state.pl script is complete.\n\n";

exit 0;


################################################################################
################################################################################


#-------------------------------------------------------------------------------
#
#  NOTES: simple command-line processing for now!
#
#  USAGE: superscope [-f <file>] [--l2 <L2 name>] [-h]
#     -f <file> = define
#     --l2 <L2 name/address>
#-------------------------------------------------------------------------------

sub example_usage
{
   print "\nUSAGE: read_dir_state.pl [-f <config file>] [--l2 <L2 name>] [-h]\n";
   print "    -addr <address>     Address to be used for directory state reads.\n";
   print "    -length <value>     Block length to be used for number of reads.\n";
   print "    -h(elp)             Display usage and exit.\n";
   print "    -n1      		  Select node 1, Shub1.\n";
   print "    -n2      		  Select node 2, Shub0.  (Vortix only)\n";
   print "    -n3      		  Select node 3, Shub0.  (Vortix only)\n";
   print "    -f <file>           Name of JTAG configuration file defining hardware\n";
   print "    --l2 <host>         Name or IP address of L2 system controller\n";
   print "    -rack <rack#>       Select brick rack # (default is 01)\n";
   print "    -slot <slot#>       Select brick slot # (default is 001)\n";
   print "    --copy <copy#>      To be used for multiple L2's on the same IP address.\n";
   print "    -verbose            Show debug information\n";
   print "\n\n";
}


#-------------------------------------------------------------------------------

sub example_parse_parameters
{

   chomp ($start_time = `date +'%T'`);
   chomp ($start_date = `date +'%A %B %e'`);
   chomp ($exc_host = `hostname`);

   $display_string = "\nStarting script execution at $start_time on $start_date.";
   $display_string .= "\nExecution host:   $exc_host.\n\n";
   $display_string .= "\n\n Command line parameters: \n";

   $argc = $#ARGV + 1;
   $i = 0;
   while ( $i < $argc )
   {
      if ( $ARGV[$i] eq '-h' )
      {  example_usage();
         exit 0;
      }

      if ( $ARGV[$i] eq '-help' )
      {  example_usage();
         exit 0;
      }

      if ( (($i + 1) == $argc ) && ($i == 0))
      {  print "Missing parameter with command line option.\n";
         print "Option: $ARGV[$i]\n";
         example_usage();
         exit 1;
      }

      if ( $ARGV[$i] eq '-f' )
      {  $i++;
         $scancfg_file = $ARGV[$i];
      }

      elsif ( $ARGV[$i] eq '--l2' )
      {  $i++;
         $l2_host = $ARGV[$i];
         $display_string .=  "\n User entered l2 hostname: $l2_host.";
      }

      elsif ( $ARGV[$i] eq '-loop' )
      {  $loop_enable = 1;
         $display_string .=  "\n Looping selected via command line arg.";
      }

      elsif ( $ARGV[$i] eq '-addr' )
      {
	$user_address = 1;
        $i++;
        $tmp_addr = "$ARGV[$i]";
        $user_entered_addr = hex ($tmp_addr);
        $display_string .=  sprintf ("\n  Directory state read address value: 0x%014llX entered via command line arg.", $user_entered_addr);
      }

      elsif ( $ARGV[$i] eq '-length' )
      {
        $i++;
        $tmp_length = "$ARGV[$i]";
        $user_length = hex ($tmp_length);
        $display_string .=  sprintf ("\n Block length value of 0x%X entered via command line arg.", $user_length);
      }

      elsif ( $ARGV[$i] eq '-n1' )
      {  $node_select = 1;
         $display_string .=  "\n Node 1 selected via command line arg.";
      }

      elsif ( $ARGV[$i] eq '-n2' )
      {  $node_select = 2;
         $display_string .=  "\n Node 2 selected via command line arg.";
      }

      elsif ( $ARGV[$i] eq '-n3' )
      {  $node_select = 3;
         $display_string .=  "\n Node 3 selected via command line arg.";
      }


      elsif ( $ARGV[$i] eq '-slot' )
      {
        $i++;
        $user_slot = $ARGV[$i];
        if ($user_slot !~ /\d+/) {
            print "\nInput Error.  Must enter a numeric slot value.\n";
            example_usage();
            exit 1;
        }
        $target_slot = sprintf ("%02d", $user_slot);
        $user_target = 1;
        $display_string .=  "\n Slot $target_slot selected via command line arg.";
      }

      elsif ( $ARGV[$i] eq '-rack' )
      {
        $i++;
        $user_rack = $ARGV[$i];
        if ($user_rack !~ /\d+/) {
            print "\nInput Error.  Must enter a numeric rack value.\n";
            example_usage();
            exit 1;
        }
        $target_rack = sprintf ("%03d", $user_rack);
        $display_string .=  "\n Rack $target_rack selected via command line arg.";
        $user_target = 1;
      }

      elsif ( $ARGV[$i] eq '-verbose' )
      {
	$verbose_mode = 1;
      }
      elsif ( $ARGV[$i] eq '--copy' )
      {
        $i++;
        $tmp_copy_num = $ARGV[$i];
        if (($tmp_copy_num !~ /\d+/) || ( $tmp_copy_num >= $SYSCO_COPY_INVALID )) {
            print "\nInput Error.  Must enter a numeric copy value less than $SYSCO_COPY_INVALID.";
            print "\n  Copy value entered:  $tmp_copy_num\n\n";
            example_usage();
            exit 1;
        }
        $l2_copy_num = $tmp_copy_num;
        $display_string .=  "\n L2 copy value of $l2_copy_num selected via command line arg.";
      }

      else
      {  print "Unknown command line option specified.\n";
         print "Option: $ARGV[$i]\n";
         example_usage();
         exit 1;
      }

      $i++;
   }

  print "$display_string\n\n";

}

sub get_node_id
{
  my $reg_ptr;
  my $node_value = 0;

  $reg_ptr = reg_create_mmr_inst("shub_01", 'SH_SHUB_ID');
  reg_mmr_read($reg_ptr);

  $node_value = reg_get_field_value($reg_ptr, 'NODE_ID');

  printf "\nThe NASID value of the current shub is 0x%03X\n", $node_value;
  
  return $node_value;
}

sub parse_scan_config {

  print "\nIn parse_scan_config function\n" if ($verbose_mode == 1);
  my $c_brick_count = 0;
  my $opus_brick_count = 0;
  my $vortix_brick_count = 0;
  my $mc_brick_count = 0;
  my $r_brick_count = 0;
  my $io_brick_count = 0;
  my $tio_brick_count = 0;
  my $brick_count = 0;
  my $pic_count = 0;
  my $tio_count = 0;

  if ($node_select == 0)  	{ $target_plane = 0; }
  elsif ($node_select == 1)     { $target_plane = 1; }
  elsif ($node_select == 2)     { $target_plane = 2; }
  elsif ($node_select == 3)     { $target_plane = 3; }

  foreach $new_brick (@module_names)  {         #  This array is defined in superscope libraries

    $mref = $modules{ $new_brick };
    @buffer_for_superscope = ();
    # @chip_names = scan_module_chips($mref, 0xFFFF, 'logical', \@buffer_for_superscope);
    @chip_names = scan_module_chips($mref, 0xFFFF, 'logical');
    if ($verbose_mode == 1)  {
      printf "\n->Module name: $new_brick\n";
      printf "      chip list: ( @chip_names )\n";
    }

    if ($new_brick =~ /c/)  {
      @targets = split (/c/, $new_brick);
      if (($user_target == 0)  || (($user_target == 1) && ($targets[0] eq $target_rack) && ($targets[1] eq $target_slot)))  {
        $l1_c_brick_list[$c_brick_count]->{RACK} = $targets[0];
        $l1_c_brick_list[$c_brick_count]->{SLOT} = $targets[1];
        $c_brick_count++;
        $l1_mc_brick_list[$mc_brick_count]->{RACK} = $targets[0];
        $l1_mc_brick_list[$mc_brick_count]->{SLOT} = $targets[1];
        $mc_brick_count++;
      }
      else  { 
        printf "  Skipping C Brick - Does not match user designated Rack and Slot values.\n"  if ($verbose_mode == 1);
        next; 
      }
    }
    elsif ($new_brick =~ /m/)  {
      @targets = split (/m/, $new_brick);
      if (($user_target == 0)  || (($user_target == 1) && ($targets[0] eq $target_rack) && ($targets[1] eq $target_slot)))  {
        $l1_mc_brick_list[$mc_brick_count]->{RACK} = $targets[0];
        $l1_mc_brick_list[$mc_brick_count]->{SLOT} = $targets[1];
        $mc_brick_count++;
      }
      else  {
        printf "  Skipping M Brick - Does not match user designated Rack and Slot values.\n"  if ($verbose_mode == 1);
        next;
      }
    }
    else {
        next;
    }

    foreach $curr_name ( @chip_names )
    {
      printf "\n->Chip: $curr_name\n" if ($verbose_mode == 1);
      @target = split(":", $curr_name);

      printf "target[0]=$target[0]  target[1]=$target[1]  target[2]=$target[2]\n" if ($verbose_mode == 1);
      if ( $target[2] =~ /^SHUB0/i )
      {
        if ( (($target_plane == 0) && ($target[1] eq 'NODE') )
          || (($target_plane == 0) && ($target[1] eq 'NODE0'))
          || (($target_plane == 2) && ($target[1] eq 'NODE2'))
          || (($target_plane == 3) && ($target[1] eq 'NODE3')) )  {
          $target_list[$node_count]->{MODULE} = "$target[0]";
          $target_list[$node_count]->{NODE} = "$target[1]";
          $target_list[$node_count]->{CHIP} = "$target[2]";
          printf("ADD SHUB0: brick=$target[0] board=$target[1] chip=$target[2]\n")  if ($verbose_mode == 1);
          $node_count++;
          $shub_count++;
        }
      }
      elsif ( $target[2] =~ /^SHUB1/i )
      {
        if ($target_plane == 1)  {
          $target_list[$node_count]->{MODULE} = "$target[0]";
          $target_list[$node_count]->{NODE} = "$target[1]";
          $target_list[$node_count]->{CHIP} = "$target[2]";
          $node_count++;
          $shub_count++;
          printf("ADD SHUB1: brick=$target[0] board=$target[1] chip=$target[2]\n")  if ($verbose_mode == 1);
        }
      }
      elsif ( $target[2] =~ /^SHUB/i )
      {
        printf("ADD SHUB: brick=$target[0] board=$target[1] chip=$target[2]\n")  if ($verbose_mode == 1);
        $shub_count++;
      }
    }
  }
}
