#!/stand/sysco/tools/bin/perl
#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 2003-2004 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
#-------------------------------------------------------------------------------
# @(#)$Id: rtr_clr.pl,v 1.2 2006/10/10 18:30:10 meyer Exp $
#
#  FILE: rtr_clr.pl
#
#  DESCRIPTION:  This file contains supporting PERL subroutines for initializing
#     NL4R MMRs that contain error state.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO               WHAT
#     --------  ----------------  -----------------------------------------
#     03/01/05  Karen Beighley    Initial version of script created: V1.0
#     06/14/05  Gary Meyer	  Updates to enable first_error disable error interrupts
#     06/28/05  Gary Meyer	  Updates to enable first_error disable error interrupts.
#     10/10/06  Gary Meyer	  Added R4_PORTx_LLP_ERR1 mmrs.
#
#-------------------------------------------------------------------------------

$SCRIPT_VER = "1.1";


#-------------------------------------------------------------------------------
#  Modules required for SuperScope MMR scripts
#-------------------------------------------------------------------------------

use lib "/stand/sysco/lib/sscope";

use common_defs;
use fmt_subs;
use jtag_subs;
use scancfg;
use reg_subs;
use mmr_subs;
use sysco_subs;
use error_regs;
use pktgen;
use shub2_01_mmrs;
use tio_01_mmrs;
use nl4rtr_01_mmrs;

#-------------------------------------------------------------------------------
#  Global variables for this script
#-------------------------------------------------------------------------------

$debug_parm = 0;          # User-specified debug parameter: encoded
$debug_flag = 0;          # Debug flag to enable messages
$verbose_mode = 0;

$num_base = 'hex';    # Default numeric base for access windows

# JTAG configuration file name - discover if no file specified
$scancfg_file      = "";   # No default CFG file (discover)
$scancfg_brk_types = "";   # Supported brick types for discovery
$scancfg_part_str  = "";   # Supported partition list for discovery

# MMR operations can use low-level L1 MMR commands (newer L1 firmware)
$mmr_flag = $TRUE;
$mmr_flag_specified = $FALSE;

# Default L2 connection name
$l2_host     = "";
$l2_copy_num = $SYSCO_COPY_INVALID;

$prompt_flag = $FALSE;
$init_flag   = $FALSE;

$tar_flag  = $FALSE;         # Target a specific brick in configuration
$tar_rack  = -1;             # Rack number of current target brick
$tar_slot  = -1;             # Slot number of current target brick


@rtr_names_clr =
  ( 
	"R4_LB_ERROR_DETAIL_1",
	"R4_LB_ERROR_DETAIL_2",
	"R4_LB_ERROR_SUMMARY",
	"R4_LB_FIRST_ERROR",
	"R4_PORTA_ERROR_DETAIL_CLEAR",
	"R4_PORTA_ERROR_SUMMARY",
	"R4_PORTA_FIRST_ERROR",
	"R4_PORTB_ERROR_DETAIL_CLEAR",
	"R4_PORTB_ERROR_SUMMARY",
	"R4_PORTB_FIRST_ERROR",
	"R4_PORTC_ERROR_DETAIL_CLEAR",
	"R4_PORTC_ERROR_SUMMARY",
	"R4_PORTC_FIRST_ERROR",
	"R4_PORTD_ERROR_DETAIL_CLEAR",
	"R4_PORTD_ERROR_SUMMARY",
	"R4_PORTD_FIRST_ERROR",
	"R4_PORTE_ERROR_DETAIL_CLEAR",
	"R4_PORTE_ERROR_SUMMARY",
	"R4_PORTE_FIRST_ERROR",
	"R4_PORTF_ERROR_DETAIL_CLEAR",
	"R4_PORTF_ERROR_SUMMARY",
	"R4_PORTF_FIRST_ERROR",
	"R4_PORTG_ERROR_DETAIL_CLEAR",
	"R4_PORTG_ERROR_SUMMARY",
	"R4_PORTG_FIRST_ERROR",
	"R4_PORTH_ERROR_DETAIL_CLEAR",
	"R4_PORTH_ERROR_SUMMARY",
	"R4_PORTH_FIRST_ERROR",
	"R4_ASIC_FIRST_ERROR",
	"R4_ASIC_FIRST_ERROR_TIMESTAMP",
	"R4_PORTA_LLP_ERR1",
	"R4_PORTB_LLP_ERR1",
	"R4_PORTC_LLP_ERR1",
	"R4_PORTD_LLP_ERR1",
	"R4_PORTE_LLP_ERR1",
	"R4_PORTF_LLP_ERR1",
	"R4_PORTG_LLP_ERR1",
	"R4_PORTH_LLP_ERR1"

  );


#-------------------------------------------------------------------------------
#  Process the command-line parameters (support "-f" and "--l2")
#-------------------------------------------------------------------------------

print "\nNL4R initialization script $SCRIPT_VER - reset using JTAG via L2/L1s\n";

example_parse_parameters();  # Simple parameter list


#-------------------------------------------------------------------------------
#  Enable debug messages if the user provided a debug parameter
#-------------------------------------------------------------------------------

$tmp = $debug_parm & 0xFF;
if ( $tmp )
{  $debug_flag = $tmp;
}
$tmp = ($debug_parm >> 8) & 0xFF;
if ( $tmp )
{  jtag_lib_msg_level($tmp);
}


#-------------------------------------------------------------------------------
#  Initialize the JTAG connection, load scan configuration and reset SICs/TAPs
#-------------------------------------------------------------------------------

jtag_lib_msg_level(0x00);      # JTAG library debug message control

if ( $l2_host eq "" )       # No L2 hardware: emulation mode
{  jtag_connect($JTAG_ACCESS_NONE, "localhost", $l2_port);
   print "\n***** NO HARDWARE CONNECTION - SOFTWARE EMULATION *****\n";
   print "Use the \"--l2 <L2 address>\" command-line parameter to specify a real L2.\n";
   mmr_access($MMR_ACCESS_NONE);
}
else
{  jtag_connect($JTAG_ACCESS_SYSCO, $l2_host, $l2_port, $l2_copy_num);
   mmr_access($MMR_ACCESS_JTAG, $mmr_flag);
}
jtag_set_data_just($JTAG_JUST_RIGHT);

#
#  Read the scan configuration from file or hardware
#
scancfg_load($scancfg_file, $TRUE, $scancfg_brk_types, $scancfg_part_str);

if ($debug_flag & $DEBUG_GENERAL)
{  print "\n-------------------------------------------------------------------\n";
   print "\nCURRENT SCAN CHAIN CONFIGURATION:\n";
   scancfg_dump($FALSE);
   print "\n-------------------------------------------------------------------\n";
}

#
#  Initialize the JTAG TAP(s) and SIC(s)
#
scancfg_hardware_init();

#
#  Open the other system controller connection
#
if ( $l2_host ne "" )
{
   sysco_open_connection($l2_host, $DEFAULT_SYSCO_PORT, $l2_copy_num);
}


################################################################################
#
#  REAL WORK OF SCRIPT GOES HERE
#
################################################################################

@list = scan_build_chip_list('nl4rtr');

foreach $entry ( @list )
{  @parms = split(':', $entry);
   $mref = $modules{ $parms[0] };
   if ( !defined($mref) )
   {  $emess = "Unable to find module pointer: $parms[0]\n";
      $emess .= "\nVerify the module is present in the system and HAS POWER ON!";
      $ecode = -1;
      &$comm_err_fun($emess);
      goto end_label;
   }

   $tar_module = $$mref{NAME};
   $tar_rack   = $$mref{RACK};
   $tar_slot   = $$mref{SLOT};
   $tar_board  = $parms[1];
   $tar_chip   = $parms[2];

   rtr_mmr_init($mref, $tar_board, $tar_chip);

}


#-------------------------------------------------------------------------------
#  Close JTAG connection and cleanup
#-------------------------------------------------------------------------------

end_label:

if ( defined($jtag_init) )   # JTAG/scan module included?
{  jtag_disconnect();
}  

if ( defined($sysco_init) )  # System controller module included?
{  sysco_close_connection();
}  

print "\n\n   Script completed.\n\n";

exit 0;


################################################################################
################################################################################

sub rtr_mmr_init
{
#   printf "\nrtr_mmr_init ( @_ )\n";

   my $mref      = $_[0];  # Reference to module structure
   my $bname     = $_[1];  # Board name on module/brick (i.e. "NODE" or "MOATB")
   my $cname     = $_[2];  # Logical chip name on board (i.e. "SHUB0" or "TIO0")
   my ($oname, $mode, %hdata);

   print "\n";
   print "------------------------------------------------------------------\n";
   print "--  NL4R MMR INITIALIZATION SEQUENCE FOR $tar_module:$tar_board:$tar_chip\n";
   print "------------------------------------------------------------------\n";

   mmr_target($SELECT, $$mref{NAME}, $bname, $cname);
                                                                                                                                                   
   @addr = ();
   @data = ();
    
   for ($i = 0; $i <= $#rtr_names_clr; $i++)
   {  $reg_ptr = reg_create_mmr_inst("nl4rtr_01", $rtr_names_clr[$i]);
      $addr[$i] = reg_addr($reg_ptr);
      if ($rtr_names_clr[$i] =~ /DETAIL_CLR/) {
        $data[$i] = 1;
      }
      else {
        $data[$i] = 0;
      }
      if (($debug_flag & $DEBUG_GENERAL) || ($verbose_mode == 1))
      {  printf "   %-30s @ 0x%016llX\n", $rtr_names_clr[$i], $addr[$i];
      }
   }
   mmr_write_group(\@addr, \@data); # Write list of addresses
}


################################################################################
################################################################################

#-------------------------------------------------------------------------------

sub scan_build_chip_list
{
   my $type = $_[0];  # Chip type prefix to search for
   my @data = ();
   my ($chn_list, $chn_ptr, $chip_list, $chip_ptr, $name, $c, $i);

   foreach $mod_ptr ( @module_ptrs )
   {  $chn_list = $$mod_ptr{CHN_REFS}; # List of chain definition pointers
      for ($c = 0; $c <= $#$chn_list; $c++)
      {  $chn_ptr = $$chn_list[$c];
         $chip_list = $$chn_ptr[$CHN_IND_CREFS];
         for ($i = 0; $i <= $#$chip_list; $i++)
         {  $chip_ptr  = $$chip_list[$i];  # Pointer to chip definition
            $curr_type = $$chip_ptr[$CMP_IND_DESC];
            if ( $curr_type =~ /^$type/i )
            {  $name = sprintf("%s:%s:%s", $$mod_ptr{NAME}, $$chip_ptr[$CMP_IND_BOARD], $$chip_ptr[$CMP_IND_LOG]);
               push (@data, $name);       # Add chip name to list (ARRAY)
            }
         }
      }
   }

   return(@data);
}


#-------------------------------------------------------------------------------
#
#  NOTES: simple command-line processing for now!
#
#  USAGE: rtr_clr.pl [-f <file>] [--l2 <L2 name>] [-M <yes|no>] [-h]
#
#-------------------------------------------------------------------------------

sub example_usage
{
   print "\nUSAGE: rtr_clr [--l2 <L2 name>]\n";

   print "\nNOTES: The NL4 R-brick reset script supports the following parameters:\n";
   print "-h|-help         Display script usage and exit.\n";
   print "-ver             Display script version and exit.\n";
   print "--l2 <host>      Name or IP address of L2 system controller\n";
   print "                   Software emulation mode if no L2 controller specified\n";
   print "-m <host>        Machine name - assumes l2-machine_name for L2 target.\n";
   print "-f <file>        Name of JTAG configuration file (.cfg) defining hardware\n";
   print "                   Configuration discovered from hardware if no file specified.\n";
   print "-b <text>        List of supported bricks types to discover if no configuration file.\n";
   print "\n";
}


#-------------------------------------------------------------------------------

sub example_parse_parameters
{
   $argc = $#ARGV + 1;
   $i = 0;
   $machine_name = "None";
   while ( $i < $argc )
   {
      if ( ($ARGV[$i] eq '-h') || ($ARGV[$i] eq '-help') )
      {  example_usage();
         exit 0;
      }
      if ( ($ARGV[$i] eq '-ver') || ($ARGV[$i] eq '-version') )
      {  print "\n";
         exit 0;
      }

      # if ( ($i + 1) == $argc )
      # {  print "\nERROR - Missing parameter with command line option.\n";
         # print "Option: $ARGV[$i]\n";
         # exit 1;
      # }

      if ( $ARGV[$i] eq '-f' )
      {  $i++;
         $scancfg_file = $ARGV[$i];
      }
      elsif ( $ARGV[$i] eq '-b' )
      {  $i++;
         $scancfg_brk_types = $ARGV[$i];
      }
      elsif ( $ARGV[$i] eq '-m' )
      {  $i++;
         $machine_name = $ARGV[$i];
	 $machine_name =~ s/\s*//g;
	 $l2_host = "l2-" . "$machine_name";
      }
      elsif ( $ARGV[$i] eq '-part' )
      {  $i++;
         $scancfg_part_str = $ARGV[$i];
      }
      elsif ( $ARGV[$i] eq '-debug' )
      {  $i++;
         $debug_parm = convert_text2num($ARGV[$i]);
      }
      elsif ( $ARGV[$i] eq '-verbose' )
      {  
         $verbose_mode = 1;
      }
      elsif ( $ARGV[$i] eq '-p' )
      {  $i++;
         if ( ($ARGV[$i] eq 'yes') || ($ARGV[$i] eq 'y') )
         {  $prompt_flag = $TRUE;
         }
         elsif ( ($ARGV[$i] eq 'no') || ($ARGV[$i] eq 'n') )
         {  $prompt_flag = $FALSE;
         }
         else
         {  print "\nERROR - Unknown parameter specified with \"-p\" option: $ARGV[$i]\n";
            print "The \"-p\" option supports a parameter of \"yes\" or \"no\".\n";
            exit 1;
         }
      }
      elsif ( $ARGV[$i] eq '-T' )
      {  $i++;
         @parms = split('\.', $ARGV[$i]);
         if ( $#parms != 1 )
         {  print "\nERROR - Invalid parameter specified with \"-T\" option: $ARGV[$i]\n";
            exit 1;
         }
         $tar_rack = convert_text2num($parms[0]);
         $tar_slot = convert_text2num($parms[1]);
         $tar_flag = $TRUE;
      }
      elsif ( $ARGV[$i] eq '-init' )
      {  $i++;
         if ( ($ARGV[$i] eq 'yes') || ($ARGV[$i] eq 'y') )
         {  $init_flag = $TRUE;
         }
         elsif ( ($ARGV[$i] eq 'no') || ($ARGV[$i] eq 'n') )
         {  $init_flag = $FALSE;
         }
         else
         {  print "\nERROR - Unknown parameter specified with \"-p\" option: $ARGV[$i]\n";
            print "The \"-p\" option supports a parameter of \"yes\" or \"no\".\n";
            exit 1;
         }
      }
      elsif ( $ARGV[$i] eq '-M' )
      {  $i++;
         if ( ($ARGV[$i] eq 'yes') || ($ARGV[$i] eq 'y') )
         {  $mmr_flag = $TRUE;
         }
         elsif ( ($ARGV[$i] eq 'no') || ($ARGV[$i] eq 'n') )
         {  $mmr_flag = $FALSE;
         }
         else
         {  print "\nERROR - Unknown parameter specified with \"-M\" option: $ARGV[$i]\n";
            print "The \"-M\" option supports a parameter of \"yes\" or \"no\".\n";
            exit 1;
         }
      }
      elsif ( $ARGV[$i] eq '--l2' )
      {  $i++;
         $l2_host = $ARGV[$i];
      }
      elsif ( $ARGV[$i] eq '--copy' )
      {  $i++;
         $tmp = convert_text2num($ARGV[$i]);
         if ( $tmp == $SYSCO_COPY_INVALID )
         {  print "\nERROR - Invalid parameter specified with \"--copy\" option: $ARGV[$i]\n";
            print "        The \"--copy\" option supports a nonzero index for biasing the USB port.\n";
            exit 1;
         }
         $l2_copy_num = $tmp;
      }

      elsif ( $ARGV[$i] eq '-nl3' )
      {  $i++;
         $nl4r_port_cfg = nl4r_process_port_string($ARGV[$i]);
         if ( $nl4r_port_cfg == -1 )
         {  print "\nERROR - Unknown parameter specified with \"-nl3\" option: $ARGV[$i]\n";
            print "The \"-nl3\" option supports a string containing port characters (A-H).\n";
            exit 1;
         }
      }
      elsif ( $ARGV[$i] eq '-tp' )
      {  $i++;
         $tmp = convert_text2num($ARGV[$i]);
         $nl4r_tp_data = $tmp & 0x3F;
      }
      elsif ( $ARGV[$i] eq '-fence' )
      {  $i++;
         $tmp = convert_text2num($ARGV[$i]);
         $nl4r_fence_val = $tmp & 0xFFFFFF;
      }
      elsif ( $ARGV[$i] eq '-eclk' )
      {  $i++;
         if ( ($ARGV[$i] eq 'yes') || ($ARGV[$i] =~ /^y/i) )
         {  $nl4r_eclk_sel = $TRUE;
         }
         elsif ( ($ARGV[$i] eq 'no') || ($ARGV[$i] =~ /^n/i) )
         {  $nl4r_eclk_sel = $FALSE;
         }
         else
         {  print "\nERROR - Unknown parameter specified with \"-eclk\" option: $ARGV[$i]\n";
            print "The \"-eclk\" option supports a parameter of \"yes\" or \"no\".\n";
            exit 1;
         }
      }
      else
      {  print "\nERROR - Unknown command line option specified.\n";
         print "Option: $ARGV[$i]\n";
         exit 1;
      }

      $i++;
   }
}


#-------------------------------------------------------------------------------
