#!/stand/sysco/tools/bin/perl
#
#==============================================================================
#
#  Unpublished copyright (c) 2004-2005, Silicon Graphics, Inc.
#  THIS IS UNPUBLISHED CONFIDENTIAL AND PROPRIETARY SOURCE CODE OF SGI.
#                                                                      
#  The copyright notice above does  not evidence any actual or intended
#  publication  or  disclosure  of  this source  code,  which  includes
#  information that is confidential  and/or proprietary, and is a trade
#  secret, of  Silicon Graphics, Inc.   ANY REPRODUCTION, MODIFICATION,
#  DISTRIBUTION, PUBLIC  PERFORMANCE, OR  PUBLIC DISPLAY OF  OR THROUGH
#  USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS  WRITTEN CONSENT  OF
#  SILICON GRAPHICS, INC.  IS  STRICTLY PROHIBITED, AND IN VIOLATION OF
#  APPLICABLE  LAWS   AND  INTERNATIONAL  TREATIES.    THE  RECEIPT  OR
#  POSSESSION OF  THIS SOURCE CODE AND/OR RELATED  INFORMATION DOES NOT
#  CONVEY OR IMPLY ANY RIGHTS  TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS
#  CONTENTS,  OR TO  MANUFACTURE, USE,  OR  SELL ANYTHING  THAT IT  MAY
#  DESCRIBE, IN WHOLE OR IN PART.
#
#------------------------------------------------------------------------------
#
#ident $Id: wirestat.pl,v 1.12 2006/02/28 22:17:34 oster Exp $
#
#------------------------------------------------------------------------------
#
#  FILE: wirestat.pl
#
#  DESCRIPTION:  post-processing script for smdk wirestat data
#
#  REVISION HISTORY:
#
#  Rev.   Date      Author            Description
#  -----  --------  ----------------  ------------------------------------------
#  0.0.1  02/21/06  Darin Oster       Created.
#  1.0.0  02/23/06  Darin Oster       Basic dataset compare working.
#  1.0.1  02/23/06  Darin Oster       Asthetics
#  1.0.2  02/24/06  Darin Oster       Enhanced with sort feature.
#  1.0.3  02/24/06  Darin Oster       Revised hash key names.    
#  1.0.4  02/24/06  Darin Oster       Updated help screen.
#  1.0.5  02/27/06  Darin Oster       Implemented recursive sort for major speedup.
#  1.0.6  02/27/06  Darin Oster       Added analyze link groups feature.
#  1.0.7  02/27/06  Darin Oster       Enhanced analyze link groups feature.
#==============================================================================

#------------------------------------------------------------------------------
#  Include Necessary Library Modules
#------------------------------------------------------------------------------
use Math::BigFloat;
use lib "/stand/sysco/lib/sscope";

$app_ver  = "1.0.7";
$app_name = "wirestat.pl $app_ver - SN2 Wirestat Script";

$SHUB2=1;

use common_defs;
use fmt_subs;
use Getopt::Long;

$DISABLE = 0;
$ENABLE  = 1;


#------------------------------------------------------------------------------
# Define test list.
#------------------------------------------------------------------------------

$BER_INIT = "?.??e-??";
$BER_ZERO = Math::BigFloat->new(0);

#------------------------------------------------------------------------------
# Define command line options and set default values
#------------------------------------------------------------------------------
$VERB_NONE    = 0;
$VERB_ERROR   = 1;
$VERB_WARNING = 2;
$VERB_NORMAL  = 3;
$VERB_DETAIL  = 4;
$VERB_DEBUG   = 5;

local $verbosity = $VERB_NORMAL;

undef $op_print_help;

local $timestamp;

local $write_output_to_file = $FALSE;
local $compare_filter_spec  = "a";
local $compare_filter_sev   = undef;
local $compare_filter_ber   = undef;
local $output_filter_spec   = "a";
local $output_filter_sev    = undef;
local $output_filter_ber    = undef;
undef $input_data_spec;
undef $user_title;
local @input_filename = ();
local $output_format = "normal";
local $timestamp;
local $sort_data_spec = "P_BER";
local $analyze_link_groups = 0;

$HELP_SUMMARY = "\
SCRIPT USAGE:\
\
  wirestat -nofile -data filenameA[,filenameB]\
\
DESCRIPTION:\
\
  This script will format the output of the \"dscr 1 wiresummary 3\" smdk screen\
  and can perform a comparison between two BER data samples. An option is available\
  for selecting an alternate \"excel\"\ output format intended for importing into\
  a spreadsheet to allow for flexible sorting of links by splitting the link\
  identifiers into separate fields.\
\
SCRIPT OPTIONS:\
\
  option           arguments  description\
  ----------------- --------- -------------------------------------------------\
  -help                       Print this screen\
  -v               <level>    Select how verbose to be\
  -t               <text>     Add title to script log\
  -data       <fileA[,fileB]> Specify 1 or 2 data files containing wiresummary data\
  -file                       Set to enable creation of timestamped log file\
  -format          <string>   Specify \"excel\" for alternate output format\
  -analyze                    Set to analyze sort-specified groups of links\
  -cfilter         <letter>   Specify a-z to filter small compare differences\
  -sort            <list>     By default, the data is sorted by peak BER. Use this\
                              option to specify an alternate sort sequence. Choices\
                              of fields to sort by include the following:\
                               r_vnode, r_port, r_rack, r_slot, r_board, r_slab,\
                               d_vnode, d_port, d_rack, d_slot, d_board, d_slab,\
                               util, c_ber, p_ber, sbits, mbits,\
                               max, sbe, mbe, we, mb2, mb3, mb4, and mb>\
                              Use \"-\" prefix on any fields to reverse sort.\
\
EXAMPLES:\
\
  # format single data set; sort by p_ber, r_slab, r_board, and finally r_port\
  wirestat.pl -data infileA -sort p_ber,r_slab,r_board,r_port\
\
  # format single data set; sort by p_ber, d_slab, d_board, and finally d_port\
  wirestat.pl -data infileA -sort p_ber,d_slab,d_board,d_port\
\
  # compare two data sets; sort by c_ber; ignore differences < e
  wirestat.pl -data infileA,infileB -cf e -sort c_ber\
\
  # compare two data sets; sort by -p_ber, and r_port; ignore differences < d
  wirestat.pl -data infileA,infileB -cf d -sort -p_ber,r_port\
\
  # same as above, only output with separated channel descriptors for excel
  wirestat.pl -data infileA,infileB -cf d -sort -p_ber,r_port -format excel\
\n";

#==============================================================================
#  Script Entry
#==============================================================================
if(!parse_command_line_options()) { exit; }

if(defined $op_print_help) {
  print_help_screen($op_print_help);
} 

check_file_space("wirestat.o");

chomp ($timestamp = `date +'%a_%b_%e_%T'`);
$timestamp =~ s/\s+/0/g; #  Replace spaces with "0"
printv($VERB_NORMAL, "\n$app_name STARTED at $timestamp\n");

if(defined $user_title) {
  print "\nScript Purpose:  $user_title\n\n";
}


#------------------------------------------------------------------------------
#  INPUT DATA
#------------------------------------------------------------------------------
local (%dataA, %dataB, %dataC);
local (@keysA, @keysB, @keysC);

if(!defined $input_filename[0]) {
  printv($VERB_ERROR, "No filename specified.\n");
}

if(defined $input_filename[1]) {
  $analyze_link_groups = 0;
}
  
input_wirestat_data($input_filename[0], \%dataA, \%dataB);
sort_wirestat_data(\%dataA, \@keysA, $sort_data_spec);

if(defined $input_filename[1]) {
  input_wirestat_data($input_filename[1], \%dataB, \%dataA);
  compare_wirestat_data();
  sort_wirestat_data(\%dataB, \@keysB, $sort_data_spec);
  sort_wirestat_data(\%dataC, \@keysC, $sort_data_spec);
}

if(!$analyze_link_groups) {
  output_wirestat_data(\%dataA, \@keysA, "");
  if(defined $input_filename[1]) {
    output_wirestat_data(\%dataB, \@keysB, "");
    output_wirestat_data(\%dataC, \@keysC, "");
  }
}


#------------------------------------------------------------------------------
#  Normal Exit
#------------------------------------------------------------------------------

execution_done(0);


#==============================================================================
# Subroutines
#==============================================================================

#------------------------------------------------------------------------------
#  bubble_hash_sort   
#------------------------------------------------------------------------------
sub bubble_hash_sort
{
  my ($hash_ref, $link_keys_ref, $sort_key, $sort_bias) = @_;
  my $sorted;
  my $last;
  my $link;

  $sorted = $FALSE;
  $last   = scalar(@$link_keys_ref);
  while ($sorted==$FALSE) {
      $sorted = $TRUE;
      $last--;
      for ($link=0; $link<$last; $link++) {
          $key0 = $$link_keys_ref[$link];
          $key1 = $$link_keys_ref[$link+1];

          # sort numbers
          if(($sort_key =~ /BER/) || ($sort_key =~ /UTIL/)) {
              if($sort_bias) {
                if($$hash_ref{$key0}{$sort_key} > $$hash_ref{$key1}{$sort_key}) {
                  $$link_keys_ref[$link]   = $key1;
                  $$link_keys_ref[$link+1] = $key0;
                  $sorted = $FALSE;
                }
              }
              elsif($$hash_ref{$key0}{$sort_key} < $$hash_ref{$key1}{$sort_key}) {
                $$link_keys_ref[$link]   = $key1;
                $$link_keys_ref[$link+1] = $key0;
                $sorted = $FALSE;
              }

          # sort strings
          } else {
              if($sort_bias) {
                if(($$hash_ref{$key0}{$sort_key}) lt ($$hash_ref{$key1}{$sort_key})) {
                  $$link_keys_ref[$link]   = $key1;
                  $$link_keys_ref[$link+1] = $key0;
                  $sorted = $FALSE;
                }
              }
              elsif(($$hash_ref{$key0}{$sort_key}) gt ($$hash_ref{$key1}{$sort_key})) {
                $$link_keys_ref[$link]   = $key1;
                $$link_keys_ref[$link+1] = $key0;
                $sorted = $FALSE;
              }
          }
      }
  }
}

#------------------------------------------------------------------------------
#  recursive_hash_sort
#------------------------------------------------------------------------------
sub recursive_hash_sort
{
  my ($hash_ref, $link_keys_ref, $sort_keys_ref) = @_;
  my ($sort_key, $sort_bias);
  my $index;
  my $hash_key_length;
  my $subset_length;
  my @subset = ();
  my ($valueA, $valueB);
  my @tmp_sort_keys;

  $hash_key_length = scalar(@$link_keys_ref);
  $sort_key = undef;

  while(!defined $$hash_ref{@$link_keys_ref[0]}{$sort_key}) {
    $sort_key = shift @$sort_keys_ref;
    return unless defined $sort_key;
    if($sort_bias = ($sort_key =~ /^-/)) {
      ($sort_key = $sort_key) =~ s/^.//;
    }
  }

  bubble_hash_sort($hash_ref, $link_keys_ref, $sort_key, $sort_bias);

  if(scalar(@$sort_keys_ref)) {

    if($sort_key =~ /ber/) { # do not subdivide for BER sort keys
      @tmp_sort_keys = (@$sort_keys_ref);
      recursive_hash_sort($hash_ref, $link_keys_ref, \@tmp_sort_keys);
    }
    else { # build sub-list, sort and splice subset if > 1 element
      for($index=0; $index<$hash_key_length; ) {
        @subset = ();
        $valueA = "$$hash_ref{$$link_keys_ref[$index]}{$sort_key}";
        $valueB = $valueA;
        while(($valueB eq $valueA) && ($index<$hash_key_length)) {
          push @subset, $$link_keys_ref[$index];
          $index++;
          $valueB = "$$hash_ref{$$link_keys_ref[$index]}{$sort_key}";
        }
        $subset_length = scalar(@subset);
        if($subset_length > 1) {
          @tmp_sort_keys = (@$sort_keys_ref);
          recursive_hash_sort($hash_ref, \@subset, \@tmp_sort_keys);
          splice(@$link_keys_ref, ($index-$subset_length), $subset_length, @subset);
        }
      }
    }
  } elsif($analyze_link_groups) {
      output_wirestat_data($hash_ref,  $link_keys_ref, "; analysis of sort-specified link group");
      analyze_wirestat_data($hash_ref, $link_keys_ref);
  }
}


#------------------------------------------------------------------------------
#  sort_wirestat_data
#------------------------------------------------------------------------------
sub sort_wirestat_data
{
  my $hash_ref      = $_[0];
  my $link_keys_ref = $_[1];
  my @sort_keys     = split /,/, $_[2];
  my $key;

  foreach $key (keys %$hash_ref) {
#   if(($$hash_ref{$key}{"P_BER"} ne $BER_ZERO) && ($$hash_ref{$key}{"C_BER"} ne $BER_ZERO) )  {
      push @$link_keys_ref, $key;
#   }
  }

  recursive_hash_sort($hash_ref, $link_keys_ref, \@sort_keys);
}


#------------------------------------------------------------------------------
#  compare_wirestat_string
#------------------------------------------------------------------------------
sub compare_wirestat_string
{
  my @charsA = split //, $_[0];
  my @charsB = split //, $_[1];
  my @charsC = "abc";

  for($i=0; $i<scalar(@charsA); $i++) {
    $charsC[$i] = compare_wirestat_char($charsA[$i], $charsB[$i]);
  }

  return(join "", @charsC); 
}

#------------------------------------------------------------------------------
#  compare_wirestat_char
#------------------------------------------------------------------------------
sub compare_wirestat_char
{
    my $chA = $_[0];
    my $chB = $_[1];
    my $chC = "";
    my $intA;
    my $intB;
    my $intC;

    $chA =~ tr/A-Z0./a-z``/;
    $chB =~ tr/A-Z0./a-z``/;

    # test for valid char
    if(($chA =~ tr/a-z`//c) || ($chA =~ tr/a-z`//c)) {
      printv($VERB_ERROR, "Unexpected char passed to compare_wirestat_char($_[0],$_[1])\n");
    }


    $intA  = ord($chA);
    $intB  = ord($chB);
    $intSF = ord($compare_filter_sev) - 97;

    if(($intA - $intB) > $intSF) {
       $chC = sprintf("%s", chr(($intA-$intB) + 64));
       printv($VERB_DEBUG, "agtb compare_wirestat_char($_[0],$_[1])  ($intA) ($intB) ($chC)\n");
    }
    elsif(($intB - $intA) > $intSF) {
       $chC = sprintf("%s", chr(($intB-$intA) + 96));
       printv($VERB_DEBUG, "altb compare_wirestat_char($_[0],$_[1])  ($intA) ($intB) ($chC)\n");
    }
    else {
       $chC = ".";
    }

    return($chC);
}


#------------------------------------------------------------------------------
#  Compare the data sets
#------------------------------------------------------------------------------
sub compare_wirestat_data 
{
  my $key;

  foreach $key (keys %dataA) {

      $dataC{$key} = {

          "RECEIVER" => $dataA{$key}{"RECEIVER"},
          "R_VNODE"  => $dataA{$key}{"R_VNODE"},
          "R_PORT"   => $dataA{$key}{"R_PORT"},
          "R_RACK"   => $dataA{$key}{"R_RACK"},
          "R_TYPE"   => $dataA{$key}{"R_TYPE"},
          "R_SLOT"   => $dataA{$key}{"R_SLOT"},
          "R_BOARD"  => $dataA{$key}{"R_BOARD"},
          "R_SLAB"   => $dataA{$key}{"R_SLAB"},

          "DRIVER"   => $dataA{$key}{"DRIVER"},
          "D_VNODE"  => $dataA{$key}{"D_VNODE"},
          "D_PORT"   => $dataA{$key}{"D_PORT"},
          "D_RACK"   => $dataA{$key}{"D_RACK"},
          "D_TYPE"   => $dataA{$key}{"D_TYPE"},
          "D_SLOT"   => $dataA{$key}{"D_SLOT"},
          "D_BOARD"  => $dataA{$key}{"D_BOARD"},
          "D_SLAB"   => $dataA{$key}{"D_SLAB"},

          "UTIL"     => ($dataA{$key}{"UTIL"} - $dataB{$key}{"UTIL"}),
          "MAX"      => ($dataA{$key}{"MAX"}  - $dataB{$key}{"MAX"}),

          "C_BER"    => ($dataA{$key}{"C_BER"} - $dataB{$key}{"C_BER"}),
          "P_BER"    => ($dataA{$key}{"P_BER"} - $dataB{$key}{"P_BER"}),

          "SBITS"    => compare_wirestat_string($dataA{$key}{"SBITS"}, $dataB{$key}{"SBITS"}),
          "MBITS"    => compare_wirestat_string($dataA{$key}{"MBITS"}, $dataB{$key}{"MBITS"}),

          "SBE"      => compare_wirestat_char($dataA{$key}{"SBE"}, $dataB{$key}{"SBE"}),
          "MBE"      => compare_wirestat_char($dataA{$key}{"MBE"}, $dataB{$key}{"MBE"}),
           "WE"      => compare_wirestat_char($dataA{$key}{ "WE"}, $dataB{$key}{ "WE"}),
          "MB2"      => compare_wirestat_char($dataA{$key}{"MB2"}, $dataB{$key}{"MB2"}),
          "MB3"      => compare_wirestat_char($dataA{$key}{"MB3"}, $dataB{$key}{"MB3"}),
          "MB4"      => compare_wirestat_char($dataA{$key}{"MB4"}, $dataB{$key}{"MB4"}),
          "MB>"      => compare_wirestat_char($dataA{$key}{"MB>"}, $dataB{$key}{"MB>"}),
     };

     if( ($dataA{$key}{"UTIL"} == 999) || ($dataB{$key}{"UTIL"} == 999) ) {
          $dataC{$key}{"UTIL"} = 999;
     }
  }
}

#------------------------------------------------------------------------------
#  Dump linkstat hash
#------------------------------------------------------------------------------
sub output_wirestat_data
{
  my ($key, $subkey, $value);
  my ($ber_peak, $ber_cumu, $util);
  my $mesg;
  my $hash_keys = $_[1];

  my $h_ref = $_[0];

  if($h_ref == \%dataA) {
    $data_set_description = "INPUT DATA FROM $input_filename[0]$_[2]";
  } elsif ($h_ref == \%dataB) {
    $data_set_description = "INPUT DATA FROM $input_filename[1]$_[2]";
  } else {
    $data_set_description = "DIFFERENCE FROM $input_filename[0] - $input_filename[1]$_[2]";
  }


  if($output_format eq "excel") {

    printv($VERB_NORMAL, "\n NUMAlink Wire Summary:  $data_set_description\n");
    printv($VERB_NORMAL, "_____________________________________________________________________________________________________________________________________________________________________________\n");
    printv($VERB_NORMAL, "                                                                     <---------- SINGLE BIT FAILURES ----------> <--------- MULTIPLE BIT FAILURES --------->  M S M   <-MBE->\n");
    printv($VERB_NORMAL, "receiver------------> driver-------------->   Cumulative     Peak    3333333333 2222222222 1111111111 0000000000 3333333333 2222222222 1111111111 0000000000  A B B W HSTGRAM\n");
    printv($VERB_NORMAL, "vnode p rck t st bd s vnode p rck t st bd s util    BER       BER    9876543210 9876543210 9876543210 9876543210 9876543210 9876543210 9876543210 9876543210  X E E E 2 3 4 >\n");
    printv($VERB_NORMAL, "----- - --- - -- -- - ----- - --- - -- -- - ---- --------- --------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- -- - - - - - - -\n");

  } else {

    printv($VERB_NORMAL, "\n NUMAlink Wire Summary:  $data_set_description\n");
    printv($VERB_NORMAL, "_________________________________________________________________________________________________________________________________________________________________________\n");
    printv($VERB_NORMAL, "                                                                 <---------- SINGLE BIT FAILURES ----------> <--------- MULTIPLE BIT FAILURES --------->  M S M   <-MBE->\n");
    printv($VERB_NORMAL, "rcv     module      driver  module        Cumulative     Peak    3333333333 2222222222 1111111111 0000000000 3333333333 2222222222 1111111111 0000000000  A B B W HSTGRAM\n");
    printv($VERB_NORMAL, "channel ^slot#slab  channel ^slot#slab  util    BER       BER    9876543210 9876543210 9876543210 9876543210 9876543210 9876543210 9876543210 9876543210  X E E E 2 3 4 >\n");
    printv($VERB_NORMAL, "------- ----------- ------- ----------- ---- --------- --------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- -- - - - - - - -\n");

  }

  foreach $key (@$hash_keys) {

    $peak_ber = sprintf("%9.2e", $$h_ref{$key}{"P_BER"});
    $cumu_ber = sprintf("%9.2e", $$h_ref{$key}{"C_BER"});
    $util     = ($$h_ref{$key}{"UTIL"}==999) ? "   ." : sprintf("%4d", $$h_ref{$key}{"UTIL"});

    # if($peak_ber =~ /0.00e/) { $peak_ber = " .      "; }
    # if($cumu_ber =~ /0.00e/) { $cumu_ber = " .      "; }

    if($output_format eq "excel") {

      $mesg = sprintf("%5s %1s %3s %1s %2s %2s %1s %5s %1s %3s %1s %2s %2s %1s",
          $$h_ref{$key}{"R_VNODE"},
          substr($$h_ref{$key}{"R_PORT"}, 1, 1),
          $$h_ref{$key}{"R_RACK"},
          $$h_ref{$key}{"R_TYPE"},
          $$h_ref{$key}{"R_SLOT"},
          $$h_ref{$key}{"R_BOARD"},
          $$h_ref{$key}{"R_SLAB"},
          $$h_ref{$key}{"D_VNODE"},
          substr($$h_ref{$key}{"D_PORT"}, 1, 1),
          $$h_ref{$key}{"D_RACK"},
          $$h_ref{$key}{"D_TYPE"},
          $$h_ref{$key}{"D_SLOT"},
          $$h_ref{$key}{"D_BOARD"},
          $$h_ref{$key}{"D_SLAB"});
    } else {
      $mesg = sprintf("%19s %19s", $$h_ref{$key}{"RECEIVER"}, $$h_ref{$key}{"DRIVER"});
    }

    $mesg .= sprintf(" %4s %9s %9s %10s %10s %10s %10s %10s %10s %10s %10s %2d %s %s %s %s %s %s %s %s\n",
               $util, $cumu_ber, $peak_ber,
               substr($$h_ref{$key}{"SBITS"}, 30, 10),
               substr($$h_ref{$key}{"SBITS"}, 20, 10),
               substr($$h_ref{$key}{"SBITS"}, 10, 10),
               substr($$h_ref{$key}{"SBITS"},  0, 10),
               substr($$h_ref{$key}{"MBITS"}, 30, 10),
               substr($$h_ref{$key}{"MBITS"}, 20, 10),
               substr($$h_ref{$key}{"MBITS"}, 10, 10),
               substr($$h_ref{$key}{"MBITS"},  0, 10),
               $$h_ref{$key}{"MAX"},
               $$h_ref{$key}{"SBE"},
               $$h_ref{$key}{"MBE"},
               $$h_ref{$key}{"WE"},
               $$h_ref{$key}{"MB2"},
               $$h_ref{$key}{"MB3"},
               $$h_ref{$key}{"MB4"},
               $$h_ref{$key}{"MB>"});

#   if($mesg =~ /0.00e.00  0.00e.00/i) {
#   if($mesg =~ /0.00e.00  0.00e.00 .......... .......... .......... .......... .......... .......... .......... ..........  0 . . . . . . ./) {
#   if($mesg =~ /.......... .......... .......... .......... .......... .......... .......... ..........  0 . . . . . . ./) {
#     printv($VERB_DETAIL, "mesg");
#   } else {
#     printv($VERB_NORMAL, $mesg);
#   }
    printv($VERB_NORMAL, $mesg);
  }

  if($output_format eq "excel") {
    printv($VERB_NORMAL, "_____________________________________________________________________________________________________________________________________________________________________________\n\n");
  } else {
    printv($VERB_NORMAL, "_________________________________________________________________________________________________________________________________________________________________________\n\n");
  }
}

#------------------------------------------------------------------------------
# Print out usage information
#------------------------------------------------------------------------------
sub input_wirestat_data {

  my ($filename, $href_x, $href_y) = @_;
  my @linkd = ();

  unless(open $infile, $filename) {
    printv($VERB_ERROR, "Unable to open $filename\n");
  }

  printv($VERB_NORMAL, "\nReading wiresummary data from filename \"$filename\".\n");
  printv($VERB_NORMAL, "#------------------------------------------------------------------------------\n");

  while(<$infile>) {
    if($_ =~ /^# /) {
      printv($VERB_NORMAL, $_);
    }
    elsif($_ =~ /^.....\.. /) {
      $link_record = $_;
      chomp($link_record);
      $link_record = $link_record . <$infile>;
      chomp($link_record);
      if(substr($link_record, 44, 8) eq "        ") {  # replace blank ber field
         substr($link_record, 44, 8) =  "0.00E+00";
      }
      if(substr($link_record, 53, 8) eq "        ") {  # replace blank ber field
         substr($link_record, 53, 8) =  "0.00E+00";
      }
      $link_record =~ s/^  *//; # Remove leading spaces
      @linkd = split /  */, $link_record;
      if(scalar(@linkd)!=23) {
        printv($VERB_ERROR, "ERROR: Unexpected record length or format.\n($link_record)\n");
      }
      printv($VERB_DEBUG, "LINK_RECORD ($link_record)\n");
      printv($VERB_DEBUG, "LINK_DATA (@linkd)\n");
      for($i=0; $i<scalar(@linkd); $i++) {
        printv($VERB_DEBUG, "LINK_DATA[$i] = ($linkd[$i])\n");
      }

      $key = "$linkd[0] $linkd[1] $linkd[2]";

      $$href_x{$key} = {

          "RECEIVER" => "$linkd[0] $linkd[1]",
          "R_VNODE"  => substr ($linkd[0],  0, -2),
          "R_PORT"   => substr ($linkd[0],  0,  1) . substr ($linkd[0], -1, 1) . substr ($linkd[1], 3, 1),
          "R_RACK"   => substr ($linkd[1],  0,  3),
          "R_TYPE"   => substr ($linkd[1],  3,  1),
          "R_SLOT"   => substr ($linkd[1],  4,  2),
          "R_BOARD"  => substr ($linkd[1],  7,  2),
          "R_SLAB"   => substr ($linkd[1], 10,  1),

          "DRIVER"   => "$linkd[2] $linkd[3]",
          "D_VNODE"  => substr ($linkd[2],  0, -2),
          "D_PORT"   => substr ($linkd[2],  0,  1) . substr ($linkd[2], -1, 1) . substr ($linkd[3], 3, 1),
          "D_RACK"   => substr ($linkd[3],  0,  3),
          "D_TYPE"   => substr ($linkd[3],  3,  1),
          "D_SLOT"   => substr ($linkd[3],  4,  2),
          "D_BOARD"  => substr ($linkd[3],  7,  2),
          "D_SLAB"   => substr ($linkd[3], 10,  1),

          "UTIL"     => $linkd[4],
          "C_BER"    => Math::BigFloat->new($linkd[5]),
          "P_BER"    => Math::BigFloat->new($linkd[6]),

          "SBITS"    => "$linkd[10]$linkd[ 9]$linkd[ 8]$linkd[ 7]",
          "MBITS"    => "$linkd[14]$linkd[13]$linkd[12]$linkd[11]",
          "MAX"      => $linkd[15],
          "SBE"      => $linkd[16],
          "MBE"      => $linkd[17],
          "WE"       => $linkd[18],
          "MB2"      => $linkd[19],
          "MB3"      => $linkd[20],
          "MB4"      => $linkd[21],
          "MB>"      => $linkd[22]
      };

      if(!defined $$href_y{$key}) {

        $$href_y{$key} = {

          "RECEIVER" => "$linkd[0] $linkd[1]",
          "R_VNODE"  => substr ($linkd[0],  0, -2),
          "R_PORT"   => substr ($linkd[2],  0,  1) . substr ($linkd[0], -1, 1) . substr ($linkd[3], 3, 1),
          "R_RACK"   => substr ($linkd[1],  0,  3),
          "R_TYPE"   => substr ($linkd[1],  3,  1),
          "R_SLOT"   => substr ($linkd[1],  4,  2),
          "R_BOARD"  => substr ($linkd[1],  7,  2),
          "R_SLAB"   => substr ($linkd[1], 10,  1),

          "DRIVER"   => "$linkd[2] $linkd[3]",
          "D_VNODE"  => substr ($linkd[2],  0, -2),
          "D_PORT"   => substr ($linkd[2],  0,  1) . substr ($linkd[2], -1, 1) . substr ($linkd[3], 3, 1),
          "D_RACK"   => substr ($linkd[3],  0,  3),
          "D_TYPE"   => substr ($linkd[3],  3,  1),
          "D_SLOT"   => substr ($linkd[3],  4,  2),
          "D_BOARD"  => substr ($linkd[3],  7,  2),
          "D_SLAB"   => substr ($linkd[3], 10,  1),

          "C_BER"    => Math::BigFloat->new("0"),
          "P_BER"    => Math::BigFloat->new("0"),

          "UTIL"     => 999,
          "MAX"      => 0,
          "SBITS"    => "........................................",
          "MBITS"    => "........................................",
          "SBE"      => '.', "MBE"  => '.', "WE"   => '.',
          "MB2"      => '.', "MB3"  => '.', "MB4"  => '.', "MB>" => '.'
        };
      }
    }
  }
  printv($VERB_NORMAL, "#------------------------------------------------------------------------------\n");
}


#------------------------------------------------------------------------------
# Print out usage information
#------------------------------------------------------------------------------
sub print_help_screen {

  print $HELP_SUMMARY;

  exit;
}

#------------------------------------------------------------------------------
# Parse the command line to input user options
#------------------------------------------------------------------------------
sub parse_command_line_options {

  my $result;
  my %optctl;
  my $ber_str;

  %optctl = (

    help            => \$op_print_help,
    verbosity       => \$verbosity,
    t               => \$user_title,
    file            => \$write_output_to_file,
    data            => \$input_data_spec,
    sort            => \$sort_data_spec,
    cfilter         => \$compare_filter_spec,
    ofilter         => \$output_filter_spec,
    format          => \$output_format,
    analyze         => \$analyze_link_groups,

  );

  $Getopt::Long::autoabbrev;

  $result = GetOptions(\%optctl, "<>", \&print_help_screen,

    "help:s",
    "verbosity=s",
    "t=s",
    "file!",
    "data=s",
    "sort=s",
    "cfilter=s",
    "ofilter=s",
    "format=s",
    "analyze!",

  );

  if(!result) {
    return 0;
  }

  #----------------------------------------------------------------------------
  #  REPEAT-OPERATION PARAMETERS
  #----------------------------------------------------------------------------
  if(defined $op_print_help) {
    return $result;
  }

  @input_filename = split /,/, $input_data_spec;

  $sort_data_spec =~ tr/a-z/A-Z/;  # make input case insensitive

  $compare_filter_spec =~ tr/A-Z/a-z/;
  ($compare_filter_sev,$ber_str) = split /,/, $compare_filter_spec;
  $compare_filter_ber = Math::BigFloat->new($ber_str);

  $output_filter_spec =~ tr/A-Z/a-z/;
  ($output_filter_sev, $ber_str) = split /,/, $output_filter_spec;
  $output_filter_ber = Math::BigFloat->new($ber_str);

  return $result;    
}


#------------------------------------------------------------------------------
#  printv(int verbosity_level, char *message); 
#------------------------------------------------------------------------------
sub printv {

  if($verbosity >= $_[0]) {
    print $_[1];
  }

  if($_[0] == $VERB_ERROR) {
    print "SCRIPT ABORTED\n";
    execution_done(1);
  }
}


#-------------------------------------------------------------------------------#
#  check_file_space subroutine
#
#  Supplied args:
#       0 - directory name (derived from the name of the machine to be dumped)
#
#-------------------------------------------------------------------------------
sub check_file_space
{
  my $dir = "$_[0]";
  chomp ($current_dir = `pwd`);
  my $base_dir = ".";
  my $ok_so_far = 1;

  chomp (my $log_date = `date +%y_%a_%b_%d_%k_%M`);
  $log_date =~ s/\s+/0/g;        #  Replace spaces with "0"

  $log_name = "$dir" . "_" . "$log_date";

  $base_dir = "$dir";
  if ((! -d "$base_dir") && ($ok_so_far == 1))  {
    print " Attemping to create directory:  $base_dir.\n";
    if ((mkdir "$base_dir", 0777) != 1)  {
      print "ERROR!  Unable to create $base_dir\n";
      print "  Saving $log_name in $current_dir\n";
      $ok_so_far = 0;
      $base_dir = $current_dir;
    }
    else  {
      chmod(0777, "$base_dir");
    }
  }

  #
  #  Open the output file
  #
  $complete_log_name = "$base_dir/$log_name";
  if ( $write_output_to_file ) {
    open(DMP_LOG, ">$complete_log_name") || die "Unable to open $complete_log_name for writing: $!\n";
#   print "\n Log file will be saved to:  $complete_log_name\n\n";
    select DMP_LOG;
  }
  else {
    open(DMP_LOG, "> /dev/stdout ") || die "Unable to open STDOUT for writing: $!\n";
  }

# print DMP_LOG "\n  $app_name\n\n";
}


#-------------------------------------------------------------------------------
#
#  closes output files and prints completed message
#
#-------------------------------------------------------------------------------
sub execution_done  {

  my $exit_val = $_[0];

  chomp ($timestamp = `date +'%a_%b_%e_%T'`);
  $timestamp =~ s/\s+/0/g; #  Replace spaces with "0"
  printv($VERB_NORMAL, "\n$app_name COMPLETED at $timestamp\n");

  if(defined $user_title) {
    print "\nScript Purpose:  $user_title\n\n";
  }

  if ( $write_output_to_file )
  { select STDOUT;
    printf("\n Log file saved to:  $complete_log_name\n\n");
    close DMP_LOG;
  }

  exit $exit_val;
}



#------------------------------------------------------------------------------
#  Analyze wirestat data
#------------------------------------------------------------------------------
sub analyze_wirestat_data
{
  my $key;
  my $h_ref     = $_[0];
  my $hash_keys = $_[1];
  my $severity;
  my $mesg;
  my ($sbe_str, $mbe_str, $sbe_sev_str, $mbe_sev_str);
  my @sbes     = (0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0);
  my @mbes     = (0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0);
  my @sbes_sev = (0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0);
  my @mbes_sev = (0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,0,0);
  my $noOfLinks = scalar(@$hash_keys);

  foreach $key (@$hash_keys) {
    for($bit=0; $bit<40; $bit++) {
      $severity = substr($$h_ref{$key}{"SBITS"}, $bit, 1),
      $severity =~ tr/A-Z0./a-z``/;
      if(ord($severity) >= ord($compare_filter_sev)) {
        $sbes[$bit]++;
        $sbes_sev[$bit]+=(ord($severity)-96);
      }
      $severity = substr($$h_ref{$key}{"MBITS"}, $bit, 1),
      $severity =~ tr/A-Z0./a-z``/;
      if(ord($severity) >= ord($compare_filter_sev)) {
        $mbes[$bit]++;
        $mbes_sev[$bit]+=(ord($severity)-96);
      }
    }
  }

  $sbe_str = "";
  $mbe_str = "";
  $sbe_sev_str = "";
  $mbe_sev_str = "";
  for($bit=0; $bit<40; $bit++) {
    $sbe_str     .= ($sbes[$bit]) ? sprintf("%x", (($sbes[$bit]*10)+($noOfLinks>>1))/$noOfLinks) : ".";
    $mbe_str     .= ($mbes[$bit]) ? sprintf("%x", (($mbes[$bit]*10)+($noOfLinks>>1))/$noOfLinks) : ".";
    $sbe_sev_str .= ($sbes[$bit]) ? sprintf("%c", (($sbes_sev[$bit]+($sbes[$bit]>>1))/$sbes[$bit])+96) : ".";
    $mbe_sev_str .= ($mbes[$bit]) ? sprintf("%c", (($mbes_sev[$bit]+($mbes[$bit]>>1))/$mbes[$bit])+96) : ".";
  }

  $mesg = ($output_format eq "excel") ?
         sprintf("TOTAL LINKS: %-13d", $noOfLinks):
         sprintf("TOTAL LINKS: %-11d", $noOfLinks);

  $mesg .= sprintf("(percentage of links with severity >= $compare_filter_sev) %10s %10s %10s %10s %10s %10s %10s %10s\n",
            substr($sbe_str, 30, 10), substr($sbe_str, 20, 10), substr($sbe_str, 10, 10), substr($sbe_str,  0, 10),
            substr($mbe_str, 30, 10), substr($mbe_str, 20, 10), substr($mbe_str, 10, 10), substr($mbe_str,  0, 10));

  $mesg .= ($output_format eq "excel") ?
         sprintf("TOTAL LINKS: %-13d", $noOfLinks):
         sprintf("TOTAL LINKS: %-11d", $noOfLinks);
  
  $mesg .= sprintf("    (average severity for failing links) %10s %10s %10s %10s %10s %10s %10s %10s\n",
            substr($sbe_sev_str, 30, 10), substr($sbe_sev_str, 20, 10), substr($sbe_sev_str, 10, 10), substr($sbe_sev_str,  0, 10),
            substr($mbe_sev_str, 30, 10), substr($mbe_sev_str, 20, 10), substr($mbe_sev_str, 10, 10), substr($mbe_sev_str,  0, 10));
  
  printv($VERB_NORMAL, $mesg);


  if($output_format eq "excel") {
    printv($VERB_NORMAL, "_____________________________________________________________________________________________________________________________________________________________________________\n\n");
  } else {
    printv($VERB_NORMAL, "_________________________________________________________________________________________________________________________________________________________________________\n\n");
  }
}

