#
# ip34_membist.tcl
#
# KK 
# 99/06/17
#
# Mod history:
# 1999/07/16 - added BIST_READ and error ADDR and DATA dumping
# 1999/9/9   - added DLL enable on DIMM's
# 1999/09/30 - changed initialization for each bank and dimm
# 2001/08/24 - KK - copied from ip35_membist
# 2002/07/09 - KK - copied from ip35_membist
#
# runs the MEMBIST on the BEDROCK
# 
#############################################################################
#
#   (C) COPYRIGHT SILICON GRAPHICS, INC.
#   UNPUBLISHED PROPRIETARY INFORMATION
#   ALL RIGHTS RESERVED UNDER THE COPYRIGHT LAWS OF THE UNITED STATES.
#
#   Portions of this software may still be in development.  The existence
#   of the software still in development is not a commitment of actual
#   release or support by Silicon Graphics, Inc.  Silicon Graphics, Inc.
#   assumes no liability for any damages resulting from attempts to use any
#   product not officially released.  If it is released, the final form of
#   the product and the time of official release is at the discretion of
#   Silicon Graphics, Inc.      
#   
#############################################################################
#   
#
#
# FORMAT OF THE MEMBIST TDR - BSDL VERSION:
#  the fields are broken up as follows:
#
#  |-+---+----------+----------+--+----+----------+-|
#  |1| 3 |   32     |   32     |2 | 4  | 32       |1|
#  |-+---+----------+----------+--+----+----------+-|
#   |  |     |          |       |   |     |        |
#   |  |     |          |       |   |     |        +reserved for resync
#   |  |     |          |       |   |     +MB: Data Lo
#   |  |     |          |       |   +MB: Data Hi
#   |  |     |          |       +MB:Addr          
#   |  |     |          +DB
#   |  |     +IB Data (see below)
#   |  +IB Addr (see below)
#   +reserved for resync
#
# the IB Data breakout depends on the IB Addr; however, the scan tool
# doesn't recognize the IB Data breakout

#  IB: MEMORY CONFIG
#
#          +---+----+-+-+--+----+-+-+--+----+-+-+--+----+-+-+--+ 
#   TDI -> |000|3210|0|0|10|3210|0|0|10|3210|0|0|10|3210|0|0|10|
#          +---+----+-+-+--+----+-+-+--+----+-+-+--+----+-+-+--+ 
#            |    |  | |  |   |  | |  |   |  | |  |   |  | |  |
#            |    |  | |  |   |  | |  |   |  | |  |   |  | |  +Dimm0 bnkEnab
#            |    |  | |  |   |  | |  |   |  | |  |   |  | +reserved
#            |    |  | |  |   |  | |  |   |  | |  |   |  Dimm0DRAM Width
#            |    |  | |  |   |  | |  |   |  | |  |   +Dimm0BankSize

#* IB: DIMM CONFIG                                                            */
#*     WRITE                                                                  */
#*                                                                            */
#*         3 3 3    2 1 1   1 1 1                                             */
#*         4 2 1    0 9 8   6 5 4                  0                          */
#*        +---+------+-+-----+-+--------------------+                         */
#* TDI -> |110|  0   | |     |0|                    | -> TDO                  */
#*        +---+------+-+-----+-+--------------------+                         */
#*         ^          ^   ^          ^                                        */
#*         |          |   |          |                                        */
#*         |          |   DIMM #     DIMM Mode bits                           */
#*         |          |             (0x002b memory, 0x0029 directory)         */
#*         |          Directory                                               */
#*         |          DIMM sel.                                               */
#*         |                                                                  */
#*         Address select (3'b110)                                            */
#*                                                                            */
#* IB: REFRESH CTL                                                            */
#*     WRITE                                                                  */
#*         3 3 3 3          1 1                                               */
#*         4 2 1 0          2 1                    0                          */
#*        +---+-+------------+----------------------+                         */
#* TDI -> |010| |        0   |                      | -> TDO                  */
#*        +---+-+------------+----------------------+                         */
#*         ^   ^                     ^                                        */
#*         |   |                     |                                        */
#*         |   Refresh enable     Refresh rate                                */
#*         |                                                                  */
#*         Address select (3'b010)                    
#
#*3:0   BIST_DATA    RW    4-bit initial BIST data pattern                    */
#*      WRITE                                                                 */
#*         3 3 3                                                              */
#*         4 2 1                        7 6 5 4 3  0                          */
#*        +---+------------------------+-+-+-+-+----+                         */
#* TDI -> |100|         0              | | | | |    | -> TDO                  */
#*        +---+------------------------+-+-+-+-+----+                         */
#*         ^                            ^ ^ ^ ^   ^        a:BIST_NIBBLE      */
#*         |                            | | | |   |        b:BIST_BYTE        */
#*         |                            | | | |   Pattern  c:BIST_CYCLE       */
#*         |                            | c b a                               */
#*         Address select (3'b100)      BIST_WRITE                            */
#*                                                                            */
#* IB: BIST CTL                                                               */
#*     WRITE                                                                  */
#*         3 3 3               1                                              */
#*         4 2 1               0 9 8 7 6 5 4 3 2 1 0                          */
#*        +---+-----------------+---+-+---+-+-+-+-+-+                         */
#* TDI -> |011|         0       |   | |   | |0| | | | -> TDO                  */
#*        +---+-----------------+---+-+---+-+-+-+-+-+                         */
#*         ^                              ^  ^ ^ ^ ^                          */
#*         |                              |  | | | |                          */
#*         |                              |  | | | BIST_START                 */
#*         |                              |  | | BIST_STOP                    */
#*         |                              |  | BIST_RST                       */
#*         |                              BANK_NUM                            */
#*         Address select (3'b011)      Dimm_NUM                              */
#*                                                                            */
#L
#* IB: BIST STATUS                                                            */
#*     READ                                                                   */
#*         3 3                                                                */
#*         4 2                                     0                          */
#*        +---+---------------------------------+-+-+                         */
#* TDI -> |x01|         0                       | | | -> TDO                  */
#*        +---+---------------------------------+-+-+                         */
#*         ^                                     ^ ^                          */
#*         |                                     | |                          */
#*         Address select (3'bx01)               | BIST_PASSED                */
#*                                               |                            */
#*                                               BIST_COMPLETE                */
#* IB: BIST_STATUS
#*     READ 
#*   Addr 3'bx11  BIST_AB_ERR_ADDR 63-32
#*   Addr 3'bx10  BIST_AB_ERR_ADDR 31-00

source $env(SCAN_PATH_DATA)/diags/generic/isttools.tcl

############################
# Target Parameters
############################
set mod 001c01
set brd NODE0
set chip BR

############################
# diagnostic parameters
############################
set bistCycle   0	;# toggle each word
set bistByte    0	;# toggle each byte
set bistNibble  0	;# toggle each nibble
set bistPattern 0x4	;# Data pattern
set firstDimm   0
set lastDimm    0
set firstBank   1
set lastBank    1
set stopOnError 0
set restartOnError 1		;# restarts BIST readback instead of starting over

set iterLimit   2
set errorLimit   4    ;# 0 keep running 
set iterPattern 0    ;# pattern is iteration count ?

set onlyWrite   0		;# for looping on writes
set onlyRead    0    ;# for looping on reads

set outputMode  3	   ;# 0: don't print anything, 1 print pass counts \
                        2: print error info , 3: print everything


########################################################

set iterCount 0
set numPasses 0
set numFails  0
set numErrors 0
set failed    0

set db 0
set mbAddr 0
set mbDataHi 0
set mbDataLo 0
set resync 0


########################################################
############################
# Config and Select 
############################

SEL MOD $mod
SEL BRD $brd
SEL CHIP $chip

RESET -H
RESET

INST GMR
GOINST

INST MEMORY_BIST_TDR
GOINST



# initialize memory DIMMS
set ibAddr 0x6        ;# DIMM_CONFIG Addr
for {set ibDimm $firstDimm} {$ibDimm <= $lastDimm} {incr ibDimm 1} {
	for { set bankNum $firstBank} { $bankNum <= $lastBank} { incr bankNum 1} { 
		puts "initializing memory DIMM $ibDimm bank $bankNum"
	
		set ibDirDimm 0x0
		set ibDimmMode 0x2000 ;# access SDRAM EMRS ctl reg: enable DLL
		set ibData [ expr ($ibDirDimm << 19) | ($ibDimm << 17) | \
		                                       ($bankNum << 16) | $ibDimmMode ]
		DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
		GODATA
	
	# SDRAM MRS Ctl Reg: CAS latency = 2, interleaved burst, latency = 8
		set ibDimmMode 0x2b 
		set ibData [ expr ($ibDirDimm << 19) | ($ibDimm << 17) | \
		                                       ($bankNum << 16) | $ibDimmMode ]
		DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
		GODATA

	#initialize directory DIMMS
		puts "initializing directory DIMM $ibDimm bank $bankNum"
		set ibDirDimm 1
	
		set ibDimmMode 0x2000 ;# access SDRAM EMRS ctl reg: enable DLL
		set ibData [ expr ($ibDirDimm << 19) | ($ibDimm << 17) | \
		                                       ($bankNum << 16) | $ibDimmMode ]
		DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
		GODATA
	
	# SDRAM MRS Ctl Reg: CAS latency = 2, interleaved burst, latency = 2
		set ibDimmMode 0x29
		set ibData [ expr ($ibDirDimm << 19) | ($ibDimm << 17) | \
		                                       ($bankNum << 16) | $ibDimmMode ]
		DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
		GODATA
	}
}


# configure memory 
puts ""
puts "Configuring memory - bank enables and DRAM widths"
set dimm3BankSize    1  ;# 1 = 64 MB
set dimm3DRAMWidth   0  ;# x8 DRAM
set dimm3BankEnable  3  ;# both
set dimm2BankSize    1  ;# 1 = 64 MB
set dimm2DRAMWidth   0  ;# x8 DRAM
set dimm2BankEnable  3  ;# both
set dimm1BankSize    1  ;# 1 = 64 MB
set dimm1DRAMWidth   0  ;# x8 DRAM
set dimm1BankEnable  3  ;# both
set dimm0BankSize    2  ;# 1 = 64 MB
set dimm0DRAMWidth   0  ;# x8 DRAM
set dimm0BankEnable  3  ;# both
set ibAddr 0
set ibData [ expr \
 ($dimm3BankSize << 28) | ($dimm3DRAMWidth << 27) | ($dimm3BankEnable << 24) |\
 ($dimm2BankSize << 20) | ($dimm2DRAMWidth << 19) | ($dimm2BankEnable << 16) |\
 ($dimm1BankSize << 12) | ($dimm1DRAMWidth << 11) | ($dimm1BankEnable <<  8) |\
 ($dimm0BankSize <<  4) | ($dimm0DRAMWidth <<  3) | ($dimm0BankEnable )  ]

DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
GODATA

puts ""
puts "Configuring memory - memory timing parameters"
set rpConfig      0   ;# 3memory cycles
set rcdConfig     0
set mbNegEdge     7
set sampleTime    2
set delayMuxSel   2
set phaseDelay    1
set dbNegEdge     1
set cpuProtIgnore 0
set ioProtEn      0
set ccEnable      1
set dimm0Sel      0

set ibAddr 1
set ibData [ expr \
 ($rpConfig << 29) | ($rcdConfig << 28) | ($mbNegEdge << 24) | \
 ($sampleTime << 20) | ($delayMuxSel << 18) | ($phaseDelay << 17) | \
 ($dbNegEdge << 16) | ($cpuProtIgnore << 15) | ($ioProtEn << 14) | \
 ($ccEnable << 12) |($dimm0Sel << 32) ]
DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
GODATA

# Enable Refresh
puts ""
puts "Enabling Refresh"
set ibAddr 0x2   ;#refresh addr
set refreshRate 0x100      ;# default is 0x800
puts "refresh rate is $refreshRate"
set ibData [ expr (1 << 31) | $refreshRate ]
DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
GODATA


while { $iterLimit == 0 || $iterCount != $iterLimit } {

#########################
# WRITE OUT
#########################

if { $onlyRead == 0 } {
# Write to BIST_DATA: BIST_WRITE | BIST[NIBBLE|BYTE|CYCLE] | Pattern 
if { $outputMode > 2 } {
	puts ""
	puts "Writing data pattern to BIST_DATA"
}
set ibAddr 0x4    ;# BIST_DATA
set bistWrite   1
if { $iterPattern != 0 } { set bistPattern $iterCount }
set ibData [ expr ($bistWrite << 7) | ($bistCycle << 6)  | \
                  ($bistByte << 5)  | ($bistNibble << 4) | \
                  ($bistPattern) ]
DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
GODATA

# Write to BIST_CTL: BIST-START on Bank 0 DIMM 0 (for now)
for { set curDimm $firstDimm} { $curDimm <= $lastDimm } { incr curDimm} {
	for {set curBank $firstBank} { $curBank <= $lastBank } { incr curBank} {
	if { $outputMode > 2} {
		puts "Writing to BIST_CTL- dimm: $curDimm bank: $curBank "
	}
	set ibAddr 0x3    ;# BIST_CTL
	set bistStart 1
	set bistStop  0
	set bistRst   0
	set bankNum   [ expr ( $curBank & 1) ]
	set dimmNum   $curDimm
	set ibData [ expr ($dimmNum << 5) | ($bankNum << 4) \
	                | ($bistRst << 2) | \
	                  ($bistStop << 1) | $bistStart ] 
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA


# Poll on IB STATUS 
set ibAddr 0x5
set ibData 0
DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
GODATA

set val 0
set writeDone 0
while { $writeDone == 0 } {
	#scan in status DR first, then poll
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "IBData" val
	if { [ expr ($val & 0x1)]  != 0 } { 
		set writeDone 1 
	}
	if { $outputMode > 2} {
		puts "BIST_WRITE poll: IB data is $val"
	}
}

if { $val != 3 } { 
	if { $outputMode > 1 } { 
		puts "************* Memory BIST Write error val = $val "
		puts "***** exiting script, enter your own commands"
	}
	return 
} else {
	if { $outputMode > 2 } {
		puts "Write completed successfully for dimm: $curDimm  bank: $curBank"
	}
}
	} ;# for curBank
} ;# for curDimm

} ;# if onlyRead

#########################
# read back
#########################

if  { $onlyWrite == 0 } {
if {$outputMode > 2 } {
	puts ""
	puts "Reading back data"
}
# Write to BIST_DATA: !BIST_WRITE | BIST[NIBBLE|BYTE|CYCLE] | Pattern
if {$outputMode > 2 } {
	puts ""
	puts "Writing data pattern to BIST_DATA"
}
set ibAddr 0x4    ;# BIST_DATA
set bistWrite   0
if { $iterPattern != 0 } { set bistPattern $iterCount }
set ibData [ expr ($bistWrite << 7) | ($bistCycle << 6)  | \
                  ($bistByte << 5)  | ($bistNibble << 4) | \
                  ($bistPattern) ]
DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
GODATA

set errCount 0

for { set curDimm $firstDimm} { $curDimm <= $lastDimm } { incr curDimm} {
	for {set curBank $firstBank} { $curBank <= $lastBank } { incr curBank} {
# Write to BIST_CTL: BIST-START on Bank 0 DIMM 0 (for now)
	if {$outputMode > 2 } {
		puts "Writing to BIST_CTL- dimm: $curDimm bank: $curBank "
	}
	set ibAddr 0x3    ;# BIST_CTL
	set bistStart 1
	set bistStop  0
	set bistRst   0
	set bankNum   [ expr ( $curBank & 1) ]
	set dimmNum   $curDimm
	set ibData [ expr ($dimmNum << 5) | ($bankNum << 4) | \
	                  ($bistRst << 2) | \
	                  ($bistStop << 1) | $bistStart ] 
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA


set testDone 0

while { $testDone == 0 } {
set ibAddr 0x5
set ibData 0
DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
GODATA

set pollDone 0
set val 0
#scan in status DR first, then poll
while { $pollDone == 0 } {
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "IBData" val
	if { $val != 0 } { 
		set pollDone 1 
	}
	if {$outputMode > 2 } {
		puts "BIST_READ poll: IB data is $val"
	}
}

if { $val != 3 } { 
if { $outputMode > 1 } {
	puts "************* Memory BIST Read error: val = $val "
## dump error info	
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "DBData" val
	puts "DB data = $val "

######
## Now, do I have to do an extra scan or will the proper data get
## latched (UPDATEd) after the CAPTURE?
######
puts "Memory Data bits 35 - 0 "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataHI" val
	puts "MB data Hi  = $val "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataLO" val
	puts "MB data Lo  = $val "

	set mbAddr 0x1
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA
	GODATA
puts "Memory Data bits 71 - 36 "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataHI" val
	puts "MB data Hi  = $val "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataLO" val
	puts "MB data Lo  = $val "

	set mbAddr 0x2
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA
	GODATA
puts "Memory Data bits 107 - 72 "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataHI" val
	puts "MB data Hi  = $val "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataLO" val
	puts "MB data Lo  = $val "

	set mbAddr 0x3
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA
	GODATA
puts "Memory Data bits 143 - 108 "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataHI" val
	puts "MB data Hi  = $val "
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "MBDataLO" val
	puts "MB data Lo  = $val "

puts "BIST Error Address 63-32"
	set ibAddr 0x3
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA
	GODATA
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "IBData" val
	puts "BIST_AB_ERR_ADDR 63-32  = $val "
	set xxx [ expr ($val & 0x7fff) ]
	puts "BE_RAS_ADDR = $xxx"
	if { $val & 0x10000 } { ;# Memory error 
		puts "Memory Data error"
	}
	if { $val & 0x20000 } { ;# Directory error 
		puts "Directory Data error"
	}
puts "BIST Error Address 31-00"
	set ibAddr 0x2
	DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
	GODATA
	GODATA
	InfoFldVal [ INFO CHIP $chip DATA MEMORY_BIST OUT ] "IBData" val
	puts "BIST_AB_ERR_ADDR 31-00  = $val "
	set xxx [ expr ($val & 0x7fff) ]
	puts "BE_DB_CAS_ADDR = [format "%#x" $xxx] "
	set xxx [ expr ($val & 0x7fff0000) >> 16 ]
	puts "BE_MB_CAS_ADDR = [format "%#x" $xxx ] "
} ;# outputMode > 1

	incr numErrors
	if { $outputMode > 1 } { puts "Error count: $numErrors" }
	if { $numErrors == $errorLimit } { 
		puts "Error limit reached:  "
	puts "Iteration: $iterCount  Pass: $numPasses  Fail: $numFails  Errors: $numErrors"
		puts "Script stopped: enter further commands. "
		return 
	}
	set failed 1

	set pollDone  0

	if { $stopOnError == 1 } { 
		set testDone 1
	} else {

	if { $restartOnError != 0 } {
	# restart the BIST

		if {$outputMode > 2 } { puts "restarting BIST" }
		set ibAddr 0x3    ;# BIST_CTL
		set bistStart 1
		set bistStop  0
		set bistRst   0
		set bankNum   [ expr ( $curBank & 1) ]
		set dimmNum   $curDimm
		set ibData [ expr ($dimmNum << 8) | ($bankNum << 4) |  	\
                    ($bistRst << 2 ) | ($bistStop << 1) | $bistStart ] 
		DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
		GODATA

	} else {
		set testDone 1
		if { $outputMode > 2 } { puts "resetting BIST" }
		set ibAddr 0x3    ;# BIST_CTL
		set bistStart 0
		set bistStop  0
		set bistRst   1
		set bankNum   [ expr ( $curBank & 1) ]
		set dimmNum   $curDimm
		set ibData [ expr ($dimmNum << 8) | ($bankNum << 4) |  \
  	                  ($bistRst << 2 ) | ($bistStop << 1) | $bistStart ] 
		DATA $resync.$ibAddr.$ibData.$db.$mbAddr.$mbDataHi.$mbDataLo.$resync
		GODATA
	}

	} ;# restartOnError

} else {    ;# if {val != 3}
	puts "readback completed successfully for dimm: $curDimm  bank: $curBank"
	set testDone 1
}


} ;# while { $testDone == 0 }


	} ;# for curBank
} ;#for curDimm

} ;# if $onlyWrite == 0

if { $failed == 1 } {
	incr numFails
	set failed 0
} else {
	incr numPasses
}

incr iterCount
if { $outputMode != 0 } {
	puts "Iteration: $iterCount  Pass: $numPasses  Fail: $numFails  Errors: $numErrors"
	if { $numErrors != 0 && $stopOnError != 0 } {
		puts "script stopped, enter additional commands"
		return;
	}
}


} ;# while iterCount < iterLimit
puts ""
puts "Script done, enter additional commands"


