#
# isttools.tcl
#
# Kurt Kermes
# 99/07/08
#
# Generic tools that will help the user manipulate fields and so forth.

#
#############################################################################
#
#   (C) COPYRIGHT SILICON GRAPHICS, INC.
#   UNPUBLISHED PROPRIETARY INFORMATION
#   ALL RIGHTS RESERVED UNDER THE COPYRIGHT LAWS OF THE UNITED STATES.
#
#   Portions of this software may still be in development.  The existence
#   of the software still in development is not a commitment of actual
#   release or support by Silicon Graphics, Inc.  Silicon Graphics, Inc.
#   assumes no liability for any damages resulting from attempts to use any
#   product not officially released.  If it is released, the final form of
#   the product and the time of official release is at the discretion of
#   Silicon Graphics, Inc.
#
#############################################################################
#

#
# Mod history:
#  1999/10/04 - KK - Added Break64 - breaks up a 64-bit hex int 
#                  into 2 32-bit hex ins
#  2000/12/05 - KK - Fixed Break64 to output 0x0 in Hi word if
#                    value <= 10 bits
#                  - Fixed Build64 to prepend 0x0 if Hi word == 0x0
#  2001/02/21 - KK - Added globals for select mechanism

# 
# proc InfoFldVal
#
# KK
# 99/07/08
# 
# Given an INFO string and a field to search, returns value of matching
# field. 
#
# the regexp line  will return a 1 if expression is matched
#

proc InfoFldVal { string field val } {
	upvar $val lval
	global mod modid brd brdidx chip
	set status [ regexp "(^|\n| *)$field *:(\[^\n\]+)\n" $string match beg lval ]
	set val [ string trim $lval ]
	return $status
}

#
#############################################################################
# proc Break64
# KK 99/10/04
# 
# Given a string variable which holds a 64-bit hexadecimal quantity
# break the quantity into two 32-bit hex values.
# int32Hi is the high 32 bits, and int32Lo is the low 32 bits.
# the second and third variables need not be defined before 
# calling this proc.
#############################################################################
#

proc Break64 { int64 int32Hi int32Lo } {
	upvar int32Hi lHi int32Lo lLo 
	global mod modid brd brdidx chip
	set lLo 0
	set lHi 0
#puts "int64   is $int64"
#puts "int32Hi is $int32Hi"
	set int64 [ string trim $int64 ]
	set strlen [ string length $int64 ]
#puts "strlen is $strlen"
	if { $strlen > 9 } {
		set lLo [ string range $int64 [expr ($strlen-8)] $strlen ]
	} else {
		set lLo [ string range $int64 2 $strlen ]
	}
	if { [ string match "0x*" $lLo ] == 0 } {
		set lLo 0x$lLo
	} else {
		 set int32Hi 0
	}
	if { $strlen > 10 } {
		set lHi [ string range $int64 0 [expr ($strlen-9) ] ]
	} else {
		set lHi 0x0
	}
#puts "lHi is $lHi"
#puts "lLo is $lLo"

}


#
#############################################################################
# Build64
# KK 2000/07/24
#
# Given two 32-bit hex strings, combine the upper bits and lower
# 32 bits into a single 64-bit hex string.
#############################################################################
#

proc Build64 { intHi int32Lo } {
set l64 ""
#evaluate intHi and int32Lo into hex strings
set int32Lo [ format "%08x" $int32Lo ]
set intHi [ format "%#x" $intHi ]
#cat intHi and int32Lo into int64 
if { $intHi == 0x0 } {
	set intHi 0x
}
append l64 $intHi $int32Lo
#puts "l64 is $l64"
return $l64

}
