#
# ip35_debug.tcl
#
# Kurt Kermes
# 99/11/30
#
# selects which DEBUG_PORT data goes out the MICTOR connector
# This script works on Bedrock
#
# Mod history: 
#  2001/03/13 - KK - Changed RESET sequence and MOD select
#  2001/08/23 - KK - renamed BRD/CHIP
#
#############################################################################
#
#   (C) COPYRIGHT SILICON GRAPHICS, INC.
#   UNPUBLISHED PROPRIETARY INFORMATION
#   ALL RIGHTS RESERVED UNDER THE COPYRIGHT LAWS OF THE UNITED STATES.
#
#   Portions of this software may still be in development.  The existence
#   of the software still in development is not a commitment of actual
#   release or support by Silicon Graphics, Inc.  Silicon Graphics, Inc.
#   assumes no liability for any damages resulting from attempts to use any
#   product not officially released.  If it is released, the final form of
#   the product and the time of official release is at the discretion of
#   Silicon Graphics, Inc.
#
#############################################################################
#


#
# DEBUG_SEL 5:3 decodes
#
set LB  0
set XB  1
set MD  2
set NI  3 
set PI1 4
set PI0 5
set II  6

#
# DEBUG_SEL 2:0 decodes
#

# LB

set LB_RQ      0
set LB_RP      1
set LB_LPQRQ0  2
set LB_LPQRQ1  3
set LB_LOQRP0  4
set LB_LOQRP1  5
set LB_FSM     6
set LB_PIO     7


# XB

set XB_DATA    0
set XB_ARB     1
set XB_LQ      2
set XB_IQ      3
set XB_NQ      4
set XB_MOQ     5
set XB_PQ1     6
set XB_PQ0     7


# MD needs IB_DEBUG bit set

# NI

set NI_REG_MISC     0
set NI_RECV0        1
set NI_RECV1        2
set NI_SEND0        3
set NI_SEND1        4
set NI_LLP          5
set NI_LLPREG       6
set NI_RECV2        7


# PI

set PI_REPLY_STATUS 0
set PI_WRRDY        1
set PI_CTRL         2
set PI_PIQPOQ       3
set PI_IRB          4
set PI_RRB          5
set PI_WRB          6
set PI_FCAM         7


# II needs a SUBMENU field set in IDBSS reg


############################################################################
# script parameters
#
# debugPort: the active port A - H  ( DEBUG_ADDR bits 9:6 )
#  this also doubles as the source port is you're specifying routing
#  use $portX for debugging 
#  use $fromX when specifying a port source 
# debugRouteTo  : specify xbar route destination
#    SET THIS TO 0 if not using this 
# debugAddr: defined in the spec for specific signal
#            
############################################################################
set debugBlock   $PI0
set debugAddr    $PI_REPLY_STATUS   
############################################################################
############################################################################

set mod $mod
set brd NODE
set chip BR

SEL MOD  $mod
SEL BRD  $brd
SEL CHIP $chip

RESET -H
RESET

set  debugAddr [ (1 << 6 ) | expr ($debugBlock << 3)  | ($debugAddr) ]
INST DEBUG_SEL_TDR
GOINST

DATA $debugAddr
GODATA

puts "debug port is set up"
puts "Enter additional script commands if you want "
puts "INST DEBUG_DATA_TDR  "
puts "GOINST "
puts "GODATA"
puts "might be a nice sequence. "
