# c_llp.tcl
#
# KK
# 1999/10/04
#
# Mod history:
#  2001/03/13 - KK - modified for new module select
#  2001/08/23 - KK - Changed BRD/CHIP names
#
# CBrick LLP test script for NI/II port
#
#
#############################################################################
#
#   (C) COPYRIGHT SILICON GRAPHICS, INC.
#   UNPUBLISHED PROPRIETARY INFORMATION
#   ALL RIGHTS RESERVED UNDER THE COPYRIGHT LAWS OF THE UNITED STATES.
#
#   Portions of this software may still be in development.  The existence
#   of the software still in development is not a commitment of actual
#   release or support by Silicon Graphics, Inc.  Silicon Graphics, Inc.
#   assumes no liability for any damages resulting from attempts to use any
#   product not officially released.  If it is released, the final form of
#   the product and the time of official release is at the discretion of
#   Silicon Graphics, Inc.
#
#############################################################################
#


# Format of each LLPTDR:
# Each port contains a 58-bit entry with the following fields: 
# +-+-+-+-+---+-+-+--------------------+--------+--------------------+-+
# |X|M|F|C|FLT|E|V|DATA[20] (RO)       |MASK[8] |SEED[20] (WO)       |X|
# +-+-+-+-+---+-+-+--------------------+--------+--------------------+-+
#  | | | |  |  | |                                                    
#  | | | |  |  | TestValid : indicates valid packet has been captured (RO)
#  | | | |  |  TestCBerror : indicates captured packet had CRC error (RO) 
#  | | | | TestFlit[3] : selects which 20-bit flit to capture (WO)
#  | | | TestClear : clears TestValid & TestCBerr; arms receiver (WO)
#  | | TestErrorCapture : enables capture of 1st micropacket w/ error (WO)
#  | MODE : update mode (0=load parameters, 1=initiate test) (WO)
#  |  MASK determines which flit to invert 
#  + resync latch - also at other end   

############################################################################
# parameters
############################################################################
set flit   0        ;# which flit to select
set mask   0        ;# which flit(s) to invert
set seed   0x12345  ;# data to be sent
set errCap 1        ;# capture 1st uPkt with error
set llp    CORE     ;# either CORE or IO for NI or II testing

set stopOnError   0 ;# 
set ignoreCRCErrs 1 ;# don't test the CRC error flag
set compareData   1 ;# are we just looping or do we want to check data?
set iterLimit     5 ;# set to 0 for infinite running 
set errorLimit    0 ;# set to 0 for infinite errors to be tallied

## the following *Cycle features are optional if you don't want to
# do fixed patterns
set patternCycleLen   1	   ;# set to > 0 to use a list of pattern seeds
set flitCycleLen      4    ;# set to > 0 to cycle thru a list of flits
set maskCycleLen      0    ;# set to > 0 to cycle thru a list of masks

set outputMode        7    ;# 0: don't print anything, 1 print pass counts \
                              2: print error info , 3: print both
set clearRetryLimit   3	   ;# 0 to keep retries
set testRetryLimit    3	   ;# 0 to keep retries

# if you want to run a sequence of patterns, masks, flits, set these
# as well as the appropriate *CycleLen variables:

#set patternList { 0x0 0x55555 0xAAAAA 0xFFFFF 0x33333 0xCCCCC }
set patternList { 0x0 0x1 0x2 0x4 0x8 0x10 0x20 0x40 0x80 0x100 0x200 \
                  0x400 0x800 0x1000 0x2000 0x4000 0x8000             \
                  0x10000 0x20000 0x40000 0x80000 }
set flitList { 0 1 2 3 4 5 6 7 }
set maskList { 0x1 0x2 0x4 0x8 0x10 0x20 0x40 0x80 }

############################################################################
# end of user-settable parameters
############################################################################

set iterCount 0
set numPasses 0
set numFails  0
set valid 0
set cbErr 0
set retryCount 0
set totalRetryCount 0 
set testFailed 0
set oldseed 0

set patternCycleIdx 0
set flitCycleIdx 0
set maskCycleIdx 0

set mod $mod   
set brd NODE
set chip BR

source $env(SCAN_PATH_DATA)/diags/generic/isttools.tcl

SEL MOD  $mod
SEL BRD  $brd
SEL CHIP $chip

RESET -H			;# the -H seems 2 b required to load insts ?????
RESET

INST GMR    
GOINST

if { [ string compare $llp "CORE" ] == 0 } {
	INST LLPTEST_CORE_TDR
} else {
	INST LLPTEST_IO_TDR
}

INST LLPTEST_CORE_TDR
GOINST

set X      0
set mode   1
set clear  1        ;# clear 

set clr      $X.0.$errCap.$clear.$flit.0.0.0.$mask.$seed.$X
set idle     0.0.0.0.0.0.0.0.0.0.0
set go       0.1.0.0.0.0.0.0.0.0.0

if { $patternCycleLen != 0 } {
	set seed [ lindex $patternList $patternCycleIdx ]
	set oldseed $seed
	incr patternCycleIdx
	if { $patternCycleIdx == $patternCycleLen } { set patternCycleIdx 0 }
}

if { $flitCycleLen != 0 } {
	set flit [ lindex $flitList $flitCycleIdx ]
	incr flitCycleIdx
	if { $flitCycleIdx == $flitCycleLen } { set flitCycleIdx 0 }
}

if { $maskCycleLen != 0 } {
	set mask [ lindex $maskList $maskCycleIdx ]
	incr maskCycleIdx
	if { $maskCycleIdx == $maskCycleLen } { set maskCycleIdx 0 }
}

set clrLoad  $X.0.$errCap.1.$flit.0.0.0.$mask.$seed.$X

# Quiet and arm
# verify that port is clear by assuring TestValid  & CBErr is clear

while { $valid == 0 || \
           $cbErr == 0 && $retryCount != $clearRetryLimit } { 
	if { $outputMode & 4 } {
		puts "SCAN: seed = $seed  mask = $mask   flit = $flit"
	}

	DATA $clrLoad
	GODATA
	GODATA
	InfoFldVal [ INFO CHIP $chip DATA LLPTEST_$llp OUT ] "TestValid" valid
	InfoFldVal [ INFO CHIP $chip DATA LLPTEST_$llp OUT ] "RcvdCRCError" cbErr
	if { $valid == 0 && $cbErr == 0 } { 
		break 
	} else {
		incr retryCount
		if { $outputMode & 4 } {
			puts "Retrying the clear: valid = $valid; cbErr is $cbErr"
		}
	}
}

if { $retryCount == $clearRetryLimit } {
	puts "Error: clear retry limit reached - at least one of the flags is set"
	puts "ValidFlag = $valid  ; RcvdCRCError = $cbErr"
	puts "Test done - enter additional commands."
	return
}

set retryCount 0
set clrLoadOld $clrLoad

#
# main test loop
#
while { ($iterCount < $iterLimit) } { 
	DATA $go
	GODATA

	# Scan in new data values and get back results from receiver
	# if you didn't get back a TestValid, just start test all over again --
	# Put back the old data and continue - you'll never see testValid.

	if { $patternCycleLen != 0 } {
		set seed [ lindex $patternList $patternCycleIdx ]
		incr patternCycleIdx
		if { $patternCycleIdx == $patternCycleLen } { set patternCycleIdx 0 }
	}
	
	if { $flitCycleLen != 0 } {
		set flit [ lindex $flitList $flitCycleIdx ]
		incr flitCycleIdx
		if { $flitCycleIdx == $flitCycleLen } { set flitCycleIdx 0 }
	}
	
	if { $maskCycleLen != 0 } {
		set mask [ lindex $maskList $maskCycleIdx ]
		incr maskCycleIdx
		if { $maskCycleIdx == $maskCycleLen } { set maskCycleIdx 0 }
	}

	set clrLoad $X.0.$errCap.1.$flit.0.0.0.$mask.$seed.$X
	DATA $clrLoad
	GODATA

	if { $outputMode & 4 } {
		puts " [ INFO CHIP $chip DATA LLPTEST_$llp OUT ] "
	}

# Analyze the data scanned out by the previous load
# if we need to retry, we'll put in the previous data

	InfoFldVal [ INFO CHIP $chip DATA LLPTEST_$llp OUT ] "TestValid" valid
	if { $valid == 0 } {
		incr totalRetryCount
		incr retryCount
		if { $retryCount == $testRetryLimit } { ;# error
			set testFailed 1
			incr numFails
			if { $outputMode & 1 } {
				puts "Error: Test Retry Limit reached -- "
				puts "Seed: $seed   Mask: $mask  FlitSelect: $flit"
			}
			if { $stopOnError > 0 } {
				puts "Test done - enter additional commands."
				return
			}
			set retryCount 0
		} else {
			DATA $clrLoadOld
			GODATA
			continue
		}
	} else {
		set retryCount 0
# check for errors
		if { $ignoreCRCErrs == 0 } {
			InfoFldVal  \
		    [ INFO CHIP $chip DATA LLPTEST_$llp OUT ] "RcvdCRCError" cbErr
			if { $cbErr == 1 } {
				set testFailed 1
				incr numFails
				if { $outputMode & 1 } {
					puts "Error: Received CRC Error"
					puts "Seed: $seed   Mask: $mask  FlitSelect: $flit"
				}
			}
		}
		if { $compareData == 1 } {
			# calculate expData
			if { [ expr ( $mask >> $flit ) ]  == 1 } {
				set expData [ expr (0xfffff - $oldseed) ] 
				set expData [ format "0x%x" $expData ]
			} else {
				set expData $oldseed
			}
			InfoFldVal [ INFO CHIP $chip DATA LLPTEST_$llp OUT ] "Data" actData
			if { $actData != $expData } {
				set testFailed 1
				incr numFails
				if { $outputMode & 2 } {
					puts "Data miscompare: Act: $actData   Exp: $expData"
				}
			}
		}
		if { $numFails > 0 && $stopOnError > 0 } {
			puts "Test done - enter additional commands."
			return
		}
		set retryCount 0
	} ;# if { $valid == 0 }

# print out current pass/fail counts
	if { $testFailed == 0 } {
		incr numPasses
	} else {
		set testFailed 0
	}
	incr iterCount
	if { $outputMode & 1 } {
		puts -nonewline "Iteration $iterCount --  "
		puts -nonewline "Passes: $numPasses  Fails: $numFails  "
		puts "   numRetries:  $totalRetryCount "
		puts " "
	}

	if { $outputMode & 4 } {
		puts "SCAN: seed = $seed  mask = $mask   flit = $flit"
	}
	set clrLoadOld $clrLoad
	set oldseed $seed

 } ;# while iterCount < iterLimit

puts "script complete, enter additional commands"
