#
# di_bist_persist.tcl
#
# Kurt Kermes
# 2000/03/31
#
# script for running Synergy DI (L4$) BIST
# This extends the core BIST code by allowing multiple passes,
# error dumps, read only, write only, and multiple reads/write
#

source $env(SCAN_PATH_DATA)/diags/ip37/syn_procs.tcl


############################################################################
# script parameters
# You may modify the string after 'set <variable>'
#
############################################################################

############################################################################
############################################################################
# parameter initialization
#    user may put in value here-------------vvvvvvvvv
############################################################################

#set mod SYN1CHN 
#set brd SYN1BRD 
#set chip U0 

# memCfg - bit 0: enable ECC
#          bit 1: enable Speculative Page Open
#          bit 2: enable CAS bit 8
set memCfg          6 
set mrsBurst        3 
set mrsBurstType    1 
set mrsCASLatency   2 
set mrsTest         0 
set mrsDLLReset     1 
set emrsDLLDisable  0 
set emrsDrvrImpCtl  1  ;# 0:norm 1:weak
set emrsQFCCtl  0     ;# 0:disab 1:enab
set rfrshVal    0x3FFF 
set wrTimeoutVal 0x10 

set bistAddr 0x3fff00
set bistData 0xffff
set bistCycle 0x0

# superBankMask  1: test SB 0   2: test SB 1   3: test SB 0 and 1
set superBankMask 0x1


set numReadsPerWrite 10
set restartOnError 0
set stopOnError 1
set iterLimit 3
set errorLimit 0
set iterPattern 0
set onlyWrite 0
set onlyRead 0
set outputMode 0
	
############################################################################
############################################################################
# procs
############################################################################
############################################################################
proc di_bist_write { bnk data cycle } {

puts "di_bist_write - superbank $bnk  pattern: $data"

# BIST_DATA - set the write bit, cycle and data patterns
set datalo [ expr  1 << 17 | $cycle << 16 | $data ]
set addr   [ expr $bnk * 0x200 + 0x658 ]
pio_write $addr 0 $datalo

# set start in BIST_CTL

set address [ expr $bnk * 0x200 + 0x640 ]
pio_write $address 0 1

# wait for completion - poll di_bist_status
set address [ expr $bnk * 0x200 + 0x648 ]
set $datalo 0
while { [expr  $datalo & 2]  != 2 } {
	pio_read $address 0 $datalo
	puts "di_bist_write: POLL status: $datalo"
}

}

############################################################################
# di_bist_read - 
#    do a readback - 
#    return when the BIST_DONE bit is set.
############################################################################
proc di_bist_read { bnk data cycle } {

puts "di_bist_read - superbank $bnk"

# BIST_DATA - clear the write bit, cycle and data patterns
set datalo [ expr $cycle << 16 | $data ]
set addr [ expr $bnk * 0x200 + 0x658 ]
pio_write $addr 0 $datalo

# set start in BIST_CTL

set address [ expr $bnk * 0x200 + 0x640 ]
pio_write $address 0 1

# wait for completion - poll di_bist_status
set address [ expr  $bnk * 0x200 + 0x648 ]
set datalo 0
while { [expr  $datalo & 2]  != 2 } {
	pio_read $address 0 $datalo
	puts "di_bist_read: POLL status: $datalo"
}
puts "di_bist_read: final status : $datalo"

	return $datalo

}

############################################################################
# di_bist_addr
############################################################################
proc di_bist_addr { bnk address } {

puts "di_bist_addr: address is $address"
# BIST ADDR - set the starting address
pio_write [ expr $bnk * 0x200 + 0x650 ] 0 $address

}

############################################################################
#
############################################################################
proc di_memory_config { bnk } {

global memCfg
puts "di_memory_config"

set addr [ expr $bnk * 0x200 + 0x600 ]
pio_write $addr 0 $memCfg

}

############################################################################
#
############################################################################
proc di_mrs_config { bnk } {

global mrsBurst mrsBurstType mrsCASLatency mrsTest mrsDLLReset
puts "di_mrs_config"

set datahi 0x80000000
set datalo [ expr   $mrsDLLReset << 8   | \
                    $mrsTest << 7       | \
                    $mrsCASLatency << 4 | \
                    $mrsBurstType  << 3 | \
                    $mrsBurst   ]
        
set addr [ expr $bnk * 0x200 + 0x608 ]
pio_write $addr $datahi $datalo

# poll for completion
pio_read $addr $datahi $datalo 
puts "di_mrs_config: datahi is $datahi"
if { [expr $datahi & 0x80000000] == 0x80000000 } {
	pio_read $addr $datahi $datalo
	puts "di_mrs_config: datahi is $datahi"
	if { [expr $datahi & 0x80000000] == 0x80000000 } {
		error "di_mrs_config: timeout on mrs config start"
	}
}

} 


############################################################################
# di_emrs_config
############################################################################
proc di_emrs_config { bnk } {

global emrsDLLDisable
puts "di_emrs_config"
set datahi 0x80000000

set datalo [ expr 0x2 | $emrsDLLDisable ]
set addr [ expr $bnk * 0x200 + 0x610 ]
pio_write $addr $datahi $datalo

# poll for completion
pio_read $addr $datahi $datalo 
puts "di_emrs_config: datahi is $datahi"
if { [expr $datahi & 0x80000000] == 0x80000000 } {
	pio_read $addr $datahi $datalo
	puts "di_emrs_config: datahi is $datahi"
	if { [expr $datahi & 0x80000000] == 0x80000000 } {
		error "di_emrs_config: timeout on emrs config start"
	}
}

}


############################################################################
#
############################################################################
proc di_refresh_control { bnk } {

global rfrshVal
puts "di_refresh_ctrl - rfrshVal is $rfrshVal"
set datahi 0x80000000

set addr [ expr $bnk * 0x200 + 0x618 ]
pio_write $addr $datahi $rfrshVal

}


############################################################################
#
############################################################################
proc di_write_timeout_ctrl { bnk } {

global wrTimeoutVal
puts "di_write_timeout_ctrl"
set datahi 0x80000000

set addr [ expr $bnk * 0x200 + 0x638 ]
pio_write $addr $datahi $wrTimeoutVal

}


############################################################################
# di_clear_err_regs - 
############################################################################
proc di_clear_err_regs { bnk } {

puts "di_clear_err_regs"
set addr [ expr $bnk * 0x200 + 0x660 ]
pio_write $addr 0 0
set addr [ expr $bnk * 0x200 + 0x668 ]
pio_write $addr 0 0
set addr [ expr $bnk * 0x200 + 0x670 ]
pio_write $addr 0 0
set addr [ expr $bnk * 0x200 + 0x678 ]
pio_write $addr 0 0

}

############################################################################
#
############################################################################
proc di_init { } {

global superBankMask
set val 0
#
# Initialize memory
#
#SIMACC GETTIME val
#puts "b-4 emrs_config: time is $val"
if { $superBankMask & 1 } { di_emrs_config 0 }
if { $superBankMask & 2 } { di_emrs_config 1 }

#SIMACC GETTIME val
#puts "before refresh_ctl: time is $val"
if { $superBankMask & 1 } { di_refresh_control 0 }
if { $superBankMask & 2 } { di_refresh_control 1 }

#SIMACC GETTIME val
#puts "time is $val"
if { $superBankMask & 1 } { di_mrs_config 0 }
if { $superBankMask & 2 } { di_mrs_config 1 }

#SIMACC GETTIME val
#puts "time is $val"
if { $superBankMask & 1 } { di_memory_config 0 }
if { $superBankMask & 2 } { di_memory_config 1 }

#SIMACC GETTIME val
#puts "time is $val"
if { $superBankMask & 1 } { di_write_timeout_ctrl 0 }
if { $superBankMask & 2 } { di_write_timeout_ctrl 1 }

# don't really need to do this, but for completeness sake, 
if { $superBankMask & 1 } { di_clear_err_regs 0 }
if { $superBankMask & 2 } { di_clear_err_regs 1 }

}

############################################################################
# get_error_info - 
# dumps out error information
#  - actual data received, failing address
#  - calculate expected data (if toggled) and difference
#  optionally puts failing info into array for further analysis
############################################################################
proc get_error_info { bnk failAddr } {

upvar failAddr lFailAddr
puts "get_error_info - superbank $bnk"
	set datahi 0
	set datalo 0
	set addr [ expr $bnk * 0x200 + 0x650 ]  ;# DI_BIST_ADDR
	pio_read $addr $datahi $datalo
	puts "   bist_addr: datalo is $datalo"
puts "   Bank: [expr ($datalo>> 20) & 0x3]   Row: [expr ($datalo >> 8) & 0xfff]  Top Column:  [expr ($datalo >> 3) & 0x1f] Bottom Column: [ expr ($datalo) & 7]"
	set lFailAddr $datalo
	set addr [ expr $bnk * 0x200 + 0x658 ]  ;# DI_BIST_DATA
	pio_read $addr $datahi $datalo
	puts "   bist_data: datalo is $datalo"
	set addr [ expr $bnk * 0x200 + 0x660 ]  ;# DI_BIST_DRAM_UE_ERROR1
	pio_read $addr $datahi $datalo
	puts "   bist_ue_error1: datalo is $datalo"
	puts "   bist_ue_error1: datahi is $datahi"
	set addr [ expr $bnk * 0x200 + 0x668 ]  ;# DI_BIST_DRAM_UE_ERROR2
	pio_read $addr $datahi $datalo
	puts "   bist_ue_error2: datalo is $datalo"
	puts "   bist_ue_error2: datahi is $datahi"
	set addr [ expr $bnk * 0x200 + 0x670 ]  ;# DI_BIST_DRAM_CE_ERROR1
	pio_read $addr $datahi $datalo
	puts "   bist_ce_error1: datalo is $datalo"
	puts "   bist_ce_error1: datahi is $datahi"
	set addr [ expr $bnk * 0x200 + 0x678 ]  ;# DI_BIST_DRAM_CE_ERROR2
	pio_read $addr $datahi $datalo
	puts "   bist_ce_error2: datalo is $datalo"
	puts "   bist_ce_error2: datahi is $datahi"
}

############################################################################
############################################################################

############################################################################
# script beginning
############################################################################
#RESET -H
#RESET

#SEL MOD $mod
#SEL BRD $brd
#SEL CHIP $chip

#SIC 0
#SIC ENAB 1

############################################################################
di_init
set iterCount  0
set passCount  0
set errorCount 0
set curSBank   0
set errorOcc   0
set failAddr   0
set val "Hey, this ain't no simulator"

for { set iterCount 1 } { $iterCount <= $iterLimit } { incr iterCount } {

	for { set curSBank 0 } { $curSBank < 2 } { incr curSBank } {
		if { [ expr (1 << $curSBank) & $superBankMask ] == 0 } {
			continue
		} 
		#
		# Bist write
		#
		if { $onlyRead == 0 } { 
			di_bist_addr $curSBank $bistAddr
			#SIMACC GETTIME val
#			puts "time is $val"
			di_bist_write $curSBank $bistData $bistCycle
		}
		
		set readDone 0
		#
		# Bist read
		#
		if { $onlyWrite == 0 } { 
			for { set readCount 0 } { $readCount < $numReadsPerWrite } { incr readCount } {
				# set BIST address
				#SIMACC GETTIME val
	#			puts "time is $val"
				di_bist_addr $curSBank $bistAddr
				
				#SIMACC GETTIME val
	#			puts "time is $val"
				set status [ di_bist_read $curSBank $bistData $bistCycle ]
	puts "read done, status is $status"
				
				#
				# check status
				#
				while { $readDone == 0 } {
					if { $status != 0x3 } {
						get_error_info $curSBank $failAddr  
						set errorOcc 1
						incr errorCount
						if { $errorCount == $errorLimit } {
							puts "*** errorLimit reached, ending script"
							return
						}
						if { $stopOnError == 1 } {
							puts "*** stop on error set, ending script"
							return
						}
						if { $restartOnError == 1 } {
							di_bist_addr $curSBank $failAddr
							set status [ di_bist_read $curSBank $bistData $bistCycle ]
						}
					} else {
						set readDone 1
					}
				} ;# while readDone
			}
		}  ;# if onlyWrite
	} ;# for curSBank
	if { $errorOcc == 1} {
		incr errorCount	
		set errorOcc 0
	} else {
		incr passCount
	}
	puts "***** Iteration $iterCount completed:   passes: $passCount  errors: $errorCount"	
} ;# for iterCount

############################################################################
############################################################################

