# ip37_dump_br_latches_batch.tcl

# KK
# 2001/02/23
#

# Dumps out Bedrock latch chains into external files.
# 

# User can specify a list of regs to dump, optionally
# outputting the bit breakouts.
#
#       (C) COPYRIGHT SILICON GRAPHICS, INC.
#       UNPUBLISHED PROPRIETARY INFORMATION.
#       ALL RIGHTS RESERVED.

# Mod history:
# 2001/04/06 - KK - modified from tk version to
#                 - run from a scantool command line
#                 - dump to a different directory
#                 - dump all latches into a single file

set version 0.6

#######################################################################
# script parameters
#######################################################################
set brd IP37
set chip BR

# User can select which chains get dumped and processed:
# This selection is done by chain number, which corresponds to 
# the clock domain 
# Chain 0 holds latches in the md, lb, ni and xb chiplets
# chain 1 holds latches in the ii, and some ni chiplets
# chain 2 holds latches in the pi chiplet
set chain0Sel 1
set chain1Sel 1
set chain2Sel 1

set showSingleLatches  1

set outFileMain   "$env(SCAN_USER_DATA)/ip37_dumps/br_dump_$mod"
set outFileChain0 ""
set outFileChain1 ""
set outFileChain2 ""

set msgWinOut 0

#######################################################################
# internal parameters
#######################################################################
set chainInv   0   ;# invert the value of the chain?
set chainCir   1   ;# circulate chain?
set gmocType   0
#set chainSel   0   ;# which chain are we accessing? 0-5 is valid.
set lpraAccess 0  ;# keep this clear.
set testSpeed  0


set inFileChain0 $env(SCAN_PATH_DATA)/diags/generic/br_chain0.alpha
set inFileChain1 $env(SCAN_PATH_DATA)/diags/generic/br_chain1.alpha
set inFileChain2 $env(SCAN_PATH_DATA)/diags/generic/br_chain2.alpha

#######################################################################
#######################################################################


#######################################################################
# procs
#######################################################################
#######################################################################
# xBuild64 - standin for Build64 - sets zero to 0x0000000000000000
#                                           not 0x00000000
#######################################################################
proc xBuild64 {intHi int32Lo } {
set l64 ""
#evaluate intHi and int32Lo into hex strings
set int32Lo [ format "%08x" $int32Lo ]
set intHi [ format "%#x" $intHi ]
#cat intHi and int32Lo into int64
if { $intHi == 0x0 } {
        set intHi 0x00000000
}
append l64 $intHi $int32Lo
#puts "l64 is $l64"
return $l64

}


#######################################################################
# dump - performs the scan operations that dumps the selected scan chain
#        to a REG
#######################################################################
proc dump {chainSel} {

global mod modid brd brdidx chnidx chip
global msgf

foreach ::fd $::fdlist { puts $::fd "* * * * * * Scanning in SCAN_CTL_TDR" }

INST SCAN_CTL_TDR
GOINST
set scanCtlReg [ expr ($::testSpeed << 10) | ($::lpraAccess << 9) | \
                      ($::chainInv << 8) | ($::chainCir << 7)     |  \
                      ($::gmocType << 4) | ($chainSel) ]
DATA $scanCtlReg
GODATA

set regName BR_CHAIN$chainSel
REG $regName
set regNameLower [ string tolower $regName ] ;# til I fix that bug

################################# put the internal chain in TDR
foreach ::fd $::fdlist { puts $::fd "* * * * * * Scanning in SCAN_TDR Inst" }
INST SCAN_TDR
GOINST

foreach ::fd $::fdlist { 
	puts $::fd "* * * * * * Getting internal latch chain $chainSel" 
}
GODATA

foreach ::fd $::fdlist {
   puts $::fd " "
   puts $::fd "Loading Data Into Array"
   puts $::fd " "
}
update 

# clear out latchArr so we only load one chain at a time
if { [ array exists ::latchArr ] } { unset ::latchArr }

set nbits [ INFO CHIP $::chip REG $regNameLower NFLD ]
for { set i 0 } { $i < $nbits } { incr i } {
   set structDesc [ INFO CHIP $::chip REG $regNameLower.$i STRUCT ]
   regexp {(.*):} $structDesc match latchName
   set value     [ INFO CHIP $::chip REG $regNameLower.$i OUT ]
   set ::latchArr($latchName) $value
}

foreach ::fd $::fdlist {
   puts $::fd " "
   puts $::fd "Loaded $i latches Into Array"
   puts $::fd " "
}
update idletasks


# probably want to deallocate the REG here
REG DELETE [ string tolower $regName ]


} ;# proc



#######################################################################
# read_n_print - reads in the latch REG
#                and formats it for dumping to file
#######################################################################
proc read_n_print {chain} {

	global msgf
	global msgWinOut
	
############################### open proper input file
switch $chain {
	0 { set outFile $::outFileChain0; set inFile $::inFileChain0 }
	1 { set outFile $::outFileChain1; set inFile $::inFileChain1 }
	2 { set outFile $::outFileChain2; set inFile $::inFileChain2 }
}
if { [ catch { open $inFile r } inFileID ] } {
	puts stderr "Cannot open $inFile"
	exit
}

############################### Add additional chain-specific output file

if { [string length $outFile ] > 0 } {
	file mkdir $env(SCAN_USER_DATA)/ip37_dumps
	if { [catch { open $outFile w } outFileID] } {
		puts stderr "Cannot open $outFile"
		exit 
	}
	lappend ::fdlist $outFileID
}
foreach ::fd $::fdlist { puts $::fd \
"         Bedrock Latch dump of chain $chain on [ exec date ] " }
update idletasks

####################################### Start dumping

set valArr u
set curLeafBase u
set curBranch u
set lastLeaf u
set linenum 0
while { [ gets $inFileID line ] >= 0 } {
	incr linenum
	set spLine [ split $line / ]
	set branch [ join [lrange $spLine 0  [expr [llength $spLine] -2]]  / ]
#puts "Branch is $branch"
#puts "CurBranch is $curBranch"
	if { [ string compare $branch $curBranch ] != 0 } {
		if { [string compare $curLeafBase "u"] } {
         set i [ expr [array size valArr] - 1]
         while {$i >=0} {
            if { [ expr $i & 1 ] != 0 } {
               set int32hi [ format "%#0x " $valArr($i) ]
               set int32lo [ format "%#0x " $valArr([expr $i-1]) ]
               set int64 [ xBuild64 $int32hi $int32lo ]
#puts stderr "branch is $lastLeaf"
#puts stderr "Build64: $int64 < $int32hi + $int32lo "
               foreach ::fd $::fdlist { puts -nonewline $::fd "$int64 " }
               incr i -2
            } else {
               foreach ::fd $::fdlist {
                  puts -nonewline $::fd [ format "%#0x " $valArr($i) ]
               }
               incr i -1
            }
         }
		}
		foreach ::fd $::fdlist { puts $::fd ""}
		update 
		set lastLeaf 1
		set curLeafBase u

		foreach ::fd $::fdlist { puts $::fd "$branch" }
		update idletasks
		set curBranch $branch
	}

# process the leaf
# Check for subscript, then make sure it's a numeric subscript, some
# latches are like PI_DE_DATA_reg[21][G] or PI_TOP_Block_reg[E]
	set leaf [ lindex $spLine [ expr [llength $spLine] -1 ] ]
#puts "leaf is $leaf"
	set tmp [ string first \[ $leaf ]
	if { $tmp >= 0 } {
		set leafBase [ string range $leaf 0 [expr $tmp - 1]  ]
	} else {
		if { $::showSingleLatches == 0 } { continue }
		set leafBase $leaf
	}
#puts "leafBase is $leafBase"
	if { [ string compare $leafBase $curLeafBase ] != 0 } {
#puts "curLeafBase is $curLeafBase"
		if { $lastLeaf != 1} {
			if { [string compare $curLeafBase "u"] } {
				set i [ expr [array size valArr] - 1]
				while {$i >=0} {
					if { [ expr $i & 1 ] != 0 } {
						set int32hi [ format "%#0x " $valArr($i) ]
						set int32lo [ format "%#0x " $valArr([expr $i-1]) ]
						set int64 [ xBuild64 $int32hi $int32lo ]
#puts stderr "leafBase is $leafBase; i is $i"
#puts stderr "hi: [ format "%#0x " $valArr($i) ] lo: [ format "%#0x " $valArr([expr $i-1]) ]"
#puts stderr "xBuild64: $int64 < $int32hi + $int32lo "
						foreach ::fd $::fdlist { puts -nonewline $::fd "$int64 " }
						incr i -2
					} else {
						foreach ::fd $::fdlist {
							puts -nonewline $::fd [ format "%#0x " $valArr($i) ]
						}
					}
					incr i -1
				}
			}
			foreach ::fd $::fdlist { puts $::fd "" }
			update idletasks
			set curLeafBase u
		} else { set lastLeaf 0 }
		foreach ::fd $::fdlist { puts -nonewline $::fd " +--- $leafBase: "}
		update idletasks
		set curLeafBase $leafBase
		unset valArr
		set valArr(0) 0
	}
#puts "curLeafBase is $curLeafBase"
	set subscript u
	set line1 [ string trim $line ]
	set bit $::latchArr($line1)
	if { [ regexp {\[([0-9]+)\]} $leaf match subscript]  } {
#     puts "Leaf: $leaf  Subscript $subscript"
# get latch value
# calculate
		set numWord [ expr $subscript / 32 ]
#puts "numWord is $numWord"
		if { [expr $numWord + 1] > [array size valArr] } {
			for { set j [ array size valArr] } { $j < $numWord + 1 } {incr j} {
				set valArr($j) 0
			}
		}
		set valArr($numWord) [ expr $valArr($numWord) | $bit << ($subscript % 32) ]
	} else {   ;# no subscript, just a single latch
		set valArr(0) $bit
	}
} ;# while

# flush out last leaf
for { set i [expr [ array size valArr] -1 ] } { $i >= 0 } { incr i -1} {
	foreach ::fd $::fdlist { 
		puts -nonewline $::fd [ format "%#08x " $valArr($i) ] 
	}
}
foreach ::fd $::fdlist { puts $::fd "" }
update idletasks

foreach ::fd $::fdlist {
	puts $::fd "\n\n * * * * * * Dump of chain $chain complete.\n "
}

######################################## close chain-specific output file
#if { [string length $::outFileMain ] > 0 } { flush $::outFileMainID }

# if we're only dumping to one file, don't delete chain-specific fd
if { [string length $outFile ] > 0 } { 
	flush $outFileID; close $outFileID 
	set tmp [ expr [ llength $::fdlist ] - 1]
	set tmpfdlist [ lreplace $::fdlist $tmp $tmp ]
	set ::fdlist $tmpfdlist
}

close $inFileID

}   ;# proc

#######################################################################
#######################################################################

#######################################################################
# main code
#######################################################################

	SEL MOD $::mod
	SEL BRD $::brd
	SEL CHIP $::chip

	RESET -H

puts "Module $mod: Starting latch dump at [ exec date ]"
	
	################################# take care of file I/O
	set ::fdlist {}
	if { [string length $::outFileMain ] > 0 } {
# check for directory
		file mkdir $env(SCAN_USER_DATA)/ip37_dumps
#set outFileMain   "$env(SCAN_USER_DATA)/br_dumps/br_dump_$mod"
		if { [catch { open $::outFileMain w } ::outFileMainID ] } {
			puts stderr "Cannot open $::outFileMain"
			return
		}
		lappend ::fdlist $::outFileMainID
	}
	foreach ::fd $::fdlist { puts $::fd \
	" ################Bedrock Latch dump of $::mod on [ exec date ] " }
	
	################################ stop clocks
	foreach ::fd $::fdlist { puts $::fd "* * * * * * Stopping clocks..." }
	foreach ::fd $::fdlist { puts $::fd "* * * * * * Setting Mode reg" }
	INST MODE_REG_TDR
	GOINST
	DATA 0x1F
	GODATA
	
puts "fdlist is $fdlist"
	# reduces risk of repeating list elems
	if { $::chain0Sel != 0 } {
		foreach ::fd $::fdlist { puts $::fd "* * * * * Dumping chain 2 " }
		dump 2
		read_n_print 2
	}
puts "fdlist is $fdlist"
	if { $::chain1Sel != 0 } {
		foreach ::fd $::fdlist { puts $::fd "* * * * * Dumping chain 1 " }
		dump 1
		read_n_print 1
	}
puts "fdlist is $fdlist"
	if { $::chain2Sel != 0 } {
		foreach ::fd $::fdlist { puts $::fd "* * * * * Dumping chain 0 " }
		dump 0
		read_n_print 0
	}
puts "fdlist is $fdlist"
	if { [string length $::outFileMain ] > 0 } { close $::outFileMainID }

puts "Module $mod: End of latch dump at [ exec date ]"

exit
############################################################################
############################################################################

