# ip37_dump_syn1_mmr.tcl
# 
#
# Kurt Kermes
# 2000/04/21
#
# script for dumping out a set of Synergy registers
#
# User can specify a list of regs to dump, optionally
# outputting the bit breakouts.
#
#       (C) COPYRIGHT SILICON GRAPHICS, INC.
#       UNPUBLISHED PROPRIETARY INFORMATION.
#       ALL RIGHTS RESERVED.
#
# Mod History:
#     2001/04/21 - KK - cleaned up to allow additional scripting

source $env(SCAN_PATH_DATA)/diags/ip37/syn_procs.tcl

############################################################################
# script parameters
# You may modify the string after 'set <variable>'
#
############################################################################

############################################################################
############################################################################
# parameter initialization
#    user may put in value here-------------vvvvvvvvv
############################################################################

set brd IP37
set chip SYN1

# reglist - lists the set of chiplets you want to dump out.
# valid is all, lb, si, fi, di, ce
# note - You must keep the 'list' keyword in the line.
set reglist [list all ] 

# showFields - if 1, outputs field breakouts
set showFields 0 

# showDescs - if 1, outputs the Register descriptions and if
#              showFields is 1, outputs the field descriptions
set showDescs 0 

# outFileName - if not an empty string, will also dump the
#               contents to the named file
set outFileName "$env(SCAN_USER_DATA)/ip37_dumps/syn1_dump_mmr_$mod" 

############################################################################
# globals
############################################################################
set modid u
set brdidx u

############################################################################
# procs
############################################################################

############################################################################
# ExtractField
############################################################################
proc ExtractField {bitpos bitwidth datahi datalo } {

# check 2 c f we cross the 32-bit boundary
	if { $bitpos < 32 && [expr ($bitpos + $bitwidth)] > 32 } {
		# concatenate the two strings - rem Tcl can't process > 32 bit ints
		set widthhi [ expr $bitwidth - (32 - $bitpos) ]
		set widthlo [ expr $bitwidth - $widthhi ]
		set smshft [expr $bitpos % 4]  ;# how far we shift the nibble
		set tmphi $datahi 
		if { $widthlo == 32 } {
			set mask 0xffffffff
		} else {
			set mask  [ expr (1 << $widthlo) - 1 ]
		}
		set loval [ expr ($datalo >> $bitpos) & $mask ]

		if { $widthhi == 32 } {
			set mask 0xffffffff
		} else {
			set mask  [ expr ( 1 << $widthhi) - 1 ]
		}
		set hival [ expr ($datahi & $mask) ]

		set smmask [ expr (1 << $smshft) - 1 ]
		set tmp    [ expr ($hival & $smmask) ] 
		set hival  [ expr ($hival >> $smshft) ]
		set loval  [ expr ($loval | $tmp << ($widthlo / 4) + $smshft) ]
		set val    [ format "0x%x%x" $hival $loval ]
	} else {
		if { $bitpos >= 32 } { 
			set mask [ expr (1 << $bitwidth) - 1 ]
			set val [expr $datahi >> $bitpos]
			set val [ format "0x%x" [expr $val & $mask]]
		} else {
			if { $bitwidth == 32 } {
				set mask 0xffffffff
			} else {
				set mask [ expr (1 << $bitwidth) - 1 ]
			}
			set val [expr $datalo >> $bitpos]
			set val [format "0x%x" [expr $val & $mask]]
		}
	}
	return $val
}


############################################################################
# DumpBlock - dump the contents of the block name in the parameter
############################################################################
proc DumpBlock {blockName} {
	global showFields
	global showDescs
	global fileID
	global fdlist
	global mod modid brd brdidx

	seek $fileID 0 start

	set curState REG ;# valid states: REG, INT1, INT2
	set addr     0
	set bitpos   0
	set bitwidth 0
	set val      0
	set datahi   0x00000102
	set datalo   0x80000801
	set regName  x
	set fldName  x

# get to beginning of block
	while { [ gets $fileID line] >= 0 } {
		set test "BEGIN [ string toupper $blockName ]"
		if { [string compare $line $test] } { 
			continue 
		} else {
			break
		}
	}
	if { [ eof $fileID ] == 1 } {
		return
	}

	foreach fd $fdlist {
		puts $fd " "
		puts $fd "***** Dumping Block $blockName *****"
		puts $fd " "
	}

# get lines until eof or next BEGIN marker	
	while { [ gets $fileID line] >= 0 } {
		if { [ regexp ^BEGIN $line] > 0 } { 
			return 
		}
		#([A-Za-z0-9_]+) (\(0x[0-9A-Fa-f]+\)) 
		if { [regexp {register ([A-Za-z0-9_]+)(\(0x[0-9A-Fa-f]+\))} $line \
		                                                 match regName val] } { 
			set curState REG
			set val [ string trim $val () ]
			pio_read $val $datahi $datalo
			foreach fd $fdlist {
				puts $fd "$regName (address $val)  = [format "0x%08x%08x" $datahi $datalo]"
			}
		} elseif { $curState == "REG" && $showFields == 1 && \
			         [regexp {integer ([A-Za-z0-9_]+)(\([0-9]+\))} $line \
			                                          match fldName bitwidth] } {
			set curState INT1
			set bitpos 0
			set bitwidth [ string trim $bitwidth () ]
			set fldVal [ExtractField $bitpos $bitwidth $datahi $datalo]
			foreach fd $fdlist {
				puts $fd "            $fldName = $fldVal"
			}
			set bitpos [expr $bitpos + $bitwidth]
		} elseif { ($curState == "INT1" || $curState == "INT2" ) &&  \
		            $showFields == 1 && \
			         [regexp {integer ([A-Za-z0-9_]+)(\([0-9]+\))} $line \
			                                          match fldName bitwidth] } {
			set curState INT2
			set bitwidth [ string trim $bitwidth () ]
			set fldVal [ExtractField $bitpos $bitwidth $datahi $datalo]
			foreach fd $fdlist {
				puts $fd "   $fldName = $fldVal"
			}
			set bitpos [expr $bitpos + $bitwidth]
		} elseif { [regexp {nop(\([0-9]+\))} $line match bitwidth] } {
			set bitwidth [ string trim $bitwidth () ]
			set bitpos [expr $bitpos + $bitwidth]
		}
	}
}


############################################################################
############################################################################

############################################################################
# script beginning
############################################################################

SEL MOD  $mod
SEL BRD  $brd
SEL CHIP $chip

RESET -H
RESET

# get synergy rev
INST IDCODE
GOINST
GODATA
set version [ INFO CHIP $chip DATA IDCODE.REV_LEV OUT ]
set synVer [ expr $version ]
puts "Detected Synergy version $synVer"

if [ catch { open "$env(SCAN_PATH_DATA)/diags/ip37/reg_set_$synVer" r } fileID] {
	puts stderr "Cannot open reg_set_$synVer"
	return
} 

set fdlist [ list stdout ]
if { [string length $outFileName] > 0 } {
	file mkdir $env(SCAN_USER_DATA)/ip37_dumps
	if [ catch { open $outFileName w } outFileID] {
		puts stderr "Cannot open $outFileName"
		return
	}
	lappend fdlist $outFileID
} 

puts "Dumping Module $mod - Board $brd - Chip $chip"

#process list
if {[ lsearch $reglist all ]  != -1 } {
	DumpBlock LB
	DumpBlock SI
	DumpBlock FI
	DumpBlock DI
	DumpBlock CE
} else {
	if { [lsearch $reglist lb ] >= 0 } {
		DumpBlock LB
	}
	if { [lsearch $reglist si ] >= 0 } {
		DumpBlock SI
	}
	if { [lsearch $reglist fi ] >= 0 } {
		DumpBlock FI
	}
	if { [lsearch $reglist di ] >= 0 } {
		DumpBlock DI
	}
	if { [lsearch $reglist ce ] >= 0 } {
		DumpBlock CE
	}

	foreach fd $fdlist {
		puts $fd " "
		puts $fd "***** Dump complete *****"
		puts $fd " "
	}
}
exit

############################################################################
############################################################################

