#
# rtr_debug.tcl
#
# Kurt Kermes
# 99/09/15
#
# selects which DEBUG_PORT data goes out the MICTOR connector
#
# Mod history:
#  2001/08/23 - KK - Changed BRD/CHIP names
#  2001/09/04 - KK - Added clock stop trigger
#
#############################################################################
#
#   (C) COPYRIGHT SILICON GRAPHICS, INC.
#   UNPUBLISHED PROPRIETARY INFORMATION
#   ALL RIGHTS RESERVED UNDER THE COPYRIGHT LAWS OF THE UNITED STATES.
#
#   Portions of this software may still be in development.  The existence
#   of the software still in development is not a commitment of actual
#   release or support by Silicon Graphics, Inc.  Silicon Graphics, Inc.
#   assumes no liability for any damages resulting from attempts to use any
#   product not officially released.  If it is released, the final form of
#   the product and the time of official release is at the discretion of
#   Silicon Graphics, Inc.
#
#############################################################################
#


set portA   0
set portB   1
set portC   2
set portD   3
set portE   4
set portF   5
set portG   6
set portH   7
set Arbiter 8
set LB      9
set pcl00   A
set pcl00a  B
set pclFF   C
set pclA5   D
set pcl5A   E
set pcl00c  F

set fromA  0
set fromB  1
set fromC  2
set fromD  3
set fromE  4
set fromF  5
set fromG  6
set fromH  7
set fromL  9

set toA  1
set toB  2
set toC  3
set toD  4
set toE  5
set toF  6
set toG  7
set toH  8
set disableRoute 0

############################################################################
# script parameters
#
# debugPort: the active port A - H  ( DEBUG_ADDR bits 9:6 )
#  this also doubles as the source port is you're specifying routing
#  use $portX for debugging 
#  use $fromX when specifying a port source 
# debugRouteTo  : specify xbar route destination
#    SET THIS TO 0 if not using this 
# debugAddr: defined in the spec for specific signal
#            
############################################################################
set debugPort      $portH ;# you can use $fromA if you're specifying route
set debugRouteTo   $disableRoute
set debugAddr      0x1

set setTrigClkStop 0x1
set clkstopMask    0x4000
set clkstopData    0x4000

############################################################################
############################################################################

set brd R
set chip RTR

RESET -H
RESET

SEL MOD  $mod
SEL BRD  $brd
SEL CHIP $chip

INST SYSCO_D
GOINST

if {$debugRouteTo != $disableRoute } {
	set debugAddr [ expr (debugPort << 6) | ($debugAddr & 0xf) ]
} else {
	# verify that we aren't routing to same port we're coming from 
	if { ($debugRouteTo == $toA && $debugPort == $fromA) || \
		($debugRouteTo == $toB && $debugPort == $fromB) || \
		($debugRouteTo == $toC && $debugPort == $fromC) || \
		($debugRouteTo == $toD && $debugPort == $fromD) || \
		($debugRouteTo == $toE && $debugPort == $fromE) || \
		($debugRouteTo == $toF && $debugPort == $fromF) || \
		($debugRouteTo == $toG && $debugPort == $fromG) || \
		($debugRouteTo == $toH && $debugPort == $fromH) } {
		puts "Warning: invalid combination of specified routing !"
		puts "Identical 'TO' and 'FROM' ports are invalid!"
	}

set debugAddr [ expr ($debugPort << 6) | ($debugRouteTo << 2) | \
                     ($debugAddr) ]

}

DATA $debugAddr
GODATA

puts "debug port is set up: "

if { $setTrigClkStop != 0 } {
	INST MODE_REG
	GOINST
	DATA 0x2
	GODATA

	INST CLK_STOP_TRIG_DATA
	GOINST
	DATA $clkstopData
	GODATA

	INST CLK_STOP_TRIG_MASK
	GOINST
	DATA $clkstopMask
	GODATA

	puts "Triggered clock stop is set up"
}
puts "Enter additional script commands if you want "
