# r_dump_br_latches.tcl

# KK
# 2001/04/12
#

# Dumps out Router latch chains into external files.
# 

# User can specify a list of regs to dump, optionally
# outputting the bit breakouts.
#
#       (C) COPYRIGHT SILICON GRAPHICS, INC.
#       UNPUBLISHED PROPRIETARY INFORMATION.
#       ALL RIGHTS RESERVED.

# Mod history:
# 2001/04/12 - KK - modified from ip35_dump_br_latches_batch.tcl
# 2001/04/16 - KK - added code to handle inversions to 2422 latch inputs
# 2001/08/15 - KK - added code to handle inversions to 2727 latch inputs
#                   (new latch inversion file)

set version 0.5

#######################################################################
# script parameters
#######################################################################
set brd R
set chip RTR

# User can select which chains get dumped and processed:
# This selection is done by chain number, which corresponds to 
# the clock domain 
# Chain 0 holds latches in the md, lb, ni and xb chiplets
# chain 1 holds latches in the ii, and some ni chiplets
# chain 2 holds latches in the pi chiplet
set chain0Sel 1

set showSingleLatches  1

set outFileMain   "$env(SCAN_USER_DATA)/rtr_dumps/rtr_dump_$mod"
set outFileChain0 ""

set inversionsFile "$env(SCAN_PATH_DATA)/diags/rbrick/rtr_inversions.new"
set msgWinOut 0

#######################################################################
# internal parameters
#######################################################################
set slowMode   0
set lpraAccess 0
set chainSel   0
set gmocType   0
set chainCir   1
set chainInv   0

set inFileChain0 $env(SCAN_PATH_DATA)/diags/generic/rtr_chain0.alpha

#######################################################################
#######################################################################


#######################################################################
# procs
#######################################################################

#######################################################################
# dump - performs the scan operations that dumps the selected scan chain
#        to a REG
#######################################################################
proc dump {chainSel} {

global mod modid brd brdidx chnidx chip
global msgf

foreach ::fd $::fdlist { puts $::fd "* * * * * * Scanning in SCAN_CTL_TDR" }

INST SCAN_CTL_TDR
GOINST
set scanCtlReg  [ expr ($::chainInv << 9) | ($::chainCir << 8) | \
                       ($::gmocType << 5) | ($::chainSel << 2) | \
                       ($::lpraAccess << 1) | ($::slowMode) ]

DATA $scanCtlReg
GODATA

set regName RTR_CHAIN$chainSel
REG $regName
set regNameLower [ string tolower $regName ] ;# til I fix that bug

################################# put the internal chain in TDR
foreach ::fd $::fdlist { puts $::fd "* * * * * * Scanning in SCAN_TDR Inst" }
INST SCAN_TDR
GOINST

foreach ::fd $::fdlist { 
	puts $::fd "* * * * * * Getting internal latch chain $chainSel" 
}
GODATA

foreach ::fd $::fdlist {
   puts $::fd " "
   puts $::fd "Loading Data Into Array"
   puts $::fd " "
}
update 

set invertList {}
if { [ catch { open $::inversionsFile r } invFileID ] } {
	puts stderr "Cannot open $::inversionsFile"
	return
}
while { [ gets $invFileID line ] >= 0 } {
	set x  [ expr $line ]
	lappend invertList $x
}
close $invFileID

set invert 1

set invertListIdx 0

set invertLatchPos [ lindex $invertList $invertListIdx ]

# clear out latchArr so we only load one chain at a time
if { [ array exists ::latchArr ] } { unset ::latchArr }

set nbits [ INFO CHIP $::chip REG $regNameLower NFLD ]
for { set i 0 } { $i < $nbits } { incr i } {
	set structDesc [ INFO CHIP $::chip REG $regNameLower.$i STRUCT ]
	regexp {(.*):} $structDesc match latchName
	set latchNameLen [ string length $latchName ] 
	if { $i % 1000 == 0 } {
		puts "Checking $i latches / 47070 for inversions "
	}

	set value  [ INFO CHIP $::chip REG $regNameLower.$i OUT ]

	if { $invert == 1 } { 
#		puts "Inverting latch $i : [ string range $latchName [expr $latchNameLen - 40] [ expr $latchNameLen - 1] ] : val= $value" 
	} 
	set value [ expr $value ^ $invert ]
	set ::latchArr($latchName) $value

	if { $i == $invertLatchPos  } {
		set invert [ expr $invert ^ 1 ]
		incr invertListIdx 
		if { $invertListIdx >= [llength $invertList] } { 
			;# puts "ASSERTION FAIL: invertListIdx = $invertListIdx" 
		} else {
			set invertLatchPos [ lindex $invertList $invertListIdx ]
			incr invertLatchPos -1
		}
	}
	update
}


foreach ::fd $::fdlist {
   puts $::fd " "
   puts $::fd "Loaded $i latches Into Array"
   puts $::fd " "
}
update idletasks


# probably want to deallocate the REG here
REG DELETE [ string tolower $regName ]


} ;# proc



#######################################################################
# read_n_print - reads in the latch REG
#                and formats it for dumping to file
#######################################################################
proc read_n_print {chain} {

	global msgf
	global msgWinOut
	
############################### open proper input file
switch $chain {
	0 { set outFile $::outFileChain0; set inFile $::inFileChain0 }
	1 { set outFile $::outFileChain1; set inFile $::inFileChain1 }
	2 { set outFile $::outFileChain2; set inFile $::inFileChain2 }
}
if { [ catch { open $inFile r } inFileID ] } {
	puts stderr "Cannot open $inFile"
	return
}

############################### Add additional chain-specific output file

if { [string length $outFile ] > 0 } {
	if { [catch { open $outFile w } outFileID] } {
		puts stderr "Cannot open $outFile"
		return 
	}
	lappend ::fdlist $outFileID
}
foreach ::fd $::fdlist { puts $::fd \
"         Router Latch dump of chain $chain on [ exec date ] " }
update idletasks

####################################### Start dumping

set valArr u
set curLeafBase u
set curBranch u
set lastLeaf u
set linenum 0
while { [ gets $inFileID line ] >= 0 } {
	incr linenum
	if { $linenum % 1000 == 0 } {
		puts "Processed $linenum / 47070 latches"
	}
	set spLine [ split $line / ]
	set branch [ join [lrange $spLine 0  [expr [llength $spLine] -2]]  / ]
#puts "Branch is $branch"
#puts "CurBranch is $curBranch"
	if { [ string compare $branch $curBranch ] != 0 } {
		if { [string compare $curLeafBase "u"] } {
			for { set i  [ expr [array size valArr] - 1] } \
			                                     { $i >= 0 } { incr i -1 } {
				foreach ::fd $::fdlist {
					puts -nonewline $::fd [ format "%#0x " $valArr($i) ]
				}

			}
		}
		foreach ::fd $::fdlist { puts $::fd ""}
		update 
		set lastLeaf 1
		set curLeafBase u

		foreach ::fd $::fdlist { puts $::fd "$branch" }
		update idletasks
		set curBranch $branch
	}

# process the leaf
# Check for subscript, then make sure it's a numeric subscript, some
# latches are like PI_DE_DATA_reg[21][G] or PI_TOP_Block_reg[E]
	set leaf [ lindex $spLine [ expr [llength $spLine] -1 ] ]
#puts "leaf is $leaf"
	set tmp [ string first \[ $leaf ]
	if { $tmp >= 0 } {
		set leafBase [ string range $leaf 0 [expr $tmp - 1]  ]
	} else {
		if { $::showSingleLatches == 0 } { continue }
		set leafBase $leaf
	}
#puts "leafBase is $leafBase"
	if { [ string compare $leafBase $curLeafBase ] != 0 } {
#puts "curLeafBase is $curLeafBase"
		if { $lastLeaf != 1} {
			if { [string compare $curLeafBase "u"] } {
				for { set i [ expr [array size valArr] - 1] } \
				                                     { $i >= 0 } { incr i -1 } {
					foreach ::fd $::fdlist {
						puts -nonewline $::fd [ format "%#0x " $valArr($i) ]
					}
				}
				foreach ::fd $::fdlist { puts $::fd "" }
				update idletasks
				set curLeafBase u
			}
		} else { set lastLeaf 0 }
		foreach ::fd $::fdlist { puts -nonewline $::fd " +--- $leafBase: "}
		update idletasks
		set curLeafBase $leafBase
		unset valArr
		set valArr(0) 0
	}
#puts "curLeafBase is $curLeafBase"
	set subscript u
	set line1 [ string trim $line ]
	set bit $::latchArr($line1)
	if { [ regexp {\[([0-9]+)\]} $leaf match subscript]  } {
#     puts "Leaf: $leaf  Subscript $subscript"
# get latch value
# calculate
		set numWord [ expr $subscript / 32 ]
#puts "numWord is $numWord"
		if { [expr $numWord + 1] > [array size valArr] } {
			for { set j [ array size valArr] } { $j < $numWord + 1 } {incr j} {
				set valArr($j) 0
			}
		}
		set valArr($numWord) [ expr $valArr($numWord) | $bit << ($subscript % 32) ]
	} else {   ;# no subscript, just a single latch
		set valArr(0) $bit
	}
} ;# while

# flush out last leaf
for { set i [expr [ array size valArr] -1 ] } { $i >= 0 } { incr i -1} {
	foreach ::fd $::fdlist { 
		puts -nonewline $::fd [ format "%#08x " $valArr($i) ] 
	}
}
foreach ::fd $::fdlist { puts $::fd "" }
update idletasks

foreach ::fd $::fdlist {
	puts $::fd "\n\n * * * * * * Dump of chain $chain complete.\n "
}

######################################## close chain-specific output file
if { [string length $::outFileMain ] > 0 } { flush $::outFileMainID }

# if we're only dumping to one file, don't delete chain-specific fd
if { [string length $outFile ] > 0 } { 
	flush $outFileID; close $outFileID 
	set tmp [ expr [ llength $::fdlist ] - 1]
	set tmpfdlist [ lreplace $::fdlist $tmp $tmp ]
	set ::fdlist $tmpfdlist
}

close $inFileID

}   ;# proc

#######################################################################
#######################################################################

#######################################################################
# main code
#######################################################################

	global mod modid brd brdidx chnidx chip
	SEL MOD $::mod
	SEL BRD $::brd
	SEL CHIP $::chip

	RESET -H

puts "Module $mod: Starting latch dump at [ exec date ]"
	
	################################# take care of file I/O
	set ::fdlist {}
	if { [string length $::outFileMain ] > 0 } {
# check for directory
		file mkdir $env(SCAN_USER_DATA)/rtr_dumps
		if { [catch { open $::outFileMain w } ::outFileMainID ] } {
			puts stderr "Cannot open $::outFileMain"
			return
		}
		lappend ::fdlist $::outFileMainID
	}
	foreach ::fd $::fdlist { puts $::fd \
	" ################Router Latch dump of $::mod on [ exec date ] " }
	
	################################ stop clocks


	foreach ::fd $::fdlist { puts $::fd "* * * * * * Stopping clocks..." }
	foreach ::fd $::fdlist { puts $::fd "* * * * * * Setting Mode reg" }
	INST MODE_REG
	GOINST
	DATA 1
	GODATA


	
puts "fdlist is $fdlist"
	# reduces risk of repeating list elems
	if { $::chain0Sel != 0 } {
		foreach ::fd $::fdlist { puts $::fd "* * * * * Dumping chain 0 " }
		dump 0
		read_n_print 0
	}
puts "fdlist is $fdlist"
	if { [string length $::outFileMain ] > 0 } { close $::outFileMainID }

puts "Module $mod: End of Router latch dump at [ exec date ]"

#exit
############################################################################
############################################################################

