##!/usr/local/bin/tclsh 
# for testing
#
# internal.tcl
#
# Kurt Kermes
# 2000/09/18
#
# script for accessing the internal scan chain latches of the 
# SN2TC
#
# User can optionally set a latch name to read/write
#             whether to invert or circulate the data
#
# Mod History: Version 0.5
#
#############################################################################
#
#   (C) COPYRIGHT SILICON GRAPHICS, INC.
#   UNPUBLISHED PROPRIETARY INFORMATION
#   ALL RIGHTS RESERVED UNDER THE COPYRIGHT LAWS OF THE UNITED STATES.
#
#   Portions of this software may still be in development.  The existence
#   of the software still in development is not a commitment of actual
#   release or support by Silicon Graphics, Inc.  Silicon Graphics, Inc.
#   assumes no liability for any damages resulting from attempts to use any
#   product not officially released.  If it is released, the final form of
#   the product and the time of official release is at the discretion of 
#   Silicon Graphics, Inc.
#
#############################################################################
#

############################################################################
# parameter initialization
############################################################################

set mod  SN2TC  
set brd  SN2TB
set chip U1

#chainName - which internal chain are we accessing
set chainNum  0

# latchVal - select which latch to deal with 
set latchVal 0

# chainInv - if you want to invert the value of the chain, do so here
set chainInv 0

# chainCir - if you want to circulate the chain so that the 
#             data coming out goes back into the internal chain
set chainCir 1

# scanActionMask - bit 0 - scan data out; bit 1 scan data in
set scanActionMask 1


############################################################################
############################################################################
# main
############################################################################
############################################################################

SEL MOD  $mod
SEL BRD  $brd
SEL CHIP $chip

# stop clocks
# load scan control parameters

INST SCAN_CTL
GOINST

set dum_ctl_ts 0 ;#
set jsel_a     0 ;#
set jsel_b     1 ;#
set jsel_c     1 ;#
set jsel_tp    4 ;#
set jclkstop   1 ;#
set CIR        0 ;#
set INV        0 ;#
set CL2_config 0 ;#

INST SCAN_CTL
GOINST
DATA $dum_ctl_ts.$jsel_a.$jsel_b.$jsel_c.$jsel_tp.$jclkstop.$CIR.$INV.$CL2_config
GODATA


set chainName SN2TC_CH$chainNum
set chainInst SCAN_TDR$chainNum
set regName $chainName
set regNameLower [ string tolower $regName ] ;# til I fix that bug
REG $regName

if { $scanActionMask & 1} {
# peform a scan-out
	INST SCAN_TDR$chainNum
	GOINST
	puts "scanning out chain..."
	GODATA

	puts "Done scanning out chain"

	puts "Loading data into array..."
	set nbits [ INFO CHIP $chip REG $regNameLower NFLD ]
	for { set i 0 } { $i < $nbits } { incr i } {
		set structDesc [ INFO CHIP $chip REG $regNameLower.$i STRUCT ]
		regexp {(.*):} $structDesc match latchName
		set value     [ INFO CHIP $chip REG $regNameLower.$i OUT ]
		set latcharr($latchName) $value
	}
} 

if { $scanActionMask & 2 } {
	# peform a scan-in	 

# if you want to write the value of the latch, do it here.
	REGDATA $latchName  $latchVal

	INST SCAN_TDR$chainNum
	GOINST
	puts "Scanning in chain"
	GODATA
	puts "Done scanning in chain"

}


puts "internal chain script complete, enter additional commands"
