#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: common_defs.pm,v 1.5 2005/10/18 22:13:07 beighley Exp $


#-------------------------------------------------------------------------------
#
#  FILE: common_defs.pm
#
#  DESCRIPTION:  This file contains PERL constants and subroutines that are
#     common to the SuperScope command modules.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  ----------------------------------------------
#     10/29/01  Karen Beighley  Original source code.
#     12/11/01  Karen Beighley  Minor changes; close all connections on error.
#     02/25/02  Karen Beighley  Added comman error function pointer.
#     ...
#     04/07/03  Karen Beighley  Minor change to close emulator connection.
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  CONSTANTS - The following values *cannot* change!
#-------------------------------------------------------------------------------

$FALSE           = 0;
$TRUE            = 1;

$DESELECT        = 0;
$SELECT          = 1;
$APPEND          = 2;

$MEM_ADDR_INCR    = 8;
$MEM_ADDR_INCR_CL = 128;

$MEM_ACCESS_SMDK = 0;
$MEM_ACCESS_JTAG = 1;
$MEM_ACCESS_PROM = 2;
$MEM_ACCESS_NONE = 3;

$MMR_ACCESS_SMDK = 0;
$MMR_ACCESS_JTAG = 1;
$MMR_ACCESS_PROM = 2;
$MMR_ACCESS_NONE = 3;

$DEBUG_NONE      = 0x00;
$DEBUG_GENERAL   = 0x01;
$DEBUG_FUNCTIONS = 0x02;
$DEBUG_DETAILS   = 0x04;
$DEBUG_VERBOSE   = 0x08;
$DEBUG_ALL       = 0x0F;

$DATA_FMT_DEC    = 0;
$DATA_FMT_OCT    = 1;
$DATA_FMT_HEX    = 2;
$DATA_FMT_HEX8   = 3;
$DATA_FMT_HEX16  = 4;
$DATA_FMT_HEX32  = 5;
$DATA_FMT_HEX64  = 6;

@DATA_FMT_STR = ( "%lld", "%#llo", "0x%llX", "0x%02llX", "0x%04llX", 
                  "0x%08llX", "0x%016llX" );

#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES
#-------------------------------------------------------------------------------

$debug_flag = $DEBUG_FUNCTIONS | $DEBUG_GENERAL;
$debug_flag = 0;

$ecode = 0;
$emess = "";

$comm_err_fun = \&dsp_error_and_exit;


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: dsp_error_and_exit()
#
#-------------------------------------------------------------------------------

sub dsp_error_and_exit
{  my $num = $#_ + 1;

   print "\nERROR - Script execution stopped due to error.\n";

   if ( $num > 0 )
   {  $err_string = $_[0];
      @err_msgs   = split('\n', $err_string);
      foreach $mess (@err_msgs)
      {  print "        $mess\n";
      }
   }
   if ( $num > 1 )
   {  $err_string = $_[1];
      if ( length($err_string) > 1 )
      {  $mess = "Last response in buffer from system maybe useful.\n";
         print "        $mess\n";
         printf "--- START LAST RESPONSE FROM SYSTEM ---\n";
         printf "$err_string\n";
         printf "--- END LAST RESPONSE FROM SYSTEM ---\n";
      }
   }
   print "\n";

   if ( defined($sysco_init) )   # system controller module included?
   {  sysco_close_connection();
   }

   if ( defined($jtag_init) )   # JTAG/scan module included?
   {  jtag_disconnect();
   }

   if ( defined($la_init) )   # logic analyzer module included?
   {  la_disconnect();
   }

   if ( defined($vstation_init) )  # VStation emulator module included?
   {  vstation_server_disconnect();
   }

# call cleanup routine

   exit -1;
}

#-------------------------------------------------------------------------------

sub hash_dump
{  my($cref) = $_[0];
   my(%ccs)  = %$cref;

   print "\nDISPLAYING CONTENTS OF HASH STRUCTURE:\n";
   foreach $key (keys %ccs)
   {  printf "%-20s = $ccs{$key}\n", $key;
   }
}


#-------------------------------------------------------------------------------

1;
