# 
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
# 
# @(#)$Id: connect.pm,v 1.2 2004/01/15 21:24:55 beighley Exp $


#-------------------------------------------------------------------------------
#  CONSTANTS - These values cannot change!
#-------------------------------------------------------------------------------

$FALSE = 0;
$TRUE  = 1;

$SELECT_L2_HARDWARE = 0;
$SELECT_L2_NONE     = 1;


#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES FOR THIS MODULE
#-------------------------------------------------------------------------------

$l2_selection = 0;

my $mw_connect = 0;


#-------------------------------------------------------------------------------

sub create_connect_window
{
   print "\ncreate_connect_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $win_h  = $_[0]; # Top-level window handle
   my $cc_ref = $_[1]; # Reference to cascade menu command option

   #
   #  Enable the cascade menu command
   #
   $$cc_ref->configure(-state => 'disabled');

   #
   #  Create the main window
   #
   $mw_connect = $win_h->Toplevel(-title => "System Controller Connection Window");
   $mw_connect->bind('<Destroy>' => [\&destroy_connect_window, $mw_connect, $cc_ref]);

   #
   #  Create the menubar for the window
   #
   $menubar  = $mw_connect->Menu(-type => 'menubar');
   $mw_connect->configure(-menu => $menubar);
   $casmenu = $menubar->cascade(-label => '~File', -tearoff => 0);
   $casmenu->command(-label => 'Close', -command => [ $mw_connect => 'destroy' ]);

   #
   #  Create the header label for the window
   #
   $frame = $mw_connect->Frame(-borderwidth => 10);
   $label = $frame->Label(-anchor => 'w', -text => 'System Controller Connection for JTAG Operations');
   $label->pack(-side => 'left', -expand => 'no');
   $frame->pack(-side => 'top', -fill => 'both');

   $l2_selection = (($l2_host eq "") ? $SELECT_L2_NONE : $SELECT_L2_HARDWARE);

   #
   #  Create the system controller selection and entry widgets
   #
   $frame = $mw_connect->Frame(-borderwidth => 10);
   $sub_frame = $frame->Frame();
   $btn = $sub_frame->Radiobutton(-text => 'L2 system controller IP address or name:',
            -variable => \$l2_selection, -value => $SELECT_L2_HARDWARE);
   $btn->pack(-side => 'left');
   $entry = $sub_frame->Entry(-width => 20, -textvariable => \$l2_host);
   $entry->pack(-side => 'left', -fill => 'both');
   $sub_frame->pack(-side => 'top', -fill => 'both');
   $sub_frame = $frame->Frame();
   $btn = $sub_frame->Radiobutton(-text => 'Software emulation: no connection to hardware.',
            -variable => \$l2_selection, -value => $SELECT_L2_NONE);
   $btn->pack(-side => 'left');
   $sub_frame->pack(-side => 'top', -fill => 'both');
   $frame->pack(-side => 'top', -fill => 'both');


   #
   #  Create the action buttons at the bottom of the window
   #
   $frame = $mw_connect->Frame(-borderwidth => 10);
   $btn   = $frame->Button(-text => 'Connect', -width => 15,
               -command => [\&cbf_connect_action, $TRUE, \$l2_selection]);
   $btn->pack(-side => 'left', -padx => 5);
   $btn   = $frame->Button(-text => 'Disconnect', -width => 15,
               -command => [\&cbf_connect_action, $FALSE, \$l2_selection]);
   $btn->pack(-side => 'left', -padx => 5);
   $label = $frame->Label(-anchor => 'w', -width => 8);
   $label->pack(-side => 'left', -expand => 'no');
   $btn = $frame->Button(-text => 'Close Window', -width => 15,
             -command => [$mw_connect => 'destroy'] );
   $btn->pack(-side => 'left', -padx => 5);
   $frame->pack(-side => 'top', -fill => 'both');
}

#-------------------------------------------------------------------------------

sub destroy_connect_window
{
   print "\ndestroy_connect_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $rsvd_ref  = $_[0];  # Widget reference when invoked as callback
   my $top_win   = $_[1];  # Reference to top-level window
   my $cc_ref    = $_[2]; # Reference to cascade menu command option

   return if ( $rsvd_ref != $top_win );

   #
   #  Enable the cascade menu command
   #
   $$cc_ref->configure(-state => 'normal');
}


#-------------------------------------------------------------------------------

sub cbf_connect_action
{
   print "\ncbf_connect_action( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $flag   = $_[0];  # Connection flag: TRUE=connect, FALSE=disconnect
   my $select = $_[1];  # Reference to current L2 selection value

   #
   #  User action: connect
   #
   if ( $flag )
   {
      #  Close an exisiting connection (if open)
      if ( defined($jtag_init) )
      {  jtag_disconnect();
      }

      #  Open new connection
     if ( $$select == $SELECT_L2_NONE )  # No L2 hardware: emulation mode
     {  
        cbf_connect_action($FALSE, $select);
     }
     elsif ( $l2_host eq "" )
     {
        printf "\nERROR - INVALID L2 NAME SPECIFIED!\n";

        cbf_connect_action($FALSE, $select);
     }
     else
     {  jtag_connect($JTAG_ACCESS_SYSCO, $l2_host, $l2_port);
        if ( $ecode < 0 )  # Error encountered
        {  $ecode = 0;
           $emess = "";
           cbf_connect_action($FALSE, $select);
        }
        else
        {  mmr_access($MMR_ACCESS_JTAG, $mmr_flag);
        }
     }
   }
   #
   #  User action: disconnect
   #
   else # ( $flag == $FALSE )
   {
      #  Close an exisiting connection (if open)
      if ( defined($jtag_init) )
      {  jtag_disconnect();
      }

      #  Open software emulation connection
      jtag_connect($JTAG_ACCESS_NONE, "localhost", $l2_port);
      mmr_access($MMR_ACCESS_NONE);

      #  Update the selection radiobutton (no hardware)
      $$select = $SELECT_L2_NONE;
   }
}

#-------------------------------------------------------------------------------

1;
