# 
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
#-------------------------------------------------------------------------------
#
#  FILE: dbgsearch.pm
#
#  DESCRIPTION:  This file contains PERL for the testpoint search utility.
#
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     10/17/03  Karen Beighley  Added modification history to source file.
#     11/10/03  Karen Beighley  Minor fix of typo.
#     01/08/04  Karen Beighley  Added functionality to history buttons.
#
#-------------------------------------------------------------------------------
#
# 
# @(#)$Id: dbgsearch.pm,v 1.3 2004/01/08 17:37:14 beighley Exp $

#-------------------------------------------------------------------------------
#  CONSTANTS - These should not change!
#-------------------------------------------------------------------------------

$TSW_WINDOW      =  0;  # TP search window data: main window handle
$TSW_NAME_TEXT   =  1;  # TP search window data: 
$TSW_LEVEL_TEXT  =  2;  # TP search window data: 
$TSW_MODE_TEXT   =  3;  # TP search window data: 
$TSW_MAX_TEXT    =  4;  # TP search window data: 
$TSW_RSLT_WID    =  5;  # TP search window data: 
$TSW_REF_NAMES   =  6;  # TP search window data: 
$TSW_REF_PORTS   =  7;  # TP search window data: 


$TP_SEARCH_MATCH_TEXT   = 0x01;  # Value must match define in C-library
$TP_SEARCH_MATCH_PREFIX = 0x02;  # Value must match define in C-library
$TP_SEARCH_MATCH_SUBSTR = 0x04;  # Value must match define in C-library

$TP_SEARCH_STORE_ALL    = 0x10;  # Value must match define in C-library
$TP_SEARCH_STORE_FIRST  = 0x20;  # Value must match define in C-library


#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES
#-------------------------------------------------------------------------------

my @tp_search_mode_names = ( "begin with search text (prefix)",
                             "match the search text (exact name)",
                             "contain search text (substring)" );

my $tp_search_title = "Debug Port Testpoint Search Utility";


#-------------------------------------------------------------------------------

sub create_tp_search_window
{
   printf "\ncreate_tp_search_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return($FALSE) if ( $#_ != 2 );
   my $main_win     = $_[0]; # Reference to main window for GUI
   my $mdw_ref      = $_[1]; # Reference to main debug window parameters
   my $cmd_ref      = $_[2]; # Reference to command button to disable/enable
   my $chip_name    = $$mdw_ref[$MDWD_CHIP_NAME]; # Full chip name
   my $chip_type    = $$mdw_ref[$MDWD_CHIP_NAME]; # Chip type/description
   my $dpd_list     = $$mdw_ref[$MDWD_DPD_LIST];  # List with debug port information for chip
   my @tsw_data     = (0, "", "", "", 0, 0, 0, 0);
   my $bwidth       = 12;
   my ($win_h, $menubar, $casmenu, $frame, $sub_frame, $label, $entry, 
       $btn, $omenu, $tps_text, $i, $dpd_ref);

   #---------------------------------------
   #  Build the debug port arrays (names and pointers)
   #---------------------------------------
   my @dp_names = ();
   my @dp_ptrs  = ();
   for ($i = 0; $i <= $#$dpd_list; $i++)
   {  $dpd_ref      = $$dpd_list[$i];
      $dp_names[$i] = $$dpd_ref[$DPD_NAME];
      $dp_ptrs[$i]  = $$dpd_ref[$DPD_DPORT_PTR];
   }

   #---------------------------------------
   #  Create main window for selection
   #---------------------------------------
   $text = $tp_search_title . ": $chip_name";
   $win_h = $main_win->Toplevel(-title => $text);
   $win_h->bind('<Destroy>' => [\&destroy_tp_search_window, \@tsw_data, $cmd_ref, $mdw_ref]);
   if ( $$cmd_ref != 0 )
   {  $$cmd_ref->configure(-state => 'disable');
   }
   $$mdw_ref[$MDWD_REF_TSWIN] = $win_h;

   #---------------------------------------
   #  Create the menu bar
   #---------------------------------------
   $menubar  = $win_h->Menu(-type => 'menubar');
   $win_h->configure(-menu => $menubar);
   $casmenu = $menubar->cascade(-label => '~File', -tearoff => 0);
   $casmenu->command(-label => 'Close', -command => [$win_h => 'destroy']);

   #---------------------------------------
   #  Create the main window widgets
   #---------------------------------------
   $frame = $win_h->Frame(-borderwidth => 10);

   $sub_frame = $frame->Frame();
   $label = $sub_frame->Label(-anchor => 'w', -text => 'Debug level:');
   $label->pack(-side => 'left', -fill => 'both');
   $omenu = $sub_frame->Optionmenu(-options => \@dp_names, -width => 30,
               -textvariable => \$tsw_data[$TSW_LEVEL_TEXT]);
   $omenu->pack(-side => 'left');
   $label = $sub_frame->Label(-anchor => 'w', -text => '  ');
   $label->pack(-side => 'left');
   $omenu = $sub_frame->Optionmenu(-options => \@tp_search_mode_names, -width => 30,
               -textvariable => \$tsw_data[$TSW_MODE_TEXT]);
   $omenu->pack(-side => 'right');
   $label = $sub_frame->Label(-anchor => 'w', -text => 'Find testpoints that:');
   $label->pack(-side => 'right', -fill => 'both');
   $sub_frame->pack(-side => 'top', -fill => 'both');

   $label = $frame->Label(-anchor => 'w', -text => "");
   $label->pack(-side => 'top');

   $sub_frame = $frame->Frame();
   $label = $sub_frame->Label(-anchor => 'w', -text => "Search text:");
   $label->pack(-side => 'left');
   $entry = $sub_frame->Entry(-width => 26, -textvariable => \$tsw_data[$TSW_NAME_TEXT]);
   $entry->pack(-side => 'left', -fill => 'both', -expand => 'yes');
   $label = $sub_frame->Label(-anchor => 'w', -text => '  ');
   $label->pack(-side => 'left');
   $entry = $sub_frame->Entry(-width => 7, -textvariable => \$tsw_data[$TSW_MAX_TEXT]);
   $entry->pack(-side => 'right', -fill => 'both');
   $label = $sub_frame->Label(-anchor => 'w', -text => "Display maximum:");
   $label->pack(-side => 'right');

   $sub_frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   $frame->pack(-side => 'top', -fill => 'both');

   #---------------------------------------
   #  Create a frame with the history widgets
   #---------------------------------------
   $frame = $win_h->Frame(-borderwidth => 10);
   $label = $frame->Label(-anchor => 'w', -text => "Search Results:");
   $label->pack(-side => 'top', -expand => 'no', -fill => 'both');
   $tps_text = $frame->Scrolled('ROText', -scrollbars => 'se', -height => 12, -wrap => 'no');
   $tps_text->pack(-side => 'top', -expand => 'yes', -fill => 'both');
   $frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   #---------------------------------------
   #  Create a frame and action buttons
   #---------------------------------------
   $frame = $win_h->Frame(-borderwidth => 10);

   $btn   = $frame->Button(-text => 'Start Search', -width => $bwidth,
      -command => [\&tp_search_cbf_run, \@tsw_data]);
   $btn->pack(-side => 'left', -padx => 5);

   $btn   = $frame->Button(-text => 'Clear Results', -width => $bwidth,
      -command => [\&tp_search_cbf_hist_btn, \@tsw_data, 'clear']);
   $btn->pack(-side => 'left', -padx => 5);

   $btn   = $frame->Button(-text => 'Save Results', -width => $bwidth,
      -command => [\&tp_search_cbf_hist_btn, \@tsw_data, 'save']);
   $btn->pack(-side => 'left', -padx => 5);

   $btn = $frame->Button(-text => 'Close Window', -width => $bwidth, -command => [$win_h => 'destroy']);
   $btn->pack(-side => 'right', -padx => 5);

   $frame->pack(-side => 'top', -fill => 'both');

   #---------------------------------------
   #  Store widget handles and variables
   #---------------------------------------
   $tsw_data[$TSW_WINDOW]     = $win_h;
   $tsw_data[$TSW_NAME_TEXT]  = "";
   $tsw_data[$TSW_LEVEL_TEXT] = $dp_names[0];
   $tsw_data[$TSW_MODE_TEXT]  = $tp_search_mode_names[0];
   $tsw_data[$TSW_MAX_TEXT]   = 25;
   $tsw_data[$TSW_RSLT_WID]   = $tps_text;
   $tsw_data[$TSW_REF_NAMES]  = \@dp_names;
   $tsw_data[$TSW_REF_PORTS]  = \@dp_ptrs;

   return(\@tsw_data);
}


#------------------------------------------------------------------------------

sub destroy_tp_search_window
{
   printf "\ndestroy_tp_search_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $curr_wid = $_[0]; # Current widget/window handle from Tk
   my $tsw_ref  = $_[1]; # Reference to group window data
   my $cmd_ref  = $_[2]; # Command button reference
   my $mdw_ref  = $_[3]; # Reference to main debug window parameters
   my $win      = $$tsw_ref[$TSW_WINDOW];

   return if ( $curr_wid != $win ); # Return if not main window

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "tsw_ref = $tsw_ref = ( @$tsw_ref )\n";
      printf "mdw_ref = $mdw_ref = ( @$mdw_ref )\n";
      printf "cmd_ref = $cmd_ref -> $$cmd_ref\n";
   }

   #---------------------------------------
   #  Clear out the GUI widget handles
   #---------------------------------------
   $$tsw_ref[$TSW_WINDOW]     = 0;
   $$mdw_ref[$MDWD_REF_TSWIN] = 0;

   #---------------------------------------
   #  Enable button from parent window
   #---------------------------------------
   if ( $$cmd_ref != 0 )
   {  $$cmd_ref->configure(-state => 'normal');
   }
}


#-------------------------------------------------------------------------------

sub tp_search_cbf_hist_btn
{
   printf "\ntp_search_cbf_hist_btn( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $tsw_ref  = $_[0]; # Reference to testpoint search window data
   my $mode     = $_[1];  # Mode: 'save' or 'clear'
   my $win_h    = $$tsw_ref[$TSW_WINDOW];
   my $rslt_wid = $$tsw_ref[$TSW_RSLT_WID];

   #---------------------------------------
   #  Clear the command history buffer
   #---------------------------------------
   if ( $mode eq 'clear' )
   {  $rslt_wid->delete('1.0', 'end');
      return;
   }

   #---------------------------------------
   #  Continue: save the history buffer
   #---------------------------------------
   my ($out_file, $file_id, $text);

   #---------------------------------------
   #  Get the contents of the command history
   #---------------------------------------
   $text = $rslt_wid->get('1.0', 'end');

   #---------------------------------------
   #  Get the output file name from user
   #---------------------------------------
   $out_file = $win_h->getSaveFile();
   return if ( !defined($out_file) );

   #---------------------------------------
   #  Open the output file and write the text
   #---------------------------------------
   $file_id = sprintf("hist%08lX", time());
   if ( !open($file_id, ">$out_file") )
   {  $text = "Unable to open output file for writing.\nFile: $out_file\nCommand history not saved.";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
         -title => 'ICC Monitor: open file error');
      return;
   }
   printf $file_id $text;
   close($file_id);
  
   print "\nSaved search results: $out_file\n";
}


#-------------------------------------------------------------------------------

sub tp_search_cbf_run
{
   printf "\ntp_search_cbf_run( @_ )\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   my $tsw_ref  = $_[0]; # Reference to testpoint search window data
   my $win_h    = $$tsw_ref[$TSW_WINDOW];
   my $dp_names = $$tsw_ref[$TSW_REF_NAMES];
   my $dp_ptrs  = $$tsw_ref[$TSW_REF_PORTS];
   my $rslt_wid = $$tsw_ref[$TSW_RSLT_WID];
   my $mode     = $TP_SEARCH_STORE_FIRST;
   my $max_tp   = $$tsw_ref[$TSW_MAX_TEXT] + 0;
   my ($i, $found, $text, $name);

   #---------------------------------------
   #  Check the output file name
   #---------------------------------------
   $name = $$tsw_ref[$TSW_NAME_TEXT];
   $name =~ s/^\s+//g;  # Remove all leading whitespace
   $name =~ s/\s+$//g;  # Remove all trailing whitespace
   if ( length($name) < 1 )
   {  $text  = "Invalid or missing search string specified.";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
         -title => "Error: $tp_search_title");
      return;
   }
   if ( ($max_tp <= 0) || ( $max_tp > 1000) )
   {  $text  = "Invalid or missing display maximum specified.\nMaximum must be in range: (1 <= max <= 1000).";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
         -title => "Error: $tp_search_title");
      return;
   }

   $text = "################################################################\n";
   $rslt_wid->insert('end', $text);
   $text = "###   BEGIN DEBUG PORT TESTPOINT SEARCH                      ###\n";
   $rslt_wid->insert('end', $text);
   $text = "################################################################\n";
   $rslt_wid->insert('end', $text);
   $text = "\n";
   $rslt_wid->insert('end', $text);
   $text = sprintf("Debug level: %s\n", $$tsw_ref[$TSW_LEVEL_TEXT]);
   $rslt_wid->insert('end', $text);
   $text = sprintf("Search text: \"%s\"\n", $name);
   $rslt_wid->insert('end', $text);
   $text = sprintf("Search mode: match testpoints that %s\n", $$tsw_ref[$TSW_MODE_TEXT]);
   $rslt_wid->insert('end', $text);
   $text = "Display max: $max_tp testpoints\n";
   $rslt_wid->insert('end', $text);
   $rslt_wid->see('end');

   #---------------------------------------
   #  Find the debug port index
   #---------------------------------------
   for ($i = 0, $found = $FALSE; (($i <= $#$dp_names) && !$found); $i++)
   {  if ( $$dp_names[$i] eq $$tsw_ref[$TSW_LEVEL_TEXT] )
      {  $found = $TRUE;
      }
   }
   $i = ( !$found ? 0 : ($i - 1) );
   
   #---------------------------------------
   #  Determine the search mode
   #---------------------------------------
   if ( $$tsw_ref[$TSW_MODE_TEXT] =~ /^match/ )
   {  $mode |= $TP_SEARCH_MATCH_TEXT;
   }
   elsif ( $$tsw_ref[$TSW_MODE_TEXT] =~ /^contain/ )
   {  $mode |= $TP_SEARCH_MATCH_SUBSTR;
   }
   else
   {  $mode |= $TP_SEARCH_MATCH_PREFIX;
   }

   #---------------------------------------
   #  Perform the testpoint search
   #---------------------------------------
   my @list = ();
   my $num = pcf_dport_testpoint_search($$dp_ptrs[$i], $name, $mode, $max_tp, \@list);

   $tmp_text = (($max_tp == $num) ?  " (display maximum reached)" : "" );
   if ( $num == 0 )
   {  $text = "\n-> No matching testpoints were found during search.\n";
   }
   elsif ( $num == 1 )
   {  $text = sprintf("\n-> Found %u matching testpoint\n", $num);   
   }
   else
   {  $text = sprintf("\n-> Found %u matching testpoints\n", $num);   
   }
   $rslt_wid->insert('end', $text);

   for ($i = 0; $i < $num; $i++)
   {  $rslt_wid->insert('end', $list[$i]);
      $rslt_wid->insert('end', "\n");
   }
   $rslt_wid->insert('end', "\n");
   $rslt_wid->see('end');
}


#-------------------------------------------------------------------------------

