# 
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
# 
# @(#)$Id: drawplot.pm,v 1.2 2003/04/01 17:35:21 beighley Exp $
#-------------------------------------------------------------------------------
#
#  FILE: drawplot.pm
#
#  DESCRIPTION:  This file contains PERL constants and subroutines that are
#     needed to create a window with a simple plot or histogram.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  ----------------------------------------------
#     11/20/02  Karen Beighley  Header added for initial file submission
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  CONSTANTS - These cannot be changed!!!
#-------------------------------------------------------------------------------

$DGW_MODE_NONE   = 0x00;  # Dataset display mode: no mode bits set
$DGW_MODE_SHOW   = 0x01;  # Dataset display mode: show data points
$DGW_MODE_LABELS = 0x02;  # Dataset display mode: show value labels
$DGW_MODE_LINES  = 0x04;  # Dataset display mode: draw lines between data points

$DGW_WINDOW     =  0;  # Data graph window: main window handle
$DGW_CANVAS     =  1;  # Data graph window: canvas that contains the graph
$DGW_CB_XGRID   =  2;  # Data graph window: 
$DGW_CB_YGRID   =  3;  # Data graph window: 
$DGW_CB_LABELS  =  4;  # Data graph window: 
$DGW_NUM_BASE   =  5;  # Data graph window: 
$DGW_DATA_REF   =  6;  # Data graph window: reference to array of pointers
$DGW_TEXT_LO    =  7;  # Data graph window: text variable for entry
$DGW_TEXT_UP    =  8;  # Data graph window: text variable for entry
$DGW_TEXT_INT   =  9;  # Data graph window: text variable for entry
$DGW_X_NUM      = 10;  # Data graph window: number of entries on X-axis
$DGW_X_INCR     = 11;  # Data graph window: position increment between X-axis entries
$DGW_X_BASE     = 12;  # Data graph window: base position of X-axis on canvas
$DGW_Y_LO_BOUND = 13;  # Data graph window: lower-bound on Y-axis
$DGW_Y_UP_BOUND = 14;  # Data graph window: upper-bound on Y-axis
$DGW_Y_INTERVAL = 15;  # Data graph window: interval for marking Y-axis
$DGW_Y_BASE     = 16;  # Data graph window: base position of Y-axis on canvas
$DGW_Y_SPAN     = 17;  # Data graph window: total span of Y-axis on canvas

@avail_colors = ( 'blue', 'red', 'purple', 'brown', 'orange', 'bisque4', 'SeaGreen4', 'sienna2', 'DeepPink3',
                  'gray60', 'green2', 'DeepSkyBlue2', 'magenta1', 'cornsilk3', 'Gold2', 'SlateBlue3');

#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES
#-------------------------------------------------------------------------------

$hist_dflt_x_incr  = 45;
$graph_dflt_x_incr = 40;

$default_x_base = 100;

$default_y_base = 100;
$default_y_span = 600;

$default_y_lo_bound = 0;   # Y-axis start value 
$default_y_up_bound = 200; # Maximum value for Y-axis
$default_y_interval = 20;  # Value per Y-axis mark: should divide equally into ($y_up_bound - $y_lo_bound)


#-------------------------------------------------------------------------------

sub create_histogram_window
{
   printf "\ncreate_histogram_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $data_ref  = $_[0];  # Reference to HASH structure with parameters
   my $title     = $_[1];  # Title for the window
   my $sub_close = $_[2];  # Reference to subroutine to call when closed
   my ($win_h, $menubar, $casmenu, $casmenu2, $curr_color, $canvas, $can_frame, 
       $frame, $label, $entry, $bwidth, $btn, $i);
   my $val_ref    = $$data_ref{VALS};  # Reference to array of variable names
   my ($y_lo_bound, $y_up_bound, $y_interval) = histogram_calc_y_default($val_ref);

   #---------------------------------------
   #  Initialize array for window variables
   #---------------------------------------
   my $num_base = 'dec';
   my @dg_data  = ();
   $dg_data[$DGW_CB_XGRID]   = 0;
   $dg_data[$DGW_CB_YGRID]   = 1;
   $dg_data[$DGW_CB_LABELS]  = 1;
   $dg_data[$DGW_NUM_BASE]   = $num_base;
   $dg_data[$DGW_DATA_REF]   = $data_ref;
   $dg_data[$DGW_TEXT_LO]    = $y_lo_bound;
   $dg_data[$DGW_TEXT_UP]    = $y_up_bound;
   $dg_data[$DGW_TEXT_INT]   = $y_interval;
   $dg_data[$DGW_X_NUM]      = $#$val_ref + 1;
   $dg_data[$DGW_X_INCR]     = $hist_dflt_x_incr;
   $dg_data[$DGW_X_BASE]     = $default_x_base;
   $dg_data[$DGW_Y_BASE]     = $default_y_base;
   $dg_data[$DGW_Y_SPAN]     = $default_y_span;
   $dg_data[$DGW_Y_LO_BOUND] = $y_lo_bound;
   $dg_data[$DGW_Y_UP_BOUND] = $y_up_bound;
   $dg_data[$DGW_Y_INTERVAL] = $y_interval;

   if ( !exists($$data_ref{COLOR}) )
   {  $$data_ref{COLOR} = $avail_colors[0];
   }

   #---------------------------------------
   #  Create the main window
   #---------------------------------------
   $win_h = MainWindow->new(-title => $title);
   $win_h->bind('<Destroy>' => [\&destroy_histogram_window, \@dg_data, $sub_close]);

   #---------------------------------------
   #  Create the menu bar
   #---------------------------------------
   $menubar  = $win_h->Menu(-type => 'menubar');
   $win_h->configure(-menu => $menubar);
   $casmenu = $menubar->cascade(-label => '~File', -tearoff => 0);
   $casmenu->command(-label => 'Close', -command => [$win_h => 'destroy']);
   $casmenu  = $menubar->cascade(-label => '~Options', -tearoff => 0);
   $casmenu->checkbutton(-label => "Display X-axis background lines", -variable => \$dg_data[$DGW_CB_XGRID],
      -command => [\&histogram_cb_xygrid, \@dg_data, \$dg_data[$DGW_CB_XGRID], 'X']);
   $casmenu->checkbutton(-label => "Display Y-axis background lines", -variable => \$dg_data[$DGW_CB_YGRID],
      -command => [\&histogram_cb_xygrid, \@dg_data, \$dg_data[$DGW_CB_YGRID], 'Y']);
   $casmenu->checkbutton(-label => "Display value labels", -variable => \$dg_data[$DGW_CB_LABELS],
      -command => [\&histogram_cb_labels, \@dg_data, \$dg_data[$DGW_CB_LABELS]]);
   $curr_color = $$data_ref{COLOR};
   $casmenu2 = $casmenu->cascade(-label => "Graph color", -tearoff => 0);
   for ($i = 0; $i <= $#avail_colors; $i++)
   {   $casmenu2->radiobutton(-label => $avail_colors[$i], -variable => \$curr_color, 
          -value => $avail_colors[$i], command => [\&histogram_cbf_set_color, \@dg_data, \$curr_color ]);
   }
   $casmenu = $menubar->cascade(-label => '~Settings', -tearoff => 0);
   $casmenu2 = $casmenu->cascade(-label => 'Numeric base', -tearoff => 0);
   for ($i = 0; $i <= $#base_opt_labs; $i++)
   {  $casmenu2->radiobutton(-label => $base_opt_labs[$i],
         -value => $base_opt_vals[$i], -variable => \$num_base,
         -command => [\&histogram_cbf_set_num_base, \@dg_data, \$num_base]);
   }

   #---------------------------------------
   #  Create the canvas for the graph
   #---------------------------------------
   $canvas = $win_h->Scrolled('Canvas', -scrollbars => 'se',
                -scrollregion => ['0c', '0c', '30c', '20c'],
                -borderwidth => 2, -relief => 'sunken');
   $can_frame = $canvas->Frame();

   #---------------------------------------
   #  Store window data for future use
   #---------------------------------------
   $dg_data[$DGW_WINDOW]     = $win_h;
   $dg_data[$DGW_CANVAS]     = $canvas;

   #---------------------------------------
   #  Draw the graph on the canvas
   #---------------------------------------
   histogram_draw(\@dg_data);

   #---------------------------------------
   #  Pack the frame and canvas
   #---------------------------------------
   $can_frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');
   $canvas->create('window', 4, 4, -anchor => 'nw', -window => $can_frame);
   $canvas->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   #---------------------------------------
   #  Create the Y-axis parameter widgets
   #---------------------------------------
   $frame = $win_h->Frame(-borderwidth => 10);
   $label = $frame->Label(-text => 'Y-axis Parameters:', -anchor => 'w');
   $label->pack(-side => 'left');
   $label = $frame->Label(-text => 'Lower bound=', -width => 12, -anchor => 'e');
   $label->pack(-side => 'left');
   $entry = $frame->Entry(-width => 12, -textvariable => \$dg_data[$DGW_TEXT_LO]);
   $entry->pack(-side => 'left', -expand => 'yes', -fill => 'both');
   $label = $frame->Label(-text => 'Upper bound=', -width => 12, -anchor => 'e');
   $label->pack(-side => 'left');
   $entry = $frame->Entry(-width => 12, -textvariable => \$dg_data[$DGW_TEXT_UP]);
   $entry->pack(-side => 'left', -expand => 'yes', -fill => 'both');
   $label = $frame->Label(-text => 'Mark increment=', -width => 15, -anchor => 'e');
   $label->pack(-side => 'left');
   $entry = $frame->Entry(-width => 12, -textvariable => \$dg_data[$DGW_TEXT_INT]);
   $entry->pack(-side => 'left', -expand => 'yes', -fill => 'both');
   $frame->pack(-side => 'top', -fill => 'both');

   #---------------------------------------
   #  Create the action buttons
   #---------------------------------------
   $bwidth = 16;
   $frame = $win_h->Frame(-borderwidth => 10);
   $btn = $frame->Button(-text => 'Apply Parameters', -width => $bwidth,
             -command => [\&histogram_cbf_btn_apply, \@dg_data]);
   $btn->pack(-side => 'left');
   $btn = $frame->Button(-text => 'Default Parameters', -width => $bwidth,
             -command => [\&histogram_cbf_btn_default, \@dg_data]);
   $btn->pack(-side => 'left', -padx => 10);
   $btn = $frame->Button(-text => 'Close Window', -width => $bwidth,
             -command => [$win_h => 'destroy']);
   $btn->pack(-side => 'right');
   $frame->pack(-side => 'top', -fill => 'both');

   return(\@dg_data);
}

#-------------------------------------------------------------------------------

sub destroy_histogram_window
{
   printf "\ndestroy_histogram_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $obj_ref   = $_[0];  # Widget or window object being destroyed
   my $dgw_ref   = $_[1];  # Reference to data graph window structure
   my $sub_close = $_[2];  # Reference to subroutine
   my $win_ref   = $$dgw_ref[$DGW_WINDOW];    # Main window for graph

   return if ( $obj_ref != $win_ref );

   #---------------------------------------
   #  Call "cancel" function (if defined)
   #---------------------------------------
   if ( defined($sub_close) && (ref($sub_close) eq 'CODE') )
   {  &$sub_close($dgw_ref);
   }

   $$dgw_ref[$DGW_WINDOW] = 0;

   # Destroy the structure pointed to by "dgw_ref" ???
}

#-------------------------------------------------------------------------------

sub histogram_draw
{
   printf "\nhistogram_draw( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $canvas   = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my $data_ref = $$dgw_ref[$DGW_DATA_REF];  # Reference to array of pointers
   my $var_ref  = $$data_ref{VARS};  # Reference to array of variable names
   my $val_ref  = $$data_ref{VALS};  # Reference to array of frequency counters
   my ($mode, $color, $i, $x, $y, $px, $py, $text, $item, $x_num, $x_incr, $x_base, 
       $x_max, $y_base, $y_span, $y_lo_bound, $y_up_bound, $y_interval, $y_num, $y_incr, $y_max);
   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "dgw_ref = $dgw_ref = ( @$dgw_ref )\n";
      printf "var_ref = $var_ref = ( @$var_ref )\n";
      printf "val_ref = $val_ref = ( @$val_ref )\n";
   }

   #---------------------------------------
   #  Extract/calculate X-axis details
   #---------------------------------------
   $x_num      = $$dgw_ref[$DGW_X_NUM];
   $x_incr     = $$dgw_ref[$DGW_X_INCR];
   $x_base     = $$dgw_ref[$DGW_X_BASE];
   $x_max      = $x_base + ($x_incr * ($x_num + 1));

   #---------------------------------------
   #  Extract/calculate Y-axis details
   #---------------------------------------
   $y_base     = $$dgw_ref[$DGW_Y_BASE];
   $y_span     = $$dgw_ref[$DGW_Y_SPAN];
   $y_lo_bound = $$dgw_ref[$DGW_Y_LO_BOUND];
   $y_up_bound = $$dgw_ref[$DGW_Y_UP_BOUND];
   $y_interval = $$dgw_ref[$DGW_Y_INTERVAL];
   $y_num      = (($y_up_bound - $y_lo_bound) / $y_interval);  # Number of marks on the Y-axis
   $y_incr     = ($y_span / $y_num);
   $y_max      = $y_base + $y_span;

   #---------------------------------------
   #  Create labels/lines on the X axis (horizontal)
   #---------------------------------------
   for ($i = 0; $i <= $x_num; $i++) 
   {  $x = $x_base + (($i + 1) * $x_incr);
      if ( $$dgw_ref[$DGW_CB_XGRID] )
      {  $item = $canvas->createLine($x, $y_max, $x, $y_base, -width => 1, -fill => 'grey');
         $canvas->addtag('xgrid', 'withtag', $item);
      }
      $item = $canvas->createLine($x, $y_max, $x, ($y_max - 5), -width => 3);
      $canvas->addtag('border', 'withtag', $item);
      $text = $$var_ref[$i];
      $item = $canvas->createText($x, ($y_max + 10), -text => $text); #, -anchor => 'n',
      $canvas->addtag('border', 'withtag', $item);
   }
   $x = $x_base + ($i * $x_incr);
   if ( $$dgw_ref[$DGW_CB_XGRID] )
   {  $item = $canvas->createLine($x, $y_max, $x, $y_base, -width => 1, -fill => 'grey');
      $canvas->addtag('xgrid', 'withtag', $item);
   }
   $item = $canvas->createLine($x, $y_max, $x, ($y_max - 5), -width => 3);
   $canvas->addtag('border', 'withtag', $item);

   #---------------------------------------
   #  Create labels/lines on the Y axis (veritcal)
   #---------------------------------------
   $i = 0;
   $y = $y_max - ($i * $y_incr);
   $text = convert_num2text((($i * $y_interval) + $y_lo_bound), 32, $$dgw_ref[$DGW_NUM_BASE]);
   $item = $canvas->createText(($x_base - 4), $y, -text => $text, -anchor => 'e');
   $canvas->addtag('border', 'withtag', $item);
   for ($i = 1; $i <= $y_num; $i++) 
   {  $y = $y_max - ($i * $y_incr);
      if ( $$dgw_ref[$DGW_CB_YGRID] )
      {  $item = $canvas->createLine($x_base, $y, $x_max, $y, -width => 1, -fill => 'grey');
         $canvas->addtag('ygrid', 'withtag', $item);
      }
      $item = $canvas->createLine($x_base, $y, ($x_base + 5), $y, -width => 3);
      $canvas->addtag('border', 'withtag', $item);
      $text = convert_num2text((($i * $y_interval) + $y_lo_bound), 32, $$dgw_ref[$DGW_NUM_BASE]);
      $item = $canvas->createText(($x_base - 4), $y, -text => $text, -anchor => 'e');
      $canvas->addtag('border', 'withtag', $item);
   }
   if ( $$dgw_ref[$DGW_CB_YGRID] )
   {  $item = $canvas->createLine($x_base, $y_base, $x_max, $y_base, -width => 1, -fill => 'grey');
      $canvas->addtag('ygrid', 'withtag', $item);
   }
   $item = $canvas->createLine($x_base, $y_base, ($x_base + 5), $y_base, -width => 3);
   $canvas->addtag('border', 'withtag', $item);
   $text = convert_num2text($y_up_bound, 32, $$dgw_ref[$DGW_NUM_BASE]);
   $item = $canvas->createText(($x_base - 4), $y_base, -text => $text, -anchor => 'e');
   $canvas->addtag('border', 'withtag', $item);

   #---------------------------------------
   #  Create outside border lines
   #---------------------------------------
   $item = $canvas->createLine($x_base, $y_max,  $x_max,  $y_max, -width => 2);
   $canvas->addtag('border', 'withtag', $item);
   $item = $canvas->createLine($x_base, $y_base, $x_base, $y_max, -width => 2);
   $canvas->addtag('border', 'withtag', $item);

   #---------------------------------------
   #  Create the title line(s)
   #---------------------------------------
   $text = ( exists($$data_ref{TITLE}) ? $$data_ref{TITLE} : "Histogram" );
   $item = $canvas->createText(($x_base + $x_incr), ($y_base - 60), -text => $text, 
              -justify => 'left', -anchor => 'w');
   $canvas->addtag('border', 'withtag', $item);
   if ( exists($$data_ref{SUBTITLE}) )
   {  $item = $canvas->createText(($x_base + $x_incr + 10), ($y_base - 45), -text => $$data_ref{SUBTITLE},
                 -justify => 'left', -anchor => 'w');
      $canvas->addtag('border', 'withtag', $item);
   }

   #---------------------------------------
   #  Process each variable in dataset
   #---------------------------------------
   $color = $$data_ref{COLOR}; # Color for this specific dataset
   $mode  = $$data_ref{MODE};  # Mode/parameters for this specific dataset

   #---------------------------------------
   #  Display the variables and values
   #---------------------------------------
   for ($i = 0; $i < $x_num; $i++)
   {  $px   = $i + 1;           # Variable/sample number
      $py   = $$val_ref[$i];    # Actual counter value for this sample
      $text = convert_num2text($py, 4, $$dgw_ref[$DGW_NUM_BASE]);
      if ( $py > $y_up_bound )
      {  printf("WARNING - The following data point exceeds upper bound for the Y-axis:");
         printf " ( 0x%08X, 0x%08X ) = ( %lu, %lu )\n", $px, $py, $px, $py;
         $y = ($y_base - 20);
      }
      elsif ( $py < $y_lo_bound )
      {  printf("WARNING - The following data point below lower bound for the Y-axis:");
         printf " ( 0x%08X, 0x%08X ) = ( %lu, %lu )\n", $px, $py, $px, $py;
         $y  = $y_max;
         $py = $py - $y_lo_bound;
      }
      else
      {  $py = $py - $y_lo_bound;
         $y = $y_max  - (($py * $y_incr) / $y_interval);
      }

      $x = $x_base + ($px * $x_incr);
      if ( ($py > 0) && ($y <= ($y_max - 2)) )
      {  $item = $canvas->createRectangle($x-5, $y_max-2, $x+5, $y, -width => 1, 
                    -outline => $color, -fill => $color);
         $canvas->addtag("dataset", 'withtag', $item);
      }
      if ( $mode & $DGW_MODE_LABELS )
      {  $item = $canvas->createText($x, ($y-10), -text => $text, -fill => $color);
         $canvas->addtag("labels", 'withtag', $item);
      }
   }


   #---------------------------------------
   #  Display a legend of current values
   #---------------------------------------
   $x = $x_max  + $x_incr + 10;
   $y = $y_base;
   $item = $canvas->createText($x+15, $y, -text => "Current Sample Value:", -justify => 'left');
   $canvas->addtag('border', 'withtag', $item);
   for ($i = 0, $y += 20; $i < $x_num; $i++, $y += 20)
   {  $val  = $$val_ref[$i];    # Actual counter value for this sample

      $text = sprintf("%4s =", $$var_ref[$i]);
      $item = $canvas->createText($x, $y, -text => $text, -justify => 'right', -anchor => 'e');
      $canvas->addtag('border', 'withtag', $item);

      $text = convert_num2text($val, 4, $$dgw_ref[$DGW_NUM_BASE]);
      $item = $canvas->createText($x+80, $y, -text => $text, -justify => 'right', -anchor => 'e');
      $canvas->addtag('border', 'withtag', $item);
   }

}


#-------------------------------------------------------------------------------

sub histogram_refresh
{
   print "\nhistogram_refresh( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph

   #---------------------------------------
   #  Delete the existing graph
   #---------------------------------------
   $canvas->delete('xgrid');
   $canvas->delete('ygrid');
   $canvas->delete('dataset');
   $canvas->delete('labels');
   $canvas->delete('border');

   #---------------------------------------
   #  Redraw the graph with the new color
   #---------------------------------------
   histogram_draw($dgw_ref); # Draw data on the canvas
}


#-------------------------------------------------------------------------------

sub histogram_cb_xygrid
{
   print "\nhistogram_cb_xygrid( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $vref    = $_[1];  # Reference to checkbutton variablee
   my $type    = $_[2];  # Reference to checkbutton variablee
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my $tagname;

   if ( $$vref )  # Checkbutton is TRUE
   {  histogram_draw($dgw_ref); # Draw data on the canvas
   }
   else  # Checkbutton is FALSE
   {  $tagname = ( ($type eq 'X') ? 'xgrid' : 'ygrid' );
      $canvas->delete($tagname);
   }
}

#-------------------------------------------------------------------------------

sub histogram_cb_labels
{
   print "\nhistogram_cb_labels( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $vref    = $_[1];  # Reference to checkbutton variablee
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph

   if ( $$vref )  # Checkbutton is TRUE
   {  histogram_draw($dgw_ref); # Draw data on the canvas
   }
   else  # Checkbutton is FALSE
   {  $canvas->delete("labels");
   }
}

#-------------------------------------------------------------------------------

sub histogram_cbf_btn_apply
{
   print "\nhistogram_cbf_btn_apply( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $win_h   = $$dgw_ref[$DGW_WINDOW];    # Main window for graph
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph

   #---------------------------------------
   #  Extract user-specified parameters
   #---------------------------------------
   my $tmp_lo  = convert_text2num($$dgw_ref[$DGW_TEXT_LO]);
   my $tmp_up  = convert_text2num($$dgw_ref[$DGW_TEXT_UP]);
   my $tmp_int = convert_text2num($$dgw_ref[$DGW_TEXT_INT]);

   #---------------------------------------
   #  Check the parameters: valid?
   #---------------------------------------
   if ( $tmp_up <= $tmp_lo )
   {  $emess = "Invalid Y-axis upper and lower bounds: lower bound must be less than the upper bound.\n";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $emess,
               -title => 'Y-axis Parameter Error');
      return;
   }

   #---------------------------------------
   #  Check the "mark increment"
   #---------------------------------------
   $num_int = int( (($tmp_up - $tmp_lo) / $tmp_int) + 0.5 );
   if ( $num_int > 40 )
   {  $text = sprintf("The number of Y-axis intervals exeeds the recommended limit of 40.\nWith specified mark value, there would be %u intevals, which may take significant time to draw/refresh. Do you want to apply these settings?", $num_int);
      $rv   = $win_h->messageBox(-icon => 'question', -type => 'YesNo', -message => $text,
              -title => 'Y-axis Parameter Question');
      return if ( $rv eq 'No' );
   }

   #---------------------------------------
   #  Store the new parameters
   #---------------------------------------
   $$dgw_ref[$DGW_Y_LO_BOUND] = $tmp_lo;
   $$dgw_ref[$DGW_Y_UP_BOUND] = $tmp_up;
   $$dgw_ref[$DGW_Y_INTERVAL] = $tmp_int;

   #---------------------------------------
   #  Delete the existing graph and redraw
   #---------------------------------------
   histogram_refresh($dgw_ref);
}

#-------------------------------------------------------------------------------

sub histogram_cbf_btn_default
{
   print "\nhistogram_cbf_btn_default( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $data_ref = $$dgw_ref[$DGW_DATA_REF];  # Reference to array of pointers
   my $val_ref  = $$data_ref{VALS};  # Reference to array of frequency counters
   my ($y_lo_bound, $y_up_bound, $y_interval) = histogram_calc_y_default($val_ref);

   #---------------------------------------
   #  Reset the Y-axis parameters
   #---------------------------------------
   $$dgw_ref[$DGW_TEXT_LO]  = $y_lo_bound;
   $$dgw_ref[$DGW_TEXT_UP]  = $y_up_bound;
   $$dgw_ref[$DGW_TEXT_INT] = $y_interval;
}

#-------------------------------------------------------------------------------

sub histogram_cbf_set_color
{
   print "\nhistogram_cbf_set_color( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $vref     = $_[1];  # Reference to variable with selected color
   my $canvas   = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my $data_ref = $$dgw_ref[$DGW_DATA_REF];  # Reference to array of pointers
   
   #---------------------------------------
   #  Delete the existing data and labels
   #---------------------------------------
   $canvas->delete('dataset');
   $canvas->delete('labels');

   #---------------------------------------
   #  Set the new color
   #---------------------------------------
   $$data_ref{COLOR} = $$vref;

   #---------------------------------------
   #  Redraw the graph with the new color
   #---------------------------------------
   histogram_draw($dgw_ref); # Draw data on the canvas
}

#-------------------------------------------------------------------------------

sub histogram_cbf_set_num_base
{
   print "\nhistogram_cbf_set_num_base( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $vref     = $_[1];  # Reference to variable with selected color
   
   #---------------------------------------
   #  Set the new numeric base
   #---------------------------------------
   $$dgw_ref[$DGW_NUM_BASE] = $$vref;

   #---------------------------------------
   #  Redraw the graph with the new parameter
   #---------------------------------------
   histogram_refresh($dgw_ref); # Draw data on the canvas
}

#-------------------------------------------------------------------------------

sub histogram_calc_y_default
{  
   printf "\nhistogram_calc_y_default( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $val_ref = $_[0];  # Reference to array with histogram values 
   my ($i, $min, $max, $tmp);

   #---------------------------------------
   #  Find the minimum and maximum values
   #---------------------------------------
   $min = 0;
   $max = 0;
   for ($i = 0; $i <= $#$val_ref; $i++)
   {  if ( $$val_ref[$i] < $min )
      {  $min = $$val_ref[$i];
      }
      if ( $$val_ref[$i] > $max )
      {  $max = $$val_ref[$i];
      }
   }

   #---------------------------------------
   #  Round maximum Y-value to next 100
   #---------------------------------------
   $tmp = int($max / 100);
   if ( ($tmp == 0) || (($tmp % 100) != 0) )
   {  $tmp++;
   }
   $tmp *= 100;
   return(0, $tmp, ($tmp / 10));
}

################################################################################

sub create_graph_window
{
   printf "\ncreate_graph_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $data_ref  = $_[0];  # Reference to array of HASH pointers
   my $title     = $_[1];  # Title for the window
   my $sub_close = $_[2];  # Reference to subroutine to call when closed
   my $def_list = $$data_ref{DEFS};  # List of definitions (per dataset)

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "data_ref = $data_ref\n";
      printf "def_list = $def_list = ( @$def_list )\n";
   }

   #---------------------------------------
   #  Determine the default axis parameters
   #---------------------------------------
   my $x_num = graph_calc_x_num($data_ref);
   my ($y_lo_bound, $y_up_bound, $y_interval) = graph_calc_y_default($data_ref);

   #---------------------------------------
   #  Initialize array for window variables
   #---------------------------------------
   my $num_base = 'dec';
   my @dg_data  = ();
   $dg_data[$DGW_CB_XGRID]   = 1;
   $dg_data[$DGW_CB_YGRID]   = 1;
   $dg_data[$DGW_CB_LABELS]  = 1;
   $dg_data[$DGW_NUM_BASE]   = $num_base;
   $dg_data[$DGW_DATA_REF]   = $data_ref;
   $dg_data[$DGW_TEXT_LO]    = $y_lo_bound;
   $dg_data[$DGW_TEXT_UP]    = $y_up_bound;
   $dg_data[$DGW_TEXT_INT]   = $y_interval;
   $dg_data[$DGW_X_NUM]      = $x_num;
   $dg_data[$DGW_X_INCR]     = $graph_dflt_x_incr;
   $dg_data[$DGW_X_BASE]     = $default_x_base;
   $dg_data[$DGW_Y_BASE]     = $default_y_base;
   $dg_data[$DGW_Y_SPAN]     = $default_y_span;
   $dg_data[$DGW_Y_LO_BOUND] = $y_lo_bound;
   $dg_data[$DGW_Y_UP_BOUND] = $y_up_bound;
   $dg_data[$DGW_Y_INTERVAL] = $y_interval;

   #---------------------------------------
   #  Create the main window
   #---------------------------------------
   $win_h = MainWindow->new(-title => $title);
   $win_h->bind('<Destroy>' => [\&destroy_graph_window, \@dg_data, $sub_close]);

   #---------------------------------------
   #  Create the menu bar
   #---------------------------------------
   $menubar  = $win_h->Menu(-type => 'menubar');
   $win_h->configure(-menu => $menubar);
   $casmenu = $menubar->cascade(-label => '~File', -tearoff => 0);
   $casmenu->command(-label => 'Close', -command => [$win_h => 'destroy']);

   $casmenu  = $menubar->cascade(-label => '~Options', -tearoff => 0);
   $casmenu->checkbutton(-label => "Display X-axis background lines", -variable => \$dg_data[$DGW_CB_XGRID],
      -command => [\&graph_cb_xygrid, \@dg_data, \$dg_data[$DGW_CB_XGRID], 'X']);
   $casmenu->checkbutton(-label => "Display Y-axis background lines", -variable => \$dg_data[$DGW_CB_YGRID],
      -command => [\&graph_cb_xygrid, \@dg_data, \$dg_data[$DGW_CB_YGRID], 'Y']);

   #---------------------------------------
   #  Create the menubar dataset settings
   #---------------------------------------
   for ($n = 0; $n <= $#$def_list; $n++)
   {  $var_ref    = $$def_list[$n];
      $name       = $$var_ref{NAME};
      $mref       = \$$var_ref{MODE};  # Reference to mode parameter for dataset
      my @cb_vals = ( 0, 0, 0 );
      $cb_vals[0] = (($$mref & $DGW_MODE_SHOW)   ? 1 : 0);
      $cb_vals[1] = (($$mref & $DGW_MODE_LABELS) ? 1 : 0);
      $cb_vals[2] = (($$mref & $DGW_MODE_LINES)  ? 1 : 0);
      $casmenu2 = $casmenu->cascade(-label => $name, -tearoff => 0);
      $cb0 = $casmenu2->checkbutton(-label => "Display counter", -variable => \$cb_vals[0]);
      $cb1 = $casmenu2->checkbutton(-label => "Display value labels",  -variable => \$cb_vals[1]);
      $cb2 = $casmenu2->checkbutton(-label => "Display lines between points",   -variable => \$cb_vals[2]);
      if ( $cb_vals[0] == 0 )
      {  $cb1->configure(-state => 'disable');
         $cb2->configure(-state => 'disable');
      }
      $cb0->configure(-command => [\&graph_cb_variable, \@dg_data, \$cb_vals[0], $n, $mref, $cb1, $cb2]);
      $cb1->configure(-command => [\&graph_cb_labels,  \@dg_data, \$cb_vals[1], $n, $mref]);
      $cb2->configure(-command => [\&graph_cb_lines,   \@dg_data, \$cb_vals[2], $n, $mref]);
   }

   #---------------------------------------
   #  Create the menubar options for setting the numerical base
   #---------------------------------------
   $casmenu = $menubar->cascade(-label => '~Settings', -tearoff => 0);
   $casmenu2 = $casmenu->cascade(-label => 'Numeric base', -tearoff => 0);
   for ($i = 0; $i <= $#base_opt_labs; $i++)
   {  $casmenu2->radiobutton(-label => $base_opt_labs[$i],
         -value => $base_opt_vals[$i], -variable => \$num_base,
         -command => [\&graph_cbf_set_num_base, \@dg_data, \$num_base]);
   }

   #---------------------------------------
   #  Create the canvas for the graph
   #---------------------------------------
   $canvas = $win_h->Scrolled('Canvas', -scrollbars => 'se',
                -scrollregion => ['0c', '0c', '30c', '25c'],
                -borderwidth => 2, -relief => 'sunken');
   $can_frame = $canvas->Frame();

   #---------------------------------------
   #  Store window data for future use
   #---------------------------------------
   $dg_data[$DGW_WINDOW]   = $win_h;
   $dg_data[$DGW_CANVAS]   = $canvas;

   #---------------------------------------
   #  Draw the graph on the canvas
   #---------------------------------------
   graph_draw(\@dg_data);

   #---------------------------------------
   #  Pack the frame and canvas
   #---------------------------------------
   $can_frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');
   $canvas->create('window', 4, 4, -anchor => 'nw', -window => $can_frame);
   $canvas->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   #---------------------------------------
   #  Create the Y-axis parameter widgets
   #---------------------------------------
   $frame = $win_h->Frame(-borderwidth => 10);
   $label = $frame->Label(-text => 'Y-axis Parameters:', -anchor => 'w');
   $label->pack(-side => 'left');
   $label = $frame->Label(-text => 'Lower bound=', -width => 12, -anchor => 'e');
   $label->pack(-side => 'left');
   $entry = $frame->Entry(-width => 12, -textvariable => \$dg_data[$DGW_TEXT_LO]);
   $entry->pack(-side => 'left', -expand => 'yes', -fill => 'both');
   $label = $frame->Label(-text => 'Upper bound=', -width => 12, -anchor => 'e');
   $label->pack(-side => 'left');
   $entry = $frame->Entry(-width => 12, -textvariable => \$dg_data[$DGW_TEXT_UP]);
   $entry->pack(-side => 'left', -expand => 'yes', -fill => 'both');
   $label = $frame->Label(-text => 'Mark increment=', -width => 15, -anchor => 'e');
   $label->pack(-side => 'left');
   $entry = $frame->Entry(-width => 12, -textvariable => \$dg_data[$DGW_TEXT_INT]);
   $entry->pack(-side => 'left', -expand => 'yes', -fill => 'both');
   $frame->pack(-side => 'top', -fill => 'both');

   #---------------------------------------
   #  Create the action buttons
   #---------------------------------------
   $bwidth = 16;
   $frame = $win_h->Frame(-borderwidth => 10);
   $btn = $frame->Button(-text => 'Apply Parameters', -width => $bwidth,
             -command => [\&graph_cbf_btn_apply, \@dg_data]);
   $btn->pack(-side => 'left');
   $btn = $frame->Button(-text => 'Default Parameters', -width => $bwidth,
             -command => [\&graph_cbf_btn_default, \@dg_data]);
   $btn->pack(-side => 'left', -padx => 10);
   $btn = $frame->Button(-text => 'Close Window', -width => $bwidth,
             -command => [$win_h => 'destroy']);
   $btn->pack(-side => 'right');
   $frame->pack(-side => 'top', -fill => 'both');

   return(\@dg_data);
}

#-------------------------------------------------------------------------------

sub destroy_graph_window
{
   printf "\ndestroy_graph_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $obj_ref   = $_[0];  # Widget or window object being destroyed
   my $dgw_ref   = $_[1];  # Reference to data graph window structure
   my $sub_close = $_[2];  # Reference to subroutine
   my $win_ref   = $$dgw_ref[$DGW_WINDOW];    # Main window for graph

   return if ( $obj_ref != $win_ref );

   #---------------------------------------
   #  Call "cancel" function (if defined)
   #---------------------------------------
   if ( defined($sub_close) && (ref($sub_close) eq 'CODE') )
   {  &$sub_close($dgw_ref);
   }

   $$dgw_ref[$DGW_WINDOW] = 0;

   # Destroy the structure pointed to by "dgw_ref" ???
}

#-------------------------------------------------------------------------------

sub graph_draw
{
   printf "\ngraph_draw( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $canvas   = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my $data_ref = $$dgw_ref[$DGW_DATA_REF];  # Reference to array of pointers
   my $def_list = $$data_ref{DEFS};          # List of definitions (per dataset)
   my ($x, $y, $px, $py, $last_x, $last_y, $n, $i, $var_ref, $val_ref, $name, 
       $color, $mode, $item, $text, $x_num, $x_incr, $x_base, $x_max, $y_base, 
       $y_span, $y_lo_bound, $y_up_bound, $y_interval, $y_num, $y_incr, $y_max);
   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "dgw_ref  = $dgw_ref = ( @$dgw_ref )\n";
      printf "def_list = $def_list = ( @$def_list )\n";
   }

   #---------------------------------------
   #  Extract/calculate X-axis details
   #---------------------------------------
   $x_num      = graph_calc_x_num($data_ref);
   $$dgw_ref[$DGW_X_NUM] = $x_num;
   $x_incr     = $$dgw_ref[$DGW_X_INCR];
   $x_base     = $$dgw_ref[$DGW_X_BASE];
   $x_max      = $x_base + ($x_incr * ($x_num + 1));

   #---------------------------------------
   #  Extract/calculate Y-axis details
   #---------------------------------------
   $y_base     = $$dgw_ref[$DGW_Y_BASE];
   $y_span     = $$dgw_ref[$DGW_Y_SPAN];
   $y_lo_bound = $$dgw_ref[$DGW_Y_LO_BOUND];
   $y_up_bound = $$dgw_ref[$DGW_Y_UP_BOUND];
   $y_interval = $$dgw_ref[$DGW_Y_INTERVAL];
   $y_num      = (($y_up_bound - $y_lo_bound) / $y_interval);  # Number of marks on the Y-axis
   $y_incr     = ($y_span / $y_num);
   $y_max      = $y_base + $y_span;

   #---------------------------------------
   #  Create labels/lines on the X axis (horizontal)
   #---------------------------------------
   $n = ( exists($$data_ref{NUM}) ? $$data_ref{NUM} : 0 );
   for ($i = 1; $i <= $x_num; $i++, $n++) 
   {  $x    = $x_base + ($i * $x_incr);
      if ( $$dgw_ref[$DGW_CB_XGRID] )
      {  $item = $canvas->createLine($x, $y_max, $x, $y_base, -width => 1, -fill => 'grey');
         $canvas->addtag('xgrid', 'withtag', $item);
      }
      $item = $canvas->createLine($x, $y_max, $x, ($y_max - 5), -width => 3);
      $canvas->addtag('border', 'withtag', $item);
      $text = sprintf("T%u", $n);
      $item = $canvas->createText($x, ($y_max + 10), -text => $text); #, -anchor => 'n',
      $canvas->addtag('border', 'withtag', $item);
   }
   $x = $x_base + ($i * $x_incr);
   if ( $$dgw_ref[$DGW_CB_XGRID] )
   {  $item = $canvas->createLine($x, $y_max, $x, $y_base, -width => 1, -fill => 'grey');
      $canvas->addtag('xgrid', 'withtag', $item);
   }
   $item = $canvas->createLine($x, $y_max, $x, ($y_max - 5), -width => 3);
   $canvas->addtag('border', 'withtag', $item);

   #---------------------------------------
   #  Create labels/lines on the Y axis (veritcal)
   #---------------------------------------
   $i = 0;
   $y = $y_max - ($i * $y_incr);
   $text = convert_num2text((($i * $y_interval) + $y_lo_bound), 32, $$dgw_ref[$DGW_NUM_BASE]);
   $item = $canvas->createText(($x_base - 4), $y, -text => $text, -anchor => 'e');
   $canvas->addtag('border', 'withtag', $item);
   for ($i = 1; $i <= $y_num; $i++) 
   {  $y = $y_max - ($i * $y_incr);
      if ( $$dgw_ref[$DGW_CB_YGRID] )
      {  $item = $canvas->createLine($x_base, $y, $x_max, $y, -width => 1, -fill => 'grey');
         $canvas->addtag('ygrid', 'withtag', $item);
      }
      $item = $canvas->createLine($x_base, $y, ($x_base + 5), $y, -width => 3);
      $canvas->addtag('border', 'withtag', $item);
      $text = convert_num2text((($i * $y_interval) + $y_lo_bound), 32, $$dgw_ref[$DGW_NUM_BASE]);
      $item = $canvas->createText(($x_base - 4), $y, -text => $text, -anchor => 'e');
      $canvas->addtag('border', 'withtag', $item);
   }
   if ( $$dgw_ref[$DGW_CB_YGRID] )
   {  $item = $canvas->createLine($x_base, $y_base, $x_max, $y_base, -width => 1, -fill => 'grey');
      $canvas->addtag('ygrid', 'withtag', $item);
   }
   $item = $canvas->createLine($x_base, $y_base, ($x_base + 5), $y_base, -width => 3);
   $canvas->addtag('border', 'withtag', $item);
   $text = convert_num2text($y_up_bound, 32, $$dgw_ref[$DGW_NUM_BASE]);
   $item = $canvas->createText(($x_base - 4), $y_base, -text => $text, -anchor => 'e');
   $canvas->addtag('border', 'withtag', $item);

   #---------------------------------------
   #  Create outside border lines and headers
   #---------------------------------------
   $item = $canvas->createLine($x_base, $y_max,  $x_max,  $y_max, -width => 2);
   $canvas->addtag('border', 'withtag', $item);
   $item = $canvas->createLine($x_base, $y_base, $x_base, $y_max, -width => 2);
   $canvas->addtag('border', 'withtag', $item);

   #---------------------------------------
   #  Create the title line(s)
   #---------------------------------------
   $text = ( exists($$data_ref{TITLE}) ? $$data_ref{TITLE} : "Histogram" );
   $item = $canvas->createText(($x_base + $x_incr), ($y_base - 60), -text => $text, 
              -justify => 'left', -anchor => 'w');
   $canvas->addtag('border', 'withtag', $item);
   if ( exists($$data_ref{SUBTITLE}) )
   {  $item = $canvas->createText(($x_base + $x_incr + 10), ($y_base - 45), -text => $$data_ref{SUBTITLE},
                 -justify => 'left', -anchor => 'w');
      $canvas->addtag('border', 'withtag', $item);
   }

   #---------------------------------------
   #  Process each dataset
   #---------------------------------------
   for ($n = 0; $n <= $#$def_list; $n++)
   {  
      $var_ref = $$def_list[$n];   # Pointer to HASH structure for dataset
      $val_ref = $$var_ref{DATA};  # Pointer to actual values for dataset
      $color   = $$var_ref{COLOR}; # Color for this specific dataset
      $mode    = $$var_ref{MODE};  # Mode/parameters for this specific dataset

      #---------------------------------------
      #  Skip this dataset if "hidden"
      #---------------------------------------
      next if ( ($mode & $DGW_MODE_SHOW) == 0 );
      $last_x  = 0; 
      $last_y  = 0;

      #---------------------------------------
      #  Display the dataset (if enabled)
      #---------------------------------------
      for ($i = 0; $i <= $#$val_ref; $i++)
      {  $px   = $i + 1;           # Time sample number
         $py   = $$val_ref[$i];   # Actual variable value for this sample
         $text = convert_num2text($py, 4, $$dgw_ref[$DGW_NUM_BASE]); # Actual value for label
         if ( ($py < $y_lo_bound) || ($py > $y_up_bound) )
         {  printf("WARNING - The following data point exceeds bounds for the Y-axis:");
            printf " ( 0x%08X, 0x%08X ) = ( %lu, %lu )\n", $px, $py, $px, $py;
            $y = ( ($py < $y_lo_bound) ? ($y_max + 40) : ($y_base - 20) );
         }
         else
         {  $py = $py - $y_lo_bound;
            $y = $y_max  - (($py * $y_incr) / $y_interval);
         }

         $x = $x_base + ($px * $x_incr);
         $item = $canvas->createOval($x-3, $y-3, $x+3, $y+3, -width => 1,
   			   -outline => 'black', -fill => $color);
         $canvas->addtag("dataset$n", 'withtag', $item);
         if ( $mode & $DGW_MODE_LABELS )
         {  $item = $canvas->createText($x, ($y-10), -text => $text, -fill => $color);
            $canvas->addtag("labels$n", 'withtag', $item);
         }
         if ( $mode & $DGW_MODE_LINES )
         {  if ($i > 0 )
            {  $item = $canvas->createLine($last_x, $last_y,  $x,  $y, -width => 1, -fill => $color);
               $canvas->addtag("lines$n", 'withtag', $item);
            }
            $last_x = $x;
            $last_y = $y;
         }

      }
   }

   #---------------------------------------
   #  Display the dataset legend
   #---------------------------------------
   $x = $x_max  + 20;
   $y = $y_base;
   $item = $canvas->createText($x+30, $y, -text => "Graph Legend:", -justify => 'left');
   $canvas->addtag('border', 'withtag', $item);
   for ($n = 0, $y += 20; $n <= $#$def_list; $n++, $y += 20)
   {  $var_ref = $$def_list[$n];   # Pointer to HASH structure for dataset
      $name    = $$var_ref{NAME};
      $color   = $$var_ref{COLOR};
      $item = $canvas->createOval($x-3, $y-3, $x+3, $y+3, -width => 1, -outline => 'black', -fill => $color);
      $canvas->addtag('border', 'withtag', $item);
      $item = $canvas->createText($x+6, $y, -text => $name, -fill => $color, -justify => 'left', -anchor => 'w');
      $canvas->addtag('border', 'withtag', $item);
   }
}

#-------------------------------------------------------------------------------

sub graph_refresh
{
   print "\ngraph_refresh( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $canvas   = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my $data_ref = $$dgw_ref[$DGW_DATA_REF];  # Reference to array of pointers
   my $def_list = $$data_ref{DEFS};          # List of definitions (per dataset)
   my $ind;

   #---------------------------------------
   #  Delete the existing graph
   #---------------------------------------
   $canvas->delete('xgrid');
   $canvas->delete('ygrid');
   $canvas->delete('border');
   for ($ind = 0; $ind <= $#$def_list; $ind++)
   {  $canvas->delete("dataset$ind");
      $canvas->delete("labels$ind");
      $canvas->delete("lines$ind");
   }

   #---------------------------------------
   #  Redraw the graph with the new color
   #---------------------------------------
   graph_draw($dgw_ref); # Draw data on the canvas
}


#-------------------------------------------------------------------------------

sub graph_cb_xygrid
{
   print "\ngraph_xygrid( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $vref    = $_[1];  # Reference to checkbutton variablee
   my $type    = $_[2];  # Reference to checkbutton variablee
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my $tagname;

   if ( $$vref )  # Checkbutton is TRUE
   {  graph_draw($dgw_ref); # Draw data on the canvas
   }
   else  # Checkbutton is FALSE
   {  $tagname = ( ($type eq 'X') ? 'xgrid' : 'ygrid' );
      $canvas->delete($tagname);
   }
}


#-------------------------------------------------------------------------------

sub graph_cb_variable
{
   print "\ngraph_cb_variable( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $vref    = $_[1];  # Reference to checkbutton variablee
   my $ind     = $_[2];  # Index to data entry affected
   my $mref    = $_[3];  # Reference to mode parameter for dataset
   my $cb1     = $_[4];  # Checkbutton to enable/disable
   my $cb2     = $_[5];  # Checkbutton to enable/disable
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph

   if ( $$vref )  # Checkbutton is TRUE
   {  $cb1->configure(-state => 'normal');
      $cb2->configure(-state => 'normal');
      $$mref |= $DGW_MODE_SHOW;
      graph_draw($dgw_ref); # Draw data on the canvas
   }
   else  # Checkbutton is FALSE
   {  $cb1->configure(-state => 'disable');
      $cb2->configure(-state => 'disable');
      $$mref &= ~($DGW_MODE_SHOW);
      $canvas->delete("dataset$ind");
      $canvas->delete("labels$ind");
      $canvas->delete("lines$ind");
   }
}

#-------------------------------------------------------------------------------

sub graph_cb_labels
{
   print "\ngraph_cb_labels( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $vref    = $_[1];  # Reference to checkbutton variablee
   my $ind     = $_[2];  # Index to data entry affected
   my $mref    = $_[3];  # Reference to mode parameter for dataset
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph

   if ( $$vref )  # Checkbutton is TRUE
   {  $$mref |= $DGW_MODE_LABELS;
      graph_draw($dgw_ref); # Draw data on the canvas
   }
   else  # Checkbutton is FALSE
   {  $$mref &= ~($DGW_MODE_LABELS);
      $canvas->delete("labels$ind");
   }
}


#-------------------------------------------------------------------------------

sub graph_cb_lines
{
   print "\ngraph_cb_lines( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $vref    = $_[1];  # Reference to checkbutton variablee
   my $ind     = $_[2];  # Index to data entry affected
   my $mref    = $_[3];  # Reference to mode parameter for dataset
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph


   if ( $$vref )  # Checkbutton is TRUE
   {  $$mref |= $DGW_MODE_LINES;
      graph_draw($dgw_ref); # Draw data on the canvas
   }
   else  # Checkbutton is FALSE
   {  $$mref &= ~($DGW_MODE_LINES);
      $canvas->delete("lines$ind");
   }
}

#-------------------------------------------------------------------------------

sub graph_cbf_set_num_base
{
   print "\ngraph_cbf_set_num_base( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $vref     = $_[1];  # Reference to variable with selected color
   
   #---------------------------------------
   #  Set the new numeric base
   #---------------------------------------
   $$dgw_ref[$DGW_NUM_BASE] = $$vref;

   #---------------------------------------
   #  Redraw the graph with the new parameter
   #---------------------------------------
   graph_refresh($dgw_ref); # Draw data on the canvas
}

#-------------------------------------------------------------------------------

sub graph_cbf_btn_apply
{
   print "\ngraph_cbf_btn_apply( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref = $_[0];  # Reference to data graph window structure
   my $win_h   = $$dgw_ref[$DGW_WINDOW];    # Main window for graph
   my $canvas  = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my ($num_int, $text, $rv);

   #---------------------------------------
   #  Extract user-specified parameters
   #---------------------------------------
   my $tmp_lo  = convert_text2num($$dgw_ref[$DGW_TEXT_LO]);
   my $tmp_up  = convert_text2num($$dgw_ref[$DGW_TEXT_UP]);
   my $tmp_int = convert_text2num($$dgw_ref[$DGW_TEXT_INT]);

   #---------------------------------------
   #  Check the parameters: valid?
   #---------------------------------------
   if ( $tmp_up <= $tmp_lo )
   {  $emess = "Invalid Y-axis upper and lower bounds: lower bound must be less than the upper bound.\n";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $emess,
               -title => 'Y-axis Parameter Error');
      return;
   }

   #---------------------------------------
   #  Check the "mark increment"
   #---------------------------------------
   $num_int = int( (($tmp_up - $tmp_lo) / $tmp_int) + 0.5 );
   if ( $num_int > 40 )
   {  $text = sprintf("The number of Y-axis intervals exeeds the recommended limit of 40.\nWith specified mark value, there would be %u intevals, which may take significant time to draw/refresh. Do you want to apply these settings?", $num_int);
      $rv   = $win_h->messageBox(-icon => 'question', -type => 'YesNo', -message => $text,
              -title => 'Y-axis Parameter Question');
      return if ( $rv eq 'No' );
   }

   #---------------------------------------
   #  Store the new parameters
   #---------------------------------------
   $$dgw_ref[$DGW_Y_LO_BOUND] = $tmp_lo;
   $$dgw_ref[$DGW_Y_UP_BOUND] = $tmp_up;
   $$dgw_ref[$DGW_Y_INTERVAL] = $tmp_int;

   #---------------------------------------
   #  Delete the existing graph and redraw
   #---------------------------------------
   graph_refresh($dgw_ref);
}

#-------------------------------------------------------------------------------

sub graph_cbf_btn_default
{
   print "\ngraph_cbf_btn_default( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $dgw_ref  = $_[0];  # Reference to data graph window structure
   my $canvas   = $$dgw_ref[$DGW_CANVAS];    # Canvas to hold graph
   my $data_ref = $$dgw_ref[$DGW_DATA_REF];  # Reference to array of pointers

   my ($y_lo_bound, $y_up_bound, $y_interval) = graph_calc_y_default($data_ref);

   #---------------------------------------
   #  Reset the Y-axis parameters
   #---------------------------------------
   $$dgw_ref[$DGW_TEXT_LO]  = $y_lo_bound;
   $$dgw_ref[$DGW_TEXT_UP]  = $y_up_bound;
   $$dgw_ref[$DGW_TEXT_INT] = $y_interval;
}

#-------------------------------------------------------------------------------

sub graph_calc_x_num
{  
   printf "\ngraph_calc_x_num( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $data_ref = $_[0];             # Reference to array of pointers
   my $def_list = $$data_ref{DEFS};  # List of definitions (per dataset)
   my ($i, $j, $tmp, $max, $def_ptr, $val_ref);


   #---------------------------------------
   #  Find the maximum number of samples
   #---------------------------------------
   $max = ( exists($$data_ref{MAX}) ? $$data_ref{MAX} : 0 );
   for ($j = 0; $j <= $#$def_list; $j++)
   {  $def_ptr = $$def_list[$j];
      $val_ref = $$def_ptr{DATA};
      $tmp     = ( $#$val_ref + 1 );
      if ( $tmp > $max )
      {  $max = $tmp;
      }
   }
   return($max);
}

#-------------------------------------------------------------------------------

sub graph_calc_y_default
{  
   printf "\ngraph_calc_y_default( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $data_ref = $_[0];             # Reference to array of pointers
   my $def_list = $$data_ref{DEFS};  # List of definitions (per dataset)
   my ($i, $j, $def_ptr, $val_ref, $min, $max, $tmp);

   #---------------------------------------
   #  Find the minimum and maximum values
   #---------------------------------------
   $min = 0;
   $max = 0;
   for ($j = 0; $j <= $#$def_list; $j++)
   {  $def_ptr = $$def_list[$j];
      $val_ref = $$def_ptr{DATA};
      for ($i = 0; $i <= $#$val_ref; $i++)
      {  if ( $$val_ref[$i] < $min )
         {  $min = $$val_ref[$i];
         }
         if ( $$val_ref[$i] > $max )
         {  $max = $$val_ref[$i];
         }
      }
   }

   #---------------------------------------
   #  Round maximum Y-value to next 100
   #---------------------------------------
   $tmp = int($max / 100);
   if ( ($tmp == 0) || (($tmp % 100) != 0) )
   {  $tmp++;
   }
   $tmp *= 100;
   return(0, $tmp, ($tmp / 10));
}

#-------------------------------------------------------------------------------

1;
