# 
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
# 
# @(#)$Id: ecc.pm,v 1.3 2004/03/29 17:56:21 ncrumbak Exp $

#-------------------------------------------------------------------------------
#
#  FILE: ecc.dm
#
#  DESCRIPTION:  This file contains supporting PERL subroutines for the error
#     log decode utility for SHUB.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO               WHAT
#     --------  ----------------  -----------------------------------------
#     01/22/03  Nelson Crumbaker  Original source code for submission.
#     08/21/03  Nelson Crumbaker  Return -1 on error instead of 72.
#-------------------------------------------------------------------------------

@ECC = (
    0xC1,       # Bit 0
    0xC2,       # Bit 1
    0xC4,       # Bit 2
    0xC8,       # Bit 3
    0xA1,       # Bit 4
    0xA2,       # Bit 5
    0xA4,       # Bit 6
    0xA8,       # Bit 7
    0x91,       # Bit 8
    0x92,       # Bit 9
    0x94,       # Bit 10
    0x98,       # Bit 11
    0x61,       # Bit 12
    0x62,       # Bit 13
    0x64,       # Bit 14
    0x68,       # Bit 15
    0x51,       # Bit 16
    0x52,       # Bit 17
    0x54,       # Bit 18
    0x58,       # Bit 19
    0x31,       # Bit 20
    0x32,       # Bit 21
    0x34,       # Bit 22
    0x38,       # Bit 23
    0xF8,       # Bit 24
    0x4F,       # Bit 25
    0x70,       # Bit 26
    0xD0,       # Bit 27
    0x0E,       # Bit 28
    0x0B,       # Bit 29
    0xF1,       # Bit 30
    0x2F,       # Bit 31
    0x1C,       # Bit 32
    0x2C,       # Bit 33
    0x4C,       # Bit 34
    0x8C,       # Bit 35
    0x1A,       # Bit 36
    0x2A,       # Bit 37
    0x4A,       # Bit 38
    0x8A,       # Bit 39
    0x19,       # Bit 40
    0x29,       # Bit 41
    0x49,       # Bit 42
    0x89,       # Bit 43
    0x16,       # Bit 44
    0x26,       # Bit 45
    0x46,       # Bit 46
    0x86,       # Bit 47
    0x15,       # Bit 48
    0x25,       # Bit 49
    0x45,       # Bit 50
    0x85,       # Bit 51
    0x13,       # Bit 52
    0x23,       # Bit 53
    0x43,       # Bit 54
    0x83,       # Bit 55
    0x8F,       # Bit 56
    0xF4,       # Bit 57
    0x07,       # Bit 58
    0x0D,       # Bit 59
    0xE0,       # Bit 60
    0xB0,       # Bit 61
    0x1F,       # Bit 62
    0xF2,       # Bit 63
    0x01,       # Bit 64
    0x02,       # Bit 65
    0x04,       # Bit 66
    0x08,       # Bit 67
    0x10,       # Bit 68
    0x20,       # Bit 69
    0x40,       # Bit 70
    0x80        # Bit 71
);

@DIR_ECC = (
    0x4C,	# Bit 00, GRP2 Bit 00, GRP1 Bit 04
    0x29,	# Bit 01, GRP2 Bit 01, GRP1 Bit 05
    0x02,	# Bit 02, GRP2 Bit 02, GRP1 Bit 06
    0x10,	# Bit 03, GRP2 Bit 03, GRP1 Bit 07
    0x1C,	# Bit 04, GRP2 Bit 08, GRP1 Bit 12
    0x01,	# Bit 05, GRP2 Bit 09, GRP1 Bit 13
    0x2A,	# Bit 06, GRP2 Bit 10, GRP1 Bit 14
    0x40,	# Bit 07, GRP2 Bit 11, GRP1 Bit 15
    0x19,	# Bit 08, GRP2 Bit 16, GRP1 Bit 20
    0x4A,	# Bit 09, GRP2 Bit 17, GRP1 Bit 21
    0x04,	# Bit 10, GRP2 Bit 18, GRP1 Bit 22
    0x20,	# Bit 11, GRP2 Bit 19, GRP1 Bit 23
    0x31,	# Bit 12, GRP2 Bit 24, GRP1 Bit 28
    0x32,	# Bit 13, GRP2 Bit 25, GRP1 Bit 29
    0x34,	# Bit 14, GRP2 Bit 26, GRP1 Bit 30
    0x38,	# Bit 15, GRP2 Bit 27, GRP1 Bit 31
    0x51,	# Bit 16, GRP2 Bit 32, GRP1 Bit 36
    0x52,	# Bit 17, GRP2 Bit 33, GRP1 Bit 37
    0x54,	# Bit 18, GRP2 Bit 34, GRP1 Bit 38
    0x58,	# Bit 19, GRP2 Bit 35, GRP1 Bit 39
    0x61,	# Bit 20, GRP2 Bit 40, GRP1 Bit 44
    0x62,	# Bit 21, GRP2 Bit 41, GRP1 Bit 45
    0x64,	# Bit 22, GRP2 Bit 42, GRP1 Bit 46
    0x68,	# Bit 23, GRP2 Bit 43, GRP1 Bit 47
    0x0B,	# Bit 24, GRP2 Bit 48, GRP1 Bit 52
    0x13,	# Bit 25, GRP2 Bit 49, GRP1 Bit 53
    0x23,	# Bit 26, GRP2 Bit 50, GRP1 Bit 54
    0x43,	# Bit 27, GRP2 Bit 51, GRP1 Bit 55
    0x0D,	# Bit 28, GRP2 Bit 56, GRP1 Bit 60
    0x15,	# Bit 29, GRP2 Bit 57, GRP1 Bit 61
    0x25,	# Bit 30, GRP2 Bit 58, GRP1 Bit 62
    0x45,	# Bit 31, GRP2 Bit 59, GRP1 Bit 63
    0x0E,	# Bit 32, GRP2 Bit 64, GRP1 Bit 68
    0x16,	# Bit 33, GRP2 Bit 65, GRP1 Bit 69
    0x26,	# Bit 34, GRP2 Bit 66, GRP1 Bit 70
    0x46	# Bit 35, GRP2 Bit 67, GRP1 Bit 71
);

sub gen_dir_syndrome
{
   local $dir_entry = $_[0];
   local $syndrome;
   local $bit;

   $syndrome = 0;
   for ($bit=0; $bit<36; $bit++)
   {
      if ( ( 0x1 << $bit ) & $dir_entry )
      {
         $syndrome ^= $DIR_ECC[$bit];
      }
   }

   return ($syndrome);
}

sub get_dir_bit_from_syndrome
{
   local $syndrome = $_[0];
   local $bit;

   for($bit=0;$bit<36;$bit++)
   {
      if ( $DIR_ECC[$bit] == $syndrome )
      {
         return($bit);
      }
   }

   return(-1); # Returns -1 on error.
}


sub get_bit_from_syndrome
{
   local $syndrome = $_[0];
   local $bit;

   for($bit=0;$bit<72;$bit++)
   {
      if ( $ECC[$bit] == $syndrome )
      {
         return($bit);
      }
   }

   return(-1); # Returns -1 on error.
}

1;
