#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: icd_access.pm,v 1.5 2004/04/21 18:34:20 beighley Exp $
#-------------------------------------------------------------------------------
#
#  FILE: icd_access.pm
#
#  DESCRIPTION:  This file contains PERL subroutines to create an GUI for a
#    user to perform stop clock and internal scan chain operations.  The
#    user can target specific chips, and select which internal scan chains
#    should be dumped for each chip.
#
#  NOTES: The "common_defs.pm" and "namesel.pm" file is required.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     09/08/03  Karen Beighley  Original source code submitted for release.
#     10/17/03  Karen Beighley  Modified parameters to create_namesel_window().
#     02/10/04  Karen Beighley  Modified to support multi-chain modules.
#     04/21/04  Karen Beighley  Additional chip parameters passed to C-library.
#
#-------------------------------------------------------------------------------

use namesel;

#-------------------------------------------------------------------------------
#  CONSTANTS - These values cannot change!
#-------------------------------------------------------------------------------

$ICW_WINDOW     =  0;  # Group window data: main window handle
$ICW_WIN_LIST   =  1;  # Group window data: list of "subordinate" windows
$ICW_NAME_TEXT  =  2;  # Group window data: name of output file
$ICW_LIST_REF   =  3;  # Group window data: reference to array of chip definitions
$ICW_SORT_REF   =  4;  # Group window data: reference to sorted array of definitions
$ICW_LISTBOX    =  5;  # Group window data: listbox handle
$ICW_STOP_MODE  =  6;  # Group window data: stop-clock mode - all or targeted bricks
$ICW_STOP_FLAG  =  7;  # Group window data: user wants to stop sequence
$ICW_BTN_MODIFY =  8;  # Group window data: button handle
$ICW_BTN_ALL    =  9;  # Group window data: button handle
$ICW_BTN_NONE   = 10;  # Group window data: button handle
$ICW_BTN_COPY   = 11;  # Group window data: button handle
$ICW_BTN_MVUP   = 12;  # Group window data: button handle
$ICW_BTN_MVDN   = 13;  # Group window data: button handle
$ICW_BTN_BROWSE = 14;  # Group window data: button handle
$ICW_BTN_BOTH   = 15;  # Group window data: button handle
$ICW_BTN_STOP   = 16;  # Group window data: button handle
$ICW_BTN_READ   = 17;  # Group window data: button handle
$ICW_BTN_CANCEL = 18;  # Group window data: button handle
$ICW_BTN_CUSTOM = 19;  # Group window data: button handle
$ICW_HIST_WID   = 20;  # Group window data: history widget handle
$ICW_BTN_CLEAR  = 21;  # Group window data: button handle
$ICW_BTN_SAVE   = 22;  # Group window data: button handle
$ICW_BTN_CLOSE  = 23;  # Group window data: button handle
$ICW_COLOR_BCK  = 24;  # Group window data: 
$ICW_COLOR_ACT  = 25;  # Group window data: 
$ICW_CASMENU0   = 26;  # Group window data: 
$ICW_CASMENU1   = 27;  # Group window data: 
$ICW_CASMENU2   = 28;  # Group window data: 

$ICD_MOD_NAME  = 0; # ICD data: name of module
$ICD_BRD_NAME  = 1; # ICD data: name of board in module
$ICD_CHP_NAME  = 2; # ICD data: name of specific chip on board
$ICD_TYPE      = 3; # ICD data: type definition: shub
$ICD_DESC      = 4; # ICD data: complete description: shub_02
$ICD_INDEX     = 5; # ICD data: cross reference index number
$ICD_LIST_REF  = 6; # ICD data: reference to list of selected chains

$STOP_MODE_ALL = 0;
$STOP_MODE_DEF = 1;


#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES - Tables of data for each supported chip type/description
#-------------------------------------------------------------------------------

my $icd_setup  = $FALSE;
my %icd_summs  = ( );
my %icd_names  = ( );
my %icd_defsel = ( );

my @chip_icd_ptrs = ();

my $icd_cancel_mess = "\n-> Sequence cancelled by user; results may be incomplete.\n";
my $icd_error_mess  = "\n-> Sequence terminated with error condition.\n";

#-------------------------------------------------------------------------------

sub create_sys_icd_window
{
   printf "\ncreate_sys_icd_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return($FALSE) if ( $#_ != 1 );
   my $main_win     = $_[0]; # Reference to main window for GUI
   my $cmd_ref      = $_[1]; # Reference to command button to disable/enable
   my $list_ref     = \@chip_icd_ptrs; # Reference to array of chip ICD data
   my @win_list     = ();
   my @icw_data     = (0, \@win_list, "scandump.dmp", 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
   my $bwidth       = 12;
   my @ordered_list = @$list_ref;
   my ($frame, $sub_frame, $btn, $label, $icd_log, 
       $entry, $def_lbox, $def_ref, $chn_text, $text, $win_h, $menubar, $casmenu, $name);

   #---------------------------------------
   #  Initialize the data structures
   #---------------------------------------
   if ( icdef_init_data_structures() == $FALSE )
   {  return;
   }

   #---------------------------------------
   #  Create main window for selection
   #---------------------------------------
   $text  = "Stop Clock and Internal Scan Chain Utility";
   $win_h = $main_win->Toplevel(-title => $text);
   $win_h->bind('<Destroy>' => [\&destroy_sys_icd_window, \@icw_data, $cmd_ref]);
   if ( $$cmd_ref != 0 )
   {  $$cmd_ref->configure(-state => 'disable');
   }

   #---------------------------------------
   #  Create the menu bar
   #---------------------------------------
   $menubar  = $win_h->Menu(-type => 'menubar');
   $win_h->configure(-menu => $menubar);
   $casmenu = $menubar->cascade(-label => '~File', -tearoff => 0);
   $icw_data[$ICW_CASMENU0]  = $casmenu;
   $casmenu->command(-label => 'Load Internal Chain Definition File ...',
      -command => [\&icdef_file_load, \@icw_data]);
   $casmenu->command(-label => 'Save Internal Chain Definition(s) to File ...',
      -command => [\&icdef_file_save, \@icw_data]);
   $casmenu->separator();
   $casmenu->command(-label => 'Close', -command => [$win_h => 'destroy']);

   $casmenu = $menubar->cascade(-label => '~Options', -tearoff => 0);
   $icw_data[$ICW_CASMENU1]  = $casmenu;
   $rc = $casmenu->cascade(-label => 'Stop Clocks', -tearoff => 0);
   $rc->radiobutton(-label => 'All modules in defined order', -value => $STOP_MODE_ALL, 
           -variable => \$icw_data[$ICW_STOP_MODE]);
   $rc->radiobutton(-label => 'Modules with valid chain(s)', -value => $STOP_MODE_DEF, 
           -variable => \$icw_data[$ICW_STOP_MODE]);

   $casmenu = $menubar->cascade(-label => '~Definitions', -tearoff => 0);
   $icw_data[$ICW_CASMENU2]  = $casmenu;
   $casmenu->command(-label => 'Select All Chains for each Chip',
      -command => [\&icdef_cbf_reset_all, \@icw_data, 'all']);
   $casmenu->command(-label => 'Select No Chains for each Chip',
      -command => [\&icdef_cbf_reset_all, \@icw_data, 'none']);
   $casmenu->separator();
   $casmenu->command(-label => 'Reset Chip Order',
      -command => [\&icdef_cbf_reset_all, \@icw_data, 'order']);
   $casmenu->command(-label => 'Reset Chip Order and Select All Chains',
      -command => [\&icdef_cbf_reset_all, \@icw_data, 'reset']);

   #---------------------------------------
   #  Create the listbox and buttons for definitions
   #---------------------------------------
   $frame = $win_h->Frame(-borderwidth => 10);

   $sub_frame = $frame->Frame();
   $label = $sub_frame->Label(-anchor => 'w', -text => "Internal Scan Chain Definitions:");
   $label->pack(-side => 'top', -fill => 'both');
   $def_lbox = $sub_frame->Scrolled('Listbox', -scrollbars => 'se', #-height => 12, 
                -font => 'Clean 12 bold', -selectmode => 'single');
   $def_lbox->pack(-side => 'top', -fill => 'both', -expand => 'yes');
   $sub_frame->pack(-side => 'left', -fill => 'both', -expand => 'yes');

   $sub_frame = $frame->Frame();
   $btn = $sub_frame->Button(-text => 'Modify', -width => $bwidth,
      -command => [\&icdef_cbf_modify, \@icw_data]);
   $icw_data[$ICW_BTN_MODIFY]  = $btn;
   $btn->pack(-side => 'top', -padx => 5, -pady => 3);
   $btn = $sub_frame->Button(-text => 'All Chains', -width => $bwidth,
      -command => [\&icdef_cbf_all, \@icw_data]);
   $icw_data[$ICW_BTN_ALL]  = $btn;
   $btn->pack(-side => 'top', -padx => 5, -pady => 3);
   $btn = $sub_frame->Button(-text => 'No Chains', -width => $bwidth,
      -command => [\&icdef_cbf_none, \@icw_data]);
   $icw_data[$ICW_BTN_NONE]  = $btn;
   $btn->pack(-side => 'top', -padx => 5, -pady => 3);
   $btn = $sub_frame->Button(-text => 'Copy', -width => $bwidth,
      -command => [\&icdef_cbf_copy, \@icw_data]);
   $icw_data[$ICW_BTN_COPY]  = $btn;
   $btn->pack(-side => 'top', -padx => 5, -pady => 3);
   $label = $sub_frame->Label(-anchor => 'w', -text => '');
   $label->pack(-side => 'top');
   $btn = $sub_frame->Button(-text => 'Move Up', -width => $bwidth,
      -command => [\&icdef_cbf_move_up, \@icw_data]);
   $icw_data[$ICW_BTN_MVUP]  = $btn;
   $btn->pack(-side => 'top', -padx => 5, -pady => 3);
   $btn = $sub_frame->Button(-text => 'Move Down', -width => $bwidth,
      -command => [\&icdef_cbf_move_down, \@icw_data]);
   $icw_data[$ICW_BTN_MVDN]  = $btn;
   $btn->pack(-side => 'top', -padx => 5, -pady => 3);
   $sub_frame->pack(-side => 'right', -fill => 'both');

   $frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   #---------------------------------------
   #  Add current entries to listbox
   #---------------------------------------
   foreach $def_ref ( @$list_ref )
   {  $chn_text = icdef_build_chain_list($def_ref);
      $name = sprintf("%s:%s:%s", $$def_ref[$ICD_MOD_NAME], $$def_ref[$ICD_BRD_NAME],
                 $$def_ref[$ICD_CHP_NAME]);
      $text = sprintf("%-20s ($chn_text)", $name);
      $def_lbox->insert('end', $text);
   }
   $def_lbox->selection('set', 0);

   #---------------------------------------
   #  Create a frame to File name parameter
   #---------------------------------------
   $frame = $win_h->Frame(-borderwidth => 10);
   $label = $frame->Label(-anchor => 'w', -text => 'Output File:');
   $label->pack(-side => 'left', -expand => 'no');
   $entry = $frame->Entry(-width => 80, -textvariable => \$icw_data[$ICW_NAME_TEXT]);
   $entry->pack(-side => 'left', -expand => 'yes', -fill => 'both');
   $btn = $frame->Button(-text => 'Browse', -width => $bwidth,
      -command => [\&icdef_cbf_browse, \@icw_data]);
   $icw_data[$ICW_BTN_BROWSE]  = $btn;
   $btn->pack(-side => 'right', -padx => 5);
   $frame->pack(-side => 'top', -fill => 'both');

   #---------------------------------------
   #  Create a frame to hold the action buttons
   #---------------------------------------
   $bwidth = 22;
   $frame = $win_h->Frame(-borderwidth => 10);
   $btn = $frame->Button(-text => 'Stop Clocks & Read Chains', -width => $bwidth,
      -command => [\&icdef_cbf_run, \@icw_data, 'both' ]);
   $icw_data[$ICW_BTN_BOTH]  = $btn;
   $icw_data[$ICW_COLOR_BCK] = $btn->cget(-background);
   $icw_data[$ICW_COLOR_ACT] = $btn->cget(-activebackground);
   $btn->configure(-background => 'green', -activebackground => 'green');
   $bwidth = 12;
   $btn->pack(-side => 'left', -padx => 5);
   $btn = $frame->Button(-text => 'Stop Clocks', -width => $bwidth,
      -command => [\&icdef_cbf_run, \@icw_data, 'stop' ]);
   $icw_data[$ICW_BTN_STOP]  = $btn;
   $btn->pack(-side => 'left', -padx => 5);
   $btn = $frame->Button(-text => 'Read Chains', -width => $bwidth,
      -command => [\&icdef_cbf_run, \@icw_data, 'read' ]);
   $icw_data[$ICW_BTN_READ]  = $btn;
   $btn->pack(-side => 'left', -padx => 5);
   $btn = $frame->Button(-text => 'Cancel', -width => $bwidth, -state => 'disabled',
      -command => [\&icdef_cbf_cancel, \@icw_data ]);
   $icw_data[$ICW_BTN_CANCEL]  = $btn;
   $btn->pack(-side => 'left', -padx => 5);
   $btn = $frame->Button(-text => 'Advanced', -width => $bwidth, -state => 'disabled');
   $btn->pack(-side => 'right', -padx => 5);
   $icw_data[$ICW_BTN_CUSTOM]  = $btn;
   $frame->pack(side => 'top', -fill => 'both', -expand => 'no');

   #
   #  Create a frame with the history/log widgets
   #
   $frame = $win_h->Frame(-borderwidth => 10);
   $label = $frame->Label(-anchor => 'w', -text => "Command History:");
   $label->pack(-side => 'top', -expand => 'no', -fill => 'both');
   $icd_log = $frame->Scrolled('ROText', -scrollbars => 'se', -height => 12, -wrap => 'no');
   $icd_log->pack(-side => 'top', -expand => 'yes', -fill => 'both');
   $frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   #
   #  Create a frame with some action buttons
   #
   $bwidth = 12;
   $frame = $win_h->Frame(-borderwidth => 10);
   $btn   = $frame->Button(-text => 'Clear History', -width => $bwidth,
      -command => [\&icdef_cbf_hist_btn, \@icw_data, 'clear']);
   $icw_data[$ICW_BTN_CLEAR]  = $btn;
   $btn->pack(-side => 'left', -padx => 5);
   $btn   = $frame->Button(-text => 'Save History', -width => $bwidth,
      -command => [\&icdef_cbf_hist_btn, \@icw_data, 'save']);
   $icw_data[$ICW_BTN_SAVE]  = $btn;
   $btn->pack(-side => 'left', -padx => 5);
   $btn = $frame->Button(-text => 'Close Window', -width => $bwidth, -command => [$win_h => 'destroy']);
   $icw_data[$ICW_BTN_CLOSE]  = $btn;
   $btn->pack(-side => 'right', -padx => 5);
   $frame->pack(-side => 'top', -fill => 'both');


   #---------------------------------------
   #  Store widget handles and variables
   #---------------------------------------
   $icw_data[$ICW_WINDOW]    = $win_h;
   $icw_data[$ICW_LIST_REF]  = $list_ref;
   $icw_data[$ICW_SORT_REF]  = \@ordered_list;
   $icw_data[$ICW_LISTBOX]   = $def_lbox;
   $icw_data[$ICW_HIST_WID]  = $icd_log;
   $icw_data[$ICW_STOP_MODE] = $STOP_MODE_ALL;
   $icw_data[$ICW_STOP_FLAG] = $FALSE;


   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "\nicw_data = ( @icw_data )\n\n";
   }

   return($TRUE);
}

#------------------------------------------------------------------------------

sub destroy_sys_icd_window
{
   printf "\ndestroy_sys_icd_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $curr_wid = $_[0]; # Current widget/window handle from Tk
   my $iwd_ref  = $_[1]; # Reference to group window data
   my $cmd_ref  = $_[2]; # Command button reference
   my $win      = $$iwd_ref[$ICW_WINDOW];

   return if ( $curr_wid != $win ); # Return if not main window

   #---------------------------------------
   #  Clear out the GUI widget handles
   #---------------------------------------
   $$iwd_ref[$ICW_WINDOW]   = 0;
   $$iwd_ref[$ICW_BTN_BOTH] = 0;
   $$iwd_ref[$ICW_BTN_STOP] = 0;
   $$iwd_ref[$ICW_BTN_READ] = 0;
   $$iwd_ref[$ICW_LISTBOX]  = 0;

   #---------------------------------------
   #  Enable button from parent window
   #---------------------------------------
   if ( $$cmd_ref != 0 )
   {  $$cmd_ref->configure(-state => 'normal');
   }

   #---------------------------------------
   #  Destroy the group selection window (if open)
   #---------------------------------------
   my $win_list = $$iwd_ref[$ICW_WIN_LIST];
   if ( $#$win_list >= 0 )
   {  $win_ref = pop(@$win_list);
      while ( defined($win_ref) )
      {  $win_ref->destroy();
         $win_ref = pop(@$win_list);
      }
   }
}


#-------------------------------------------------------------------------------

sub icdef_init_data_structures
{
   printf "\nicdef_init_data_structures( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   #---------------------------------------
   #  Have the structures already been initialized?
   #---------------------------------------
   return($TRUE) if ( $icd_setup );
   my ($parm, $i, $type, $mref, $chn_list, $c, $chn_ptr, $chip_list, $chip_ptr);

   #---------------------------------------
   #  Get the list of supported chip types
   #---------------------------------------
   my @chip_descs = jtag_icd_get_descs();
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      $ecode = 0;
      $emess = "";
      return($FALSE);
   }

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "\n-> Building the internal chain description tables for each support type:\n";
      printf "   ( @chip_descs )\n";
   }

   #---------------------------------------
   #  Build the internal chain tables
   #---------------------------------------
   for ($i = 0; $i <= $#chip_descs; $i++)
   {  $type = $chip_descs[$i];
      if ( scancfg_check_for_chip_type($type) )
      {  my @data = ();
         $parm = 'name';
         pcf_icd_get_parms($i, $parm, \@data, $ecode, $emess);
         if ( $ecode < 0 )
         {  &$comm_err_fun($emess);
            $ecode = 0;
            $emess = "";
            return($FALSE);
         }
         $icd_names{$type} = \@data;

         my @sel_data = ();
         for ($j = 0; $j <= $#data; $j++)
         {  push(@sel_data, $j);
         }
         $icd_defsel{$type} = \@sel_data;

         my @data = ();
         $parm = 'summary';
         pcf_icd_get_parms($i, $parm, \@data, $ecode, $emess);
         if ( $ecode < 0 )
         {  &$comm_err_fun($emess);
            $ecode = 0;
            $emess = "";
            return($FALSE);
         }
         $icd_summs{$type} = \@data;
      }
   }

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "\n-> Building the definition structure for each supported chip:\n";
   }

   #---------------------------------------
   #  Build the definition structures
   #---------------------------------------
   foreach $mref ( @module_ptrs )
   {  $chn_list = $$mref{CHN_REFS}; # List of chain definition pointers
      for ($c = 0; $c <= $#$chn_list; $c++)
      {  $chn_ptr = $$chn_list[$c];
         $chip_list = $$chn_ptr[$CHN_IND_CREFS];
         foreach $chip_ptr ( @$chip_list )
         {  $curr_type = $$chip_ptr[$CMP_IND_DESC];
            for ($i = 0, $found = $FALSE; (($i <= $#chip_descs) && !$found); $i++)
            {  if ( $curr_type =~ /^$chip_descs[$i]/i )
               {  $found = $TRUE;
               }
            }
            if ( $found ) # Chip type supports internal scan chain operations
            {  $i--;
               my @def_data = ( );
               $def_data[$ICD_MOD_NAME] = $$mref{NAME};
               $def_data[$ICD_BRD_NAME] = $$chip_ptr[$CMP_IND_BOARD];
               $def_data[$ICD_CHP_NAME] = $$chip_ptr[$CMP_IND_LOG];
               $def_data[$ICD_TYPE]     = $chip_descs[$i];
               $def_data[$ICD_DESC]     = $$chip_ptr[$CMP_IND_DESC];
               $def_data[$ICD_INDEX]    = $i;
               $def_data[$ICD_LIST_REF] = $icd_defsel{ $chip_descs[$i] };
               push(@chip_icd_ptrs, \@def_data);
            }
         }
      }
   }

   if ( $debug_flag & $DEBUG_DETAILS )
   {   printf "   Number of chips that support internal scan = %d\n", ($#chip_icd_ptrs + 1);
      foreach $def_ref (@chip_icd_ptrs)
      {  printf "   def_ref = $def_ref = ( @$def_ref )\n";
      }
   }

   #---------------------------------------
   #  Check number of supported chips
   #---------------------------------------
   if ( $#chip_icd_ptrs < 0 )
   {  $ecode = -1;
      $emess = "None of the chips in the current configuration support internal scan chain operations.";
      &$comm_err_fun($emess);
      $ecode = 0;
      $emess = "";
      return($FALSE);
   }

   #---------------------------------------
   #  Set the initialize flag
   #---------------------------------------
   $icd_setup = $TRUE;
   return($TRUE);
}


#-------------------------------------------------------------------------------

sub icdef_build_chain_list
{  
   printf "\nicdef_build_chain_list( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $def_ref  = $_[0];  # Reference to definition structure
   my ($ind, $type, $name_ptr, $text, $sel_list, $i);

   $type     = $$def_ref[$ICD_TYPE];
   $name_ptr = $icd_names{$type};   
   $text     = "";
   $sel_list = $$def_ref[$ICD_LIST_REF];
   if ( $sel_list == 0 )
   {  return($text);
   }
   elsif ( $#$sel_list == -1 )
   {  return($text);
   }
   $ind  = $$sel_list[0];
   $text = $$name_ptr[$ind];
   for ($i = 1; $i <= $#$sel_list; $i++)
   {  $ind  = $$sel_list[$i];
      $text .= ', ' . $$name_ptr[$ind];
   }
   return($text);
}


#-------------------------------------------------------------------------------

sub icdef_find_def_index
{
   printf "\nicdef_find_def_index( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $def_ref  = $_[1]; # Reference to definition instance
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];
   my ($ind, $found);

   #---------------------------------------
   #  Find index of definition in list
   #---------------------------------------
   for ($ind=0, $found=$FALSE; (($ind <= $#$list_ref) && !$found); $ind++)
   {  if ( $$list_ref[$ind] == $def_ref )
      {  $found = $TRUE;
      }
   }
   return ( $found ? ($ind - 1) : -1 );
}


#-------------------------------------------------------------------------------

sub icdef_update_definition
{
   printf "\nicdef_update_definition( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $ind      = $_[1]; # Index of definition to update
   my $sel_ref  = 0;
   if ( $#_ >= 2 )
   {  $sel_ref = $_[2];  # Optional: reference to new selection list
   } 
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];

   #---------------------------------------
   #  Update structure and build string
   #---------------------------------------
   my $def_ref  = $$list_ref[$ind];
   if ( ref($sel_ref) eq 'ARRAY' )
   {  $$def_ref[$ICD_LIST_REF] = $sel_ref;
   }
   my $chn_text = icdef_build_chain_list($def_ref);
   my $name = sprintf("%s:%s:%s", $$def_ref[$ICD_MOD_NAME], $$def_ref[$ICD_BRD_NAME], $$def_ref[$ICD_CHP_NAME]);
   my $text = sprintf("%-20s ($chn_text)", $name);

   #---------------------------------------
   #  Update the string in the listbox
   #---------------------------------------
   $$iwd_ref[$ICW_LISTBOX]->delete($ind);
   $$iwd_ref[$ICW_LISTBOX]->insert($ind, $text);
   $$iwd_ref[$ICW_LISTBOX]->selectionClear(0, 'end');
   $$iwd_ref[$ICW_LISTBOX]->selection('set', $ind);
}


#-------------------------------------------------------------------------------

sub icdef_set_widget_state
{
   printf "\nicdef_set_widget_state( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return if ( $#_ != 1 );
   my $iwd_ref = $_[0];  # Reference to selection window data
   my $btn_state;

   #---------------------------------------
   #  Determine state: starting or stopping sequence
   #---------------------------------------
   if ( $_[1] eq 'stop' )
   {  $$iwd_ref[$ICW_BTN_CANCEL]->configure(-state => 'disabled',
         -background => $$iwd_ref[$ICW_COLOR_BCK],   
         -activebackground => $$iwd_ref[$ICW_COLOR_ACT]);
      $$iwd_ref[$ICW_BTN_BOTH]->configure(
         -background => 'green',
         -activebackground => 'green');
      $btn_state = 'normal';
   }
   else
   {  $$iwd_ref[$ICW_BTN_CANCEL]->configure(-state => 'normal',
         -background => 'red',
         -activebackground => 'red');
      $$iwd_ref[$ICW_BTN_BOTH]->configure(
         -background => $$iwd_ref[$ICW_COLOR_BCK],   
         -activebackground => $$iwd_ref[$ICW_COLOR_ACT]);
      $btn_state = 'disabled';
   }

   #---------------------------------------
   #  Set the buttons to the specified state
   #---------------------------------------
   $$iwd_ref[$ICW_BTN_BOTH]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_STOP]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_READ]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_MODIFY]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_ALL]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_NONE]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_COPY]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_MVUP]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_MVDN]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_BROWSE]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_CLEAR]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_SAVE]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_BTN_CLOSE]->configure(-state => $btn_state);

   #---------------------------------------
   #  Set the menus to the specified state
   #---------------------------------------
   $$iwd_ref[$ICW_CASMENU0]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_CASMENU1]->configure(-state => $btn_state);
   $$iwd_ref[$ICW_CASMENU2]->configure(-state => $btn_state);
}


#-------------------------------------------------------------------------------

sub icdef_set_cur_definition
{
   printf "\nicdef_set_cur_definition( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $def_ref  = $_[1]; # Reference to definition instance

   #---------------------------------------
   #  Find index of definition in list
   #---------------------------------------
   my $ind = icdef_find_def_index($iwd_ref, $def_ref);
   return if ( $ind == -1 );

   #---------------------------------------
   #  Reset the listbox selection
   #---------------------------------------
   $$iwd_ref[$ICW_LISTBOX]->selectionClear(0, 'end');
   $$iwd_ref[$ICW_LISTBOX]->selection('set', $ind);
}

#-------------------------------------------------------------------------------

sub icdef_cbf_reset_all
{
   printf "\nicdef_cbf_reset_all( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $mode     = $_[1]; # Reset mode: all, none, order or reset
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];
   my $sort_ref = $$iwd_ref[$ICW_SORT_REF];
   my ($i, $j, $def_ref, $type, $name_ptr, $cur, $chn_text, $text, $name);

   #---------------------------------------
   #  Reset the chip order in the listbox
   #---------------------------------------
   if ( ($mode eq 'reset') || ($mode eq 'order') )
   {  for ($i = 0; $i <= $#$sort_ref; $i++)
      {  $$list_ref[$i] = $$sort_ref[$i];
      }
   }

   #---------------------------------------
   #  Reset the chain definitions for each
   #---------------------------------------
   if ( $mode ne 'order' )
   {  for ($i = 0; $i <= $#$list_ref; $i++)
      {  $def_ref  = $$list_ref[$i];
         $type     = $$def_ref[$ICD_TYPE];
         $name_ptr = $icd_names{$type}; 
         if ( $mode ne 'none' )
         {  $$def_ref[$ICD_LIST_REF] = $icd_defsel{$type};
         }
         else
         {  my @sel_data = ();
            $$def_ref[$ICD_LIST_REF] = \@sel_data;
         }
      }
   }

   #---------------------------------------
   #  Determine original listbox selection index
   #---------------------------------------
   if ( ($mode eq 'order') || ($mode eq 'reset') )
   {  $cur = 0;
   }
   else
   {  $cur = $$iwd_ref[$ICW_LISTBOX]->curselection();
      if ( !defined($cur) )
      {  $cur = 0;
      }
   }

   #---------------------------------------
   #  Delete listbox entries and re-insert
   #---------------------------------------
   $$iwd_ref[$ICW_LISTBOX]->delete(0, 'end');
   for ($i = 0; $i <= $#$list_ref; $i++)
   {  $def_ref  = $$list_ref[$i];
      $chn_text = icdef_build_chain_list($def_ref);
      $name = sprintf("%s:%s:%s", $$def_ref[$ICD_MOD_NAME], $$def_ref[$ICD_BRD_NAME], $$def_ref[$ICD_CHP_NAME]);
      $text = sprintf("%-20s ($chn_text)", $name);
      $$iwd_ref[$ICW_LISTBOX]->insert('end', $text);
   }
   $$iwd_ref[$ICW_LISTBOX]->selection('set', $cur);
}

#-------------------------------------------------------------------------------

sub icdef_cbf_cancel
{
   printf "\nicdef_cbf_cancel( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data

   $$iwd_ref[$ICW_STOP_FLAG] = $TRUE;
}


#-------------------------------------------------------------------------------

sub icdef_cbf_run
{
   printf "\nicdef_cbf_run( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $mode     = $_[1]; # Mode of operation: 'both', 'stop' or 'read'
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];
   my $win_h    = $$iwd_ref[$ICW_WINDOW];
   my $res_wid  = $$iwd_ref[$ICW_HIST_WID];  # Reference to command results widget
   my $out_name = $$iwd_ref[$ICW_NAME_TEXT];
   my ($def_ref, $sel_list, $i, $found, $time_stamp);
   my @mod_list   = ();
   my @parms      = ();
   my $chip_count = 0;

   #---------------------------------------
   #  Check the output file name
   #---------------------------------------
   $out_name =~ s/^\s+//g;  # Remove all leading whitespace
   $out_name =~ s/\s+$//g;  # Remove all trailing whitespace
   if ( length($out_name) < 1 )
   {  $text  = "Invalid or missing output file name specified.";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
         -title => 'Error: No Output File Specified');
      return;
   }

   #---------------------------------------
   #  Build list of modules names and count chips
   #---------------------------------------
   foreach $def_ref ( @$list_ref )
   {  $sel_list = $$def_ref[$ICD_LIST_REF];
      if ( ($#$sel_list >= 0) || ($$iwd_ref[$ICW_STOP_MODE] == $STOP_MODE_ALL) )
      {  for ($i = 0, $found = $FALSE; (($i <= $#mod_list) && !$found); $i++)
         {  if ( $$def_ref[$ICD_MOD_NAME] eq $mod_list[$i] )
            {  $found = $TRUE;
            }
         }
         if ( !$found )
         {  push(@mod_list, $$def_ref[$ICD_MOD_NAME]);
         }
      } 
      if ( $#$sel_list >= 0 )
      {  $chip_count += 1;
      }
   }

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "\n-> processed definitions :\n";
      printf "module_list = ( @mod_list )\n";
      printf "chip_count = $chip_count\n";
   }

   #---------------------------------------
   #  Error message if no modules or chains
   #---------------------------------------
   if ( $#mod_list < 0 )
   {  $text  = "No modules defined with valid chains.";
      $text .= "\nTo perform operation, define scan chain(s) or change stop clock option.";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
         -title => 'Error: No Targets for Operation');
      return;
   }
   if ( ($mode eq 'read') && ($chip_count == 0) )
   {  $text  = "No modules defined with valid chains.";
      $text .= "\nTo perform operation, define scan chain(s).";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
         -title => 'Error: No Targets for Operation');
      return;
   }

   #---------------------------------------
   #  Confirm the user wants to continue
   #---------------------------------------
   $text = "The system's normal operation will be corrupted.";
   if ( ($mode eq 'both') && ($chip_count == 0) )
   {  $title = "Confirm: Stop Clocks";
      $text .= "\nNOTE: no chains are defined for read operation.";
   }
   elsif ( $mode eq 'stop' )
   {  $title = "Confirm: Stop Clocks";
   }
   elsif ( $mode eq 'read' )
   {  $title = "Confirm: Read Internal Scan Chains";
      $text .= "\nNOTE: the clocks must be stopped before reading scan chains.";
   }
   else
   {  $title = "Confirm: Stop Clocks and Read Internal Scan Chains";
   }
   $text .= "\nDo you want to continue?";
   $rv = $win_h->messageBox(-icon => 'question', -type => 'YesNo', -message => $text, -title => $title);
   if ( $rv eq 'No' )
   {  return;
   }

   #---------------------------------------
   #  Disable the buttons during operation
   #---------------------------------------
   icdef_set_widget_state($iwd_ref, 'start');
   $$iwd_ref[$ICW_STOP_FLAG] = $FALSE;

   #---------------------------------------
   #  Open the output file and display the header
   #---------------------------------------
   $time_stamp = jtag_dump_file_open($out_name);
   if ( $ecode < 0 )
   {  $text = $icd_error_mess;
      $res_wid->insert('end', $text);
      printf $text;
      $res_wid->see('end');
      jtag_dump_file_add($text);
      goto icdef_cbf_run_done_label;
   }

   $text = "\n";
   $res_wid->insert('end', $text);
   printf $text;
   $text = "################################################################\n";
   $res_wid->insert('end', $text);
   printf $text;
   $text = "###   BEGIN SEQUENCE @ $time_stamp              ###\n";
   $res_wid->insert('end', $text);
   printf $text;
   $text = "################################################################\n";
   $res_wid->insert('end', $text);
   printf $text;
   $res_wid->see('end');
   $win_h->update();

   #---------------------------------------
   #  Check to see if user CANCEL'ed operation
   #---------------------------------------
   if ( $$iwd_ref[$ICW_STOP_FLAG] )
   {  $text = $icd_cancel_mess;
      $res_wid->insert('end', $text);
      printf $text;
      $res_wid->see('end');
      jtag_dump_file_add($text);
      goto icdef_cbf_run_done_label;
   }

   #---------------------------------------
   #  Stop the clocks on targeted module(s)
   #---------------------------------------
   if ( ($mode eq 'both') || ($mode eq 'stop') )
   {  $text = "\n-> Issuing JTAG operations to STOP clocks :\n";
      $res_wid->insert('end', $text);
      $text = "   ( @mod_list )\n";
      $res_wid->insert('end', $text);
      $res_wid->see('end');
      $win_h->update();

      jtag_stop_clocks(\@mod_list);
      if ( $ecode < 0 )
      {  $text = $icd_error_mess;
         $res_wid->insert('end', $text);
         printf $text;
         $res_wid->see('end');
         jtag_dump_file_add($text);
         goto icdef_cbf_run_done_label;
      }

      $text = "\n-> Clocks should now be stopped on listed bricks\n";
      $res_wid->insert('end', $text);
      $res_wid->see('end');
      $win_h->update();
   }

   #---------------------------------------
   #  Check to see if user CANCEL'ed operation
   #---------------------------------------
   if ( $$iwd_ref[$ICW_STOP_FLAG] )
   {  $text = $icd_cancel_mess;
      $res_wid->insert('end', $text);
      printf $text;
      $res_wid->see('end');
      jtag_dump_file_add($text);
      goto icdef_cbf_run_done_label;
   }

   #---------------------------------------
   #  Read the internal scan chain(s)
   #---------------------------------------
   if ( (($mode eq 'both') || ($mode eq 'read')) && ($chip_count > 0) )
   {
      $text = "\n-> Reading internal scan chain data to file :\n";
      $res_wid->insert('end', $text);
      $res_wid->see('end');
      $win_h->update();

      foreach $def_ref ( @$list_ref )
      {  
         $sel_list = $$def_ref[$ICD_LIST_REF];
         if ( $#$sel_list >= 0 )
         {
         $chn_text = icdef_build_chain_list($def_ref);
         $mname    = $$def_ref[$ICD_MOD_NAME];
         $name     = sprintf("%s:%s:%s", $mname, $$def_ref[$ICD_BRD_NAME], $$def_ref[$ICD_CHP_NAME]);
         $text     = sprintf("   %-20s ... processing ... please wait ...", $name);
         $res_wid->insert('end', $text);
         $res_wid->see('end');
         $win_h->update();

         #---------------------------------------
         #  Check to see if user CANCEL'ed operation
         #---------------------------------------
         if ( $$iwd_ref[$ICW_STOP_FLAG] )
         {  $text = $icd_cancel_mess;
            $res_wid->insert('end', $text);
            printf $text;
            $res_wid->see('end');
            jtag_dump_file_add($text);
            goto icdef_cbf_run_done_label;
         }

         if ( !(exists($modules{$mname})) )
         {  $emess = "Unknown module passed to subroutine icdef_cbf_run().";
            $emess .= "\nModule: $mname";
            $emess .= "\nModule not found in current configuration.";
            &$comm_err_fun($emess);
            $text = $icd_error_mess;
            $res_wid->insert('end', $text);
            printf $text;
            $res_wid->see('end');
            jtag_dump_file_add($text);
            goto icdef_cbf_run_done_label;
            return;
         }
         $mref = $modules{$mname};

         $dref = scan_module_chip_details($mref, $$def_ref[$ICD_BRD_NAME], 
                    $$def_ref[$ICD_CHP_NAME], 'logical');
         if ( !defined($dref) )
         {  $emess = "Unknown target board/chip passed to subroutine icdef_cbf_run().";
            $emess .= "\nModule: $mname";
            $emess .= "\nBoard:  $$def_ref[$ICD_BRD_NAME]";
            $emess .= "\nChip:   $$def_ref[$ICD_CHP_NAME]";
            $emess .= "\nLocation not found in current configuration.";
            &$comm_err_fun($emess);
            $text = $icd_error_mess;
            $res_wid->insert('end', $text);
            printf $text;
            $res_wid->see('end');
            jtag_dump_file_add($text);
            goto icdef_cbf_run_done_label;
            return;
         }
         if ( $debug_flag & $DEBUG_DETAILS )
         {  printf("%-20s = $dref = ( @$dref )\n", $name);
         }

         @parms = ();
         $parms[0] = $$dref[$DETAIL_IND_MLOC];
         $parms[1] = $$dref[$DETAIL_IND_ILEN];
         $parms[2] = $$dref[$DETAIL_IND_IPOS];
         $parms[3] = $$dref[$DETAIL_IND_CLEN];
         $parms[4] = $$dref[$DETAIL_IND_CPOS];
         $parms[5] = 0x02;  # mode = INTCHN_MODE_CIR
#        $parms[5] = 0x00;  # mode = normal
         $parms[6] = $$dref[$DETAIL_IND_MNUM]; # Number number/index in configuration
         $parms[7] = $$dref[$DETAIL_IND_CNUM]; # Chain number/index on module

         jtag_dump_chip($name, $$dref[$DETAIL_IND_TYPE], $$def_ref[$ICD_INDEX], \@parms, $sel_list);
         if ( $ecode < 0 )
         {  $text = $icd_error_mess;
            $res_wid->insert('end', $text);
            printf $text;
            $res_wid->see('end');
            jtag_dump_file_add($text);
            goto icdef_cbf_run_done_label;
         }

         $text     = "($chn_text)\n";
         $res_wid->delete('end - 35 chars', 'end');
         $res_wid->insert('end', $text);
         $res_wid->see('end');
         $win_h->update();

         #---------------------------------------
         #  Check to see if user CANCEL'ed operation
         #---------------------------------------
         if ( $$iwd_ref[$ICW_STOP_FLAG] )
         {  $text = $icd_cancel_mess;
            $res_wid->insert('end', $text);
            printf $text;
            $res_wid->see('end');
            jtag_dump_file_add($text);
            goto icdef_cbf_run_done_label;
         }

         }
      }

   }

icdef_cbf_run_done_label:

   $time_stamp = jtag_dump_file_close();

   $text = "\n-> Results saved to file : $out_name\n";
   $res_wid->insert('end', $text);
   $res_wid->see('end');

   $text = "\n";
   $res_wid->insert('end', $text);
   printf $text;
   $text = "################################################################\n";
   $res_wid->insert('end', $text);
   printf $text;
   $text = "###   END SEQUENCE @ $time_stamp                ###\n";
   $res_wid->insert('end', $text);
   printf $text;
   $text = "################################################################\n";
   $res_wid->insert('end', $text);
   printf $text;
   $res_wid->see('end');

   icdef_set_widget_state($iwd_ref, 'stop');
}


#-------------------------------------------------------------------------------

sub icdef_cbf_browse
{
   printf "\nicdef_cbf_browse( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $win_h    = $$iwd_ref[$ICW_WINDOW];

   my @types = ( ['Internal Scan Chain Dump (Text)', '.dmp'],
                 ['All Files',                     , '*'   ]  );
   my $out_file = $win_h->getSaveFile(
                  -title => 'Save Internal Chain Data',
                  -filetypes => \@types,
                  -defaultextension => '.dmp'
                   );
#   my $out_file = $win_h->getSaveFile();
   return if ( !defined($out_file) );

   $$iwd_ref[$ICW_NAME_TEXT] = $out_file;
}


#-------------------------------------------------------------------------------

sub icdef_cbf_modify
{
   printf "\nicdef_cbf_modify( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];

   #---------------------------------------
   #  Return if no definition is selected
   #---------------------------------------
   my $cur = $$iwd_ref[$ICW_LISTBOX]->curselection();
   return if ( !defined($cur) );

   #---------------------------------------
   #  Create window to modify definition
   #---------------------------------------
   my $def_ref  = $$list_ref[$cur];
   my $type     = $$def_ref[$ICD_TYPE];
   my $name_ptr = $icd_summs{$type}; 
   my $curr_ptr = $$def_ref[$ICD_LIST_REF];
   my $sel_win  = 0;
   my $name     = sprintf("%s:%s:%s", $$def_ref[$ICD_MOD_NAME], $$def_ref[$ICD_BRD_NAME], $$def_ref[$ICD_CHP_NAME]);
   my $hdr      = "$name Internal Scan Chains";
   my @parms    = ( $iwd_ref, $def_ref, 0, 0 );
   my $mode     = $NAMESEL_MODE_DEFAULT | $NAMESEL_MODE_EMPTY_LIST;

   create_namesel_window($hdr, $mode, "Accept", $name_ptr, "", $curr_ptr, 
      \&icdef_modify_create_sub, \&icdef_modify_cancel_sub, \@parms, \$sel_win);

   #---------------------------------------
   #  Store the window handle 
   #---------------------------------------
   $parms[3]    = $sel_win;
   my $win_list = $$iwd_ref[$ICW_WIN_LIST];
   push(@$win_list, $sel_win);
}


#-------------------------------------------------------------------------------

sub icdef_cbf_none
{
   printf "\nicdef_cbf_none( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data

   #---------------------------------------
   #  Return if no definition is selected
   #---------------------------------------
   my $cur = $$iwd_ref[$ICW_LISTBOX]->curselection();
   return if ( !defined($cur) );

   #---------------------------------------
   #  Remove all chains and update
   #---------------------------------------
   my @sel_data = ();
   icdef_update_definition($iwd_ref, $cur, \@sel_data);
}


#-------------------------------------------------------------------------------

sub icdef_cbf_all
{
   printf "\nicdef_cbf_all( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data

   #---------------------------------------
   #  Return if no definition is selected
   #---------------------------------------
   my $cur = $$iwd_ref[$ICW_LISTBOX]->curselection();
   return if ( !defined($cur) );

   #---------------------------------------
   #  List all chains and update
   #---------------------------------------
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];
   my $def_ref  = $$list_ref[$cur];
   my $type     = $$def_ref[$ICD_TYPE];
   icdef_update_definition($iwd_ref, $cur, $icd_defsel{$type});
}


#-------------------------------------------------------------------------------

sub icdef_cbf_copy
{
   printf "\nicdef_cbf_copy( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $lbox     = $$iwd_ref[$ICW_LISTBOX];
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];

   #---------------------------------------
   #  Return if no definition is selected
   #---------------------------------------
   my $cur = $lbox->curselection();
   return if ( !defined($cur) );

   #---------------------------------------
   #  Build lists of similar chips
   #---------------------------------------
   my $def_ref  = $$list_ref[$cur];
   my $type     = $$def_ref[$ICD_TYPE];
   my @names    = ();
   my @refs     = ();
   my ($i, $name);
   for ($i = 0; $i <= $#$list_ref; $i++)
   {  $ref = $$list_ref[$i];
      if ( ($i != $cur) && ($$ref[$ICD_TYPE] eq $type) )
      {  $name = sprintf("%s:%s:%s", $$ref[$ICD_MOD_NAME], $$ref[$ICD_BRD_NAME], $$ref[$ICD_CHP_NAME]);
         push(@names, $name);
         push(@refs,  $ref);
      }
   }
   if ( $#names == -1 )
   {  my $text = "There are no other chips of type \"$type\" in the configuration.";
      $$iwd_ref[$ICW_WINDOW]->messageBox(-icon => 'error', -type => 'OK',
            -message => $text, -title => 'Error: Copy Internal Scan Chain Definition');
      return;
   }
   
   #---------------------------------------
   #  Create window to copy definition
   #---------------------------------------
   my @sel_data = ();
   my $sel_win  = 0;
   $name        = sprintf("%s:%s:%s", $$def_ref[$ICD_MOD_NAME], $$def_ref[$ICD_BRD_NAME], $$def_ref[$ICD_CHP_NAME]);
   my $hdr      = "for Copy of $name Chain Definition";
   my @parms    = ( $iwd_ref, $def_ref, \@refs, 0 );
   my $mode     = $NAMESEL_MODE_DEFAULT;

   create_namesel_window($hdr, $mode, "Copy Definition", \@names, "", \@sel_data,
      \&icdef_copy_create_sub, \&icdef_copy_cancel_sub, \@parms, \$sel_win);

   #---------------------------------------
   #  Store the window handle 
   #---------------------------------------
   $parms[3]    = $sel_win;
   my $win_list = $$iwd_ref[$ICW_WIN_LIST];
   push(@$win_list, $sel_win);
}


#-------------------------------------------------------------------------------

sub icdef_cbf_move_up
{
   printf "\nicdef_cbf_move_up( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $lbox     = $$iwd_ref[$ICW_LISTBOX];

   #---------------------------------------
   #  Return if no definition is selected
   #---------------------------------------
   my $cur = $lbox->curselection();
   return if ( !defined($cur) );
   return if ( $cur == 0 );

   #---------------------------------------
   #  Delete the selected entry and insert in new position
   #---------------------------------------
   my $pos  = $cur - 1;
   my $text = $lbox->get($cur);
   $lbox->delete($cur);
   $lbox->insert($pos, $text);
   $lbox->selectionClear(0, 'end');
   $lbox->selection('set', $pos);

   #---------------------------------------
   #  Update the list of pointers to match
   #---------------------------------------
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];
   my $tmp = $$list_ref[$pos];
   $$list_ref[$pos] = $$list_ref[$cur];
   $$list_ref[$cur] = $tmp;
}


#-------------------------------------------------------------------------------

sub icdef_cbf_move_down
{
   printf "\nicdef_cbf_move_down( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $lbox     = $$iwd_ref[$ICW_LISTBOX];
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];

   #---------------------------------------
   #  Return if no definition is selected
   #---------------------------------------
   my $cur = $lbox->curselection();
   return if ( !defined($cur) );
   return if ( $cur == $#$list_ref );

   #---------------------------------------
   #  Delete the selected entry and insert in new position
   #---------------------------------------
   my $pos  = $cur + 1;
   my $text = $lbox->get($cur);
   $lbox->delete($cur);
   $lbox->insert($pos, $text);
   $lbox->selectionClear(0, 'end');
   $lbox->selection('set', $pos);

   #---------------------------------------
   #  Update the list of pointers to match
   #---------------------------------------
   my $tmp = $$list_ref[$pos];
   $$list_ref[$pos] = $$list_ref[$cur];
   $$list_ref[$cur] = $tmp;
}


#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

sub icdef_modify_create_sub
{
   printf "\nicdef_modify_create_sub( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm_ref = $_[0];  # Parameter registered for callback subroutine
   my $name     = $_[1];  # Name or title for group of selected strings/names
   my $aref     = $_[2];  # Reference to array with indices of selected strings/names

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "parm : $parm_ref = ( @$parm_ref )\n";
      printf "name : $name\n";
      printf "list : $aref = ( @$aref )\n";
   }

   #---------------------------------------
   #  Extract the parameters and find index
   #---------------------------------------
   my $iwd_ref  = $$parm_ref[0]; # Reference to group window data
   my $def_ref  = $$parm_ref[1]; # Reference to definition instance
   my $tmp      = $$parm_ref[2]; # Unused
   my $win_ref  = $$parm_ref[3]; # Reference to the name selection window

   #---------------------------------------
   #  Remove selection window from list
   #---------------------------------------
   if ( icdef_delete_sel_window_ref($iwd_ref, $win_ref) == $FALSE )
   {  return;
   }

   #---------------------------------------
   #  Find index of definition in list
   #---------------------------------------
   my $ind = icdef_find_def_index($iwd_ref, $def_ref);
   return if ( $ind == -1 );

   #---------------------------------------
   #  Update the definition (chain list)
   #---------------------------------------
   icdef_update_definition($iwd_ref, $ind, $aref);
}


#-------------------------------------------------------------------------------

sub icdef_modify_cancel_sub
{
   printf "\nicdef_modify_cancel_sub( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm_ref = $_[0];  # Parameter registered for callback subroutine

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "parm : $parm_ref = ( @$parm_ref )\n";
   }

   #---------------------------------------
   #  Extract the parameters and find index
   #---------------------------------------
   my $iwd_ref  = $$parm_ref[0]; # Reference to group window data
   my $def_ref  = $$parm_ref[1]; # Reference to definition instance
   my $tmp      = $$parm_ref[2]; # Unused
   my $win_ref  = $$parm_ref[3]; # Reference to the name selection window

   #---------------------------------------
   #  Remove selection window from list
   #---------------------------------------
   if ( icdef_delete_sel_window_ref($iwd_ref, $win_ref) == $FALSE )
   {  return;
   }

   #---------------------------------------
   #  Reset the selected definition
   #---------------------------------------
   icdef_set_cur_definition($iwd_ref, $def_ref);
}


#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

sub icdef_copy_create_sub
{
   printf "\nicdef_copy_create_sub( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm_ref = $_[0];  # Parameter registered for callback subroutine
   my $name     = $_[1];  # Name or title for group of selected strings/names
   my $aref     = $_[2];  # Reference to array with indices of selected strings/names

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "parm : $parm_ref = ( @$parm_ref )\n";
      printf "name : $name\n";
      printf "list : $aref = ( @$aref )\n";
   }

   #---------------------------------------
   #  Extract the parameters 
   #---------------------------------------
   my $iwd_ref  = $$parm_ref[0]; # Reference to group window data
   my $def_ref  = $$parm_ref[1]; # Reference to definition instance
   my $ptr_list = $$parm_ref[2]; # List of similar definitions
   my $win_ref  = $$parm_ref[3]; # Reference to the name selection window
   my ($loc, $ind);

   #---------------------------------------
   #  Remove selection window from list
   #---------------------------------------
   if ( icdef_delete_sel_window_ref($iwd_ref, $win_ref) == $FALSE )
   {  return;
   }

   return if ( $#$aref == -1 );  # Return if none selected

   #---------------------------------------
   #  Copy the selected list for the specific chips
   #---------------------------------------
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];
   my $sel_list = $$def_ref[$ICD_LIST_REF]; # List of selected chains
   foreach $loc ( @$aref )
   {  $cur_ref = $$ptr_list[$loc];
      $ind     = icdef_find_def_index($iwd_ref, $cur_ref);
      if ( $ind != -1 )
      {  icdef_update_definition($iwd_ref, $ind, $sel_list);
      }
   }

   #---------------------------------------
   #  Reset the selected definition
   #---------------------------------------
   icdef_set_cur_definition($iwd_ref, $def_ref);
}


#-------------------------------------------------------------------------------

sub icdef_copy_cancel_sub
{
   printf "\nicdef_copy_cancel_sub( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm_ref = $_[0];  # Parameter registered for callback subroutine

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "parm : $parm_ref = ( @$parm_ref )\n";
   }

   #---------------------------------------
   #  Extract the parameters 
   #---------------------------------------
   my $iwd_ref  = $$parm_ref[0]; # Reference to group window data
   my $def_ref  = $$parm_ref[1]; # Reference to definition instance
   my $ptr_list = $$parm_ref[2]; # List of similar definitions
   my $win_ref  = $$parm_ref[3]; # Reference to the name selection window

   #---------------------------------------
   #  Remove selection window from list
   #---------------------------------------
   if ( icdef_delete_sel_window_ref($iwd_ref, $win_ref) == $FALSE )
   {  return;
   }

   #---------------------------------------
   #  Reset the selected definition
   #---------------------------------------
   icdef_set_cur_definition($iwd_ref, $def_ref);
}


#-------------------------------------------------------------------------------

sub icdef_delete_sel_window_ref
{
   printf "\nicdef_delete_sel_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0]; # Reference to group window data
   my $win_ref  = $_[1]; # Reference to the name selection window
   my ($ind, $found);

   #---------------------------------------
   #  Remove selection window from list
   #---------------------------------------
   my $win_list = $$iwd_ref[$ICW_WIN_LIST];
   for ($ind=0, $found=$FALSE; (($ind <= $#$win_list) && !$found); $ind++)
   {  if ( $$win_list[$ind] == $win_ref )
      {  $found = $TRUE;
      }
   }
   return($FALSE) if ( !$found );
   $ind--;
   splice(@$win_list, $ind, 1);
   return($TRUE);
}

#-------------------------------------------------------------------------------

sub icdef_cbf_hist_btn
{
   print "\nicdef_cbf_hist_btn( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return if ( $#_ != 1 );
   my $iwd_ref  = $_[0];  # Reference to window data
   my $mode     = $_[1];  # Mode: 'save' or 'clear'
   if ( $debug_flag & $DEBUG_GENERAL )
   {  printf "CWD_REF  = $iwd_ref = ( @$iwd_ref )\n";
      printf "MODE     = $mode\n";
   }
   my $win_h      = $$iwd_ref[$ICW_WINDOW];    # Main window for dialog boxes
   my $res_widget = $$iwd_ref[$ICW_HIST_WID];  # Reference to command results widget

   #---------------------------------------
   #  Clear the command history buffer
   #---------------------------------------
   if ( $mode eq 'clear' )
   {  $res_widget->delete('1.0', 'end');
      return;
   }

   #---------------------------------------
   #  Continue: save the command history buffer
   #---------------------------------------
   my ($out_file, $file_id, $text);

   #---------------------------------------
   #  Get the contents of the command history
   #---------------------------------------
   $text = $res_widget->get('1.0', 'end');

   #---------------------------------------
   #  Get the output file name from user
   #---------------------------------------
   my @types = ( ['History or Log file (Text)', '.txt'],
                 ['All Files',                     , '*'   ]  );
    $out_file = $win_h->getSaveFile(
                  -title => 'Save Sequence History Data',
                  -filetypes => \@types,
                  -defaultextension => '.txt'
                   );
#   $out_file = $win_h->getSaveFile();
   return if ( !defined($out_file) );

   #---------------------------------------
   #  Open the output file and write the text
   #---------------------------------------
   $file_id = sprintf("hist%08lX", time());
   if ( !open($file_id, ">$out_file") )
   {  $text = "Unable to open output file for writing.\nFile: $out_file\nCommand history not saved.";
      $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
         -title => 'SuperScope: open file error');
      return;
   }
   printf $file_id $text;
   close($file_id);

   print "\nSaved command results: $out_file\n";
}


#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

sub icdef_file_load
{
   printf "\nicdef_file_load( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0];  # Reference to window data
   my $win_h    = $$iwd_ref[$ICW_WINDOW];    # Main window for dialog boxes
   my $text;

   $text = "This operation is not yet implemented...sorry!\n";
   $win_h->messageBox(-icon => 'error', -type => 'OK', -message => $text,
      -title => 'SuperScope: load file error');
}


#-------------------------------------------------------------------------------

sub icdef_file_save
{
   printf "\nicdef_file_save( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $iwd_ref  = $_[0];  # Reference to window data
   my $win_h    = $$iwd_ref[$ICW_WINDOW];    # Main window for dialog boxes
   my $list_ref = $$iwd_ref[$ICW_LIST_REF];
   my ($def_ref, $chn_text, $name, $text);

   #---------------------------------------
   #  Prompt the user for the file name
   #---------------------------------------
   my @types = ( ['Internal Chain Definition (Text)', '.icd'] ,
                 ['All Files',                      , '*'   ]  );
   my $out_file = $win_h->getSaveFile(
                  -title => 'Save Internal Chain Definition',
                  -filetypes => \@types,
                  -defaultextension => '.icd'
                   );
#   my $out_file = $win_h->getSaveFile();
   return if ( !defined($out_file) );

   #---------------------------------------
   #  Open the output file for writing
   #---------------------------------------
   if ( !open(WIN_OUT_FILE, ">$out_file") )
   {  $emess = "Unable to open output file for writing.";
      $emess .= "\nFile: $out_file";
      &comm_error_fun($emess); # TEMP!!!
      return;
   }

   #---------------------------------------
   #  Write the file header
   #---------------------------------------
   printf WIN_OUT_FILE "#\n";
   printf WIN_OUT_FILE "#  Internal Chain Definition File\n";
   printf WIN_OUT_FILE "#\n";
   printf WIN_OUT_FILE "#  The internal chain defintion file (*.icd) allows the user to customize\n";
   printf WIN_OUT_FILE "#  which internal scan chains are dumped.  Different internal chains can be defined \n";
   printf WIN_OUT_FILE "#  for different chips in the system (i.e. each chip could be defined differently).\n";
   printf WIN_OUT_FILE "#\n";
   printf WIN_OUT_FILE "#  Each line in this file must start with the keyword \"CHIP\" or \"TYPE\" and\n";
   printf WIN_OUT_FILE "#  be in the following format (lines that begin with '#' are comment lines):\n";
   printf WIN_OUT_FILE "#  CHIP <target chip> (<list of internal chains>)\n";
   printf WIN_OUT_FILE "#  TYPE <target type> (<list of internal chains>)\n";
   printf WIN_OUT_FILE "#\n";
   printf WIN_OUT_FILE "#\n";
   printf WIN_OUT_FILE "#  Dump a subset of internal chains from a specific chip in the system:\n";
   printf WIN_OUT_FILE "#     The target name is of the format <brick>:<board>:<chip> and\n";
   printf WIN_OUT_FILE "#        must match an entry in the scan chain configuration file.\n";
   printf WIN_OUT_FILE "#     The names of the chains must be from the \"defined list\" for the chip type.\n";
   printf WIN_OUT_FILE "#     The chains are dumped in the order specified within parenthesis.\n";
   printf WIN_OUT_FILE "#     An asterix '*' causes all internal chains to be dumped.\n";
   printf WIN_OUT_FILE "#\n";
   printf WIN_OUT_FILE "#KEY  TARGET                 INTERNAL CHAINS TO DUMP USING JTAG\n";
   printf WIN_OUT_FILE "#---  ---------------------- ---------------------------------------------------\n";

   #---------------------------------------
   #  Process each definition in list
   #---------------------------------------
   foreach $def_ref ( @$list_ref )
   {  $chn_text = icdef_build_chain_list($def_ref);
      $name = sprintf("%s:%s:%s", $$def_ref[$ICD_MOD_NAME], $$def_ref[$ICD_BRD_NAME], $$def_ref[$ICD_CHP_NAME]);
      printf WIN_OUT_FILE "CHIP  %-22s ($chn_text)\n", $name;
   }
   close(WIN_OUT_FILE);

   print "\nSaved internal scan chain configuration: $out_file\n";
}


#-------------------------------------------------------------------------------

1;
