#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#

#-------------------------------------------------------------------------------
#
#  FILE: ip41_memory_bit_map.dm
#
#  DESCRIPTION:  This file contains supporting PERL subroutines for the error
#     log decode utility for SHUB.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO               WHAT
#     --------  ----------------  -----------------------------------------
#     08/21/03  Nelson Crumbaker  Original source code for submission.
#     07/13/04  Nelson Crumbaker  Added x4 DRAM CHIP Information.
#-------------------------------------------------------------------------------

 @IP41_MEMORY_BIT_MAP = (
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    X4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [  24, "LEFT",   33,   24,    3,    6,   33,   24,    3,    6  ], # 0
    [  25, "LEFT",   35,   25,    3,    6,   35,   25,    3,    6  ], # 1
    [  26, "LEFT",   39,   26,    3,    6,   39,   26,    3,    6  ], # 2
    [  27, "LEFT",   40,   27,    3,    6,   40,   27,    3,    6  ], # 3
    [  32, "LEFT",   44,   64,    8,   16,   53,   32,    4,    8  ], # 4
    [  33, "LEFT",   45,   65,    8,   16,   55,   33,    4,    8  ], # 5
    [  34, "LEFT",   49,   66,    8,   16,   57,   34,    4,    8  ], # 6
    [  35, "LEFT",   51,   67,    8,   16,   60,   35,    4,    8  ], # 7
    [  40, "LEFT",   61,   40,    5,   10,   61,   40,    5,   10  ], # 8
    [  41, "LEFT",   64,   41,    5,   10,   64,   41,    5,   10  ], # 9
    [  42, "LEFT",   68,   42,    5,   10,   68,   42,    5,   10  ], # 10
    [  43, "LEFT",   69,   43,    5,   10,   69,   43,    5,   10  ], # 11
    [   0, "RIGHT", 179,   63,    7,   15,   88,   59,    7,   14  ], # 12
    [   1, "RIGHT",  87,   58,    7,   14,   87,   58,    7,   14  ], # 13
    [   2, "RIGHT",  84,   57,    7,   14,   84,   57,    7,   14  ], # 14
    [   3, "RIGHT",  83,   56,    7,   14,   83,   56,    7,   14  ], # 15
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [   0, "LEFT",    2,    0,    0,    0,    2,    0,    0,    0  ], # 16
    [   1, "LEFT",    4,    1,    0,    0,    4,    1,    0,    0  ], # 17
    [   2, "LEFT",    6,    2,    0,    0,    6,    2,    0,    0  ], # 18
    [   3, "LEFT",    8,    3,    0,    0,    8,    3,    0,    0  ], # 19
    [   8, "LEFT",   12,    8,    1,    2,   12,    8,    1,    2  ], # 20
    [   9, "LEFT",   13,    9,    1,    2,   13,    9,    1,    2  ], # 21
    [  10, "LEFT",   19,   10,    1,    2,   19,   10,    1,    2  ], # 22
    [  11, "LEFT",   20,   11,    1,    2,   20,   11,    1,    2  ], # 23
    [  10, "RIGHT", 165,   52,    6,   13,  165,   52,    6,   13  ], # 24
    [  11, "RIGHT",  72,   48,    6,   12,   72,   48,    6,   12  ], # 25
    [  16, "LEFT",   23,   16,    2,    4,   23,   16,    2,    4  ], # 26
    [  17, "LEFT",   24,   17,    2,    4,   24,   17,    2,    4  ], # 27 
    [  18, "LEFT",   28,   18,    2,    4,   28,   18,    2,    4  ], # 28
    [  19, "LEFT",   31,   19,    2,    4,   31,   19,    2,    4  ], # 29
    [  18, "RIGHT",  64,   41,    5,   10,   64,   41,    5,   10  ], # 30
    [  19, "RIGHT",  61,   40,    5,   10,   61,   40,    5,   10  ], # 31
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [  24, "RIGHT", 151,   39,    4,    9,  151,   39,    4,    9  ], # 32
    [  25, "RIGHT",  57,   34,    4,    8,   57,   34,    4,    8  ], # 33
    [  26, "RIGHT", 147,   37,    4,    9,  147,   37,    4,    9  ], # 34
    [  27, "RIGHT",  55,   33,    4,    8,   55,   33,    4,    8  ], # 35
    [  32, "RIGHT",  51,   67,    8,   16,  133,   31,    3,    7  ], # 36
    [  33, "RIGHT",  49,   66,    8,   16,   40,   27,    3,    6  ], # 37
    [  34, "RIGHT",  45,   65,    8,   16,   35,   25,    3,    6  ], # 38
    [  35, "RIGHT", 135,   69,    8,   17,   33,   24,    3,    6  ], # 39
    [  56, "RIGHT",  99,    7,    0,    1,   99,    7,    0,    1  ], # 40
    [  57, "RIGHT",  98,    6,    0,    1,   98,    6,    0,    1  ], # 41
    [  58, "RIGHT",  94,    4,    0,    1,   94,    4,    0,    1  ], # 42
    [  59, "RIGHT",   2,    0,    0,    0,    2,    0,    0,    0  ], # 43
    [  64, "RIGHT", 133,   31,    3,    7,   51,   67,    8,   16  ], # 44
    [  65, "RIGHT",  40,   27,    3,    6,   49,   66,    8,   16  ], # 45
    [  66, "RIGHT",  35,   25,    3,    6,   45,   65,    8,   16  ], # 46
    [  67, "RIGHT",  33,   24,    3,    6,  135,   69,    8,   17  ], # 47
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [  48, "LEFT",   72,   48,    6,   12,   72,   48,    6,   12  ], # 48
    [  49, "LEFT",   73,   49,    6,   12,   73,   49,    6,   12  ], # 49
    [  50, "LEFT",   79,   50,    6,   12,   79,   50,    6,   12  ], # 50
    [  51, "LEFT",   80,   51,    6,   12,   80,   51,    6,   12  ], # 51
    [  56, "LEFT",   83,   56,    7,   14,   83,   56,    7,   14  ], # 52
    [  57, "LEFT",   84,   57,    7,   14,   84,   57,    7,   14  ], # 53
    [  58, "LEFT",   87,   58,    7,   14,   87,   58,    7,   14  ], # 54
    [  59, "LEFT",   88,   59,    7,   14,   88,   59,    7,   14  ], # 55
    [  50, "RIGHT", 105,   12,    1,    3,  105,   12,    1,    3  ], # 56
    [  51, "RIGHT",  12,    8,    1,    2,   12,    8,    1,    2  ], # 57
    [  64, "LEFT",   53,   32,    4,    8,   44,   64,    8,   16  ], # 58
    [  65, "LEFT",   55,   33,    4,    8,   45,   65,    8,   16  ], # 59
    [  66, "LEFT",   57,   34,    4,    8,   49,   66,    8,   16  ], # 60
    [  67, "LEFT",   60,   35,    4,    8,   51,   67,    8,   16  ], # 61
    [  42, "RIGHT", 117,   21,    2,    5,  117,   21,    2,    5  ], # 62
    [  43, "RIGHT",  23,   16,    2,    4,   24,   17,    2,    4  ], # 63
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [  28, "LEFT",  126,   28,    3,    7,  126,   28,    3,    7  ], # 64
    [  29, "LEFT",  127,   29,    3,    7,  127,   29,    3,    7  ], # 65
    [  30, "LEFT",  131,   30,    3,    7,  131,   30,    3,    7  ], # 66
    [  31, "LEFT",  133,   31,    3,    7,  133,   31,    3,    7  ], # 67
    [  36, "LEFT",  134,   68,    8,   17,  146,   36,    4,    9  ], # 68
    [  37, "LEFT",  135,   69,    8,   17,  147,   37,    4,    9  ], # 69
    [  38, "LEFT",  142,   70,    8,   17,  150,   38,    4,    9  ], # 70
    [  39, "LEFT",  144,   71,    8,   17,  151,   39,    4,    9  ], # 71
    [  44, "LEFT",  153,   44,    5,   11,  153,   44,    5,   11  ], # 72
    [  45, "LEFT",  155,   45,    5,   11,  155,   45,    5,   11  ], # 73
    [  46, "LEFT",  161,   46,    5,   11,  161,   46,    5,   11  ], # 74
    [  47, "LEFT",  162,   47,    5,   11,  162,   47,    5,   11  ], # 75
    [   4, "RIGHT",  88,   59,    7,   14,  179,   63,    7,   15  ], # 76
    [   5, "RIGHT", 178,   62,    7,   15,  178,   62,    7,   15  ], # 77
    [   6, "RIGHT", 174,   60,    7,   15,  174,   60,    7,   15  ], # 78
    [   7, "RIGHT", 175,   61,    7,   15,  175,   61,    7,   15  ], # 79
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [   4, "LEFT",   94,    4,    0,    1,   94,    4,    0,    1  ], # 80
    [   5, "LEFT",   95,    5,    0,    1,   95,    5,    0,    1  ], # 81
    [   6, "LEFT",   98,    6,    0,    1,   98,    6,    0,    1  ], # 82
    [   7, "LEFT",   99,    7,    0,    1,   99,    7,    0,    1  ], # 83
    [  12, "LEFT",  105,   12,    1,    3,  105,   12,    1,    3  ], # 84
    [  13, "LEFT",  106,   13,    1,    3,  106,   13,    1,    3  ], # 85
    [  14, "LEFT",  109,   14,    1,    3,  109,   14,    1,    3  ], # 86
    [  15, "LEFT",  110,   15,    1,    3,  110,   15,    1,    3  ], # 87
    [  14, "RIGHT",  79,   50,    6,   12,  170,   54,    6,   13  ], # 88
    [  15, "RIGHT",  73,   49,    6,   12,   73,   49,    6,   12  ], # 89
    [  20, "LEFT",  114,   20,    2,    5,  114,   20,    2,    5  ], # 90
    [  21, "LEFT",  117,   21,    2,    5,  117,   21,    2,    5  ], # 91
    [  22, "LEFT",  121,   22,    2,    5,  121,   22,    2,    5  ], # 92
    [  23, "LEFT",  123,   23,    2,    5,  123,   23,    2,    5  ], # 93
    [  22, "RIGHT", 155,   45,    5,   11,  155,   45,    5,   11  ], # 94
    [  23, "RIGHT", 153,   44,    5,   11,  153,   44,    5,   11  ], # 95
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [  28, "RIGHT",  60,   35,    4,    8,   60,   35,    4,    8  ], # 96
    [  29, "RIGHT", 150,   38,    4,    9,  150,   38,    4,    9  ], # 97
    [  30, "RIGHT", 146,   36,    4,    9,  146,   36,    4,    9  ], # 98
    [  31, "RIGHT",  53,   32,    4,    8,   53,   32,    4,    8  ], # 99
    [  36, "RIGHT", 144,   71,    8,   17,  131,   30,    3,    7  ], # 100
    [  37, "RIGHT", 142,   70,    8,   17,   39,   26,    3,    6  ], # 101
    [  38, "RIGHT",  44,   64,    8,   16,  127,   29,    3,    7  ], # 102
    [  39, "RIGHT", 134,   68,    8,   17,  126,   28,    3,    7  ], # 103
    [  60, "RIGHT",   8,    3,    0,    0,    8,    3,    0,    0  ], # 104
    [  61, "RIGHT",   6,    2,    0,    0,    4,    1,    0,    0  ], # 105
    [  62, "RIGHT",   4,    1,    0,    0,    6,    2,    0,    0  ], # 106
    [  63, "RIGHT",  95,    5,    0,    1,   95,    5,    0,    1  ], # 107
    [  68, "RIGHT", 131,   30,    3,    7,  144,   71,    8,   17  ], # 108
    [  69, "RIGHT",  39,   26,    3,    6,  142,   70,    8,   17  ], # 109
    [  70, "RIGHT", 127,   29,    3,    7,   44,   64,    8,   16  ], # 110
    [  71, "RIGHT", 126,   28,    3,    7,  134,   68,    8,   17  ], # 111
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [  52, "LEFT",  165,   52,    6,   13,  165,   52,    6,   13  ], # 112
    [  53, "LEFT",  166,   53,    6,   13,  166,   53,    6,   13  ], # 113
    [  54, "LEFT",  170,   54,    6,   13,  170,   54,    6,   13  ], # 114
    [  55, "LEFT",  171,   55,    6,   13,  171,   55,    6,   13  ], # 115
    [  60, "LEFT",  174,   60,    7,   15,  174,   60,    7,   15  ], # 116
    [  61, "LEFT",  175,   61,    7,   15,  175,   61,    7,   15  ], # 117
    [  62, "LEFT",  178,   62,    7,   15,  178,   62,    7,   15  ], # 118
    [  63, "LEFT",  179,   63,    7,   15,  179,   63,    7,   15  ], # 119
    [  54, "RIGHT", 106,   13,    1,    3,  106,   13,    1,    3  ], # 120
    [  55, "RIGHT",  13,    9,    1,    2,   13,    9,    1,    2  ], # 121
    [  68, "LEFT",  146,   36,    4,    9,  134,   68,    8,   17  ], # 122
    [  69, "LEFT",  147,   37,    4,    9,  135,   69,    8,   17  ], # 123
    [  70, "LEFT",  150,   38,    4,    9,  142,   70,    8,   17  ], # 124
    [  71, "LEFT",  151,   39,    4,    9,  144,   71,    8,   17  ], # 125
    [  46, "RIGHT",  24,   17,    2,    4,   23,   16,    2,    4  ], # 126
    [  47, "RIGHT", 114,   20,    2,    5,  114,   20,    2,    5  ], # 127
############################################################################
#                   XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB   LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4    FSB   #
#     BIT   DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM   BIT   #
############################################################################
    [  40, "RIGHT", 123,   23,    2,    5,  123,   23,    2,    5  ], # cb0
    [  48, "RIGHT",  20,   11,    1,    2,   20,   11,    1,    2  ], # cb1
    [   8, "RIGHT", 171,   55,    6,   13,  171,   55,    6,   13  ], # cb2
    [  16, "RIGHT", 162,   47,    5,   11,  161,   46,    5,   11  ], # cb3
    [  17, "RIGHT",  69,   43,    5,   10,   69,   43,    5,   10  ], # cb4
    [   9, "RIGHT",  80,   51,    6,   12,   80,   51,    6,   12  ], # cb5
    [  49, "RIGHT", 110,   15,    1,    3,  110,   15,    1,    3  ], # cb6
    [  41, "RIGHT",  28,   18,    2,    4,   28,   18,    2,    4  ], # cb7
    [  44, "RIGHT",  31,   19,    2,    4,   31,   19,    2,    4  ], # cb8
    [  52, "RIGHT",  19,   10,    1,    2,   19,   10,    1,    2  ], # cb9
    [  12, "RIGHT", 170,   54,    6,   13,   79,   50,    6,   12  ], # cb10
    [  20, "RIGHT", 161,   46,    5,   11,  162,   47,    5,   11  ], # cb11
    [  21, "RIGHT",  68,   42,    5,   10,   68,   42,    5,   10  ], # cb12
    [  13, "RIGHT", 166,   53,    6,   13,  166,   53,    6,   13  ], # cb13
    [  53, "RIGHT", 109,   14,    1,    3,  109,   14,    1,    3  ], # cb14
    [  45, "RIGHT", 121,   22,    2,    5,  121,   22,    2,    5  ], # cb15
);  
    
1;

