#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 2005 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#

#-------------------------------------------------------------------------------
#
#  FILE: ip63_memory_bit_map.pm
#
#  DESCRIPTION:  This file contains supporting PERL subroutines for the error
#     log decode utility for SHUB.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO               WHAT
#     --------  ----------------  -----------------------------------------
#     05/25/05  Jeff Olson        Original source code for submission.
#-------------------------------------------------------------------------------

 @IP63_MEMORY_BIT_MAP = (
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [  0,  "LEFT",     3,    0,    0,    0,    3,    0,    0,    0  ], # 0
    [  1,  "LEFT",     4,    1,    0,    0,    4,    1,    0,    0  ], # 1
    [  2,  "LEFT",     9,    2,    0,    0,    9,    2,    0,    0  ], # 2
    [  3,  "LEFT",    10,    3,    0,    0,   10,    3,    0,    0  ], # 3
    [  8,  "LEFT",    12,    8,    1,    1,   12,    8,    1,    1  ], # 4
    [  9,  "LEFT",    13,    9,    1,    1,   13,    9,    1,    1  ], # 5
    [  10, "LEFT",    21,   10,    1,    1,   21,   10,    1,    1  ], # 6
    [  11, "LEFT",    22,   11,    1,    1,   22,   11,    1,    1  ], # 7
    [  16, "LEFT",    24,   16,    2,    2,   24,   16,    2,    2  ], # 8
    [  17, "LEFT",    25,   17,    2,    2,   25,   17,    2,    2  ], # 9
    [  18, "LEFT",    30,   18,    2,    2,   30,   18,    2,    2  ], # 10
    [  19, "LEFT",    31,   19,    2,    2,   31,   19,    2,    2  ], # 11
    [  24, "LEFT",    33,   24,    3,    3,   33,   24,    3,    3  ], # 12
    [  25, "LEFT",    34,   25,    3,    3,   34,   25,    3,    3  ], # 13
    [  26, "LEFT",    39,   26,    3,    3,   39,   26,    3,    3  ], # 14
    [  27, "LEFT",    40,   27,    3,    3,   40,   27,    3,    3  ], # 15
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [  32,  "LEFT",   80,   32,    4,    4,   80,   32,    4,    4  ], # 16
    [  33,  "LEFT",   81,   33,    4,    4,   81,   33,    4,    4  ], # 17
    [  34,  "LEFT",   86,   34,    4,    4,   86,   34,    4,    4  ], # 18
    [  35,  "LEFT",   87,   35,    4,    4,   87,   35,    4,    4  ], # 19
    [  40,  "LEFT",   89,   40,    5,    5,   89,   40,    5,    5  ], # 20
    [  41,  "LEFT",   90,   41,    5,    5,   90,   41,    5,    5  ], # 21
    [  42,  "LEFT",   95,   42,    5,    5,   95,   42,    5,    5  ], # 22
    [  43,  "LEFT",   96,   43,    5,    5,   96,   43,    5,    5  ], # 23
    [  48,  "LEFT",   98,   48,    6,    6,   98,   48,    6,    6  ], # 24
    [  49,  "LEFT",   99,   49,    6,    6,   99,   49,    6,    6  ], # 25
    [  56,  "LEFT",  110,   56,    7,    7,  110,   56,    7,    7  ], # 26
    [  57,  "LEFT",  111,   57,    7,    7,  111,   57,    7,    7  ], # 27
    [  58,  "LEFT",  116,   58,    7,    7,  116,   58,    7,    7  ], # 28
    [  59,  "LEFT",  117,   59,    7,    7,  117,   59,    7,    7  ], # 29
    [  64,  "LEFT",   42,   64,    8,    8,   42,   64,    8,    8  ], # 30
    [  65,  "LEFT",   43,   65,    8,    8,   43,   65,    8,    8  ], # 31
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [   0, "RIGHT",    3,    0,    0,    0,    3,    0,    0,    0  ], # 32
    [   1, "RIGHT",    4,    1,    0,    0,    4,    1,    0,    0  ], # 33
    [   2, "RIGHT",    9,    2,    0,    0,    9,    2,    0,    0  ], # 34
    [   3, "RIGHT",   10,    3,    0,    0,   10,    3,    0,    0  ], # 35
    [   8, "RIGHT",   12,    8,    1,    1,   12,    8,    1,    1  ], # 36
    [   9, "RIGHT",   13,    9,    1,    1,   13,    9,    1,    1  ], # 37
    [  10, "RIGHT",   21,   10,    1,    1,   21,   10,    1,    1  ], # 38
    [  11, "RIGHT",   22,   11,    1,    1,   22,   11,    1,    1  ], # 39
    [  16, "RIGHT",   24,   16,    2,    2,   24,   16,    2,    2  ], # 40
    [  17, "RIGHT",   25,   17,    2,    2,   25,   17,    2,    2  ], # 41
    [  18, "RIGHT",   30,   18,    2,    2,   30,   18,    2,    2  ], # 42
    [  19, "RIGHT",   31,   19,    2,    2,   31,   19,    2,    2  ], # 43
    [  24, "RIGHT",   33,   24,    3,    3,   33,   24,    3,    3  ], # 44
    [  25, "RIGHT",   34,   25,    3,    3,   34,   25,    3,    3  ], # 45
    [  26, "RIGHT",   39,   26,    3,    3,   39,   26,    3,    3  ], # 46
    [  27, "RIGHT",   40,   27,    3,    3,   40,   27,    3,    3  ], # 47
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [  32, "RIGHT",   80,   32,    4,    4,   80,   32,    4,    4  ], # 48
    [  33, "RIGHT",   81,   33,    4,    4,   81,   33,    4,    4  ], # 49
    [  34, "RIGHT",   86,   34,    4,    4,   86,   34,    4,    4  ], # 50
    [  35, "RIGHT",   87,   35,    4,    4,   87,   35,    4,    4  ], # 51
    [  40, "RIGHT",   89,   40,    5,    5,   89,   40,    5,    5  ], # 52
    [  41, "RIGHT",   90,   41,    5,    5,   90,   41,    5,    5  ], # 53
    [  42, "RIGHT",   95,   42,    5,    5,   95,   42,    5,    5  ], # 54
    [  43, "RIGHT",   96,   43,    5,    5,   96,   43,    5,    5  ], # 55
    [  48, "RIGHT",   98,   48,    6,    6,   98,   48,    6,    6  ], # 56
    [  49, "RIGHT",   99,   49,    6,    6,   99,   49,    6,    6  ], # 57
    [  56, "RIGHT",  110,   56,    7,    7,  110,   56,    7,    7  ], # 58
    [  57, "RIGHT",  111,   57,    7,    7,  111,   57,    7,    7  ], # 59
    [  58, "RIGHT",  116,   58,    7,    7,  116,   58,    7,    7  ], # 60
    [  59, "RIGHT",  117,   59,    7,    7,  117,   59,    7,    7  ], # 61
    [  64, "RIGHT",   42,   64,    8,    8,   42,   64,    8,    8  ], # 62
    [  65, "RIGHT",   43,   65,    8,    8,   43,   65,    8,    8  ], # 63
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [   4,  "LEFT",  122,    4,    0,    9,  122,    4,    0,    9  ], # 64
    [   5,  "LEFT",  123,    5,    0,    9,  123,    5,    0,    9  ], # 65
    [   6,  "LEFT",  128,    6,    0,    9,  128,    6,    0,    9  ], # 66
    [   7,  "LEFT",  129,    7,    0,    9,  129,    7,    0,    9  ], # 67
    [  12,  "LEFT",  131,   12,    1,   10,  131,   12,    1,   10  ], # 68
    [  13,  "LEFT",  132,   13,    1,   10,  132,   13,    1,   10  ], # 69
    [  14,  "LEFT",  140,   14,    1,   10,  140,   14,    1,   10  ], # 70
    [  15,  "LEFT",  141,   15,    1,   10,  141,   15,    1,   10  ], # 71
    [  20,  "LEFT",  143,   20,    2,   11,  143,   20,    2,   11  ], # 72
    [  21,  "LEFT",  144,   21,    2,   11,  144,   21,    2,   11  ], # 73
    [  22,  "LEFT",  149,   22,    2,   11,  149,   22,    2,   11  ], # 74
    [  23,  "LEFT",  150,   23,    2,   11,  150,   23,    2,   11  ], # 75
    [  28,  "LEFT",  152,   28,    3,   12,  152,   28,    3,   12  ], # 76
    [  29,  "LEFT",  153,   29,    3,   12,  153,   29,    3,   12  ], # 77
    [  30,  "LEFT",  158,   30,    3,   12,  158,   30,    3,   12  ], # 78
    [  31,  "LEFT",  159,   31,    3,   12,  159,   31,    3,   12  ], # 79
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [  36,  "LEFT",  199,   36,    4,   13,  199,   36,    4,   13  ], # 80
    [  37,  "LEFT",  200,   37,    4,   13,  200,   37,    4,   13  ], # 81
    [  38,  "LEFT",  205,   38,    4,   13,  205,   38,    4,   13  ], # 82
    [  39,  "LEFT",  206,   49,    4,   13,  206,   39,    4,   13  ], # 83
    [  44,  "LEFT",  208,   44,    5,   14,  208,   44,    5,   14  ], # 84
    [  45,  "LEFT",  209,   45,    5,   14,  209,   45,    5,   14  ], # 85
    [  46,  "LEFT",  214,   46,    5,   14,  214,   46,    5,   14  ], # 86
    [  47,  "LEFT",  215,   47,    5,   14,  215,   47,    5,   14  ], # 87
    [  52,  "LEFT",  217,   52,    6,   15,  217,   52,    6,   15  ], # 88
    [  53,  "LEFT",  218,   53,    6,   15,  218,   53,    6,   15  ], # 89
    [  60,  "LEFT",  229,   60,    7,   16,  229,   60,    7,   16  ], # 90
    [  61,  "LEFT",  230,   61,    7,   16,  230,   61,    7,   16  ], # 91
    [  62,  "LEFT",  235,   62,    7,   16,  235,   62,    7,   16  ], # 92
    [  63,  "LEFT",  236,   63,    7,   16,  236,   63,    7,   16  ], # 93
    [  68,  "LEFT",  161,   68,    8,   17,  161,   68,    8,   17  ], # 94
    [  69,  "LEFT",  162,   69,    8,   17,  162,   69,    8,   17  ], # 95
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [   4, "RIGHT",  122,    4,    0,    9,  122,    4,    0,    9  ], # 96
    [   5, "RIGHT",  123,    5,    0,    9,  123,    5,    0,    9  ], # 97
    [   6, "RIGHT",  128,    6,    0,    9,  128,    6,    0,    9  ], # 98
    [   7, "RIGHT",  129,    7,    0,    9,  129,    7,    0,    9  ], # 99
    [  12, "RIGHT",  131,   12,    1,   10,  131,   12,    1,   10  ], # 100
    [  13, "RIGHT",  132,   13,    1,   10,  132,   13,    1,   10  ], # 101
    [  14, "RIGHT",  140,   14,    1,   10,  140,   14,    1,   10  ], # 102
    [  15, "RIGHT",  141,   15,    1,   10,  141,   15,    1,   10  ], # 103
    [  20, "RIGHT",  143,   20,    2,   11,  143,   20,    2,   11  ], # 104
    [  21, "RIGHT",  144,   21,    2,   11,  144,   21,    2,   11  ], # 105
    [  22, "RIGHT",  149,   22,    2,   11,  149,   22,    2,   11  ], # 106
    [  23, "RIGHT",  150,   23,    2,   11,  150,   23,    2,   11  ], # 107
    [  28, "RIGHT",  152,   28,    3,   12,  152,   28,    3,   12  ], # 108
    [  29, "RIGHT",  153,   29,    3,   12,  153,   29,    3,   12  ], # 109
    [  30, "RIGHT",  158,   30,    3,   12,  158,   30,    3,   12  ], # 110
    [  31, "RIGHT",  159,   31,    3,   12,  159,   31,    3,   12  ], # 111
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [  36, "RIGHT",  165,   36,    4,   13,  165,   36,    4,   13  ], # 112
    [  37, "RIGHT",  166,   37,    4,   13,  166,   37,    4,   13  ], # 113
    [  38, "RIGHT",  170,   38,    4,   13,  170,   38,    4,   13  ], # 114
    [  39, "RIGHT",  171,   39,    4,   13,  171,   39,    4,   13  ], # 115
    [  44, "RIGHT",  174,   44,    5,   14,  174,   44,    5,   14  ], # 116
    [  45, "RIGHT",  175,   45,    5,   14,  175,   45,    5,   14  ], # 117
    [  46, "RIGHT",  178,   46,    5,   14,  178,   46,    5,   14  ], # 118
    [  47, "RIGHT",  179,   47,    5,   14,  179,   47,    5,   14  ], # 119
    [  52, "RIGHT",  170,   52,    6,   15,  170,   52,    6,   15  ], # 120
    [  53, "RIGHT",  171,   53,    6,   15,  171,   53,    6,   15  ], # 121
    [  60, "RIGHT",  134,   60,    7,   16,  134,   60,    7,   16  ], # 122
    [  61, "RIGHT",  135,   61,    7,   16,  135,   61,    7,   16  ], # 123
    [  62, "RIGHT",  142,   62,    7,   16,  142,   62,    7,   16  ], # 124
    [  63, "RIGHT",  144,   63,    7,   16,  144,   63,    7,   16  ], # 125
    [  68, "RIGHT",  161,   68,    8,   17,  161,   68,    8,   17  ], # 126
    [  69, "RIGHT",  162,   69,    8,   17,  162,   69,    8,   17  ], # 127
#############################################################################
#                    XBUS  XBUS  XBUS  XBUS  YBUS  YBUS  YBUS  YBUS         #
#    SHUB    LorR    DIMM  DIMM   x8    x4   DIMM  DIMM   x8    x4     FSB  #
#     BIT    DIMM     PIN   BIT  DRAM  DRAM   PIN   BIT  DRAM  DRAM    BIT  #
#############################################################################
    [  67, "RIGHT",   49,   67,    8,    8,   49,   67,    8,    8  ], # 128
    [  51, "RIGHT",  108,   51,    6,    6,  108,   51,    6,    6  ], # 129
    [  50, "LEFT",   107,   50,    6,    6,  107,   50,    6,    6  ], # 130
    [  66, "LEFT",    48,   66,    8,    8,   48,   66,    8,    8  ], # 131
    [  67, "LEFT",    49,   67,    8,    8,   49,   67,    8,    8  ], # 132
    [  51, "LEFT",   108,   51,    6,    6,  108,   51,    6,    6  ], # 133
    [  50, "RIGHT",  107,   50,    6,    6,  107,   50,    6,    6  ], # 134
    [  66, "RIGHT",   48,   66,    8,    8,   48,   66,    8,    8  ], # 135
    [  71, "RIGHT",  168,   71,    8,   17,  168,   71,    8,   17  ], # 136
    [  55, "RIGHT",  227,   55,    6,   15,  227,   55,    6,   15  ], # 137
    [  54, "LEFT",   226,   54,    6,   15,  226,   54,    6,   15  ], # 138
    [  70, "LEFT",   167,   70,    8,   17,  167,   70,    8,   17  ], # 139
    [  71, "LEFT",   168,   71,    8,   17,  168,   71,    8,   17  ], # 140
    [  55, "LEFT",   227,   55,    6,   15,  227,   55,    6,   15  ], # 141
    [  54, "RIGHT",  226,   54,    6,   15,  226,   54,    6,   15  ], # 142
    [  70, "RIGHT",  167,   70,    8,   17,  167,   70,    8,   17  ], # 143
);  
    
1;

