#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: la_subs.pm,v 1.4 2004/05/07 18:44:14 beighley Exp $


#-------------------------------------------------------------------------------
#
#  FILE: la_subs.pm
#
#  DESCRIPTION:  This file contains PERL subroutines to interact with an     
#     external logic analyzer (Tektronix).
#
#  NOTES: The "common_defs.pm" file is required when this file is used.
#
#  SUBROUTINES:  The following subroutines are contained in this file:
#     (expected parameter usage displayed; some have optional parameters)
#     la_connect($la_host)      
#     la_connect($la_host, $la_port)
#     la_disconnect()
#     la_system_load($sys_fname)
#     la_system_save($sys_fname, $comments)
#     la_system_save($sys_fname, $comments, $LA_SAVE_DATA_YES)
#     la_system_save($sys_fname, $comments, $LA_SAVE_DATA_NO)
#     @groups = la_module_get_group_names($module)
#     la_module_set_channel_group($module, $group, $channels)
#     la_module_set_channel_name($module, $channel, $name)
#     la_system_run()
#     la_system_run($TRUE|$FALSE)
#     la_system_stop()
#     la_system_stop($TRUE|$FALSE)
#     $status = la_system_get_status()
#     $num = la_module_get_num_samples($module)
#     $num = la_module_get_num_samples($module, $dataset)
#     $num = la_module_get_trigger_sample($module)
#     $num = la_module_get_trigger_sample($module, $dataset)
#     @lad = la_module_get_text_data($module, $start, $num)
#     @lad = la_module_get_text_data($module, $start, $num, $glist)
#     la_module_save_data($module, $first_sample, $num, $filename)
#     la_module_save_data($module, $first_sample, $num, $filename, $mode)
#
#     $str  = la_text_data_get_hexstr(\@lad, $sample_index, $group_index)
#     $val  = hexstr_to_value($text)
#     @data = hexstr_to_array($text)
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     11/26/01  Karen Beighley  Original source code documented for use.
#     11/27/01  Karen Beighley  Added wait parameter to *_run() and *_stop().
#     12/03/01  Karen Beighley  Modified "la_module_get_text_data" function 
#                                  to support optional group list parameter.
#     05/21/02  Karen Beighley  Added subroutines to get or set a group name,
#                                  and to set channel name(s).
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES
#-------------------------------------------------------------------------------

$la_init   = $FALSE;
$la_host   = "localhost";
$la_port   = 6543;
$la_module = "LA 1";

@default_la_chnmap = 
  ( "A0(0)", "A0(1)", "A0(2)", "A0(3)", "A0(4)", "A0(5)", "A0(6)", "A0(7)",
    "A1(0)", "A1(1)", "A1(2)", "A1(3)", "A1(4)", "A1(5)", "A1(6)", "A1(7)",
    "A2(0)", "A2(1)", "A2(2)", "A2(3)", "A2(4)", "A2(5)", "A2(6)", "A2(7)",
    "A3(0)", "A3(1)", "A3(2)", "A3(3)", "A3(4)", "A3(5)", "A3(6)", "A3(7)",
    "D0(0)", "D0(1)", "D0(2)", "D0(3)", "D0(4)", "D0(5)", "D0(6)", "D0(7)",
    "D1(0)", "D1(1)", "D1(2)", "D1(3)", "D1(4)", "D1(5)", "D1(6)", "D1(7)",
    "D2(0)", "D2(1)", "D2(2)", "D2(3)", "D2(4)", "D2(5)", "D2(6)", "D2(7)",
    "D3(0)", "D3(1)", "D3(2)", "D3(3)", "D3(4)", "D3(5)", "D3(6)", "D3(7)" );

@la_chnmap = 
  ( "A0(0)", "A0(1)", "A0(2)", "A0(3)", "A0(4)", "A0(5)", "A0(6)", "A0(7)",
    "A1(0)", "A1(1)", "A1(2)", "A1(3)", "A1(4)", "A1(5)", "A1(6)", "A1(7)",
    "A2(0)", "A2(1)", "A2(2)", "A2(3)", "A2(4)", "A2(5)", "A2(6)", "A2(7)",
    "A3(0)", "A3(1)", "A3(2)", "A3(3)", "A3(4)", "A3(5)", "A3(6)", "A3(7)",
    "D0(0)", "D0(1)", "D0(2)", "D0(3)", "D0(4)", "D0(5)", "D0(6)", "D0(7)",
    "D1(0)", "D1(1)", "D1(2)", "D1(3)", "D1(4)", "D1(5)", "D1(6)", "D1(7)",
    "D2(0)", "D2(1)", "D2(2)", "D2(3)", "D2(4)", "D2(5)", "D2(6)", "D2(7)",
    "D3(0)", "D3(1)", "D3(2)", "D3(3)", "D3(4)", "D3(5)", "D3(6)", "D3(7)" );

$LA_CHAN_NAME_LEN = 20;

$LA_SAVE_DATA_NO=0;
$LA_SAVE_DATA_YES=1;

$LA_FILE_APPEND=0;
$LA_FILE_OVERWRITE=1;

$LA_STATUS_RUNNING=0;
$LA_STATUS_IDLE=1;
$LA_STATUS_INVALID=2;

$TLA700_DATASET_MAIN=0;
$TLA700_DATASET_MAGNIVU=1;
$TLA700_DATASET_GLITCH=2;


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_connect()
#
#    USAGE: la_connect($la_host)             
#           la_connect($la_host, $la_port)
#
#    DESCRIPTION: Open a connection to the specified logic analyzer.
#       The port is assumed to be 8001 if not passed as a parameter.
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_connect
{
   my $num = $#_ + 1;
   print "\nla_connect(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   la_disconnect()  if ( $la_init == $TRUE );

   if ( $num == 2 )
   {  $la_port = $_[1];
   }
   elsif ( $num != 1 )
   {  $emess = "Invalid parameters passed to subroutine la_connect().";
      &$comm_err_fun($emess);
      return;
   }
   $la_host = $_[0];

   printf "\nOpening connection to logic analyzer: host=$la_host, port=$la_port\n";

   pcf_la_connect($la_host, $la_port, $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }

   printf "\tSuccessful connection and initialization with logic analyzer.\n";

   $la_init = $TRUE;
   return;
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_disconnect()
#
#    USAGE: la_disconnect()
#
#    DESCRIPTION: Close a connection to the logic analyzer (pre-opened). 
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_disconnect
{
   print "\nla_disconnect(@_)\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $la_init == $TRUE )
   {  pcf_la_disconnect();
      printf "\nClosing connection to logic analyzer: host=$la_host, port=$la_port\n";
   }
   $la_init = $FALSE;
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_default_chnmap()
#
#    USAGE: la_default_chnmap()
#
#    DESCRIPTION: Reset the channel configuration mapping to the default.
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_default_chnmap
{ 
   print "\nla_default_chnmap(@_)\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $aref = ( ($#_ > -1) ? $_[0] : \@$la_chnmap);
   my $i;
   for ($i = 0; $i <= $#default_la_chnmap; $i++)
   {  $$aref[$i] = $default_la_chnmap[$i];
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_system_load()
#
#    USAGE: la_system_load($sys_fname)
#
#    DESCRIPTION: The subroutine loads the LA with the specified system file
#       (Note: this file name must be a complete path to a destination file 
#       on the LA).
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_system_load
{
   my $num = $#_ + 1;
   print "\nla_system_load(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 1 )
   {  $emess = "Invalid parameters passed to subroutine la_system_load().";
      &$comm_err_fun($emess);
      return;
   }

   pcf_la_system_load($_[0], $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_system_save()
#
#    USAGE: la_system_save($sys_fname, $comments)
#           la_system_save($sys_fname, $comments, $LA_SAVE_DATA_YES)
#           la_system_save($sys_fname, $comments, $LA_SAVE_DATA_NO)
#
#    DESCRIPTION: The subroutine saves the TLA system to a file.  The last
#       parameter dictates if the acquisition data should be saved in the file:
#       $LA_SAVE_DATA_YES or $LA_SAVE_DATA_NO. If the last parameter is not 
#       specified, then $LA_SAVE_DATA_YES is used as the default.
#       (Note: The file name must be a complete path to a destination file 
#       on the LA).
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_system_save
{
   my $num = $#_ + 1;
   print "\nla_system_save(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   my $save_mode = $LA_SAVE_DATA_YES;
   if ( $num == 3 )
   {  $save_mode = $_[2];
      if (($save_mode != $LA_SAVE_DATA_NO) && ($save_mode != $LA_SAVE_DATA_YES))
      {  $emess = "Invalid save parameter passed to subroutine la_system_save().";
         &$comm_err_fun($emess);
         return;
      }
   }
   elsif ( $num != 2 )
   {  $emess = "Invalid parameters passed to subroutine la_system_save().";
      &$comm_err_fun($emess);
      return;
   }

   pcf_la_system_save($_[0], $_[1], $save_mode, $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_system_run()
#
#    USAGE: la_system_run()
#           la_system_run($TRUE)
#           la_system_run($FALSE)
#
#    DESCRIPTION: The subroutine starts a data acquisition operation.
#       The LA may not run immediately.  The optional parameter determines
#       if the subroutine should check the LA status and wait (loop) until
#       it becomes "RUNNING" before returning.  The LA status is not checked
#       if no parameter is given or if $FALSE is specified.
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_system_run
{
   my $num = $#_ + 1;
   print "\nla_system_run(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);
   #
   #  Check and extract the subroutine parameters
   #
   my $wait = $FALSE;
   if ( $num == 1 )
   {  $wait = $_[0];
      if ( ($wait != $TRUE) && ($wait != $FALSE) ) 
      {  $emess = "Invalid wait flag passed to subroutine la_system_run().";
         &$comm_err_fun($emess);
         return;
      }
   }
   elsif ( $num != 0 )
   {  $emess = "Invalid parameters passed to subroutine la_system_run().";
      &$comm_err_fun($emess);
      return;
   }

   pcf_la_system_run($ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }

   return if ( $wait == $FALSE );

   #
   #  Wait for the LA status to become RUNNING
   #
   while ( la_system_get_status() != $LA_STATUS_RUNNING )
   {
      printf "WAIT: LA status IDLE (not RUNNING)\n" if ($debug_flag & $DEBUG_GENERAL);
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_system_stop()
#
#    USAGE: la_system_stop()
#           la_system_stop($TRUE)
#           la_system_stop($FALSE)
#
#    DESCRIPTION: The subroutine stops a data acquisition operation.
#       The LA may not stop immediately.  The optional parameter determines
#       if the subroutine should check the LA status and wait (loop) until
#       it becomes "IDLE" before returning.  The LA status is not checked
#       if no parameter is given or if $FALSE is specified.
#
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_system_stop
{
   my $num = $#_ + 1;
   print "\nla_system_stop(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   #
   #  Check and extract the subroutine parameters
   #
   my $wait = $FALSE;
   if ( $num == 1 )
   {  $wait = $_[0];
      if ( ($wait != $TRUE) && ($wait != $FALSE) ) 
      {  $emess = "Invalid wait flag passed to subroutine la_system_stop().";
         &$comm_err_fun($emess);
         return;
      }
   }
   elsif ( $num != 0 )
   {  $emess = "Invalid parameters passed to subroutine la_system_stop().";
      &$comm_err_fun($emess);
      return;
   }

   #
   #  Issue the STOP command to the LA
   #
   pcf_la_system_stop($ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }

   return if ( $wait == $FALSE );

   #
   #  Wait for the LA status to become IDLE
   #
   while ( la_system_get_status() != $LA_STATUS_IDLE )
   {
      printf "WAIT: LA status RUNNING (not IDLE)\n" if ($debug_flag & $DEBUG_GENERAL);
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_system_get_status()
#
#    USAGE: $status = la_system_get_status()
#
#    DESCRIPTION: The subroutine checks the current run status of the LA.
#
#    RETURN VALUE: The current run status of the LA: 
#                  $LA_STATUS_RUNNING or $LA_STATUS_IDLE
#
#-------------------------------------------------------------------------------

sub la_system_get_status
{
   my $num = $#_ + 1;
   print "\nla_system_get_status(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 0 )
   {  $emess = "Invalid parameters passed to subroutine la_system_get_status().";
      &$comm_err_fun($emess);
      return($LA_STATUS_INVALID);
   }

   my $rv = pcf_la_system_get_status($ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return($LA_STATUS_INVALID);
   }
   return( $rv );
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_module_get_num_samples()
#
#    USAGE: $num = la_module_get_num_samples($module)
#           $num = la_module_get_num_samples($module, $dataset)
#
#    DESCRIPTION: The subroutine will get the number of samples in 
#       acquition memory for the specified module.  
#
#       Valid datasets: $TLA_DATASET_MAIN, $TLA_DATASET_MAGNIVU or 
#       $TLA_DATASET_GLITCH.  If no dataset is specified, then
#       $TLA_DATASET_MAIN is used.
#
#    RETURN VALUE: The number of samples in the module's acquisition memory.
#
#-------------------------------------------------------------------------------

sub la_module_get_num_samples
{
   my $num = $#_ + 1;
   print "\nla_module_get_num_samples(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   my $mod_ds = $TLA700_DATASET_MAIN;

   if ( $num == 2 )
   {  $mod_ds = $_[1];
      if ( ($mod_ds != $TLA700_DATASET_MAIN) &&
           ($mod_ds != $TLA700_DATASET_MAGNIVU) &&
           ($mod_ds != $TLA700_DATASET_GLITCH) )
      {  $emess = "Invalid dataset parameter passed to la_module_get_num_samples().";
         &$comm_err_fun($emess);
         return(0);
      }
   }
   elsif ( $num != 1 )
   {  $emess = "Invalid parameters passed to subroutine la_module_get_num_samples().";
      &$comm_err_fun($emess);
      return(0);
   }

   my $rv = pcf_la_module_get_num_samples($_[0], $mod_ds, $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return(0);
   }
   return( $rv );
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_module_get_trigger_sample()
#
#    USAGE: $num = la_module_get_trigger_sample($module)
#           $num = la_module_get_trigger_sample($module, $dataset)
#
#    DESCRIPTION: The subroutine will get the sample number of the
#       trigger sample of the specified module.
#
#       Valid datasets: $TLA_DATASET_MAIN, $TLA_DATASET_MAGNIVU or 
#       $TLA_DATASET_GLITCH.  If no dataset is specified, then
#       $TLA_DATASET_MAIN is used.
#
#    RETURN VALUE: The sample number of the trigger sample.
#
#-------------------------------------------------------------------------------

sub la_module_get_trigger_sample
{
   my $num = $#_ + 1;
   print "\nla_module_get_trigger_sample(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   my $mod_ds = $TLA700_DATASET_MAIN;

   if ( $num == 2 )
   {  $mod_ds = $_[1];
      if ( ($mod_ds != $TLA700_DATASET_MAIN) &&
           ($mod_ds != $TLA700_DATASET_MAGNIVU) &&
           ($mod_ds != $TLA700_DATASET_GLITCH) )
      {  $emess = "Invalid dataset parameter passed to la_module_get_trigger_sample().";
         &$comm_err_fun($emess);
         return(0);
      }
   }
   elsif ( $num != 1 )
   {  $emess = "Invalid parameters passed to subroutine la_module_get_trigger_sample().";
      &$comm_err_fun($emess);
      return(0);
   }

   my $rv = pcf_la_module_get_trigger_sample($_[0], $mod_ds, $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return(0);
   }
   return( $rv );
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_module_get_group_names()
#
#    USAGE: @groups = la_module_get_group_names($module)
#
#    DESCRIPTION: The subroutine will get the names of all groups defined
#       in the setup for the specified module.
#
#    RETURN VALUE: An array containing the module group names.
#
#-------------------------------------------------------------------------------

sub la_module_get_group_names
{
   my $num = $#_ + 1;
   print "\nla_module_get_group_names(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 1 )
   {  $emess = "Invalid parameters passed to subroutine la_module_get_group_names().";
      &$comm_err_fun($emess);
      return;
   }

   my $str = pcf_la_module_get_group_names($_[0], $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }

   my @rv_array = split(",", $str);
   return( @rv_array );
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_module_set_channel_group()
#
#    USAGE: la_module_set_channel_group($module, $group, $channels)
#
#    DESCRIPTION: The subroutine will get set the channel list for a channel
#       group for the specified module.  The channels are specified in a 
#       comma-delimited string (no spaces allowed).
#
#    EXAMPLE:
#       la_module_set_channel_group("LA 2", "mode", "A1(2),A1(3),A1(4)");
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_module_set_channel_group
{
   my $num = $#_ + 1;
   print "\nla_module_set_channel_group(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 3 )
   {  $emess = "Invalid parameters passed to subroutine la_module_set_channel_group().";
      &$comm_err_fun($emess);
      return;
   }

   pcf_la_module_set_channel_group($_[0], $_[1], $_[2], $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_module_set_channel_name()
#
#    USAGE: la_module_set_channel_name($module, $channel, $name)
#
#    DESCRIPTION: The subroutine will get set the user defined name for a
#       channel of the specified module.  One or more channels can be defined
#       in the string using commas (no spaces allowed).  Each channel name
#       must be explicitly provided (no bit ranges supported).
#
#    EXAMPLES:
#       la_module_set_channel_name("LA 1", "A0(0)", "LA_DVLD_L");
#       la_module_set_channel_name("LA 1", "A0(0),A0(1)", "LA_DVLD_L,LA_DVLD_H");
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_module_set_channel_name
{
   my $num = $#_ + 1;
   print "\nla_module_set_channel_name(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 3 )
   {  $emess = "Invalid parameters passed to subroutine la_module_set_channel_name().";
      &$comm_err_fun($emess);
      return;
   }

   my @clist = split(",", $_[1]);
   my @nlist = split(",", $_[2]);

   if ( $#clist != $#nlist )
   {  $emess = "Inconsistent parameters passed to subroutine la_module_set_channel_name().";
      $emess .= "\nThe number of channels does not match the number of names specified.";
      &$comm_err_fun($emess);
      return;
   }

   pcf_la_module_set_channel_name($_[0], $_[1], $_[2], $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_module_get_text_data()
#
#    USAGE: @lad = la_module_get_text_data($module, $start, $num)
#           @lad = la_module_get_text_data($module, $start, $num, $glist)
#
#    DESCRIPTION: The subroutine will read the acquisition data from the
#       specified module (using the first sample and number of samples
#       as defined by the parameters).
#
#       If the group list parameter ($glist) is not specified, then the
#       data for all groups is returned (order matches the module's group
#       defines on the LA).
#
#       If the group list ($glist) is provided, then data is returned for the 
#       those groups specified.  The group list parameter must be a string
#       containing the names seperated with a comma; no additional white
#       space should be included.  The group names much match those given
#       in the LA setup window; use "Timestamp" to get time stamp data.
#
#    RETURN VALUE: Any array containing the acquisition data from the module.
#
#-------------------------------------------------------------------------------

sub la_module_get_text_data
{
   my $num = $#_ + 1;
   print "\nla_module_get_text_data(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   my $glist = "";
   if ( $num == 4 )
   {  $glist = $_[3];
   }
   elsif ( $num != 3 )
   {  $emess = "Invalid parameters passed to subroutine la_module_get_text_data().";
      &$comm_err_fun($emess);
      return;
   }

   my $str = pcf_la_module_get_text_data($_[0], $_[1], $_[2], $glist, $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }

   my @rv_array = split("\n", $str);
   return( @rv_array );
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_module_save_data()
#
#    USAGE: la_module_save_data($module, $first_sample, $num, $filename)
#           la_module_save_data($module, $first_sample, $num, $filename, $mode)
#
#    DESCRIPTION: The subroutine will save the acquisition data of a module
#       to the specified file on the LA (Note: this file name must be a
#       complete path to a destination file on the LA).
#
#       Valid modes: $LA_FILE_APPEND or $LA_FILE_OVERWRITE
#       If no mode is specified, then $LA_FILE_OVERWRITE is used.
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub la_module_save_data
{
   my $num = $#_ + 1;
   print "\nla_module_save_data(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   my $mode = $LA_FILE_OVERWRITE;

   if ( $num == 5 )
   {  $mode = $_[4];
      if ( ($mode != $LA_FILE_APPEND) &&
           ($mode != $LA_FILE_OVERWRITE) )
      {  $emess = "Invalid mode parameter passed to la_module_save_data().";
         &$comm_err_fun($emess);
         return;
      }
   }
   elsif ( $num != 4 )
   {  $emess = "Invalid parameters passed to subroutine la_module_save_data().";
      &$comm_err_fun($emess);
      return;
   }

   pcf_la_module_save_data($_[0], $_[1], $_[2], $_[3], $mode, $ecode, $emess);
   if ( $ecode < 0 )
   {  &$comm_err_fun($emess);
      return;
   }
   return;
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: la_text_data_get_hexstr()
#
#    USAGE: $str = la_text_data_get_hexstr(\@lad, $sample_index, $group_index)
#
#    DESCRIPTION: The subroutine will extract the text entry of a group for a
#       specific sample from the LA text data.
#
#    RETURN VALUE: Text string containing hexidecimal value of the group for 
#    a specific sample from the LA text data (i.e. "FEFFDFFB", "3E07", "1").
#
#-------------------------------------------------------------------------------

sub la_text_data_get_hexstr
{
   my $num = $#_ + 1;
   print "\nla_text_data_get_hexstr(@_) : num=$num\n"  if ($debug_flag & $DEBUG_FUNCTIONS);

   #
   #  Verify the number and type of the command-line parameters
   #
   if ( $num != 3 )
   {  $emess = "Invalid parameters passed to subroutine la_text_data_get_hexstr().";
      &$comm_err_fun($emess);
      return;
   }
   my($aref, $s_ind, $g_ind) = @_;
   if ( ref($aref) ne 'ARRAY' )
   {  $emess = "Invalid parameter passed to subroutine la_text_data_get_hexstr().";
      $emess .= "\nFirst parameter must be ARRAY references.";
      &$comm_err_fun($emess);
      return;
   }
   my @data = @$aref;
   $num = $#data + 1;
   print "number of samples in data array: $num\n" if ( $debug_flag & $DEBUG_GENERAL );
   if ( $s_ind >= $num )
   {  $emess = "Invalid parameters passed to subroutine la_text_data_get_hexstr().";
      $emess .= "\nSample index exceeds number of entries in data array.";
      &$comm_err_fun($emess);
      return;
   }

   #
   #  Split the specific sample into individual groups
   #
   my $sample = $data[$s_ind];
   print "sample = \"$sample\"\n" if ( $debug_flag & $DEBUG_GENERAL );
   my @group = split(",", $sample);
   $num = $#group + 1;
   print "number of group values in sample: $num\n" if ( $debug_flag & $DEBUG_GENERAL );
   if ( $g_ind >= $num )
   {  $emess = "Invalid parameters passed to subroutine la_text_data_get_hexstr().";
      $emess .= "\nGroup index exceeds number of entries in sample data.";
      &$comm_err_fun($emess);
      return;
   }
   print "group entry: $group[$g_ind]\n" if ( $debug_flag & $DEBUG_GENERAL );

   return( $group[$g_ind] );
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: hexstr_to_value()
#
#    USAGE: $val = hexstr_to_value($text)
#
#    DESCRIPTION: The subroutine will convert a hexidecimal string to a
#       scalar value.  If the string has more than 8 characters (represents
#       a value greater than 32-bits), then only the last 8 characters are used.
#
#    RETURN VALUE: The scalar version of the hexidecimal string.
#
#    EXAMPLES:
#       $val = hexstr_to_value("123456");
#       -> value: 0x00123456 = 1193046
#       $val = hexstr_to_value("123456789ABCDEF45678");
#       -> value: 0xDEF45678 = -554412424
#
#-------------------------------------------------------------------------------

sub hexstr_to_value
{
   print "\nhexstr_to_value(@_)\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   if ( $#_ != 0 )
   {  return(0);
   }
   my $hstr = $_[0];
   my $len  = length($hstr);
   return(0) if ( $len == 0 );
   if ( $debug_flag & $DEBUG_GENERAL )
   {  print "hexstring  = \"$hstr\"\n";
      print "length     = $len\n";
   }
   my $vstr = "";
   if ( $len >= 8 )
   {  $len -= 8;
      $vstr = substr($hstr, $len, 8);
   }
   else
   {  $vstr = substr($hstr, 0, $len);
   }

   return(hex($vstr));
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: hexstr_to_array()
#
#    USAGE: @data = hexstr_to_array($text)
#
#    DESCRIPTION: The subroutine will convert a hexidecimal string to an
#       array of scalar values.  The number of entries in the array depends
#       on the number of hex characeters in the string.
#
#    RETURN VALUE: An array of scalar values from the hexidecimal string.
#
#    EXAMPLES:
#       @data = hexstr_to_array("123456")
#       -> data = (0x00123456)
#       @data = hexstr_to_array("123456789ABCDEF45678")
#       -> data = (0xDEF45678 0x56789ABC 0x00001234)
#       @data = hexstr_to_array("55555444444443333333322222222")
#       -> data = (0x22222222 0x33333333 0x44444444 0x00055555)
#
#-------------------------------------------------------------------------------

sub hexstr_to_array
{
   print "\nhexstr_to_array(@_)\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my @adata = ();
   if ( $#_ != 0 )
   {  return(@adata);
   }
   my $hstr = $_[0];
   my $len  = length($hstr);
   if ( $debug_flag & $DEBUG_GENERAL )
   {  print "hexstring = \"$hstr\"\n";
      print "length    = $len\n";
   }
   my $vstr = "";
   my $val  = 0;

   while ( $len >= 8 )
   {  $len -= 8;
      $vstr = substr($hstr, $len, 8);
      $val = hex($vstr);
#      unshift(@adata, $val);
      push(@adata, $val);
   }
   if ( $len > 0 )
   {  $vstr = substr($hstr, 0, $len);
      $val = hex($vstr);
#      unshift(@adata, $val);
      push(@adata, $val);
   }

   return(@adata);
}

#-------------------------------------------------------------------------------

1;
