################################################################################
#   LBI (PROGRAMED FPGA) OPCODE/TDR DEFINITION
################################################################################


$lbi_01_tdr_defined = 0;

#-----------------------------------
#  TDR DEFINITION: BYPASS
#-----------------------------------

%lbi_01_flds_00000 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%lbi_01_tdr_00000 =
(  NAME   => "BYPASS",
   NOTE   => "BYPASS JTAG Register",
   OPNAME => "BYPASS",
   OPCODE => 0x1FF,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00000
);


#-----------------------------------
#  TDR DEFINITION: IDCODE
#-----------------------------------

%lbi_01_flds_00001 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "VERSION",          => [   0,  28,   4, 0x0000, "Version number"         ],
   "PARTNUMBER",       => [   1,  12,  16, 0x0000, "Partnumber"             ],
   "MANUFACTURER",     => [   2,   1,  11, 0x0000, "Manufacturer Code"      ],
   "LSB",              => [   3,   0,   1, 0x0000, "Mandatory LSB"          ]
);

%lbi_01_tdr_00001 =
(  NAME   => "IDCODE",
   NOTE   => "JTAG Device Identification Register",
   OPNAME => "IDCODE",
   OPCODE => 0x01,
   LENGTH => 32,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00001
);


#-----------------------------------
#  TDR DEFINITION: PSIC LOAD_CER
#-----------------------------------

%lbi_01_flds_00003 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "ENABLE",          => [   0,   0,  10, 0x0000, ""                       ]
);

%lbi_01_tdr_00003 =
(  NAME   => "LOAD_CER",
   NOTE   => "SIC chain enable register",
   OPNAME => "LOAD_CER",
   OPCODE => 0x08,
   LENGTH => 10,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00003
);


#-----------------------------------
#  TDR DEFINITION: INIT_MEM
#-----------------------------------

%lbi_01_flds_00006 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA",             => [   0,   0,   2, 0x0000, "Memory initialization/status" ]
);

%lbi_01_tdr_00006 =
(  NAME   => "INIT_MEM",
   NOTE   => "Memory initialization and status",
   OPNAME => "INIT_MEM",
   OPCODE => 0x10,
   LENGTH => 3,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00006
);


#-----------------------------------
#  TDR DEFINITION: WR_MEMADDR
#-----------------------------------

%lbi_01_flds_00007 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "ADDR",             => [   0,   0,  24, 0x0000, "Local memory address"   ]
);

%lbi_01_tdr_00007 =
(  NAME   => "WR_MEMADDR",
   NOTE   => "Address for memory write operation",
   OPNAME => "WR_MEMADDR",
   OPCODE => 0x11,
   LENGTH => 24,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00007
);


#-----------------------------------
#  TDR DEFINITION: RD_MEMADDR
#-----------------------------------

%lbi_01_flds_00008 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "ADDR",             => [   0,   0,  24, 0x0000, "Local memory address"   ]
);

%lbi_01_tdr_00008 =
(  NAME   => "RD_MEMADDR",
   NOTE   => "Address for memory read operation",
   OPNAME => "RD_MEMADDR",
   OPCODE => 0x12,
   LENGTH => 24,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00008
);


#-----------------------------------
#  TDR DEFINITION: WR_MEMDATA
#-----------------------------------

%lbi_01_flds_00009 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA",             => [   0,   0,  16, 0x0000, "Memory access data"   ]
);

%lbi_01_tdr_00009 =
(  NAME   => "WR_MEMDATA",
   NOTE   => "Data for memory write operation",
   OPNAME => "WR_MEMDATA",
   OPCODE => 0x13,
   LENGTH => 16,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00009
);


#-----------------------------------
#  TDR DEFINITION: RD_MEMDATA
#-----------------------------------

%lbi_01_flds_00010 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA",             => [   0,   0,  16, 0x0000, "Memory access data"   ]
);

%lbi_01_tdr_00010 =
(  NAME   => "RD_MEMDATA",
   NOTE   => "Data from memory read operation",
   OPNAME => "RD_MEMDATA",
   OPCODE => 0x14,
   LENGTH => 16,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_00010
);


#-----------------------------------
#  TDR DEFINITION: WR_MEMERR
#-----------------------------------

%lbi_01_flds_00011 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DISABLE"           => [   0,  56,   1, 0x0000, ""                       ],
   "COR_ERR"           => [   1,  55,   1, 0x0000, ""                       ],
   "UNC_ERR"           => [   2,  54,   1, 0x0000, ""                       ],
   "OVERRUN"           => [   3,  53,   1, 0x0000, ""                       ],
   "ADDR"              => [   4,  29,  24, 0x0000, ""                       ],
   "DATA"              => [   5,  13,  16, 0x0000, ""                       ],
   "ECC"               => [   6,   1,  12, 0x0000, ""                       ],
   "FORCE"             => [   7,   0,   1, 0x0000, ""                       ]
);

%lbi_01_tdr_00011 =
(  NAME   => "WR_MEMERR",
   NOTE   => "Memory error information",
   OPNAME => "WR_MEMERR",
   OPCODE => 0x15,
   LENGTH => 57,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%lbi_01_flds_00011
);


#-----------------------------------
#  TDR DEFINITION: RD_MEMERR
#-----------------------------------

%lbi_01_flds_00012 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DISABLE"           => [   0,  56,   1, 0x0000, ""                       ],
   "COR_ERR"           => [   1,  55,   1, 0x0000, ""                       ],
   "UNC_ERR"           => [   2,  54,   1, 0x0000, ""                       ],
   "OVERRUN"           => [   3,  53,   1, 0x0000, ""                       ],
   "ADDR"              => [   4,  29,  24, 0x0000, ""                       ],
   "DATA"              => [   5,  13,  16, 0x0000, ""                       ],
   "ECC"               => [   6,   1,  12, 0x0000, ""                       ],
   "FORCE"             => [   7,   0,   1, 0x0000, ""                       ]
);

%lbi_01_tdr_00012 =
(  NAME   => "RD_MEMERR",
   NOTE   => "Memory error information",
   OPNAME => "RD_MEMERR",
   OPCODE => 0x16,
   LENGTH => 57,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%lbi_01_flds_00012
);


#-----------------------------------
#  TDR DEFINITION: RD_MEMUPD_0
#                  RD_MEMUPD_1
#-----------------------------------

%lbi_01_flds_memupd =
#  NAME                     IND  POS    LEN  MODE    DESCRIPTION
#  -------------------      ---  ---    ---  ------  -----------------------
(
   "0x01F000"           => [    0, 7936,  256, 0x0000, ""            ],
   "0x01E000"           => [    1, 7680,  256, 0x0000, ""            ],
   "0x01D000"           => [    2, 7424,  256, 0x0000, ""            ],
   "0x01C000"           => [    3, 7168,  256, 0x0000, ""            ],
   "0x01B000"           => [    4, 6912,  256, 0x0000, ""            ],
   "0x01A000"           => [    5, 6656,  256, 0x0000, ""            ],
   "0x019000"           => [    6, 6400,  256, 0x0000, ""            ],
   "0x018000"           => [    7, 6144,  256, 0x0000, ""            ],
   "0x017000"           => [    8, 5888,  256, 0x0000, ""            ],
   "0x016000"           => [    9, 5632,  256, 0x0000, ""            ],
   "0x015000"           => [   10, 5376,  256, 0x0000, ""            ],
   "0x014000"           => [   11, 5120,  256, 0x0000, ""            ],
   "0x013000"           => [   12, 4864,  256, 0x0000, ""            ],
   "0x012000"           => [   13, 4608,  256, 0x0000, ""            ],
   "0x011000"           => [   14, 4352,  256, 0x0000, ""            ],
   "0x010000"           => [   15, 4096,  256, 0x0000, ""            ],
   "0x00F000"           => [   16, 3840,  256, 0x0000, ""            ],
   "0x00E000"           => [   17, 3584,  256, 0x0000, ""            ],
   "0x00D000"           => [   18, 3328,  256, 0x0000, ""            ],
   "0x00C000"           => [   19, 3072,  256, 0x0000, ""            ],
   "0x00B000"           => [   20, 2816,  256, 0x0000, ""            ],
   "0x00A000"           => [   21, 2560,  256, 0x0000, ""            ],
   "0x009000"           => [   22, 2304,  256, 0x0000, ""            ],
   "0x008000"           => [   23, 2048,  256, 0x0000, ""            ],
   "0x007000"           => [   24, 1792,  256, 0x0000, ""            ],
   "0x006000"           => [   25, 1536,  256, 0x0000, ""            ],
   "0x005000"           => [   26, 1280,  256, 0x0000, ""            ],
   "0x004000"           => [   27, 1024,  256, 0x0000, ""            ],
   "0x003000"           => [   28,  768,  256, 0x0000, ""            ],
   "0x002000"           => [   29,  512,  256, 0x0000, ""            ],
   "0x001000"           => [   30,  256,  256, 0x0000, ""            ],
   "0x000000"           => [   31,    0,  256, 0x0000, ""            ]
);


%lbi_01_tdr_00094 =
(  NAME   => "RD_MEMUPD_0",
   NOTE   => "Read private memory update status",
   OPNAME => "RD_MEMUPD_0",
   OPCODE => 0x17,
   LENGTH => 8192,
   RESET  => [ 0x00000000 ],    # FIXME, need 8192 bit field
   FIELDS => \%lbi_01_flds_memupd
);


%lbi_01_tdr_00095 =
(  NAME   => "RD_MEMUPD_1",
   NOTE   => "Read private memory update status",
   OPNAME => "RD_MEMUPD_1",
   OPCODE => 0x18,
   LENGTH => 8192,
   RESET  => [ 0x00000000 ],    # FIXME, need 8192 bit field
   FIELDS => \%lbi_01_flds_memupd
);


#-----------------------------------
#  TDR DEFINITION: RD_LEDSTAT
#-----------------------------------

%lbi_01_flds_00013 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "LED_13_OVR",       => [   0,  79,   1, 0x0000, ""                     ],
   "LED_13_UPD",       => [   1,  78,   1, 0x0000, ""                     ],
   "LED_12_OVR",       => [   2,  77,   1, 0x0000, ""                     ],
   "LED_12_UPD",       => [   3,  76,   1, 0x0000, ""                     ],
   "LED_11_OVR",       => [   4,  75,   1, 0x0000, ""                     ],
   "LED_11_UPD",       => [   5,  74,   1, 0x0000, ""                     ],
   "LED_10_OVR",       => [   6,  73,   1, 0x0000, ""                     ],
   "LED_10_UPD",       => [   7,  72,   1, 0x0000, ""                     ],
   "LED_03_OVR",       => [   8,  71,   1, 0x0000, ""                     ],
   "LED_03_UPD",       => [   9,  70,   1, 0x0000, ""                     ],
   "LED_02_OVR",       => [  10,  69,   1, 0x0000, ""                     ],
   "LED_02_UPD",       => [  11,  68,   1, 0x0000, ""                     ],
   "LED_01_OVR",       => [  12,  67,   1, 0x0000, ""                     ],
   "LED_01_UPD",       => [  13,  66,   1, 0x0000, ""                     ],
   "LED_00_OVR",       => [  14,  65,   1, 0x0000, ""                     ],
   "LED_00_UPD",       => [  15,  64,   1, 0x0000, ""                     ],
   "LED_13",           => [  16,  56,   8, 0x0000, ""                     ],
   "LED_12",           => [  17,  48,   8, 0x0000, ""                     ],
   "LED_11",           => [  18,  40,   8, 0x0000, ""                     ],
   "LED_10",           => [  19,  32,   8, 0x0000, ""                     ],
   "LED_03",           => [  20,  24,   8, 0x0000, ""                     ],
   "LED_02",           => [  21,  16,   8, 0x0000, ""                     ],
   "LED_01",           => [  22,   8,   8, 0x0000, ""                     ],
   "LED_00",           => [  23,   0,   8, 0x0000, ""                     ]
);

%lbi_01_tdr_00013 =
(  NAME   => "RD_LEDSTAT",
   NOTE   => "Read LED registers",
   OPNAME => "RD_LEDSTAT",
   OPCODE => 0x19,
   LENGTH => 80,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%lbi_01_flds_00013
);


#-----------------------------------
#  SHARED TDR FIELD DEFINITIONS FOR
#     WR_IODATA_Px
#     RD_IODATA_Px
#     WR_IOIMASK_Px
#     RD_IOIMASK_Px
#     RD_IOISTAT_Px
#-----------------------------------

%lbi_01_flds_iodata =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,   8, 0x0000, "I/O pin data"        ],
);

%lbi_01_flds_ioimask =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "IMASK"             => [   0,   0,   8, 0x0000, "Interrupt Mask"    ],
);

%lbi_01_flds_ioistat =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "ISTAT"             => [   0,   0,   8, 0x0000, "Interrupt Status"  ],
);

#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P0
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00014 =
(  NAME   => "WR_IODATA_P0",
   NOTE   => "Write the Port 0 I/O pin values",
   OPNAME => "WR_IODATA_P0",
   OPCODE => 0x09,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P0
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00015 =
(  NAME   => "RD_IODATA_P0",
   NOTE   => "Read the Port 0 I/O pin values",
   OPNAME => "RD_IODATA_P0",
   OPCODE => 0x0A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P0
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00016 =
(  NAME   => "WR_IOIMASK_P0",
   NOTE   => "Write the Port 0 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P0",
   OPCODE => 0x0B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P0
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00017 =
(  NAME   => "RD_IOIMASK_P0",
   NOTE   => "Read the Port 0 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P0",
   OPCODE => 0x0C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P0
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00018 =
(  NAME   => "RD_IOISTAT_P0",
   NOTE   => "Read the Port 0 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P0",
   OPCODE => 0x0D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P1
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00019 =
(  NAME   => "WR_IODATA_P1",
   NOTE   => "Write the Port 1 I/O pin values",
   OPNAME => "WR_IODATA_P1",
   OPCODE => 0x29,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P1
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00020 =
(  NAME   => "RD_IODATA_P1",
   NOTE   => "Read the Port 1 I/O pin values",
   OPNAME => "RD_IODATA_P1",
   OPCODE => 0x2A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P1
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00021 =
(  NAME   => "WR_IOIMASK_P1",
   NOTE   => "Write the Port 1 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P1",
   OPCODE => 0x2B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P1
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00022 =
(  NAME   => "RD_IOIMASK_P1",
   NOTE   => "Read the Port 1 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P1",
   OPCODE => 0x2C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P1
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00023 =
(  NAME   => "RD_IOISTAT_P1",
   NOTE   => "Read the Port 1 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P1",
   OPCODE => 0x2D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P2
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00024 =
(  NAME   => "WR_IODATA_P2",
   NOTE   => "Write the Port 2 I/O pin values",
   OPNAME => "WR_IODATA_P2",
   OPCODE => 0x49,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P2
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00025 =
(  NAME   => "RD_IODATA_P2",
   NOTE   => "Read the Port 2 I/O pin values",
   OPNAME => "RD_IODATA_P2",
   OPCODE => 0x4A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P2
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00026 =
(  NAME   => "WR_IOIMASK_P2",
   NOTE   => "Write the Port 2 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P2",
   OPCODE => 0x4B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P2
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00027 =
(  NAME   => "RD_IOIMASK_P2",
   NOTE   => "Read the Port 2 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P2",
   OPCODE => 0x4C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P2
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00028 =
(  NAME   => "RD_IOISTAT_P2",
   NOTE   => "Read the Port 2 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P2",
   OPCODE => 0x4D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P3
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00029 =
(  NAME   => "WR_IODATA_P3",
   NOTE   => "Write the Port 3 I/O pin values",
   OPNAME => "WR_IODATA_P3",
   OPCODE => 0x69,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P3
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00030 =
(  NAME   => "RD_IODATA_P3",
   NOTE   => "Read the Port 3 I/O pin values",
   OPNAME => "RD_IODATA_P3",
   OPCODE => 0x6A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P3
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00031 =
(  NAME   => "WR_IOIMASK_P3",
   NOTE   => "Write the Port 3 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P3",
   OPCODE => 0x6B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P3
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00032 =
(  NAME   => "RD_IOIMASK_P3",
   NOTE   => "Read the Port 3 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P3",
   OPCODE => 0x6C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P3
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00033 =
(  NAME   => "RD_IOISTAT_P3",
   NOTE   => "Read the Port 3 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P3",
   OPCODE => 0x6D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P4
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00034 =
(  NAME   => "WR_IODATA_P4",
   NOTE   => "Write the Port 4 I/O pin values",
   OPNAME => "WR_IODATA_P4",
   OPCODE => 0x89,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P4
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00035 =
(  NAME   => "RD_IODATA_P4",
   NOTE   => "Read the Port 4 I/O pin values",
   OPNAME => "RD_IODATA_P4",
   OPCODE => 0x8A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P4
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00036 =
(  NAME   => "WR_IOIMASK_P4",
   NOTE   => "Write the Port 4 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P4",
   OPCODE => 0x8B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P4
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00037 =
(  NAME   => "RD_IOIMASK_P4",
   NOTE   => "Read the Port 4 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P4",
   OPCODE => 0x8C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P4
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00038 =
(  NAME   => "RD_IOISTAT_P4",
   NOTE   => "Read the Port 4 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P4",
   OPCODE => 0x8D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P5
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00039 =
(  NAME   => "WR_IODATA_P5",
   NOTE   => "Write the Port 5 I/O pin values",
   OPNAME => "WR_IODATA_P5",
   OPCODE => 0xA9,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P5
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00040 =
(  NAME   => "RD_IODATA_P5",
   NOTE   => "Read the Port 5 I/O pin values",
   OPNAME => "RD_IODATA_P5",
   OPCODE => 0xAA,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P5
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00041 =
(  NAME   => "WR_IOIMASK_P5",
   NOTE   => "Write the Port 5 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P5",
   OPCODE => 0xAB,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P5
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00042 =
(  NAME   => "RD_IOIMASK_P5",
   NOTE   => "Read the Port 5 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P5",
   OPCODE => 0xAC,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P5
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00043 =
(  NAME   => "RD_IOISTAT_P5",
   NOTE   => "Read the Port 5 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P5",
   OPCODE => 0xAD,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P6
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00044 =
(  NAME   => "WR_IODATA_P6",
   NOTE   => "Write the Port 6 I/O pin values",
   OPNAME => "WR_IODATA_P6",
   OPCODE => 0xC9,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P6
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00045 =
(  NAME   => "RD_IODATA_P6",
   NOTE   => "Read the Port 6 I/O pin values",
   OPNAME => "RD_IODATA_P6",
   OPCODE => 0xCA,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P6
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00046 =
(  NAME   => "WR_IOIMASK_P6",
   NOTE   => "Write the Port 6 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P6",
   OPCODE => 0xCB,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P6
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00047 =
(  NAME   => "RD_IOIMASK_P6",
   NOTE   => "Read the Port 6 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P6",
   OPCODE => 0xCC,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P6
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00048 =
(  NAME   => "RD_IOISTAT_P6",
   NOTE   => "Read the Port 6 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P6",
   OPCODE => 0xCD,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P7
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00049 =
(  NAME   => "WR_IODATA_P7",
   NOTE   => "Write the Port 7 I/O pin values",
   OPNAME => "WR_IODATA_P7",
   OPCODE => 0xE9,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P7
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00050 =
(  NAME   => "RD_IODATA_P7",
   NOTE   => "Read the Port 7 I/O pin values",
   OPNAME => "RD_IODATA_P7",
   OPCODE => 0xEA,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P7
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00051 =
(  NAME   => "WR_IOIMASK_P7",
   NOTE   => "Write the Port 7 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P7",
   OPCODE => 0xEB,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P7
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00052 =
(  NAME   => "RD_IOIMASK_P7",
   NOTE   => "Read the Port 7 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P7",
   OPCODE => 0xEC,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P7
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00053 =
(  NAME   => "RD_IOISTAT_P7",
   NOTE   => "Read the Port 7 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P7",
   OPCODE => 0xED,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P8
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00054 =
(  NAME   => "WR_IODATA_P8",
   NOTE   => "Write the Port 8 I/O pin values",
   OPNAME => "WR_IODATA_P8",
   OPCODE => 0x109,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P8
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00055 =
(  NAME   => "RD_IODATA_P8",
   NOTE   => "Read the Port 8 I/O pin values",
   OPNAME => "RD_IODATA_P8",
   OPCODE => 0x10A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P8
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00056 =
(  NAME   => "WR_IOIMASK_P8",
   NOTE   => "Write the Port 8 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P8",
   OPCODE => 0x10B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P8
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00057 =
(  NAME   => "RD_IOIMASK_P8",
   NOTE   => "Read the Port 8 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P8",
   OPCODE => 0x10C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P8
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00058 =
(  NAME   => "RD_IOISTAT_P8",
   NOTE   => "Read the Port 8 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P8",
   OPCODE => 0x10D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P9
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00059 =
(  NAME   => "WR_IODATA_P9",
   NOTE   => "Write the Port 9 I/O pin values",
   OPNAME => "WR_IODATA_P9",
   OPCODE => 0x129,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P9
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00060 =
(  NAME   => "RD_IODATA_P9",
   NOTE   => "Read the Port 9 I/O pin values",
   OPNAME => "RD_IODATA_P9",
   OPCODE => 0x12A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P9
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00061 =
(  NAME   => "WR_IOIMASK_P9",
   NOTE   => "Write the Port 9 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P9",
   OPCODE => 0x12B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P9
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00062 =
(  NAME   => "RD_IOIMASK_P9",
   NOTE   => "Read the Port 9 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P9",
   OPCODE => 0x12C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P9
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00063 =
(  NAME   => "RD_IOISTAT_P9",
   NOTE   => "Read the Port 9 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P9",
   OPCODE => 0x12D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P10
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00064 =
(  NAME   => "WR_IODATA_P10",
   NOTE   => "Write the Port 10 I/O pin values",
   OPNAME => "WR_IODATA_P10",
   OPCODE => 0x149,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P10
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00065 =
(  NAME   => "RD_IODATA_P10",
   NOTE   => "Read the Port 10 I/O pin values",
   OPNAME => "RD_IODATA_P10",
   OPCODE => 0x14A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P10
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00066 =
(  NAME   => "WR_IOIMASK_P10",
   NOTE   => "Write the Port 10 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P10",
   OPCODE => 0x14B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P10
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00067 =
(  NAME   => "RD_IOIMASK_P10",
   NOTE   => "Read the Port 10 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P10",
   OPCODE => 0x14C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P10
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00068 =
(  NAME   => "RD_IOISTAT_P10",
   NOTE   => "Read the Port 10 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P10",
   OPCODE => 0x14D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P11
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00069 =
(  NAME   => "WR_IODATA_P11",
   NOTE   => "Write the Port 11 I/O pin values",
   OPNAME => "WR_IODATA_P11",
   OPCODE => 0x169,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P11
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00070 =
(  NAME   => "RD_IODATA_P11",
   NOTE   => "Read the Port 11 I/O pin values",
   OPNAME => "RD_IODATA_P11",
   OPCODE => 0x16A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P11
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00071 =
(  NAME   => "WR_IOIMASK_P11",
   NOTE   => "Write the Port 11 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P11",
   OPCODE => 0x16B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P11
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00072 =
(  NAME   => "RD_IOIMASK_P11",
   NOTE   => "Read the Port 11 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P11",
   OPCODE => 0x16C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P11
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00073 =
(  NAME   => "RD_IOISTAT_P11",
   NOTE   => "Read the Port 11 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P11",
   OPCODE => 0x16D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P12
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00074 =
(  NAME   => "WR_IODATA_P12",
   NOTE   => "Write the Port 12 I/O pin values",
   OPNAME => "WR_IODATA_P12",
   OPCODE => 0x189,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P12
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00075 =
(  NAME   => "RD_IODATA_P12",
   NOTE   => "Read the Port 12 I/O pin values",
   OPNAME => "RD_IODATA_P12",
   OPCODE => 0x18A,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P12
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00076 =
(  NAME   => "WR_IOIMASK_P12",
   NOTE   => "Write the Port 12 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P12",
   OPCODE => 0x18B,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P12
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00077 =
(  NAME   => "RD_IOIMASK_P12",
   NOTE   => "Read the Port 12 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P12",
   OPCODE => 0x18C,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P12
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00078 =
(  NAME   => "RD_IOISTAT_P12",
   NOTE   => "Read the Port 12 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P12",
   OPCODE => 0x18D,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P13
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00079 =
(  NAME   => "WR_IODATA_P13",
   NOTE   => "Write the Port 13 I/O pin values",
   OPNAME => "WR_IODATA_P13",
   OPCODE => 0x1A9,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P13
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00080 =
(  NAME   => "RD_IODATA_P13",
   NOTE   => "Read the Port 13 I/O pin values",
   OPNAME => "RD_IODATA_P13",
   OPCODE => 0x1AA,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P13
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00081 =
(  NAME   => "WR_IOIMASK_P13",
   NOTE   => "Write the Port 13 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P13",
   OPCODE => 0x1AB,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P13
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00082 =
(  NAME   => "RD_IOIMASK_P13",
   NOTE   => "Read the Port 13 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P13",
   OPCODE => 0x1AC,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P13
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00083 =
(  NAME   => "RD_IOISTAT_P13",
   NOTE   => "Read the Port 13 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P13",
   OPCODE => 0x1AD,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P14
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00084 =
(  NAME   => "WR_IODATA_P14",
   NOTE   => "Write the Port 14 I/O pin values",
   OPNAME => "WR_IODATA_P14",
   OPCODE => 0x1C9,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P14
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00085 =
(  NAME   => "RD_IODATA_P14",
   NOTE   => "Read the Port 14 I/O pin values",
   OPNAME => "RD_IODATA_P14",
   OPCODE => 0x1CA,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P14
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00086 =
(  NAME   => "WR_IOIMASK_P14",
   NOTE   => "Write the Port 14 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P14",
   OPCODE => 0x1CB,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P14
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00087 =
(  NAME   => "RD_IOIMASK_P14",
   NOTE   => "Read the Port 14 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P14",
   OPCODE => 0x1CC,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P14
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00088 =
(  NAME   => "RD_IOISTAT_P14",
   NOTE   => "Read the Port 14 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P14",
   OPCODE => 0x1CD,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IODATA_P15
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00089 =
(  NAME   => "WR_IODATA_P15",
   NOTE   => "Write the Port 15 I/O pin values",
   OPNAME => "WR_IODATA_P15",
   OPCODE => 0x1E9,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IODATA_P15
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00090 =
(  NAME   => "RD_IODATA_P15",
   NOTE   => "Read the Port 15 I/O pin values",
   OPNAME => "RD_IODATA_P15",
   OPCODE => 0x1EA,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_iodata
);


#-----------------------------------
#  TDR DEFINITION: PSIC WR_IOIMASK_P15
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00091 =
(  NAME   => "WR_IOIMASK_P15",
   NOTE   => "Write the Port 15 I/O interrupt mask value",
   OPNAME => "WR_IOIMASK_P15",
   OPCODE => 0x1EB,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOIMASK_P15
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00092 =
(  NAME   => "RD_IOIMASK_P15",
   NOTE   => "Read the Port 15 I/O interrupt mask value",
   OPNAME => "RD_IOIMASK_P15",
   OPCODE => 0x1EC,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioimask
);


#-----------------------------------
#  TDR DEFINITION: PSIC RD_IOISTAT_P15
#  (uses shared field definitions from above)
#-----------------------------------

%lbi_01_tdr_00093 =
(  NAME   => "RD_IOISTAT_P15",
   NOTE   => "Read the Port 15 I/O interrupt status",
   OPNAME => "RD_IOISTAT_P15",
   OPCODE => 0x1ED,
   LENGTH => 8,
   RESET  => [ 0x00000000 ],
   FIELDS => \%lbi_01_flds_ioistat
);



#-----------------------------------
#  HASH TABLE: LOOKUP TDR BY OPNAME
#-----------------------------------

%lbi_01_tdrs =
(  $lbi_01_tdr_00000{OPNAME} => \%lbi_01_tdr_00000,
   $lbi_01_tdr_00001{OPNAME} => \%lbi_01_tdr_00001,
   $lbi_01_tdr_00003{OPNAME} => \%lbi_01_tdr_00003,
   $lbi_01_tdr_00006{OPNAME} => \%lbi_01_tdr_00006,
   $lbi_01_tdr_00007{OPNAME} => \%lbi_01_tdr_00007,
   $lbi_01_tdr_00008{OPNAME} => \%lbi_01_tdr_00008,
   $lbi_01_tdr_00009{OPNAME} => \%lbi_01_tdr_00009,
   $lbi_01_tdr_00010{OPNAME} => \%lbi_01_tdr_00010,
   $lbi_01_tdr_00011{OPNAME} => \%lbi_01_tdr_00011,
   $lbi_01_tdr_00012{OPNAME} => \%lbi_01_tdr_00012,
   $lbi_01_tdr_00013{OPNAME} => \%lbi_01_tdr_00013,
   $lbi_01_tdr_00014{OPNAME} => \%lbi_01_tdr_00014,
   $lbi_01_tdr_00015{OPNAME} => \%lbi_01_tdr_00015,
   $lbi_01_tdr_00016{OPNAME} => \%lbi_01_tdr_00016,
   $lbi_01_tdr_00017{OPNAME} => \%lbi_01_tdr_00017,
   $lbi_01_tdr_00018{OPNAME} => \%lbi_01_tdr_00018,
   $lbi_01_tdr_00019{OPNAME} => \%lbi_01_tdr_00019,
   $lbi_01_tdr_00020{OPNAME} => \%lbi_01_tdr_00020,
   $lbi_01_tdr_00021{OPNAME} => \%lbi_01_tdr_00021,
   $lbi_01_tdr_00022{OPNAME} => \%lbi_01_tdr_00022,
   $lbi_01_tdr_00023{OPNAME} => \%lbi_01_tdr_00023,
   $lbi_01_tdr_00024{OPNAME} => \%lbi_01_tdr_00024,
   $lbi_01_tdr_00025{OPNAME} => \%lbi_01_tdr_00025,
   $lbi_01_tdr_00026{OPNAME} => \%lbi_01_tdr_00026,
   $lbi_01_tdr_00027{OPNAME} => \%lbi_01_tdr_00027,
   $lbi_01_tdr_00028{OPNAME} => \%lbi_01_tdr_00028,
   $lbi_01_tdr_00029{OPNAME} => \%lbi_01_tdr_00029,
   $lbi_01_tdr_00030{OPNAME} => \%lbi_01_tdr_00030,
   $lbi_01_tdr_00031{OPNAME} => \%lbi_01_tdr_00031,
   $lbi_01_tdr_00032{OPNAME} => \%lbi_01_tdr_00032,
   $lbi_01_tdr_00033{OPNAME} => \%lbi_01_tdr_00033,
   $lbi_01_tdr_00034{OPNAME} => \%lbi_01_tdr_00034,
   $lbi_01_tdr_00035{OPNAME} => \%lbi_01_tdr_00035,
   $lbi_01_tdr_00036{OPNAME} => \%lbi_01_tdr_00036,
   $lbi_01_tdr_00037{OPNAME} => \%lbi_01_tdr_00037,
   $lbi_01_tdr_00038{OPNAME} => \%lbi_01_tdr_00038,
   $lbi_01_tdr_00039{OPNAME} => \%lbi_01_tdr_00039,
   $lbi_01_tdr_00040{OPNAME} => \%lbi_01_tdr_00040,
   $lbi_01_tdr_00041{OPNAME} => \%lbi_01_tdr_00041,
   $lbi_01_tdr_00042{OPNAME} => \%lbi_01_tdr_00042,
   $lbi_01_tdr_00043{OPNAME} => \%lbi_01_tdr_00043,
   $lbi_01_tdr_00044{OPNAME} => \%lbi_01_tdr_00044,
   $lbi_01_tdr_00045{OPNAME} => \%lbi_01_tdr_00045,
   $lbi_01_tdr_00046{OPNAME} => \%lbi_01_tdr_00046,
   $lbi_01_tdr_00047{OPNAME} => \%lbi_01_tdr_00047,
   $lbi_01_tdr_00048{OPNAME} => \%lbi_01_tdr_00048,
   $lbi_01_tdr_00049{OPNAME} => \%lbi_01_tdr_00049,
   $lbi_01_tdr_00050{OPNAME} => \%lbi_01_tdr_00050,
   $lbi_01_tdr_00051{OPNAME} => \%lbi_01_tdr_00051,
   $lbi_01_tdr_00052{OPNAME} => \%lbi_01_tdr_00052,
   $lbi_01_tdr_00053{OPNAME} => \%lbi_01_tdr_00053,
   $lbi_01_tdr_00054{OPNAME} => \%lbi_01_tdr_00054,
   $lbi_01_tdr_00055{OPNAME} => \%lbi_01_tdr_00055,
   $lbi_01_tdr_00056{OPNAME} => \%lbi_01_tdr_00056,
   $lbi_01_tdr_00057{OPNAME} => \%lbi_01_tdr_00057,
   $lbi_01_tdr_00058{OPNAME} => \%lbi_01_tdr_00058,
   $lbi_01_tdr_00059{OPNAME} => \%lbi_01_tdr_00059,
   $lbi_01_tdr_00060{OPNAME} => \%lbi_01_tdr_00060,
   $lbi_01_tdr_00061{OPNAME} => \%lbi_01_tdr_00061,
   $lbi_01_tdr_00062{OPNAME} => \%lbi_01_tdr_00062,
   $lbi_01_tdr_00063{OPNAME} => \%lbi_01_tdr_00063,
   $lbi_01_tdr_00064{OPNAME} => \%lbi_01_tdr_00064,
   $lbi_01_tdr_00065{OPNAME} => \%lbi_01_tdr_00065,
   $lbi_01_tdr_00066{OPNAME} => \%lbi_01_tdr_00066,
   $lbi_01_tdr_00067{OPNAME} => \%lbi_01_tdr_00067,
   $lbi_01_tdr_00068{OPNAME} => \%lbi_01_tdr_00068,
   $lbi_01_tdr_00069{OPNAME} => \%lbi_01_tdr_00069,
   $lbi_01_tdr_00070{OPNAME} => \%lbi_01_tdr_00070,
   $lbi_01_tdr_00071{OPNAME} => \%lbi_01_tdr_00071,
   $lbi_01_tdr_00072{OPNAME} => \%lbi_01_tdr_00072,
   $lbi_01_tdr_00073{OPNAME} => \%lbi_01_tdr_00073,
   $lbi_01_tdr_00074{OPNAME} => \%lbi_01_tdr_00074,
   $lbi_01_tdr_00075{OPNAME} => \%lbi_01_tdr_00075,
   $lbi_01_tdr_00076{OPNAME} => \%lbi_01_tdr_00076,
   $lbi_01_tdr_00077{OPNAME} => \%lbi_01_tdr_00077,
   $lbi_01_tdr_00078{OPNAME} => \%lbi_01_tdr_00078,
   $lbi_01_tdr_00079{OPNAME} => \%lbi_01_tdr_00079,
   $lbi_01_tdr_00080{OPNAME} => \%lbi_01_tdr_00080,
   $lbi_01_tdr_00081{OPNAME} => \%lbi_01_tdr_00081,
   $lbi_01_tdr_00082{OPNAME} => \%lbi_01_tdr_00082,
   $lbi_01_tdr_00083{OPNAME} => \%lbi_01_tdr_00083,
   $lbi_01_tdr_00084{OPNAME} => \%lbi_01_tdr_00084,
   $lbi_01_tdr_00085{OPNAME} => \%lbi_01_tdr_00085,
   $lbi_01_tdr_00086{OPNAME} => \%lbi_01_tdr_00086,
   $lbi_01_tdr_00087{OPNAME} => \%lbi_01_tdr_00087,
   $lbi_01_tdr_00088{OPNAME} => \%lbi_01_tdr_00088,
   $lbi_01_tdr_00089{OPNAME} => \%lbi_01_tdr_00089,
   $lbi_01_tdr_00090{OPNAME} => \%lbi_01_tdr_00090,
   $lbi_01_tdr_00091{OPNAME} => \%lbi_01_tdr_00091,
   $lbi_01_tdr_00092{OPNAME} => \%lbi_01_tdr_00092,
   $lbi_01_tdr_00093{OPNAME} => \%lbi_01_tdr_00093,
   $lbi_01_tdr_00094{OPNAME} => \%lbi_01_tdr_00094,
   $lbi_01_tdr_00095{OPNAME} => \%lbi_01_tdr_00095
);


#-----------------------------------
#  ARRAY: STORE OPNAMES IN "ORDER"
#-----------------------------------

@lbi_01_opnames =
(  $lbi_01_tdr_00000{OPNAME},
   $lbi_01_tdr_00001{OPNAME},
   $lbi_01_tdr_00003{OPNAME},
   $lbi_01_tdr_00006{OPNAME},
   $lbi_01_tdr_00007{OPNAME},
   $lbi_01_tdr_00008{OPNAME},
   $lbi_01_tdr_00009{OPNAME},
   $lbi_01_tdr_00010{OPNAME},
   $lbi_01_tdr_00011{OPNAME},
   $lbi_01_tdr_00012{OPNAME},
   $lbi_01_tdr_00013{OPNAME},
   $lbi_01_tdr_00014{OPNAME},
   $lbi_01_tdr_00094{OPNAME},
   $lbi_01_tdr_00095{OPNAME},
   $lbi_01_tdr_00015{OPNAME},
   $lbi_01_tdr_00016{OPNAME},
   $lbi_01_tdr_00017{OPNAME},
   $lbi_01_tdr_00018{OPNAME},
   $lbi_01_tdr_00019{OPNAME},
   $lbi_01_tdr_00020{OPNAME},
   $lbi_01_tdr_00021{OPNAME},
   $lbi_01_tdr_00022{OPNAME},
   $lbi_01_tdr_00023{OPNAME},
   $lbi_01_tdr_00024{OPNAME},
   $lbi_01_tdr_00025{OPNAME},
   $lbi_01_tdr_00026{OPNAME},
   $lbi_01_tdr_00027{OPNAME},
   $lbi_01_tdr_00028{OPNAME},
   $lbi_01_tdr_00029{OPNAME},
   $lbi_01_tdr_00030{OPNAME},
   $lbi_01_tdr_00031{OPNAME},
   $lbi_01_tdr_00032{OPNAME},
   $lbi_01_tdr_00033{OPNAME},
   $lbi_01_tdr_00034{OPNAME},
   $lbi_01_tdr_00035{OPNAME},
   $lbi_01_tdr_00036{OPNAME},
   $lbi_01_tdr_00037{OPNAME},
   $lbi_01_tdr_00038{OPNAME},
   $lbi_01_tdr_00039{OPNAME},
   $lbi_01_tdr_00040{OPNAME},
   $lbi_01_tdr_00041{OPNAME},
   $lbi_01_tdr_00042{OPNAME},
   $lbi_01_tdr_00043{OPNAME},
   $lbi_01_tdr_00044{OPNAME},
   $lbi_01_tdr_00045{OPNAME},
   $lbi_01_tdr_00046{OPNAME},
   $lbi_01_tdr_00047{OPNAME},
   $lbi_01_tdr_00048{OPNAME},
   $lbi_01_tdr_00049{OPNAME},
   $lbi_01_tdr_00050{OPNAME},
   $lbi_01_tdr_00051{OPNAME},
   $lbi_01_tdr_00052{OPNAME},
   $lbi_01_tdr_00053{OPNAME},
   $lbi_01_tdr_00054{OPNAME},
   $lbi_01_tdr_00055{OPNAME},
   $lbi_01_tdr_00056{OPNAME},
   $lbi_01_tdr_00057{OPNAME},
   $lbi_01_tdr_00058{OPNAME},
   $lbi_01_tdr_00059{OPNAME},
   $lbi_01_tdr_00060{OPNAME},
   $lbi_01_tdr_00061{OPNAME},
   $lbi_01_tdr_00062{OPNAME},
   $lbi_01_tdr_00063{OPNAME},
   $lbi_01_tdr_00064{OPNAME},
   $lbi_01_tdr_00065{OPNAME},
   $lbi_01_tdr_00066{OPNAME},
   $lbi_01_tdr_00067{OPNAME},
   $lbi_01_tdr_00068{OPNAME},
   $lbi_01_tdr_00069{OPNAME},
   $lbi_01_tdr_00070{OPNAME},
   $lbi_01_tdr_00071{OPNAME},
   $lbi_01_tdr_00072{OPNAME},
   $lbi_01_tdr_00073{OPNAME},
   $lbi_01_tdr_00074{OPNAME},
   $lbi_01_tdr_00075{OPNAME},
   $lbi_01_tdr_00076{OPNAME},
   $lbi_01_tdr_00077{OPNAME},
   $lbi_01_tdr_00078{OPNAME},
   $lbi_01_tdr_00079{OPNAME},
   $lbi_01_tdr_00080{OPNAME},
   $lbi_01_tdr_00081{OPNAME},
   $lbi_01_tdr_00082{OPNAME},
   $lbi_01_tdr_00083{OPNAME},
   $lbi_01_tdr_00084{OPNAME},
   $lbi_01_tdr_00085{OPNAME},
   $lbi_01_tdr_00086{OPNAME},
   $lbi_01_tdr_00087{OPNAME},
   $lbi_01_tdr_00088{OPNAME},
   $lbi_01_tdr_00089{OPNAME},
   $lbi_01_tdr_00090{OPNAME},
   $lbi_01_tdr_00091{OPNAME},
   $lbi_01_tdr_00092{OPNAME},
   $lbi_01_tdr_00093{OPNAME}
);


