################################################################################
#  
#  Modules required for SuperScope
#
################################################################################
use common_defs;
use jtag_subs;
use scancfg;
use shub_01_mmrs;
use reg_subs;
use mmr_subs;
use shub_subs;
use pktgen;
use error_regs;
use fmt_subs;
use ecc;

############### Define Global Variables ###############################
$FALSE  = 0;
$TRUE   = 1;

$f100MHz_CL_2_0 = 0;
$f133MHz_CL_2_0 = 1;
$f133MHz_CL_2_5 = 2;
$f160MHz_CL_2_0 = 3;
$f160MHz_CL_2_5 = 4;
$f160MHz_CL_3_0 = 5;
$f200MHz_CL_2_0 = 6;
$f200MHz_CL_2_5 = 7;
$f200MHz_CL_3_0 = 8;

# Valid Bist Command Values
$DIMM_INIT_MODE_SET   = 0x0;
$DIMM_INIT_REFRESH    = 0x1;
$DIMM_INIT_PRECHARGE  = 0x2;
$DIMM_INIT_BURST_TERM = 0x6;
$DIMM_INIT_NOP        = 0x7;
$FILL_DIR             = 0x20;
$FILL_DIR_ACCESS      = 0x40;
$FILL_DATA            = 0x30;
$DIMM_BIST_READ       = 0x10;
$READ_DIR_PAIR        = 0x50;

# Burst Type, used for DIMM Initialization Sequence
$BURST_LENGTH_2 = 1;
$BURST_LENGTH_4 = 2;
$BURST_LENGTH_8 = 3;

# Burst Type, used for DIMM Initialization Sequence
$SEQUENTIAL = 0;
$INTERLEAVE = 1;

# Cas Latency
$CL2_0 = 2;
$CL2_5 = 6;
$CL3_0 = 3;

$X = 1;
$Y = 0;

$LEFT  = 0;
$RIGHT = 1;

$DIRA = 0;
$DIRB = 1;

$BUSYI = 0x60;
$BSYIW = 0x62;
$BUSY  = 0x61;
$BSYEI = 0x63;
$BSYUW = 0x64;
$BSYUR = 0x65;

$BSYF  = 0x66;
$BSYFN = 0x67;
$BSYFY = 0x68;
$BSYG  = 0x69;
$BSYGN = 0x6a;
$BSYGY = 0x6b;

$BSYX  = 0x6c;
$BSYXN = 0x6d;
$BSYXY = 0x6e;
$BSYN  = 0x6f;
$BSYNN = 0x70;
$BSYNY = 0x71;

$BSYEN = 0x72;
$BSYEY = 0x73;
$BSYSN = 0x74;
$BSYSY = 0x75;
$BSYUN = 0x76;
$BSYUY = 0x77;

################ MD MEMORY CONFIG MMRS ##########################
$x_dim_cfg = reg_create_mmr_inst("shub_01", "SH_X_DIMM_CFG");
$y_dim_cfg = reg_create_mmr_inst("shub_01", "SH_Y_DIMM_CFG");
$j_dim_cfg = reg_create_mmr_inst("shub_01", "SH_JNR_DIMM_CFG");

$x_cfg      = reg_create_mmr_inst("shub_01", "SH_X_CFG");
$y_cfg      = reg_create_mmr_inst("shub_01", "SH_Y_CFG");
$x_dqct_cfg = reg_create_mmr_inst("shub_01", "SH_X_DQCT_CFG");
$y_dqct_cfg = reg_create_mmr_inst("shub_01", "SH_Y_DQCT_CFG");
$x_rfr      = reg_create_mmr_inst("shub_01", "SH_X_REFRESH_CONTROL");
$y_rfr      = reg_create_mmr_inst("shub_01", "SH_Y_REFRESH_CONTROL");
$rd_blk     = reg_create_mmr_inst("shub_01", "SH_MEM_RED_BLACK");
$lp_dir_cfg = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_CONFIG");
$rp_dir_cfg = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_CONFIG");

############## MD BIST DATA PATTERN MMRS ###############################
$lp_xbist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_XBIST_H");
$lp_xbist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_XBIST_L");
$rp_xbist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_XBIST_H");
$rp_xbist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_XBIST_L");

$lp_ybist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_YBIST_H");
$lp_ybist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_YBIST_L");
$rp_ybist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_YBIST_H");
$rp_ybist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_YBIST_L");

$ls_xbist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_XBIST_H");
$ls_xbist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_XBIST_L");
$rs_xbist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_XBIST_H");
$rs_xbist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_XBIST_L");

$ls_ybist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_YBIST_H");
$ls_ybist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_YBIST_L");
$rs_ybist_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_YBIST_H");
$rs_ybist_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_YBIST_L");

############## MD BIST ERROR MMRS ###################################
$lp_xbist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_XBIST_ERR_H");
$lp_xbist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_XBIST_ERR_L");
$rp_xbist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_XBIST_ERR_H");
$rp_xbist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_XBIST_ERR_L");

$lp_ybist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_YBIST_ERR_H");
$lp_ybist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_YBIST_ERR_L");
$rp_ybist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_YBIST_ERR_H");
$rp_ybist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_YBIST_ERR_L");

$ls_xbist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_XBIST_ERR_H");
$ls_xbist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_XBIST_ERR_L");
$rs_xbist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_XBIST_ERR_H");
$rs_xbist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_XBIST_ERR_L");

$ls_ybist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_YBIST_ERR_H");
$ls_ybist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQLS_MMR_YBIST_ERR_L");
$rs_ybist_err_h = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_YBIST_ERR_H");
$rs_ybist_err_l = reg_create_mmr_inst("shub_01", "SH_MD_DQRS_MMR_YBIST_ERR_L");

############## MD BIST WRITE DIRECTORY ENTRY MMRS ###################################
$dqlp_piowd_dir =  reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_PIOWD_DIR_ENTRY");
$dqlp_piowd_ecc =  reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_PIOWD_DIR_ECC");
$dqrp_piowd_dir =  reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_PIOWD_DIR_ENTRY");
$dqrp_piowd_ecc =  reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_PIOWD_DIR_ECC");

############## MD BIST READ DIRECTORY ENTRY MMRS ###################################
$lp_xdir_ecc   = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_XPIORD_XDIR_ECC");
$lp_xdir_entry = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_XPIORD_XDIR_ENTRY");
$lp_ydir_ecc   = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_YPIORD_YDIR_ECC");
$lp_ydir_entry = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_YPIORD_YDIR_ENTRY");
$rp_xdir_ecc   = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_XPIORD_XDIR_ECC");
$rp_xdir_entry = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_XPIORD_XDIR_ENTRY");
$rp_ydir_ecc   = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_YPIORD_YDIR_ECC");
$rp_ydir_entry = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_YPIORD_YDIR_ENTRY");

############## MD SUMMARY ERROR MMRS ###################################
$sh_mmrbist_err = reg_create_mmr_inst("shub_01", "SH_MMRBIST_ERR");
$sum_err        = reg_create_mmr_inst("shub_01", "SH_MEM_ERROR_SUMMARY");
$sum_err_al     = reg_create_mmr_inst("shub_01", "SH_MEM_ERROR_SUMMARY_ALIAS");
$ovfl_err       = reg_create_mmr_inst("shub_01", "SH_MEM_ERROR_OVERFLOW");
$ovfl_err_al    = reg_create_mmr_inst("shub_01", "SH_MEM_ERROR_OVERFLOW_ALIAS");

################ MD MEMORY PROTECTION ##########################
$lp_priv0 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC0");
$rp_priv0 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC0");
$lp_priv1 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC1");
$rp_priv1 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC1");
$lp_priv2 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC2");
$rp_priv2 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC2");
$lp_priv3 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC3");
$rp_priv3 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC3");
$lp_priv4 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC4");
$rp_priv4 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC4");
$lp_priv5 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC5");
$rp_priv5 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC5");
$lp_priv6 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC6");
$rp_priv6 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC6");
$lp_priv7 = reg_create_mmr_inst("shub_01", "SH_MD_DQLP_MMR_DIR_PRIVEC7");
$rp_priv7 = reg_create_mmr_inst("shub_01", "SH_MD_DQRP_MMR_DIR_PRIVEC7");

################ MD BIST CONTROL MMRS ##########################
$mbist_base = reg_create_mmr_inst("shub_01", "SH_MMRBIST_BASE");
$mbist_ctl = reg_create_mmr_inst("shub_01", "SH_MMRBIST_CTL");

########################################################################
# Writes the three MD DIMM Configuration MMRs
# Usage: write_dimm_cfg_mmrs(dimm_size, dimm_banks, dimm_groups, freq);
########################################################################
sub write_dimm_cfg_mmrs
{
  my $dimm_size   = $_[0];
  my $two_banks   = $_[1];
  my $dimm_groups = $_[2];
  my $freq        = $_[3];

  my $my_dimm_size = 7; # 7 = UNUSED DIMM SIZE
  if($dimm_size == 256) {
     $my_dimm_size = 2;
  } elsif($dimm_size == 512) {
     $my_dimm_size = 3;
  } elsif($dimm_size == 1024) {
     $my_dimm_size = 4;
  } elsif($dimm_size == 2048) {
     $my_dimm_size = 5;
  }

  my $dimm0_size = 7;
  my $dimm0_2bk  = 0;
  my $dimm0_rev  = 0;
  my $dimm0_cs   = 0;
  my $dimm1_size = 7;
  my $dimm1_2bk  = 0;
  my $dimm1_rev  = 0;
  my $dimm1_cs   = 1;
  my $dimm2_size = 7;
  my $dimm2_2bk  = 0;
  my $dimm2_rev  = 0;
  my $dimm2_cs   = 2;
  my $dimm3_size = 7;
  my $dimm3_2bk  = 0;
  my $dimm3_rev  = 0;
  my $dimm3_cs   = 3;

  for($z = 0; $z < $dimm_groups; $z++) {
    if($z == 0) {
      $dimm0_size = $my_dimm_size;
      $dimm0_2bk  = $two_banks;
    } elsif($z == 1) {
      $dimm1_size = $my_dimm_size;
      $dimm1_2bk  = $two_banks;
    } elsif($z == 2) {
      $dimm2_size = $my_dimm_size;
      $dimm2_2bk  = $two_banks;
    } elsif($z == 3) {
      $dimm3_size = $my_dimm_size;
      $dimm3_2bk  = $two_banks;
    }
  }

  reg_set_field_value($j_dim_cfg, "DIMM0_SIZE", $dimm0_size);
  reg_set_field_value($j_dim_cfg, "DIMM0_2BK", 	$dimm0_2bk);
  reg_set_field_value($j_dim_cfg, "DIMM0_REV", 	$dimm0_rev);
  reg_set_field_value($j_dim_cfg, "DIMM0_CS", 	$dimm0_cs);
  reg_set_field_value($j_dim_cfg, "DIMM1_SIZE", $dimm1_size);
  reg_set_field_value($j_dim_cfg, "DIMM1_2BK", 	$dimm1_2bk);
  reg_set_field_value($j_dim_cfg, "DIMM1_REV", 	$dimm1_rev);
  reg_set_field_value($j_dim_cfg, "DIMM1_CS", 	$dimm1_cs);
  reg_set_field_value($j_dim_cfg, "DIMM2_SIZE", $dimm2_size);
  reg_set_field_value($j_dim_cfg, "DIMM2_2BK", 	$dimm2_2bk);
  reg_set_field_value($j_dim_cfg, "DIMM2_REV", 	$dimm2_rev);
  reg_set_field_value($j_dim_cfg, "DIMM2_CS", 	$dimm2_cs);
  reg_set_field_value($j_dim_cfg, "DIMM3_SIZE", $dimm3_size);
  reg_set_field_value($j_dim_cfg, "DIMM3_2BK", 	$dimm3_2bk);
  reg_set_field_value($j_dim_cfg, "DIMM3_REV", 	$dimm3_rev);
  reg_set_field_value($j_dim_cfg, "DIMM3_CS", 	$dimm3_cs);
  reg_set_field_value($j_dim_cfg, "FREQ", 	$freq);

  reg_set_field_value($x_dim_cfg, "DIMM0_SIZE", $dimm0_size);
  reg_set_field_value($x_dim_cfg, "DIMM0_2BK", 	$dimm0_2bk);
  reg_set_field_value($x_dim_cfg, "DIMM0_REV", 	$dimm0_rev);
  reg_set_field_value($x_dim_cfg, "DIMM0_CS", 	$dimm0_cs);
  reg_set_field_value($x_dim_cfg, "DIMM1_SIZE", $dimm1_size);
  reg_set_field_value($x_dim_cfg, "DIMM1_2BK", 	$dimm1_2bk);
  reg_set_field_value($x_dim_cfg, "DIMM1_REV", 	$dimm1_rev);
  reg_set_field_value($x_dim_cfg, "DIMM1_CS", 	$dimm1_cs);
  reg_set_field_value($x_dim_cfg, "DIMM2_SIZE", $dimm2_size);
  reg_set_field_value($x_dim_cfg, "DIMM2_2BK", 	$dimm2_2bk);
  reg_set_field_value($x_dim_cfg, "DIMM2_REV", 	$dimm2_rev);
  reg_set_field_value($x_dim_cfg, "DIMM2_CS", 	$dimm2_cs);
  reg_set_field_value($x_dim_cfg, "DIMM3_SIZE", $dimm3_size);
  reg_set_field_value($x_dim_cfg, "DIMM3_2BK", 	$dimm3_2bk);
  reg_set_field_value($x_dim_cfg, "DIMM3_REV", 	$dimm3_rev);
  reg_set_field_value($x_dim_cfg, "DIMM3_CS", 	$dimm3_cs);
  reg_set_field_value($x_dim_cfg, "FREQ", 	$freq);

  reg_set_field_value($y_dim_cfg, "DIMM0_SIZE", $dimm0_size);
  reg_set_field_value($y_dim_cfg, "DIMM0_2BK", 	$dimm0_2bk);
  reg_set_field_value($y_dim_cfg, "DIMM0_REV", 	$dimm0_rev);
  reg_set_field_value($y_dim_cfg, "DIMM0_CS", 	$dimm0_cs);
  reg_set_field_value($y_dim_cfg, "DIMM1_SIZE", $dimm1_size);
  reg_set_field_value($y_dim_cfg, "DIMM1_2BK", 	$dimm1_2bk);
  reg_set_field_value($y_dim_cfg, "DIMM1_REV", 	$dimm1_rev);
  reg_set_field_value($y_dim_cfg, "DIMM1_CS", 	$dimm1_cs);
  reg_set_field_value($y_dim_cfg, "DIMM2_SIZE", $dimm2_size);
  reg_set_field_value($y_dim_cfg, "DIMM2_2BK", 	$dimm2_2bk);
  reg_set_field_value($y_dim_cfg, "DIMM2_REV", 	$dimm2_rev);
  reg_set_field_value($y_dim_cfg, "DIMM2_CS", 	$dimm2_cs);
  reg_set_field_value($y_dim_cfg, "DIMM3_SIZE", $dimm3_size);
  reg_set_field_value($y_dim_cfg, "DIMM3_2BK", 	$dimm3_2bk);
  reg_set_field_value($y_dim_cfg, "DIMM3_REV", 	$dimm3_rev);
  reg_set_field_value($y_dim_cfg, "DIMM3_CS", 	$dimm3_cs);
  reg_set_field_value($y_dim_cfg, "FREQ", 	$freq);

  reg_mmr_write($j_dim_cfg);
  reg_mmr_write($y_dim_cfg); 
  reg_mmr_write($x_dim_cfg); 

  my $cfg_data = 0;

  $cfg_data = reg_mmr_read($j_dim_cfg);
  printf"\nJNR_DIMM_CFG = 0x%016llX\n", $cfg_data;

  $cfg_data = reg_mmr_read($x_dim_cfg);
  printf"X_DIMM_CFG   = 0x%016llX\n", $cfg_data;

  $cfg_data = reg_mmr_read($y_dim_cfg);
  printf"X_DIMM_CFG   = 0x%016llX\n\n", $cfg_data;

}

########################################################################
# Sets the bist data pattern registers.
# Usage: bist_set_data(l_p, l_s, r_p, r_s);
########################################################################
sub  bist_set_data
{
  # The four variables below hold the data used to pattern memory from the
  # bist pattern registers.  All of the *_p registers hold 32 bits of data
  # pattern and all of the *_s registers hold 40 bits of data.  Together they
  # make up the 72 bits written to the dimms.  The data can be set to any value.
  my $l_p = $_[0];
  my $l_s = $_[1];
  my $r_p = $_[2];
  my $r_s = $_[3];

  my $inv = 0;
  my $rot = 0;

  # Arm all of the bist error capturing.  This bit arms the bist error
  # capture registers.
  # NOTE: The Arming bits are only found in the *_h mmrs.
  reg_set_field_value($rp_xbist_h, "ARM", 1);
  reg_set_field_value($rp_ybist_h, "ARM", 1);
  reg_set_field_value($rs_xbist_h, "ARM", 1);
  reg_set_field_value($rs_ybist_h, "ARM", 1);
  reg_set_field_value($ls_xbist_h, "ARM", 1);
  reg_set_field_value($ls_ybist_h, "ARM", 1);
  reg_set_field_value($lp_xbist_h, "ARM", 1);
  reg_set_field_value($lp_ybist_h, "ARM", 1);

  # Set up the X BIST Pattern variables
  reg_set_field_value($lp_xbist_h, "PAT", $l_p);
  reg_set_field_value($lp_xbist_h, "INV", $inv);
  reg_set_field_value($lp_xbist_h, "ROT", $rot);

  reg_set_field_value($rp_xbist_h, "PAT", $r_p);
  reg_set_field_value($rp_xbist_h, "INV", $inv);
  reg_set_field_value($rp_xbist_h, "ROT", $rot);

  reg_set_field_value($ls_xbist_h, "PAT", $l_s);
  reg_set_field_value($ls_xbist_h, "INV", $inv);
  reg_set_field_value($ls_xbist_h, "ROT", $rot);

  reg_set_field_value($rs_xbist_h, "PAT", $r_s);
  reg_set_field_value($rs_xbist_h, "INV", $inv);
  reg_set_field_value($rs_xbist_h, "ROT", $rot);

  reg_set_field_value($lp_xbist_l, "PAT", ~($l_p));
  reg_set_field_value($lp_xbist_l, "INV", $inv);
  reg_set_field_value($lp_xbist_l, "ROT", $rot);

  reg_set_field_value($rp_xbist_l, "PAT", ~($r_p));
  reg_set_field_value($rp_xbist_l, "INV", $inv);
  reg_set_field_value($rp_xbist_l, "ROT", $rot);

  reg_set_field_value($ls_xbist_l, "PAT", ~($l_s));
  reg_set_field_value($ls_xbist_l, "INV", $inv);
  reg_set_field_value($ls_xbist_l, "ROT", $rot);

  reg_set_field_value($rs_xbist_l, "PAT", ~($r_s));
  reg_set_field_value($rs_xbist_l, "INV", $inv);
  reg_set_field_value($rs_xbist_l, "ROT", $rot);

  # Set up the Y BIST Pattern variables
  reg_set_field_value($lp_ybist_h, "PAT", ~($l_p));
  reg_set_field_value($lp_ybist_h, "INV", $inv);
  reg_set_field_value($lp_ybist_h, "ROT", $rot);

  reg_set_field_value($rp_ybist_h, "PAT", ~($r_p));
  reg_set_field_value($rp_ybist_h, "INV", $inv);
  reg_set_field_value($rp_ybist_h, "ROT", $rot);

  reg_set_field_value($ls_ybist_h, "PAT", ~($l_s));
  reg_set_field_value($ls_ybist_h, "INV", $inv);
  reg_set_field_value($ls_ybist_h, "ROT", $rot);

  reg_set_field_value($rs_ybist_h, "PAT", ~($r_s));
  reg_set_field_value($rs_ybist_h, "INV", $inv);
  reg_set_field_value($rs_ybist_h, "ROT", $rot);

  reg_set_field_value($lp_ybist_l, "PAT", ($l_p));
  reg_set_field_value($lp_ybist_l, "INV", $inv);
  reg_set_field_value($lp_ybist_l, "ROT", $rot);

  reg_set_field_value($rp_ybist_l, "PAT", ($r_p));
  reg_set_field_value($rp_ybist_l, "INV", $inv);
  reg_set_field_value($rp_ybist_l, "ROT", $rot);

  reg_set_field_value($ls_ybist_l, "PAT", ($l_s));
  reg_set_field_value($ls_ybist_l, "INV", $inv);
  reg_set_field_value($ls_ybist_l, "ROT", $rot);

  reg_set_field_value($rs_ybist_l, "PAT", ($r_s));
  reg_set_field_value($rs_ybist_l, "INV", $inv);
  reg_set_field_value($rs_ybist_l, "ROT", $rot);

  # Write the Pattern MMRs
  reg_mmr_write($lp_xbist_h);
  reg_mmr_write($lp_xbist_l);
  reg_mmr_write($rp_xbist_h);
  reg_mmr_write($rp_xbist_l);

  reg_mmr_write($ls_xbist_h);
  reg_mmr_write($ls_xbist_l);
  reg_mmr_write($rs_xbist_h);
  reg_mmr_write($rs_xbist_l);

  reg_mmr_write($lp_ybist_h);
  reg_mmr_write($lp_ybist_l);
  reg_mmr_write($rp_ybist_h);
  reg_mmr_write($rp_ybist_l);

  reg_mmr_write($ls_ybist_h);
  reg_mmr_write($ls_ybist_l);
  reg_mmr_write($rs_ybist_h);
  reg_mmr_write($rs_ybist_l);

  #read_bist_data_mmrs();
  #check_data_bist_error_mmrs();
}

########################################################################
# Function to read out the bist pattern MMRs.
# Usage: read_bist_data_mmrs();
########################################################################
sub read_bist_data_mmrs
{
  my $data_pat = 0;
  
  $data_pat = reg_mmr_read($lp_xbist_h);
  printf"DQLP_MMR_XBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($lp_xbist_l);
  printf"DQLP_MMR_XBIST_L = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rp_xbist_h);
  printf"DQRP_MMR_XBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rp_xbist_l);
  printf"DQRP_MMR_XBIST_L = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($lp_ybist_h);
  printf"DQLP_MMR_YBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($lp_ybist_l);
  printf"DQLP_MMR_YBIST_L = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rp_ybist_h);
  printf"DQRP_MMR_YBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rp_ybist_l);
  printf"DQRP_MMR_YBIST_L = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($ls_xbist_h);
  printf"DQLS_MMR_XBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($ls_xbist_l);
  printf"DQLS_MMR_XBIST_L = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rs_xbist_h);
  printf"DQRS_MMR_XBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rs_xbist_l);
  printf"DQRS_MMR_XBIST_L = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($ls_ybist_h);
  printf"DQLS_MMR_YBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($ls_ybist_l);
  printf"DQLS_MMR_YBIST_L = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rs_ybist_h);
  printf"DQRS_MMR_YBIST_H = 0x%016llX\n", $data_pat;
  $data_pat = reg_mmr_read($rs_ybist_l);
  printf"DQRS_MMR_YBIST_L = 0x%016llX\n", $data_pat;

}

########################################################################
# Sets the Directory Entry pattern MMRs
# Usage: set_dir_pat_mmrs(dira, dirb, acc, pri, ecca, eccb);
########################################################################
sub set_dir_pat_mmrs
{

  my $dira = $_[0];
  my $dirb = $_[1];
  my $acc  = $_[2];
  my $pri  = $_[3];
  my $ecca = $_[4];
  my $eccb = $_[5];

  # Set the Left Directory Entry write fields for the Left DQs
  reg_set_field_value($dqlp_piowd_dir, "ACC", $acc);
  reg_set_field_value($dqlp_piowd_dir, "PRI", $pri);
  reg_set_field_value($dqlp_piowd_dir, "DIRA", $dirb);
  reg_set_field_value($dqlp_piowd_dir, "DIRB", $dira);

  # Set the Left Directory ECC write fields for the Left DQs
  reg_set_field_value($dqlp_piowd_ecc, "ECCA", $eccb);
  reg_set_field_value($dqlp_piowd_ecc, "ECCB", $ecca);

  # Set the Right Directory Entry write fields for the Right DQs
  reg_set_field_value($dqrp_piowd_dir, "ACC", $acc);
  reg_set_field_value($dqrp_piowd_dir, "PRI", $pri);
  reg_set_field_value($dqrp_piowd_dir, "DIRA", $dirb);
  reg_set_field_value($dqrp_piowd_dir, "DIRB", $dira);

  # Set the Left Directory ECC write fields for the Left DQs
  reg_set_field_value($dqrp_piowd_ecc, "ECCB", $ecca);
  reg_set_field_value($dqrp_piowd_ecc, "ECCA", $eccb);

  reg_mmr_write($dqlp_piowd_dir);
  reg_mmr_write($dqlp_piowd_ecc);
  reg_mmr_write($dqrp_piowd_dir);
  reg_mmr_write($dqrp_piowd_ecc);
}

########################################################################
# Reads the Directory Entry MMRs from the READ_DIR_PAIR bist command
# Usage: $x = read_bist_dir_mmrs(read_ecc);
# NOTE: if x > 0, errors occured
########################################################################
sub read_bist_dir_mmrs
{
  my $read_ecc = $_[0];

  my $k = 0;
  my $ret_val = 0;
  my $mmr_data = 0;
  my $mmr_name = "NONAME";
  my $dira = 0;
  my $dirb = 0;
  my $ecca = 0;
  my $eccb = 0;
  my $pri  = 0;
  my $acc  = 0;
  my $cor  = 0;
  my $unc  = 0;

  for($k = 0; $k < 4; $k++) {
    if($k == 0) {
      $mmr_data = reg_mmr_read($lp_xdir_entry);
      $mmr_name = "DQLP_XDIR_ENTRY";
      $dira = reg_get_field_value($lp_xdir_entry, "DIRA");
      $dirb = reg_get_field_value($lp_xdir_entry, "DIRB");
      $pri  = reg_get_field_value($lp_xdir_entry, "PRI");
      $acc  = reg_get_field_value($lp_xdir_entry, "ACC");
      $cor  = reg_get_field_value($lp_xdir_entry, "COR");
      $unc  = reg_get_field_value($lp_xdir_entry, "UNC");
    } elsif($k == 1) {
      $mmr_data = reg_mmr_read($rp_xdir_entry);
      $mmr_name = "DQRP_XDIR_ENTRY";
      $dira = reg_get_field_value($rp_xdir_entry, "DIRA");
      $dirb = reg_get_field_value($rp_xdir_entry, "DIRB");
      $pri  = reg_get_field_value($rp_xdir_entry, "PRI");
      $acc  = reg_get_field_value($rp_xdir_entry, "ACC");
      $cor  = reg_get_field_value($rp_xdir_entry, "COR");
      $unc  = reg_get_field_value($rp_xdir_entry, "UNC");
    } elsif($k == 2) {
      $mmr_data = reg_mmr_read($lp_ydir_entry);
      $mmr_name = "DQLP_YDIR_ENTRY";
      $dira = reg_get_field_value($lp_ydir_entry, "DIRA");
      $dirb = reg_get_field_value($lp_ydir_entry, "DIRB");
      $pri  = reg_get_field_value($lp_ydir_entry, "PRI");
      $acc  = reg_get_field_value($lp_ydir_entry, "ACC");
      $cor  = reg_get_field_value($lp_ydir_entry, "COR");
      $unc  = reg_get_field_value($lp_ydir_entry, "UNC");
    } elsif($k == 3) {
      $mmr_data = reg_mmr_read($rp_ydir_entry);
      $mmr_name = "DQRP_YDIR_ENTRY";
      $dira = reg_get_field_value($rp_ydir_entry, "DIRA");
      $dirb = reg_get_field_value($rp_ydir_entry, "DIRB");
      $pri  = reg_get_field_value($rp_ydir_entry, "PRI");
      $acc  = reg_get_field_value($rp_ydir_entry, "ACC");
      $cor  = reg_get_field_value($rp_ydir_entry, "COR");
      $unc  = reg_get_field_value($rp_ydir_entry, "UNC");
    }

    if($verbose) {
      printf"\t$mmr_name = ";
      printf"DIRA = 0x%07llX\n", $dira;
      printf"\t\t\t  DIRB = 0x%07llX\n", $dirb;
      printf"\t\t\t  PRI  = 0x%01llX\n", $pri;
      printf"\t\t\t  ACC  = 0x%01llX\n", $acc;
      printf"\t\t\t  COR  = 0x$cor\n";
      printf"\t\t\t  UNC  = 0x$unc\n";
    } else {
      #printf"\t$mmr_name = ";
      #printf"0x%016llX\n", $mmr_data;
    }

# There is no reset on the correctable and uncorrectable bits in these
# mmrs so their values cannot be trusted.
#    if($cor == 1) {
#      print"\t\t\t  ERROR: Correctable Error in Directory Entry Data!!!\n";
#      $ret_val++;
#    }
#    if($unc == 1) {
#      print"\t\t\t  ERROR: Uncorrectable Error in Directory Entry Data!!!\n";
#      $ret_val++;
#    }

  }

  if($read_ecc) {
    my $k = 0;
    for($k = 0; $k< 4; $k++) {

      if($k == 0) {
        $mmr_data = reg_mmr_read($lp_xdir_ecc);
        $mmr_name = "DQLP_XDIR_ECC";
        $ecca = reg_get_field_value($lp_xdir_ecc, "ECCA");
        $eccb = reg_get_field_value($lp_xdir_ecc, "ECCB");
      } elsif($k == 1) {
        $mmr_data = reg_mmr_read($rp_xdir_ecc);
        $mmr_name = "DQRP_XDIR_ECC";
        $ecca = reg_get_field_value($rp_xdir_ecc, "ECCA");
        $eccb = reg_get_field_value($rp_xdir_ecc, "ECCB");
      } elsif($k == 2) {
        $mmr_data = reg_mmr_read($lp_ydir_ecc);
        $mmr_name = "DQLP_YDIR_ECC";
        $ecca = reg_get_field_value($lp_ydir_ecc, "ECCA");
        $eccb = reg_get_field_value($lp_ydir_ecc, "ECCB");
      } elsif($k == 3) {
        $mmr_data = reg_mmr_read($rp_ydir_ecc);
        $mmr_name = "DQRP_YDIR_ECC";
        $ecca = reg_get_field_value($rp_ydir_ecc, "ECCA");
        $eccb = reg_get_field_value($rp_ydir_ecc, "ECCB");
      }

      printf"\t$mmr_name =\t";
      printf"ECCA = 0x%07llX\n", $ecca;
      printf"\t\tECCB = 0x%07llX\n", $eccb;
    }
  }

  return $ret_val;
}

########################################################################
# Checks the SH_MMRBIST_ERR MMR for directory bist errors.
# Usage: $x = check_dir_bist_error_mmrs();
# NOTE: if $x > 0, errors occured
########################################################################
sub check_dir_bist_error_mmrs
{
  print"\tChecking for dir errors in BIST Error MMR: ";
  my $bist_err     = 0;
  my $top_bist_err = 0;
  my $ret_val      = 0;

  $top_bist_err = reg_mmr_read($sh_mmrbist_err);
  if($top_bist_err != 0x0) {
    my $a = reg_get_field_value($sh_mmrbist_err, "DETECTED");
    my $b = reg_get_field_value($sh_mmrbist_err, "MULTIPLE_DETECTED");
    my $c = reg_get_field_value($sh_mmrbist_err, "ADDR");
    my $d = reg_get_field_value($sh_mmrbist_err, "CANCELLED");
    printf"\n\tERROR : SH_MMRBIST_ERR :";
    print" DETECTED      : 0x$a\n";
    printf"\t\t\t\t MULT_DETECTED : 0x$b\n";
    printf"\t\t\t\t ADDR          : 0x%08llX\n", $c << 3;
    printf"\t\t\t\t CANCELLED     : 0x$d\n";
    $ret_val = 1;
  }

  if($ret_val == 0) {
    print"\tPASS!\n";
  } else {
    print"\tFAIL!\n";
  }

  return $ret_val;
}

########################################################################
# Checks the SH_MMRBIST_ERR MMR and DQ compare patter MMRs
# for data bist errors.
# Usage: check_data_bist_error_mmrs();
########################################################################
sub check_data_bist_error_mmrs
{
  print"\tChecking for data errors in BIST Error MMRs: ";
  my $bist_err = 0;
  my $valid_err = 0;
  my $more_err = 0;
  my $pat = 0;
  my $mmrname = "NONAME";
  my $top_bist_err = 0;
  my $ret_val = 0;

  $top_bist_err = reg_mmr_read($sh_mmrbist_err);
  if($top_bist_err != 0x0) {
    my $a = reg_get_field_value($sh_mmrbist_err, "DETECTED");
    my $b = reg_get_field_value($sh_mmrbist_err, "MULTIPLE_DETECTED");
    my $c = reg_get_field_value($sh_mmrbist_err, "ADDR");
    my $d = reg_get_field_value($sh_mmrbist_err, "CANCELLED");
    printf"\n\tERROR : SH_MMRBIST_ERR :";
    print" DETECTED      : 0x$a\n";
    printf"\t\t\t\t MULT_DETECTED : 0x$b\n";
    printf"\t\t\t\t ADDR          : 0x%08llX\n", $c << 3;
    printf"\t\t\t\t CANCELLED     : 0x$d\n";
    $ret_val = 1;
  }

  if(1) {
    for($err = 0; $err < 16; $err++) {
      if($err == 0) {
        $bist_err  = reg_mmr_read($lp_xbist_err_h);
        $valid_err = reg_get_field_value($lp_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($lp_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($lp_xbist_err_h, "PAT");
        $data_pat =  reg_mmr_read($lp_xbist_h);
        $mmr_name = "DQLP_MMR_XBIST_ERR_H";

      } elsif($err == 1) {
        $bist_err  = reg_mmr_read($lp_xbist_err_l);
        $valid_err = reg_get_field_value($lp_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($lp_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($lp_xbist_err_l, "PAT");
        $data_pat =  reg_mmr_read($lp_xbist_l);
        $mmr_name = "DQLP_MMR_XBIST_ERR_L";

      } elsif($err == 2) {
        $bist_err  = reg_mmr_read($rp_xbist_err_h);
        $valid_err = reg_get_field_value($rp_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($rp_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($rp_xbist_err_h, "PAT");
        $data_pat =  reg_mmr_read($rp_xbist_h);
        $mmr_name = "DQRP_MMR_XBIST_ERR_H";

      } elsif($err == 3) {
        $bist_err  = reg_mmr_read($rp_xbist_err_l);
        $valid_err = reg_get_field_value($rp_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($rp_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($rp_xbist_err_l, "PAT");
        $data_pat =  reg_mmr_read($rp_xbist_l);
        $mmr_name = "DQRP_MMR_XBIST_ERR_L";

      } elsif($err == 4) {
        $bist_err  = reg_mmr_read($lp_ybist_err_h);
        $valid_err = reg_get_field_value($lp_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($lp_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($lp_ybist_err_h, "PAT");
        $data_pat =  reg_mmr_read($lp_ybist_h);
        $mmr_name = "DQLP_MMR_YBIST_ERR_H";

      } elsif($err == 5) {
        $bist_err  = reg_mmr_read($lp_ybist_err_l);
        $valid_err = reg_get_field_value($lp_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($lp_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($lp_ybist_err_l, "PAT");
        $data_pat =  reg_mmr_read($lp_ybist_l);
        $mmr_name = "DQLP_MMR_YBIST_ERR_L";

      } elsif($err == 6) {
        $bist_err  = reg_mmr_read($rp_ybist_err_h);
        $valid_err = reg_get_field_value($rp_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($rp_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($rp_ybist_err_h, "PAT");
        $data_pat =  reg_mmr_read($rp_ybist_h);
        $mmr_name = "DQRP_MMR_YBIST_ERR_H";

      } elsif($err == 7) {
        $bist_err  = reg_mmr_read($rp_ybist_err_l);
        $valid_err = reg_get_field_value($rp_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($rp_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($rp_ybist_err_l, "PAT");
        $data_pat =  reg_mmr_read($rp_ybist_l);
        $mmr_name = "DQRP_MMR_YBIST_ERR_L";

      } elsif($err == 8) {
        $bist_err  = reg_mmr_read($ls_xbist_err_h);
        $valid_err = reg_get_field_value($ls_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($ls_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($ls_xbist_err_h, "PAT");
        $data_pat =  reg_mmr_read($ls_xbist_h);
        $mmr_name = "DQLS_MMR_XBIST_ERR_H";

      } elsif($err == 9) {
        $bist_err  = reg_mmr_read($ls_xbist_err_l);
        $valid_err = reg_get_field_value($ls_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($ls_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($ls_xbist_err_l, "PAT");
        $data_pat =  reg_mmr_read($ls_xbist_l);
        $mmr_name = "DQLS_MMR_XBIST_ERR_L";

      } elsif($err == 10) {
        $bist_err  = reg_mmr_read($rs_xbist_err_h);
        $valid_err = reg_get_field_value($rs_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($rs_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($rs_xbist_err_h, "PAT");
        $data_pat =  reg_mmr_read($rs_xbist_h);
        $mmr_name = "DQRS_MMR_XBIST_ERR_H";

      } elsif($err == 11) {
        $bist_err  = reg_mmr_read($rs_xbist_err_l);
        $valid_err = reg_get_field_value($rs_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($rs_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($rs_xbist_err_l, "PAT");
        $data_pat =  reg_mmr_read($rs_xbist_l);
        $mmr_name = "DQRS_MMR_XBIST_ERR_L";

      } elsif($err == 12) {
        $bist_err  = reg_mmr_read($ls_ybist_err_h);
        $valid_err = reg_get_field_value($ls_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($ls_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($ls_ybist_err_h, "PAT");
        $data_pat =  reg_mmr_read($ls_ybist_h);
        $mmr_name = "DQLS_MMR_YBIST_ERR_H";

      } elsif($err == 13) {
        $bist_err  = reg_mmr_read($ls_ybist_err_l);
        $valid_err = reg_get_field_value($ls_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($ls_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($ls_ybist_err_l, "PAT");
        $data_pat =  reg_mmr_read($ls_ybist_l);
        $mmr_name = "DQLS_MMR_YBIST_ERR_L";

      } elsif($err == 14) {
        $bist_err  = reg_mmr_read($rs_ybist_err_h);
        $valid_err = reg_get_field_value($rs_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($rs_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($rs_ybist_err_h, "PAT");
        $data_pat =  reg_mmr_read($rs_ybist_h);
        $mmr_name = "DQRS_MMR_YBIST_ERR_H";

      } elsif($err == 15) {
        $bist_err  = reg_mmr_read($rs_ybist_err_l);
        $valid_err = reg_get_field_value($rs_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($rs_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($rs_ybist_err_l, "PAT");
        $data_pat =  reg_mmr_read($rs_ybist_l);
        $mmr_name = "DQRS_MMR_YBIST_ERR_L";
      }

      if($valid_err) { 
        printf"\t $mmr_name :\t";
        print"VAL  : 0x$valid_err\n";
        print"\t\t\t\tMORE : 0x$more_err\n";
        printf"\t\t\t\tPAT : 0x%010llX\n", $pat;
        printf"\t\t\t\tEXP : 0x%010llX\n", $data_pat;
      }
    }
  }

  if($ret_val == 0) {
    print"PASS!\n";
  } else {
    print"FAIL!\n";
  }

  return $ret_val;
}

########################################################################
# Checks the SH_MMRBIST_ERR MMR for bist errors.
# Usage: check_bist_error_mmrs();
########################################################################
sub check_bist_error_mmrs
{
  print"\tChecking for errors in BIST Error MMR: ";
  my $top_bist_err = 0;
  my $ret_val = 0;
  my $a = 0;
  my $b = 0;
  my $c = 0;

  $top_bist_err = reg_mmr_read($sh_mmrbist_err);
  if($top_bist_err != 0x0) {
    my $a = reg_get_field_value($sh_mmrbist_err, "DETECTED");
    my $b = reg_get_field_value($sh_mmrbist_err, "MULTIPLE_DETECTED");
    my $c = reg_get_field_value($sh_mmrbist_err, "ADDR");
    my $d = reg_get_field_value($sh_mmrbist_err, "CANCELLED");
    printf"\n\tERROR : SH_MMRBIST_ERR :";
    printf" DETECTED      : 0x$a\n";
    printf"\t\t\t\t MULT_DETECTED : 0x$b\n";
    printf"\t\t\t\t ADDR          : 0x%08llX\n", $c << 3; # shifting makes it a $line addr
    printf"\t\t\t\t CANCELLED     : 0x$d\n";
    $ret_val = 1;

    if($verbose) {
      more_bist_error_mmrs();
    }
  }

  if($ret_val == 0) {
    print"\tPASS!\n";
  } else {
    print"\tFAIL!\n";
  }

  return $ret_val;
}

########################################################################
# If the verbose output is selected, the function prints out the bist
# miscompare mmrs.
# Usage: more_bist_error_mmrs();
########################################################################
sub more_bist_error_mmrs
{
  my $err = 0;
  my $bist_err = 0;
  my $valid_err = 0;
  my $more_err = 0;
  my $pat = 0;
  my $mmrname = "NONAME";

    for($err = 0; $err < 16; $err++) {
      if($err == 0) {
        $bist_err  = reg_mmr_read($lp_xbist_err_h);
        $valid_err = reg_get_field_value($lp_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($lp_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($lp_xbist_err_h, "PAT");
        $mmr_name = "DQLP_MMR_XBIST_ERR_H";

      } elsif($err == 1) {
        $bist_err  = reg_mmr_read($lp_xbist_err_l);
        $valid_err = reg_get_field_value($lp_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($lp_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($lp_xbist_err_l, "PAT");
        $mmr_name = "DQLP_MMR_XBIST_ERR_L";

      } elsif($err == 2) {
        $bist_err  = reg_mmr_read($rp_xbist_err_h);
        $valid_err = reg_get_field_value($rp_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($rp_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($rp_xbist_err_h, "PAT");
        $mmr_name = "DQRP_MMR_XBIST_ERR_H";

      } elsif($err == 3) {
        $bist_err  = reg_mmr_read($rp_xbist_err_l);
        $valid_err = reg_get_field_value($rp_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($rp_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($rp_xbist_err_l, "PAT");
        $mmr_name = "DQRP_MMR_XBIST_ERR_L";

      } elsif($err == 4) {
        $bist_err  = reg_mmr_read($lp_ybist_err_h);
        $valid_err = reg_get_field_value($lp_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($lp_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($lp_ybist_err_h, "PAT");
        $mmr_name = "DQLP_MMR_YBIST_ERR_H";

      } elsif($err == 5) {
        $bist_err  = reg_mmr_read($lp_ybist_err_l);
        $valid_err = reg_get_field_value($lp_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($lp_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($lp_ybist_err_l, "PAT");
        $mmr_name = "DQLP_MMR_YBIST_ERR_L";

      } elsif($err == 6) {
        $bist_err  = reg_mmr_read($rp_ybist_err_h);
        $valid_err = reg_get_field_value($rp_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($rp_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($rp_ybist_err_h, "PAT");
        $mmr_name = "DQRP_MMR_YBIST_ERR_H";

      } elsif($err == 7) {
        $bist_err  = reg_mmr_read($rp_ybist_err_l);
        $valid_err = reg_get_field_value($rp_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($rp_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($rp_ybist_err_l, "PAT");
        $mmr_name = "DQRP_MMR_YBIST_ERR_L";

      } elsif($err == 8) {
        $bist_err  = reg_mmr_read($ls_xbist_err_h);
        $valid_err = reg_get_field_value($ls_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($ls_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($ls_xbist_err_h, "PAT");
        $mmr_name = "DQLS_MMR_XBIST_ERR_H";

      } elsif($err == 9) {
        $bist_err  = reg_mmr_read($ls_xbist_err_l);
        $valid_err = reg_get_field_value($ls_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($ls_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($ls_xbist_err_l, "PAT");
        $mmr_name = "DQLS_MMR_XBIST_ERR_L";

      } elsif($err == 10) {
        $bist_err  = reg_mmr_read($rs_xbist_err_h);
        $valid_err = reg_get_field_value($rs_xbist_err_h, "VAL");
        $more_err  = reg_get_field_value($rs_xbist_err_h, "MORE");
        $pat       = reg_get_field_value($rs_xbist_err_h, "PAT");
        $mmr_name = "DQRS_MMR_XBIST_ERR_H";

      } elsif($err == 11) {
        $bist_err  = reg_mmr_read($rs_xbist_err_l);
        $valid_err = reg_get_field_value($rs_xbist_err_l, "VAL");
        $more_err  = reg_get_field_value($rs_xbist_err_l, "MORE");
        $pat       = reg_get_field_value($rs_xbist_err_l, "PAT");
        $mmr_name = "DQRS_MMR_XBIST_ERR_L";

      } elsif($err == 12) {
        $bist_err  = reg_mmr_read($ls_xbist_err_h);
        $valid_err = reg_get_field_value($ls_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($ls_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($ls_ybist_err_h, "PAT");
        $mmr_name = "DQLS_MMR_YBIST_ERR_H";

      } elsif($err == 13) {
        $bist_err  = reg_mmr_read($ls_ybist_err_l);
        $valid_err = reg_get_field_value($ls_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($ls_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($ls_xbist_err_l, "PAT");
        $mmr_name = "DQLS_MMR_YBIST_ERR_L";

      } elsif($err == 14) {
        $bist_err  = reg_mmr_read($rs_ybist_err_h);
        $valid_err = reg_get_field_value($rs_ybist_err_h, "VAL");
        $more_err  = reg_get_field_value($rs_ybist_err_h, "MORE");
        $pat       = reg_get_field_value($rs_ybist_err_h, "PAT");
        $mmr_name = "DQRS_MMR_YBIST_ERR_H";
      } elsif($err == 15) {
        $bist_err  = reg_mmr_read($rs_ybist_err_l);
        $valid_err = reg_get_field_value($rs_ybist_err_l, "VAL");
        $more_err  = reg_get_field_value($rs_ybist_err_l, "MORE");
        $pat       = reg_get_field_value($rs_ybist_err_l, "PAT");
        $mmr_name = "DQRS_MMR_YBIST_ERR_L";
      }

      if($valid_err || $more_err) {
        printf"\t $mmr_name :\t";
        print"VAL  : 0x$valid_err\n";
        print"\t\t\t\tMORE : 0x$more_err\n";
        printf"\t\t\t\tPAT : 0x%010llX\n", $pat;
      }
    }
}

########################################################################
# Looks for errors in the SH_MEM_ERROR_SUMMARY and SH_MEM_ERROR_OVERFLOW MMRs
# Usage: check_summary_err_mmr();
########################################################################
sub check_summary_err_mmr
{
  print"\tChecking for errors in SUMMARY MMR: ";
  my $sum_err_data = 0;
  my $ovfl_err_data = 0;
  my $ret_val  = 0;

  $sum_err_data = reg_mmr_read($sum_err);
  if($verbose) {
    print"\n";
    printf"\tMEM_ERROR_SUMMARY  = 0x%016llX\n", $sum_err_data;
  }
  if($sum_err_data != 0) {
    $ret_val = 1;
  }
  $ovfl_err_data = reg_mmr_read($ovfl_err);
  if($verbose) {
    printf"\tMEM_ERROR_OVERFLOW = 0x%016llX\n", $ovfl_err_data;
  }

  if($ret_val == 0) {
    print"\tPASS!\n";
  } else {
    print"\tFAIL!\n";
  }

  return $ret_val;
}

########################################################################
# Clears the MEM_ERROR_SUMMARY and MEM_ERROR_OVERFLOW MMRs
# Usage: clear_summary_err_mmr();
########################################################################
sub clear_summary_err_mmr
{
  #reg_set_field_value($sum_err_al, "NONEXIST_ADDR", 0x1);
  #reg_set_field_value($sum_err_al, "DIR_ACC", 0x1);
  #reg_set_field_value($ovfl_err_al, "NONEXIST_ADDR", 0x1);
  #reg_set_field_value($ovfl_err_al, "DIR_ACC", 0x1);

  reg_set_value($sh_mmrbist_err, 0x0);
  reg_set_value($sum_err_al,     0xffffffffffffffff);
  reg_set_value($ovfl_err_al,    0xffffffffffffffff);

  reg_mmr_write($sum_err_al);
  reg_mmr_write($ovfl_err_al);
  reg_mmr_write($sh_mmrbist_err);
  print"Cleared out MEM_ERROR_SUMMARY MMRs.\n";
}

########################################################################
# Forms the address used for the bist engine given a dimm group number
# and dimm physical banks.
# Usage: form_bist_address(cs, dimm_bank)
########################################################################
sub  form_bist_address
{
  my $cs   = $_[0];
  my $dimm_bank = $_[1];

  my $address = 0;
  # CS is always bits 34 & 35
  $address = $cs << 34;

  # Bank bit is A12.  This is unused in a 2 bank config for MRS.
  # Only RAS bist are used.
  $address |= $dimm_bank << 12;

  return $address;
}

########################################################################
# Starts the bist engine with a given command, d-word block length, and address.
# Usage: start_bist_engine(cmd, block_length, addr);
########################################################################
sub start_bist_engine
{
  my $cmd          = $_[0];
  my $block_length = $_[1];
  my $base_addr    = $_[2];

  # Valid Bist Command Values
  # DIMM_INIT_MODE_SET   = 0x0;
  # DIMM_INIT_REFRESH    = 0x1;
  # DIMM_INIT_PRECHARGE  = 0x2;
  # DIMM_INIT_BURST_TERM = 0x6;
  # DIMM_INIT_NOP        = 0x7;
  # FILL_DIR             = 0x20
  # FILL_DATA            = 0x30
  # DIMM_BIST_READ       = 0x10
  # READ_DIR_PAIR        = 0x50

  # Set up and write the bist base address mmr
  reg_set_value($mbist_base, $base_addr);
  reg_mmr_write($mbist_base);

  # Set up and write the bist control mmr
  # NOTE: The bist engine ignores the bottom 6 bits of the block_length field
  # for a directory fill.
  reg_set_field_value($mbist_ctl, "CMD", $cmd);
  reg_set_field_value($mbist_ctl, "BLOCK_LENGTH", $block_length);
  reg_set_field_value($mbist_ctl, "IN_PROGRESS", 1);

  reg_mmr_write($mbist_ctl);

  # Monitor the "in_progress" bit in the bist control register to verify the
  # bist engine is finished.
  mbist_wait_loop(1);

}

########################################################################
# Waits for the 'in_progress' bit in the bist control register to clear.
# Usage: mbist_wait_loop();
########################################################################
sub mbist_wait_loop
{
  my $check_err  = $_[0];
  my $in_prog    = 1;
  my $read_loops = 0;

  while ($in_prog == 1)
  {
     $ctl_read_val = reg_mmr_read($mbist_ctl);
     $in_prog      = reg_get_field_value($mbist_ctl, "IN_PROGRESS");
     $eflag        = reg_get_field_value($mbist_ctl, "FAIL");

     if ( $in_prog ) {   sleep 2;
         $read_loops++;
         if ( $read_loops > 5 ) {   
             $emess = "\n\n ERROR:  Timed out waiting for BIST engine to finish!!!\n\n";
             dsp_error_and_exit($emess);
         }
     }
  }

  if ( $check_err != 1 ) {
    if ( $eflag ) {
        print"\nERROR:  FAIL bit set in BIST control register!!!\n\n";
    }
  }
}

###############################################################################
# read_mem() reads out a selected number of cachelines from a given address 
# from which to start reading.  The number of cachelines read is always an even
# number so the selected number must be greater than one.  It returns an array
# of arrays that contains the address and its corresponding data.
#
# Usage: @array = read_mem($address_to_start_from, $num_of_cachelines_to_read);
###############################################################################
sub read_mem {

  my $address = $_[0];
  my $max_cachelines = $_[1];

  my $block_length = 0x20;
  my $cmd = 0x10; # DIMM_BIST_READ = 0x10
  my $base_addr = 0;
  my $dw_addr = 0;
  my $main_iter = 0;

  my $t = 0; my $x = 0; my $y = 0; my $w = 0;

  clear_summary_err_mmr();

  print"\n";
  print"*************************************************************\n";
  print"Block Memory Read via BIST Engine\n";
  print"*************************************************************\n";
@X_addr = ();
@X_dw = ();
@Y_addr = ();
@Y_dw = ();
@X_ecc  = ();
@Y_ecc  = ();
@Main = ();

  for($cachelines = 0; $cachelines < ($max_cachelines / 2); $cachelines++) {


      $address &= 0xfffffff00;
      printf"address = 0x%09llX\n", $address;

      for($t = 0; $t < 4; $t++) {
        $base_addr = $address;
        if($t == 1) {
          $base_addr |= 0x20;
        } elsif($t == 2) {
          $base_addr |= 0x40;
        } elsif($t == 3) {
          $base_addr |= 0x60;
        }

        bist_set_data(0x1234567879abcdef, 0x1234567879abcdef, 0x1234567879abcdef, 0x1234567879abcdef);
        start_bist_engine($cmd, $block_length, $base_addr);

        @dw_array = ();
        @dw_array = parse_bist_err_mmrs();

        $dw_addr = $base_addr;
        for($w = 0; $w < 4; $w++) {
          push (@X_addr, $dw_addr);
          push (@X_dw,   $dw_array[$w]);
          $dw_addr += 0x8;
        }

        $dw_addr = $base_addr | 0x80;
        for($w = 4; $w < 8; $w++) {
          push (@Y_addr, $dw_addr);
          push (@Y_dw,   $dw_array[$w]);
          $dw_addr += 0x8;

        }

	for($w = 8; $w < 10; $w++) {
	  push (@X_ecc, $dw_array[$w]);
	}
	for($w = 10; $w < 12; $w++) {
	  push (@Y_ecc, $dw_array[$w]);
	}

        $array_length = @X_addr;
        if($array_length == 16) {

	  $ecc = 0;
          for($x = 0; $x < 16; $x++) {
	    $Main[$main_iter] = [$X_addr[$x], $X_dw[$x], ((($X_ecc[$ecc])>>(($x%2)*8))&0xff)];
	    $main_iter++;
	    if($x % 2) {
	      $ecc++;
	    }
          }

	  $ecc = 0;
          for($y = 0; $y < 16; $y++) {
	    $Main[$main_iter] = [$Y_addr[$y], $Y_dw[$y], ((($Y_ecc[$ecc])>>(($y%2)*8))&0xff)];
	    $main_iter++;
	    if($y % 2) {
	      $ecc++;
	    }
          }

          @X_addr = ();
          @X_dw   = ();
          @Y_addr = ();
          @Y_dw   = ();
          @X_ecc  = ();
          @Y_ecc  = ();
        }
      }
      $address += 0x100;
  }

#printf"0x%09llX, 0x%016llX, 0x%02lX\n", $Main[31][0], $Main[31][1], $Main[31][2];
  return @Main;
}

########################################################################
# If the verbose output is selected, the function prints out the bist
# miscompare mmrs.
# Usage: parse_bist_err_mmrs();
########################################################################
sub parse_bist_err_mmrs
{
  my $err = 0;
  my $bist_err = 0;
  my $valid_err = 0;
  my $more_err = 0;
  my $pat = 0;
  my $mmrname = "NONAME";
  my $left_x_p_h  = 0;
  my $left_x_s_h  = 0;
  my $right_x_p_h = 0;
  my $right_x_s_h = 0;
  my $left_x_p_l  = 0;
  my $left_x_s_l  = 0;
  my $right_x_p_l = 0;
  my $right_x_s_l = 0;

  my $left_y_p_h  = 0;
  my $left_y_s_h  = 0;
  my $right_y_p_h = 0;
  my $right_y_s_h = 0;
  my $left_y_p_l  = 0;
  my $left_y_s_l  = 0;
  my $right_y_p_l = 0;
  my $right_y_s_l = 0;

  for($err = 0; $err < 16; $err++) {

    if($err == 0) {
      $bist_err  = reg_mmr_read($lp_xbist_err_h);
      $valid_err = reg_get_field_value($lp_xbist_err_h, "VAL");
      $more_err  = reg_get_field_value($lp_xbist_err_h, "MORE");
      $pat       = reg_get_field_value($lp_xbist_err_h, "PAT");
      $mmr_name = "DQLP_MMR_XBIST_ERR_H";
      $left_x_p_h = $pat;

    } elsif($err == 1) {
      $bist_err  = reg_mmr_read($ls_xbist_err_h);
      $valid_err = reg_get_field_value($ls_xbist_err_h, "VAL");
      $more_err  = reg_get_field_value($ls_xbist_err_h, "MORE");
      $pat       = reg_get_field_value($ls_xbist_err_h, "PAT");
      $mmr_name = "DQLS_MMR_XBIST_ERR_H";
      $left_x_s_h = $pat;

    } elsif($err == 2) {
      $bist_err  = reg_mmr_read($rp_xbist_err_h);
      $valid_err = reg_get_field_value($rp_xbist_err_h, "VAL");
      $more_err  = reg_get_field_value($rp_xbist_err_h, "MORE");
      $pat       = reg_get_field_value($rp_xbist_err_h, "PAT");
      $mmr_name = "DQRP_MMR_XBIST_ERR_H";
      $right_x_p_h = $pat;
    } elsif($err == 3) {
      $bist_err  = reg_mmr_read($rs_xbist_err_h);
      $valid_err = reg_get_field_value($rs_xbist_err_h, "VAL");
      $more_err  = reg_get_field_value($rs_xbist_err_h, "MORE");
      $pat       = reg_get_field_value($rs_xbist_err_h, "PAT");
      $mmr_name = "DQRS_MMR_XBIST_ERR_H";
      $right_x_s_h = $pat;

    } elsif($err == 4) {
      $bist_err  = reg_mmr_read($lp_xbist_err_l);
      $valid_err = reg_get_field_value($lp_xbist_err_l, "VAL");
      $more_err  = reg_get_field_value($lp_xbist_err_l, "MORE");
      $pat       = reg_get_field_value($lp_xbist_err_l, "PAT");
      $mmr_name = "DQLP_MMR_XBIST_ERR_L";
      $left_x_p_l = $pat;

    } elsif($err == 5) {
      $bist_err  = reg_mmr_read($ls_xbist_err_l);
      $valid_err = reg_get_field_value($ls_xbist_err_l, "VAL");
      $more_err  = reg_get_field_value($ls_xbist_err_l, "MORE");
      $pat       = reg_get_field_value($ls_xbist_err_l, "PAT");
      $mmr_name = "DQLS_MMR_XBIST_ERR_L";
      $left_x_s_l = $pat;

    } elsif($err == 6) {
      $bist_err  = reg_mmr_read($rp_xbist_err_l);
      $valid_err = reg_get_field_value($rp_xbist_err_l, "VAL");
      $more_err  = reg_get_field_value($rp_xbist_err_l, "MORE");
      $pat       = reg_get_field_value($rp_xbist_err_l, "PAT");
      $mmr_name = "DQRP_MMR_XBIST_ERR_L";
      $right_x_p_l = $pat;

    } elsif($err == 7) {
      $bist_err  = reg_mmr_read($rs_xbist_err_l);
      $valid_err = reg_get_field_value($rs_xbist_err_l, "VAL");
      $more_err  = reg_get_field_value($rs_xbist_err_l, "MORE");
      $pat       = reg_get_field_value($rs_xbist_err_l, "PAT");
      $mmr_name = "DQRS_MMR_XBIST_ERR_L";
      $right_x_s_l = $pat;

    } elsif($err == 8) {
      $bist_err  = reg_mmr_read($lp_ybist_err_h);
      $valid_err = reg_get_field_value($lp_ybist_err_h, "VAL");
      $more_err  = reg_get_field_value($lp_ybist_err_h, "MORE");
      $pat       = reg_get_field_value($lp_ybist_err_h, "PAT");
      $mmr_name = "DQLP_MMR_YBIST_ERR_H";
      $left_y_p_h = $pat;

    } elsif($err == 9) {
      $bist_err  = reg_mmr_read($ls_ybist_err_h);
      $valid_err = reg_get_field_value($ls_ybist_err_h, "VAL");
      $more_err  = reg_get_field_value($ls_ybist_err_h, "MORE");
      $pat       = reg_get_field_value($ls_ybist_err_h, "PAT");
      $mmr_name = "DQLS_MMR_YBIST_ERR_H";
      $left_y_s_h = $pat;

    } elsif($err == 10) {
      $bist_err  = reg_mmr_read($rp_ybist_err_h);
      $valid_err = reg_get_field_value($rp_ybist_err_h, "VAL");
      $more_err  = reg_get_field_value($rp_ybist_err_h, "MORE");
      $pat       = reg_get_field_value($rp_ybist_err_h, "PAT");
      $mmr_name = "DQRP_MMR_YBIST_ERR_H";
      $right_y_p_h = $pat;

    } elsif($err == 11) {
      $bist_err  = reg_mmr_read($rs_ybist_err_h);
      $valid_err = reg_get_field_value($rs_ybist_err_h, "VAL");
      $more_err  = reg_get_field_value($rs_ybist_err_h, "MORE");
      $pat       = reg_get_field_value($rs_ybist_err_h, "PAT");
      $mmr_name = "DQRS_MMR_YBIST_ERR_H";
      $right_y_s_h = $pat;

    } elsif($err == 12) {
      $bist_err  = reg_mmr_read($lp_ybist_err_l);
      $valid_err = reg_get_field_value($lp_ybist_err_l, "VAL");
      $more_err  = reg_get_field_value($lp_ybist_err_l, "MORE");
      $pat       = reg_get_field_value($lp_ybist_err_l, "PAT");
      $mmr_name = "DQLP_MMR_YBIST_ERR_L";
      $left_y_p_l = $pat;

    } elsif($err == 13) {
      $bist_err  = reg_mmr_read($ls_ybist_err_l);
      $valid_err = reg_get_field_value($ls_ybist_err_l, "VAL");
      $more_err  = reg_get_field_value($ls_ybist_err_l, "MORE");
      $pat       = reg_get_field_value($ls_ybist_err_l, "PAT");
      $mmr_name = "DQLS_MMR_YBIST_ERR_L";
      $left_y_s_l = $pat;

    } elsif($err == 14) {
      $bist_err  = reg_mmr_read($rp_ybist_err_l);
      $valid_err = reg_get_field_value($rp_ybist_err_l, "VAL");
      $more_err  = reg_get_field_value($rp_ybist_err_l, "MORE");
      $pat       = reg_get_field_value($rp_ybist_err_l, "PAT");
      $mmr_name = "DQRP_MMR_YBIST_ERR_L";
      $right_y_p_l = $pat;

    } elsif($err == 15) {
      $bist_err  = reg_mmr_read($rs_ybist_err_l);
      $valid_err = reg_get_field_value($rs_ybist_err_l, "VAL");
      $more_err  = reg_get_field_value($rs_ybist_err_l, "MORE");
      $pat       = reg_get_field_value($rs_ybist_err_l, "PAT");
      $mmr_name = "DQRS_MMR_YBIST_ERR_L";
      $right_y_s_l = $pat;
    }

    if($verbose) {
      if($valid_err || $more_err) {
        printf"\t $mmr_name :\t";
        print"VAL  : 0x$valid_err\n";
        print"\t\t\t\tMORE : 0x$more_err\n";
        printf"\t\t\t\tPAT : 0x%010llX\n", $pat;
      }
    }
  }

  my $dw0  = 0;
  my $dw1  = 0;
  my $ecc1 = 0;
  my $dw2  = 0;
  my $dw3  = 0;
  my $ecc2 = 0;
  my $dw4  = 0;
  my $dw5  = 0;
  my $ecc3 = 0;
  my $dw6  = 0;
  my $dw7  = 0;
  my $ecc4 = 0;

  unscramble_data($left_x_p_h, $left_x_s_h,
                  $right_x_p_h, $right_x_s_h,
                  $dw0, $dw1, $ecc1);
  unscramble_data($left_x_p_l, $left_x_s_l,
                  $right_x_p_l, $right_x_s_l,
                  $dw2, $dw3, $ecc2);

  unscramble_data($left_y_p_h, $left_y_s_h,
                  $right_y_p_h, $right_y_s_h,
                  $dw4, $dw5, $ecc3);
  unscramble_data( $left_y_p_l, $left_y_s_l,
                  $right_y_p_l, $right_y_s_l,
                  $dw6, $dw7, $ecc4);

  @dword_array = ();
  $dword_array[0]  = $dw0;   # X Bus
  $dword_array[1]  = $dw1;   # X Bus
  $dword_array[2]  = $dw2;   # X Bus
  $dword_array[3]  = $dw3;   # X Bus
  $dword_array[4]  = $dw4;   # Y Bus
  $dword_array[5]  = $dw5;   # Y Bus
  $dword_array[6]  = $dw6;   # Y Bus
  $dword_array[7]  = $dw7;   # Y Bus
  $dword_array[8]  = $ecc1;  # X Bus
  $dword_array[9]  = $ecc2;  # X Bus
  $dword_array[10] = $ecc3;  # Y Bus
  $dword_array[11] = $ecc4;  # Y Bus

#printf"parse_bist_err_mmrs() dword_array8  = 0x%016llX\n", $dword_array[8];
#printf"parse_bist_err_mmrs() dword_array9  = 0x%016llX\n", $dword_array[9];
#printf"parse_bist_err_mmrs() dword_array10 = 0x%016llX\n", $dword_array[10];
#printf"parse_bist_err_mmrs() dword_array11 = 0x%016llX\n", $dword_array[11];

  return @dword_array;
}

########################################################################
# unscramble_data() takes the data from the bist error registers and 
# unscrambles it so it is in the form of the fsb data.
#
# Usage: unscramble_data($left_primary_data, $left_secondary_data,
#                         $right_primary_data, $right_secondary_data);
########################################################################
sub unscramble_data
{
  my $left_p  = $_[0];
  my $left_s  = $_[1];
  my $right_p = $_[2];
  my $right_s = $_[3];

  my $data0 = 0;
  my $data1 = 0;

  my $DimmBit = 0;
  my $LorR = 0;

  my $ecc = 0;
  my $z = 0;

  for($i = 0; $i < 64; $i++) {
        $DimmBit = $DoubleWordData[$i][0];
        $LorR    = $DoubleWordData[$i][1];

        if($LorR eq "LEFT") {
          if($DimmBit >= 0 && $DimmBit <= 39) {
             $data0 |= ((($left_s >> $DimmBit) & 0x1) << $i);

          } elsif($DimmBit > 39 && $DimmBit <= 71) {
             $data0 |= ((($left_p >> ($DimmBit-40)) & 0x1) << $i);
          }

        } else {
          if($DimmBit >= 0 && $DimmBit <= 39) {
             $data0 |= ((($right_s >> $DimmBit) & 0x1) << $i);

          } elsif($DimmBit > 39 && $DimmBit <= 71) {
             $data0 |= ((($right_p >> ($DimmBit-40)) & 0x1) << $i);
          }
        }
  }


  for($i = 64; $i < 128; $i++) {
        $DimmBit = $DoubleWordData[$i][0];
        $LorR    = $DoubleWordData[$i][1];

        if($LorR eq "LEFT") {
          if($DimmBit >= 0 && $DimmBit <= 39) {
             $data1 |= ((($left_s >> $DimmBit) & 0x1) << ($i - 64));
          }
          elsif($DimmBit > 39 && $DimmBit <= 71) {
             $data1 |= ((($left_p >> ($DimmBit-40)) & 0x1) << ($i - 64));
          }

        } else {
          if($DimmBit >= 0 && $DimmBit <= 39) {
             $data1 |= ((($right_s >> $DimmBit) & 0x1) << ($i - 64));
          }
          elsif($DimmBit > 39 && $DimmBit <= 71) {
             $data1 |= ((($right_p >> ($DimmBit-40)) & 0x1) << ($i - 64));
          }
        }
  }

  for($z = 0; $z < 16; $z++) {
        $DimmBit = $DoubleWordECC[$z][0];

          if($DimmBit >= 0 && $DimmBit <= 39) {
             $ecc |= ((($right_s >> $DimmBit) & 0x1) << $z);
          }
          elsif($DimmBit > 39 && $DimmBit <= 71) {
             $ecc |= ((($right_p >> ($DimmBit-40)) & 0x1) << $z);
          }
  }

  $_[4] = $data0;
  $_[5] = $data1;
  $_[6] = $ecc;
}

########################################################################
# scramble_data() takes the data and scrambles it.
#
# Usage: scramble_data($data0, $data1 
#                      $left_primary_data,  $left_secondary_data,
#                      $right_primary_data, $right_secondary_data);
########################################################################
sub scramble_data
{
  $data0 = $_[0];
  $data1 = $_[1];

#  printf"data0 = 0x%016llX\n", $data0;
#  printf"data1 = 0x%016llX\n", $data1;
#  printf"\n";

  $left_p  = 0;
  $left_s  = 0;
  $right_p = 0;
  $right_s = 0;

  for($i = 0; $i < 64; $i++) {
    #printf"DoubleWordData[$i][0] = $DoubleWordData[$i][0]\n";
    $DimmBit = $DoubleWordData[$i][0];

    if($DoubleWordData[$i][1] eq "LEFT") {
        if($DimmBit >= 0 && $DimmBit <= 39) {
          $left_s  |= ((($data0 >> $i) & 0x1) << $DimmBit);
        } else {
          $left_p  |= ((($data0 >> $i) & 0x1) << ($DimmBit - 40));
        }
    } else {
        if($DimmBit >= 0 && $DimmBit <= 39) {
          $right_s |= ((($data0 >> $i) & 0x1) << $DimmBit);
        } else {
          $right_p |= ((($data0 >> $i) & 0x1) << ($DimmBit - 40));
        }
    }
  }

  for($i = 64; $i < 128; $i++) {
    #printf"DoubleWordData[$i][0] = $DoubleWordData[$i][0]\n";
    $DimmBit = $DoubleWordData[$i][0];

    if($DoubleWordData[$i][1] eq "LEFT") {
        if($DimmBit >= 0 && $DimmBit <= 39) {
          $left_s  |= ((($data1 >> ($i - 64)) & 0x1) << $DimmBit);
        } else {
          $left_p  |= ((($data1 >> ($i - 64)) & 0x1) << ($DimmBit - 40));
        }
    } else {
        if($DimmBit >= 0 && $DimmBit <= 39) {
          $right_s |= ((($data1 >> ($i - 64)) & 0x1) << $DimmBit);
        } else {
          $right_p |= ((($data1 >> ($i - 64)) & 0x1) << ($DimmBit - 40));
        }
    }
  }

#  printf"left_p  = 0x%010llX\n", $left_p;
#  printf"left_s  = 0x%010llX\n", $left_s;
#  printf"right_p = 0x%010llX\n", $right_p;
#  printf"right_s = 0x%010llX\n", $right_s;

  $_[2] = $left_p;
  $_[3] = $left_s;
  $_[4] = $right_p;
  $_[5] = $right_s;
}

#############################################################################
# form_dram_init_address() Forms the DRAM init address used by the bist
# engine to initialized the dimms.
# 
# Usage: $address = form_dram_init_address(control, cs, bank, emrs_flag);
#############################################################################
sub form_dram_init_address
{
  my $control   = $_[0];
  my $cs        = $_[1];
  my $bank      = $_[2];
  my $emrs_flag = $_[3];
  my $address   = 0;

  # CS is alwasy bits 34 & 35
  $address = ($cs & 0x3) << 34;

  # Bank bit is A12.  This is used in a two bank configuration,
  # only RAS bist are used.
  $address |= ($bank & 0x1) << 12;

  # RAS bits 0 & 1 come from address 10 & 11
  $address |= ($control & 0x3) << 10;

  # RAS bits 2-12 are address bits 13-23
  $address |= (($control >> 2) & 0x7ff) << 13;

  # RAS bit 13 is address bit 26
  $address |= (($control >> 13) & 0x1) << 26;

  # EMR is BA = b'01.  Address bits 8 & 9.
  if($emrs_flag == 1) {
    $address |= 0x1 << 8;
  }

  return $address;
}

#############################################################################
# set_dimm_refresh_value() Sets the refresh control value.
#
# Usage: 
# $ref_mmr_data = set_dimm_refresh_value($max_dimm_groups, $two_dimm_banks);
#############################################################################
sub set_dimm_refresh_value {
  my $max_dimm_groups = $_[0];
  my $two_dimm_banks  = $_[1];
  my $ref_mmr_data;

  # Refresh control register. Number of dimm groups/banks
  # needs to be enabled in x_refresh_control_enable
  #
  #  group #      banks   value
  #    1           1     0x01
  #    1           2     0x03
  #    2           1     0x04
  #    2           2     0x0c
  #    3           1     0x10
  #    3           2     0x30
  #    4           1     0x40
  #    4           2     0xc0
  #
  if($max_dimm_groups == 1) {
    $ref_mmr_data = 0x1CC301;
    if($two_dimm_banks == 1) {
      $ref_mmr_data |= 0x02;
    }
  } elsif($max_dimm_groups == 2) {
    $ref_mmr_data = 0x1CC305;
    if($two_dimm_banks == 1) {
      $ref_mmr_data |= 0x0a;
    }
  } elsif($max_dimm_groups == 3) {
    $ref_mmr_data = 0x1CC315;
    if($two_dimm_banks == 1) {
      $ref_mmr_data |= 0x2a;
    }
  } elsif($max_dimm_groups == 4) {
    $ref_mmr_data = 0x1CC355;
    if($two_dimm_banks == 1) {
      $ref_mmr_data |= 0xaa;
    }
  }

  return $ref_mmr_data;
}

############################################################################################
# OUTPUT_MMR_VALUE()
############################################################################################
sub output_mmr_value
{
    my ($mmreg) = @_;
    $mmval = reg_mmr_read( $mmreg );
    printf "%-30s (0x%09llX) = 0x%016llX\n", reg_name($mmreg), reg_addr($mmreg), $mmval;
}

 @DoubleWordData = (
    [  24, "LEFT"  ], # 0
    [  25, "LEFT"  ], # 1
    [  26, "LEFT"  ], # 2
    [  27, "LEFT"  ], # 3
    [  32, "LEFT"  ], # 4
    [  33, "LEFT"  ], # 5
    [  34, "LEFT"  ], # 6
    [  35, "LEFT"  ], # 7
    [  40, "LEFT"  ], # 8
    [  41, "LEFT"  ], # 9
    [  42, "LEFT"  ], # 10
    [  43, "LEFT"  ], # 11
    [   0, "RIGHT" ], # 12
    [   1, "RIGHT" ], # 13
    [   2, "RIGHT" ], # 14
    [   3, "RIGHT" ], # 15
    [   0, "LEFT"  ], # 16
    [   1, "LEFT"  ], # 17
    [   2, "LEFT"  ], # 18
    [   3, "LEFT"  ], # 19
    [   8, "LEFT"  ], # 20
    [   9, "LEFT"  ], # 21
    [  10, "LEFT"  ], # 22
    [  11, "LEFT"  ], # 23
    [  10, "RIGHT" ], # 24
    [  11, "RIGHT" ], # 25
    [  16, "LEFT"  ], # 26
    [  17, "LEFT"  ], # 27
    [  18, "LEFT"  ], # 28
    [  19, "LEFT"  ], # 29
    [  18, "RIGHT" ], # 30
    [  19, "RIGHT" ], # 31
    [  24, "RIGHT" ], # 32
    [  25, "RIGHT" ], # 33
    [  26, "RIGHT" ], # 34
    [  27, "RIGHT" ], # 35
    [  32, "RIGHT" ], # 36
    [  33, "RIGHT" ], # 37
    [  34, "RIGHT" ], # 38
    [  35, "RIGHT" ], # 39
    [  56, "RIGHT" ], # 40
    [  57, "RIGHT" ], # 41
    [  58, "RIGHT" ], # 42
    [  59, "RIGHT" ], # 43
    [  64, "RIGHT" ], # 44
    [  65, "RIGHT" ], # 45
    [  66, "RIGHT" ], # 46
    [  67, "RIGHT" ], # 47
    [  48, "LEFT"  ], # 48
    [  49, "LEFT"  ], # 49
    [  50, "LEFT"  ], # 50
    [  51, "LEFT"  ], # 51
    [  56, "LEFT"  ], # 52
    [  57, "LEFT"  ], # 53
    [  58, "LEFT"  ], # 54
    [  59, "LEFT"  ], # 55
    [  50, "RIGHT" ], # 56
    [  51, "RIGHT" ], # 57
    [  64, "LEFT"  ], # 58
    [  65, "LEFT"  ], # 59
    [  66, "LEFT"  ], # 60
    [  67, "LEFT"  ], # 61
    [  42, "RIGHT" ], # 62
    [  43, "RIGHT" ], # 63
    [  28, "LEFT"  ], # 64
    [  29, "LEFT"  ], # 65
    [  30, "LEFT"  ], # 66
    [  31, "LEFT"  ], # 67
    [  36, "LEFT"  ], # 68
    [  37, "LEFT"  ], # 69
    [  38, "LEFT"  ], # 70
    [  39, "LEFT"  ], # 71
    [  44, "LEFT"  ], # 72
    [  45, "LEFT"  ], # 73
    [  46, "LEFT"  ], # 74
    [  47, "LEFT"  ], # 75
    [   4, "RIGHT" ], # 76
    [   5, "RIGHT" ], # 77
    [   6, "RIGHT" ], # 78
    [   7, "RIGHT" ], # 79
    [   4, "LEFT"  ], # 80
    [   5, "LEFT"  ], # 81
    [   6, "LEFT"  ], # 82
    [   7, "LEFT"  ], # 83
    [  12, "LEFT"  ], # 84
    [  13, "LEFT"  ], # 85
    [  14, "LEFT"  ], # 86
    [  15, "LEFT"  ], # 87
    [  14, "RIGHT" ], # 88
    [  15, "RIGHT" ], # 89
    [  20, "LEFT"  ], # 90
    [  21, "LEFT"  ], # 91
    [  22, "LEFT"  ], # 92
    [  23, "LEFT"  ], # 93
    [  22, "RIGHT" ], # 94
    [  23, "RIGHT" ], # 95
    [  28, "RIGHT" ], # 96
    [  29, "RIGHT" ], # 97
    [  30, "RIGHT" ], # 98
    [  31, "RIGHT" ], # 99
    [  36, "RIGHT" ], # 100
    [  37, "RIGHT" ], # 101
    [  38, "RIGHT" ], # 102
    [  39, "RIGHT" ], # 103
    [  60, "RIGHT" ], # 104
    [  61, "RIGHT" ], # 105
    [  62, "RIGHT" ], # 106
    [  63, "RIGHT" ], # 107
    [  68, "RIGHT" ], # 108
    [  69, "RIGHT" ], # 109
    [  70, "RIGHT" ], # 110
    [  71, "RIGHT" ], # 111
    [  52, "LEFT"  ], # 112
    [  53, "LEFT"  ], # 113
    [  54, "LEFT"  ], # 114
    [  55, "LEFT"  ], # 115
    [  60, "LEFT"  ], # 116
    [  61, "LEFT"  ], # 117
    [  62, "LEFT"  ], # 118
    [  63, "LEFT"  ], # 119
    [  54, "RIGHT" ], # 120
    [  55, "RIGHT" ], # 121
    [  68, "LEFT"  ], # 122
    [  69, "LEFT"  ], # 123
    [  70, "LEFT"  ], # 124
    [  71, "LEFT"  ], # 125
    [  46, "RIGHT" ], # 126
    [  47, "RIGHT" ], # 127
);

@DoubleWordECC = (
    [  40, "RIGHT" ], # 0
    [  48, "RIGHT" ], # 1
    [   8, "RIGHT" ], # 2
    [  16, "RIGHT" ], # 3
    [  17, "RIGHT" ], # 4
    [   9, "RIGHT" ], # 5
    [  49, "RIGHT" ], # 6
    [  41, "RIGHT" ], # 7
    [  44, "RIGHT" ], # 8
    [  52, "RIGHT" ], # 9
    [  12, "RIGHT" ], # 10
    [  20, "RIGHT" ], # 11
    [  21, "RIGHT" ], # 12
    [  13, "RIGHT" ], # 13
    [  53, "RIGHT" ], # 14
    [  45, "RIGHT" ], # 15
);

1;

