#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 2003-2004 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: multi_chip.pm,v 1.2 2004/02/10 23:12:51 beighley Exp $
#
#-------------------------------------------------------------------------------
#
#  FILE: multi_chip.pm
#
#  DESCRIPTION:  This file contains PERL subroutines to build multi-chip
#     target lists, and to read/write MMRs from multiple chips.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  ------------------------------------------------
#     12/11/03  Karen Beighley  Subroutines moved into this file for checkin.
#     02/10/04  Karen Beighley  Modified to support multi-chain modules.
#
#-------------------------------------------------------------------------------

$TARGET_BRICK = 0;
$TARGET_BOARD = 1;
$TARGET_CHIP  = 2;
$TARGET_RACK  = 3;
$TARGET_SLOT  = 4;
$TARGET_CNUM  = 5;
$TARGET_CPOS  = 6;
$TARGET_DREF  = 7;

#-------------------------------------------------------------------------------

sub multi_chip_read_mmr_list
{
   printf "\nmulti_chip_read_mmr_list( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $list_ref  = $_[0];
   my $beg_ind   = $_[1];
   my $max       = $_[2];
   my $addr_list = $_[3];
   my ($num, $i, $j, $ref, @data_list, @ptrs);

   #---------------------------------------
   #  Set the MMR target list
   #---------------------------------------
   $num = multi_chip_set_mmr_target($list_ref, $beg_ind, $max);
   printf "-> Number of MMR targets selected = %u\n", $num if ( $debug_flag & $DEBUG_DETAILS );
   return($FALSE) if ( $num < 1 );

   #---------------------------------------
   #  Read the list of MMRs for each target
   #---------------------------------------
   if ( $num == 1 )
   {  @data_list = mmr_read_group($addr_list);
      @ptrs = ();
      $ptrs[0] = \@data_list;
   }
   else
   {  @ptrs = mmr_read_group($addr_list);
   }

   #---------------------------------------
   #  Store the reference to the MMR data
   #---------------------------------------
   for ($i = 0, $j = 0; $j < $num; $i++)
   {  $ref = $$list_ref[$i+$beg_ind];
      if ( $ref != 0 )
      {  $$ref[$TARGET_DREF] = $ptrs[$j];
         $j++;
      }
   }

   return($TRUE);
}


#-------------------------------------------------------------------------------

sub multi_chip_write_mmr_list
{
   printf "\nmulti_chip_write_mmr_list( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $list_ref  = $_[0];
   my $beg_ind   = $_[1];
   my $max       = $_[2];
   my $addr_list = $_[3];
   my $data_list = $_[4];

   #---------------------------------------
   #  Set the MMR target list
   #---------------------------------------
   my $num = multi_chip_set_mmr_target($list_ref, $beg_ind, $max);
   printf "-> Number of MMR targets selected = %u\n", $num if ( $debug_flag & $DEBUG_DETAILS );
   return($FALSE) if ( $num < 1 );

   #---------------------------------------
   #  Read the list of MMRs for each target
   #---------------------------------------
   mmr_write_group($addr_list, $data_list);

   return($TRUE);
}


#-------------------------------------------------------------------------------

sub multi_chip_set_mmr_target
{
   printf "\nmulti_chip_set_mmr_target( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $list_ref = $_[0];
   my $beg_ind  = $_[1];
   my $num      = $_[2];

   #---------------------------------------
   #  Determine maximum index
   #---------------------------------------
   my @data = ();
   my $max_ind = $#$list_ref + 1;
   if ( ($beg_ind + $num) > $max_ind )
   {  $num = $max_ind - $beg_ind;
   }

   #---------------------------------------
   #  Process the target list
   #---------------------------------------
   for ($i = 0, $j = 0; $i < $num; $i++)
   {  $ref = $$list_ref[$i+$beg_ind];
      if ( $ref != 0 )
      {  $data[$j++] = $$ref[0];
         $data[$j++] = $$ref[1];
         $data[$j++] = $$ref[2];
      }
   }

   return ( 0 ) if ( $j == 0 );
   return( mmr_target($SELECT, @data) );
}


#-------------------------------------------------------------------------------

sub multi_chip_build_target_list
{
   printf "\nmulti_chip_build_target_list( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my @list = ();
   return(\@list) if ( $#_ != 3 );
   
   my $type  = $_[0];
   my $bname = $_[1];
   my $cname = $_[2];
   my $ntype = ( $_[3] eq 'physical' ? $CMP_IND_PHY : $CMP_IND_LOG );
   my $num   = 0;
   my ($chip_ptr, $name, $ref, $i, $j, $k, $found, $chn_ind, $chip_ind);

   #---------------------------------------
   #  Process each valid module/brick in the system
   #---------------------------------------
   foreach $brick ( @module_names )
   {  $mref = $modules{$brick};

      #---------------------------------------
      #  Match brick type and chip name
      #---------------------------------------
      if ( $$mref{TYPE} eq $type )
      {  $chip_ptr = scan_module_chip_ptr($mref, $bname, $cname, $ntype);
         if ( defined($chip_ptr) )
         {  my @data   = ();
            $data[$TARGET_BRICK] = $brick;
            $data[$TARGET_BOARD] = $$chip_ptr[$CMP_IND_BOARD];
            $data[$TARGET_CHIP]  = $$chip_ptr[$ntype]; 
            $data[$TARGET_RACK]  = $$mref{RACK}; 
            $data[$TARGET_SLOT]  = $$mref{SLOT};
            ($chn_ind, $chp_ind) = scan_module_find_chip($mref, $bname, $cname, $ntype);
            $data[$TARGET_CNUM]  = $chn_ind;
            $data[$TARGET_CPOS]  = $chp_ind;
            $data[$TARGET_DREF]  = 0;
            if ( $num == 0 )
            {  $list[$num] = \@data;
            }
            else
            {  for ($j = 0, $found = $FALSE; (($j < $num) && !$found); $j++)
               {  $ref = $list[$j];
                  $found = ( (($data[$TARGET_SLOT] < $$ref[$TARGET_SLOT]) ||
                             (($data[$TARGET_SLOT] == $$ref[$TARGET_SLOT]) && 
                              ($data[$TARGET_RACK] < $$ref[$TARGET_RACK]))) ? $TRUE : $FALSE );
               }
               if ( $found  )
               {  $j--;
                  for ($k = $num; $k > $j; $k--)
                  {  $list[$k] = $list[($k-1)];
                  }
               }
               $list[$j] = \@data; 
            }
            $num += 1;
         }
      }
   }

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf("\n-> Target List (#chips=%u)\n", $num);
      for ($i = 0; $i < $num; $i++)
      {  $ref = $list[$i];
         printf("%03u : ( @$ref )\n", $i);
      }
   }

   return(\@list);
}


#-------------------------------------------------------------------------------

sub multi_chip_copy_target_list
{
   printf "\nmulti_chip_copy_target_list( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my @list = ();
   return(\@list) if ( $#_ != 3 );

   my $list_ref = $_[0];
   my $bname    = $_[1];
   my $cname    = $_[2];
   my $ntype    = ( $_[3] eq 'physical' ? $CMP_IND_PHY : $CMP_IND_LOG );
   my $num      = 0;
   my ($b, $ref, $mref, $chip_ptr, $chn_ind, $chp_ind, $i);

   #---------------------------------------
   #  Process each module/brick in the list
   #---------------------------------------
   for ($b = 0; $b <= $#$list_ref; $b++)
   {  $ref  = $$list_ref[$b];
      if ( $ref == 0 )
      {  $list[$b] = 0;
      }
      else
      {  $mref = $modules{ $$ref[$TARGET_BRICK] };
         $chip_ptr = scan_module_chip_ptr($mref, $bname, $cname, $ntype);
         if ( defined($chip_ptr) )
         {  my @data   = ();
            $data[$TARGET_BRICK] = $$ref[$TARGET_BRICK];
            $data[$TARGET_BOARD] = $$chip_ptr[$CMP_IND_BOARD];
            $data[$TARGET_CHIP]  = $$chip_ptr[$ntype]; 
            $data[$TARGET_RACK]  = $$mref{RACK}; 
            $data[$TARGET_SLOT]  = $$mref{SLOT};
            ($chn_ind, $chp_ind) = scan_module_find_chip($mref, $bname, $cname, $ntype);
            $data[$TARGET_CNUM]  = $chn_ind;
            $data[$TARGET_CPOS]  = $chp_ind;
            $data[$TARGET_DREF]  = 0;
            $list[$b] = \@data;
            $num += 1;
         }
         else
         {  $list[$b] = 0;
         }
      }
   }

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf("\n-> Target List (#chips=%u)\n", ($#list + 1));
      for ($i = 0; $i <= $#list; $i++)
      {  $ref = $list[$i];
         if ( $ref == 0 )
         {  printf("%03u : NONE\n", $i);
         }
         else
         {  printf("%03u : ( @$ref )\n", $i);
         }
      }
   }

   return(\@list);
}


#-------------------------------------------------------------------------------

1;
