#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 2002-2005 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: nl4r_funcs.pm,v 1.16 2006/12/20 07:47:50 meyer Exp $
#
#  FILE: nl4r_funcs.pm
#
#  DESCRIPTION:  This file contains PERL routines to dump and parse nl4r registers.
#
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     05/19/06  Gary Meyer      Original source code submitted for release.  Updated for 1955 chassis.
#     05/23/06  Gary Meyer      Updated detail dump for file_mode.
#     05/27/06  Gary Meyer      Changes for exit state enumeration.
#     06/23/06  Gary Meyer      Added error handling for mmr_target() call.  Changed router naming in 
#				$current_location for width in the time stamp list in the Final summary.
#     06/28/06  Gary Meyer	Updates for NL4R error detail regs found in the fru file.
#     07/12/06  Gary Meyer	Updated so routing type errors don't get promoted in the FRU list.
#				Also, generating a fru entry for the source of a misrouted packet.
#     07/24/06  Gary Meyer      Update to ensure all NASID values use 4 characters.  Added tollhouse 
#				chassis type info.
#     08/21/06  Gary Meyer      Update to only flag MCAST_REP_GATHER errors if router error state has 
#				been initialized by prom.
#     08/31/06  Gary Meyer	Update to cover the case where all jtag reads fail.
#     09/06/06  Gary Meyer	Update to display all fields in the R4_STATUS_REV_ID mmr.
#     10/16/06  Gary Meyer	Added decodes for various Metarouter types.  Initialize $SSCOPE_ERR
#				before doing any jtag accesses.
#     10/31/06  Gary Meyer	Added R4_TSF_DIAG_MSG_CTL mmr.
#     11/10/06  Gary Meyer	Corrected bug, where a flagged Port E error summary was incorrectly preloading
#                               FRU location information.
#     11/28/06  Gary Meyer	Cosmetic changes to improve the formatting of fru information.
#     12/19/06  Gary Meyer      Changes to ensure correct fru scoring for Tollhouse IRU routers.
#     12/20/06  Gary Meyer      Updated to correctly report data corruption errors in the fru REASON.
#
#
#
#-------------------------------------------------------------------------------


####################################################################################
#
#	This subroutine reads a list of NL4R MMR's.
#
#	The NL4R registers in the list are derived from /stand/sysco/lib/sscope/nl4rtr_01_mmrs.pm
#
#
#  GEM  Need to add fru entry from error detail, at least for illegal VC errors.
#       see errdmp_8851_6 on cores.csd in /cores/people/mikeh/2637053.APAC.  Good example
#       of an INVALID_GLOBAL_LU_VCH1 error on port D.  
#       Will need to pass a mask to nl4r_dump_error_detail() that would contain which VC
#       the first error occurred on, and that function could return the NASID value found
#       in the captured header.
#
####################################################################################

# Define port connections for Tollhouse Chassis

  @BLADE_LIST_0L = (	"Blade 4, NI0",		# port b  RTR0, Left Side  (191010 & 1955)
			"Blade 3, NI0",		# port c
			"Blade 2, NI0",		# port d
			"Blade 1, NI0",		# port e
			"Blade 0, NI0" );	# port f

  @BLADE_LIST_1L = (    "Blade 9, NI0",		# port b  RTR1, Left Side  (191010 only)
                        "Blade 8, NI0",		# port c
                        "Blade 7, NI0",		# port d
                        "Blade 6, NI0",		# port e
                        "Blade 5, NI0" );	# port f

  @BLADE_LIST_0R = (    "Blade 5, NI1",		# port b  RTR0, Right Side  (191010 only)
                        "Blade 6, NI1",		# port c
                        "Blade 7, NI1",		# port d
                        "Blade 8, NI1",		# port e
                        "Blade 9, NI1" );	# port f

  @BLADE_LIST_1R = (    "Blade 0, NI1",		# port b  RTR1, Right Side  (191010 only, 1955 RTR0)
                        "Blade 1, NI1",		# port c
                        "Blade 2, NI1",		# port d
                        "Blade 3, NI1",		# port e
                        "Blade 4, NI1" );	# port f
  

sub read_nl4r_mmrs
{
  print "\n\nEntering read_nl4r_mmrs() function\n\n" if ($verbose_mode == 1);

  @nl4r_mmr_names = ();

  if ($abbrev_mode == 1)  { 
    @nl4r_mmr_names = ( 
			"R4_PROTECTION_CONFIG",
			"R4_STATUS_REV_ID"); 
  }
  else {
    @nl4r_mmr_names = ( 	
			"R4_PROTECTION_CONFIG",
			"R4_STATUS_REV_ID",
			"R4_GLOBAL_PARM0",
			"R4_GLOBAL_PARM1",
			"R4_RTC_SOURCE",

                        "R4_SCRATCH0",
                        "R4_SCRATCH1",
                        "R4_SCRATCH2",
                        "R4_SCRATCH3",
			"R4_SCRATCH4",
			"R4_SCRATCH5",
                        "R4_SCRATCH6",
                        "R4_SCRATCH7",
                        "R4_SCRATCH8",
			"R4_SCRATCH9",

			"R4_TSF_DIAG_MSG_CTL",

			"R4_ASIC_FIRST_ERROR",
			"R4_ASIC_FIRST_ERROR_TIMESTAMP",
                        "R4_LB_FIRST_ERROR",
                        "R4_LB_ERROR_SUMMARY",
			"R4_LB_ERROR_REPORT_MASK",
                        "R4_LB_ERROR_DETAIL_1",
                        "R4_LB_ERROR_DETAIL_2",

			"R4_PORTA_LLP_STATE",
			"R4_PORTB_LLP_STATE",
			"R4_PORTC_LLP_STATE",
			"R4_PORTD_LLP_STATE",
			"R4_PORTE_LLP_STATE",
			"R4_PORTF_LLP_STATE",
			"R4_PORTG_LLP_STATE",
			"R4_PORTH_LLP_STATE",

			"R4_PORTA_FIRST_ERROR",
			"R4_PORTA_ERROR_SUMMARY",
			"R4_PORTA_ERROR_REPORT_MASK",
			"R4_PORTA_ERROR_DETAIL",
			"R4_PORTB_FIRST_ERROR",
			"R4_PORTB_ERROR_SUMMARY",
			"R4_PORTB_ERROR_REPORT_MASK",
			"R4_PORTB_ERROR_DETAIL",
			"R4_PORTC_FIRST_ERROR",
			"R4_PORTC_ERROR_SUMMARY",
			"R4_PORTC_ERROR_REPORT_MASK",
			"R4_PORTC_ERROR_DETAIL",
			"R4_PORTD_FIRST_ERROR",
			"R4_PORTD_ERROR_SUMMARY",
			"R4_PORTD_ERROR_REPORT_MASK",
			"R4_PORTD_ERROR_DETAIL",
			"R4_PORTE_FIRST_ERROR",
			"R4_PORTE_ERROR_SUMMARY",
			"R4_PORTE_ERROR_REPORT_MASK",
			"R4_PORTE_ERROR_DETAIL",
			"R4_PORTF_FIRST_ERROR",
			"R4_PORTF_ERROR_SUMMARY",
			"R4_PORTF_ERROR_REPORT_MASK",
			"R4_PORTF_ERROR_DETAIL",
			"R4_PORTG_FIRST_ERROR",
			"R4_PORTG_ERROR_SUMMARY",
			"R4_PORTG_ERROR_REPORT_MASK",
			"R4_PORTG_ERROR_DETAIL",
			"R4_PORTH_FIRST_ERROR",
			"R4_PORTH_ERROR_SUMMARY",
			"R4_PORTH_ERROR_REPORT_MASK",
			"R4_PORTH_ERROR_DETAIL",
			
			"R4_PORTA_PARMS",
			"R4_PORTA_LLP_CONFIG",
			"R4_PORTA_LLP_TEST_CTL",
			"R4_PORTA_LLP_ERR1",
			"R4_PORTA_LLP_ERR2",
			"R4_PORTA_CREDIT_STATE",
			"R4_PORTA_CREDIT",
			"R4_PORTB_PARMS",
			"R4_PORTB_LLP_CONFIG",
			"R4_PORTB_LLP_TEST_CTL",
			"R4_PORTB_LLP_ERR1",
			"R4_PORTB_LLP_ERR2",
			"R4_PORTB_CREDIT_STATE",
			"R4_PORTB_CREDIT",
			"R4_PORTC_PARMS",
			"R4_PORTC_LLP_CONFIG",
			"R4_PORTC_LLP_TEST_CTL",
			"R4_PORTC_LLP_ERR1",
			"R4_PORTC_LLP_ERR2",
			"R4_PORTC_CREDIT_STATE",
			"R4_PORTC_CREDIT",
			"R4_PORTD_PARMS",
			"R4_PORTD_LLP_CONFIG",
			"R4_PORTD_LLP_TEST_CTL",
			"R4_PORTD_LLP_ERR1",
			"R4_PORTD_LLP_ERR2",
			"R4_PORTD_CREDIT_STATE",
			"R4_PORTD_CREDIT",
			"R4_PORTE_PARMS",
			"R4_PORTE_LLP_CONFIG",
			"R4_PORTE_LLP_TEST_CTL",
			"R4_PORTE_LLP_ERR1",
			"R4_PORTE_LLP_ERR2",
			"R4_PORTE_CREDIT_STATE",
			"R4_PORTE_CREDIT",
			"R4_PORTF_PARMS",
			"R4_PORTF_LLP_CONFIG",
			"R4_PORTF_LLP_TEST_CTL",
			"R4_PORTF_LLP_ERR1",
			"R4_PORTF_LLP_ERR2",
			"R4_PORTF_CREDIT_STATE",
			"R4_PORTF_CREDIT",
			"R4_PORTG_PARMS",
			"R4_PORTG_LLP_CONFIG",
			"R4_PORTG_LLP_TEST_CTL",
			"R4_PORTG_LLP_ERR1",
			"R4_PORTG_LLP_ERR2",
			"R4_PORTG_CREDIT_STATE",
			"R4_PORTG_CREDIT",
			"R4_PORTH_PARMS",
			"R4_PORTH_LLP_CONFIG",
			"R4_PORTH_LLP_TEST_CTL",
			"R4_PORTH_LLP_ERR1",
			"R4_PORTH_LLP_ERR2",
			"R4_PORTH_CREDIT_STATE",
			"R4_PORTH_CREDIT",

			"R4_PORTA_RST_MASK",
			"R4_PORTB_RST_MASK",
			"R4_PORTC_RST_MASK",
                        "R4_PORTD_RST_MASK",
			"R4_PORTE_RST_MASK",
			"R4_PORTF_RST_MASK",
			"R4_PORTG_RST_MASK",
			"R4_PORTH_RST_MASK"
		);

	  if ($fast_mode == 0) {
	     @nl4r_mmr_names = (
			 @nl4r_mmr_names,
			"R4_MCAST_GATHER_CONFIG",
			"R4_MCAST_DEST_VEC_0",
			"R4_MCAST_DEST_VEC_1",
			"R4_MCAST_DEST_VEC_2",
			"R4_MCAST_DEST_VEC_3",
			"R4_MCAST_TRANS_TABLE_0",
			"R4_MCAST_TRANS_TABLE_1",
			"R4_MCAST_TRANS_TABLE_2",
			"R4_MCAST_TRANS_TABLE_3",
			"R4_MCAST_TRANS_TABLE_4",
			"R4_MCAST_TRANS_TABLE_5",
			"R4_MCAST_TRANS_TABLE_6",
			"R4_MCAST_TRANS_TABLE_7",
			"R4_MCAST_TRANS_TABLE_8",
			"R4_MCAST_TRANS_TABLE_9",
			"R4_MCAST_TRANS_TABLE_10",
			"R4_MCAST_TRANS_TABLE_11",
			"R4_MCAST_TRANS_TABLE_12",
			"R4_MCAST_TRANS_TABLE_13",
			"R4_MCAST_TRANS_TABLE_14",
			"R4_MCAST_TRANS_TABLE_15",
			"R4_MCAST_REP_GATHER_BUF",		# 16 deep
			"R4_MCAST_DEST_TABLE"			# 16 deep
		);
	}


    if ($rperf_flag == 1)  {
  	@nl4r_mmr_names = (     @nl4r_mmr_names, 
				"R4_PORT_PERF_WIN",

				"R4_PORTA_PERF_CNTR0",
				"R4_PORTA_PERF_CNTR1",
				"R4_PORTA_PERF_CNTR2",
				"R4_PORTA_PERF_CNTR3",
				"R4_PORTA_PERF_CNTR4",
				"R4_PORTA_PERF_CNTR5",
				"R4_PORTA_PERF_CNTR6",
				"R4_PORTA_PERF_CNTR7",
				"R4_PORTA_PERF_CNTR8",
				"R4_PORTA_PERF_CNTR9",
				"R4_PORTA_PERF_CNTR10",
				"R4_PORTA_PERF_CNTR11",
				"R4_PORTA_PERF_CNTR12",
				"R4_PORTA_PERF_CNTR13",
				"R4_PORTA_PERF_CNTR14",
				"R4_PORTA_PERF_CNTR15",
				"R4_PORTA_PERF_CNTR16",
				"R4_PORTA_PERF_CNTR17",
				"R4_PORTA_PERF_CNTR18",
				"R4_PORTA_PERF_CNTR19",
				"R4_PORTA_PERF_CNTR20",
				"R4_PORTA_PERF_CNTR21",
				"R4_PORTA_PERF_CNTR22",
				"R4_PORTA_PERF_CNTR23",
				"R4_PORTA_PERF_CNTR24",
				"R4_PORTA_PERF_CNTR25",
				"R4_PORTA_PERF_CNTR26",
				"R4_PORTA_PERF_CNTR27",
				"R4_PORTA_PERF_CNTR28",
				"R4_PORTA_PERF_CNTR29",
				"R4_PORTA_PERF_CNTR30",
				"R4_PORTA_PERF_CNTR31",
				"R4_PORTA_PERF_CNTR32",
				"R4_PORTA_PERF_CNTR33",
				"R4_PORTA_PERF_CNTR34",
				"R4_PORTA_PERF_CNTR35",
				"R4_PORTA_PERF_CNTR36",

				"R4_PORTB_PERF_CNTR0",
				"R4_PORTB_PERF_CNTR1",
				"R4_PORTB_PERF_CNTR2",
				"R4_PORTB_PERF_CNTR3",
				"R4_PORTB_PERF_CNTR4",
				"R4_PORTB_PERF_CNTR5",
				"R4_PORTB_PERF_CNTR6",
				"R4_PORTB_PERF_CNTR7",
				"R4_PORTB_PERF_CNTR8",
				"R4_PORTB_PERF_CNTR9",
				"R4_PORTB_PERF_CNTR10",
				"R4_PORTB_PERF_CNTR11",
				"R4_PORTB_PERF_CNTR12",
				"R4_PORTB_PERF_CNTR13",
				"R4_PORTB_PERF_CNTR14",
				"R4_PORTB_PERF_CNTR15",
				"R4_PORTB_PERF_CNTR16",
				"R4_PORTB_PERF_CNTR17",
				"R4_PORTB_PERF_CNTR18",
				"R4_PORTB_PERF_CNTR19",
				"R4_PORTB_PERF_CNTR20",
				"R4_PORTB_PERF_CNTR21",
				"R4_PORTB_PERF_CNTR22",
				"R4_PORTB_PERF_CNTR23",
				"R4_PORTB_PERF_CNTR24",
				"R4_PORTB_PERF_CNTR25",
				"R4_PORTB_PERF_CNTR26",
				"R4_PORTB_PERF_CNTR27",
				"R4_PORTB_PERF_CNTR28",
				"R4_PORTB_PERF_CNTR29",
				"R4_PORTB_PERF_CNTR30",
				"R4_PORTB_PERF_CNTR31",
				"R4_PORTB_PERF_CNTR32",
				"R4_PORTB_PERF_CNTR33",
				"R4_PORTB_PERF_CNTR34",
				"R4_PORTB_PERF_CNTR35",
				"R4_PORTB_PERF_CNTR36",

				"R4_PORTC_PERF_CNTR0",
				"R4_PORTC_PERF_CNTR1",
				"R4_PORTC_PERF_CNTR2",
				"R4_PORTC_PERF_CNTR3",
				"R4_PORTC_PERF_CNTR4",
				"R4_PORTC_PERF_CNTR5",
				"R4_PORTC_PERF_CNTR6",
				"R4_PORTC_PERF_CNTR7",
				"R4_PORTC_PERF_CNTR8",
				"R4_PORTC_PERF_CNTR9",
				"R4_PORTC_PERF_CNTR10",
				"R4_PORTC_PERF_CNTR11",
				"R4_PORTC_PERF_CNTR12",
				"R4_PORTC_PERF_CNTR13",
				"R4_PORTC_PERF_CNTR14",
				"R4_PORTC_PERF_CNTR15",
				"R4_PORTC_PERF_CNTR16",
				"R4_PORTC_PERF_CNTR17",
				"R4_PORTC_PERF_CNTR18",
				"R4_PORTC_PERF_CNTR19",
				"R4_PORTC_PERF_CNTR20",
				"R4_PORTC_PERF_CNTR21",
				"R4_PORTC_PERF_CNTR22",
				"R4_PORTC_PERF_CNTR23",
				"R4_PORTC_PERF_CNTR24",
				"R4_PORTC_PERF_CNTR25",
				"R4_PORTC_PERF_CNTR26",
				"R4_PORTC_PERF_CNTR27",
				"R4_PORTC_PERF_CNTR28",
				"R4_PORTC_PERF_CNTR29",
				"R4_PORTC_PERF_CNTR30",
				"R4_PORTC_PERF_CNTR31",
				"R4_PORTC_PERF_CNTR32",
				"R4_PORTC_PERF_CNTR33",
				"R4_PORTC_PERF_CNTR34",
				"R4_PORTC_PERF_CNTR35",
				"R4_PORTC_PERF_CNTR36",

				"R4_PORTD_PERF_CNTR0",
				"R4_PORTD_PERF_CNTR1",
				"R4_PORTD_PERF_CNTR2",
				"R4_PORTD_PERF_CNTR3",
				"R4_PORTD_PERF_CNTR4",
				"R4_PORTD_PERF_CNTR5",
				"R4_PORTD_PERF_CNTR6",
				"R4_PORTD_PERF_CNTR7",
				"R4_PORTD_PERF_CNTR8",
				"R4_PORTD_PERF_CNTR9",
				"R4_PORTD_PERF_CNTR10",
				"R4_PORTD_PERF_CNTR11",
				"R4_PORTD_PERF_CNTR12",
				"R4_PORTD_PERF_CNTR13",
				"R4_PORTD_PERF_CNTR14",
				"R4_PORTD_PERF_CNTR15",
				"R4_PORTD_PERF_CNTR16",
				"R4_PORTD_PERF_CNTR17",
				"R4_PORTD_PERF_CNTR18",
				"R4_PORTD_PERF_CNTR19",
				"R4_PORTD_PERF_CNTR20",
				"R4_PORTD_PERF_CNTR21",
				"R4_PORTD_PERF_CNTR22",
				"R4_PORTD_PERF_CNTR23",
				"R4_PORTD_PERF_CNTR24",
				"R4_PORTD_PERF_CNTR25",
				"R4_PORTD_PERF_CNTR26",
				"R4_PORTD_PERF_CNTR27",
				"R4_PORTD_PERF_CNTR28",
				"R4_PORTD_PERF_CNTR29",
				"R4_PORTD_PERF_CNTR30",
				"R4_PORTD_PERF_CNTR31",
				"R4_PORTD_PERF_CNTR32",
				"R4_PORTD_PERF_CNTR33",
				"R4_PORTD_PERF_CNTR34",
				"R4_PORTD_PERF_CNTR35",
				"R4_PORTD_PERF_CNTR36",

				"R4_PORTE_PERF_CNTR0",
				"R4_PORTE_PERF_CNTR1",
				"R4_PORTE_PERF_CNTR2",
				"R4_PORTE_PERF_CNTR3",
				"R4_PORTE_PERF_CNTR4",
				"R4_PORTE_PERF_CNTR5",
				"R4_PORTE_PERF_CNTR6",
				"R4_PORTE_PERF_CNTR7",
				"R4_PORTE_PERF_CNTR8",
				"R4_PORTE_PERF_CNTR9",
				"R4_PORTE_PERF_CNTR10",
				"R4_PORTE_PERF_CNTR11",
				"R4_PORTE_PERF_CNTR12",
				"R4_PORTE_PERF_CNTR13",
				"R4_PORTE_PERF_CNTR14",
				"R4_PORTE_PERF_CNTR15",
				"R4_PORTE_PERF_CNTR16",
				"R4_PORTE_PERF_CNTR17",
				"R4_PORTE_PERF_CNTR18",
				"R4_PORTE_PERF_CNTR19",
				"R4_PORTE_PERF_CNTR20",
				"R4_PORTE_PERF_CNTR21",
				"R4_PORTE_PERF_CNTR22",
				"R4_PORTE_PERF_CNTR23",
				"R4_PORTE_PERF_CNTR24",
				"R4_PORTE_PERF_CNTR25",
				"R4_PORTE_PERF_CNTR26",
				"R4_PORTE_PERF_CNTR27",
				"R4_PORTE_PERF_CNTR28",
				"R4_PORTE_PERF_CNTR29",
				"R4_PORTE_PERF_CNTR30",
				"R4_PORTE_PERF_CNTR31",
				"R4_PORTE_PERF_CNTR32",
				"R4_PORTE_PERF_CNTR33",
				"R4_PORTE_PERF_CNTR34",
				"R4_PORTE_PERF_CNTR35",
				"R4_PORTE_PERF_CNTR36",

				"R4_PORTF_PERF_CNTR0",
				"R4_PORTF_PERF_CNTR1",
				"R4_PORTF_PERF_CNTR2",
				"R4_PORTF_PERF_CNTR3",
				"R4_PORTF_PERF_CNTR4",
				"R4_PORTF_PERF_CNTR5",
				"R4_PORTF_PERF_CNTR6",
				"R4_PORTF_PERF_CNTR7",
				"R4_PORTF_PERF_CNTR8",
				"R4_PORTF_PERF_CNTR9",
				"R4_PORTF_PERF_CNTR10",
				"R4_PORTF_PERF_CNTR11",
				"R4_PORTF_PERF_CNTR12",
				"R4_PORTF_PERF_CNTR13",
				"R4_PORTF_PERF_CNTR14",
				"R4_PORTF_PERF_CNTR15",
				"R4_PORTF_PERF_CNTR16",
				"R4_PORTF_PERF_CNTR17",
				"R4_PORTF_PERF_CNTR18",
				"R4_PORTF_PERF_CNTR19",
				"R4_PORTF_PERF_CNTR20",
				"R4_PORTF_PERF_CNTR21",
				"R4_PORTF_PERF_CNTR22",
				"R4_PORTF_PERF_CNTR23",
				"R4_PORTF_PERF_CNTR24",
				"R4_PORTF_PERF_CNTR25",
				"R4_PORTF_PERF_CNTR26",
				"R4_PORTF_PERF_CNTR27",
				"R4_PORTF_PERF_CNTR28",
				"R4_PORTF_PERF_CNTR29",
				"R4_PORTF_PERF_CNTR30",
				"R4_PORTF_PERF_CNTR31",
				"R4_PORTF_PERF_CNTR32",
				"R4_PORTF_PERF_CNTR33",
				"R4_PORTF_PERF_CNTR34",
				"R4_PORTF_PERF_CNTR35",
				"R4_PORTF_PERF_CNTR36",

				"R4_PORTG_PERF_CNTR0",
				"R4_PORTG_PERF_CNTR1",
				"R4_PORTG_PERF_CNTR2",
				"R4_PORTG_PERF_CNTR3",
				"R4_PORTG_PERF_CNTR4",
				"R4_PORTG_PERF_CNTR5",
				"R4_PORTG_PERF_CNTR6",
				"R4_PORTG_PERF_CNTR7",
				"R4_PORTG_PERF_CNTR8",
				"R4_PORTG_PERF_CNTR9",
				"R4_PORTG_PERF_CNTR10",
				"R4_PORTG_PERF_CNTR11",
				"R4_PORTG_PERF_CNTR12",
				"R4_PORTG_PERF_CNTR13",
				"R4_PORTG_PERF_CNTR14",
				"R4_PORTG_PERF_CNTR15",
				"R4_PORTG_PERF_CNTR16",
				"R4_PORTG_PERF_CNTR17",
				"R4_PORTG_PERF_CNTR18",
				"R4_PORTG_PERF_CNTR19",
				"R4_PORTG_PERF_CNTR20",
				"R4_PORTG_PERF_CNTR21",
				"R4_PORTG_PERF_CNTR22",
				"R4_PORTG_PERF_CNTR23",
				"R4_PORTG_PERF_CNTR24",
				"R4_PORTG_PERF_CNTR25",
				"R4_PORTG_PERF_CNTR26",
				"R4_PORTG_PERF_CNTR27",
				"R4_PORTG_PERF_CNTR28",
				"R4_PORTG_PERF_CNTR29",
				"R4_PORTG_PERF_CNTR30",
				"R4_PORTG_PERF_CNTR31",
				"R4_PORTG_PERF_CNTR32",
				"R4_PORTG_PERF_CNTR33",
				"R4_PORTG_PERF_CNTR34",
				"R4_PORTG_PERF_CNTR35",
				"R4_PORTG_PERF_CNTR36",

				"R4_PORTH_PERF_CNTR0",
				"R4_PORTH_PERF_CNTR1",
				"R4_PORTH_PERF_CNTR2",
				"R4_PORTH_PERF_CNTR3",
				"R4_PORTH_PERF_CNTR4",
				"R4_PORTH_PERF_CNTR5",
				"R4_PORTH_PERF_CNTR6",
				"R4_PORTH_PERF_CNTR7",
				"R4_PORTH_PERF_CNTR8",
				"R4_PORTH_PERF_CNTR9",
				"R4_PORTH_PERF_CNTR10",
				"R4_PORTH_PERF_CNTR11",
				"R4_PORTH_PERF_CNTR12",
				"R4_PORTH_PERF_CNTR13",
				"R4_PORTH_PERF_CNTR14",
				"R4_PORTH_PERF_CNTR15",
				"R4_PORTH_PERF_CNTR16",
				"R4_PORTH_PERF_CNTR17",
				"R4_PORTH_PERF_CNTR18",
				"R4_PORTH_PERF_CNTR19",
				"R4_PORTH_PERF_CNTR20",
				"R4_PORTH_PERF_CNTR21",
				"R4_PORTH_PERF_CNTR22",
				"R4_PORTH_PERF_CNTR23",
				"R4_PORTH_PERF_CNTR24",
				"R4_PORTH_PERF_CNTR25",
				"R4_PORTH_PERF_CNTR26",
				"R4_PORTH_PERF_CNTR27",
				"R4_PORTH_PERF_CNTR28",
				"R4_PORTH_PERF_CNTR29",
				"R4_PORTH_PERF_CNTR30",
				"R4_PORTH_PERF_CNTR31",
				"R4_PORTH_PERF_CNTR32",
				"R4_PORTH_PERF_CNTR33",
				"R4_PORTH_PERF_CNTR34",
				"R4_PORTH_PERF_CNTR35",
				"R4_PORTH_PERF_CNTR36",

				"R4_PERFORMANCE_COUNTER0",
				"R4_PERFORMANCE_COUNTER1",
				"R4_PERFORMANCE_COUNTER2",
				"R4_PERFORMANCE_COUNTER3",
				"R4_PERFORMANCE_COUNTER4",
				"R4_PERFORMANCE_COUNTER5",
				"R4_PERFORMANCE_COUNTER6",
				"R4_PERFORMANCE_COUNTER7"  );

    }
  }
  @nl4r_reg_list = ();

#--------------------------------------------
#  Create the register structure for each MMR
#--------------------------------------------
  for ($i = 0; $i <= $#nl4r_mmr_names; $i++) {
    $name    = $nl4r_mmr_names[$i];
    $reg_ptr = reg_create_mmr_inst("nl4rtr_01", $name);
    if ( !reg_is_valid($reg_ptr) )
    {  $emess = "Unable to create register instance: $name.\n";
      dsp_error_and_exit($emess);
    }
    $nl4r_reg_list[$i] = $reg_ptr;
  }
 
  $nl4r_cnt = 0;
  $nl4r_read_in_progress = 1;

  while (defined $nl4r_target_list[$nl4r_cnt]) {
    $SSCOPE_ERR = 0;
    mmr_target($SELECT, $nl4r_target_list[$nl4r_cnt]->{MODULE},
                      $nl4r_target_list[$nl4r_cnt]->{NODE},
                      $nl4r_target_list[$nl4r_cnt]->{CHIP});

    if ($SSCOPE_ERR == 1)  {
      printf "\n mmr_target\(\) failed.  Aborting mmr read for %s of %s\n\n",
        $nl4r_target_list[$nl4r_cnt]->{NODE}, $nl4r_target_list[$nl4r_cnt]->{MODULE};
      $SSCOPE_ERR = 0;
      $nl4r_cnt++;
      next;
    }


    $nl4r_target_list[$nl4r_cnt]->{XFR_ERR_CNT} = 0;


#--------------------------------------------
#  Read the MMRs (use L1 list operation)
#--------------------------------------------
    my $xfr_complete = 0;
    my $xfr_f_cnt = 0;
    while (($xfr_complete == 0) && ($xfr_f_cnt < 3) && ($nl4r_target_list[$nl4r_cnt]->{XFR_ERR_CNT} < 6))  {
      $SSCOPE_ERR = 0;
      reg_list_mmr_read(\@nl4r_reg_list);
      if ($SSCOPE_ERR == 1)  {
        $nl4r_target_list[$nl4r_cnt]->{XFR_ERR_CNT}++;
        $xfr_f_cnt++;
        $SSCOPE_ERR = 0;
      }
      else {
        $xfr_complete = 1;
      }
    }

    parse_nl4r_mmr_list(\@nl4r_reg_list);

    get_nl4r_arb_tp($nl4r_cnt, $port_alive_mask);
 
    if ($rperf_flag == 1)  {
        printf ("\n Sending %d ports worth of %d values to nl4r_perf_report\(\)\n", $prt_cnt, $perf_reg_cnt) if ($verbose_mode == 1);
	nl4r_perf_report( $nl4r_target_list[$nl4r_cnt]->{NODE}, 
			  $nl4r_target_list[$nl4r_cnt]->{MODULE},
			  $lb_clk_value, $nasid_value,
			  \@r4_perf_array);
    }

    if ($dump_luts == 1)  {  dump_valid_lut('NL4R', $nasid_value);  }

    $nl4r_cnt++;
  }

  $nl4r_read_in_progress = 0;

}

#-------------------------------------------------------------------------------------
#
#  This function will display the contents of each register and report any errors.
#
#  Call this function from read_nl4r_mmrs() for reading from actual hardware or
#  parse_mmr_data() when reading mmr info from a file.
#  An array of hashes should be supplied that contains the register info.
#
#-------------------------------------------------------------------------------------

sub parse_nl4r_mmr_list {

    my $nl4r_reg_list_ref = $_[0];
    my @nl4r_reg_list = ();
    @nl4r_reg_list = @$nl4r_reg_list_ref;
 
    my $rtr_node = $nl4r_target_list[$nl4r_cnt]->{NODE};
    $rtr_node =~ s/ROUTER/RTR/;
    my $chip_loc = $nl4r_target_list[$nl4r_cnt]->{CHIP};
    $chip_loc =~ s/RTR//;
    my $rtr_loc = "$rtr_node" . ":" . "$chip_loc";
    $current_location = sprintf("%s, %s", $nl4r_target_list[$nl4r_cnt]->{MODULE}, $rtr_loc);
    my $vortix_flag = 0;
    my $tollhouse_flag = 0;	# 191010 or 1955 chassis
    my $chassis_type = "";

    if ($nl4r_target_list[$nl4r_cnt]->{MODULE} =~ /c/) {
      if (($rtr_loc =~ /R_TOP/) || ($rtr_loc =~ /R_BOT/)) {
	$vortix_flag = 1;
      }
      else {
	$tollhouse_flag = 1;
	if ($nl4r_target_list[$nl4r_cnt]->{CHASSIS} == 2) {			# 191010
	  $chassis_type = "191010";
	  if ($rtr_loc =~ /RTR_L:0/)    { *blade_list = \@BLADE_LIST_0L; }
	  elsif ($rtr_loc =~ /RTR_L:1/) { *blade_list = \@BLADE_LIST_1L; }
	  elsif ($rtr_loc =~ /RTR_R:0/) { *blade_list = \@BLADE_LIST_0R; }
	  elsif ($rtr_loc =~ /RTR_R:1/) { *blade_list = \@BLADE_LIST_1R; }
        }
	elsif ($nl4r_target_list[$nl4r_cnt]->{CHASSIS} == 1)  {			# 1955
	  $chassis_type = "1955";
	  if ($rtr_loc =~ /RTR_L:0/)    { *blade_list = \@BLADE_LIST_0L; }
	  elsif ($rtr_loc =~ /RTR_R:0/) { *blade_list = \@BLADE_LIST_1R; }
	}
      }
    }

    $prefix		= "0x0";
    $nasid_value 	= 0;
    $mmr_display = "";
    my $mmr_data_invalid = 0;
    my $bad_read_cnt = 0;

    my $prom_init_complete = 0;
    my $get_lb_error_detail = 0;
    my $get_prta_error_detail = 0;
    my $get_prtb_error_detail = 0;
    my $get_prtc_error_detail = 0;
    my $get_prtd_error_detail = 0;
    my $get_prte_error_detail = 0;
    my $get_prtf_error_detail = 0;
    my $get_prtg_error_detail = 0;
    my $get_prth_error_detail = 0;
    my $check_prta_credits    = 0;
    my $check_prtb_credits    = 0;
    my $check_prtc_credits    = 0;
    my $check_prtd_credits    = 0;
    my $check_prte_credits    = 0;
    my $check_prtf_credits    = 0;
    my $check_prtg_credits    = 0;
    my $check_prth_credits    = 0;
    my $prta_fru_added        = 0;
    my $prtb_fru_added        = 0;
    my $prtc_fru_added        = 0;
    my $prtd_fru_added        = 0;
    my $prte_fru_added        = 0;
    my $prtf_fru_added        = 0;
    my $prtg_fru_added        = 0;
    my $prth_fru_added        = 0;
    my $prta_rl_err_val       = 0;
    my $prtb_rl_err_val       = 0;
    my $prtc_rl_err_val       = 0;
    my $prtd_rl_err_val       = 0;
    my $prte_rl_err_val       = 0;
    my $prtf_rl_err_val       = 0;
    my $prtg_rl_err_val       = 0;
    my $prth_rl_err_val       = 0;
    my $valid_port_err_mask   = 0x63ffffffffff;		# mask out sn, cb, retry count != 0 errors
    my $pkt_flow_err_mask     = 0x0000fff0;		# mask of packet flow type errors found in first error
    my $lut_err_mask          = 0x3fff0000;		# mask of routing or lut type errors
    my $port_first_err_mask   = 0;			# indicates which port is found in the first error

    my $bcval		      = 0x200000000000;		# CORESENTBADCRC_CNT_NON_ZERO value in First error reg
							# note, with this type of error, the captured error detail
							# won't reflect the damaged header, since that has already
							# been release by the time the error is detected.
    my @r4_perf_array 	= ();
    my $prt_cnt 	= 0;
    my $perf_reg_cnt 	= 0;
    my $perf_idx 	= 0;
    my $lb_clk_value 	= 0;
    my $rtr_hdr		= "";
    my $nl4r_type	= "Undefined";		#  0 for Regular, 1 for Meta
    my $rtr_output	= "";
    my $link_up_cnt	= 0;
    my $port_alive_mask	= 0;
    my $max_cred	= 0x24;
    my $min_cred	= 0x06;
    my $i = 0;

    for ($i = 0; $i <= $#nl4r_reg_list; $i++)  {       # parse mmrs that were read 
      my $notify = "";
      my $mmr_printed = 0;
      $reg_ptr = $nl4r_reg_list[$i];
      my $mmr_data = reg_get_value($reg_ptr);
      my $mmr_name = reg_name($reg_ptr);
      my $mmr_addr = reg_addr($reg_ptr);

      if ($mmr_data == $mmr_addr)  {
          printf "Suspect bad read:  Address: 0x%X      Value read: 0x%X\n", $mmr_addr, $mmr_data;                    # GEM:  DEBUG
          $bad_read_cnt++;
          $mmr_data_invalid = ($bad_read_cnt > 10) ? 1 : 0;
      }

      if ( $mmr_name =~ /PROTECTION/ )  {
	  $nasid_value = ($bad_read_cnt == 0) ? reg_get_field_value($reg_ptr, 'NODE_ID') : 0xDEAD;
	  $prefix = sprintf ("0x%04X", $nasid_value);
	  $nl4r_target_list[$nl4r_cnt]->{NASID} = $nasid_value;
      }

      if ( $mmr_name =~ /REV_ID/ )  {
	  my $type_val = reg_get_field_value($reg_ptr, 'POSITION');
	  if ($type_val == 0xe )  {
	      $nl4r_type = "MetaRouter";		# Meta
	  }
	  elsif ($type_val == 0xa )  {
	      $nl4r_type = "Level 1 MetaRouter";		# Meta
	  }
	  elsif ($type_val == 0x6 )  {
	      $nl4r_type = "Level 2 MetaRouter";		# Meta
	  }
	  elsif ($type_val == 0x2 )  {
	      $nl4r_type = "Level 3 MetaRouter";		# Meta
	  }
	  elsif ($type_val == 0xc )  {
	      $nl4r_type = "Repeater MetaRouter";		# Meta
	  }
	  elsif ($type_val == 0xf )  {
	      $nl4r_type = "Regular";		# Regular or Ordinary
	  }

	  if ($tollhouse_flag == 1) {
  	    $rtr_hdr = sprintf "\n\nReading NL4R Registers on %s of %s   NASID: 0x%04X   Type: %s   Chassis Type: %s\n\n",
        	$rtr_loc, $nl4r_target_list[$nl4r_cnt]->{MODULE}, $nasid_value, $nl4r_type, $chassis_type;
  	    $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} = sprintf "\n\nError Registers found on %s of %s    NASID: 0x%04X   Type: %s   Chassis Type: %s\n\n",
        	$rtr_loc, $nl4r_target_list[$nl4r_cnt]->{MODULE}, $nasid_value, $nl4r_type, $chassis_type;
  	  }
	  else {
  	    $rtr_hdr = sprintf "\n\nReading NL4R Registers on %s of %s   NASID: 0x%04X   Type: %s\n\n",
        	$rtr_loc, $nl4r_target_list[$nl4r_cnt]->{MODULE}, $nasid_value, $nl4r_type;
  	    $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} = sprintf "\n\nError Registers found on %s of %s    NASID: 0x%04X   Type: %s\n\n",
        	$rtr_loc, $nl4r_target_list[$nl4r_cnt]->{MODULE}, $nasid_value, $nl4r_type;
  	  }

	  my $new_nasid = 1;

		#  Will likely have duplicate NL4R Nasids.

	  if (($shub_version > 2) && ($nl4r_type =~ /Meta/))  {	# ignore duplicate NASID check for Altix systems
  	    foreach $lcl_nasid (@nl4r_nasid_list) {
    	      if ($nasid_value == $lcl_nasid)  {
      	          $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= sprintf "\n   %s: ERROR!! NASID 0x%04X is already defined.\n", $prefix, $nasid_value;
      	          $any_nl4r_error = 1;
      	          $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
      	          $rtr_output .=  "ALERT!! This Nasid value is already defined!!\n\n";
		  $new_nasid = 0;
      	          last;
    	      }
  	    }
  	  }
	  if ($new_nasid == 1)  {
  	    push(@nl4r_nasid_list, $nasid_value);
	  }
	  if (($tollhouse_flag == 1) && ($verbose_mode == 1))  {
	    print "\n\n Blade List for $rtr_loc\n";
	    foreach $b (@blade_list)  {
	      print "$b\n";
	    }
	  }
      }

      if ($mmr_name =~ /R4_SCRATCH0\b/) {
	if ((($mmr_data >> 40) & 1) == 1) {
	  $prom_init_complete = 1;
	}
	else {
	  $rtr_hdr .= "\t The value in R4_SCRATCH0 indicates the error state on this router\n";
	  $rtr_hdr .= "\t has not yet been cleared by Prom.\n";
	  $rtr_hdr .= "\t Any errors, except dynamic LLP credit state, will be ignored.\n\n";
	}
      }

      if ($mmr_name =~ /MCAST_REP_GATHER/)  {
        my $gath_err = 0;
        my $gath_err_info = "";
        if ($prom_init_complete == 1) {
	  ($mmr_display, $gath_err, $gath_err_info) = dump_table_mmr($mmr_name, 'nl4rtr_01', 16, 'ERR');
	}
	else {
	  ($mmr_display, $gath_err, $gath_err_info) = dump_table_mmr($mmr_name, 'nl4rtr_01', 16);
	}
	$mmr_printed = 1;
	if (($gath_err == 1) && ($shub_version == 2))  {
	  $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= $gath_err_info;
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $global_err_val |= $NL4R_HW_ERR;
	}
      }

      if ($mmr_name =~ /MCAST_DEST_TABLE/) {
	($mmr_display, $scratch1, $scratch2) = dump_table_mmr($mmr_name, 'nl4rtr_01', 16);
	$mmr_printed = 1;
      }

      if ($prom_init_complete == 1)  {		#  need to qualify any errors before flagging them

        if (($mmr_name =~ /R4_ASIC_FIRST_ERROR\b/ ) && ($mmr_data != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
          $notify = "  <<<<<<<";
	  $port_first_err_mask = $mmr_data & 0xff;
        }

        if (($mmr_name =~ /TIMESTAMP/ ) && ($mmr_data != 0) && ($nl4r_target_list[$nl4r_cnt]->{ERROR} == 1)) {
	  $notify = "  <<<<<<<";
	  $time_list[$t_stamps]->{NASID} = $nasid_value;
	  $time_list[$t_stamps]->{REG} = $mmr_name;
	  $time_list[$t_stamps]->{TIME} = (reg_get_field_value($reg_ptr, 'RTC_VALUE')) << 8;  # must adjust the raw RTC value
          $time_list[$t_stamps]->{LOCATION} = $current_location;
          $t_stamps++;
        }

        if (($mmr_name =~ /R4_LB_FIRST_ERROR/ ) && ($mmr_data != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $get_lb_error_detail = 1;
          $notify = "  <<<<<<<";
	  $fru_list[$fru_count]->{NASID}    = $nasid_value;
	  $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	  $fru_list[$fru_count]->{LOCATION} = $current_location;
	  $fru_list[$fru_count]->{REASON}   = "NL4R LB Error.  ";
	  printf "\n%s: NL4R LB Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
          $fru_count++;

        }
        if (($get_lb_error_detail == 1) && ($mmr_name =~ /R4_LB_ERROR_DETAIL_/)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'all', $prefix);
	  $global_err_val |= $NL4R_HW_ERR;
	  if ($mmr_name =~ /R4_LB_ERROR_DETAIL_1/)  {
	    my $cmd = reg_get_field_value($reg_ptr, 'COMMAND');
	    my $addr = (reg_get_field_value($reg_ptr, 'ADDRESS')) << 3;
	    $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= sprintf("\n    Actual address: 0x%014llX    SN2NET command type: %s\n", $addr, $sn2net_command_list[$cmd]); 
	  }
          $notify = "  <<<<<<<";
        }
      }

      if (($mmr_name =~ /LLP_STATE/) && (($mmr_data & 0xf) == 0xe))  {
	if ($mmr_name =~ /PORTA/)  {  $check_prta_credits = 1;  $port_alive_mask += 0x1; }
	if ($mmr_name =~ /PORTB/)  {  $check_prtb_credits = 1;  $port_alive_mask += 0x2; }
	if ($mmr_name =~ /PORTC/)  {  $check_prtc_credits = 1;  $port_alive_mask += 0x4; }
	if ($mmr_name =~ /PORTD/)  {  $check_prtd_credits = 1;  $port_alive_mask += 0x8; }
	if ($mmr_name =~ /PORTE/)  {  $check_prte_credits = 1;  $port_alive_mask += 0x10; }
	if ($mmr_name =~ /PORTF/)  {  $check_prtf_credits = 1;  $port_alive_mask += 0x20; }
	if ($mmr_name =~ /PORTG/)  {  $check_prtg_credits = 1;  $port_alive_mask += 0x40; }
	if ($mmr_name =~ /PORTH/)  {  $check_prth_credits = 1;  $port_alive_mask += 0x80; }
 	$link_up_cnt++;
      }

	#  Check if any internal IP57 to Router connections aren't initialized in a Vortix Brick

      if (($mmr_name =~ /LLP_STATE/) && ($vortix_flag == 1) && (($mmr_data & 0xf) != 0xe) 
	 && (($mmr_name =~ /PORTB/) || ($mmr_name =~ /PORTC/)
	    || ($mmr_name =~ /PORTD/) || ($mmr_name =~ /PORTE/)) ) {
	  my $port_name = $mmr_name;
	  $port_name =~ s/R4_//;
	  $port_name =~ s/_LLP_STATE//;
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
          $notify = "  <<<<<<<";
	  $fru_list[$fru_count]->{NASID}    = $nasid_value;
	  $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	  $fru_list[$fru_count]->{LOCATION} = $current_location;
	  $fru_list[$fru_count]->{REASON}   = "Internal Shub-RTR $port_name connection on Vortix brick not initialized.  ";
	  printf "\n%s: NL4R Vortix %s Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $port_name, $nasid_value, $fru_count;
          $fru_count++;
      }
	#  Check if any internal 191010 RTR to RTR connections aren't initialized

      if (($mmr_name =~ /PORTA_LLP_STATE/) && ($tollhouse_flag == 1) && (($mmr_data & 0xf) != 0xe) 
	 && ($nl4r_target_list[$nl4r_cnt]->{CHASSIS} == 2))  {		# generate this error only on 191010 chassis
	  my $port_name = 'Port A';
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
          $notify = "  <<<<<<<";
	  $fru_list[$fru_count]->{NASID}    = $nasid_value;
	  $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	  $fru_list[$fru_count]->{LOCATION} = $current_location;
	  $fru_list[$fru_count]->{REASON}   = "Internal RTR-RTR $port_name connection on Dual Router Blade not initialized.  ";
	  printf "\n%s: NL4R 191010 %s Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $port_name, $nasid_value, $fru_count;
          $fru_count++;
      }

      if (($mmr_name =~ /R4_PORTA_CREDIT_STATE/ ) && ($check_prta_credits)) {
        if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
        	$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($mmr_name =~ /R4_PORTB_CREDIT_STATE/ ) && ($check_prtb_credits)) {
        if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
        	$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($mmr_name =~ /R4_PORTC_CREDIT_STATE/ ) && ($check_prtc_credits)) {
        if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
        	$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($mmr_name =~ /R4_PORTD_CREDIT_STATE/ ) && ($check_prtd_credits)) {
        if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
        	$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($mmr_name =~ /R4_PORTE_CREDIT_STATE/ ) && ($check_prte_credits)) {
        if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
        	$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($mmr_name =~ /R4_PORTF_CREDIT_STATE/ ) && ($check_prtf_credits)) {
        if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
        	$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($mmr_name =~ /R4_PORTG_CREDIT_STATE/ ) && ($check_prtg_credits)) {
        if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
           || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
		$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($mmr_name =~ /R4_PORTH_CREDIT_STATE/ ) && ($check_prth_credits)) {
	if (  ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) < $min_cred )
	   || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH0') ) > $max_cred )
	   || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) < $min_cred )
	   || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH1') ) > $max_cred )
	   || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) < $min_cred )
	   || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH2') ) > $max_cred )
	   || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) < $min_cred )
	   || ((reg_get_field_value($reg_ptr, 'CREDITS_VCH3') ) > $max_cred ) )  {
		$nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= lcl_reg_write_value($reg_ptr, 'rt_crd', $prefix);	
		$nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
		$any_nl4r_error = 1;
		$notify = "  <<<<<<<";
		$nl4r_target_list[$nl4r_cnt]->{CREDITS_LOW} = 1;
		$global_err_val |= $NL4R_CRD_ERR;
	}
      }

      if (($prom_init_complete == 1) && ($mmr_data_invalid == 0))  {	#  need to qualify any errors before flagging them

        if ((($mmr_name =~ /R4_PORTA_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTA_ERROR_SUMMARY/ ))
	    && ($check_prta_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  my $add_rtr_fru = 0;
	  my $fru_reason = "";
	  $get_prta_error_detail = 1;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prta_fru_added) != 0)  {
	    $fru_list[$fru_count]->{NASID}    = $nasid_value;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	    $fru_list[$fru_count]->{LOCATION} = $current_location;
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prta_fru_added & 8) != 8)) {
	         $fru_reason = "NL4R PORTA LLP Retry Timeout Error. ";
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prta_fru_added & $bcval) != $bcval)) {
	         $fru_reason = "NL4R PORTA Data Corruption from Core Error. ";
		 $add_rtr_fru = 10;
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prta_fru_added & 1) != 1)) {
	         $fru_reason = "NL4R PORTA Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prta_fru_added & 1) != 1) ) {
	         $fru_reason = "NL4R PORTA detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prta_fru_added & 1) != 1) ) {
	         $fru_reason = "NL4R PORTA detected Packet Routing Error. ";
		 $prta_rl_err_val = $mmr_data & $lut_err_mask;
	    }
	    else {
	        $fru_reason = "NL4R PORTA Error. ";
	    }
	    $fru_list[$fru_count]->{REASON} = $fru_reason;
	    if ((($port_first_err_mask & 0x1) == 0x1) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
	    if ($vortix_flag == 1)  {  
		if ($add_rtr_fru == 0) {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect External Channel. "; 
		}
		else {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
		}
	    }
	    if ($tollhouse_flag == 1)  {  
		if ($add_rtr_fru == 0) {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect on-board RTR to RTR Channel. "; 
		  $add_rtr_fru = 1;
		}
		else {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
		}
	    }
	    printf "\n%s: NL4R PORTA Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
	    $prta_fru_added = $mmr_data;
	    $global_err_val |= $NL4R_HW_ERR;
	    if ($add_rtr_fru == 1) {
		my $other_rtr = $current_location; 
		if ($current_location =~ /:1/) {
		  $other_rtr =~ s/:1/:0/;
		}
		else {
		  $other_rtr =~ s/:0/:1/;
		}
	        $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	        $fru_list[$fru_count]->{LOCATION} = $other_rtr;
	        $fru_list[$fru_count]->{REASON} = "Source router of $current_location $fru_reason ";
            	$fru_count++;
            }
          }
        }
        if (($get_prta_error_detail == 1) && ($mmr_name =~ /R4_PORTA_ERROR_DETAIL/)) {
	  my $dtl_name = "R4_PORTA_ERROR_DETAIL";
	  if ($file_mode == 0)  {
	    $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail ( 
				$nl4r_target_list[$nl4r_cnt]->{MODULE},
				$nl4r_target_list[$nl4r_cnt]->{NODE},
				$prefix, $dtl_name, 32, $prta_rl_err_val);
	  }
          else {
	    my $dtl_cnt = 0;
	    my @dtl_addr = ();
	    my @dtl_data = ();
	    while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {	    # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
	      $dtl_cnt++;
	      if ($dtl_cnt < 32) {
		$i++;
	        $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
	        if ($mmr_name !~ /$dtl_name/) {
		  $i--;
	        }
	      }
	    }
	    $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prta_rl_err_val);
          }
	  $mmr_printed = 1;
        }

        if ((($mmr_name =~ /R4_PORTB_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTB_ERROR_SUMMARY/ ))
	    && ($check_prtb_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  $get_prtb_error_detail = 1;
	  my $bad_rtr = 0;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prtb_fru_added) != 0)  {
	    $fru_list[$fru_count]->{NASID}    = $nasid_value;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	    $fru_list[$fru_count]->{LOCATION} = $current_location;
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prtb_fru_added & 8) != 8)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTB LLP Retry Timeout Error. ";
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prtb_fru_added & $bcval) != $bcval)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTB Data Corruption from Core Error. ";
		$bad_rtr = 1; 
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prtb_fru_added & 1) != 1)) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTB Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prtb_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTB detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prtb_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTB detected Packet Routing Error. ";
		 $prtb_rl_err_val = $mmr_data & $lut_err_mask;
	    }
	    else {
	    	$fru_list[$fru_count]->{REASON}   = "NL4R PORTB Error. ";
	    }
	    if ((($port_first_err_mask & 0x2) == 0x2) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
	    if ($vortix_flag == 1)  {  
                if ($bad_rtr == 0) {
	          $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect Internal \(Opposite side\) Channel. "; 
	        }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
	    }
            if ($tollhouse_flag == 1)  {
                if ($bad_rtr == 0) {
	          $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect Internal RTR to $blade_list[0]. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }
	    printf "\n%s: NL4R PORTB Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
	    $prtb_fru_added = $mmr_data;
	    $global_err_val |= $NL4R_HW_ERR;
          }
        }
        if (($get_prtb_error_detail == 1) && ($mmr_name =~ /R4_PORTB_ERROR_DETAIL/)) {
          my $dtl_name = "R4_PORTB_ERROR_DETAIL";
          if ($file_mode == 0)  {
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail (
                                $nl4r_target_list[$nl4r_cnt]->{MODULE},
                                $nl4r_target_list[$nl4r_cnt]->{NODE},
                                $prefix, $dtl_name, 32, $prtb_rl_err_val);
          }
          else {
            my $dtl_cnt = 0;
            my @dtl_addr = ();
            my @dtl_data = ();
            while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {         # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
              $dtl_cnt++;
              if ($dtl_cnt < 32) {
                $i++;
                $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
                if ($mmr_name !~ /$dtl_name/) {
                  $i--;
                }
              }
            }
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prtb_rl_err_val);
          }
          $mmr_printed = 1;
        }


        if ((($mmr_name =~ /R4_PORTC_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTC_ERROR_SUMMARY/ ))
	    && ($check_prtc_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  $get_prtc_error_detail = 1;
	  my $bad_rtr = 0;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prtc_fru_added) != 0)  {
	    $fru_list[$fru_count]->{NASID}    = $nasid_value;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	    $fru_list[$fru_count]->{LOCATION} = $current_location;
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prtc_fru_added & 8) != 8)) {
                $fru_list[$fru_count]->{REASON}   = "NL4R PORTC LLP Retry Timeout Error. ";
            }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prtc_fru_added & $bcval) != $bcval)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTC Data Corruption from Core Error. ";
		$bad_rtr = 1; 
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prtc_fru_added & 1) != 1)) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTC Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prtc_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTC detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prtc_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTC detected Packet Routing Error. ";
		 $prtc_rl_err_val = $mmr_data & $lut_err_mask;
	    }
            else {
		$fru_list[$fru_count]->{REASON}   = "NL4R PORTC Error. ";
	    }
	    if ((($port_first_err_mask & 0x4) == 0x4) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
            if ($vortix_flag == 1)  {
                if ($bad_rtr == 0) {
	          $fru_list[$fru_count]->{REASON} .=  "\n\t\t\t\t\t\tSuspect Internal \(Same side\) Channel. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }
            if ($tollhouse_flag == 1)  {
                if ($bad_rtr == 0) {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect Internal RTR to $blade_list[1]. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }

	    printf "\n%s: NL4R PORTC Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
            $prtc_fru_added = $mmr_data;
	    $global_err_val |= $NL4R_HW_ERR;
	  }
        }
        if (($get_prtc_error_detail == 1) && ($mmr_name =~ /R4_PORTC_ERROR_DETAIL/)) {
          my $dtl_name = "R4_PORTC_ERROR_DETAIL";
          if ($file_mode == 0)  {
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail (
                                $nl4r_target_list[$nl4r_cnt]->{MODULE},
                                $nl4r_target_list[$nl4r_cnt]->{NODE},
                                $prefix, $dtl_name, 32, $prtc_rl_err_val);
          }
          else {
            my $dtl_cnt = 0;
            my @dtl_addr = ();
            my @dtl_data = ();
            while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {         # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
              $dtl_cnt++;
              if ($dtl_cnt < 32) {
                $i++;
                $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
                if ($mmr_name !~ /$dtl_name/) {
                  $i--;
                }
              }
            }
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prtc_rl_err_val);
          }
          $mmr_printed = 1;
        }


        if ((($mmr_name =~ /R4_PORTD_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTD_ERROR_SUMMARY/ ))
	    && ($check_prtd_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  $get_prtd_error_detail = 1;
	  my $bad_rtr = 0;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prtd_fru_added) != 0)  {
	    $fru_list[$fru_count]->{NASID}    = $nasid_value;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	    $fru_list[$fru_count]->{LOCATION} = $current_location;
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prtd_fru_added & 8) != 8)) {
                $fru_list[$fru_count]->{REASON}   = "NL4R PORTD LLP Retry Timeout Error. ";
            }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prtd_fru_added & $bcval) != $bcval)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTD Data Corruption from Core Error. ";
		$bad_rtr = 1; 
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prtd_fru_added & 1) != 1)) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTD Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prtd_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTD detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prtd_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTD detected Packet Routing Error. ";
		 $prtd_rl_err_val = $mmr_data & $lut_err_mask;
	    }
            else {
		$fru_list[$fru_count]->{REASON}   = "NL4R PORTD Error. ";
	    }
	    if ((($port_first_err_mask & 0x8) == 0x8) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
            if ($vortix_flag == 1)  {
                if ($bad_rtr == 0) {
                  $fru_list[$fru_count]->{REASON} .=  "\n\t\t\t\t\t\tSuspect Internal \(Same side\) Channel. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }
            if ($tollhouse_flag == 1)  {
                if ($bad_rtr == 0) {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect Internal RTR to $blade_list[2]. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }

	    printf "\n%s: NL4R PORTD Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
            $prtd_fru_added = $mmr_data;
	    $global_err_val |= $NL4R_HW_ERR;
	  }
        }
        if (($get_prtd_error_detail == 1) && ($mmr_name =~ /R4_PORTD_ERROR_DETAIL/)) {
          my $dtl_name = "R4_PORTD_ERROR_DETAIL";
          if ($file_mode == 0)  {
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail (
                                $nl4r_target_list[$nl4r_cnt]->{MODULE},
                                $nl4r_target_list[$nl4r_cnt]->{NODE},
                                $prefix, $dtl_name, 32, $prtd_rl_err_val);
          }
          else {
            my $dtl_cnt = 0;
            my @dtl_addr = ();
            my @dtl_data = ();
            while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {         # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
              $dtl_cnt++;
              if ($dtl_cnt < 32) {
                $i++;
                $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
                if ($mmr_name !~ /$dtl_name/) {
                  $i--;
                }
              }
            }
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prtd_rl_err_val);
          }
          $mmr_printed = 1;
        }


        if ((($mmr_name =~ /R4_PORTE_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTE_ERROR_SUMMARY/ ))
	    && ($check_prte_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  $get_prte_error_detail = 1;
	  my $bad_rtr = 0;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prte_fru_added) != 0)  {
            $fru_list[$fru_count]->{NASID}    = $nasid_value;
            $fru_list[$fru_count]->{LOCATION} = $current_location;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prte_fru_added & 8) != 8)) {
                $fru_list[$fru_count]->{REASON}   = "NL4R PORTE LLP Retry Timeout Error. ";
            }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prte_fru_added & $bcval) != $bcval)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTE Data Corruption from Core Error. ";
		$bad_rtr = 1; 
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prte_fru_added & 1) != 1)) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTE Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prte_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTE detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prte_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTE detected Packet Routing Error. ";
		 $prte_rl_err_val = $mmr_data & $lut_err_mask;
	    }
            else {
		$fru_list[$fru_count]->{REASON}   = "NL4R PORTE Error. ";
	    }
	    if ((($port_first_err_mask & 0x10) == 0x10) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
            if ($vortix_flag == 1)  {
                if ($bad_rtr == 0) {
                  $fru_list[$fru_count]->{REASON} .=  "\n\t\t\t\t\t\tSuspect Internal \(Opposite side\) Channel. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }
            if ($tollhouse_flag == 1)  {
                if ($bad_rtr == 0) {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect Internal RTR to $blade_list[3]. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }

	    printf "\n%s: NL4R PORTE Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
            $prte_fru_added = $mmr_data;
	    $global_err_val |= $NL4R_HW_ERR;
	  }
        }
        if (($get_prte_error_detail == 1) && ($mmr_name =~ /R4_PORTE_ERROR_DETAIL/)) {
          my $dtl_name = "R4_PORTE_ERROR_DETAIL";
          if ($file_mode == 0)  {
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail (
                                $nl4r_target_list[$nl4r_cnt]->{MODULE},
                                $nl4r_target_list[$nl4r_cnt]->{NODE},
                                $prefix, $dtl_name, 32, $prte_rl_err_val);
          }
          else {
            my $dtl_cnt = 0;
            my @dtl_addr = ();
            my @dtl_data = ();
            while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {         # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
              $dtl_cnt++;
              if ($dtl_cnt < 32) {
                $i++;
                $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
                if ($mmr_name !~ /$dtl_name/) {
                  $i--;
                }
              }
            }
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prte_rl_err_val);
          }
          $mmr_printed = 1;
        }


        if ((($mmr_name =~ /R4_PORTF_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTF_ERROR_SUMMARY/ ))
	    && ($check_prtf_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  $get_prtf_error_detail = 1;
	  my $bad_rtr = 0;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prtf_fru_added) != 0)  {
            $fru_list[$fru_count]->{NASID}    = $nasid_value;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
            $fru_list[$fru_count]->{LOCATION} = $current_location;
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prtf_fru_added & 8) != 8)) {
                $fru_list[$fru_count]->{REASON}   = "NL4R PORTF LLP Retry Timeout Error. ";
            }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prtf_fru_added & $bcval) != $bcval)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTF Data Corruption from Core Error. ";
		$bad_rtr = 1; 
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prtf_fru_added & 1) != 1)) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTF Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prtf_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTF detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prtf_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTF detected Packet Routing Error. ";
		 $prtf_rl_err_val = $mmr_data & $lut_err_mask;
	    }
            else {
		$fru_list[$fru_count]->{REASON}   = "NL4R PORTF Error. ";
	    }
	    if ((($port_first_err_mask & 0x20) == 0x20) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
            if ($vortix_flag == 1)  {
                if ($bad_rtr == 0) {
                  $fru_list[$fru_count]->{REASON} .=  "\n\t\t\t\t\t\tSuspect External Channel. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }
            if ($tollhouse_flag == 1)  {
                if ($bad_rtr == 0) {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect Internal RTR to $blade_list[4]. ";
                }
                else {
                  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tThis router should be replaced. ";
                }
            }

	    printf "\n%s: NL4R PORTF Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
            $prtf_fru_added = $mmr_data;
	    $global_err_val |= $NL4R_HW_ERR;
	  }
        }
        if (($get_prtf_error_detail == 1) && ($mmr_name =~ /R4_PORTF_ERROR_DETAIL/)) {
          my $dtl_name = "R4_PORTF_ERROR_DETAIL";
          if ($file_mode == 0)  {
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail (
                                $nl4r_target_list[$nl4r_cnt]->{MODULE},
                                $nl4r_target_list[$nl4r_cnt]->{NODE},
                                $prefix, $dtl_name, 32, $prtf_rl_err_val);
          }
          else {
            my $dtl_cnt = 0;
            my @dtl_addr = ();
            my @dtl_data = ();
            while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {         # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
              $dtl_cnt++;
              if ($dtl_cnt < 32) {
                $i++;
                $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
                if ($mmr_name !~ /$dtl_name/) {
                  $i--;
                }
              }
            }
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prtf_rl_err_val);
          }
          $mmr_printed = 1;
        }


        if ((($mmr_name =~ /R4_PORTG_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTG_ERROR_SUMMARY/ ))
	    && ($check_prtg_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  $get_prtg_error_detail = 1;
	  my $bad_rtr = 0;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prtg_fru_added) != 0)  {
	    $fru_list[$fru_count]->{NASID}    = $nasid_value;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	    $fru_list[$fru_count]->{LOCATION} = $current_location;
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prtg_fru_added & 8) != 8)) {
                $fru_list[$fru_count]->{REASON}   = "NL4R PORTG LLP Retry Timeout Error. ";
            }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prtg_fru_added & $bcval) != $bcval)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTG Data Corruption from Core Error. ";
		$bad_rtr = 1; 
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prtg_fru_added & 1) != 1)) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTG Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prtg_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTG detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prtg_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTG detected Packet Routing Error. ";
		 $prtg_rl_err_val = $mmr_data & $lut_err_mask;
	    }
            else {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTG Error. ";
	    }
	    if ((($port_first_err_mask & 0x40) == 0x40) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
	    if (($vortix_flag == 1) || ($tollhouse_flag == 1))  {  
		if ($bad_rtr == 0) {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect External Channel. "; 
		}
		else {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tReplace this router. "; 
		}
	    }
	    printf "\n%s: NL4R PORTG Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
            $prtg_fru_added = $mmr_data;
	    $global_err_val |= $NL4R_HW_ERR;
	  }
        }
        if (($get_prtg_error_detail == 1) && ($mmr_name =~ /R4_PORTG_ERROR_DETAIL/)) {
          my $dtl_name = "R4_PORTG_ERROR_DETAIL";
          if ($file_mode == 0)  {
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail (
                                $nl4r_target_list[$nl4r_cnt]->{MODULE},
                                $nl4r_target_list[$nl4r_cnt]->{NODE},
                                $prefix, $dtl_name, 32, $prtg_rl_err_val);
          }
          else {
            my $dtl_cnt = 0;
            my @dtl_addr = ();
            my @dtl_data = ();
            while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {         # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
              $dtl_cnt++;
              if ($dtl_cnt < 32) {
                $i++;
                $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
                if ($mmr_name !~ /$dtl_name/) {
                  $i--;
                }
              }
            }
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prtg_rl_err_val);
          }
          $mmr_printed = 1;
        }


        if ((($mmr_name =~ /R4_PORTH_FIRST_ERROR/ ) || ($mmr_name =~ /R4_PORTH_ERROR_SUMMARY/ ))
	    && ($check_prth_credits) && (($mmr_data & $valid_port_err_mask) != 0)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	  $any_nl4r_error = 1;
	  $notify = "  <<<<<<<";
	  $get_prth_error_detail = 1;
	  my $bad_rtr = 0;
	  if ((($mmr_data & $valid_port_err_mask) ^ $prth_fru_added) != 0)  {
	    $fru_list[$fru_count]->{NASID}    = $nasid_value;
	    $fru_list[$fru_count]->{DEVICE}   = "NL4R";
	    $fru_list[$fru_count]->{LOCATION} = $current_location;
	    if (((reg_get_field_value($reg_ptr, 'RETRY_TO')) == 1) && (($prth_fru_added & 8) != 8)) {
                $fru_list[$fru_count]->{REASON}   = "NL4R PORTH LLP Retry Timeout Error. ";
            }
	    elsif (((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_CNT_NON_ZERO')) == 1) && (($prth_fru_added & $bcval) != $bcval)) {
	        $fru_list[$fru_count]->{REASON}   = "NL4R PORTH Data Corruption from Core Error. ";
		$bad_rtr = 1; 
	    }
	    elsif (((reg_get_field_value($reg_ptr, 'POWER_NOT_OK')) == 1) && (($prth_fru_added & 1) != 1)) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTH Cable Problem. ";
	    }
	    elsif ( (($mmr_data & $pkt_flow_err_mask) > 0) && (($prth_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTH detected Packet Flow Error. ";
	    }
	    elsif ( (($mmr_data & $lut_err_mask) > 0) && (($prth_fru_added & 1) != 1) ) {
	         $fru_list[$fru_count]->{REASON}   = "NL4R PORTH detected Packet Routing Error. ";
		 $prth_rl_err_val = $mmr_data & $lut_err_mask;
	    }
            else {
		$fru_list[$fru_count]->{REASON}   = "NL4R PORTH Error. ";
	    }
            $prth_fru_added = $mmr_data;
	    if ((($port_first_err_mask & 0x80) == 0x80) && ($mmr_name =~ FIRST)) {
		$fru_list[$fru_count]->{REASON} .= "First Error on this RTR. ";
	    }
	    if (($vortix_flag == 1) || ($tollhouse_flag == 1))  {  
		if ($bad_rtr == 0) {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tSuspect External Channel. "; 
		}
		else {
		  $fru_list[$fru_count]->{REASON} .= "\n\t\t\t\t\t\tReplace this router. "; 
		}
	    }
	    printf "\n%s: NL4R PORTH Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
            $fru_count++;
	    $global_err_val |= $NL4R_HW_ERR;
	  }
        }
        if (($get_prth_error_detail == 1) && ($mmr_name =~ /R4_PORTH_ERROR_DETAIL/)) {
          my $dtl_name = "R4_PORTH_ERROR_DETAIL";
          if ($file_mode == 0)  {
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= nl4r_dump_error_detail (
                                $nl4r_target_list[$nl4r_cnt]->{MODULE},
                                $nl4r_target_list[$nl4r_cnt]->{NODE},
                                $prefix, $dtl_name, 32, $prth_rl_err_val);
          }
          else {
            my $dtl_cnt = 0;
            my @dtl_addr = ();
            my @dtl_data = ();
            while (($mmr_name =~ /$dtl_name/) && ($dtl_cnt < 32)) {         # get all 32 detail regs
              $dtl_data[$dtl_cnt] = reg_get_value($reg_ptr);
              $dtl_addr[$dtl_cnt] = (reg_addr($reg_ptr)) + ($dtl_cnt * 8);
              $dtl_cnt++;
              if ($dtl_cnt < 32) {
                $i++;
                $reg_ptr = $nl4r_reg_list[$i];
                $mmr_name = reg_name($reg_ptr);
                if ($mmr_name !~ /$dtl_name/) {
                  $i--;
                }
              }
            }
            $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= parse_port_err_detail(\@dtl_addr, \@dtl_data, $prefix, $dtl_name, $dtl_cnt, $prth_rl_err_val);
          }
          $mmr_printed = 1;
        }


        if (($mmr_name =~ /LLP_ERR/)  && ($mmr_data != 0)
	   && (  (($mmr_name =~ /PORTA_LLP_ERR/) && ($check_prta_credits == 1))
	      || (($mmr_name =~ /PORTB_LLP_ERR/) && ($check_prtb_credits == 1))
	      || (($mmr_name =~ /PORTC_LLP_ERR/) && ($check_prtc_credits == 1))
	      || (($mmr_name =~ /PORTD_LLP_ERR/) && ($check_prtd_credits == 1))
	      || (($mmr_name =~ /PORTE_LLP_ERR/) && ($check_prte_credits == 1))
	      || (($mmr_name =~ /PORTF_LLP_ERR/) && ($check_prtf_credits == 1))
	      || (($mmr_name =~ /PORTG_LLP_ERR/) && ($check_prtg_credits == 1))
	      || (($mmr_name =~ /PORTH_LLP_ERR/) && ($check_prth_credits == 1)) ))  {

	  my $failure_found = 0;
	  if ($mmr_name =~ /ERR1/) {
	    if ( ((reg_get_field_value($reg_ptr, 'RX_SN_ERR_COUNT')) > 0x8000)
	        || ((reg_get_field_value($reg_ptr, 'RX_CB_ERR_COUNT')) > 0x8000)
	        || ((reg_get_field_value($reg_ptr, 'RETRY_COUNT')) > 0x8000) ) {
		  $failure_found = 1;
	          $global_err_val |= $NL4R_CSR_ERR;
	    }
	  }
	  elsif ($mmr_name =~ /ERR2/) {
	    if ( ((reg_get_field_value($reg_ptr, 'RETRY_TIMEOUT')) == 1)
	        || ((reg_get_field_value($reg_ptr, 'CORESENTBADCRC_COUNT')) > 0)) {
		  $failure_found = 1;
	          $global_err_val |= $NL4R_HW_ERR;
	    }
	  }
	  if ($failure_found)  {
	    $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
	    $any_nl4r_error = 1;
	    $notify = "  <<<<<<<";
	  }
        }

      }		#  end of "qualify" block

      if ($mmr_printed == 0)  {
        if (    ($mmr_name =~ /PERF_CNTR/) 
	   || ($mmr_name =~ /PERFORMANCE/)
	   || ($mmr_name =~ /R4_PORT._ERROR_DETAIL/)
	   || ($mmr_name =~ /SCRATCH/) )  {
          $mmr_display = sprintf "%s: %-22s      @ 0x%09llX = 0x%016llX\n", 
	          $prefix, $mmr_name, reg_addr($reg_ptr), $mmr_data;
        }
        elsif (($mmr_name =~ /CREDIT_STATE/) || ($mmr_name =~ /REV_ID/))  {
    	     # parameter: 'all' for all fields, 'none' for no fields, 'valid' for non-zero fields
          $mmr_display = lcl_reg_write_value($reg_ptr, 'all', $prefix, 0, $notify);
        }
        else  {
          $mmr_display = lcl_reg_write_value($reg_ptr, 'valid', $prefix, 0, $notify);
        }
      }

      $rtr_output .= "$mmr_display";

      if (($notify =~ /<<<<</) && ($mmr_name !~ /CREDIT_STATE/) && ($mmr_name !~ /R4_LB_ERROR_DETAIL/)) {
	  $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= "\n$mmr_display";
      }

      if ( $mmr_name =~ /PERF_CNTR/)  {
	$perf_idx = $perf_reg_cnt % 37;
	$r4_perf_array[$prt_cnt][$perf_idx] = $mmr_data;
	$perf_reg_cnt++;
	if (($perf_reg_cnt % 37) == 0)  {
	  $prt_cnt++;
	}
      }

      if ( $mmr_name =~ /PERF_WIN/)  {
	$lb_clk_value = reg_get_field_value($reg_ptr, 'CNT');
      }
    }

    print "$rtr_hdr";
    print "$rtr_output\n\n\n";

    if (($link_up_cnt == 0) && ($mmr_data_invalid == 0)) {
      $fru_list[$fru_count]->{NASID}    = $nasid_value;
      $fru_list[$fru_count]->{DEVICE}   = "NL4R";
      $fru_list[$fru_count]->{LOCATION} = $current_location;
      $fru_list[$fru_count]->{REASON}   = "NL4R: No ports have initialized properly. ";
      printf "\n%s: NL4R No Links OK Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
      $fru_count++;
    }

    if ($bad_read_cnt > 0) {
      print "\n\nALERT:  Found $bad_read_cnt suspect reads out of $#nl4r_reg_list mmrs.\n\n";  #  GEM  Debug
    }

    if ($bad_read_cnt == $i) {
      $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= "\n\n     Invalid Data \(address values\) found in all MMR's on this node.\n";
      $nl4r_target_list[$nl4r_cnt]->{ERR_SUMM} .= "     Error report for this node will not be generated due to corrupted data from JTAG.\n";
      $any_nl4r_error = 1;
      $nl4r_target_list[$nl4r_cnt]->{ERROR} = 1;
      $global_err_val |= $NL4R_HW_ERR;

      $fru_list[$fru_count]->{NASID} = $nasid_value;
      $fru_list[$fru_count]->{REASON} = "Fatal unknown error causing JTAG reads to fail on this Router.";
      $fru_list[$fru_count]->{DEVICE}   = "SHUB2";
      $fru_list[$fru_count]->{LOCATION} = $current_location;
      printf "\n%s: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
      $fru_count++;
    }
}

####################################################################################
#
#  subroutine:  nl4r_dump_error_detail
#
#  This function is called to dump the group of 32 NL4R port error detail MMR's.
#
#
####################################################################################

sub nl4r_dump_error_detail
{
   printf "\nnl4r_dump_error_detail( @_ )\n" if ($verbose_mode == 1);

   my $module_name = $_[0];  	# Reference to module structure
   my $bname     = $_[1];  	# Board name on module/brick (i.e. "NODE" or "MOATB")
   my $prefix    = "$_[2]";  	# Nasid value string
   my $base_name = $_[3];  	# MMR name
   my $num       = $_[4];  	# Number of MMRs
   my $rl_err_val  = $_[5];  	# router or lut error value
   my ($i, $a, $reg_ptr, @addr, @data);
   my $hdr0;
   my $hdr1;
   my $found_hdr = 0;
   my $caption = "";
   my $tmp_display = "";

   $reg_ptr = reg_create_mmr_inst("nl4rtr_01", $base_name);
   if ( !reg_is_valid($reg_ptr) )
   {  $emess = "Invalid MMR name specified to subroutine nl4r_dump_error_mmrs().";
      $emess .= "\nMMR name specified: \"$base_name\"";
      &$comm_err_fun($emess);
      return;
   }

   if ($verbose_mode == 1)  {
     printf "\n-> Reading list of %d MMRs beginning with: %s\n", $num, $base_name;
   }

   @addr = ();
   for ($i = 0, $a = reg_addr($reg_ptr); $i < $num; $i++, $a += 8)
   {  $addr[$i] = $a;
   }
   my $xfr_complete = 0;
   my $xfr_f_cnt = 0;
   while (($xfr_complete == 0) && ($xfr_f_cnt < 3) && ($nl4r_target_list[$nl4r_cnt]->{XFR_ERR_CNT} < 6))  {
     $SSCOPE_ERR = 0;
     @data = mmr_read_group(\@addr);
     if ($SSCOPE_ERR == 1)  {
        $nl4r_target_list[$nl4r_cnt]->{XFR_ERR_CNT}++;
        $xfr_f_cnt++;
        $SSCOPE_ERR = 0;
     }
     else {
        $xfr_complete = 1;
     }
   }

   my $err_sum = parse_port_err_detail(\@addr, \@data, $prefix, $base_name, $num, $rl_err_val);

   return ($err_sum);
}


sub parse_port_err_detail {

   my $prt_detail_addr_ref = $_[0];
   my $prt_detail_data_ref = $_[1];
   my $prefix    = "$_[2]";  		# Nasid value string
   my $base_name = $_[3];  		# MMR name
   my $count     = $_[4];  		# Number of MMRs
   my $rl_err    = $_[5];  		# router or LUT error value
   $mmr_display = "\n";

   my @detail_addr_list = ();
   @detail_addr_list = @$prt_detail_addr_ref;

   my @detail_data_list = ();
   @detail_data_list = @$prt_detail_data_ref;
   my $err_detail_info = "\n\n";
   my $vc_value = 0xf;

# GEM need to add in a VC error type check, and a LUT error value check,
# then add to fru list if the VC error detail matches the VC indicated in
# the First Error info, AND if there is a LUT error indicated in the DIR
# field

   for (my $dtl_idx = 0; $dtl_idx < $count; $dtl_idx++)
   {  
      $tmp_display = sprintf "%s:  %s[%02u]  @ 0x%09llX = 0x%016llX\n", $prefix, $base_name, $dtl_idx, $detail_addr_list[$dtl_idx], $detail_data_list[$dtl_idx];
      if    (($dtl_idx >= 0) && ($dtl_idx < 4))  {  $caption = "\n  Virtual Channel 0, First Error\n";  $vc_value = 0; }
      elsif (($dtl_idx >= 4) && ($dtl_idx < 8))  {  $caption = "\n  Virtual Channel 0, Second Error\n"; $vc_value = 0; }
      elsif (($dtl_idx >= 8) && ($dtl_idx < 12)) {  $caption = "\n  Virtual Channel 1, First Error\n";  $vc_value = 1; }
      elsif (($dtl_idx >= 12) && ($dtl_idx < 16)){  $caption = "\n  Virtual Channel 1, Second Error\n"; $vc_value = 1; }
      elsif (($dtl_idx >= 16) && ($dtl_idx < 20)){  $caption = "\n  Virtual Channel 2, First Error\n";  $vc_value = 2; }
      elsif (($dtl_idx >= 20) && ($dtl_idx < 24)){  $caption = "\n  Virtual Channel 2, Second Error\n"; $vc_value = 2; }
      elsif (($dtl_idx >= 24) && ($dtl_idx < 28)){  $caption = "\n  Virtual Channel 3, First Error\n";  $vc_value = 3; }
      elsif (($dtl_idx >= 28) && ($dtl_idx < 32)){  $caption = "\n  Virtual Channel 3, Second Error\n"; $vc_value = 3; }
      if ( (($dtl_idx % 4) == 0 ) && ( $detail_data_list[$dtl_idx] != 0 ))  {		#  Header bits 63:0
	$found_hdr = 1;
	$hdr0 = $detail_data_list[$dtl_idx];
	$err_detail_info .= "  $caption";
	$err_detail_info .= "$tmp_display";
      }
      if ((($dtl_idx % 4) == 1 ) && ( $found_hdr == 1 ))  {		#  Header bits 127:64
	$err_detail_info .= "$tmp_display";
	$hdr1 = $detail_data_list[$dtl_idx];
      }
      if ((($dtl_idx % 4) == 2 ) && ( $found_hdr == 1 ))  {		#  Sideband bits 9:0
	$err_detail_info .= "$tmp_display";
	$err_detail_info .= decode_shub2_sn2net_header($hdr0, $hdr1);	#  display header fields

	my $pois_dir = ($hdr0 >> 12) & 0x1;				# dir field indicates packet was poisoned
	my $vc_chk = vc_error_check($rl_err, $vc_value);		# check whether this VC logged errors
	if (($pois_dir == 1) && ($vc_chk == 1)) {
	  my $detail_nasid = (($hdr1 >> 16) & 0x7FF) | ((($hdr1 >> 29) & 0x7) << 11);
	  my $detail_nasid_type = (($detail_nasid & 1) == 1) ? "TIO" : "SHUB";
	  my $detail_chiplet = ($hdr1 >> 27) & 0x3;
	  my $detail_chiplet_type = (($detail_nasid & 1) == 1) ? $tio_chiplet_list[$detail_chiplet] : $shub2_chiplet_list[$detail_chiplet];
          my $port_name = $base_name;
	  $port_name =~ s/R4_PORT//;
	  $port_name =~ s/_ERROR_DETAIL//;
	  $fru_list[$fru_count]->{NASID}    = $detail_nasid;
	  $fru_list[$fru_count]->{DEVICE}   = $detail_nasid_type;
	  $fru_list[$fru_count]->{REASON}   = sprintf "Misrouted packet from %s chiplet on NASID 0x%04X.", $detail_chiplet_type, $detail_nasid;
	  $fru_list[$fru_count]->{REASON}  .= sprintf "\n\t\t\t\t\t\tDetected by Port %s, NL4R at %s.  ",  $port_name, $current_location ;
	  printf "\n%s: NL4R Port %s Error: Added Remote NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $port_name, $detail_nasid, $fru_count;
 	  $fru_count++;
	}
	
	$found_hdr = 0;						#  Clear values
	$hdr0 = 0;
	$hdr1 = 0;
      }
	
      $mmr_display .= $tmp_display;
   }

   $err_detail_info .= "\n\n";

   return($err_detail_info);

}
 


sub get_nl4r_arb_tp  {

  if ($verbose_mode == 1)  { print "\nEntering get_nl4r_arb_tp function.\n"; }

  my $rnode_val = $_[0];
  my $port_mask = $_[1];

  my $menu_count   = 12;
  my $eval_count    = 40;
  my @tp_mmr_names  = ();
  my @tp_clr_mmr_names  = ();
  my @tp_reg_list   = ();
  my @tp_sel_0    = ();
  my @tp_data_0   = ();
  my @tp_sel_1    = ();
  my @tp_data_1   = ();
  my @tp_sel_2    = ();
  my @tp_data_2   = ();
  my @tp_sel_3    = ();
  my @tp_data_3   = ();
  my @tp_sel_4    = ();
  my @tp_data_4   = ();
  my @tp_sel_5    = ();
  my @tp_data_5   = ();
  my @tp_sel_6    = ();
  my @tp_data_6   = ();
  my @tp_sel_7    = ();
  my @tp_data_7   = ();
  my @tp_sel_8    = ();
  my @tp_data_8   = ();
  my @tp_sel_9    = ();
  my @tp_data_9   = ();
  my @tp_sel_10   = ();
  my @tp_data_10  = ();

  my $rcv_hang    = 0;		# RCV Ports A-H
  my $send_hang   = 0;		# Send Ports A-H

  my $port_ad_compare_mask = 0;
  my $port_eh_compare_mask = 0;
  my $port_ad_alive_mask = $port_mask & 0xf;
  my $port_eh_alive_mask = ($port_mask >> 4) & 0xf;;
 
  for (my $pcm = 0; $pcm < 4; $pcm++) {
    if ((($port_ad_alive_mask >> $pcm) & 1) == 1) {
      $port_ad_compare_mask += (0xffff << (16 * $pcm));
    }
    if ((($port_eh_alive_mask >> $pcm) & 1) == 1) {
      $port_eh_compare_mask += (0xffff << (16 * $pcm));
    }
  }

	#  Initial set of mmrs to clear
  @tp_clr_mmr_names = (
	"R4_DEBUG_PORT_CONFIG",
	"R4_ROOT_TEST_POINT_SELECT",
	"R4_ROOT_TEST_POINT_TRIGGER_SELECT",
	"R4_BRANCH1_TEST_POINT_SELECT",
	"R4_BRANCH1_TEST_POINT_TRIGGER_SELECT",
	"R4_BRANCH2_TEST_POINT_SELECT",
	"R4_BRANCH2_TEST_POINT_TRIGGER_SELECT",
	"R4_LB0_TEST_POINT_SELECT_512",
	"R4_LB1_TEST_POINT_SELECT_512",
	"R4_ARBL_TEST_POINT_SELECT_512",
	"R4_ARBR_TEST_POINT_SELECT_512",
	"R4_CDQ0_TEST_POINT_SELECT_512",
	"R4_CDQ1_TEST_POINT_SELECT_512",
	"R4_CDQ2_TEST_POINT_SELECT_512",
	"R4_CDQ3_TEST_POINT_SELECT_512",
	"R4_PORTA_TEST_POINT_SELECT_512",
	"R4_PORTB_TEST_POINT_SELECT_512",
	"R4_PORTC_TEST_POINT_SELECT_512",
	"R4_PORTD_TEST_POINT_SELECT_512",
	"R4_PORTE_TEST_POINT_SELECT_512",
	"R4_PORTF_TEST_POINT_SELECT_512",
	"R4_PORTG_TEST_POINT_SELECT_512",
	"R4_PORTH_TEST_POINT_SELECT_512" 
  ); 

	# MMR's for MIP state info
  @tp_mmr_names = (
	"R4_ROOT_TEST_POINT_SELECT",
	"R4_ROOT_TEST_POINT_TRIGGER_SELECT",
	"R4_BRANCH1_TEST_POINT_SELECT",
	"R4_BRANCH1_TEST_POINT_TRIGGER_SELECT",
	"R4_BRANCH2_TEST_POINT_SELECT",
	"R4_BRANCH2_TEST_POINT_TRIGGER_SELECT",
	"R4_PORTA_TEST_POINT_SELECT_512",
	"R4_PORTB_TEST_POINT_SELECT_512",
	"R4_PORTC_TEST_POINT_SELECT_512",
	"R4_PORTD_TEST_POINT_SELECT_512",
	"R4_PORTE_TEST_POINT_SELECT_512",
	"R4_PORTF_TEST_POINT_SELECT_512",
	"R4_PORTG_TEST_POINT_SELECT_512",
	"R4_PORTH_TEST_POINT_SELECT_512", 
	"R4_ARBL_TEST_POINT_SELECT_512",
	"R4_ARBR_TEST_POINT_SELECT_512"
  ); 

	# MMR's for arb state info
  @tp_arb_names = (
        "R4_ROOT_TEST_POINT_SELECT",
        "R4_ROOT_TEST_POINT_TRIGGER_SELECT",
        "R4_BRANCH1_TEST_POINT_SELECT",
        "R4_BRANCH1_TEST_POINT_TRIGGER_SELECT",
        "R4_ARBL_TEST_POINT_SELECT_512",
        "R4_ARBR_TEST_POINT_SELECT_512"
  );

	#  First, clear all test point values

  for (my $i = 0; $i <= $#tp_clr_mmr_names; $i++) {
    my $name    = $tp_clr_mmr_names[$i];
    $reg_ptr = reg_create_mmr_inst("nl4rtr_01", $name);
    if ( !reg_is_valid($reg_ptr) )
    {  $emess = "Unable to create register instance: $name.\n";
      dsp_error_and_exit($emess);
    }
    reg_set_value($reg_ptr, 0);
    $tp_clr_reg_list[$i] = $reg_ptr;
  }

  my $xfr_complete = 0;
  my $xfr_f_cnt = 0;
  while (($xfr_complete == 0) && ($xfr_f_cnt < 3) && ($nl4r_target_list[$rnode_val]->{XFR_ERR_CNT} < 6))  {
    $SSCOPE_ERR = 0;
    reg_list_mmr_write(\@tp_clr_reg_list);
    if ($SSCOPE_ERR == 1)  {
      $xfr_f_cnt++;
      $nl4r_target_list[$rnode_val]->{XFR_ERR_CNT}++;
      $SSCOPE_ERR = 0;
      print "\n Error detected after first reg_list_mmr_write() attempt in get_nl4r_arb_tp()\n";
    }
    else {
      $xfr_complete = 1;
    }
  }

#  RcvMIP State CONFIG FOR PORTS D, C, B, A, VC0-3
  @tp_sel_0 = (
	0x7161514131211101,	# R4_ROOT_TEST_POINT_SELECT
	0x7565554535251505,	# R4_ROOT_TEST_POINT_TRIGGER_SELECT
	0x0000000000000000,	# R4_BRANCH1_TEST_POINT_SELECT
	0x0000000000000000,	# R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
	0x7161514130201000,	# R4_BRANCH2_TEST_POINT_SELECT
	0x7363534332221202,	# R4_BRANCH2_TEST_POINT_TRIGGER_SELECT
	0x7B6B5B4B3B2B1B0B,	# R4_PORTA_TEST_POINT_SELECT_512
	0x3B2B1B0B00000000,	# R4_PORTB_TEST_POINT_SELECT_512
	0x7B6B5B4B3B2B1B0B,	# R4_PORTC_TEST_POINT_SELECT_512
	0x3B2B1B0B00000000,	# R4_PORTD_TEST_POINT_SELECT_512
	0x0000000000000000,	# R4_PORTE_TEST_POINT_SELECT_512
	0x0000000000000000,	# R4_PORTF_TEST_POINT_SELECT_512
	0x0000000000000000,	# R4_PORTG_TEST_POINT_SELECT_512
	0x0000000000000000,	# R4_PORTH_TEST_POINT_SELECT_512
	0x0000000000000000, 	# R4_ARBL_TEST_POINT_SELECT_512
	0x0000000000000000	# R4_ARBR_TEST_POINT_SELECT_512
  );

#  RcvMIP State CONFIG FOR PORTS H, G, F, E, VC0-3
  @tp_sel_1 = (
        0x7161514131211101,     # R4_ROOT_TEST_POINT_SELECT
        0x7565554535251505,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x0000000000000000,     # R4_BRANCH1_TEST_POINT_SELECT
        0x0000000000000000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7565554534241404,     # R4_BRANCH2_TEST_POINT_SELECT
        0x7767574736261606,     # R4_BRANCH2_TEST_POINT_TRIGGER_SELECT
        0x7B6B5B4B3B2B1B0B,     # R4_PORTA_TEST_POINT_SELECT_512
        0x3B2B1B0B00000000,     # R4_PORTB_TEST_POINT_SELECT_512
        0x7B6B5B4B3B2B1B0B,     # R4_PORTC_TEST_POINT_SELECT_512
        0x3B2B1B0B00000000,     # R4_PORTD_TEST_POINT_SELECT_512
        0x7B6B5B4B3B2B1B0B,     # R4_PORTE_TEST_POINT_SELECT_512
        0x3B2B1B0B00000000,     # R4_PORTF_TEST_POINT_SELECT_512
        0x7B6B5B4B3B2B1B0B,     # R4_PORTG_TEST_POINT_SELECT_512
        0x3B2B1B0B00000000,     # R4_PORTH_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_ARBL_TEST_POINT_SELECT_512
        0x0000000000000000      # R4_ARBR_TEST_POINT_SELECT_512
  );

#  SendMIP State CONFIG FOR PORTS A-H, VC0-3
  @tp_sel_11 = (
        0x7161514131211101,     # R4_ROOT_TEST_POINT_SELECT
        0x0000000000000000,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x0000000000000000,     # R4_BRANCH1_TEST_POINT_SELECT
        0x0000000000000000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x0706050403020100,     # R4_BRANCH2_TEST_POINT_SELECT
        0x0000000000000000,     # R4_BRANCH2_TEST_POINT_TRIGGER_SELECT
        0x7464544434241404,     # R4_PORTA_TEST_POINT_SELECT_512
        0x0000000000000404,     # R4_PORTB_TEST_POINT_SELECT_512
        0x0000000000040404,     # R4_PORTC_TEST_POINT_SELECT_512
        0x0000000004040004,     # R4_PORTD_TEST_POINT_SELECT_512
        0x0000000404000004,     # R4_PORTE_TEST_POINT_SELECT_512
        0x0000040000000004,     # R4_PORTF_TEST_POINT_SELECT_512
        0x0004040000000004,     # R4_PORTG_TEST_POINT_SELECT_512
        0x0404000000000004,     # R4_PORTH_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_ARBL_TEST_POINT_SELECT_512
        0x0000000000000000      # R4_ARBR_TEST_POINT_SELECT_512
  );

#  ARB Left, Send 0 (Port A) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_2 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x0000000000000000,     # R4_BRANCH2_TEST_POINT_SELECT
        0x0000000000000000,     # R4_BRANCH2_TEST_POINT_TRIGGER_SELECT
        0x0000000000000000,     # R4_PORTA_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_PORTB_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_PORTC_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_PORTD_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_PORTE_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_PORTF_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_PORTG_TEST_POINT_SELECT_512
        0x0000000000000000,     # R4_PORTH_TEST_POINT_SELECT_512
        0x7868584838281808,     # R4_ARBL_TEST_POINT_SELECT_512
        0x0000000000000000      # R4_ARBR_TEST_POINT_SELECT_512
  );

#  ARB Left, Send 1 (Port B) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_3 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7969594939291909,     # R4_ARBL_TEST_POINT_SELECT_512  * only change
        0x0000000000000000      # R4_ARBR_TEST_POINT_SELECT_512
  );

#  ARB Left, Send 2 (Port G) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_4 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7A6A5A4A3A2A1A0A,     # R4_ARBL_TEST_POINT_SELECT_512  * only change
        0x0000000000000000      # R4_ARBR_TEST_POINT_SELECT_512
  );

#  ARB Left, Send 3 (Port H) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_5 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7B6B5B4B3B2B1B0B,     # R4_ARBL_TEST_POINT_SELECT_512  * only change
        0x0000000000000000      # R4_ARBR_TEST_POINT_SELECT_512
  );

#  ARB Right, Send 0 (Port C) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_6 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7161514131211101,     # R4_BRANCH1_TEST_POINT_SELECT	* change
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7B6B5B4B3B2B1B0B,     # R4_ARBL_TEST_POINT_SELECT_512
        0x7868584838281808      # R4_ARBR_TEST_POINT_SELECT_512  * change
  );

#  ARB Right, Send 1 (Port D) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_7 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7161514131211101,     # R4_BRANCH1_TEST_POINT_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7B6B5B4B3B2B1B0B,     # R4_ARBL_TEST_POINT_SELECT_512
        0x7969594939291909      # R4_ARBR_TEST_POINT_SELECT_512  * change
  );

#  ARB Right, Send 2 (Port E) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_8 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7161514131211101,     # R4_BRANCH1_TEST_POINT_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7B6B5B4B3B2B1B0B,     # R4_ARBL_TEST_POINT_SELECT_512
        0x7A6A5A4A3A2A1A0A      # R4_ARBR_TEST_POINT_SELECT_512  * change
  );

#  ARB Right, Send 3 (Port F) arbiter commit's for VC0-3, Ports A-H
  @tp_sel_9 = (
        0x7060504030201000,     # R4_ROOT_TEST_POINT_SELECT
        0x7464544434241404,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x7161514131211101,     # R4_BRANCH1_TEST_POINT_SELECT
        0x7060504030201000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x7B6B5B4B3B2B1B0B,     # R4_ARBL_TEST_POINT_SELECT_512
        0x7B6B5B4B3B2B1B0B      # R4_ARBR_TEST_POINT_SELECT_512  * change
  );

#  ARB Left, ARBMISC FIELD7 :  ARB Right, ARBMISC FIELD7 sendLB_VchBlock0,1,2,3
  @tp_sel_10 = (
        0x0000000000001000,     # R4_ROOT_TEST_POINT_SELECT
        0x0000000000000000,     # R4_ROOT_TEST_POINT_TRIGGER_SELECT
        0x0000000000001100,     # R4_BRANCH1_TEST_POINT_SELECT
        0x0000000000000000,     # R4_BRANCH1_TEST_POINT_TRIGGER_SELECT
        0x0000000000000077,     # R4_ARBL_TEST_POINT_SELECT_512
        0x0000000000007700      # R4_ARBR_TEST_POINT_SELECT_512
  );


  for (my $t = 0; $t < $menu_count; $t++)  {	# outer loop - one for each menu select setting
  #  set up the test point select values and write select mmrs
    my @tp_write_data = ();
    @tp_write_data = 	($t == 0) ? @tp_sel_0 :
			($t == 1) ? @tp_sel_1 :
			($t == 2) ? @tp_sel_2 :
			($t == 3) ? @tp_sel_3 :
			($t == 4) ? @tp_sel_4 :
			($t == 5) ? @tp_sel_5 :
			($t == 6) ? @tp_sel_6 :
			($t == 7) ? @tp_sel_7 :
			($t == 8) ? @tp_sel_8 :
			($t == 9) ? @tp_sel_9 :
			($t == 10) ? @tp_sel_10 : @tp_sel_11;

    my $tp_group_id = ($t == 0) ? "Receive Message in Progress Ports A-D":
		      ($t == 1) ? "Receive Message in Progress Ports E-H":
		      ($t == 2) ? "Port A Arbiter":
		      ($t == 3) ? "Port B Arbiter":
		      ($t == 4) ? "Port G Arbiter":
		      ($t == 5) ? "Port H Arbiter":
		      ($t == 6) ? "Port C Arbiter":
		      ($t == 7) ? "Port D Arbiter":
		      ($t == 8) ? "Port E Arbiter":
		      ($t == 9) ? "Port F Arbiter":
		      ($t == 10) ? "sendLB_VchBlock[3:0]": "Send Message in Progress Ports A-H";

    my @mmr_write_names = ();
    @mmr_write_names = (($t < 3) || ($t > 10)) ? @tp_mmr_names : @tp_arb_names;

    for (my $i = 0; $i <= $#mmr_write_names; $i++) {
      my $name = $mmr_write_names[$i];
      $reg_ptr = reg_create_mmr_inst("nl4rtr_01", $name);
      if ( !reg_is_valid($reg_ptr) )
      {  $emess = "Unable to create register instance: $name.\n";
         dsp_error_and_exit($emess);
      }
      reg_set_value($reg_ptr, $tp_write_data[$i]);
      $tp_reg_list[$i] = $reg_ptr;
    }

    my $xfr_complete = 0;
    my $xfr_f_cnt = 0;
    while (($xfr_complete == 0) && ($xfr_f_cnt < 3) && ($nl4r_target_list[$rnode_val]->{XFR_ERR_CNT} < 6))  {
      $SSCOPE_ERR = 0;
      reg_list_mmr_write(\@tp_reg_list);
      if ($SSCOPE_ERR == 1)  {
        $xfr_f_cnt++;
        $nl4r_target_list[$rnode_val]->{XFR_ERR_CNT}++;
        $SSCOPE_ERR = 0;
        print "\n Error detected after reg_list_mmr_write() attempt $t in get_nl4r_arb_tp()\n";
      }
      else {
        $xfr_complete = 1;
      }
    }

    my $dbg_port_reg_list = ();

    for (my $i = 0; $i < $eval_count; $i++)  {
      $reg_ptr = reg_create_mmr_inst("nl4rtr_01", "R4_ROOT_DEBUG_DATA");
      if ( !reg_is_valid($reg_ptr) )
      {  $emess = "\nWarning!  Unable to create register instance: R4_ROOT_DEBUG_DATA.\n";
         dsp_error_and_exit($emess);
      }
      $dbg_port_reg_list[$i] = $reg_ptr;
    }


  #  read the test point MMR
    $xfr_complete = 0;
    $xfr_f_cnt = 0;
    while (($xfr_complete == 0) && ($xfr_f_cnt < 3) && ($nl4r_target_list[$rnode_val]->{XFR_ERR_CNT} < 6))  {
      $SSCOPE_ERR = 0;
      reg_list_mmr_read(\@dbg_port_reg_list);	   # do $eval_count reads of the R4_ROOT_DEBUG_DATA mmr
      if ($SSCOPE_ERR == 1)  {
        $xfr_f_cnt++;
        $nl4r_target_list[$rnode_val]->{XFR_ERR_CNT}++;
        $SSCOPE_ERR = 0;
        print "\n Error detected after reg_list_mmr_read() attempt in get_nl4r_arb_tp()\n";
      }
      else {
        $xfr_complete = 1;
      }
    }
    my @tp_read_data = ();
    my $same_as_previous = 0;
    my $compare_mask = ($t == 0) ? $port_ad_compare_mask : 
		       ($t == 1) ? $port_eh_compare_mask : 0xffffffff;

    for (my $i = 0; $i < $eval_count; $i++)  {
      $tp_read_data[$i] = (reg_get_value($dbg_port_reg_list[$i])) & $compare_mask;
      if ($verbose_mode == 1) { printf "\nTest point values for group %d : 0x%016llX", $t, $tp_read_data[$i]};
      if (($i > 0) && ($tp_read_data[$i] > 0) && (($tp_read_data[$i] & $previous_read) == $previous_read))  {
	$same_as_previous++;
	if ($verbose_mode == 1) { print " \n Found a hit - same as previous count is now $same_as_previous\n"; }
      }
      if ($i == 0)  {
        $previous_read = $tp_read_data[$i];
      }
      else {
        $previous_read &= $tp_read_data[$i];
      }
    } 
    if ($verbose_mode == 1) { print "\n\n"; }

    if ($t == 0)     { @tp_data_0 = @tp_read_data; }
    elsif ($t == 1)  { @tp_data_1 = @tp_read_data; }
    elsif ($t == 2)  { @tp_data_2 = @tp_read_data; }
    elsif ($t == 3)  { @tp_data_3 = @tp_read_data; }
    elsif ($t == 4)  { @tp_data_4 = @tp_read_data; }
    elsif ($t == 5)  { @tp_data_5 = @tp_read_data; }
    elsif ($t == 6)  { @tp_data_6 = @tp_read_data; }
    elsif ($t == 7)  { @tp_data_7 = @tp_read_data; }
    elsif ($t == 8)  { @tp_data_8 = @tp_read_data; }
    elsif ($t == 9)  { @tp_data_9 = @tp_read_data; }
    elsif ($t == 10) { @tp_data_10 = @tp_read_data; }
    elsif ($t == 11) { @tp_data_11 = @tp_read_data; }

    if ($same_as_previous == ($eval_count - 1))  {
        $nl4r_target_list[$rnode_val]->{ERR_SUMM} .= "\nFound Possible hung condition in Test point group:  $tp_group_id\n";
	$any_nl4r_error = 1;
	$nl4r_target_list[$rnode_val]->{ERROR} = 1;
	if ($t == 0)  	 { $rcv_hang = 1; }
  	elsif ($t == 1)  { $rcv_hang = 1; }
  	elsif ($t == 10) { $send_hang = 1; }
    }

  }

  if (($rcv_hang == 1) && ($send_hang == 0) && (! defined $nl4r_target_list[$rnode_val]->{CREDITS_LOW}))  {
        $fru_list[$fru_count]->{NASID}    = $nasid_value;
 	$fru_list[$fru_count]->{DEVICE}   = "NL4R";
        $fru_list[$fru_count]->{LOCATION} = $current_location;
        $fru_list[$fru_count]->{REASON}   = "Hung NL4R in Test points: Hung at Receive Port but not at Send port.";
        printf "\n%s: Hung NL4R Error: Added NL4R NASID 0x%04X to the FRU list, entry %d.\n", $prefix, $nasid_value, $fru_count;
        $fru_count++;
  }
    

  #  Parse Test Point Data
 
  printf "\n\n RCV Message In Progress Indications  \(0=IDLE   1=IN_PROG   2=ERR_CANCEL   3=ERR_CANCEL_HEAD   4=EAT_UNTIL_TAIL   5=TAIL_TO\)\n\n";
  printf "   Port A          Port B          Port C          Port D          Port E          Port F          Port G          Port H \n";
  printf "VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3\n";   
  printf "-------------   -------------   -------------   -------------   -------------   -------------   -------------   -------------\n";

  for (my $i = 0; $i < $eval_count; $i++)  {
    my $mip_a_0 = $tp_data_0[$i] & 7;    
    my $mip_a_1 = ($tp_data_0[$i] >> 4) & 7;    
    my $mip_a_2 = ($tp_data_0[$i] >> 8) & 7;    
    my $mip_a_3 = ($tp_data_0[$i] >> 12) & 7;    
    my $mip_b_0 = ($tp_data_0[$i] >> 16) & 7;    
    my $mip_b_1 = ($tp_data_0[$i] >> 20) & 7;    
    my $mip_b_2 = ($tp_data_0[$i] >> 24) & 7;    
    my $mip_b_3 = ($tp_data_0[$i] >> 28) & 7;    
    my $mip_c_0 = ($tp_data_0[$i] >> 32) & 7;    
    my $mip_c_1 = ($tp_data_0[$i] >> 36) & 7;    
    my $mip_c_2 = ($tp_data_0[$i] >> 40) & 7;    
    my $mip_c_3 = ($tp_data_0[$i] >> 44) & 7;    
    my $mip_d_0 = ($tp_data_0[$i] >> 48) & 7;    
    my $mip_d_1 = ($tp_data_0[$i] >> 52) & 7;    
    my $mip_d_2 = ($tp_data_0[$i] >> 56) & 7;    
    my $mip_d_3 = ($tp_data_0[$i] >> 60) & 7;    

    my $mip_e_0 = $tp_data_1[$i] & 7;    
    my $mip_e_1 = ($tp_data_1[$i] >> 4) & 7;    
    my $mip_e_2 = ($tp_data_1[$i] >> 8) & 7;    
    my $mip_e_3 = ($tp_data_1[$i] >> 12) & 7;    
    my $mip_f_0 = ($tp_data_1[$i] >> 16) & 7;    
    my $mip_f_1 = ($tp_data_1[$i] >> 20) & 7;    
    my $mip_f_2 = ($tp_data_1[$i] >> 24) & 7;    
    my $mip_f_3 = ($tp_data_1[$i] >> 28) & 7;    
    my $mip_g_0 = ($tp_data_1[$i] >> 32) & 7;    
    my $mip_g_1 = ($tp_data_1[$i] >> 36) & 7;    
    my $mip_g_2 = ($tp_data_1[$i] >> 40) & 7;    
    my $mip_g_3 = ($tp_data_1[$i] >> 44) & 7;    
    my $mip_h_0 = ($tp_data_1[$i] >> 48) & 7;    
    my $mip_h_1 = ($tp_data_1[$i] >> 52) & 7;    
    my $mip_h_2 = ($tp_data_1[$i] >> 56) & 7;    
    my $mip_h_3 = ($tp_data_1[$i] >> 60) & 7;    

    printf "  %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d\n",
	$mip_a_0, $mip_a_1, $mip_a_2, $mip_a_3, $mip_b_0, $mip_b_1, $mip_b_2, $mip_b_3, $mip_c_0, $mip_c_1, $mip_c_2, $mip_c_3, $mip_d_0, $mip_d_1, $mip_d_2, $mip_d_3,
	$mip_e_0, $mip_e_1, $mip_e_2, $mip_e_3, $mip_f_0, $mip_f_1, $mip_f_2, $mip_f_3, $mip_g_0, $mip_g_1, $mip_g_2, $mip_g_3, $mip_h_0, $mip_h_1, $mip_h_2, $mip_h_3;
  }
  print "\n\n";

   printf "\n NL4 Arbitration Indications from Debug Testpoints.\n";
   printf "\nTo Send PortA   To Send PortB   To Send PortC   To Send PortD   To Send PortE   To Send PortF   To Send PortG   To Send PortH";
   printf "\nVC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3";   
   printf "\n-------------   -------------   -------------   -------------   -------------   -------------   -------------   -------------\n";

  for (my $i = 0; $i < $eval_count; $i++)  {
    my $vc0_to_a = get_port_value ($tp_data_2[$i] & 0xff); 
    my $vc1_to_a = get_port_value ( ($tp_data_2[$i] >> 8) & 0xff); 
    my $vc2_to_a = get_port_value ( ($tp_data_2[$i] >> 16) & 0xff); 
    my $vc3_to_a = get_port_value ( ($tp_data_2[$i] >> 24) & 0xff); 
    my $vc0_to_b = get_port_value ( $tp_data_3[$i] & 0xff); 
    my $vc1_to_b = get_port_value ( ($tp_data_3[$i] >> 8) & 0xff); 
    my $vc2_to_b = get_port_value ( ($tp_data_3[$i] >> 16) & 0xff); 
    my $vc3_to_b = get_port_value ( ($tp_data_3[$i] >> 24) & 0xff); 
    my $vc0_to_c = get_port_value ( $tp_data_6[$i] & 0xff); 
    my $vc1_to_c = get_port_value ( ($tp_data_6[$i] >> 8) & 0xff); 
    my $vc2_to_c = get_port_value ( ($tp_data_6[$i] >> 16) & 0xff); 
    my $vc3_to_c = get_port_value ( ($tp_data_6[$i] >> 24) & 0xff); 
    my $vc0_to_d = get_port_value ( $tp_data_7[$i] & 0xff); 
    my $vc1_to_d = get_port_value ( ($tp_data_7[$i] >> 8) & 0xff); 
    my $vc2_to_d = get_port_value ( ($tp_data_7[$i] >> 16) & 0xff); 
    my $vc3_to_d = get_port_value ( ($tp_data_7[$i] >> 24) & 0xff); 
    my $vc0_to_e = get_port_value ( $tp_data_8[$i] & 0xff); 
    my $vc1_to_e = get_port_value ( ($tp_data_8[$i] >> 8) & 0xff); 
    my $vc2_to_e = get_port_value ( ($tp_data_8[$i] >> 16) & 0xff); 
    my $vc3_to_e = get_port_value ( ($tp_data_8[$i] >> 24) & 0xff); 
    my $vc0_to_f = get_port_value ( $tp_data_9[$i] & 0xff); 
    my $vc1_to_f = get_port_value ( ($tp_data_9[$i] >> 8) & 0xff); 
    my $vc2_to_f = get_port_value ( ($tp_data_9[$i] >> 16) & 0xff); 
    my $vc3_to_f = get_port_value ( ($tp_data_9[$i] >> 24) & 0xff); 
    my $vc0_to_g = get_port_value ( $tp_data_4[$i] & 0xff); 
    my $vc1_to_g = get_port_value ( ($tp_data_4[$i] >> 8) & 0xff); 
    my $vc2_to_g = get_port_value ( ($tp_data_4[$i] >> 16) & 0xff); 
    my $vc3_to_g = get_port_value ( ($tp_data_4[$i] >> 24) & 0xff); 
    my $vc0_to_h = get_port_value ( $tp_data_5[$i] & 0xff); 
    my $vc1_to_h = get_port_value ( ($tp_data_5[$i] >> 8) & 0xff); 
    my $vc2_to_h = get_port_value ( ($tp_data_5[$i] >> 16) & 0xff); 
    my $vc3_to_h = get_port_value ( ($tp_data_5[$i] >> 24) & 0xff); 
    printf "  %s  %s  %s  %s      %s  %s  %s  %s      %s  %s  %s  %s      %s  %s  %s  %s      %s  %s  %s  %s      %s  %s  %s  %s      %s  %s  %s  %s      %s  %s  %s  %s\n",
	$vc0_to_a, $vc1_to_a, $vc2_to_a, $vc3_to_a, $vc0_to_b, $vc1_to_b, $vc2_to_b, $vc3_to_b, 
	$vc0_to_c, $vc1_to_c, $vc2_to_c, $vc3_to_c, $vc0_to_d, $vc1_to_d, $vc2_to_d, $vc3_to_d, 
	$vc0_to_e, $vc1_to_e, $vc2_to_e, $vc3_to_e, $vc0_to_f, $vc1_to_f, $vc2_to_f, $vc3_to_f, 
	$vc0_to_g, $vc1_to_g, $vc2_to_g, $vc3_to_g, $vc0_to_h, $vc1_to_h, $vc2_to_h, $vc3_to_h ;
  }
  print "\n\n";
  printf "  Router LB Port VC Blocked Indications \(1=Blocked\)\n\n";

  print "\n\n";

  printf " Send Message In Progress Indications  \(0=IDLE   1=IN_PROG\)\n\n";
  printf "   Port A          Port B          Port C          Port D          Port E          Port F          Port G          Port H \n";
  printf "VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3    VC0  1  2  3\n";   
  printf "-------------   -------------   -------------   -------------   -------------   -------------   -------------   -------------\n";
  for (my $i = 0; $i < $eval_count; $i++)  {
    my $mip_a_0 = $tp_data_11[$i] & 1;
    my $mip_a_1 = ($tp_data_11[$i] >> 1) & 1;
    my $mip_a_2 = ($tp_data_11[$i] >> 2) & 1;
    my $mip_a_3 = ($tp_data_11[$i] >> 3) & 1;
    my $mip_b_0 = ($tp_data_11[$i] >> 4) & 1;
    my $mip_b_1 = ($tp_data_11[$i] >> 5) & 1;
    my $mip_b_2 = ($tp_data_11[$i] >> 6) & 1;
    my $mip_b_3 = ($tp_data_11[$i] >> 7) & 1;
    my $mip_c_0 = ($tp_data_11[$i] >> 8) & 1;
    my $mip_c_1 = ($tp_data_11[$i] >> 9) & 1;
    my $mip_c_2 = ($tp_data_11[$i] >> 10) & 1;
    my $mip_c_3 = ($tp_data_11[$i] >> 11) & 1;
    my $mip_d_0 = ($tp_data_11[$i] >> 12) & 1;
    my $mip_d_1 = ($tp_data_11[$i] >> 13) & 1;
    my $mip_d_2 = ($tp_data_11[$i] >> 14) & 1;
    my $mip_d_3 = ($tp_data_11[$i] >> 15) & 1;
 
    my $mip_e_0 = ($tp_data_11[$i] >> 16) & 1;
    my $mip_e_1 = ($tp_data_11[$i] >> 17) & 1;
    my $mip_e_2 = ($tp_data_11[$i] >> 18) & 1;
    my $mip_e_3 = ($tp_data_11[$i] >> 19) & 1;
    my $mip_f_0 = ($tp_data_11[$i] >> 20) & 1;
    my $mip_f_1 = ($tp_data_11[$i] >> 21) & 1;
    my $mip_f_2 = ($tp_data_11[$i] >> 22) & 1;
    my $mip_f_3 = ($tp_data_11[$i] >> 23) & 1;
    my $mip_g_0 = ($tp_data_11[$i] >> 24) & 1;
    my $mip_g_1 = ($tp_data_11[$i] >> 25) & 1;
    my $mip_g_2 = ($tp_data_11[$i] >> 26) & 1;
    my $mip_g_3 = ($tp_data_11[$i] >> 27) & 1;
    my $mip_h_0 = ($tp_data_11[$i] >> 28) & 1;
    my $mip_h_1 = ($tp_data_11[$i] >> 29) & 1;
    my $mip_h_2 = ($tp_data_11[$i] >> 30) & 1;
    my $mip_h_3 = ($tp_data_11[$i] >> 31) & 1;
 
    printf "  %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d      %d  %d  %d  %d\n",
        $mip_a_0, $mip_a_1, $mip_a_2, $mip_a_3, $mip_b_0, $mip_b_1, $mip_b_2, $mip_b_3, $mip_c_0, $mip_c_1, $mip_c_2, $mip_c_3, $mip_d_0, $mip_d_1, $mip_d_2, $mip_d_3,
        $mip_e_0, $mip_e_1, $mip_e_2, $mip_e_3, $mip_f_0, $mip_f_1, $mip_f_2, $mip_f_3, $mip_g_0, $mip_g_1, $mip_g_2, $mip_g_3, $mip_h_0, $mip_h_1, $mip_h_2, $mip_h_3;
  }
  print "\n\n";

}

sub get_port_value  {

   my $port_num = $_[0];
   my $port_des =  ($port_num == 0) ?    '0' :
		   ($port_num == 1) ?    'A' :
                   ($port_num == 2) ?    'B' :
                   ($port_num == 4) ?    'C' :
                   ($port_num == 8) ?    'D' :
                   ($port_num == 0x10) ? 'E' :
                   ($port_num == 0x20) ? 'F' :
                   ($port_num == 0x40) ? 'G' :
                   ($port_num == 0x80) ? 'H' : sprintf("0x%x", $port_num);

   return($port_des);
}

# subroutine to confirm this VC's error detail contains info that
# was reported in the first error or the error summary 

sub vc_error_check {

  my $port_err_val = $_[0];		# errors logged on this port
  my $detail_vc = $_[1];		# VC found in the error detail
  my $vc_ok = 0;
  
  my $error_mask = ($detail_vc == 0) ?  0x04410000 :
	           ($detail_vc == 1) ?  0x08920000 :
	           ($detail_vc == 2) ?  0x11040000 :
	           ($detail_vc == 3) ?  0x22280000 : 0x0;

  if (($port_err_val & $error_mask) > 0) {
    $vc_ok = 1;
  }
  
  return($vc_ok);

}


#-------------------------------------------------------------------------------

1;


