#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: nl4rtr_perf.pm,v 1.2 2005/04/15 22:07:28 meyer Exp $
#-------------------------------------------------------------------------------
#
#  FILE:  nl4rtr_perf.pm
#
#
#  DESCRIPTION: contains routines & utilities for NL4R specific performance
#	reports.
#
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     04/06/04  Gary Meyer      Original source code submitted for release.
#
#-------------------------------------------------------------------------------


sub nl4r_perf_report {
  print "\n\nIn nl4r_perf_report()\n\n" if ($verbose_mode == 1);

  my $c_name    = $_[0];	#  chip name
  my $module 	= $_[1];	#  module name
  my $lb_clock  = $_[2];	#  derived from the CNT field of R4_PORT_PERF_WIN
  my $nasid_val = $_[3];	
  my $perf_regs_ref = $_[4];	#  reference to multi-dimensional list of NL4R perf regs 

  my @perf_regs = ();
  @perf_regs = @$perf_regs_ref;

  my @ports = ( 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H');
  my $num_ports = $#ports;

  printf "\n\n   **** NL4R Performance Counters for %s of %s    NASID: 0x%03X\n", $c_name, $module, $nasid_val;

  my $p = 0;
  for($p=0; $p<$num_ports; $p++) {
	printf "\n\n\tPort %s Output\n", $ports[$p];
        printf "\t  A  (Total Output AGE) \t\t0x%X\n", $perf_regs[$p][36];
        printf "\t  B  (Blocked Output VC)\t\t0x%X   0x%X   0x%X   0x%X\n", 
		$perf_regs[$p][8], $perf_regs[$p][9], $perf_regs[$p][10], $perf_regs[$p][11];

        printf "\t  Df (Output Fairness Datavalid) \t 0x%X\n", $perf_regs[$p][12];
        printf "\t  Db (Output Bypass Datavalid) \t\t 0x%X\n", $perf_regs[$p][13];
        printf "\t  M  (Granted Tails) \t\t\t 0x%X   0x%X   0x%X   0x%X\n", 
		$perf_regs[$p][20], $perf_regs[$p][21], $perf_regs[$p][22], $perf_regs[$p][23];

        printf "\t  P  (Out Msg in Progress) \t\t 0x%X   0x%X   0x%X   0x%X\n", 
		$perf_regs[$p][24], $perf_regs[$p][25], $perf_regs[$p][26], $perf_regs[$p][27];

        printf "\t  U  (Multiple Unblocked Requests) \t 0x%X\n", $perf_regs[$p][14];
        printf "\t  W  (Flowed w/ Multiple Unblocked) \t 0x%X\n", $perf_regs[$p][15]; 
        printf "\t  T  (Command Type) \t0x%X  0x%X  0x%X  0x%X  0x%X  0x%X  0x%X  0x%X\n", 
	   	$perf_regs[$p][0], $perf_regs[$p][1], $perf_regs[$p][2], $perf_regs[$p][3],
	   	$perf_regs[$p][4], $perf_regs[$p][5], $perf_regs[$p][6], $perf_regs[$p][7];

	printf "\n\tPort %s Input\n", $ports[$p];
        printf "\t  E  (Integrated Queue Depth) \t\t0x%X   0x%X   0x%X   0x%X\n", 
		$perf_regs[$p][32], $perf_regs[$p][33], $perf_regs[$p][34], $perf_regs[$p][35];

        printf "\t  G  (Grants [bypass & fairness]) \t0x%X   0x%X   0x%X   0x%X\n",
		$perf_regs[$p][16], $perf_regs[$p][17], $perf_regs[$p][18], $perf_regs[$p][19];
        printf "\t  R  (Input Requests to Arbiter) \t0x%X   0x%X   0x%X   0x%X\n",
		$perf_regs[$p][28], $perf_regs[$p][29], $perf_regs[$p][30], $perf_regs[$p][31];
	printf "              ---------------------------------\n";

  } # end for port

  printf "\n\n";
  printf "\t*****************************************************\n";
  printf "\t*            Performance Report                     *\n";
  printf "\t*****************************************************\n";

  my ($f0, $f1, $f2, $f3);
  my $tot_rtr_input_bw = 0.0;
  my $tot_rtr_output_bw = 0.0;
  my $tot_E = 0.0;
  my $tot_G = 0.0;
  my $clocks = 0xfffffffffffffff - $lb_clock;
  my $msg = "";

  if ($clocks == 0) {
      printf "\n\n *** ERROR Clocks perf counter = 0 ***\n\n";
      $clocks = 1;
  }
  printf "\t\t Time period = %d clks or %d nsec\n", $clocks, $clocks*5;

  for($p=0; $p<$num_ports; $p++) {
        printf "\n\tPort %s Flow\n", $ports[$p];
     	  # Input Port BW
        my $in_bw_vc0 = ($perf_regs[$p][16] * 3.2) / $clocks;
        my $in_bw_vc1 = ($perf_regs[$p][17] * 3.2) / $clocks;
        my $in_bw_vc2 = ($perf_regs[$p][18] * 3.2) / $clocks;
        my $in_bw_vc3 = ($perf_regs[$p][19] * 3.2) / $clocks;
        my $tot_port_bw = $in_bw_vc0 + $in_bw_vc1 + $in_bw_vc2 + $in_bw_vc3;
        printf "\t  Input B/W (GB/s)\t\t  %6.3f %6.3f %6.3f %6.3f = %6.3f\n",
	  	$in_bw_vc0, $in_bw_vc1, $in_bw_vc2, $in_bw_vc3, $tot_port_bw;
        $tot_rtr_input_bw += $tot_port_bw;

          # Output Port BW
        $tot_port_bw = ($perf_regs[$p][12] + $perf_regs[$p][13]) * 3.2 / $clocks;
        printf "\t  Output B/W (GB/s)\t\t\t\t\t %6.3f\n", $tot_port_bw;
        $tot_rtr_output_bw += $tot_port_bw;

          # Average Input Queue Depth
        $avq0dpth = $perf_regs[$p][32] / $clocks;
        $avq1dpth = $perf_regs[$p][33] / $clocks;
        $avq2dpth = $perf_regs[$p][34] / $clocks;
        $avq3dpth = $perf_regs[$p][35] / $clocks;
        printf "\t  Ave Q Depth (upkts)\t\t %6.1f %6.1f %6.1f %6.1f\n",
		$avq0dpth, $avq1dpth, $avq2dpth, $avq3dpth;

          # Average Input Queue Residency time
        if ($perf_regs[$p][16] == 0) {
            $f0 = 0.0;
	} else {
            $f0 = 5.0 * $perf_regs[$p][32] / $perf_regs[$p][16];
            $tot_E += $perf_regs[$p][32];
            $tot_G += $perf_regs[$p][16];
        }

	if ($perf_regs[$p][17] == 0) {
            $f1 = 0.0;
	} else {
            $f1 = 5.0 * $perf_regs[$p][33] / $perf_regs[$p][17]; 
	    $tot_E += $perf_regs[$p][33];
	    $tot_G += $perf_regs[$p][17];
        }

        if ($perf_regs[$p][18] == 0) {
            $f2 = 0.0;
	} else {
            $f2 = 5.0 * $perf_regs[$p][34] / $perf_regs[$p][18];
            $tot_E += $perf_regs[$p][34];
            $tot_G += $perf_regs[$p][18];
        }

        if($perf_regs[$p][19] == 0) {
            $f3 = 0.0;
	} else {
            $f3 = 5.0 * $perf_regs[$p][35] / $perf_regs[$p][19];
            $tot_E += $perf_regs[$p][35];
            $tot_G += $perf_regs[$p][19];
        }

        printf "\t  Ave Q Residency (nsec)\t %6.3f %6.3f %6.3f %6.3f\n", $f0, $f1, $f2, $f3; 

        printf "\t\t\tLocal Arbitration\n";
                # Fraction of requests granted.
        if ($perf_regs[$p][28] == 0) {
            $f0 = 0.0;
	} else {
            $f0 = (100.0 * $perf_regs[$p][16]) / ($perf_regs[$p][28]);
	}

        if ($perf_regs[$p][29] == 0) {
            $f1 = 0.0;
	} else {
            $f1 = (100.0 * $perf_regs[$p][17]) / ($perf_regs[$p][29]);
	}

        if ($perf_regs[$p][30] == 0) {
            $f2 = 0.0;
	} else {
            $f2 = (100.0 * $perf_regs[$p][18]) / ($perf_regs[$p][30]);
	}

        if ($perf_regs[$p][31] == 0) {
            $f3 = 0.0;
	} else {
            $f3 = (100.0 * $perf_regs[$p][19]) / ($perf_regs[$p][31]);
	}

        printf "\t  Percent Requests Granted\t %6.3f %6.3f %6.3f %6.3f\n", $f0, $f1, $f2, $f3;

                # Average Message in Progress (nsec)

        if ($perf_regs[$p][20] == 0) {
            $f0 = 0.0;
	} else {
            $f0 = ($perf_regs[$p][24]) / ($perf_regs[$p][20]);
	}

        if ($perf_regs[$p][21] == 0) {
            $f1 = 0.0;
	} else {
            $f1 = ($perf_regs[$p][25]) / ($perf_regs[$p][21]);
	}

        if ($perf_regs[$p][22] == 0) {
            $f2 = 0.0;
	} else {
            $f2 = ($perf_regs[$p][26]) / ($perf_regs[$p][22]);
	}

        if ($perf_regs[$p][23] == 0) {
            $f3 = 0.0;
	} else {
            $f3 = ($perf_regs[$p][27]) / ($perf_regs[$p][23]);
	}

        printf "\t  Ave Msg in Progress (nsec)\t %6.3f %6.3f %6.3f %6.3f\n", $f0, $f1, $f2, $f3;

                # Acceptor Efficiency.

        if ($perf_regs[$p][14] == 0) {
             $f0 = 0;
        } else {
             $f0 = (100.0 * ($perf_regs[$p][15])) / $perf_regs[$p][14];
        }
        printf "\t  Acceptor Efficiency       \t %6.3f\n", $f0;
                
                # Fraction of Requests Bypass
        $datvld = $perf_regs[$p][12] + $perf_regs[$p][13];
  	if ($datvld == 0) {
      	    printf "\n\n *** ERROR datvld counter = 0 for Port $ports[$p] ***\n\n";
      	    $datvld = 1;
  	}

        $f0 = $perf_regs[$p][13] / $datvld;
        printf "\t  Percent Requests Bypassed \t %6.3f\n", 100.0*$f0;

        printf "\t\t\tGlobal Arbitration\n";
                # Percent VC Blocked
        $f0 = (100.0 * $perf_regs[$p][8]) / $clocks;
        $f1 = (100.0 * $perf_regs[$p][9]) / $clocks;
        $f2 = (100.0 * $perf_regs[$p][10]) / $clocks;
        $f3 = (100.0 * $perf_regs[$p][11]) / $clocks;
        printf "\t  Percent VC Blocked       \t %6.3f %6.3f %6.3f %6.3f\n", $f0, $f1, $f2, $f3;

                # Average Output Age
        my $tails = $perf_regs[$p][20] + $perf_regs[$p][21] + $perf_regs[$p][22] + $perf_regs[$p][23];
        if ($tails == 0) {
            $f0 = 0.0;
        } else {
            $f0 = $perf_regs[$p][36] / $tails;
	}
        printf "\t  Average Output Age        \t%6.3f\n", $f0;
        
  }  # end for p[ort]

          # Total Router Bandwidth
  printf "\n\n   **** NL4R Performance Summary for %s of %s   NASID: 0x%03X\n", $c_name, $module, $nasid_val;
  printf "\n\t  Input GB/s: %d    Output GB/s: %d\n", %tot_rtr_input_bw, $tot_rtr_output_bw;
          # Total Router Average Latency
  if ($tot_G == 0) {
     printf "\n *** ERROR tot_G counter = 0 ***\n\n";
     $tot_G = 1;
  } 
  my $rtr_ave_lat = (5.0 * ($tot_E + 7.0)) / $tot_G;
  printf "\t  Average Latency: %d nsec\n\n\n", $rtr_ave_lat;

}

1;

