
$USE_NL4RTR_SUBS_PM = 1;    # Define to allow test for module inclusion

$NL4RTR_IR_LEN = 7;

$INIT_MODE_NONE             = 0x00;
$INIT_MODE_JTAG_RESET_HARD  = 0x01;
$INIT_MODE_JTAG_RESET_SOFT  = 0x02;
$INIT_MODE_LOAD_TDR_DEFAULT = 0x04;
$INIT_MODE_LLP_CONFIG       = 0x08;

@nl4rtr_llp_ports = ( 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H' );
#@nl4rtr_llp_ports = ( 'A', 'B' );

#-------------------------------------------------------------------------------
#
#    SUBROUTINE: nl4rtr_reset_init()
#
#    USAGE: nl4rtr_reset_init()
#           nl4rtr_reset_init($mode)
#           nl4rtr_reset_init($mode, $top_module)
#
#    DESCRIPNL4RTRN:  This subroutine will perform the reset and initialization
#       sequence on the previously selected (via mmr_select()) NL4RTR ASIC.
#
#    WARNING: This subroutine was used during simulator initialization only!
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub nl4rtr_reset_init
{
   my $num = $#_ + 1;
   print "\nnl4rtr_reset_init(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCNL4RTRNS);

   my $module_name  = "loopback";
   my $mode = $INIT_MODE_NONE;
   if ( $num == 1 )
   {  $mode = $_[0];
   }
   elsif ( $num == 2 )
   {  $mode = $_[0];
      $module_name = $_[1];
   }
   elsif ( $num != 0 )
   {  $emess = "Invalid parameters passed to subroutine nl4rtr_reset_init().";
      dsp_error_and_exit($emess);
      return;
   }
   printf "reset mode = 0x%02X = %u\n", $mode, $mode  if ($debug_flag & $DEBUG_GENERAL);

   my $hard_rst_reg = "$module_name.hard_reset_reg";
   my $soft_rst_reg = "$module_name.soft_reset_reg";
   my $clk_rst_reg  = "$module_name.clk_reset_reg";
   my @tdi = ();
   my @tdo = ();

   print "\n";
   print "------------------------------------------------------------------\n";
   print "--  INITIALIZING THE NL4RTR USING JTAG AND SIMULATOR ACCESS\n";
   print "------------------------------------------------------------------\n";

   if ( $mode & $INIT_MODE_JTAG_RESET_HARD )
   {  print "\n->Resetting the TAP controller via TRST...\n";
      jtag_reset($JTAG_RESET_HARD, $JTAG_STATE_RTI);
   }

   if ( $mode & $INIT_MODE_JTAG_RESET_SOFT )
   {  print "\n->Resetting the TAP controller via TMS/TCK...\n";
      jtag_reset($JTAG_RESET_SOFT, $JTAG_STATE_RTI);
   }

   print "\n->Asserting the CLK RESET signal on the simulator...\n  entity=$clk_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $clk_rst_reg, "1'b1");
   jtag_state_to_rti(1);
   print "\n->Asserting the HARD RESET signal on the simulator...\n  entity=$hard_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $hard_rst_reg, "1'b1");
   jtag_state_to_rti(1);
   print "\n->Asserting the SOFT RESET signal on the simulator...\n  entity=$soft_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $soft_rst_reg, "1'b1");
   jtag_state_to_rti(1);

   if ( $mode & $INIT_MODE_LOAD_TDR_DEFAULT )
   {  print "\n->Loading the TDR_DEFAULT instruction...\n";
      @tdi = ( 0x7C );
      @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");
   }

   print "\n->Loading the CONFIG instruction...\n";
   @tdi = ( 0x7E );
   @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");
   print "\n->Loading the CONFIG data register with hardcoded value\n";
   @tdi = ( 0x0 );
   @tdo = jtag_scan_dr(2, \@tdi);
jtag_data_display(\@tdi, "DR TDI DATA");
jtag_data_display(\@tdo, "DR TDO DATA");



   print "\n->Loading the JCLK_RST instruction...\n";
   @tdi = ( 0x7D );
   @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");

   print "\n->Loading the BYPASS instruction (clear JCLK_RST)...\n";
   @tdi = ( 0x7F );
   @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");

   print "\n->Deasserting the CLK RESET signal on the simulator...\n  entity=$clk_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $clk_rst_reg, "1'b0");


   print "\n->Loading the ICNTLR instruction...\n";
   @tdi = ( 0x06 );
   @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");

   print "\n->Loading the ICNTLR data register with RESET value\n";
   @tdi = (    0x0000, 0x00000000, 0x00000000, 0x00000000,
           0x00000000, 0x10101810, 0x10181010, 0x18101018,
           0x10101810, 0x10181010, 0x18101018);
   @tdo = jtag_scan_dr(336, \@tdi);
jtag_data_display(\@tdi, "DR TDI DATA");
jtag_data_display(\@tdo, "DR TDO DATA");

   print "\n->Loading the ICNTLR data register with hardcoded value\n";
   @tdi = (     0x0000, 0x00000000, 0x00000000, 0x00000000, 
            0x00000000, 0x00101800, 0x10180010, 0x18001018, 
            0x00101800, 0x10180010, 0x18001018);
   @tdo = jtag_scan_dr(336, \@tdi);
jtag_data_display(\@tdi, "DR TDI DATA");
jtag_data_display(\@tdo, "DR TDO DATA");

   if ( $mode & $INIT_MODE_LLP_CONFIG )
   {  
      $ir_code = 0x10;
      foreach $port ( @nl4rtr_llp_ports )
      {
         print "\n->Loading the LLP4_$port CONFIG instruction...\n";
         @tdi = ( $ir_code );
         @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");
         print "\n->Loading the LLP4_$port CONFIG data register with hardcoded value...\n";
         @tdi = ( 0x000, 0x00000000, 0x00000000, 0x0FC00000, 0x003FE03F, 0xF03F8000, 0x00610003 );
         @tdo = jtag_scan_dr(203, \@tdi);
jtag_data_display(\@tdi, "DR TDI DATA");
jtag_data_display(\@tdo, "DR TDO DATA");
         $ir_code += 1;

         print "\n->Loading the LLP4TEST_$port instruction...\n";
         @tdi = ( $ir_code );
         @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");
         print "\n->Loading the LLP4TEST_$port data register with hardcoded value...\n";
         @tdi = ( 0x00000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 );
         @tdo = jtag_scan_dr(242, \@tdi);
jtag_data_display(\@tdi, "DR TDI DATA");
jtag_data_display(\@tdo, "DR TDO DATA");
         $ir_code += 1;
      }

      $ir_code = 0x60;
      foreach $port ( @nl4rtr_llp_ports )
      {
         print "\n->Loading the LLP4_$port DLY_CTRL0 instruction...\n";
         @tdi = ( $ir_code );
         @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");
         print "\n->Loading the LLP4_$port DLY_CTRL0 data register with hardcoded value...\n";
         @tdi = ( 0x00000000 );
         @tdo = jtag_scan_dr(27, \@tdi);
jtag_data_display(\@tdi, "DR TDI DATA");
jtag_data_display(\@tdo, "DR TDO DATA");
         $ir_code += 1;

         print "\n->Loading the LLP4_$port DLY_CTRL1 instruction...\n";
         @tdi = ( $ir_code );
         @tdo = jtag_scan_ir($NL4RTR_IR_LEN, \@tdi);
jtag_data_display(\@tdi, "IR TDI DATA");
jtag_data_display(\@tdo, "IR TDO DATA");
         print "\n->Loading the LLP4_$port DLY_CTRL1 data register with hardcoded value...\n";
         @tdi = ( 0x00000000, 0x00000000 );
         @tdo = jtag_scan_dr(33, \@tdi);
jtag_data_display(\@tdi, "DR TDI DATA");
jtag_data_display(\@tdo, "DR TDO DATA");
         $ir_code += 1;
      }
   }

   print "\n->Desserting the SOFT RESET signal on the simulator...\n  entity=$soft_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $soft_rst_reg, "1'b0");
   jtag_state_to_rti(3);
   print "\n->Deasserting the HARD RESET signal on the simulator...\n  entity=$hard_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $hard_rst_reg, "1'b0");

   jtag_state_to_rti(10);


   print "------------------------------------------------------------------\n";
   print "--  RESET/INITIALIZATION SEQUENCE COMPLETE!\n";
   print "------------------------------------------------------------------\n";
}

#------------------------------------------------------------------------------

1;
