#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: orbit_01_jtdrs.pm,v 1.1 2003/04/14 19:54:38 beighley Exp $
#
#-------------------------------------------------------------------------------
#
#  FILE: orbit_01_jtdrs.pm
#
#  NOTE: Definitions of the JTAG test data registers (TDRs) for ORBIT.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     04/11/03  Karen Beighley  Original source code submitted into tree.
#
#-------------------------------------------------------------------------------

$orbit_01_tdr_defined = 0;

#-------------------------------------------------------------------------------

%orbit_01_flds_00000 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%orbit_01_tdr_00000 =
(  NAME   => "BYPASS",
   NOTE   => "BYPASS JTAG Register",
   OPNAME => "BYPASS",
   OPCODE => 0x7F,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%orbit_01_flds_00000
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00001 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "VERSION",          => [   0,  28,   4, 0x0000, "Version number"         ],
   "PARTNUMBER",       => [   1,  12,  16, 0x0000, "Partnumber"             ],
   "MANUFACTURER",     => [   2,   1,  11, 0x0000, "Manufacturer Code"      ],
   "LSB",              => [   3,   0,   1, 0x0000, "Mandatory LSB"          ]
);

%orbit_01_tdr_00001 =
(  NAME   => "IDCODE",
   NOTE   => "JTAG Device Identification Register",
   OPNAME => "IDCODE",
   OPCODE => 0x03,
   LENGTH => 32,
   RESET  => [ 0x00000000 ],
   FIELDS => \%orbit_01_flds_00001
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00002 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "PSB_CONFIG_AUX"        => [   0,  42,   2, 0x0000, "Auxiliary config bits"   ],
   "JTAG_MCI_OVERRIDE"     => [   1,  41,   1, 0x0000, "JTAG MCI override"       ],
   "JTAG_MCI_TARGET"       => [   2,  27,  14, 0x0000, "JTAG MCI target"         ],
   "JTAG_MCI_RESET_DELAY"  => [   3,  21,   6, 0x0000, "JTAG MCI reset delay"    ],
   "WRT90_OVERRIDE"        => [   4,  20,   1, 0x0000, "WRT90 override"          ],
   "WRT90_OVERRIDEA"       => [   5,  19,   1, 0x0000, "WRT90 override A"        ],
   "WRT90_TARGET"          => [   6,   5,  14, 0x0000, "WRT90 target"            ],
   "JSEL_UNUSED"           => [   7,   3,   2, 0x0000, "Unused clock select"     ],
   "NI_CLK_SEL"            => [   8,   2,   1, 0x0000, "NI clock select"         ],
   "MD_CLK_SEL"            => [   9,   0,   2, 0x0000, "MD clock divider select" ],
);

%orbit_01_tdr_00002 =
(  NAME   => "CONFIG",
   NOTE   => "Orbit Pre-reset Configuration",
   OPNAME => "CONFIG",
   OPCODE => 0x7E,
   LENGTH => 44,
   RESET  => [ 0x0000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00002
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00003 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "WRAP_JTAG_SCAN_ACCESS" => [   0,   4,  18, 0x0000, "Scan access bits for RAM wrappers" ],
   "CIR"                   => [   1,   3,   1, 0x0000, "Circulate TDO to TDI"              ],
   "INV"                   => [   2,   2,   1, 0x0000, "Invert circlated TDO value"        ],
   "CL2_CONFIG"            => [   3,   0,   2, 0x0000, "Config bit per channel (1=NL3)"    ]
);

%orbit_01_tdr_00003 =
(  NAME   => "SCAN_CTL",
   NOTE   => "Controls gmoc_mode and scan access",
   OPNAME => "SCAN_CTL",
   OPCODE => 0x05,
   LENGTH => 23,
   RESET  => [ 0x0000000 ],
   FIELDS => \%orbit_01_flds_00003
);

#-------------------------------------------------------------------------------


%orbit_01_flds_00004 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "NI0_CLK_I_TERM_H"    => [   0,   7,   1, 0x0000, "Term value for port ni0_i_clk_h" ],
   "NI0_CLK_I_TERM_L"    => [   1,   6,   1, 0x0000, "Term value for port ni0_i_clk_l" ],
   "NI0_DATA_F_TERM_H"   => [   2,   5,   1, 0x0000, "Term value for ports ni0_data_f_h[19:0], ni0_idr_b_of_h" ],
   "NI0_DATA_F_TERM_L"   => [   3,   4,   1, 0x0000, "Term value for ports ni0_data_f_l[19:0], ni0_idr_b_of_l" ],
   "NI1_CLK_I_TERM_H"    => [   4,   3,   1, 0x0000, "Term value for port ni1_i_clk_h" ],
   "NI1_CLK_I_TERM_L"    => [   5,   2,   1, 0x0000, "Term value for port ni1_i_clk_l" ],
   "NI1_DATA_F_TERM_H"   => [   6,   1,   1, 0x0000, "Term value for ports ni1_data_f_h[19:0], ni1_idr_b_of_h" ],
   "NI1_DATA_F_TERM_L"   => [   7,   0,   1, 0x0000, "Term value for ports ni1_data_f_l[19:0], ni1_idr_b_of_l" ]
);

%orbit_01_tdr_00004 =
(  NAME   => "NL3_TERM",
   NOTE   => "NL4 termination value selection",
   OPNAME => "NL3_TERM",
   OPCODE => 0x09,
   LENGTH => 8,
   RESET  => [ 0x0000000 ],
   FIELDS => \%orbit_01_flds_00004
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00005 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "START"               => [   0,  29,   1, 0x0000, "Start PIO operation to addressed MMR" ],
   "WRITE"               => [   1,  28,   1, 0x0000, "Operation is a write (1=write, 0=read)" ],
   "PIO_ADDRESS"         => [   2,   0,  28, 0x0000, "Physical address of MMR to access" ]
);

%orbit_01_tdr_00005 =
(  NAME   => "PIO_CTRL",
   NOTE   => "Control Register for PIO",
   OPNAME => "PIO_CTRL",
   OPCODE => 0x0F,
   LENGTH => 30,
   RESET  => [ 0x0000000 ],
   FIELDS => \%orbit_01_flds_00005
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00006 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ERROR"               => [   0,  65,   1, 0x0000, "Error occured with the PIO operation"   ],
   "VALID"               => [   1,  64,   1, 0x0000, "Read data is valid; operation complete" ],
   "DATA"                => [   2,   0,  64, 0x0000, "Read or write data from/for MMR"        ]
);

%orbit_01_tdr_00006 =
(  NAME   => "PIO_DATA",
   NOTE   => "Data Register for PIO",
   OPNAME => "PIO_DATA",
   OPCODE => 0x10,
   LENGTH => 66,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00006
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00007 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "MABIST_BIST_M3"      => [   0,  24,   2, 0x0000, "Reset active indicator (1=active)"     ],
   "MABIST_ABDONE"       => [   1,  18,   6, 0x0000, "Done bit per BIST controller (1=done)" ],
   "MABIST_SYSPF"        => [   2,   0,  18, 0x0000, "Status bit per SRAM (pass/fail)"       ]
);

%orbit_01_tdr_00007 =
(  NAME   => "RUNABIST",
   NOTE   => "SRAM RUNABIST control",
   OPNAME => "RUNABIST",
   OPCODE => 0x08,
   LENGTH => 26,
   RESET  => [ 0x0000000 ],
   FIELDS => \%orbit_01_flds_00007
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00008 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "JTAG_MCI_L_LOCK"   => [   0,  70,   9, 0x0000, "MCI left side lock state (observe-only)" ],
   "JTAG_MCI_R_LOCK"   => [   1,  61,   9, 0x0000, "MCI right side lock state (observe-only)" ],
   "WRT90_LOCK"        => [   2,  60,   1, 0x0000, "WRT90 lock state (observe-only)" ],
   "LOAD"              => [   3,  56,   4, 0x0000, "Select clock domain for GMXC clocks" ],
   "GMXC_SEQ"          => [   4,  36,  20, 0x0000, "Select number clocks thru (0=1 clk)" ],
   "SLOW1CLK"          => [   5,  35,   1, 0x0000, "Set for 1 slow clock for selected chain" ],
   "SLOW2CLK"          => [   6,  34,   1, 0x0000, "Set for 2 slow clocks for selected chain" ],
   "GMXC_TDR_SEL"      => [   7,   8,  26, 0x0000, "Select internal chain for GMXC clocks" ],
   "TP"                => [   8,   2,   6, 0x0000, "Select clock test point signals"    ],
   "SEL_CLKSTOP"       => [   9,   1,   1, 0x0000, "Select stop clock (0=JTAG, 1=core)" ],
   "JTAG_CLKSTOP"      => [  10,   0,   1, 0x0000, "JTAG stop system clock" ],
);

%orbit_01_tdr_00008 =
(  NAME   => "CLKCNTL",
   NOTE   => "JTAG CLKCNTL control Register",
   OPNAME => "CLKCNTL",
   OPCODE => 0x07,
   LENGTH => 79,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00008
);

#-------------------------------------------------------------------------------


%orbit_01_flds_00009 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "NI0_N_REFIO_VALUE"   => [   0, 117,   9, 0x0000, "description"            ],
   "NI0_P_REFIO_VALUE"   => [   1, 108,   9, 0x0000, "description"            ],
   "NI1_N_REFIO_VALUE"   => [   2,  99,   9, 0x0000, "description"            ],
   "NI1_P_REFIO_VALUE"   => [   3,  90,   9, 0x0000, "description"            ],
   "FSB_N_REFIO_VALUE"   => [   4,  81,   9, 0x0000, "description"            ],
   "FSB_P_REFIO_VALUE"   => [   5,  72,   9, 0x0000, "description"            ],
   "FSB_FILTER"          => [   6,  71,   1, 0x0000, "description"            ],
   "FSB_BYPASS"          => [   7,  70,   1, 0x0000, "description"            ],
   "FSB_DISABLE"         => [   8,  69,   1, 0x0000, "description"            ],
   "FSB_RESET"           => [   9,  68,   1, 0x0000, "description"            ],
   "FSB_N_TDR_VALUE"     => [  10,  60,   8, 0x0000, "description"            ],
   "FSB_P_TDR_VALUE"     => [  11,  52,   8, 0x0000, "description"            ],
   "FSB_SEL_TDR_VAL"     => [  12,  51,   1, 0x0000, "description"            ],
   "FSB_N_UPDN"          => [  13,  50,   1, 0x0000, "description"            ],
   "FSB_P_UPDN"          => [  14,  49,   1, 0x0000, "description"            ],
   "FSB_OVRD_UPDN"       => [  15,  48,   1, 0x0000, "description"            ],
   "NI1_FILTER"          => [  16,  47,   1, 0x0000, "description"            ],
   "NI1_BYPASS"          => [  17,  46,   1, 0x0000, "description"            ],
   "NI1_DISABLE"         => [  18,  45,   1, 0x0000, "description"            ],
   "NI1_RESET"           => [  19,  44,   1, 0x0000, "description"            ],
   "NI1_N_TDR_VALUE"     => [  20,  36,   8, 0x0000, "description"            ],
   "NI1_P_TDR_VALUE"     => [  21,  28,   8, 0x0000, "description"            ],
   "NI1_SEL_TDR_VAL"     => [  22,  27,   1, 0x0000, "description"            ],
   "NI1_N_UPDN"          => [  23,  26,   1, 0x0000, "description"            ],
   "NI1_P_UPDN"          => [  24,  25,   1, 0x0000, "description"            ],
   "NI1_OVRD_UPDN"       => [  25,  24,   1, 0x0000, "description"            ],
   "NI0_FILTER"          => [  26,  23,   1, 0x0000, "description"            ],
   "NI0_BYPASS"          => [  27,  22,   1, 0x0000, "description"            ],
   "NI0_DISABLE"         => [  28,  21,   1, 0x0000, "description"            ],
   "NI0_RESET"           => [  29,  20,   1, 0x0000, "description"            ],
   "NI0_N_TDR_VALUE"     => [  30,  12,   8, 0x0000, "description"            ],
   "NI0_P_TDR_VALUE"     => [  31,   4,   8, 0x0000, "description"            ],
   "NI0_SEL_TDR_VAL"     => [  32,   3,   1, 0x0000, "description"            ],
   "NI0_N_UPDN"          => [  33,   2,   1, 0x0000, "description"            ],
   "NI0_P_UPDN"          => [  34,   1,   1, 0x0000, "description"            ],
   "NI0_OVRD_UPDN"       => [  35,   0,   1, 0x0000, "description"            ]
);

%orbit_01_tdr_00009 =
(  NAME   => "ICNTLR",
   NOTE   => "Impedance Controller for APF and GTL I/O",
   OPNAME => "ICNTLR",
   OPCODE => 0x06,
   LENGTH => 126,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00009
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00010 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "DAVAIL_SEL"          => [   0,   8,   2, 0x0000, "Control timing of data available signal" ],
   "FMODE_JTAG_VAL"      => [   1,   7,   1, 0x0000, "JTAG FMODE value"                        ],
   "FMODE_JTAG_EN"       => [   2,   6,   1, 0x0000, "Enable JTAG control of FMODE bit"        ],
   "UPDATE_AFTER_CNT_EN" => [   3,   5,   1, 0x0000, "" ],
   "LLP_TUNING_CMPLT_EN" => [   4,   4,   1, 0x0000, "" ],
   "RMT_UPDATE_EN"       => [   5,   3,   1, 0x0000, "Enable remote fine update (via LLP)"     ],
   "DLINE_MAN_ADJ_EN"    => [   6,   2,   1, 0x0000, "Enable manual adjust mode"               ],
   "DLINE_MAN_HOLD"      => [   7,   1,   1, 0x0000, "Hold current delay value"                ],
   "DLY_CTRL_RESET"      => [   8,   0,   1, 0x0000, "Hold delay controller in reset if set"   ]
);

%orbit_01_tdr_00010 =
(  NAME   => "DLY_CTRL0_NI0",
   NOTE   => "DLY_CTRL0_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI0",
   OPCODE => 0x60,
   LENGTH => 10,
   RESET  => [ 0x0000000 ],
   FIELDS => \%orbit_01_flds_00010
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00011 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%orbit_01_tdr_00011 =
(  NAME   => "DLY_CTRL1_NI0",
   NOTE   => "DLY_CTRL1_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI0",
   OPCODE => 0x61,
   LENGTH => 33,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00011
);

#-------------------------------------------------------------------------------


%orbit_01_flds_00012 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "DAVAIL_SEL"          => [   0,   8,   2, 0x0000, "Control timing of data available signal" ],
   "FMODE_JTAG_VAL"      => [   1,   7,   1, 0x0000, "JTAG FMODE value"                        ],
   "FMODE_JTAG_EN"       => [   2,   6,   1, 0x0000, "Enable JTAG control of FMODE bit"        ],
   "UPDATE_AFTER_CNT_EN" => [   3,   5,   1, 0x0000, "" ],
   "LLP_TUNING_CMPLT_EN" => [   4,   4,   1, 0x0000, "" ],
   "RMT_UPDATE_EN"       => [   5,   3,   1, 0x0000, "Enable remote fine update (via LLP)"     ],
   "DLINE_MAN_ADJ_EN"    => [   6,   2,   1, 0x0000, "Enable manual adjust mode"               ],
   "DLINE_MAN_HOLD"      => [   7,   1,   1, 0x0000, "Hold current delay value"                ],
   "DLY_CTRL_RESET"      => [   8,   0,   1, 0x0000, "Hold delay controller in reset if set"   ]
);

%orbit_01_tdr_00012 =
(  NAME   => "DLY_CTRL0_NI1",
   NOTE   => "DLY_CTRL0_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI1",
   OPCODE => 0x62,
   LENGTH => 10,
   RESET  => [ 0x0000000 ],
   FIELDS => \%orbit_01_flds_00012
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00013 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%orbit_01_tdr_00013 =
(  NAME   => "DLY_CTRL1_NI1",
   NOTE   => "DLY_CTRL1_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI1",
   OPCODE => 0x63,
   LENGTH => 33,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00013
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00014 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "LLP_RESET"         => [   0,  55,   1, 0x0000, "Load or initiate test"  ],
   "ERR_CAPTURE"       => [   1,  54,   1, 0x0000, "Capture 1st mpkt w/ error" ],
   "CLEAR"             => [   2,  53,   1, 0x0000, "Clear and arm"          ],
   "FLIT_SELECT"       => [   3,  50,   3, 0x0000, "Flit selection"         ],
   "CRC_ERROR"         => [   4,  49,   1, 0x0000, "CRC error detected"     ],
   "VALID"             => [   5,  48,   1, 0x0000, "Valid flag"             ],
   "DATA"              => [   6,  28,  20, 0x0000, "Captured flit data"     ],
   "MASK"              => [   7,  20,   8, 0x0000, "Mask for packet data"   ],
   "SEED"              => [   8,   0,  20, 0x0000, "Seed for packet data"   ],
);

%orbit_01_tdr_00014 =
(  NAME   => "LLP_II",
   NOTE   => "LLP II Channel Bist",
   OPNAME => "LLP_II",
   OPCODE => 0x0C,
   LENGTH => 56,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00014
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00015 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ENABLE_USER_IGNORE"  => [   0,  55,   1, 0x0000, "Enable user ignore timer" ],
   "USER_IGNORE_TIME"    => [   1,  15,  40, 0x0000, "Ignore reset interval (if enabled)" ],
   "LINK_RESET_ENABLE"   => [   2,  14,   1, 0x0000, "Enable link reset mode" ],
   "QUEUE_CNT_ENABLE"    => [   3,  13,   1, 0x0000, "Enable queue counters to adjust initial credits" ],
   "CHARGE_UNDERFLOW"    => [   4,  12,   1, 0x0000, "DC component in AC endocde data is too small" ],
   "CHARGE_OVERFLOW"     => [   5,  11,   1, 0x0000, "DC component in AC encoded data is too large" ],
   "FAKED_ALL_SN_ERRORS" => [   6,  10,   1, 0x0000, "Asserted when fake SN counter reaches zero" ],
   "SENT_ALL_SN_ERRORS"  => [   7,   9,   1, 0x0000, "Asserted when SN counter reaches zero" ],
   "SENT_ALL_CB_ERRORS"  => [   8,   8,   1, 0x0000, "Asserted when CB counter reaches zero" ],
   "FAKE_SN_ERROR"       => [   9,   7,   1, 0x0000, "SSR pretends it received a bad SN" ],
   "SEND_SN_ERROR"       => [  10,   6,   1, 0x0000, "Inc. SN counter; SN errors may be forced" ],
   "SEND_CB_ERROR"       => [  11,   5,   1, 0x0000, "Inc. CB counter; CB errors may be forced" ],
   "ENABLE_QUAD"         => [  12,   4,   1, 0x0000, "Enable sample time select" ],
   "ENABLE_RFTU"         => [  13,   3,   1, 0x0000, "Enable fine tune update" ],
   "ENABLE_TUNING"       => [  14,   2,   1, 0x0000, "Enable PFSSR tuning" ],
   "ENABLE_AC"           => [  15,   1,   1, 0x0000, "Enable AC encoding" ],
   "BIT_MODE32"          => [  16,   0,   1, 0x0000, "LLP should use 32 bit mode (0=16b)" ],
);

%orbit_01_tdr_00015 =
(  NAME   => "LLP4_NI0",
   NOTE   => "NI0 LLP4 Configuration Data",
   OPNAME => "LLP4_NI0",
   OPCODE => 0x0A,
   LENGTH => 56,
   RESET  => [ 0x0000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00015
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00016 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ENABLE_USER_IGNORE"  => [   0,  55,   1, 0x0000, "Enable user ignore timer" ],
   "USER_IGNORE_TIME"    => [   1,  15,  40, 0x0000, "Ignore reset interval (if enabled)" ],
   "LINK_RESET_ENABLE"   => [   2,  14,   1, 0x0000, "Enable link reset mode" ],
   "QUEUE_CNT_ENABLE"    => [   3,  13,   1, 0x0000, "Enable queue counters to adjust initial credits" ],
   "CHARGE_UNDERFLOW"    => [   4,  12,   1, 0x0000, "DC component in AC endocde data is too small" ],
   "CHARGE_OVERFLOW"     => [   5,  11,   1, 0x0000, "DC component in AC encoded data is too large" ],
   "FAKED_ALL_SN_ERRORS" => [   6,  10,   1, 0x0000, "Asserted when fake SN counter reaches zero" ],
   "SENT_ALL_SN_ERRORS"  => [   7,   9,   1, 0x0000, "Asserted when SN counter reaches zero" ],
   "SENT_ALL_CB_ERRORS"  => [   8,   8,   1, 0x0000, "Asserted when CB counter reaches zero" ],
   "FAKE_SN_ERROR"       => [   9,   7,   1, 0x0000, "SSR pretends it received a bad SN" ],
   "SEND_SN_ERROR"       => [  10,   6,   1, 0x0000, "Inc. SN counter; SN errors may be forced" ],
   "SEND_CB_ERROR"       => [  11,   5,   1, 0x0000, "Inc. CB counter; CB errors may be forced" ],
   "ENABLE_QUAD"         => [  12,   4,   1, 0x0000, "Enable sample time select" ],
   "ENABLE_RFTU"         => [  13,   3,   1, 0x0000, "Enable fine tune update" ],
   "ENABLE_TUNING"       => [  14,   2,   1, 0x0000, "Enable PFSSR tuning" ],
   "ENABLE_AC"           => [  15,   1,   1, 0x0000, "Enable AC encoding" ],
   "BIT_MODE32"          => [  16,   0,   1, 0x0000, "LLP should use 32 bit mode (0=16b)" ],
);

%orbit_01_tdr_00016 =
(  NAME   => "LLP4_NI1",
   NOTE   => "NI1 LLP4 Configuration Data",
   OPNAME => "LLP4_NI1",
   OPCODE => 0x0B,
   LENGTH => 56,
   RESET  => [ 0x0000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00016
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00017 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%orbit_01_tdr_00017 =
(  NAME   => "LLP4TEST_NI0",
   NOTE   => "LLP NI0 Channel BIST",
   OPNAME => "LLP4TEST_NI0",
   OPCODE => 0x1A,
   LENGTH => 242,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00017
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00018 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%orbit_01_tdr_00018 =
(  NAME   => "LLP4TEST_NI1",
   NOTE   => "LLP NI1 Channel BIST",
   OPNAME => "LLP4TEST_NI1",
   OPCODE => 0x1B,
   LENGTH => 242,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00018
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00019 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%orbit_01_tdr_00019 =
(  NAME   => "TDR_DEFAULT",
   NOTE   => "Reset user defined TDR's",
   OPNAME => "TDR_DEFAULT",
   OPCODE => 0x7C,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%orbit_01_flds_00019
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00020 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%orbit_01_tdr_00020 =
(  NAME   => "JCLK_RST",
   NOTE   => "Clock Controller Reset Instruction",
   OPNAME => "JCLK_RST",
   OPCODE => 0x7D,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%orbit_01_flds_00020
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00021 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%orbit_01_tdr_00021 =
(  NAME   => "HIGHZ",
   NOTE   => "HIGHZ instruction",
   OPNAME => "HIGHZ",
   OPCODE => 0x04,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%orbit_01_flds_00021
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00022 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "NI0_PFSSD"         => [   0,   5,   1, 0x0000, "Enable internal scan chain" ],
   "NI1_PFSSD"         => [   1,   4,   1, 0x0000, "Enable internal scan chain" ],
   "II_NGSSD20"        => [   2,   3,   1, 0x0000, "Enable internal scan chain" ],
   "NI0_PFSSR"         => [   3,   2,   1, 0x0000, "Enable internal scan chain" ],
   "NI1_PFSSR"         => [   4,   1,   1, 0x0000, "Enable internal scan chain" ],
   "II_NGSSR20"        => [   5,   0,   1, 0x0000, "Enable internal scan chain" ]
);

%orbit_01_tdr_00022 =
(  NAME   => "SCAN_CHAN",
   NOTE   => "SCAN_CHAN instruction",
   OPNAME => "SCAN_CHAN",
   OPCODE => 0x39,
   LENGTH => 6,
   RESET  => [ 0x00000000 ],
   FIELDS => \%orbit_01_flds_00022
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00023 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%orbit_01_tdr_00023 =
(  NAME   => "ASIC_L1_DATA",
   NOTE   => "Orbit to L1 data register",
   OPNAME => "ASIC_L1_DATA",
   OPCODE => 0x11,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00023
);

#-------------------------------------------------------------------------------

%orbit_01_flds_00024 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%orbit_01_tdr_00024 =
(  NAME   => "L1_ASIC_DATA",
   NOTE   => "L1 to orbit data register",
   OPNAME => "L1_ASIC_DATA",
   OPCODE => 0x12,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%orbit_01_flds_00024
);

#-------------------------------------------------------------------------------

#-----------------------------------
#  HASH TABLE: LOOKUP TDR BY OPNAME
#-----------------------------------

%orbit_01_tdrs =
(  $orbit_01_tdr_00000{OPNAME} => \%orbit_01_tdr_00000,
   $orbit_01_tdr_00001{OPNAME} => \%orbit_01_tdr_00001,
   $orbit_01_tdr_00002{OPNAME} => \%orbit_01_tdr_00002,
   $orbit_01_tdr_00003{OPNAME} => \%orbit_01_tdr_00003,
   $orbit_01_tdr_00004{OPNAME} => \%orbit_01_tdr_00004,
   $orbit_01_tdr_00005{OPNAME} => \%orbit_01_tdr_00005,
   $orbit_01_tdr_00006{OPNAME} => \%orbit_01_tdr_00006,
   $orbit_01_tdr_00007{OPNAME} => \%orbit_01_tdr_00007,
   $orbit_01_tdr_00008{OPNAME} => \%orbit_01_tdr_00008,
   $orbit_01_tdr_00009{OPNAME} => \%orbit_01_tdr_00009,
   $orbit_01_tdr_00010{OPNAME} => \%orbit_01_tdr_00010,
   $orbit_01_tdr_00011{OPNAME} => \%orbit_01_tdr_00011,
   $orbit_01_tdr_00012{OPNAME} => \%orbit_01_tdr_00012,
   $orbit_01_tdr_00013{OPNAME} => \%orbit_01_tdr_00013,
   $orbit_01_tdr_00014{OPNAME} => \%orbit_01_tdr_00014,
   $orbit_01_tdr_00015{OPNAME} => \%orbit_01_tdr_00015,
   $orbit_01_tdr_00016{OPNAME} => \%orbit_01_tdr_00016,
   $orbit_01_tdr_00017{OPNAME} => \%orbit_01_tdr_00017,
   $orbit_01_tdr_00018{OPNAME} => \%orbit_01_tdr_00018,
   $orbit_01_tdr_00019{OPNAME} => \%orbit_01_tdr_00019,
   $orbit_01_tdr_00020{OPNAME} => \%orbit_01_tdr_00020,
   $orbit_01_tdr_00021{OPNAME} => \%orbit_01_tdr_00021,
   $orbit_01_tdr_00022{OPNAME} => \%orbit_01_tdr_00022,
   $orbit_01_tdr_00023{OPNAME} => \%orbit_01_tdr_00023,
   $orbit_01_tdr_00024{OPNAME} => \%orbit_01_tdr_00024
);


#-----------------------------------
#  ARRAY: STORE OPNAMES IN "ORDER"
#-----------------------------------

@orbit_01_opnames =
(  $orbit_01_tdr_00000{OPNAME},
   $orbit_01_tdr_00001{OPNAME},
   $orbit_01_tdr_00002{OPNAME},
   $orbit_01_tdr_00003{OPNAME},
   $orbit_01_tdr_00004{OPNAME},
   $orbit_01_tdr_00005{OPNAME},
   $orbit_01_tdr_00006{OPNAME},
   $orbit_01_tdr_00007{OPNAME},
   $orbit_01_tdr_00008{OPNAME},
   $orbit_01_tdr_00009{OPNAME},
   $orbit_01_tdr_00010{OPNAME},
   $orbit_01_tdr_00011{OPNAME},
   $orbit_01_tdr_00012{OPNAME},
   $orbit_01_tdr_00013{OPNAME},
   $orbit_01_tdr_00014{OPNAME},
   $orbit_01_tdr_00015{OPNAME},
   $orbit_01_tdr_00016{OPNAME},
   $orbit_01_tdr_00017{OPNAME},
   $orbit_01_tdr_00018{OPNAME},
   $orbit_01_tdr_00019{OPNAME},
   $orbit_01_tdr_00020{OPNAME},
   $orbit_01_tdr_00021{OPNAME},
   $orbit_01_tdr_00022{OPNAME},
   $orbit_01_tdr_00023{OPNAME},
   $orbit_01_tdr_00024{OPNAME}
);

#-------------------------------------------------------------------------------

1;
