#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: orbit_subs.pm,v 1.1 2003/04/29 18:31:35 beighley Exp $
#
#-------------------------------------------------------------------------------
#
#  FILE: orbit_subs.pm
#
#  DESCRIPTION:  This file contains PERL constants and subroutines that are
#     specific to the SN2 ORBIT ASIC.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     04/29/03  Karen Beighley  Original source code submitted into tree.
#
#-------------------------------------------------------------------------------

$USE_ORBIT_SUBS_PM = 1;    # Define to allow test for module inclusion

#-------------------------------------------------------------------------------
#  PACKET GENERATOR SPECIFICS - The order of the address array cannot change!
#-------------------------------------------------------------------------------

$ORBIT_CHIPLET_MD = 0;
$ORBIT_CHIPLET_LB = 1;
$ORBIT_CHIPLET_PI = 2;
$ORBIT_CHIPLET_II = 3;

@orbit_pktgen_addr =
( 0x0000000090060E80, # OC_ORBIT_ID (NODE_ID field @ bit 32)
  0x0000000090063000, # OC_DIAG_MSG_CNTRL
  0x0000000090063980, # OC_DIAG_MSG_HDR0
  0x0000000090063A00, # OC_DIAG_MSG_HDR1
  0x0000000090063080, # OC_DIAG_MSG_DATA0L
  0x0000000090063100, # OC_DIAG_MSG_DATA0U
  0x0000000090063180, # OC_DIAG_MSG_DATA1L
  0x0000000090063200, # OC_DIAG_MSG_DATA1U
  0x0000000090063280, # OC_DIAG_MSG_DATA2L
  0x0000000090063300, # OC_DIAG_MSG_DATA2U
  0x0000000090063380, # OC_DIAG_MSG_DATA3L
  0x0000000090063400, # OC_DIAG_MSG_DATA3U
  0x0000000090063480, # OC_DIAG_MSG_DATA4L
  0x0000000090063500, # OC_DIAG_MSG_DATA4U
  0x0000000090063580, # OC_DIAG_MSG_DATA5L
  0x0000000090063600, # OC_DIAG_MSG_DATA5U
  0x0000000090063680, # OC_DIAG_MSG_DATA6L
  0x0000000090063700, # OC_DIAG_MSG_DATA6U
  0x0000000090063780, # OC_DIAG_MSG_DATA7L
  0x0000000090063800, # OC_DIAG_MSG_DATA7U
  0x0000000090063880, # OC_DIAG_MSG_DATA8L
  0x0000000090063900  # OC_DIAG_MSG_DATA8U
);


#-------------------------------------------------------------------------------
#  ERROR REGISTER SPECIFICS - The order of the address array cannot change!
#-------------------------------------------------------------------------------

@orbit_ereg_addr =
(  0x0000000090050700, # OC_LB_FIRST_ERROR
   0x0000000090050280, # OC_LB_ERROR_DETAIL_1
   0x0000000090050300, # OC_LB_ERROR_DETAIL_2
   0x0000000090050380, # OC_LB_ERROR_DETAIL_3
   0x0000000090050400, # OC_LB_ERROR_DETAIL_4
   0x0000000090050480, # OC_LB_ERROR_DETAIL_5
   0x0000000090050680, # OC_LB_ERROR_SUMMARY
   0x0000000090050600, # OC_LB_ERROR_OVERFLOW
   0x0000000090050580, # OC_LB_ERROR_MASK
   0x0000000090050688, # OC_LB_ERROR_SUMMARY_ALIAS
   0x0000000090050608, # OC_LB_ERROR_OVERFLOW_ALIAS
);


#------------------------------------------------------------------------------
#
#    SUBROUTINE: orbit_pktgen_get_ids()
#
#    USAGE: @vals = orbit_pktgen_get_ids($op, $addr)
#           @vals = orbit_pktgen_get_ids($op, $addr, $src_id)
#           @vals = orbit_pktgen_get_ids($op, $addr, $src_id, $dst_id)
#           @vals = orbit_pktgen_get_ids($op, $addr, $src_id, $dst_id, $dst_cid)
#
#    DESCRIPTION:  This subroutine will determine the source and
#       destination IDs required for a packet generator request header.  The
#       source node ID, source chiplet ID, destination node ID and destination
#       chiplet ID are returned.  If no parameters are given, the chip's
#       NODE_ID MMR is read, and the NODE_ID is extracted.  If the destination
#       node ID is not provided, then the source ID is used.  If the destination
#       chiplet ID is not provided, then it is determined from the operation
#       type ('mem' or 'mmr') and address.
#
#    RETURN VALUE: Array containing source and destination IDs.
#
#------------------------------------------------------------------------------

sub orbit_pktgen_get_ids
{
   my $num = $#_ + 1;

   printf "\norbit_pktgen_get_ids(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( ($num < 2) || ($num > 5) )
   {  $emess = "Invalid parameters passed to subroutine orbit_pktgen_get_ids().";
      $ecode = -1;
      return;
   }
   my $op   = $_[0];
   my $addr = $_[1];
   my ($tmp, $src_id, $src_chiplet, $dst_id, $dst_chiplet);

   #--------------------------------------- 
   #  Determine the NODE IDs
   #--------------------------------------- 
   if ( $num == 2 ) # No source or destination: read local ID
   {  $tmp = mmr_read($orbit_pktgen_addr[$PKTGEN_IND_ID]);
      return if ( $ecode < 0 );
      $src_id = ($tmp >> 32) & 0x3FFF;
      $dst_id = $src_id;
   }
   elsif ( $num == 3 ) # Source ID provided: use for destination also
   {  $src_id = $_[2];
      $dst_id = $src_id;
   }
   else # Both source and destination ID provided
   {  $src_id = $_[2];
      $dst_id = $_[3];
   }

   #--------------------------------------- 
   #  Determine the CHIPLET IDs
   #--------------------------------------- 
   $src_chiplet = $ORBIT_CHIPLET_LB;
   if ( $num == 5 )
   {  $dst_chiplet = $_[4];
   }
   elsif ( $op eq 'mem' )
   {  $dst_chiplet = $ORBIT_CHIPLET_MD;
   }
   else
   {  $dst_chiplet = (($addr & 0x100000000) ? $ORBIT_CHIPLET_LB : $ORBIT_CHIPLET_II);
   }
   return($src_id, $src_chiplet, $dst_id, $dst_chiplet);
}


#------------------------------------------------------------------------------

1;

