# 
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
# 
# @(#)$Id: pic_01_jtdrs.pm,v 1.1 2002/04/15 16:21:13 beighley Exp $

#-------------------------------------------------------------------------------
#
#  FILE: pic_01_jtdrs.pm
#
#  NOTE: Definitions of the JTAG test data registers (TDRs) for PIC.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     04/05/02  Karen Beighley  Original source code submitted into tree.
#
#-------------------------------------------------------------------------------

$pic_01_tdr_defined = 0;

#-------------------------------------------------------------------------------

%pic_01_flds_00000 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00000 =
(  NAME   => "BYPASS",
   NOTE   => "BYPASS JTAG Register",
   OPNAME => "BYPASS",
   OPCODE => 0xFF,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00000
);

#-------------------------------------------------------------------------------

%pic_01_flds_00001 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "VERSION"           => [   0,  28,   4, 0x0000, "Version number"         ],
   "PARTNUMBER"        => [   1,  12,  16, 0x0000, "Partnumber"             ],
   "MANUFACTURER"      => [   2,   1,  11, 0x0000, "Manufacturer Code"      ],
   "LSB"               => [   3,   0,   1, 0x0000, "Mandatory LSB"          ]
);

%pic_01_tdr_00001 =
(  NAME   => "IDCODE",
   NOTE   => "JTAG Device Identification Register",
   OPNAME => "IDCODE",
   OPCODE => 0x03,
   LENGTH => 32,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00001
);

#-------------------------------------------------------------------------------

%pic_01_flds_00002 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00002 =
(  NAME   => "GMR",
   NOTE   => "Give-Me-Reset (GMR) Instruction",
   OPNAME => "GMR",
   OPCODE => 0x30,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00002
);

#-------------------------------------------------------------------------------

%pic_01_flds_00003 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD0"             => [   0,  57,   7, 0x0000, "Reserved"                  ],
   "MODE"              => [   1,  56,   1, 0x0000, "Load or initiate test"     ],
   "ERR_CAPTURE"       => [   2,  55,   1, 0x0000, "Capture 1st mpkt w/ error" ],
   "CLEAR"             => [   3,  54,   1, 0x0000, "Clear and arm"             ],
   "FLIT_SELECT"       => [   4,  51,   3, 0x0000, "Flit selection"            ],
   "CRC_ERROR"         => [   5,  50,   1, 0x0000, "CRC error detected"        ],
   "VALID"             => [   6,  49,   1, 0x0000, "Valid flag"                ],
   "DATA"              => [   7,  29,  20, 0x0000, "Captured flit data"        ],
   "MASK"              => [   8,  21,   8, 0x0000, "Mask for packet data"      ],
   "SEED"              => [   9,   1,  20, 0x0000, "Seed for packet data"      ],
   "RSVD1"             => [  10,   0,   1, 0x0000, "Reserved"                  ]
);

%pic_01_tdr_00003 =
(  NAME   => "LLP_TEST_A",
   NOTE   => "LLP A channel test register",
   OPNAME => "LLP_TEST_A",
   OPCODE => 0x40,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00003
);

#-------------------------------------------------------------------------------

%pic_01_flds_00004 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD0"             => [   0,  57,   7, 0x0000, "Reserved"                  ],
   "MODE"              => [   1,  56,   1, 0x0000, "Load or initiate test"     ],
   "ERR_CAPTURE"       => [   2,  55,   1, 0x0000, "Capture 1st mpkt w/ error" ],
   "CLEAR"             => [   3,  54,   1, 0x0000, "Clear and arm"             ],
   "FLIT_SELECT"       => [   4,  51,   3, 0x0000, "Flit selection"            ],
   "CRC_ERROR"         => [   5,  50,   1, 0x0000, "CRC error detected"        ],
   "VALID"             => [   6,  49,   1, 0x0000, "Valid flag"                ],
   "DATA"              => [   7,  29,  20, 0x0000, "Captured flit data"        ],
   "MASK"              => [   8,  21,   8, 0x0000, "Mask for packet data"      ],
   "SEED"              => [   9,   1,  20, 0x0000, "Seed for packet data"      ],
   "RSVD1"             => [  10,   0,   1, 0x0000, "Reserved"                  ]
);

%pic_01_tdr_00004 =
(  NAME   => "LLP_TEST_B",
   NOTE   => "LLP B channel test register",
   OPNAME => "LLP_TEST_B",
   OPCODE => 0x41,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00004
);

#-------------------------------------------------------------------------------

%pic_01_flds_00005 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD0"             => [   0,  57,   7, 0x0000, "Reserved"                  ],
   "MODE"              => [   1,  56,   1, 0x0000, "Load or initiate test"     ],
   "ERR_CAPTURE"       => [   2,  55,   1, 0x0000, "Capture 1st mpkt w/ error" ],
   "CLEAR"             => [   3,  54,   1, 0x0000, "Clear and arm"             ],
   "FLIT_SELECT"       => [   4,  51,   3, 0x0000, "Flit selection"            ],
   "CRC_ERROR"         => [   5,  50,   1, 0x0000, "CRC error detected"        ],
   "VALID"             => [   6,  49,   1, 0x0000, "Valid flag"                ],
   "DATA"              => [   7,  29,  20, 0x0000, "Captured flit data"        ],
   "MASK"              => [   8,  21,   8, 0x0000, "Mask for packet data"      ],
   "SEED"              => [   9,   1,  20, 0x0000, "Seed for packet data"      ],
   "RSVD1"             => [  10,   0,   1, 0x0000, "Reserved"                  ]
);

%pic_01_tdr_00005 =
(  NAME   => "LLP_TEST_C",
   NOTE   => "LLP C channel test register",
   OPNAME => "LLP_TEST_C",
   OPCODE => 0x42,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00005
);

#-------------------------------------------------------------------------------

%pic_01_flds_00006 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD0"             => [   0,  57,   7, 0x0000, "Reserved"                  ],
   "MODE"              => [   1,  56,   1, 0x0000, "Load or initiate test"     ],
   "ERR_CAPTURE"       => [   2,  55,   1, 0x0000, "Capture 1st mpkt w/ error" ],
   "CLEAR"             => [   3,  54,   1, 0x0000, "Clear and arm"             ],
   "FLIT_SELECT"       => [   4,  51,   3, 0x0000, "Flit selection"            ],
   "CRC_ERROR"         => [   5,  50,   1, 0x0000, "CRC error detected"        ],
   "VALID"             => [   6,  49,   1, 0x0000, "Valid flag"                ],
   "DATA"              => [   7,  29,  20, 0x0000, "Captured flit data"        ],
   "MASK"              => [   8,  21,   8, 0x0000, "Mask for packet data"      ],
   "SEED"              => [   9,   1,  20, 0x0000, "Seed for packet data"      ],
   "RSVD1"             => [  10,   0,   1, 0x0000, "Reserved"                  ]
);

%pic_01_tdr_00006 =
(  NAME   => "LLP_TEST_D",
   NOTE   => "LLP D channel test register",
   OPNAME => "LLP_TEST_D",
   OPCODE => 0x43,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00006
);

#-------------------------------------------------------------------------------

%pic_01_flds_00007 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00007 =
(  NAME   => "PIO_REQ",
   NOTE   => "PIO Request Instruction",
   OPNAME => "PIO_REQ",
   OPCODE => 0x10,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00007
);

#-------------------------------------------------------------------------------

%pic_01_flds_00008 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "CMD"               => [   0,   0,  64, 0x0000, ""                       ],
);

%pic_01_tdr_00008 =
(  NAME   => "PIO_CMD",
   NOTE   => "PIO Command Word Register",
   OPNAME => "PIO_CMD",
   OPCODE => 0x11,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00008
);

#-------------------------------------------------------------------------------

%pic_01_flds_00009 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%pic_01_tdr_00009 =
(  NAME   => "PIO_DATA",
   NOTE   => "PIO Data Word Register",
   OPNAME => "PIO_DATA",
   OPCODE => 0x12,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00009
);

#-------------------------------------------------------------------------------

%pic_01_flds_00010 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "ADDRESS"           => [   0,   0,  64, 0x0000, ""                       ],
);

%pic_01_tdr_00010 =
(  NAME   => "PIO_ADDR",
   NOTE   => "PIO Address Word Register",
   OPNAME => "PIO_ADDR",
   OPCODE => 0x13,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00010
);

#-------------------------------------------------------------------------------

%pic_01_flds_00011 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "XTALK_UPKT_MODE"   => [   0,  63,   1, 0x0000, "Xtalk packet issue mode"       ],
   "DEBUG_REG_SEL"     => [   1,  62,   1, 0x0000, "Debug register access mode"    ],
   "RSVD0"             => [   2,  60,   2, 0x0000, "Reserved"                      ],
   "DUMMY0"            => [   3,  59,   1, 0x0000, "Dummy_0"                       ],
   "ERROR"             => [   4,  58,   1, 0x0000, "Operation error flag"          ],
   "COMPLETE"          => [   5,  57,   1, 0x0000, "Operation complete"            ],
   "WRITE"             => [   6,  56,   1, 0x0000, "Operation: 1=write 0=read"     ],
   "RSVD1"             => [   7,  48,   8, 0x0000, "Reserved"                      ],
   "CMD_SB"            => [   8,  40,   8, 0x0000, "Xtalk command word sideband"   ],
   "DATA_SB"           => [   9,  32,   8, 0x0000, "Xtalk data word sideband"      ],
   "ADDR"              => [  10,   0,  32, 0x0000, "Register address"              ]
);

%pic_01_tdr_00011 =
(  NAME   => "PIO_CTRL",
   NOTE   => "PIO Control Register",
   OPNAME => "PIO_CTRL",
   OPCODE => 0x14,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00011
);

#-------------------------------------------------------------------------------

%pic_01_flds_00012 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00012 =
(  NAME   => "GMXC_CORE",
   NOTE   => "Give-Me-X-Clocks (GMXC_CORE) Instruction",
   OPNAME => "GMXC_CORE",
   OPCODE => 0x31,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00012
);

#-------------------------------------------------------------------------------

%pic_01_flds_00013 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00013 =
(  NAME   => "GMXC_BUS0",
   NOTE   => "BUS0 Give-Me-X-Clocks (GMXC_BUS0) Instruction",
   OPNAME => "GMXC_BUS0",
   OPCODE => 0x32,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00013
);

#-------------------------------------------------------------------------------

%pic_01_flds_00014 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00014 =
(  NAME   => "GMXC_BUS1",
   NOTE   => "BUS1 Give-Me-X-Clocks (GMXC_BUS1) Instruction",
   OPNAME => "GMXC_BUS1",
   OPCODE => 0x33,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00014
);


#-------------------------------------------------------------------------------

%pic_01_flds_00015 =
#  NAME                          IND  POS  LEN  MODE    DESCRIPTION
#  -------------------           ---  ---  ---  ------  -----------------------
(  "GMXC_ABIST"             => [   0,  63,   1, 0x0000, ""                       ],
   "RSVD0"                  => [   1,  44,  19, 0x0000, ""                       ],
   "GMXC_SEQ"               => [   2,  32,  12, 0x0000, ""                       ],
   "RSVD1"                  => [   3,  28,   4, 0x0000, ""                       ],
   "CLOCK_COUNTER_ENABLE"   => [   4,  27,   1, 0x0000, ""                       ],
   "CLOCK_COUNTER_2_ENABLE" => [   5,  26,   1, 0x0000, ""                       ],
   "CLOCK_COUNTER_MODE"     => [   6,  24,   2, 0x0000, ""                       ],
   "DUMMY3"                 => [   7,  23,   1, 0x0000, ""                       ],
   "DUMMY2"                 => [   8,  22,   1, 0x0000, ""                       ],
   "DUMMY1"                 => [   9,  21,   1, 0x0000, ""                       ],
   "DUMMY0"                 => [  10,  20,   1, 0x0000, ""                       ],
   "WRAPPER_TEST_MODE"      => [  11,  19,   1, 0x0000, ""                       ],
   "RSVD2"                  => [  12,  11,   8, 0x0000, ""                       ],
   "HALT_CORE_CLOCK"        => [  13,  10,   1, 0x0000, ""                       ],
   "HALT_PCI0_CLOCK"        => [  14,   9,   1, 0x0000, ""                       ],
   "HALT_PCI1_CLOCK"        => [  15,   8,   1, 0x0000, ""                       ],
   "RSVD3"                  => [  16,   6,   2, 0x0000, ""                       ],
   "FORCE_STOP_PCI0_CLOCK"  => [  17,   5,   1, 0x0000, ""                       ],
   "FORCE_STOP_PCI1_CLOCK"  => [  18,   4,   1, 0x0000, ""                       ],
   "RSVD4"                  => [  19,   2,   2, 0x0000, ""                       ],
   "RESET_PLL_PCI0"         => [  20,   1,   1, 0x0000, ""                       ],
   "RESET_PLL_PCI1"         => [  21,   0,   1, 0x0000, ""                       ]
);

%pic_01_tdr_00015 =
(  NAME   => "SC_CTRL",
   NOTE   => "SC_CTRL JTAG Register",
   OPNAME => "SC_CTRL",
   OPCODE => 0x08,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00015
);


#-------------------------------------------------------------------------------

%pic_01_flds_00016 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "STOPPED"           => [   0,  63,   1, 0x0000, "1=clock counter has stopped"        ],
   "C2_INUSE"          => [   1,  62,   1, 0x0000, "1=clock counter period 2 is in use" ],
   "RSVD"              => [   2,  40,  22, 0x0000, "Reserved"                           ],
   "PERIOD"            => [   3,   0,  40, 0x0000, "Period"                             ]
);

%pic_01_tdr_00016 =
(  NAME   => "CCNT1",
   NOTE   => "CCNT1 JTAG Register",
   OPNAME => "CCNT1",
   OPCODE => 0x18,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00016
);

#-------------------------------------------------------------------------------

%pic_01_flds_00017 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,  40,  24, 0x0000, "Reserved"               ],
   "PERIOD"            => [   1,   0,  40, 0x0000, "Period"                 ]
);

%pic_01_tdr_00017 =
(  NAME   => "CCNT2",
   NOTE   => "CCNT2 JTAG Register",
   OPNAME => "CCNT2",
   OPCODE => 0x19,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00017
);

#-------------------------------------------------------------------------------

%pic_01_flds_00018 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "MASK"              => [   0,  32,  32, 0x0000, ""                       ],
   "DATA"              => [   1,   0,  32, 0x0000, ""                       ]
);

%pic_01_tdr_00018 =
(  NAME   => "TRIGGER",
   NOTE   => "TRIGGER JTAG Register",
   OPNAME => "TRIGGER",
   OPCODE => 0x20,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%pic_01_flds_00018
);

#-------------------------------------------------------------------------------

%pic_01_flds_00019 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00019 =
(  NAME   => "HIGHZ",
   NOTE   => "HIGHZ JTAG Instruction",
   OPNAME => "HIGHZ",
   OPCODE => 0x04,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00019
);

#-------------------------------------------------------------------------------

%pic_01_flds_00020 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%pic_01_tdr_00020 =
(  NAME   => "CLAMP",
   NOTE   => "CLAMP JTAG Instruction",
   OPNAME => "CLAMP",
   OPCODE => 0x06,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%pic_01_flds_00020
);

#-------------------------------------------------------------------------------

#-----------------------------------
#  HASH TABLE: LOOKUP TDR BY OPNAME
#-----------------------------------

%pic_01_tdrs =
(  $pic_01_tdr_00000{OPNAME} => \%pic_01_tdr_00000,
   $pic_01_tdr_00001{OPNAME} => \%pic_01_tdr_00001,
   $pic_01_tdr_00002{OPNAME} => \%pic_01_tdr_00002,
   $pic_01_tdr_00003{OPNAME} => \%pic_01_tdr_00003,
   $pic_01_tdr_00004{OPNAME} => \%pic_01_tdr_00004,
   $pic_01_tdr_00005{OPNAME} => \%pic_01_tdr_00005,
   $pic_01_tdr_00006{OPNAME} => \%pic_01_tdr_00006,
   $pic_01_tdr_00007{OPNAME} => \%pic_01_tdr_00007,
   $pic_01_tdr_00008{OPNAME} => \%pic_01_tdr_00008,
   $pic_01_tdr_00009{OPNAME} => \%pic_01_tdr_00009,
   $pic_01_tdr_00010{OPNAME} => \%pic_01_tdr_00010,
   $pic_01_tdr_00011{OPNAME} => \%pic_01_tdr_00011,
   $pic_01_tdr_00012{OPNAME} => \%pic_01_tdr_00012,
   $pic_01_tdr_00013{OPNAME} => \%pic_01_tdr_00013,
   $pic_01_tdr_00014{OPNAME} => \%pic_01_tdr_00014,
   $pic_01_tdr_00015{OPNAME} => \%pic_01_tdr_00015,
   $pic_01_tdr_00016{OPNAME} => \%pic_01_tdr_00016,
   $pic_01_tdr_00017{OPNAME} => \%pic_01_tdr_00017,
   $pic_01_tdr_00018{OPNAME} => \%pic_01_tdr_00018,
   $pic_01_tdr_00019{OPNAME} => \%pic_01_tdr_00019,
   $pic_01_tdr_00020{OPNAME} => \%pic_01_tdr_00020
);

#-----------------------------------
#  ARRAY: STORE OPNAMES IN "ORDER"
#-----------------------------------

@pic_01_opnames =
(  $pic_01_tdr_00000{OPNAME},
   $pic_01_tdr_00001{OPNAME},
   $pic_01_tdr_00002{OPNAME},
   $pic_01_tdr_00003{OPNAME},
   $pic_01_tdr_00004{OPNAME},
   $pic_01_tdr_00005{OPNAME},
   $pic_01_tdr_00006{OPNAME},
   $pic_01_tdr_00007{OPNAME},
   $pic_01_tdr_00008{OPNAME},
   $pic_01_tdr_00009{OPNAME},
   $pic_01_tdr_00010{OPNAME},
   $pic_01_tdr_00011{OPNAME},
   $pic_01_tdr_00012{OPNAME},
   $pic_01_tdr_00013{OPNAME},
   $pic_01_tdr_00014{OPNAME},
   $pic_01_tdr_00015{OPNAME},
   $pic_01_tdr_00016{OPNAME},
   $pic_01_tdr_00017{OPNAME},
   $pic_01_tdr_00018{OPNAME},
   $pic_01_tdr_00019{OPNAME},
   $pic_01_tdr_00020{OPNAME}
);

#-------------------------------------------------------------------------------

1;
