#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
# 
# @(#)$Id: rad_subs.pm,v 1.4 2002/04/03 19:55:46 beighley Exp $


$rad_win_count = 0;  # Increment for each register access window!


################################################################################
#  The register lookup structure is a hash table where:
#     key   = full name of the register (<module>:<board>:<chip>:<reg>)
#     {key} = reference to register instance (from reg_create_mmr_inst())
#  The register lookup is a global structure holding the register instance
#     reference for each register being accessed by SuperScope
%reg_lookup = ();


################################################################################
#  The entry widget data (EWD) structure contains entries:
#  ( <MMR text>, <HEX text>, <field 0 text>, ..., <field N text> )
#  There is one EWD structure per MMR on a ASIC (shared if multiple windows)

$EWD_TEXT_MMR   = 0;	# text variable for entry widget: register
$EWD_TEXT_HEX  	= 1;	# text variable for entry widget: hex representation
$EWD_TEXT_FLD0  = 2;	# text variable for entry widget: field 0

#  The EWD lookup structure is a hash table where:
#     key   = full name of the register (<module>:<board>:<chip>:<reg>)
#     {key} = reference to two-element array: ( <count>, <EWD_reference> )
#
%ewd_lookup = ();

#  Example: $ewd_lookup{001c04:NODE:SHUB0:SH_II_INT0_CONFIG} = ARRAY(0x10a478c4)
#           ARRAY(0x10a478c4) --> ( 2 ARRAY(0x10a478f4) )
#           ARRAY(0x10a478f4) --> EWD reference which contains
#              --> ( 0x0000000000000000 0x0000 0x00000000 0x0 0x0000 0x0 0x0 )


################################################################################
#  The register access data (RAD) structure contains entries:
#  ( <ASIC name>, <reg_ref>, <frame_MMR>, <frame_fields>, <val_display>,
#    <val_select>, <entry_ref>)
#  There is one RAD structure per MMR on each window

$RAD_ASIC_NAME   = 0;   # text name of ASIC (i.e. "001c04:NODE:SHUB0")
$RAD_REG_REF     = 1;   # reference to register instance (from create_*)
$RAD_FRAME_MMR   = 2;   # frame for MMR definition
$RAD_FRAME_FLDS  = 3;   # frame for the field definition(s)
$RAD_VAL_DISPLAY = 4;   # variable for field display checkbox
$RAD_VAL_SELECT  = 5;   # variable for MMR selection checkbox
$RAD_EWD_REF     = 6;   # reference to EWD structure

#  The RAD array is an array containing the RAD references for one register
#     access window.  There is one entry for each register in the window.
#  ( <rad ref for register 0>, ..., <rad ref for register N> )
#
#  The RAD lookup is a hash table containing the reference to the RAD array
#     for each register access window created.
#     key   = window name
#     {key} = reference to RAD array
%rad_lookup  = (); 


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: rad_list_create_inst()
#
#    USAGE: rad_create_inst($win_name, $asic_name, $asic_type, $mmr_list, $base)
#
#    DESCRIPTION: This subroutine will create a list of RAD structures; one
#       RAD structure is created for each MMR in the list.
#
#    RETURN VALUE: A reference to an array of RAD references.
#
#-------------------------------------------------------------------------------

sub rad_list_create_inst
{  print "\nrad_list_create_inst( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my @rad_list = ();
   return(\@rad_list) if ( $#_ != 4 );

   my $mmr_list = $_[3];
   my $rad_ref;

   foreach $mmr_name (@$mmr_list)
   {
      $rad_ref = rad_create_inst($_[1], $_[2], $mmr_name, $_[4]);
      if ( $$rad_ref[$RAD_ASIC_NAME] ne "" )
      {  push (@rad_list, $rad_ref);
      }
   }

   $rad_lookup{$_[0]} = \@rad_list;
   return(\@rad_list);
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: rad_create_inst()
#
#    USAGE: rad_create_inst($asic_name, $asic_type, $mmr_name, $base)
#
#    DESCRIPTION: This subroutine will create a RAD structure for the
#       specific register.  The register instance and EWD (entry text)
#       entries of the RAD are shared for all RADs for a specific register.
#
#    RETURN VALUE: A reference to a register access data (RAD) structure.
#
#-------------------------------------------------------------------------------

sub rad_create_inst
{  print "\nrad_create_inst( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my @rad_array = ( "", 0, 0, 0, 0, 0, 0 );
   return(\@rad_array) if ( $#_ != 3 );

   my ($reg_ptr, $aref, $full_name);
   $full_name = $_[0] . ":" . $_[2];   # Full name of MMR

   #  Search for the MMR name in the "database"
   $reg_ptr = reg_lookup_create_inst($full_name, $_[1], $_[2]);
   return(\@rad_array) if ( !reg_is_valid($reg_ptr) );

   #  Initialize the fields of the structure
   $rad_array[$RAD_ASIC_NAME] = $_[0];
   $rad_array[$RAD_REG_REF]   = $reg_ptr;
   if ($debug_flag & $DEBUG_DETAILS)
   {  print "rad_array[RAD_ASIC_NAME] = $rad_array[$RAD_ASIC_NAME]\n";
      print "rad_array[RAD_REG_REF]   = $rad_array[$RAD_REG_REF]\n";
   }

   #
   #  Get a reference to an EWD structure from the global lookup (get/create)
   #
   $aref = ewd_create_reg_inst($full_name, $reg_ptr, $_[3]);
   $$aref[0] += 1;                        # Increment counter (# windows using)
   $rad_array[$RAD_EWD_REF] = $$aref[1];  # Store pointer for this window

   print "rad_array = ( @rad_array )\n" if ($debug_flag & $DEBUG_DETAILS);
   return( \@rad_array );
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: reg_lookup_create_inst()
#
#    USAGE: reg_lookup_create_inst($full_name, $asic_type, $mmr_name)
#
#    DESCRIPTION: This subroutine will get or create a register instance for
#       a specific register.  The global lookup table is used to determine
#       if a register instance has already been created.
#
#    RETURN VALUE: A reference to a register instance
#
#-------------------------------------------------------------------------------

sub reg_lookup_create_inst
{
   print "\nreg_lookup_create_inst( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);
 
   return if ( $#_ != 2 );
   my $full_name = $_[0];   # Full name of MMR: <brick>:<board>:<chip>:<reg>
   my $reg_ref;

   #
   #  Get the existing structure or create a new structure for the register
   #
   if ( exists $reg_lookup{$full_name} )  # entry exists for this chip:register
   {  $reg_ref = $reg_lookup{$full_name};
      return($reg_ref);
   }
   else
   {  $reg_ref = reg_create_mmr_inst($_[1], $_[2]);
      $reg_lookup{$full_name} = $reg_ref;
   }
   print "reg_ref = $reg_ref = ( @$reg_ref )\n" if ($debug_flag & $DEBUG_DETAILS);

   return($reg_ref);
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: ewd_create_reg_inst()
#
#    USAGE: rad_create_inst($full_name, $reg_ptr, $num_base)
#
#    DESCRIPTION: This subroutine will get or create an EWD structure for
#       a specific register.  The global lookup table is used to determine
#       if an EWD structure has already been created.
#
#    RETURN VALUE: A reference to an EWD structure
#
#-------------------------------------------------------------------------------

sub ewd_create_reg_inst
{  my @ewd_data = ();

   printf "\newd_create_reg_inst( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return( \@ewd_data ) if ( $#_ != 2 );
   my $full_name = $_[0];   # Full name of MMR: <brick>:<board>:<chip>:<reg>
   my $reg_ptr   = $_[1];   # Pointer to register instance structure
   my $num_base  = $_[2];   # Base for entry text representation
   my ($aref, @names, $val, $text, $nfld, $i, $len);

   #
   #  Get the existing structure or create a new structure for the register
   #
   if ( exists $ewd_lookup{$full_name} )  # entry exists for this chip:register
   {  $aref = $ewd_lookup{$full_name};
      return($aref);
   }
   else
   {  @ewd_data = ();
      @names   = reg_get_fields($reg_ptr, ORDER);# get field names in order
      $val     = reg_get_value($reg_ptr);        # get current register value
      $text    = convert_num2text($val, 64, $num_base);
      $ewd_data[$EWD_TEXT_MMR] = $text;
      $ewd_data[$EWD_TEXT_HEX] = $text;
      $nfld = reg_get_num_fields($reg_ptr);
      for ($i = 0; $i < $nfld; $i++)
      {  $val  = reg_get_field_value($reg_ptr, $names[$i]);
         $len  = reg_field_parm($reg_ptr, $names[$i], $REG_FLD_LENGTH);
         $text = convert_num2text($val, $len, $num_base);
         $ewd_data[$EWD_TEXT_FLD0 + $i] = $text;
      }
      my @data = ( 0, \@ewd_data );       # Initialize the array for lookup
      $ewd_lookup{$full_name} = \@data;
      return( \@data );                   # Return a pointer to the array
   }
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: dump_global_rad()
#
#    USAGE: dump_global_rad()
#
#    DESCRIPTION: This subroutine will display the contents of the global
#       lookup tables holding the register access data.
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub dump_global_rad
{
   my ($name, $aref);

   print "\n";
   print "-------------------------------------------------------\n";
   print "- DUMPING REG LOOKUP HASH TABLE:\n";
   print "-------------------------------------------------------\n";
   foreach $name ( keys %reg_lookup )
   {  print "\n$name = $reg_lookup{$name}\n";
      $aref = $reg_lookup{$name};
      print "array = ( @$aref )\n";
   }

   print "\n";
   print "-------------------------------------------------------\n";
   print "- DUMPING EWD LOOKUP HASH TABLE:\n";
   print "-------------------------------------------------------\n";
   foreach $name ( keys %ewd_lookup )
   {  print "\n$name = $ewd_lookup{$name}\n";
      $aref = $ewd_lookup{$name};
      print "array = ( @$aref )\n";
   }

   print "\n";
   print "-------------------------------------------------------\n";
   print "- DUMPING RAD LOOKUP HASH TABLE:\n";
   print "-------------------------------------------------------\n";
   foreach $name ( keys %rad_lookup )
   {  print "\n$name = $rad_lookup{$name}\n";
      $aref = $rad_lookup{$name};
      print "array = ( @$aref )\n";
   }
}


#-------------------------------------------------------------------------------

sub rad_dump
{  return if ( $#_ != 0 );
   my $rad_ref = $_[0];
   my $reg_ref = $$rad_ref[$RAD_REG_REF];
   my $ewd_ref = $$rad_ref[$RAD_EWD_REF];

   printf "\nREGISTER ACCESS DATA (RAD) CONTENTS:\n";
   printf "ASIC name   = \"%s\"\n", $$rad_ref[$RAD_ASIC_NAME];
   printf "reg_ref     = $reg_ref = ( @$reg_ref )\n";
   printf "frame_mmr   = $$rad_ref[$RAD_FRAME_MMR]\n";
   printf "frame_fld   = $$rad_ref[$RAD_FRAME_FLDS]\n";
   printf "val_display = $$rad_ref[$RAD_VAL_DISPLAY]\n";
   printf "val_select  = $$rad_ref[$RAD_VAL_SELECT]\n";
   printf "etext_ref   = $ewd_ref = ( @$ewd_ref )\n";

   reg_dump($reg_ref);
}

#-------------------------------------------------------------------------------

1;
