#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: regaccess.pm,v 1.18 2005/02/22 20:31:08 beighley Exp $
#-------------------------------------------------------------------------------
#
#  FILE: regaccess.pm
#
#  DESCRIPTION:  This file contains PERL subroutines to create a register
#     access window for a user to manipulate MMRs.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     12/11/03  Karen Beighley  Added modification history to perl source.
#                               Modified to use reg_mmr_read() and
#                                  reg_mmr_write() subroutines.
#     02/11/04  Karen Beighley  Added module scan chain index for resets.
#
#-------------------------------------------------------------------------------


use rad_subs;

#-------------------------------------------------------------------------------
#  CONSTANTS - These values cannot change!
#-------------------------------------------------------------------------------

$FALSE = 0;
$TRUE  = 1;


#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES FOR THIS MODULE
#-------------------------------------------------------------------------------

my $dsp_cb_label = "Display individual register fields";
my $fld_font = '-*-helvetica-medium-r-normal--*-80-*-*-*-*-*-*';



################################################################################
#
#  SUBROUTINES TO SUPPORT THE REGISTER ACCESS GUI
#
################################################################################

sub create_reg_access_window
{  
   printf "\ncreate_reg_access_window( @_ ) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm    = $_[0];  # Parameter registered for callback subroutine from selection
   my $win_cmt = $_[1];  # Name or title for window from user
   my $aref    = $_[2];  # Reference to array with indices of selected strings/names

   my $asic_name = $$parm[0]; # Full ASIC name: <brick>:<board>:<chip>
   my $asic_type = $$parm[1]; # ASIC type (i.e. "shub_01")
   my $main_win  = $$parm[2]; # Main window for application

   #---------------------------------------
   #  Extract actual MMR names into list
   #---------------------------------------
   my ($name, $asic_mmr_list, $i, $reg_ptr);
   $name = $asic_type . "_sorted_mmr_name";
   $asic_mmr_list = \@$name;
   my @reg_names  = ();
   foreach $i (@$aref)
   {  $name = $$asic_mmr_list[$i];
      push(@reg_names, $name);
   }
   print "register list: @reg_names\n" if ($debug_flag & $DEBUG_GENERAL);

   #---------------------------------------
   #  Create the register access window
   #---------------------------------------
   my $win_name  = "REG_WIN_$rad_win_count";
   my $win_title = "MMR Access Window: $asic_name";
   $win_title .= " \"$win_cmt\"" if ( $win_cmt ne "" );
   my $access_win = $main_win->Toplevel(-title => $win_title);
   $access_win->bind('<Destroy>' => [\&destroy_reg_access_window, $access_win, $win_name] );
   $rad_win_count++;       # Increment register access window counter

   #---------------------------------------
   #  Create the list of "register access data" structures (1 per register)
   #---------------------------------------
   my $rad_list  = rad_list_create_inst($win_name, $asic_name, $asic_type, 
                      \@reg_names, $num_base);
   if ( $#$rad_list < 0 )
   {  printf "\nERROR - Unable to create structures for specified register.\n";
      return($FALSE);
   }
   dump_global_rad() if ($debug_flag & $DEBUG_DETAILS);

   my ($menubar, $casmenu, $casmenu2, $rad_ref, 
       $hdr_frame, $hdr_label, $frame, $canvas, $canvas_frame, $bwidth,
       $action_frame, $btn_wrall, $btn_rdall, $btn_wrsel, $btn_rdsel, $btn_selall, 
       $btn_selnone, $btn_close);

   #  Create the menubar for the MMR access window 
   $menubar  = $access_win->Menu(-type => 'menubar');
   $access_win->configure(-menu => $menubar);
   $casmenu = $menubar->cascade(-label => '~File', -tearoff => 0);
   $casmenu->command(-label => 'Save Window Configuration (this window only)...',
      -command => [ \&window_cfg_save, $main_win, 'REG_ACCESS', $win_title, 
                    $asic_name, $asic_type, $rad_list]);
   $casmenu->command(-label => 'Save Window Contents (to text file)...',
      -command => [ \&regaccess_save_data, $main_win, $win_title,
                    $asic_name, $asic_type, $rad_list]);
   $casmenu->separator;
   $casmenu->command(-label => 'Close', -command => [ $access_win => 'destroy' ]);
   $casmenu = $menubar->cascade(-label => '~Settings', -tearoff => 0);
   $casmenu2 = $casmenu->cascade(-label => 'Numeric base', -tearoff => 0);
   for ($i = 0; $i <= $#base_opt_labs; $i++)
   {  $casmenu2->radiobutton(-label => $base_opt_labs[$i], 
         -value => $base_opt_vals[$i], -variable => \$num_base,
         -command => [\&ra_cbf_update_num_base, $rad_list]);
   }

   my @names = split(":", $asic_name);
   if ( !(exists $modules{ $names[0] })  )
   {  $emess  = "Specified module does not exist in the current configuration.";
      $emess .= "\nModule: $names[0]\n";
      superscope_error_handler($emess);
      return($FALSE);
   }
   my $mod_ref = $modules{ $names[0] };
   my ($chn_ind, $chip_ind) = scan_module_find_chip($mod_ref, $names[1], $names[2], 'logical');
   $casmenu = $menubar->cascade(-label => '~Reset', -tearoff => 0);
   $casmenu->command(-label => 'Reset chip TAP(s) via TMS',
      -command => [\&jar_cbf_reset, $mod_ref, $chn_ind, 'soft']);
   $casmenu->command(-label => 'Reset chip TAP(s) via TRST',
      -command => [\&jar_cbf_reset, $mod_ref, $chn_ind, 'hard']);
   $casmenu->command(-label => 'Reset SIC and configure chain(s)',
      -command => [\&jar_cbf_reset, $mod_ref, $chn_ind, 'sic' ]);

   $casmenu = $menubar->cascade(-label => '~Clear', -tearoff => 0);
   $casmenu->command(-label => 'Clear All MMRs (display only)', -command => [\&ra_cbf_reset_mmrs, $FALSE, $rad_list]);
   $casmenu->command(-label => 'Clear Selected MMRs (display only)', -command => [\&ra_cbf_reset_mmrs, $TRUE, $rad_list]);

   #
   #  Create the header and main canvas
   #
   $name = "MMR Access Window for " . $asic_name;
   $hdr_frame = $access_win->Frame(-borderwidth => 10);
   $hdr_label = $hdr_frame->Label(-anchor => 'w', -text => $name);
   $hdr_label->pack(-side => 'left');
   $hdr_frame->pack(-side => 'top', -expand => 'no', -fill => 'both');

   #  Determine the canvas height (based on number of registers/fields)
   my $can_height = 20;
   foreach $rad_ref ( @$rad_list )
   {  $reg_ptr = $$rad_ref[$RAD_REG_REF];   # pointer to register instance
      $num_fld = reg_get_num_fields($reg_ptr);
      $can_height += (20 + 10 + ($num_fld * 24));
   }

   #  Create the canvase for the register widgets
   $canvas = $access_win->Scrolled('Canvas', -scrollbars => 'se',
                -scrollregion => [0, 0, 100, $can_height],
                -borderwidth => 2, -relief => 'sunken');
   $canvas_frame  = $canvas->Frame();

   #
   #  Create the frames/widgets for the register information
   #
   foreach $rad_ref ( @$rad_list )
   { 
      #  Create the widgets for this register
      return($FALSE) if ( !create_reg_access_widgets($canvas_frame, $rad_ref) );
   }
   $canvas_frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');
   $canvas->create('window', 4, 4, -anchor => 'nw', -window => $canvas_frame);
   $canvas->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   #
   #  Create a frame to hold the action buttons
   #
   $action_frame = $access_win->Frame(-borderwidth => 10);
   $action_frame->pack(-fill => 'both');
   $bwidth = 12;
   $btn_wrall = $action_frame->Button(-text => 'Write All', -width => $bwidth,
               -command => [\&ra_cbf_btn_write, $asic_name, $FALSE, $rad_list]);
   $btn_wrall->pack(-side => 'left', -padx => 5);
   $btn_rdall = $action_frame->Button(-text => 'Read All', -width => $bwidth,
                -command => [\&ra_cbf_btn_read, $asic_name, $FALSE, $rad_list]);
   $btn_rdall->pack(-side => 'left', -padx => 5);
   $btn_wrsel = $action_frame->Button(-text => 'Write Selected', -width => $bwidth,
                   -command => [\&ra_cbf_btn_write, $asic_name, $TRUE, $rad_list]);
   $btn_wrsel->pack(-side => 'left', -padx => 5);
   $btn_rdsel = $action_frame->Button(-text => 'Read Selected', -width => $bwidth,
                   -command => [\&ra_cbf_btn_read, $asic_name, $TRUE, $rad_list]);
   $btn_rdsel->pack(-side => 'left', -padx => 5);
   $btn_selall = $action_frame->Button(-text => 'Select All', -width => $bwidth,
                    -command => [\&ra_cbf_btn_select, 1, $rad_list]);
   $btn_selall->pack(-side => 'left', -padx => 5);
   $btn_selnone = $action_frame->Button(-text => 'Select None', -width => $bwidth,
                    -command => [\&ra_cbf_btn_select, 0, $rad_list]);
   $btn_selnone->pack(-side => 'left', -padx => 5);
   $btn_close = $action_frame->Button(-text => 'Close Window', -width => $bwidth,
                    -command => [ $access_win => 'destroy' ]);
   $btn_close->pack(-side => 'left', -padx => 5);

   return($TRUE);
}

#-------------------------------------------------------------------------------

sub destroy_reg_access_window
{
}


#-------------------------------------------------------------------------------

sub create_reg_access_widgets
{  my $num = $#_ + 1;

   printf "\ncreate_reg_access_widgets( @_ ) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 2 )
   {  printf "\nERROR - Invalid parameters passed to function!\n";
      return($FALSE);
   }
   my $frame   = $_[0];  # top-level frame
   my $rad_ref = $_[1];  # pointer to register access data (RAD) structure
   my ($ewd_ref, $reg_ptr, $dptr, $fptr,
       $frame1, $menub1, $hexval, $label1, $label2, $check1, $check2,
       $frame2, @fnames, $row, $field, $aptr, $text, $label, $entry);

   #  Extract the pointer to the entry widget data (EWD) array
   $ewd_ref = $$rad_ref[$RAD_EWD_REF];

   #  Extract pointers to the register definition and field structures
   $reg_ptr = $$rad_ref[$RAD_REG_REF];   # pointer to register instance
   $dptr    = $$reg_ptr[$REG_INDEX_PTR]; # pointer to definition structure
   $fptr    = $$dptr{FIELDS};            # pointer to field hash table

   #  Create the frame for the MMR menu button and entry
   $frame1 = $frame->Frame(-borderwidth => 10);
   $$rad_ref[$RAD_FRAME_MMR] = $frame1;  # store in global lookup table

   #  Add the menu button and entry to the frame
   $menub1 = $frame1->Menubutton(-text => $$dptr{NAME}, -width => 40, 
                -tearoff => 0,
                -relief => 'raised', -direction => 'below', -anchor => 'e');
   $menub1->command(-label => "$$dptr{NOTE}");
   $text = sprintf("Address: 0x%09lX", $$dptr{ADDRESS});
   $menub1->command(-label => $text);
   $hexval = $frame1->Label(-textvariable => \$$ewd_ref[$EWD_TEXT_HEX], 
                ### -font => 'Fixed 10',
                -width => 24, -anchor => 'w');
   $entry = $frame1->Entry(-width => 26, ###-font => 'Fixed 10',
             -textvariable => \$$ewd_ref[$EWD_TEXT_MMR]);
   $entry->bind( '<Return>' => [\&ra_cbf_entry_reg, $reg_ptr, $ewd_ref, $TRUE, 
      $$rad_ref[$RAD_ASIC_NAME]] );
   $entry->bind( '<Tab>' => [\&ra_cbf_entry_reg, $reg_ptr, $ewd_ref, $TRUE, 
      $$rad_ref[$RAD_ASIC_NAME]] );
   $entry->bind( '<Leave>' => [\&ra_cbf_entry_reg, $reg_ptr, $ewd_ref, $TRUE,
      $$rad_ref[$RAD_ASIC_NAME]] );

   $label1 = $frame1->Label(-text => '     select', -font => $fld_font);
   $check1 = $frame1->Checkbutton(-variable => \$$rad_ref[$RAD_VAL_SELECT]);
   $label2 = $frame1->Label(-text => 'show fields', -font => $fld_font);
   $check2 = $frame1->Checkbutton(-variable => \$$rad_ref[$RAD_VAL_DISPLAY],
                -command => [\&ra_cbf_dsp_checkbtn, $rad_ref]);

   #  Pack the widgets into the frame
   $menub1->pack(-side => 'left');
   $hexval->pack(-side => 'left');
   $entry->pack(-side => 'left');
   $label1->pack(-side => 'left');
   $check1->pack(-side => 'left');
   $label2->pack(-side => 'left');
   $check2->pack(-side => 'left');
   $frame1->pack(-side => 'top', -fill => 'both');


   #  Create the frame for the field definition(s)
   $frame2 = $frame->Frame();          
   $$rad_ref[$RAD_FRAME_FLDS] = $frame2; # store in global lookup table

   #  Build the widgets for each field
   @fnames = reg_get_fields($reg_ptr, ORDER);
   $row = 0;
   foreach $field (@fnames)
   {
      $aptr   = $$fptr{$field};

      if ( $$aptr[$REG_FLD_LENGTH] == 1 )
      {  $text = sprintf("<%u>", $$aptr[$REG_FLD_START]);
      }
      else
      {  $text = sprintf("<%u:%u>", 
            ($$aptr[$REG_FLD_START] + $$aptr[$REG_FLD_LENGTH] - 1),
            $$aptr[$REG_FLD_START]);
      }
      $label = $frame2->Label(-width => 10, -anchor => 'w',  -font => $fld_font);
      Tk::grid( $label, -row => $row, -column => 0, -sticky => 'e');
      $label = $frame2->Label(-width => 40, -text => $field, -anchor => 'w', -font => $fld_font);
      Tk::grid( $label, -row => $row, -column => 1, -sticky => 'e');
      $label = $frame2->Label(-width => 8,  -text => $text,  -anchor => 'w', -font => $fld_font);
      Tk::grid( $label, -row => $row, -column => 2, -sticky => 'e');

      $entry = $frame2->Entry(-width => 26, -font   => $fld_font,
                   -textvariable => \$$ewd_ref[$EWD_TEXT_FLD0+$row]);
      $entry->bind( '<Return>' => [\&ra_cbf_entry_field, $reg_ptr, $ewd_ref, $field, $TRUE,
         $$rad_ref[$RAD_ASIC_NAME]] );
      $entry->bind( '<Tab>' => [\&ra_cbf_entry_field, $reg_ptr, $ewd_ref, $field, $TRUE,
         $$rad_ref[$RAD_ASIC_NAME]] );
      $entry->bind( '<Leave>' => [\&ra_cbf_entry_field, $reg_ptr, $ewd_ref, $field, $TRUE,
         $$rad_ref[$RAD_ASIC_NAME]] );
      Tk::grid( $entry, -row => $row, -column => 3, -sticky => 'e');

# NOTE: need to add register access fields
      $text = sprintf("%s", $$aptr[$REG_FLD_DESC]);
      $label = $frame2->Label(-text => $text, -anchor => 'w', -width => 50,
                -font   => $fld_font);
      Tk::grid( $label, -row => $row, -column => 4, -sticky => 'e');
      $frame2->gridRowconfigure(1,-weight => 1);  

      $row++;
   }

   return($TRUE);
}

#-------------------------------------------------------------------------------

sub regaccess_save_data
{
   print "\nregaccess_save_data( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return if ( $#_ != 4 );
   my $win_top   = $_[0];  # Top-level window for file dialog
   my $win_title = $_[1];  # Title of window (user description
   my $asic_name = $_[2];  # ASIC name: <brick>:<board>:<chip>
   my $asic_type = $_[3];  # ASIC type/decription (i.e. "shub_01")
   my $rad_list  = $_[4];  # Array of RAD references for the window

   #
   #  Prompt the user for the file name
   #
   my @types = ( ['SuperScope Data Files (Text)', '.txt']  );
   my $out_file = $win_top->getSaveFile(
#                  -title => 'Save Window Contents'
#                  -filetypes => \@types,
#                  -defaultextension => '.txt'
                   );
   return if ( !defined($out_file) );

   #
   #  Open the output file for writing
   #
   if ( !open(DATA_OUT_FILE, ">$out_file") )
   {  $emess = "Unable to open output file for writing.";
      $emess .= "\nFile: $out_file";
      superscope_error_handler($emess);
      return;
   }

   #
   #  Write the file header
   #
   printf DATA_OUT_FILE "#\n";
   printf DATA_OUT_FILE "# The file contains register (MMR) access contents from\n";
   printf DATA_OUT_FILE "#    the SuperScope debug environemnt.\n";
   printf DATA_OUT_FILE "#    Window title : $win_title\n";
   printf DATA_OUT_FILE "#    Target chip  : $asic_name\n";
   printf DATA_OUT_FILE "#    Chip type    : $asic_type\n";
   printf DATA_OUT_FILE "#\n";

   #
   #  Process each register (MMR) in the window
   #
   my ($rad_ref, $reg_ref);
   foreach $rad_ref (@$rad_list)
   {  $reg_ref = $$rad_ref[$RAD_REG_REF];
      reg_write_contents($reg_ref, DATA_OUT_FILE, $num_base);
   }
   close(DATA_OUT_FILE);

   print "\nSaved window contents: $out_file\n";
}


#-------------------------------------------------------------------------------

sub regaccess_update_reg
{
   print "\nregaccess_update_reg( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return if ( $#_ != 1 );
   my $asic_name = $_[0]; # Full ASIC name: <brick>:<board>:<chip>
   my $reg_ref   = $_[1]; # register pointer from reg_create_mmr_inst()
   my $full_name = $asic_name . ":" . reg_name($reg_ref);

   #
   #  Has a entry widget data structure been created for this register?
   #
   return if ( !(exists $ewd_lookup{$full_name}) );
   my $ewd_def = $ewd_lookup{$full_name};
   my $ewd_ptr = $$ewd_def[1];

   #
   #  Update the entry widget data structure
   #
   $$ewd_ptr[$EWD_TEXT_MMR] = convert_num2text(reg_get_value($reg_ref), 64, $num_base);
   ra_cbf_entry_reg(0, $reg_ref, $ewd_ptr, $FALSE, $asic_name);
}


#-------------------------------------------------------------------------------

sub ra_cbf_btn_write
{  print "\nra_cbf_btn_write(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   return if ( $#_ != 2 );
   my $asic_name = $_[0]; # Full ASIC name: <brick>:<board>:<chip>
   my $sel_flag  = $_[1]; # Flag: 1=write selected, 0=write all in window
   my $rad_list  = $_[2]; # List of register access data (RAD) to update
   my ($rad_ref, $reg_ptr, $ewd_ptr, $val, $text);

   #  Target the correct ASIC for MMR operations
   my @parms = split(":", $asic_name);
   mmr_target($SELECT, $parms[0], $parms[1], $parms[2]);
   my $atype = mmr_target_list_get_detail(0, $DETAIL_IND_TYPE);
   my $flag  = $TRUE;

   #  Process the possible registers and read those "valid"
   foreach $rad_ref (@$rad_list)
   {  $reg_ptr  = $$rad_ref[$RAD_REG_REF];    # register instance
      $ewd_ptr  = $$rad_ref[$RAD_EWD_REF];    # entry widget data string
      if ( ($sel_flag == $FALSE) ||
           (($sel_flag == $TRUE) && $$rad_ref[$RAD_VAL_SELECT]) )
      {  
         #  Any special setup for packet generator?
         if ( $flag && (reg_type($reg_ptr) == $REG_TYPE_PKTGEN) )
         {  ra_pktgen_setup($atype);
            $flag = $FALSE;
         }

         #  Write the value to the actual MMR (hardware)
         reg_mmr_write($reg_ptr);
         if ( $ecode < 0 )
         {  $text = "MMR operation(s) not completed due to error.\n";
#            $mw_history->insert('end', $text);
print $text;
            return;
         }

         #  Add line to history window
         $text = sprintf("%-24s mmr_write(0x%016llX, 0x%016llX) %s\n",
            $$rad_ref[$RAD_ASIC_NAME], reg_addr($reg_ptr), 
            reg_get_value($reg_ptr), reg_name($reg_ptr));
         # Pass in pointer to history window!
#         $mw_history->insert('end', $text);
print $text;
      }
   }
}


#-------------------------------------------------------------------------------

sub ra_cbf_btn_read
{  print "\nra_cbf_btn_read(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   return if ( $#_ != 2 );
   my $asic_name = $_[0]; # Full ASIC name: <brick>:<board>:<chip>
   my $sel_flag  = $_[1]; # Flag: 1=read selected, 0=read all in window
   my $rad_list  = $_[2]; # List of register access data (RAD) to update
   my ($rad_ref, $reg_ptr, $ewd_ptr, $val, $text);

   #  Target the correct ASIC for MMR operations
   my @parms = split(":", $asic_name);
   mmr_target($SELECT, $parms[0], $parms[1], $parms[2]);
   my $atype = mmr_target_list_get_detail(0, $DETAIL_IND_TYPE);
   my $flag  = $TRUE;

   #  Process the possible registers and read those "valid"
   foreach $rad_ref (@$rad_list)
   {  $reg_ptr  = $$rad_ref[$RAD_REG_REF];    # register instance
      $ewd_ptr  = $$rad_ref[$RAD_EWD_REF];    # entry widget data string
      if ( ($sel_flag == $FALSE) ||
           (($sel_flag == $TRUE) && $$rad_ref[$RAD_VAL_SELECT]) )
      {  
         #  Any special setup for packet generator?
         if ( $flag && (reg_type($reg_ptr) == $REG_TYPE_PKTGEN) )
         {  ra_pktgen_setup($atype);
            $flag = $FALSE;
         }

         #  Get the most recent value from the actual MMR (hardware)
         $val = reg_mmr_read($reg_ptr);
         if ( $ecode < 0 )
         {  $text = "MMR operation(s) not completed due to error.\n";
#            $mw_history->insert('end', $text);
print $text;
            return;
         }

         #  Update the display with the value read
         $$ewd_ptr[$EWD_TEXT_MMR] = convert_num2text($val, 64, $num_base);
         $$ewd_ptr[$EWD_TEXT_HEX] = convert_num2text($val, 64, $num_base);
         ra_cbf_entry_reg(0, $reg_ptr, $ewd_ptr, $TRUE, $asic_name);

         #  Add line to history window
         $text = sprintf("%-24s mmr_read(0x%016llX) %s -> 0x%016llX\n",
            $$rad_ref[$RAD_ASIC_NAME], reg_addr($reg_ptr), reg_name($reg_ptr),
            $val);
         # Pass in pointer to history window!
#         $mw_history->insert('end', $text);
print $text;
      }
   }
}


#-------------------------------------------------------------------------------

sub ra_cbf_btn_select
{  print "\nra_cbf_btn_select(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $value    = $_[0]; # Value for all selection checkboxes
   my $rad_list = $_[1]; # List of register access data (RAD) to update
   my $rad_ref;

   #  Clear the checkbox for all possible
   foreach $rad_ref (@$rad_list)
   {  $$rad_ref[$RAD_VAL_SELECT] = $value;
   }
}


#-------------------------------------------------------------------------------

sub ra_cbf_entry_reg
{  print "\nra_cbf_entry_reg(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   # NOTE: confirm #parmeters = 5, and that $_[0] is a HASH
   my $href      = $_[0];  # hash pointer w/ Tk specifics
   my $reg_ptr   = $_[1];  # register pointer from reg_create_mmr_inst()
   my $ewd_ptr   = $_[2];  # pointer to entry widget data (EWD) array
   my $upd_flag  = $_[3];  # Flag: TRUE=update other access window(s)
   my $asic_name = $_[4];  # Full ASIC name: <brick>:<board>:<chip>

   #  Extract pointer to the register definition and field hash structures
   my $mmr_dptr = $$reg_ptr[$REG_INDEX_PTR]; # pointer to register definition
   my $mmr_fptr = $$mmr_dptr{FIELDS};        # pointer to field defintion

   #  Convert the value given for the register and store (structure & GUI entry)
   my ($val, $field, $ind);
   $val = convert_text2num($$ewd_ptr[$EWD_TEXT_MMR]);
   $val = reg_set_value($reg_ptr, $val);
   $$ewd_ptr[$EWD_TEXT_MMR] = convert_num2text($val, 64, $num_base);
   $$ewd_ptr[$EWD_TEXT_HEX] = convert_num2text($val, 64, $num_base);

   #  Update the value of each field in the register
   foreach $field (keys %$mmr_fptr)
   {  $val  = reg_get_field_value($reg_ptr, $field); # field value
      $aptr = $$mmr_fptr{$field};    # pointer to field parameters (ARRAY)
      $ind  = $$aptr[$REG_FLD_INDEX]; # index value from field parameters
      $$ewd_ptr[$EWD_TEXT_FLD0+$ind] = convert_num2text($val, 
         $$aptr[$REG_FLD_LENGTH], $num_base);
   }

   #  Update other SuperScope access windows
   if ( $upd_flag )
   {  dbgaccess_update_reg($asic_name, $reg_ptr);
   }

}


#-------------------------------------------------------------------------------

sub ra_cbf_entry_field
{  print "\nra_cbf_entry_field(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   # NOTE: confirm #parmeters = 6, and that $_[0] is a HASH
   my $href      = $_[0];  # hash pointer w/ Tk specifics
   my $reg_ptr   = $_[1];  # register pointer from reg_create_mmr_inst()
   my $ewd_ptr   = $_[2];  # pointer to entry widget data (EWD) array
   my $field     = $_[3];  # name of field within the register
   my $upd_flag  = $_[4];  # Flag: TRUE=update other access window(s)
   my $asic_name = $_[5];  # Full ASIC name: <brick>:<board>:<chip>

   #  Extract pointer to the register definition and field hash structures
   my $mmr_dptr = $$reg_ptr[$REG_INDEX_PTR]; # pointer to register definition
   my $mmr_fptr = $$mmr_dptr{FIELDS};        # pointer to field defintion
   my $aptr     = $$mmr_fptr{$field};        # pointer to field parameters
   my $ent_ind  = $$aptr[$REG_FLD_INDEX];    # index value within register
   my $val      = 0;

   #  Convert the value given for the field and store (entire register & field)
   $val = convert_text2num($$ewd_ptr[$EWD_TEXT_FLD0+$ent_ind]);
   $val = reg_set_field_value($reg_ptr, $field, $val);
   $$ewd_ptr[$EWD_TEXT_MMR] = convert_num2text($val, 64, $num_base);
   $$ewd_ptr[$EWD_TEXT_HEX] = convert_num2text($val, 64, 'hex');
   $val = reg_get_field_value($reg_ptr, $field);
   $$ewd_ptr[$EWD_TEXT_FLD0+$ent_ind] = convert_num2text($val,
         $$aptr[$REG_FLD_LENGTH], $num_base);

   #  Update other SuperScope access windows
   if ( $upd_flag )
   {  dbgaccess_update_reg($asic_name, $reg_ptr);
   }
}


#-------------------------------------------------------------------------------

sub ra_cbf_dsp_checkbtn
{  print "\nra_cbf_dsp_checkbtn(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $rad_ref   = $_[0];                      # reference to RAD structure
   my $mmr_frame = $$rad_ref[$RAD_FRAME_MMR];  # frame containing MMR name/menu
   my $fld_frame = $$rad_ref[$RAD_FRAME_FLDS]; # frame containing fields

   #  Show or hide the frame containing the field definition(s)
   if ( $$rad_ref[$RAD_VAL_DISPLAY] == 0 )
   {  $fld_frame->pack("forget");
   }
   else
   {  $fld_frame->pack(-after => $mmr_frame, -fill => 'both');
   }
}


#-------------------------------------------------------------------------------

sub ra_cbf_update_num_base
{  print "\nra_cbf_update_num_base(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $rad_list = $_[0];  # pointer to the list of registers
   my ($rad_ref, $reg_ptr, $ewd_ptr);

   foreach $rad_ref ( @$rad_list )
   {
      #  Extract the pointer to the entry widget data (EWD) array
      $ewd_ptr = $$rad_ref[$RAD_EWD_REF];

      #  Extract pointer to the register definition
      $reg_ptr = $$rad_ref[$RAD_REG_REF];   # pointer to register instance

      #  Update the register entry and field(s)
      ra_cbf_entry_reg(0, $reg_ptr, $ewd_ptr, $FALSE, $$rad_ref[$RAD_ASIC_NAME]);
   }
}

#-------------------------------------------------------------------------------

sub ra_cbf_reset_mmrs
{  print "\nra_cbf_reset_mmrs(@_) : #parms=$#_\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $sel_flag = $_[0]; # Flag: 1=reset selected, 0=reset all in window
   my $rad_list = $_[1]; # List of register access data (RAD) to update
   my ($rad_ref, $reg_ptr, $ewd_ptr, $val);

   #  Process the possible registers and read those "valid"
   foreach $rad_ref (@$rad_list)
   {  #  Extract pointer to the register definition
      $reg_ptr = $$rad_ref[$RAD_REG_REF];
      #  Extract the pointer to the entry widget data (EWD) array
      $ewd_ptr = $$rad_ref[$RAD_EWD_REF];

      if ( ($sel_flag == $FALSE) ||
           (($sel_flag == $TRUE) && $$rad_ref[$RAD_VAL_SELECT]) )
      {  $val = reg_reset($reg_ptr);
         $$ewd_ptr[$EWD_TEXT_MMR] = convert_num2text($val, 64, $num_base);
         #  Update the register entry and field(s)
         ra_cbf_entry_reg(0, $reg_ptr, $ewd_ptr, $TRUE, $$rad_ref[$RAD_ASIC_NAME]);
      }
   }
}


#-------------------------------------------------------------------------------

sub ra_pktgen_setup
{
   print "\nra_pktgen_setup( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $atype = $_[0];  # Chip type: shub, nl4r, tio, shub2, etc.

   if ( $atype =~ /^tio/i )
   {  tio_pktgen_setup();
   }
}


#-------------------------------------------------------------------------------

1;

