#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 2002-2005 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: scct_funcs.pm,v 1.1 2005/06/06 22:51:53 meyer Exp $
#
#  FILE: scct_funcs.pm
#
#  DESCRIPTION:  This file contains PERL routines to dump and parse scct specific
#                registers.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     05/23/05  Gary Meyer      Original source code submitted for release.
#
#
#-------------------------------------------------------------------------------


#------------------------------------------------------------------------
#
#  get_scct_info  subroutine
#
#  This subroutine is called if scct_flag == 1 to extract scct specific information.
#  The errdmp script needs the "-scct" parameter for this to happen.
#
#------------------------------------------------------------------------

sub get_scct_info
{

    my $kk;

    $td0_myIndex     = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_LB_CMP_EXP_DATA0');
    $td0_myCpu       = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_LB_CMP_EXP_DATA1');
    $td0_tIndex      = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_LB_CMP_ENABLE0');
    $td0_tCpu        = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_LB_CMP_ENABLE1');
    $td0_activity    = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_II_CMP_EXP_DATA0');
    $td0_base        = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_II_CMP_EXP_DATA1');
    $td0_limit       = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_II_CMP_ENABLE0');
    $td0_strideA     = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_II_CMP_ENABLE1');
    $td0_strideB     = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_MD_CMP_EXP_DATA0');
    $td0_dwmask      = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_MD_CMP_EXP_DATA1');
    $td0_lines       = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_MD_CMP_ENABLE0');
    $td0_cycles      = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_MD_CMP_ENABLE1');
    $td0_seed        = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_PI_CMP_EXP_DATA0');
    $td0_key         = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_PI_CMP_EXP_DATA1');
    $td0_dir1        = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_PI_CMP_ENABLE0');
    $td0_dir2        = reg_create_mmr_inst('shub_01', 'SH_XN_IILB_PI_CMP_ENABLE1');
    $td1_myIndex     = reg_create_mmr_inst('shub_01', 'SH_XN_PI_IILB_CMP_EXP_DATA0');
    $td1_myCpu       = reg_create_mmr_inst('shub_01', 'SH_XN_PI_IILB_CMP_EXP_DATA1');
    $td1_tIndex      = reg_create_mmr_inst('shub_01', 'SH_XN_PI_IILB_CMP_ENABLE0');
    $td1_tCpu        = reg_create_mmr_inst('shub_01', 'SH_XN_PI_IILB_CMP_ENABLE1');
    $td1_activity    = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI0_CMP_EXP_DATA0');
    $td1_base        = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI0_CMP_EXP_DATA1');
    $td1_limit       = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI0_CMP_ENABLE0');
    $td1_strideA     = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI0_CMP_ENABLE1');
    $td1_strideB     = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI1_CMP_EXP_DATA0');
    $td1_dwmask      = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI1_CMP_EXP_DATA1');
    $td1_lines       = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI1_CMP_ENABLE0');
    $td1_cycles      = reg_create_mmr_inst('shub_01', 'SH_XN_PI_NI1_CMP_ENABLE1');
    $td1_seed        = reg_create_mmr_inst('shub_01', 'SH_XN_PI_SIC_CMP_EXP_HDR0');
    $td1_key         = reg_create_mmr_inst('shub_01', 'SH_XN_PI_SIC_CMP_EXP_HDR1');
    $td1_dir1        = reg_create_mmr_inst('shub_01', 'SH_XN_PI_SIC_CMP_HDR_ENABLE0');
    $td1_dir2        = reg_create_mmr_inst('shub_01', 'SH_XN_PI_SIC_CMP_HDR_ENABLE1');

    my @td01_regs=( $td0_myIndex,  $td0_myCpu,   $td0_tIndex,   $td0_tCpu,    $td0_activity,  $td0_base,
     $td0_limit,    $td0_strideA,  $td0_strideB, $td0_dwmask,   $td0_lines,   $td0_cycles,    $td0_seed,
     $td0_key,      $td0_dir1,     $td0_dir2,    $td1_myIndex,  $td1_myCpu,   $td1_tIndex,    $td1_tCpu,
     $td1_activity, $td1_base,     $td1_limit,   $td1_strideA,  $td1_strideB, $td1_dwmask,
     $td1_lines,    $td1_cycles,   $td1_seed,    $td1_key,      $td1_dir1,    $td1_dir2 );

#  Need to use reg_create_mmr_list and reg_list_mmr_read() for this section

    my $xfr_complete = 0;
    my $xfr_f_cnt = 0;
    while (($xfr_complete == 0) && ($xfr_f_cnt < 3) && ($target_list[$t_cnt]->{XFR_ERR_CNT} < 6))  {
      reg_list_mmr_read(\@td01_regs);  #  Read the MMRs as a group
      if ($SSCOPE_ERR == 1)  {
        $target_list[$t_cnt]->{XFR_ERR_CNT}++;
        $xfr_f_cnt++;
        $SSCOPE_ERR = 0;
      }
      else {
        $xfr_complete = 1;
      }
    }

    $td_myIndex   = reg_get_value($td01_regs[0]);
    $td_myCpu     = reg_get_value($td01_regs[1]);
    $td_tIndex    = reg_get_value($td01_regs[2]);
    $td_tVCpu     = reg_get_value($td01_regs[3]);
    $td_activity  = reg_get_value($td01_regs[4]);
    $td_base      = reg_get_value($td01_regs[5]);
    $td_limit     = reg_get_value($td01_regs[6]);
    $td_strideA   = reg_get_value($td01_regs[7]);
    $td_strideB   = reg_get_value($td01_regs[8]);
    $td_dwmask    = reg_get_value($td01_regs[9]);
    $td_lines     = reg_get_value($td01_regs[10]);
    $td_cycles    = reg_get_value($td01_regs[11]);
    $td_seed      = reg_get_value($td01_regs[12]);
    $td_key       = reg_get_value($td01_regs[13]);
    $td_dir1      = reg_get_value($td01_regs[14]);
    $td_dir2      = reg_get_value($td01_regs[15]);

    printf ("    {                       // scct_td\n");
    printf ("        0x%016llx, // scct_td  myIndex\n",  $td_myIndex);
    printf ("        0x%016llx, // scct_td  myCpu\n",    $td_myCpu);
    printf ("        0x%016llx, // scct_td  tIndex\n",   $td_tIndex);
    printf ("        0x%016llx, // scct_td  tVCpu\n",    $td_tVCpu);
    printf ("        0x%016llx, // scct_td  activity\n", $td_activity);
    printf ("        0x%016llx, // scct_td  base\n",     $td_base);
    printf ("        0x%016llx, // scct_td  limit\n",    $td_limit);
    printf ("        0x%016llx, // scct_td  stride\n",   $td_strideA);
    printf ("        0x%016llx, // scct_td  baseInc\n",  $td_strideB);
    printf ("        0x%016llx, // scct_td  dwmask\n",   $td_dwmask);
    printf ("        0x%016llx, // scct_td  lines\n",    $td_lines);
    printf ("        0x%016llx, // scct_td  cycles\n",   $td_cycles);
    printf ("        0x%016llx, // scct_td  seed\n",     $td_seed);
    printf ("        0x%016llx, // scct_td  key\n",      $td_key);
    printf ("        0x%016llx, // scct_td  dir1\n",     $td_dir1);
    printf ("        0x%016llx  // scct_td  dir2\n",     $td_dir2);
    printf ("    },                      // scct_td\n");

    if( (($td_base&0xffff000000000000) == 0xa000000000000000) && (($td_limit&0xffff000000000000)==0xa000000000000000) ) {
        gen_scct_addr_list($td_base, $td_limit, $td_strideA, $td_strideB, $td_lines);
    }

#  Need to use reg_create_mmr_list and reg_list_mmr_read() for this section

    $td_myIndex   = reg_get_value($td01_regs[16]);
    $td_myCpu     = reg_get_value($td01_regs[17]);
    $td_tIndex    = reg_get_value($td01_regs[18]);
    $td_tVCpu     = reg_get_value($td01_regs[19]);
    $td_activity  = reg_get_value($td01_regs[20]);
    $td_base      = reg_get_value($td01_regs[21]);
    $td_limit     = reg_get_value($td01_regs[22]);
    $td_strideA   = reg_get_value($td01_regs[23]);
    $td_strideB   = reg_get_value($td01_regs[24]);
    $td_dwmask    = reg_get_value($td01_regs[25]);
    $td_lines     = reg_get_value($td01_regs[26]);
    $td_cycles    = reg_get_value($td01_regs[27]);
    $td_seed      = reg_get_value($td01_regs[28]);
    $td_key       = reg_get_value($td01_regs[29]);
    $td_dir1      = reg_get_value($td01_regs[30]);
    $td_dir2      = reg_get_value($td01_regs[31]);

    printf ("    {                       // scct_td\n");
    printf ("        0x%016llx, // scct_td  myIndex\n",  $td_myIndex);
    printf ("        0x%016llx, // scct_td  myCpu\n",    $td_myCpu);
    printf ("        0x%016llx, // scct_td  tIndex\n",   $td_tIndex);
    printf ("        0x%016llx, // scct_td  tVCpu\n",    $td_tVCpu);
    printf ("        0x%016llx, // scct_td  activity\n", $td_activity);
    printf ("        0x%016llx, // scct_td  base\n",     $td_base);
    printf ("        0x%016llx, // scct_td  limit\n",    $td_limit);
    printf ("        0x%016llx, // scct_td  stride\n",   $td_strideA);
    printf ("        0x%016llx, // scct_td  baseInc\n",  $td_strideB);
    printf ("        0x%016llx, // scct_td  dwmask\n",   $td_dwmask);
    printf ("        0x%016llx, // scct_td  lines\n",    $td_lines);
    printf ("        0x%016llx, // scct_td  cycles\n",   $td_cycles);
    printf ("        0x%016llx, // scct_td  seed\n",     $td_seed);
    printf ("        0x%016llx, // scct_td  key\n",      $td_key);
    printf ("        0x%016llx, // scct_td  dir1\n",     $td_dir1);
    printf ("        0x%016llx  // scct_td  dir2\n",     $td_dir2);
    printf ("    },                      // scct_td\n");

    if( (($td_base&0xffff000000000000) == 0xa000000000000000) && (($td_limit&0xffff000000000000)==0xa000000000000000) ) {
        gen_scct_addr_list($td_base, $td_limit, $td_strideA, $td_strideB, $td_lines);
    }


}


#------------------------------------------------------------------------
#
#  gen_scct_addr_list  subroutine
#
#------------------------------------------------------------------------

sub gen_scct_addr_list
{

    my $a_base    = $_[0];
    my $a_limit   = $_[1];
    my $a_strideA = $_[2];
    my $a_strideB = $_[3];
    my $a_lines   = $_[4];

    if($a_base==0) {
        return(0);
    }

    if($a_strideA & 0xf000000000000000) {
       $a_strideA = ($a_strideA ^ 0xffffffffffffffff) + 1;
       $a_negstrA = 1;
    }
    else {
       $a_negstrA = 0;
    }
    if($a_strideB & 0xf000000000000000) {
       $a_strideB = ($a_strideB ^ 0xffffffffffffffff) + 1;
       $a_negstrB = 1;
    }
    else {
       $a_negstrB = 0;
    }

    $b_lines      = $a_lines + 8;
    $b_addr       = $a_base;
    $b_base       = $a_base;

    while($b_lines > 0)
    {
        if($a_base < $a_limit) {
            if($b_addr>=$a_limit) {
                if($a_negstrB) {
                    $b_addr = ($b_base -= $a_strideB);
                }
                else {
                    $b_addr = ($b_base += $a_strideB);
                }
            }
        }
        else {
            if($b_addr<$a_limit) {
                if($a_negstrB) {
                    $b_addr = ($b_base -= $a_strideB);
                }
                else {
                    $b_addr = ($b_base += $a_strideB);
                }
            }
        }

        $address = $b_addr & 0xfffffff80;
        $nodenum = ($b_addr>>38) & 0x7ff;           # Extract entire NASID
        save_address($prefix, $address, $nodenum, 'SCCT');

        if($a_negstrA) {
            $b_addr = ($b_addr -= $a_strideA);
        }
        else {
            $b_addr = ($b_addr += $a_strideA);
        }
        $b_lines--;
    }

    return(0);
}

#-------------------------------------------------------------------------------

1;
