#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
# 
# @(#)$Id: selchip.pm,v 1.1 2002/02/20 17:25:12 beighley Exp $

use Tk;

$FALSE  = 0;
$TRUE   = 1;

#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES FOR THIS MODULE
#-------------------------------------------------------------------------------

my $csel_win = 0;
my $csel_list;
my @csel_ptr = ();
my @csel_ind = ();


#-------------------------------------------------------------------------------

sub create_chip_selection_window
{ 
   printf "\ncreate_chip_selection_window( @_ )\n";

   if ( $#_ != 0 )
   {  printf "\nERROR - Invalid parameters passed to function!\n";
      return($FALSE);
   }
   my $type = $_[0];  # Chip selection for this type of access window

   my ($title, $text, $frame, $label, $bwidth, $btn);

   $title = "Chip Selection for MMR Access Window";
   $text  = "Target chip for MMR access (brick:board:chip):";

   $csel_win = MainWindow->new(-title => $title);
   $frame = $csel_win->Frame(-borderwidth => 10);
   $label = $frame->Label(-anchor => 'w', -text => $text);
   $csel_list = $frame->Scrolled('Listbox', -scrollbars => 'se', -height => 10,
                   -font => 'fixed');
   $csel_list->bind('<Double-1>' => \&cbf_chip_select);
   $label->pack(-side => 'top', -expand => 'no', -fill => 'both');
   $csel_list->pack(-side => 'top', -expand => 'yes', -fill => 'both');
   $frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');

   $bwidth = 12;
   $frame  = $csel_win->Frame(-borderwidth => 10);
   $frame->pack(-side => 'top', -fill => 'both');
   $btn    = $frame->Button(-text => 'Select', -width => $bwidth,
                -command => [\&cbf_chip_select, 0]);
   $btn->pack(-side => 'left', -padx => 5);
   $btn = $frame->Button(-text => 'Cancel', -width => $bwidth,   
                    -command => \&cbf_btn_cancel);
   $btn->pack(-side => 'left', -padx => 5);



   @csel_ptr = ();
   @csel_ind = ();

   $base = 0;
   foreach $mref (@module_ptrs)
   {
printf "\nMODULE: $$mref{NAME}\n";

      @chips = scan_module_chips($mref, $CMP_FLAG_MMR, 'logical', \@pos);

printf "chips = ( @chips )\n";
printf "index = ( @pos )\n";

printf "\nmerging lists:\n";
      for ($i = 0; $i <= $#chips; $i++)
      {  $csel_list->insert('end', $chips[$i]);
printf "i=%02u base=%02u : chip=%s\n", $i, $base, $chips[$i];
         $csel_ptr[$base+$i] = $mref;
         $csel_ind[$base+$i] = $pos[$i];
      }
      $base += $i;
   }
   $csel_list->selection('set', 0);

print "\nARRAYS:\n";
print "pointers = ( @csel_ptr )\n";
print "index    = ( @csel_ind )\n";

   return($TRUE);
}


#-------------------------------------------------------------------------------

sub cbf_chip_select
{  my $num = $#_ + 1;

   printf "\ncbf_chip_select( @_ ) : num=$num\n";

   my $cur = $csel_list->curselection();
   if ( !defined($cur) )
   {  $csel_win->messageBox(-icon => 'info', -type => 'OK',
            -message => 'No chip selected for register access window.',
            -title => 'Chip Selection Error');
      return;
   }
   my $name = $csel_list->get($cur);

   print "\nSELECTED CHIP: $cur = \"$name\"\n";

   my $ptr = $csel_ptr[$cur];
   my $ind = $csel_ind[$cur];

   print "ptr=$ptr, ind=$ind\n";

   my $aref = scan_module_chip_parms($ptr, $ind);
   print "parameters = ( @$aref )\n";

}


#-------------------------------------------------------------------------------

sub cbf_btn_cancel
{  my $num = $#_ + 1;

   printf "\ncbf_btn_cancel( @_ ) : num=$num\n";

   $csel_win->destroy();
}


#-------------------------------------------------------------------------------

1;
