#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: selreg.pm,v 1.8 2005/10/18 22:24:35 beighley Exp $


use mmrsel;

#-------------------------------------------------------------------------------
#  CONSTANTS - These values cannot change!
#-------------------------------------------------------------------------------

$FALSE = 0;
$TRUE  = 1;


#-------------------------------------------------------------------------------
#
#  Create the selection window for a MMR access window.
#
#-------------------------------------------------------------------------------

sub create_reg_selection_window
{  my $num = $#_ + 1;

   printf "\ncreate_reg_selection_window( @_ ) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 3 )
   {  printf "\nERROR - Invalid parameters passed to function!\n";
      return($FALSE);
   }
   my $main_win  = $_[0]; # Main window for application
   my $chip_name = $_[1]; # full ASIC name: <brick>:<board>:<chip>:<reg>
   my $chip_type = $_[2]; # ASIC type (i.e. "shub_01")
   my @chip_parm = ( $chip_name, $chip_type, $main_win );

   #---------------------------------------
   #  Create the MMR selection window
   #---------------------------------------
   my $mode = $MMRSEL_MODE_DEFAULT | $MMRSEL_MODE_SHOW_NAME | $MMRSEL_MODE_EMPTY_NAME;
   my @curr_list = ( );
   my $sel_win   = 0;
   create_mmrsel_window($main_win, $chip_name, $chip_type,
      "for MMR Access Window", $mode, "Create MMR Window", "", \@curr_list,
      \&create_reg_access_window, \&cancel_reg_selection_window, \@chip_parm, \$sel_win);

   return($TRUE);
}

#-------------------------------------------------------------------------------

sub cancel_reg_selection_window
{
   printf "\ncancel_reg_selection_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm = $_[0];  # Parameter registered for callback subroutine

#   printf "\nparm : $parm\n";
}


#-------------------------------------------------------------------------------
#
#  Create the selection window for a MMR read to file operation.
#
#-------------------------------------------------------------------------------

sub create_read_save_selection_window
{  my $num = $#_ + 1;

   printf "\ncreate_read_save_selection_window( @_ ) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( $num != 3 )
   {  printf "\nERROR - Invalid parameters passed to function!\n";
      return($FALSE);
   }
   my $main_win  = $_[0]; # Main window for application
   my $chip_name = $_[1]; # full ASIC name: <brick>:<board>:<chip>:<reg>
   my $chip_type = $_[2]; # ASIC type (i.e. "shub_01")
   my @chip_parm = ( $chip_name, $chip_type, $main_win );

   #---------------------------------------
   #  Create the MMR selection window
   #---------------------------------------
   my $mode = $mode = $MMRSEL_MODE_DEFAULT | $MMRSEL_MODE_SHOW_NAME | $MMRSEL_MODE_SHOW_BROWSE;
   my @curr_list = ( );
   my $sel_win   = 0;
   create_mmrsel_window($main_win, $chip_name, $chip_type,
      "MMRs to read to file", $mode, "Read and Save MMRs", "", \@curr_list,
      \&perform_mmr_read_and_save, \&cancel_read_save_selection_window, \@chip_parm, 
      \$sel_win, "Output File Name:");

   return($TRUE);
}

#-------------------------------------------------------------------------------

sub cancel_read_save_selection_window
{
   printf "\ncancel_read_save_selection_window( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm = $_[0];  # Parameter registered for callback subroutine

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "\nparm : $parm\n";
   }
}

#-------------------------------------------------------------------------------

sub perform_mmr_read_and_save
{
   printf "\nperform_mmr_read_and_save( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $parm = $_[0];  # Parameter registered for callback subroutine
   my $name = $_[1];  # Name or title for group of selected strings/names
   my $aref = $_[2];  # Reference to array with indices of selected strings/names

   if ( $debug_flag & $DEBUG_DETAILS )
   {  printf "\nparm : $parm\n";
      printf "name : $name\n";
      printf "list : ( @$aref )\n";
   }

   my $chip_name = $$parm[0]; # Full ASIC name: <brick>:<board>:<chip>
   my $chip_type = $$parm[1]; # ASIC type (i.e. "shub_01")
   my $main_win  = $$parm[2]; # Main window for application

   #---------------------------------------
   #  Target the correct ASIC for MMR operation(s)
   #---------------------------------------
   my @parms = split(":", $chip_name);
   mmr_target($SELECT, $parms[0], $parms[1], $parms[2]);

   #---------------------------------------
   #  Extract actual MMR names into list
   #---------------------------------------
   my ($tmp_name, $asic_mmr_list, $i);
   $tmp_name = $chip_type . "_sorted_mmr_name";
   $asic_mmr_list = \@$tmp_name;
   my @reg_names  = ();
   foreach $i (@$aref)
   {  $tmp_name = $$asic_mmr_list[$i];
      push(@reg_names, $tmp_name);
   }
   print "register list: @reg_names\n" if ($debug_flag & $DEBUG_GENERAL);

   #---------------------------------------
   #  Read the MMRs and save contents to file
   #---------------------------------------
   my @names = split(":", $chip_name);
   mmr_list_read_and_save($names[0], $names[1], $names[2], $chip_type,
      \@reg_names, $name);

   printf "\nRead MMRs and saved to file: $name\n";
}


#-------------------------------------------------------------------------------

1;
