#-------------------------------------------------------------------------------
#
#  FILE: shub2_01_jtdrs.pm
#
#  NOTE: Definitions of the JTAG test data registers (TDRs) for SHUB2.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     04/14/04  Karen Beighley  Original source code (ported from SHUB).
#
#-------------------------------------------------------------------------------

$shub2_01_tdr_defined = 0;

#-------------------------------------------------------------------------------

%shub2_01_flds_00000 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub2_01_tdr_00000 =
(  NAME   => "BYPASS",
   NOTE   => "BYPASS JTAG Register",
   OPNAME => "BYPASS",
   OPCODE => 0x7F,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub2_01_flds_00000
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00001 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "VERSION",          => [   0,  28,   4, 0x0000, "Version number"         ],
   "PARTNUMBER",       => [   1,  12,  16, 0x0000, "Partnumber"             ],
   "MANUFACTURER",     => [   2,   1,  11, 0x0000, "Manufacturer Code"      ],
   "LSB",              => [   3,   0,   1, 0x0000, "Mandatory LSB"          ]
);

%shub2_01_tdr_00001 =
(  NAME   => "IDCODE",
   NOTE   => "JTAG Device Identification Register",
   OPNAME => "IDCODE",
   OPCODE => 0x03,
   LENGTH => 32,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub2_01_flds_00001
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00002 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "CONFIG_SPARE"                     => [   0,  76,   4, 0x0000, ""                                            ],
   "FSB_DEBUG_CONFIG_OE"              => [   1,  75,   1, 0x0000, "Port output enable select"                   ],
   "LB_DBUG_CONFIG_OE"                => [   2,  74,   1, 0x0000, "Port output enable select"                   ],
   "GTL_CONFIG_RE"                    => [   3,  73,   1, 0x0000, "Reference Enable selection for GTL I/O"      ],
   "DQSGATE_EN"                       => [   4,  72,   1, 0x0000, ""                                            ],
   "FMAC_DDR_RCVR_CLOCK_CONFIG"       => [   5,  71,   1, 0x0000, ""                                            ],
   "R_FMAC_DDR_RCVR_DRDY_CONFIG"      => [   6,  69,   2, 0x0000, ""                                            ],
   "L_FMAC_DDR_RCVR_DRDY_CONFIG"      => [   7,  67,   2, 0x0000, ""                                            ],
   "FMAC_DDR_RCVR_FIFO_CONFIG"        => [   8,  66,   1, 0x0000, ""                                            ],
   "FMAC_TXRX_CAPTURE_PHASE"          => [   9,  65,   1, 0x0000, ""                                            ],
   "FMAC_TXRX_LAUNCH_PHASE"           => [  10,  64,   1, 0x0000, ""                                            ],
   "JTAG_MCI_OVERRIDE"                => [  11,  63,   1, 0x0000, ""                                            ],
   "JTAG_MCI_TARGET"                  => [  12,  49,  14, 0x0000, ""                                            ],
   "JTAG_MCI_RESET_DELAY"             => [  13,  42,   7, 0x0000, ""                                            ],
   "JTAG_MCI_WRTACT_LATENCY"          => [  14,  40,   2, 0x0000, ""                                            ],
   "WRT90_OVERRIDE"                   => [  15,  39,   1, 0x0000, ""                                            ],
   "WRT90_OVERRIDEA"                  => [  16,  38,   1, 0x0000, ""                                            ],
   "WRT90_TARGET"                     => [  17,  24,  14, 0x0000, ""                                            ],
   "J_SEL_OSC_FSBCLK"                 => [  18,  23,   1, 0x0000, "Selects core or ext osc/ for raw_fsbclk"     ],
   "J_SEL_OSC_CORECLK"                => [  19,  22,   1, 0x0000, "Selects core or ext osc/ for raw_coreclk"    ],
   "J_SEL_FSB_1X_BY2"                 => [  20,  21,   1, 0x0000, "Selects output freq/ for FSB clock domain"   ],
   "J_SEL_OSCFSB_CORE"                => [  21,  20,   1, 0x0000, "Selects input freq/ for FSB clock domain"    ],
   "JSEL_PI4CLK_DSKW"                 => [  22,  18,   2, 0x0000, "Selects fine tuning deskew of pi4clk"        ],
   "JSEL_PI2CLK_DSKW"                 => [  23,  16,   2, 0x0000, "Selects fine tuning deskew of pi2clk"        ],
   "JSEL_BCLKA_DSKW"                  => [  24,  14,   2, 0x0000, "Selects fine tuning deskew of bclka"         ],
   "JSEL_NETCHNCLK_DSKW"              => [  25,  12,   2, 0x0000, "Selects fine tuning deskew of netchnclk"     ],
   "JSEL_CORECLK_DSKW"                => [  26,  10,   2, 0x0000, "Selects fine tuning deskew of coreclk"       ],
   "JSEL_DRAMCLK_DSKW"                => [  27,   8,   2, 0x0000, "Selects fine tuning deskew of dramclk"       ],
   "JSEL_CORE2XCLK_DSKW"              => [  28,   6,   2, 0x0000, "Selects fine tuning deskew of core2xclk"     ],
   "JSEL_CORE2XCLK_INV"               => [  29,   5,   1, 0x0000, "Selects polarity of core2xclk (0=normal)"    ],
   "II_CLK_SEL"                       => [  30,   3,   2, 0x0000, "Selects the clock source for the IOCLK domain" ],
   "NI_CLK_SEL"                       => [  31,   2,   1, 0x0000, "Selects the clock source for the NICLK domain" ],
   "MD_CLK_SEL"                       => [  32,   0,   2, 0x0000, "Selects the divide frequency of the DRAMCLK" ],
);

%shub2_01_tdr_00002 =
(  NAME   => "CONFIG",
   NOTE   => "SHUB Pre-reset Configuration",
   OPNAME => "CONFIG",
   OPCODE => 0x7E,
   LENGTH => 80,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00002
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00003 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "SPARE"                 => [   0,   2,   3, 0x0000, ""            ],
   "CIR"                   => [   1,   1,   1, 0x0000, ""            ],
   "INV"                   => [   2,   0,   1, 0x0000, ""            ],
);

%shub2_01_tdr_00003 =
(  NAME   => "SCAN_CTL",
   NOTE   => "Controls gmoc_mode and scan access",
   OPNAME => "SCAN_CTL",
   OPCODE => 0x05,
   LENGTH => 6,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub2_01_flds_00003
);

#-------------------------------------------------------------------------------


%shub2_01_flds_00004 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "NI0_CLK_I_TERM_H"    => [   0,   3,   1, 0x0000, "description"            ],
   "NI0_CLK_I_TERM_L"    => [   1,   2,   1, 0x0000, "description"            ],
   "NI1_CLK_I_TERM_H"    => [   2,   1,   1, 0x0000, "description"            ],
   "NI1_CLK_I_TERM_L"    => [   3,   0,   1, 0x0000, "description"            ],
);

%shub2_01_tdr_00004 =
(  NAME   => "ICLK_TERM",
   NOTE   => "ICLK termination value selection",
   OPNAME => "ICLK_TERM",
   OPCODE => 0x09,
   LENGTH => 4,
   RESET  => [ 0x000000F ],
   FIELDS => \%shub2_01_flds_00004
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00005 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "START"               => [   0,  29,   1, 0x0000, "description"            ],
   "WRITE"               => [   1,  28,   1, 0x0000, "description"            ],
   "PIO_ADDRESS"         => [   2,   0,  28, 0x0000, "description"            ]
);

%shub2_01_tdr_00005 =
(  NAME   => "PIO_CTRL",
   NOTE   => "Control Register for PIO",
   OPNAME => "PIO_CTRL",
   OPCODE => 0x0F,
   LENGTH => 30,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub2_01_flds_00005
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00006 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ERROR"               => [   0,  65,   1, 0x0000, "description"            ],
   "VALID"               => [   1,  64,   1, 0x0000, "description"            ],
   "DATA"                => [   2,   0,  64, 0x0000, "description"            ]
);

%shub2_01_tdr_00006 =
(  NAME   => "PIO_DATA",
   NOTE   => "Data Register for PIO",
   OPNAME => "PIO_DATA",
   OPCODE => 0x10,
   LENGTH => 66,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00006
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00007 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "MABIST_BIST_M3"      => [   0,  20,   1, 0x0000, "description"            ],
   "MABIST_ABDONE"       => [   1,  16,   4, 0x0000, "description"            ],
   "MABIST_SYSPF"        => [   2,   0,  16, 0x0000, "description"            ]
);

%shub2_01_tdr_00007 =
(  NAME   => "RUNABIST",
   NOTE   => "SRAM RUNABIST control",
   OPNAME => "RUNABIST",
   OPCODE => 0x08,
   LENGTH => 21,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub2_01_flds_00007
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00008 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "JTAG_MCI_L_LOCK"                  => [  0,  71,   9, 0x0000, ""                                            ],
   "JTAG_MCI_R_LOCK"                  => [  1,  62,   9, 0x0000, ""                                            ],
   "WRT90_LOCK"                       => [  2,  61,   1, 0x0000, ""                                            ],
   "CLKCNTL_SPARE"                    => [  3,  57,   4, 0x0000, ""                                            ],
   "LOAD"                             => [  4,  53,   4, 0x0000, ""                                            ],
   "GMXC_SEQ"                         => [  5,  33,  20, 0x0000, ""                                            ],
   "SLOW1CLK"                         => [  6,  32,   1, 0x0000, ""                                            ],
   "SLOW2CLK"                         => [  7,  31,   1, 0x0000, ""                                            ],
   "GMXC_TDR_SEL"                     => [  8,   8,  23, 0x0000, ""                                            ],
   "TP"                               => [  9,   2,   7, 0x0000, ""                                            ],
   "SEL_CLKSTOP"                      => [ 10,   1,   1, 0x0000, ""                                            ],
   "JTAG_CLKSTOP"                     => [ 11,   0,   1, 0x0000, ""                                            ],
);

%shub2_01_tdr_00008 =
(  NAME   => "CLKCNTL",
   NOTE   => "JTAG CLKCNTL control Register",
   OPNAME => "CLKCNTL",
   OPCODE => 0x07,
   LENGTH => 80,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00008
);

#-------------------------------------------------------------------------------


%shub2_01_flds_00009 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "GTL_COMCLK_N_REFIO_VALUE"         => [ 0, 159,   9, 0x0000, ""                                            ],
   "GTL_COMCLK_P_REFIO_VALUE"         => [ 1, 150,   9, 0x0000, ""                                            ],
   "GTL_DDR_N_REFIO_VALUE"            => [ 2, 141,   9, 0x0000, ""                                            ],
   "GTL_DDR_P_REFIO_VALUE"            => [ 3, 132,   9, 0x0000, ""                                            ],
   "NI1_N_REFIO_VALUE"                => [ 4, 123,   9, 0x0000, ""                                            ],
   "NI1_P_REFIO_VALUE"                => [ 5, 114,   9, 0x0000, ""                                            ],
   "NI0_N_REFIO_VALUE"                => [ 6, 105,   9, 0x0000, ""                                            ],
   "NI0_P_REFIO_VALUE"                => [ 7,  96,   9, 0x0000, ""                                            ],
   "GTL_COMCLK_FILTER"                => [ 8,  95,   1, 0x0000, ""                                            ],
   "GTL_COMCLK_BYPASS"                => [ 9,  94,   1, 0x0000, ""                                            ],
   "GTL_COMCLK_DISABLE"               => [10,  93,   1, 0x0000, ""                                            ],
   "GTL_COMCLK_RESET"                 => [11,  92,   1, 0x0000, ""                                            ],
   "GTL_COMCLK_N_TDR_VALUE"           => [12,  84,   8, 0x0000, ""                                            ],
   "GTL_COMCLK_P_TDR_VALUE"           => [13,  76,   8, 0x0000, ""                                            ],
   "GTL_COMCLK_SEL_TDR_VAL"           => [14,  75,   1, 0x0000, ""                                            ],
   "GTL_COMCLK_N_UPDN"                => [15,  74,   1, 0x0000, ""                                            ],
   "GTL_COMCLK_P_UPDN"                => [16,  73,   1, 0x0000, ""                                            ],
   "GTL_COMCLK_OVRD_UPDN"             => [17,  72,   1, 0x0000, ""                                            ],
   "GTL_DDR_FILTER"                   => [18,  71,   1, 0x0000, ""                                            ],
   "GTL_DDR_BYPASS"                   => [19,  70,   1, 0x0000, ""                                            ],
   "GTL_DDR_DISABLE"                  => [20,  69,   1, 0x0000, ""                                            ],
   "GTL_DDR_RESET"                    => [21,  68,   1, 0x0000, ""                                            ],
   "GTL_DDR_N_TDR_VALUE"              => [22,  60,   8, 0x0000, ""                                            ],
   "GTL_DDR_P_TDR_VALUE"              => [23,  52,   8, 0x0000, ""                                            ],
   "GTL_DDR_SEL_TDR_VAL"              => [24,  51,   1, 0x0000, ""                                            ],
   "GTL_DDR_N_UPDN"                   => [25,  50,   1, 0x0000, ""                                            ],
   "GTL_DDR_P_UPDN"                   => [26,  49,   1, 0x0000, ""                                            ],
   "GTL_DDR_OVRD_UPDN"                => [27,  48,   1, 0x0000, ""                                            ],
   "NI1_FILTER"                       => [28,  47,   1, 0x0000, ""                                            ],
   "NI1_BYPASS"                       => [29,  46,   1, 0x0000, ""                                            ],
   "NI1_DISABLE"                      => [30,  45,   1, 0x0000, ""                                            ],
   "NI1_RESET"                        => [31,  44,   1, 0x0000, ""                                            ],
   "NI1_N_TDR_VALUE"                  => [32,  36,   8, 0x0000, ""                                            ],
   "NI1_P_TDR_VALUE"                  => [33,  28,   8, 0x0000, ""                                            ],
   "NI1_SEL_TDR_VAL"                  => [34,  27,   1, 0x0000, ""                                            ],
   "NI1_N_UPDN"                       => [35,  26,   1, 0x0000, ""                                            ],
   "NI1_P_UPDN"                       => [36,  25,   1, 0x0000, ""                                            ],
   "NI1_OVRD_UPDN"                    => [37,  24,   1, 0x0000, ""                                            ],
   "NI0_FILTER"                       => [38,  23,   1, 0x0000, ""                                            ],
   "NI0_BYPASS"                       => [39,  22,   1, 0x0000, ""                                            ],
   "NI0_DISABLE"                      => [40,  21,   1, 0x0000, ""                                            ],
   "NI0_RESET"                        => [41,  20,   1, 0x0000, ""                                            ],
   "NI0_N_TDR_VALUE"                  => [42,  12,   8, 0x0000, ""                                            ],
   "NI0_P_TDR_VALUE"                  => [43,   4,   8, 0x0000, ""                                            ],
   "NI0_SEL_TDR_VAL"                  => [44,   3,   1, 0x0000, ""                                            ],
   "NI0_N_UPDN"                       => [45,   2,   1, 0x0000, ""                                            ],
   "NI0_P_UPDN"                       => [46,   1,   1, 0x0000, ""                                            ],
   "NI0_OVRD_UPDN"                    => [47,   0,   1, 0x0000, ""                                            ],
);

%shub2_01_tdr_00009 =
(  NAME   => "ICNTLR",
   NOTE   => "Impedance Controller for APF and GTL I/O",
   OPNAME => "ICNTLR",
   OPCODE => 0x06,
   LENGTH => 168,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00009
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00010 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "SPARE"                            => [ 0,  23,   4, 0x0000, ""                                            ],
   "OBSERVE_ENABLE"                   => [ 1,  22,   1, 0x0000, ""                                            ],
   "DATA_CLK_MUX_SEL"                 => [ 2,  21,   1, 0x0000, ""                                            ],
   "PFSSD_SYNC_MODE"                  => [ 3,  20,   1, 0x0000, ""                                            ],
   "PFSSD_DEBUG_SEL"                  => [ 4,  18,   2, 0x0000, ""                                            ],
   "PFSSR_DEBUG_SEL"                  => [ 5,  12,   6, 0x0000, ""                                            ],
   "DAVAIL_SEL"                       => [ 6,  10,   2, 0x0000, ""                                            ],
   "CLK_ADAPT_HOLD_MODE"              => [ 7,   9,   1, 0x0000, ""                                            ],
   "SAMP_TIM_SEL"                     => [ 8,   8,   1, 0x0000, ""                                            ],
   "FMODE_JTAG_VAL"                   => [ 9,   7,   1, 0x0000, ""                                            ],
   "FMODE_JTAG_EN"                    => [10,   6,   1, 0x0000, ""                                            ],
   "UPDATE_AFTER_CNT_EN"              => [11,   5,   1, 0x0000, ""                                            ],
   "LLP_TUNING_CMPLT_EN"              => [12,   4,   1, 0x0000, ""                                            ],
   "RMT_UPDATE_EN"                    => [13,   3,   1, 0x0000, ""                                            ],
   "DLINE_MAN_ADJ_EN"                 => [14,   2,   1, 0x0000, ""                                            ],
   "DLINE_MAN_HOLD"                   => [15,   1,   1, 0x0000, ""                                            ],
   "DLY_CTRL_RESET"                   => [16,   0,   1, 0x0000, ""                                            ],
);

%shub2_01_tdr_00010 =
(  NAME   => "DLY_CTRL0_NI0",
   NOTE   => "DLY_CTRL0_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI0",
   OPCODE => 0x60,
   LENGTH => 27,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub2_01_flds_00010
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00011 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%shub2_01_tdr_00011 =
(  NAME   => "DLY_CTRL1_NI0",
   NOTE   => "DLY_CTRL1_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI0",
   OPCODE => 0x61,
   LENGTH => 33,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00011
);

#-------------------------------------------------------------------------------


%shub2_01_flds_00012 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "SPARE"                            => [ 0,  23,   4, 0x0000, ""                                            ],
   "OBSERVE_ENABLE"                   => [ 1,  22,   1, 0x0000, ""                                            ],
   "DATA_CLK_MUX_SEL"                 => [ 2,  21,   1, 0x0000, ""                                            ],
   "PFSSD_SYNC_MODE"                  => [ 3,  20,   1, 0x0000, ""                                            ],
   "PFSSD_DEBUG_SEL"                  => [ 4,  18,   2, 0x0000, ""                                            ],
   "PFSSR_DEBUG_SEL"                  => [ 5,  12,   6, 0x0000, ""                                            ],
   "DAVAIL_SEL"                       => [ 6,  10,   2, 0x0000, ""                                            ],
   "CLK_ADAPT_HOLD_MODE"              => [ 7,   9,   1, 0x0000, ""                                            ],
   "SAMP_TIM_SEL"                     => [ 8,   8,   1, 0x0000, ""                                            ],
   "FMODE_JTAG_VAL"                   => [ 9,   7,   1, 0x0000, ""                                            ],
   "FMODE_JTAG_EN"                    => [10,   6,   1, 0x0000, ""                                            ],
   "UPDATE_AFTER_CNT_EN"              => [11,   5,   1, 0x0000, ""                                            ],
   "LLP_TUNING_CMPLT_EN"              => [12,   4,   1, 0x0000, ""                                            ],
   "RMT_UPDATE_EN"                    => [13,   3,   1, 0x0000, ""                                            ],
   "DLINE_MAN_ADJ_EN"                 => [14,   2,   1, 0x0000, ""                                            ],
   "DLINE_MAN_HOLD"                   => [15,   1,   1, 0x0000, ""                                            ],
   "DLY_CTRL_RESET"                   => [16,   0,   1, 0x0000, ""                                            ],
);


%shub2_01_tdr_00012 =
(  NAME   => "DLY_CTRL0_NI1",
   NOTE   => "DLY_CTRL0_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI1",
   OPCODE => 0x62,
   LENGTH => 27,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub2_01_flds_00012
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00013 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%shub2_01_tdr_00013 =
(  NAME   => "DLY_CTRL1_NI1",
   NOTE   => "DLY_CTRL1_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI1",
   OPCODE => 0x63,
   LENGTH => 33,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00013
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00014 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%shub2_01_tdr_00014 =
(  NAME   => "RACK_SLOT",
   NOTE   => "RACK_SLOT",
   OPNAME => "RACK_SLOT",
   OPCODE => 0x13,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00014
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00015 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "CRD_MGR_ENABLE"                   => [ 0, 184,   1, 0x0000, ""                                            ],
   "ENABLE_XCLRTIME"                  => [ 1, 183,   1, 0x0000, ""                                            ],
   "XCLRTIME"                         => [ 2, 175,   8, 0x0000, ""                                            ],
   "ENABLE_REWINDTIMEOUT"             => [ 3, 174,   1, 0x0000, ""                                            ],
   "REWINDTIMEOUT"                    => [ 4, 166,   8, 0x0000, ""                                            ],
   "ENABLE_BOOTTIME"                  => [ 5, 165,   1, 0x0000, ""                                            ],
   "BOOTTIME"                         => [ 6, 157,   8, 0x0000, ""                                            ],
   "ENABLE_FT_TIME"                   => [ 7, 156,   1, 0x0000, ""                                            ],
   "FT_TIME"                          => [ 8, 140,  16, 0x0000, ""                                            ],
   "ENABLE_REWINDSYNCRATIO"           => [ 9, 139,   1, 0x0000, ""                                            ],
   "REWINDSYNCRATIO"                  => [10, 133,   6, 0x0000, ""                                            ],
   "CREDIT_VC3_TEST"                  => [11, 126,   7, 0x0000, ""                                            ],
   "CREDIT_VC2_TEST"                  => [12, 119,   7, 0x0000, ""                                            ],
   "CREDIT_VC1_TEST"                  => [13, 112,   7, 0x0000, ""                                            ],
   "CREDIT_VC0_TEST"                  => [14, 105,   7, 0x0000, ""                                            ],
   "ENABLE_LINK"                      => [15, 104,   1, 0x0000, ""                                            ],
   "ENABLE_MAXRETRY"                  => [16, 103,   1, 0x0000, ""                                            ],
   "MAXRETRY"                         => [17,  87,  16, 0x0000, ""                                            ],
   "ENABLE_FTU_TIME"                  => [18,  86,   1, 0x0000, ""                                            ],
   "FTU_TIME"                         => [19,  74,  12, 0x0000, ""                                            ],
   "ENABLE_NULLTIMEOUT"               => [20,  73,   1, 0x0000, ""                                            ],
   "NULLTIMEOUT"                      => [21,  67,   6, 0x0000, ""                                            ],
   "ENABLE_MAXBURST"                  => [22,  66,   1, 0x0000, ""                                            ],
   "MAXBURST"                         => [23,  56,  10, 0x0000, ""                                            ],
   "ENABLE_USER_IGNORE_TIME"          => [24,  55,   1, 0x0000, ""                                            ],
   "USER_IGNORE_TIME"                 => [25,  15,  40, 0x0000, ""                                            ],
   "LINK_RESET_ENABLE"                => [26,  14,   1, 0x0000, ""                                            ],
   "QUEUE_CNT_ENABLE"                 => [27,  13,   1, 0x0000, ""                                            ],
   "CHARGE_UNDERFLOW"                 => [28,  12,   1, 0x0000, ""                                            ],
   "CHARGE_OVERFLOW"                  => [29,  11,   1, 0x0000, ""                                            ],
   "FAKED_ALL_SN_ERRORS"              => [30,  10,   1, 0x0000, ""                                            ],
   "SENT_ALL_SN_ERRORS"               => [31,   9,   1, 0x0000, ""                                            ],
   "SENT_ALL_CB_ERRORS"               => [32,   8,   1, 0x0000, ""                                            ],
   "FAKE_SN_ERROR"                    => [33,   7,   1, 0x0000, ""                                            ],
   "SEND_SN_ERROR"                    => [34,   6,   1, 0x0000, ""                                            ],
   "SEND_CB_ERROR"                    => [35,   5,   1, 0x0000, ""                                            ],
   "ENABLE_QUAD"                      => [36,   4,   1, 0x0000, ""                                            ],
   "ENABLE_RFTU"                      => [37,   3,   1, 0x0000, ""                                            ],
   "ENABLE_TUNING"                    => [38,   2,   1, 0x0000, ""                                            ],
   "ENABLE_AC"                        => [39,   1,   1, 0x0000, ""                                            ],
   "BIT_MODE32"                       => [40,   0,   1, 0x0000, ""                                            ],
);

%shub2_01_tdr_00015 =
(  NAME   => "LLP4_NI0",
   NOTE   => "NI0 LLP4 Configuration Data",
   OPNAME => "LLP4_NI0",
   OPCODE => 0x0A,
   LENGTH => 185,
   RESET  => [ 0x01000000, 0x00000006, 0x0C183000, 0x00000000, 0x00000000, 0x00006011 ],
   FIELDS => \%shub2_01_flds_00015
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00016 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "CRD_MGR_ENABLE"                   => [ 0, 184,   1, 0x0000, ""                                            ],
   "ENABLE_XCLRTIME"                  => [ 1, 183,   1, 0x0000, ""                                            ],
   "XCLRTIME"                         => [ 2, 175,   8, 0x0000, ""                                            ],
   "ENABLE_REWINDTIMEOUT"             => [ 3, 174,   1, 0x0000, ""                                            ],
   "REWINDTIMEOUT"                    => [ 4, 166,   8, 0x0000, ""                                            ],
   "ENABLE_BOOTTIME"                  => [ 5, 165,   1, 0x0000, ""                                            ],
   "BOOTTIME"                         => [ 6, 157,   8, 0x0000, ""                                            ],
   "ENABLE_FT_TIME"                   => [ 7, 156,   1, 0x0000, ""                                            ],
   "FT_TIME"                          => [ 8, 140,  16, 0x0000, ""                                            ],
   "ENABLE_REWINDSYNCRATIO"           => [ 9, 139,   1, 0x0000, ""                                            ],
   "REWINDSYNCRATIO"                  => [10, 133,   6, 0x0000, ""                                            ],
   "CREDIT_VC3_TEST"                  => [11, 126,   7, 0x0000, ""                                            ],
   "CREDIT_VC2_TEST"                  => [12, 119,   7, 0x0000, ""                                            ],
   "CREDIT_VC1_TEST"                  => [13, 112,   7, 0x0000, ""                                            ],
   "CREDIT_VC0_TEST"                  => [14, 105,   7, 0x0000, ""                                            ],
   "ENABLE_LINK"                      => [15, 104,   1, 0x0000, ""                                            ],
   "ENABLE_MAXRETRY"                  => [16, 103,   1, 0x0000, ""                                            ],
   "MAXRETRY"                         => [17,  87,  16, 0x0000, ""                                            ],
   "ENABLE_FTU_TIME"                  => [18,  86,   1, 0x0000, ""                                            ],
   "FTU_TIME"                         => [19,  74,  12, 0x0000, ""                                            ],
   "ENABLE_NULLTIMEOUT"               => [20,  73,   1, 0x0000, ""                                            ],
   "NULLTIMEOUT"                      => [21,  67,   6, 0x0000, ""                                            ],
   "ENABLE_MAXBURST"                  => [22,  66,   1, 0x0000, ""                                            ],
   "MAXBURST"                         => [23,  56,  10, 0x0000, ""                                            ],
   "ENABLE_USER_IGNORE_TIME"          => [24,  55,   1, 0x0000, ""                                            ],
   "USER_IGNORE_TIME"                 => [25,  15,  40, 0x0000, ""                                            ],
   "LINK_RESET_ENABLE"                => [26,  14,   1, 0x0000, ""                                            ],
   "QUEUE_CNT_ENABLE"                 => [27,  13,   1, 0x0000, ""                                            ],
   "CHARGE_UNDERFLOW"                 => [28,  12,   1, 0x0000, ""                                            ],
   "CHARGE_OVERFLOW"                  => [29,  11,   1, 0x0000, ""                                            ],
   "FAKED_ALL_SN_ERRORS"              => [30,  10,   1, 0x0000, ""                                            ],
   "SENT_ALL_SN_ERRORS"               => [31,   9,   1, 0x0000, ""                                            ],
   "SENT_ALL_CB_ERRORS"               => [32,   8,   1, 0x0000, ""                                            ],
   "FAKE_SN_ERROR"                    => [33,   7,   1, 0x0000, ""                                            ],
   "SEND_SN_ERROR"                    => [34,   6,   1, 0x0000, ""                                            ],
   "SEND_CB_ERROR"                    => [35,   5,   1, 0x0000, ""                                            ],
   "ENABLE_QUAD"                      => [36,   4,   1, 0x0000, ""                                            ],
   "ENABLE_RFTU"                      => [37,   3,   1, 0x0000, ""                                            ],
   "ENABLE_TUNING"                    => [38,   2,   1, 0x0000, ""                                            ],
   "ENABLE_AC"                        => [39,   1,   1, 0x0000, ""                                            ],
   "BIT_MODE32"                       => [40,   0,   1, 0x0000, ""                                            ],
);

%shub2_01_tdr_00016 =
(  NAME   => "LLP4_NI1",
   NOTE   => "NI1 LLP4 Configuration Data",
   OPNAME => "LLP4_NI1",
   OPCODE => 0x0B,
   LENGTH => 185,
   RESET  => [ 0x01000000, 0x00000006, 0x0C183000, 0x00000000, 0x00000000, 0x00006011 ],
   FIELDS => \%shub2_01_flds_00016
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00017 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%shub2_01_tdr_00017 =
(  NAME   => "LLP4TEST_NI0",
   NOTE   => "LLP NI0 Channel BIST",
   OPNAME => "LLP4TEST_NI0",
   OPCODE => 0x1A,
   LENGTH => 242,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x00000000, 0x00000000, 0x00100000, 0x14BFFFFE ],
   FIELDS => \%shub2_01_flds_00017
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00018 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%shub2_01_tdr_00018 =
(  NAME   => "LLP4TEST_NI1",
   NOTE   => "LLP NI1 Channel BIST",
   OPNAME => "LLP4TEST_NI1",
   OPCODE => 0x1B,
   LENGTH => 242,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x00000000, 0x00000000, 0x00100000, 0x14BFFFFE ],
   FIELDS => \%shub2_01_flds_00018
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00019 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub2_01_tdr_00019 =
(  NAME   => "TDR_DEFAULT",
   NOTE   => "Reset user defined TDR's",
   OPNAME => "TDR_DEFAULT",
   OPCODE => 0x7C,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub2_01_flds_00019
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00020 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub2_01_tdr_00020 =
(  NAME   => "JCLK_RST",
   NOTE   => "Clock Controller Reset Instruction",
   OPNAME => "JCLK_RST",
   OPCODE => 0x7D,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub2_01_flds_00020
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00021 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub2_01_tdr_00021 =
(  NAME   => "HIGHZ",
   NOTE   => "HIGHZ instruction",
   OPNAME => "HIGHZ",
   OPCODE => 0x04,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub2_01_flds_00021
);

#-------------------------------------------------------------------------------

%shub2_01_flds_00022 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "CODE"              => [   0,   0,   3, 0x0000, "Address mode encoding"  ],
);

%shub2_01_tdr_00022 =
(  NAME   => "ADDRESS_MODE",
   NOTE   => "ADDRESS_MODE instruction",
   OPNAME => "ADDRESS_MODE",
   OPCODE => 0x0C,
   LENGTH => 3,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub2_01_flds_00022
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00023 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%shub2_01_tdr_00023 =
(  NAME   => "ASIC_L1_DATA",
   NOTE   => "SHUB to L1 data register",
   OPNAME => "ASIC_L1_DATA",
   OPCODE => 0x11,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00023
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00024 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%shub2_01_tdr_00024 =
(  NAME   => "L1_ASIC_DATA",
   NOTE   => "L1 to SHUB data register",
   OPNAME => "L1_ASIC_DATA",
   OPCODE => 0x12,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00024
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00025 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "VALUE"             => [   0,   0,  24, 0x0000, "Time to ignore any warm reset input" ],
);

%shub2_01_tdr_00025 =
(  NAME   => "WARM_FENCE",
   NOTE   => "WARM_FENCE Register",
   OPNAME => "WARM_FENCE",
   OPCODE => 0x14,
   LENGTH => 24,
   RESET  => [ 0x000F42 ],
   FIELDS => \%shub2_01_flds_00025
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00026 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RESERVED_63_60"                   => [  0,  60,   4, 0x0000, ""                                            ],
   "ENABLE_LONG_PULSE"                => [  1,  59,   1, 0x0000, "Enable long FSB reset pulse"                 ],
   "RESERVED_58_50"                   => [  2,  50,   9, 0x0000, ""                                            ],
   "A_49_3"                           => [  3,   3,  47, 0x0000, "Value on FSB address pins during reset"      ],
   "RESERVED_2_0"                     => [  4,   0,   3, 0x0000, ""                                            ],
);

%shub2_01_tdr_00026 =
(  NAME   => "FSB_CONFIG",
   NOTE   => "FSB_CONFIG instruction",
   OPNAME => "FSB_CONFIG",
   OPCODE => 0x0D,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub2_01_flds_00026
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00027 =
#  NAME                                  IND  POS  LEN  MODE    DESCRIPTION
#  ----------------------------------    ---  ---  ---  ------  -----------------------
(  "PIO_RQ_CRD_CTL_DEPTH_TDR"         => [  0,  40,   6, 0x0000, ""                                            ],
   "PI_MD_RQ_CRD_CTL_DEPTH_TDR"       => [  1,  34,   6, 0x0000, ""                                            ],
   "PI_MD_RP_CRD_CTL_DEPTH_TDR"       => [  2,  28,   6, 0x0000, ""                                            ],
   "XN_MD_RQ_CRD_CTL_DEPTH_TDR"       => [  3,  22,   6, 0x0000, ""                                            ],
   "XN_MD_RP_CRD_CTL_DEPTH_TDR"       => [  4,  16,   6, 0x0000, ""                                            ],
   "REDUCE_XN_TNUMS_DATA_TDR"         => [  5,   0,  16, 0x0000, ""                                            ],
);

%shub2_01_tdr_00027 =
(  NAME   => "JNR_CONFIG",
   NOTE   => "JNR_CONFIG instruction",
   OPNAME => "JNR_CONFIG",
   OPCODE => 0x15,
   LENGTH => 46,
   RESET  => [ 0x00000F0C, 0x910C000F ],
   FIELDS => \%shub2_01_flds_00027
);


#-------------------------------------------------------------------------------

%shub2_01_flds_00028 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "CODE"              => [   0,   0,   3, 0x0000, "Address mode encoding"  ],
);

%shub2_01_tdr_00028 =
(  NAME   => "PROCESSOR_TYPE",
   NOTE   => "PROCESSOR_TYPE instruction",
   OPNAME => "PROCESSOR_TYPE",
   OPCODE => 0x0E,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub2_01_flds_00028
);



#-------------------------------------------------------------------------------

#-----------------------------------
#  HASH TABLE: LOOKUP TDR BY OPNAME
#-----------------------------------

%shub2_01_tdrs =
(  $shub2_01_tdr_00000{OPNAME} => \%shub2_01_tdr_00000,
   $shub2_01_tdr_00001{OPNAME} => \%shub2_01_tdr_00001,
   $shub2_01_tdr_00002{OPNAME} => \%shub2_01_tdr_00002,
   $shub2_01_tdr_00003{OPNAME} => \%shub2_01_tdr_00003,
   $shub2_01_tdr_00004{OPNAME} => \%shub2_01_tdr_00004,
   $shub2_01_tdr_00005{OPNAME} => \%shub2_01_tdr_00005,
   $shub2_01_tdr_00006{OPNAME} => \%shub2_01_tdr_00006,
   $shub2_01_tdr_00007{OPNAME} => \%shub2_01_tdr_00007,
   $shub2_01_tdr_00008{OPNAME} => \%shub2_01_tdr_00008,
   $shub2_01_tdr_00009{OPNAME} => \%shub2_01_tdr_00009,
   $shub2_01_tdr_00010{OPNAME} => \%shub2_01_tdr_00010,
   $shub2_01_tdr_00011{OPNAME} => \%shub2_01_tdr_00011,
   $shub2_01_tdr_00012{OPNAME} => \%shub2_01_tdr_00012,
   $shub2_01_tdr_00013{OPNAME} => \%shub2_01_tdr_00013,
   $shub2_01_tdr_00014{OPNAME} => \%shub2_01_tdr_00014,
   $shub2_01_tdr_00015{OPNAME} => \%shub2_01_tdr_00015,
   $shub2_01_tdr_00016{OPNAME} => \%shub2_01_tdr_00016,
   $shub2_01_tdr_00017{OPNAME} => \%shub2_01_tdr_00017,
   $shub2_01_tdr_00018{OPNAME} => \%shub2_01_tdr_00018,
   $shub2_01_tdr_00019{OPNAME} => \%shub2_01_tdr_00019,
   $shub2_01_tdr_00020{OPNAME} => \%shub2_01_tdr_00020,
   $shub2_01_tdr_00021{OPNAME} => \%shub2_01_tdr_00021,
   $shub2_01_tdr_00022{OPNAME} => \%shub2_01_tdr_00022,
   $shub2_01_tdr_00023{OPNAME} => \%shub2_01_tdr_00023,
   $shub2_01_tdr_00024{OPNAME} => \%shub2_01_tdr_00024,
   $shub2_01_tdr_00025{OPNAME} => \%shub2_01_tdr_00025,
   $shub2_01_tdr_00026{OPNAME} => \%shub2_01_tdr_00026,
   $shub2_01_tdr_00027{OPNAME} => \%shub2_01_tdr_00027,
   $shub2_01_tdr_00028{OPNAME} => \%shub2_01_tdr_00028
);


#-----------------------------------
#  ARRAY: STORE OPNAMES IN "ORDER"
#-----------------------------------

@shub2_01_opnames =
(  $shub2_01_tdr_00000{OPNAME},
   $shub2_01_tdr_00001{OPNAME},
   $shub2_01_tdr_00002{OPNAME},
   $shub2_01_tdr_00003{OPNAME},
   $shub2_01_tdr_00004{OPNAME},
   $shub2_01_tdr_00005{OPNAME},
   $shub2_01_tdr_00006{OPNAME},
   $shub2_01_tdr_00007{OPNAME},
   $shub2_01_tdr_00008{OPNAME},
   $shub2_01_tdr_00009{OPNAME},
   $shub2_01_tdr_00010{OPNAME},
   $shub2_01_tdr_00011{OPNAME},
   $shub2_01_tdr_00012{OPNAME},
   $shub2_01_tdr_00013{OPNAME},
   $shub2_01_tdr_00014{OPNAME},
   $shub2_01_tdr_00015{OPNAME},
   $shub2_01_tdr_00016{OPNAME},
   $shub2_01_tdr_00017{OPNAME},
   $shub2_01_tdr_00018{OPNAME},
   $shub2_01_tdr_00019{OPNAME},
   $shub2_01_tdr_00020{OPNAME},
   $shub2_01_tdr_00021{OPNAME},
   $shub2_01_tdr_00022{OPNAME},
   $shub2_01_tdr_00023{OPNAME},
   $shub2_01_tdr_00024{OPNAME},
   $shub2_01_tdr_00025{OPNAME},
   $shub2_01_tdr_00026{OPNAME},
   $shub2_01_tdr_00027{OPNAME},
   $shub2_01_tdr_00028{OPNAME}
);

#-------------------------------------------------------------------------------

1;
