#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: shub_01_jtdrs.pm,v 1.5 2002/03/20 21:52:26 beighley Exp $

#-------------------------------------------------------------------------------
#
#  FILE: shub_01_jtdrs.pm
#
#  NOTE: Definitions of the JTAG test data registers (TDRs) for SHUB.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     02/21/02  Karen Beighley  Original source code submitted into tree.
#     03/11/02  Karen Beighley  Modified fields in SH_JTAG_DLY_CTRL1_* TDR.
#     03/19/02  Karen Beighley  Fixed NI1_CLK_I_TERM_H in NL3_TERM TDR.
#
#-------------------------------------------------------------------------------

$shub_01_tdr_defined = 0;

#-------------------------------------------------------------------------------

%shub_01_flds_00000 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub_01_tdr_00000 =
(  NAME   => "BYPASS",
   NOTE   => "BYPASS JTAG Register",
   OPNAME => "BYPASS",
   OPCODE => 0x7F,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub_01_flds_00000
);

#-------------------------------------------------------------------------------

%shub_01_flds_00001 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "VERSION",          => [   0,  28,   4, 0x0000, "Version number"         ],
   "PARTNUMBER",       => [   1,  12,  16, 0x0000, "Partnumber"             ],
   "MANUFACTURER",     => [   2,   1,  11, 0x0000, "Manufacturer Code"      ],
   "LSB",              => [   3,   0,   1, 0x0000, "Mandatory LSB"          ]
);

%shub_01_tdr_00001 =
(  NAME   => "IDCODE",
   NOTE   => "JTAG Device Identification Register",
   OPNAME => "IDCODE",
   OPCODE => 0x03,
   LENGTH => 32,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub_01_flds_00001
);

#-------------------------------------------------------------------------------

%shub_01_flds_00002 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "GTL_CONFIG_RE"         => [   0,  55,   1, 0x0000, "description"            ],
   "FSB_CFG_AUX"           => [   1,  53,   2, 0x0000, "description"            ],
   "FSB_CFG_EN_BIST"       => [   2,  52,   1, 0x0000, "description"            ],
   "FSB_CFG_OUTPUT_TRI"    => [   3,  48,   4, 0x0000, "description"            ],
   "FSB_CFG_CLOCK_RATIO"   => [   4,  43,   5, 0x0000, "description"            ],
   "FSB_CFG_EN_BUS_PARKING"=> [   5,  42,   1, 0x0000, "description"            ],
   "FSB_CFG_SAMPLE_BINIT"  => [   6,  41,   1, 0x0000, "description"            ],
   "FSB_CFG_IOQ_DEPTH"     => [   7,  40,   1, 0x0000, "description"            ],
   "JTAG_MCI_OVERRIDE"     => [   8,  39,   1, 0x0000, "description"            ],
   "JTAG_MCI_TARGET"       => [   9,  25,  14, 0x0000, "description"            ],
   "JTAG_MCI_RESET_DELAY"  => [  10,  21,   4, 0x0000, "description"            ],
   "WRT90_OVERRIDE"        => [  11,  20,   1, 0x0000, "description"            ],
   "WRT90_OVERRIDEA"       => [  12,  19,   1, 0x0000, "description"            ],
   "WRT90_TARGET"          => [  13,   5,  14, 0x0000, "description"            ],
   "II_CLK_SEL"            => [  14,   3,   2, 0x0000, "description"            ],
   "NI_CLK_SEL"            => [  15,   2,   1, 0x0000, "description"            ],
   "MD_CLK_SEL"            => [  16,   0,   2, 0x0000, "description"            ],
);

%shub_01_tdr_00002 =
(  NAME   => "CONFIG",
   NOTE   => "SHUB Pre-reset Configuration",
   OPNAME => "CONFIG",
   OPCODE => 0x7E,
   LENGTH => 56,
   RESET  => [ 0x0000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00002
);

#-------------------------------------------------------------------------------

%shub_01_flds_00003 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "WRAP_JTAG_SCAN_ACCESS" => [   0,   4,  18, 0x0000, "description"            ],
   "CIR"                   => [   1,   3,   1, 0x0000, "description"            ],
   "INV"                   => [   2,   2,   1, 0x0000, "description"            ],
   "CL2_CONFIG"            => [   3,   0,   2, 0x0000, "description"            ]
);

%shub_01_tdr_00003 =
(  NAME   => "SCAN_CTL",
   NOTE   => "Controls gmoc_mode and scan access",
   OPNAME => "SCAN_CTL",
   OPCODE => 0x05,
   LENGTH => 23,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00003
);

#-------------------------------------------------------------------------------


%shub_01_flds_00004 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "NI0_CLK_I_TERM_H"    => [   0,   7,   1, 0x0000, "description"            ],
   "NI0_CLK_I_TERM_L"    => [   1,   6,   1, 0x0000, "description"            ],
   "NI0_DATA_F_TERM_H"   => [   2,   5,   1, 0x0000, "description"            ],
   "NI0_DATA_F_TERM_L"   => [   3,   4,   1, 0x0000, "description"            ],
   "NI1_CLK_I_TERM_H"    => [   4,   3,   1, 0x0000, "description"            ],
   "NI1_CLK_I_TERM_L"    => [   5,   2,   1, 0x0000, "description"            ],
   "NI1_DATA_F_TERM_H"   => [   6,   1,   1, 0x0000, "description"            ],
   "NI1_DATA_F_TERM_L"   => [   7,   0,   1, 0x0000, "description"            ]
);

%shub_01_tdr_00004 =
(  NAME   => "NL3_TERM",
   NOTE   => "NL4 termination value selection",
   OPNAME => "NL3_TERM",
   OPCODE => 0x09,
   LENGTH => 8,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00004
);

#-------------------------------------------------------------------------------

%shub_01_flds_00005 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "START"               => [   0,  29,   1, 0x0000, "description"            ],
   "WRITE"               => [   1,  28,   1, 0x0000, "description"            ],
   "PIO_ADDRESS"         => [   2,   0,  28, 0x0000, "description"            ]
);

%shub_01_tdr_00005 =
(  NAME   => "PIO_CTRL",
   NOTE   => "Control Register for PIO",
   OPNAME => "PIO_CTRL",
   OPCODE => 0x0F,
   LENGTH => 30,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00005
);

#-------------------------------------------------------------------------------

%shub_01_flds_00006 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ERROR"               => [   0,  65,   1, 0x0000, "description"            ],
   "VALID"               => [   1,  64,   1, 0x0000, "description"            ],
   "DATA"                => [   2,   0,  64, 0x0000, "description"            ]
);

%shub_01_tdr_00006 =
(  NAME   => "PIO_DATA",
   NOTE   => "Data Register for PIO",
   OPNAME => "PIO_DATA",
   OPCODE => 0x10,
   LENGTH => 66,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00006
);

#-------------------------------------------------------------------------------

%shub_01_flds_00007 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "MABIST_BIST_M3"      => [   0,  24,   2, 0x0000, "description"            ],
   "MABIST_ABDONE"       => [   1,  18,   6, 0x0000, "description"            ],
   "MABIST_SYSPF"        => [   2,   0,  18, 0x0000, "description"            ]
);

%shub_01_tdr_00007 =
(  NAME   => "RUNABIST",
   NOTE   => "SRAM RUNABIST control",
   OPNAME => "RUNABIST",
   OPCODE => 0x08,
   LENGTH => 26,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00007
);

#-------------------------------------------------------------------------------

%shub_01_flds_00008 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "JTAG_MCI_L_LOCK"   => [   0,  67,   9, 0x0000, ""                       ],
   "JTAG_MCI_R_LOCK"   => [   1,  58,   9, 0x0000, ""                       ],
   "WRT90_LOCK"        => [   2,  57,   1, 0x0000, ""                       ],
   "LOAD"              => [   3,  53,   4, 0x0000, ""                       ],
   "GMXC_SEQ"          => [   4,  33,  20, 0x0000, ""                       ],
   "SLOW1CLK"          => [   5,  32,   1, 0x0000, ""                       ],
   "SLOW2CLK"          => [   6,  31,   1, 0x0000, ""                       ],
   "GMXC_TDR_SEL"      => [   7,   6,  25, 0x0000, ""                       ],
   "TP"                => [   8,   2,   4, 0x0000, ""                       ],
   "SEL_CLKSTOP"       => [   9,   1,   1, 0x0000, ""                       ],
   "JTAG_CLKSTOP"      => [  10,   0,   1, 0x0000, ""                       ],
);

%shub_01_tdr_00008 =
(  NAME   => "CLKCNTL",
   NOTE   => "JTAG CLKCNTL control Register",
   OPNAME => "CLKCNTL",
   OPCODE => 0x07,
   LENGTH => 76,
   RESET  => [ 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00008
);

#-------------------------------------------------------------------------------


%shub_01_flds_00009 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "NI0_N_REFIO_VALUE"   => [   0, 117,   9, 0x0000, "description"            ],
   "NI0_P_REFIO_VALUE"   => [   1, 108,   9, 0x0000, "description"            ],
   "NI1_N_REFIO_VALUE"   => [   2,  99,   9, 0x0000, "description"            ],
   "NI1_P_REFIO_VALUE"   => [   3,  90,   9, 0x0000, "description"            ],
   "FSB_N_REFIO_VALUE"   => [   4,  81,   9, 0x0000, "description"            ],
   "FSB_P_REFIO_VALUE"   => [   5,  72,   9, 0x0000, "description"            ],
   "FSB_FILTER"          => [   6,  71,   1, 0x0000, "description"            ],
   "FSB_BYPASS"          => [   7,  70,   1, 0x0000, "description"            ],
   "FSB_DISABLE"         => [   8,  69,   1, 0x0000, "description"            ],
   "FSB_RESET"           => [   9,  68,   1, 0x0000, "description"            ],
   "FSB_N_TDR_VALUE"     => [  10,  60,   8, 0x0000, "description"            ],
   "FSB_P_TDR_VALUE"     => [  11,  52,   8, 0x0000, "description"            ],
   "FSB_SEL_TDR_VAL"     => [  12,  51,   1, 0x0000, "description"            ],
   "FSB_N_UPDN"          => [  13,  50,   1, 0x0000, "description"            ],
   "FSB_P_UPDN"          => [  14,  49,   1, 0x0000, "description"            ],
   "FSB_OVRD_UPDN"       => [  15,  48,   1, 0x0000, "description"            ],
   "NI1_FILTER"          => [  16,  47,   1, 0x0000, "description"            ],
   "NI1_BYPASS"          => [  17,  46,   1, 0x0000, "description"            ],
   "NI1_DISABLE"         => [  18,  45,   1, 0x0000, "description"            ],
   "NI1_RESET"           => [  19,  44,   1, 0x0000, "description"            ],
   "NI1_N_TDR_VALUE"     => [  20,  36,   8, 0x0000, "description"            ],
   "NI1_P_TDR_VALUE"     => [  21,  28,   8, 0x0000, "description"            ],
   "NI1_SEL_TDR_VAL"     => [  22,  27,   1, 0x0000, "description"            ],
   "NI1_N_UPDN"          => [  23,  26,   1, 0x0000, "description"            ],
   "NI1_P_UPDN"          => [  24,  25,   1, 0x0000, "description"            ],
   "NI1_OVRD_UPDN"       => [  25,  24,   1, 0x0000, "description"            ],
   "NI0_FILTER"          => [  26,  23,   1, 0x0000, "description"            ],
   "NI0_BYPASS"          => [  27,  22,   1, 0x0000, "description"            ],
   "NI0_DISABLE"         => [  28,  21,   1, 0x0000, "description"            ],
   "NI0_RESET"           => [  29,  20,   1, 0x0000, "description"            ],
   "NI0_N_TDR_VALUE"     => [  30,  12,   8, 0x0000, "description"            ],
   "NI0_P_TDR_VALUE"     => [  31,   4,   8, 0x0000, "description"            ],
   "NI0_SEL_TDR_VAL"     => [  32,   3,   1, 0x0000, "description"            ],
   "NI0_N_UPDN"          => [  33,   2,   1, 0x0000, "description"            ],
   "NI0_P_UPDN"          => [  34,   1,   1, 0x0000, "description"            ],
   "NI0_OVRD_UPDN"       => [  35,   0,   1, 0x0000, "description"            ]
);

%shub_01_tdr_00009 =
(  NAME   => "ICNTLR",
   NOTE   => "Impedance Controller for APF and GTL I/O",
   OPNAME => "ICNTLR",
   OPCODE => 0x06,
   LENGTH => 126,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00009
);

#-------------------------------------------------------------------------------

%shub_01_flds_00010 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "DAVAIL_SEL"          => [   0,   8,   2, 0x0000, "description"            ],
   "FMODE_JTAG_VAL"      => [   1,   7,   1, 0x0000, "description"            ],
   "FMODE_JTAG_EN"       => [   2,   6,   1, 0x0000, "description"            ],
   "UPDATE_AFTER_CNT_EN" => [   3,   5,   1, 0x0000, "description"            ],
   "LLP_TUNING_CMPLT_EN" => [   4,   4,   1, 0x0000, "description"            ],
   "RMT_UPDATE_EN"       => [   5,   3,   1, 0x0000, "description"            ],
   "DLINE_MAN_ADJ_EN"    => [   6,   2,   1, 0x0000, "description"            ],
   "DLINE_MAN_HOLD"      => [   7,   1,   1, 0x0000, "description"            ],
   "DLY_CTRL_RESET"      => [   8,   0,   1, 0x0000, "description"            ]
);

%shub_01_tdr_00010 =
(  NAME   => "DLY_CTRL0_NI0",
   NOTE   => "DLY_CTRL0_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI0",
   OPCODE => 0x60,
   LENGTH => 10,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00010
);

#-------------------------------------------------------------------------------

%shub_01_flds_00011 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%shub_01_tdr_00011 =
(  NAME   => "DLY_CTRL1_NI0",
   NOTE   => "DLY_CTRL1_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI0",
   OPCODE => 0x61,
   LENGTH => 33,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00011
);

#-------------------------------------------------------------------------------


%shub_01_flds_00012 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "DAVAIL_SEL"          => [   0,   8,   2, 0x0000, "description"            ],
   "FMODE_JTAG_VAL"      => [   1,   7,   1, 0x0000, "description"            ],
   "FMODE_JTAG_EN"       => [   2,   6,   1, 0x0000, "description"            ],
   "UPDATE_AFTER_CNT_EN" => [   3,   5,   1, 0x0000, "description"            ],
   "LLP_TUNING_CMPLT_EN" => [   4,   4,   1, 0x0000, "description"            ],
   "RMT_UPDATE_EN"       => [   5,   3,   1, 0x0000, "description"            ],
   "DLINE_MAN_ADJ_EN"    => [   6,   2,   1, 0x0000, "description"            ],
   "DLINE_MAN_HOLD"      => [   7,   1,   1, 0x0000, "description"            ],
   "DLY_CTRL_RESET"      => [   8,   0,   1, 0x0000, "description"            ]
);

%shub_01_tdr_00012 =
(  NAME   => "DLY_CTRL0_NI1",
   NOTE   => "DLY_CTRL0_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI1",
   OPCODE => 0x62,
   LENGTH => 10,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00012
);

#-------------------------------------------------------------------------------

%shub_01_flds_00013 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%shub_01_tdr_00013 =
(  NAME   => "DLY_CTRL1_NI1",
   NOTE   => "DLY_CTRL1_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI1",
   OPCODE => 0x63,
   LENGTH => 33,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00013
);

#-------------------------------------------------------------------------------

%shub_01_flds_00014 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "JTAG_ENABLE"       => [   0,  56,   1, 0x0000, "Enables JTAG access"    ],
   "LLP_RESET"         => [   1,  55,   1, 0x0000, "Load or initiate test"  ],
   "ERR_CAPTURE"       => [   2,  54,   1, 0x0000, "Capture 1st mpkt w/ error" ],
   "CLEAR"             => [   3,  53,   1, 0x0000, "Clear and arm"          ],
   "FLIT_SELECT"       => [   4,  50,   3, 0x0000, "Flit selection"         ],
   "CRC_ERROR"         => [   5,  49,   1, 0x0000, "CRC error detected"     ],
   "VALID"             => [   6,  48,   1, 0x0000, "Valid flag"             ],
   "DATA"              => [   7,  28,  20, 0x0000, "Captured flit data"     ],
   "MASK"              => [   8,  20,   8, 0x0000, "Mask for packet data"   ],
   "SEED"              => [   9,   0,  20, 0x0000, "Seed for packet data"   ],
);

%shub_01_tdr_00014 =
(  NAME   => "LLP_II",
   NOTE   => "LLP II Channel Bist",
   OPNAME => "LLP_II",
   OPCODE => 0x0C,
   LENGTH => 57,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00014
);

#-------------------------------------------------------------------------------

%shub_01_flds_00015 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "CHARGE_UNDERFLOW"    => [   0,  12,   1, 0x0000, "description"            ],
   "CHARGE_OVERFLOW"     => [   1,  11,   1, 0x0000, "description"            ],
   "FAKED_ALL_SN_ERRORS" => [   2,  10,   1, 0x0000, "description"            ],
   "SENT_ALL_SN_ERRORS"  => [   3,   9,   1, 0x0000, "description"            ],
   "SENT_ALL_CB_ERRORS"  => [   4,   8,   1, 0x0000, "description"            ],
   "FAKE_SN_ERROR"       => [   5,   7,   1, 0x0000, "description"            ],
   "SEND_SN_ERROR"       => [   6,   6,   1, 0x0000, "description"            ],
   "SEND_CB_ERROR"       => [   7,   5,   1, 0x0000, "description"            ],
   "ENABLE_QUAD"         => [   8,   4,   1, 0x0000, "description"            ],
   "ENABLE_RFTU"         => [   9,   3,   1, 0x0000, "description"            ],
   "ENABLE_TUNING"       => [  10,   2,   1, 0x0000, "description"            ],
   "ENABLE_AC"           => [  11,   1,   1, 0x0000, "description"            ],
   "BIT_MODE32"          => [  12,   0,   1, 0x0000, "description"            ],
);

%shub_01_tdr_00015 =
(  NAME   => "LLP4_NI0",
   NOTE   => "NI0 LLP4 Configuration Data",
   OPNAME => "LLP4_NI0",
   OPCODE => 0x0A,
   LENGTH => 13,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00015
);

#-------------------------------------------------------------------------------

%shub_01_flds_00016 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "CHARGE_UNDERFLOW"    => [   0,  12,   1, 0x0000, "description"            ],
   "CHARGE_OVERFLOW"     => [   1,  11,   1, 0x0000, "description"            ],
   "FAKED_ALL_SN_ERRORS" => [   2,  10,   1, 0x0000, "description"            ],
   "SENT_ALL_SN_ERRORS"  => [   3,   9,   1, 0x0000, "description"            ],
   "SENT_ALL_CB_ERRORS"  => [   4,   8,   1, 0x0000, "description"            ],
   "FAKE_SN_ERROR"       => [   5,   7,   1, 0x0000, "description"            ],
   "SEND_SN_ERROR"       => [   6,   6,   1, 0x0000, "description"            ],
   "SEND_CB_ERROR"       => [   7,   5,   1, 0x0000, "description"            ],
   "ENABLE_QUAD"         => [   8,   4,   1, 0x0000, "description"            ],
   "ENABLE_RFTU"         => [   9,   3,   1, 0x0000, "description"            ],
   "ENABLE_TUNING"       => [  10,   2,   1, 0x0000, "description"            ],
   "ENABLE_AC"           => [  11,   1,   1, 0x0000, "description"            ],
   "BIT_MODE32"          => [  12,   0,   1, 0x0000, "description"            ],
);

%shub_01_tdr_00016 =
(  NAME   => "LLP4_NI1",
   NOTE   => "NI1 LLP4 Configuration Data",
   OPNAME => "LLP4_NI1",
   OPCODE => 0x0B,
   LENGTH => 13,
   RESET  => [ 0x0000000 ],
   FIELDS => \%shub_01_flds_00016
);

#-------------------------------------------------------------------------------

%shub_01_flds_00017 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%shub_01_tdr_00017 =
(  NAME   => "LLP4TEST_NI0",
   NOTE   => "LLP NI0 Channel BIST",
   OPNAME => "LLP4TEST_NI0",
   OPCODE => 0x1A,
   LENGTH => 242,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00017
);

#-------------------------------------------------------------------------------

%shub_01_flds_00018 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%shub_01_tdr_00018 =
(  NAME   => "LLP4TEST_NI1",
   NOTE   => "LLP NI1 Channel BIST",
   OPNAME => "LLP4TEST_NI1",
   OPCODE => 0x1B,
   LENGTH => 242,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%shub_01_flds_00018
);

#-------------------------------------------------------------------------------

%shub_01_flds_00019 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub_01_tdr_00019 =
(  NAME   => "TDR_DEFAULT",
   NOTE   => "Reset user defined TDR's",
   OPNAME => "TDR_DEFAULT",
   OPCODE => 0x7C,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub_01_flds_00019
);

#-------------------------------------------------------------------------------

%shub_01_flds_00020 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub_01_tdr_00020 =
(  NAME   => "JCLK_RST",
   NOTE   => "Clock Controller Reset Instruction",
   OPNAME => "JCLK_RST",
   OPCODE => 0x7D,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub_01_flds_00020
);

#-------------------------------------------------------------------------------

%shub_01_flds_00021 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub_01_tdr_00021 =
(  NAME   => "HIGHZ",
   NOTE   => "HIGHZ instruction",
   OPNAME => "HIGHZ",
   OPCODE => 0x04,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub_01_flds_00021
);

#-------------------------------------------------------------------------------

%shub_01_flds_00022 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%shub_01_tdr_00022 =
(  NAME   => "SCAN_CHAN",
   NOTE   => "SCAN_CHAN instruction",
   OPNAME => "SCAN_CHAN",
   OPCODE => 0x39,
   LENGTH => 1,
   RESET  => [ 0x00000000 ],
   FIELDS => \%shub_01_flds_00022
);

#-------------------------------------------------------------------------------

#-----------------------------------
#  HASH TABLE: LOOKUP TDR BY OPNAME
#-----------------------------------

%shub_01_tdrs =
(  $shub_01_tdr_00000{OPNAME} => \%shub_01_tdr_00000,
   $shub_01_tdr_00001{OPNAME} => \%shub_01_tdr_00001,
   $shub_01_tdr_00002{OPNAME} => \%shub_01_tdr_00002,
   $shub_01_tdr_00003{OPNAME} => \%shub_01_tdr_00003,
   $shub_01_tdr_00004{OPNAME} => \%shub_01_tdr_00004,
   $shub_01_tdr_00005{OPNAME} => \%shub_01_tdr_00005,
   $shub_01_tdr_00006{OPNAME} => \%shub_01_tdr_00006,
   $shub_01_tdr_00007{OPNAME} => \%shub_01_tdr_00007,
   $shub_01_tdr_00008{OPNAME} => \%shub_01_tdr_00008,
   $shub_01_tdr_00009{OPNAME} => \%shub_01_tdr_00009,
   $shub_01_tdr_00010{OPNAME} => \%shub_01_tdr_00010,
   $shub_01_tdr_00011{OPNAME} => \%shub_01_tdr_00011,
   $shub_01_tdr_00012{OPNAME} => \%shub_01_tdr_00012,
   $shub_01_tdr_00013{OPNAME} => \%shub_01_tdr_00013,
   $shub_01_tdr_00014{OPNAME} => \%shub_01_tdr_00014,
   $shub_01_tdr_00015{OPNAME} => \%shub_01_tdr_00015,
   $shub_01_tdr_00016{OPNAME} => \%shub_01_tdr_00016,
   $shub_01_tdr_00017{OPNAME} => \%shub_01_tdr_00017,
   $shub_01_tdr_00018{OPNAME} => \%shub_01_tdr_00018,
   $shub_01_tdr_00019{OPNAME} => \%shub_01_tdr_00019,
   $shub_01_tdr_00020{OPNAME} => \%shub_01_tdr_00020,
   $shub_01_tdr_00021{OPNAME} => \%shub_01_tdr_00021,
   $shub_01_tdr_00022{OPNAME} => \%shub_01_tdr_00022
);


#-----------------------------------
#  ARRAY: STORE OPNAMES IN "ORDER"
#-----------------------------------

@shub_01_opnames =
(  $shub_01_tdr_00000{OPNAME},
   $shub_01_tdr_00001{OPNAME},
   $shub_01_tdr_00002{OPNAME},
   $shub_01_tdr_00003{OPNAME},
   $shub_01_tdr_00004{OPNAME},
   $shub_01_tdr_00005{OPNAME},
   $shub_01_tdr_00006{OPNAME},
   $shub_01_tdr_00007{OPNAME},
   $shub_01_tdr_00008{OPNAME},
   $shub_01_tdr_00009{OPNAME},
   $shub_01_tdr_00010{OPNAME},
   $shub_01_tdr_00011{OPNAME},
   $shub_01_tdr_00012{OPNAME},
   $shub_01_tdr_00013{OPNAME},
   $shub_01_tdr_00014{OPNAME},
   $shub_01_tdr_00015{OPNAME},
   $shub_01_tdr_00016{OPNAME},
   $shub_01_tdr_00017{OPNAME},
   $shub_01_tdr_00018{OPNAME},
   $shub_01_tdr_00019{OPNAME},
   $shub_01_tdr_00020{OPNAME},
   $shub_01_tdr_00021{OPNAME},
   $shub_01_tdr_00022{OPNAME}
);

#-------------------------------------------------------------------------------

1;
