# 
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
# 
# @(#)$Id: shub_subs.pm,v 1.23 2006/05/15 15:29:41 meyer Exp $
#-------------------------------------------------------------------------------
#
#  FILE: shub_subs.pm
#
#  DESCRIPTION:  This file contains PERL constants and subroutines that are
#     specific to the SN2 SHUB ASIC.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  ----------------------------------------------
#     12/11/01  Karen Beighley  Original source code.
#     12/13/01  Karen Beighley  Minor change: increased entries in "shub_addr".
#     12/17/01  Karen Beighley  Added definition of $USE_SHUB_SUBS_PM.
#     11/20/02  Karen Beighley  Added Gary Meyer's "shub_tools" source with
#                                  includes decode subroutines.
#                               Initial source submission for release.
#     01/07/03  Karen Beighley  Merged changes from Gary's source files.
#     05/07/03  Gary Meyer      Improved decode of PI_ERROR_DETAIL_2
#     01/09/04  Karen Beighley  Added the interrupt enable register information.
#     03/12/04  Karen Beighley  Added support for SHUB2 chip (reset DV).
#     08/27/04  Gary Meyer	More PI_ERROR_DETAIL_2 decode changes.
#     01/24/05  Gary Meyer	Added decode_shub2_sn2net_header() for Shub2 & TIO.
#     01/24/05  Gary Meyer	Updated @sn2net_command_list so it's coherent with 
#                               //shrd_des/protocol/include/c/sn2net.h as of 1/24/05.
#     01/24/05  Gary Meyer	Updated decode_shub2_sn2net_header() for 52 addr bits.
#				Added decode_sh2_pi_err_detail().
#     06/01/05  Gary Meyer	Updated decode_pi_err_detail() for MSG_LEN error.
#     06/14/05  Gary Meyer	Updated decode_sh2_pi_err_detail() for MSG_LEN error.
#     07/02/05  Gary Meyer	Updated decode_sh2_pi_err_detail() with Source decode fix.
#     07/19/05  Gary Meyer	Minor change to output in decode_shub2_sn2net_header().
#     08/09/05  Gary Meyer	Added decode of ATTR field in decode_pi_err_detail().
#     02/13/06  Gary Meyer      Updated to use AID_DECODE for Shub2.
#
#-------------------------------------------------------------------------------

$USE_SHUB_SUBS_PM = 1;    # Define to allow test for module inclusion


#-------------------------------------------------------------------------------
#  DEPENDENCIES
#-------------------------------------------------------------------------------

if ( !defined($USE_JTAG_SUBS_PM) )
{  $emess = "Required module has not been included in PERL script.";
   $emess .= "\nThe \"use jtag_subs;\" statement must appear before using \"shub_subs\".";
   dsp_error_and_exit($emess);
   exit -1;
}


#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES
#-------------------------------------------------------------------------------

$SHUB_IR_LEN = 7;

$SHUB_MD_CHIPLET = 0;  # chiplet value for address <27:25> of PIO_CTRL field
$SHUB_LB_CHIPLET = 1;  # chiplet value for address <27:25> of PIO_CTRL field
$SHUB_PI_CHIPLET = 2;  # chiplet value for address <27:25> of PIO_CTRL field
$SHUB_XN_CHIPLET = 5;  # chiplet value for address <27:25> of PIO_CTRL field

%shub_inst_lj = ( TDR_DEFAULT   => 0x3E000000,  JCLK_RST      => 0xBE000000,
                  CONFIG        => 0x7E000000,  IDCODE        => 0xC0000000,
                  LLP4_NI0      => 0x50000000,  LLP4_NI1      => 0xD0000000,
                  DLY_CTRL0_NI0 => 0x06000000,  DLY_CTRL1_NI0 => 0x86000000,
                  DLY_CTRL0_NI1 => 0x46000000,  DLY_CTRL1_NI1 => 0xC6000000 );

%shub_dr_len  = ( TDR_DEFAULT   =>  0,  JCLK_RST      =>  0,
                  CONFIG        => 56,  IDCODE        => 32,
                  LLP4_NI0      => 13,  LLP4_NI1      => 13,
                  DLY_CTRL0_NI0 => 10,  DLY_CTRL1_NI0 => 33,
                  DLY_CTRL0_NI1 => 10,  DLY_CTRL1_NI1 => 33 );


$INIT_MODE_NONE             = 0x00;
$INIT_MODE_JTAG_RESET_HARD  = 0x01;
$INIT_MODE_JTAG_RESET_SOFT  = 0x02;
$INIT_MODE_LOAD_TDR_DEFAULT = 0x04;


#-------------------------------------------------------------------------------
#  PACKET GENERATOR SPECIFICS - The order of the address array cannot change!
#-------------------------------------------------------------------------------

$SH_CHIPLET_MD = 0;
$SH_CHIPLET_LB = 1;
$SH_CHIPLET_PI = 2;
$SH_CHIPLET_II = 3;

@shub_pktgen_addr =
( 0x0000000110060580, # SH_SHUB_ID (NODE_ID field @ bit 32)
  0x0000000110062000, # SH_DIAG_MSG_CNTRL
  0x0000000110062980, # SH_DIAG_MSG_HDR0
  0x0000000110062A00, # SH_DIAG_MSG_HDR1
  0x0000000110062080, # SH_DIAG_MSG_DATA0L
  0x0000000110062100, # SH_DIAG_MSG_DATA0U
  0x0000000110062180, # SH_DIAG_MSG_DATA1L
  0x0000000110062200, # SH_DIAG_MSG_DATA1U
  0x0000000110062280, # SH_DIAG_MSG_DATA2L
  0x0000000110062300, # SH_DIAG_MSG_DATA2U
  0x0000000110062380, # SH_DIAG_MSG_DATA3L
  0x0000000110062400, # SH_DIAG_MSG_DATA3U
  0x0000000110062480, # SH_DIAG_MSG_DATA4L
  0x0000000110062500, # SH_DIAG_MSG_DATA4U
  0x0000000110062580, # SH_DIAG_MSG_DATA5L
  0x0000000110062600, # SH_DIAG_MSG_DATA5U
  0x0000000110062680, # SH_DIAG_MSG_DATA6L
  0x0000000110062700, # SH_DIAG_MSG_DATA6U
  0x0000000110062780, # SH_DIAG_MSG_DATA7L
  0x0000000110062800, # SH_DIAG_MSG_DATA7U
  0x0000000110062880, # SH_DIAG_MSG_DATA8L
  0x0000000110062900  # SH_DIAG_MSG_DATA8U
);

#-------------------------------------------------------------------------------
#  ERROR REGISTER SPECIFICS - The order of the address array cannot change!
#-------------------------------------------------------------------------------

@shub_ereg_addr =
(  0x0000000110050600, # SH_LB_FIRST_ERROR
   0x0000000110050200, # SH_LB_ERROR_DETAIL_1
   0x0000000110050280, # SH_LB_ERROR_DETAIL_2
   0x0000000110050300, # SH_LB_ERROR_DETAIL_3
   0x0000000110050380, # SH_LB_ERROR_DETAIL_4
   0x0000000110050400, # SH_LB_ERROR_DETAIL_5
   0x0000000110050580, # SH_LB_ERROR_SUMMARY
   0x0000000110050500, # SH_LB_ERROR_OVERFLOW
   0x0000000110050480, # SH_LB_ERROR_MASK
   0x0000000110050588, # SH_LB_ERROR_SUMMARY_ALIAS
   0x0000000110050508, # SH_LB_ERROR_OVERFLOW_ALIAS
);

#-------------------------------------------------------------------------------
#  INTERRUPT ENABLE REGISTER  - The order of the address array cannot change!
#-------------------------------------------------------------------------------

@shub_int_enable_addr =
(  0x0000000110000500, # SH_LOCAL_INT0_ENABLE
   0x0000000110000600, # SH_LOCAL_INT1_ENABLE
   0x0000000110000700, # SH_LOCAL_INT2_ENABLE
   0x0000000110000800, # SH_LOCAL_INT3_ENABLE
   0x0000000110000900, # SH_LOCAL_INT4_ENABLE
   0x0000000110000a00  # SH_LOCAL_INT5_ENABLE
);


#-------------------------------------------------------------------------------
#  SN2NET header commands
#-------------------------------------------------------------------------------

@sn2net_command_list = (
		"SN2NET_RDSHD",
		"SN2NET_RDEXC",
		"SN2NET_READ",
		"SN2NET_GET",
		"SN2NET_GETF",
		"SN2NET_PFCL",
		"SN2NET_RQSH",
		"SN2NET_RQSHR",
		"SN2NET_ININV",
		"SN2NET_ININF",
		"SN2NET_INTER",
		"SN2NET_INEXC",
		"SN2NET_ERASE",
		"SN2NET_FLSH",
		"SN2NET_LINVV",
		"SN2NET_INVAL",
		"SN2NET_NCRD",
		"SN2NET_PRDI",
		"SN2NET_PCRRA",
		"SN2NET_PCRRB",
		"SN2NET_PRIHA",
		"SN2NET_PRIHB",
		"SN2NET_PRIRA",
		"SN2NET_PRIRB",
		"SN2NET_PCRDI",
		"SN2NET_PCRHA",
		"SN2NET_PCRHB",
		"SN2NET_AMOR",
		"SN2NET_RESERVED_1C",
		"SN2NET_BAR",
		"SN2NET_RDXRO",
		"SN2NET_RDXRN",
		"SN2NET_RESERVED_20",
		"SN2NET_RESERVED_21",
		"SN2NET_RESERVED_22",
		"SN2NET_RESERVED_23",
		"SN2NET_RESERVED_24",
		"SN2NET_RDXRX",
		"SN2NET_RESERVED_26",
		"SN2NET_RESERVED_27",
		"SN2NET_AMOW",
		"SN2NET_RESERVED_29",
		"SN2NET_RESERVED_2A",
		"SN2NET_RESERVED_2B",
		"SN2NET_NCWRD",
		"SN2NET_RESERVED_2D",
		"SN2NET_RESERVED_2E",
		"SN2NET_RESERVED_2F",
		"SN2NET_RESERVED_30",
		"SN2NET_PWRI",
		"SN2NET_RESERVED_32",
		"SN2NET_RESERVED_33",
		"SN2NET_PWIHA",
		"SN2NET_PWIHB",
		"SN2NET_PWIRA",
		"SN2NET_PWIRB",
		"SN2NET_GFXW1",
		"SN2NET_RESERVED_39",
		"SN2NET_RESERVED_3A",
		"SN2NET_RESERVED_3B",
		"SN2NET_RESERVED_3C",
		"SN2NET_RESERVED_3D",
		"SN2NET_LINTR",
		"SN2NET_LPTC",
		"SN2NET_WRBK",
		"SN2NET_PUT",
		"SN2NET_WRBKR",
		"SN2NET_IWE",
		"SN2NET_RESERVED_44",
		"SN2NET_RESERVED_45",
		"SN2NET_RESERVED_46",
		"SN2NET_RESERVED_47",
		"SN2NET_RESERVED_48",
		"SN2NET_RESERVED_49",
		"SN2NET_RESERVED_4A",
		"SN2NET_RESERVED_4B",
		"SN2NET_NCWRF",
		"SN2NET_RESERVED_4D",
		"SN2NET_RESERVED_4E",
		"SN2NET_RESERVED_4F",
		"SN2NET_GFXWC",
		"SN2NET_RESERVED_51",
		"SN2NET_GFXW4",
		"SN2NET_GFXW2",
		"SN2NET_RESERVED_54",
		"SN2NET_RESERVED_55",
		"SN2NET_RESERVED_56",
		"SN2NET_RESERVED_57",
		"SN2NET_PWCHA",
		"SN2NET_PWCHB",
		"SN2NET_PWCRA",
		"SN2NET_PWCRB",
		"SN2NET_PWCO",
		"SN2NET_RESERVED_5D",
		"SN2NET_RESERVED_5E",
		"SN2NET_RESERVED_5F",
		"SN2NET_RESERVED_60",
		"SN2NET_VRD",
		"SN2NET_VWR",
		"SN2NET_RESERVED_63",
		"SN2NET_RESERVED_64",
		"SN2NET_RESERVED_65",
		"SN2NET_RESERVED_66",
		"SN2NET_RESERVED_67",
		"SN2NET_MCINVA",
		"SN2NET_MCINVB",
		"SN2NET_RESERVED_6A",
		"SN2NET_RESERVED_6B",
		"SN2NET_RESERVED_6C",
		"SN2NET_RESERVED_6D",
		"SN2NET_RESERVED_6E",
		"SN2NET_RESERVED_6F",
		"SN2NET_PCWHA",
		"SN2NET_PCWHB",
		"SN2NET_PCWIA",
		"SN2NET_PCWIB",
		"SN2NET_PCWRI",
		"SN2NET_RESERVED_75",
		"SN2NET_RESERVED_76",
		"SN2NET_RESERVED_77",
		"SN2NET_BINEV",
		"SN2NET_RESERVED_79",
		"SN2NET_RESERVED_7A",
		"SN2NET_RESERVED_7B",
		"SN2NET_RESERVED_7C",
		"SN2NET_RESERVED_7D",
		"SN2NET_RESERVED_7E",
		"SN2NET_RESERVED_7F",
		"SN2NET_SACK",
		"SN2NET_EACK",
		"SN2NET_IACK",
		"SN2NET_NACKG",
		"SN2NET_WNACK",
		"SN2NET_AERR",
		"SN2NET_PERR",
		"SN2NET_IVACK",
		"SN2NET_WERR",
		"SN2NET_WBACK",
		"SN2NET_WBBAK",
		"SN2NET_NACK",
		"SN2NET_PRGE",
		"SN2NET_PRGER",
		"SN2NET_DNGRD",
		"SN2NET_DNGDR",
		"SN2NET_RESERVED_90",
		"SN2NET_PACKH",
		"SN2NET_PACKN",
		"SN2NET_PNKRA",
		"SN2NET_PNKRB",
		"SN2NET_BINVP",
		"SN2NET_GFXCR",
		"SN2NET_GFXER",
		"SN2NET_WACK",
		"SN2NET_WACKP",
		"SN2NET_IWACK",
		"SN2NET_DERRW",
		"SN2NET_BINVV",
		"SN2NET_DERRR",
		"SN2NET_PRERR",
		"SN2NET_PWERR",
		"SN2NET_RESERVED_A0",
		"SN2NET_RESERVED_A1",
		"SN2NET_RESERVED_A2",
		"SN2NET_RESERVED_A3",
		"SN2NET_RESERVED_A4",
		"SN2NET_RESERVED_A5",
		"SN2NET_RESERVED_A6",
		"SN2NET_RESERVED_A7",
		"SN2NET_RESERVED_A8",
		"SN2NET_RESERVED_A9",
		"SN2NET_NCRP",
		"SN2NET_RESERVED_AB",
		"SN2NET_RESERVED_AC",
		"SN2NET_RESERVED_AD",
		"SN2NET_RESERVED_AE",
		"SN2NET_RESERVED_AF",
		"SN2NET_RESERVED_B0",
		"SN2NET_RESERVED_B1",
		"SN2NET_RESERVED_B2",
		"SN2NET_RESERVED_B3",
		"SN2NET_PRPLY",
		"SN2NET_RESERVED_B5",
		"SN2NET_ARRP",
		"SN2NET_RESERVED_B7",
		"SN2NET_RESERVED_B8",
		"SN2NET_RESERVED_B9",
		"SN2NET_RESERVED_BA",
		"SN2NET_RESERVED_BB",
		"SN2NET_RESERVED_BC",
		"SN2NET_RESERVED_BD",
		"SN2NET_RESERVED_BE",
		"SN2NET_RESERVED_BF",
		"SN2NET_SRESP",
		"SN2NET_SRPLY",
		"SN2NET_RESERVED_C2",
		"SN2NET_RESERVED_C3",
		"SN2NET_ERESP",
		"SN2NET_RESERVED_C5",
		"SN2NET_ERPLY",
		"SN2NET_ESPEC",
		"SN2NET_IRESP",
		"SN2NET_RESERVED_C9",
		"SN2NET_IRPLY",
		"SN2NET_SHWBR",
		"SN2NET_XFER",
		"SN2NET_XFERR",
		"SN2NET_RESERVED_CE",
		"SN2NET_ISPEC",
		"SN2NET_PCRPY",
		"SN2NET_RESERVED_D1",
		"SN2NET_RESERVED_D2",
		"SN2NET_RESERVED_D3",
		"SN2NET_RESERVED_D4",
		"SN2NET_RESERVED_D5",
		"SN2NET_ERPYP",
		"SN2NET_RESERVED_D7",
		"SN2NET_BINTR",
		"SN2NET_BIEXC",
		"SN2NET_RESERVED_DA",
		"SN2NET_RESERVED_DB",
		"SN2NET_RESERVED_DC",
		"SN2NET_RESERVED_DD",
		"SN2NET_BIINF",
		"SN2NET_RESERVED_DF",
		"SN2NET_NCWAK",
		"SN2NET_PCNWA",
		"SN2NET_PCNWB",
		"SN2NET_AWAK",
		"SN2NET_PSDBK",
		"SN2NET_IIACK",
		"SN2NET_PNKWA",
		"SN2NET_PNKWB",
		"SN2NET_RESERVED_E8",
		"SN2NET_VRPLY",
		"SN2NET_VWACK",
		"SN2NET_RESERVED_EB",
		"SN2NET_RESERVED_EC",
		"SN2NET_RESERVED_ED",
		"SN2NET_RESERVED_EE",
		"SN2NET_RESERVED_EF",
		"SN2NET_RESERVED_F0",
		"SN2NET_PCNRA",
		"SN2NET_PCNRB",
		"SN2NET_PCAKN",
		"SN2NET_PCAKH",
		"SN2NET_BIINV",
		"SN2NET_BFLSH",
		"SN2NET_BERSE",
		"SN2NET_RESERVED_F8",
		"SN2NET_VERRA",
		"SN2NET_VERRC",
		"SN2NET_VERAC",
		"SN2NET_RESERVED_FC",
		"SN2NET_RESERVED_FD",
		"SN2NET_PI_CRB_TIMEOUT",
		"SN2NET_RESERVED_FF",
		"SN2NET_NONE"		);


@sn2net_chiplet_list = ( "MD", "LB", "PI", "II" );

@shub2_chiplet_list = ( "MD", "LB", "PI", "BT" );

@tio_chiplet_list = ( "ICE/AEGIS", "LB", "ICE/AEGIS", "ICE/AEGIS" );


		
#-------------------------------------------------------------------------------
#
#  subroutine:  decode_sn2net_header
#
#  arguments should be received in this order:
#
#    $hdr_lower - hex value, bits [63:0] of the header to be decoded
#    $hdr_upper - hex value, bits [63:0] of the header to be decoded
#
#
#  return value:  a string describing the fields in the SN2NET Header
#
#-------------------------------------------------------------------------------

sub decode_sn2net_header  {

    my $hdr_lower;
    my $hdr_upper;

    if ((! defined $_[0]) || (! defined $_[1])) {
      print " ERROR!  Upper & lower header values not provided decode_sn2net_header subroutine.\n";
      print " Returning without reading MD directory state.\n\n";
    }
    else  {
      $hdr_lower = $_[0];
      $hdr_upper = $_[1];
    }

    my $return_string = "";
    my $dir_msg = "";

    my $v_bit 		= $hdr_lower & 1;
    my $age 		= ($hdr_lower >> 1) & 0xFF; 
    my $dir 		= ($hdr_lower >> 9) & 0xF; 
    my $dst_node 	= ($hdr_lower >> 13) & 0x7FF; 
    my $cmd 		= ($hdr_lower >> 24) & 0xFF; 
    my $dst_chiplet	= ($hdr_lower >> 32) & 0x7; 
    my $addr_32_6	= ($hdr_lower >> 35) & 0x7FFFFFF; 

    my $suppl		= $hdr_upper & 0x3FFF;
    my $src_node	= ($hdr_upper >> 16) & 0x7FF;
    my $src_chiplet	= ($hdr_upper >> 27) & 0x7;
    my $addr_5_3	= ($hdr_upper >> 35) & 0x7;
    my $addr_49_33	= ($hdr_upper >> 38) & 0x1FFFF;
    my $echo		= ($hdr_upper >> 55) & 0x1FF;

    my $final_address = ($addr_5_3 << 3) | ($addr_32_6 << 6) | ($addr_49_33 << 33);
    if (($dir & 0x8) > 0)  {
      $dir_msg = "Poisoned Dir Field - Invalid LUT access";
    }

    $return_string = "\n\nThe SN2NET Header values derived from:\n";
    $return_string .= sprintf ("Header bits [127:64]:  0x%016llX   Header bits [63:0]: 0x%016llX\n\n", 
	$hdr_upper, $hdr_lower);
    $return_string .= sprintf("Source Node:\t\t0x%03X\n", $src_node);
    $return_string .= sprintf("Source Chiplet:\t\t0x%X    (%s)\n", $src_chiplet, $sn2net_chiplet_list[$src_chiplet]);
    $return_string .= sprintf("Command:\t\t0x%02X   (%s)\n", $cmd, $sn2net_command_list[$cmd]);
    $return_string .= sprintf("Address:\t\t0x%013llX\n", $final_address);
    $return_string .= sprintf("Supplemental:\t\t0x%04X\n", $suppl);
    $return_string .= sprintf("Echo:\t\t\t0x%03X\n", $echo);
    $return_string .= sprintf("Dest Node:\t\t0x%03X\n", $dst_node);
    $return_string .= sprintf("Dest Chiplet:\t\t0x%X    (%s)\n", $dst_chiplet, $sn2net_chiplet_list[$dst_chiplet]);
    $return_string .= sprintf("V Bit:\t\t\t0x%0X\n", $v_bit);
    $return_string .= sprintf("Age:\t\t\t0x%0X\n", $age);
    $return_string .= sprintf("Dir:\t\t\t0x%0X   %s\n", $dir, $dir_msg);

    $return_string .= "\n\n";

    return($return_string);

}


		
#-------------------------------------------------------------------------------
#
#  subroutine:  decode_shub2_sn2net_header
#
#  arguments should be received in this order:
#
#    $hdr_lower - hex value, bits [63:0] of the header to be decoded
#    $hdr_upper - hex value, bits [63:0] of the header to be decoded
#
#
#  return value:  a string describing the fields in the Shub 2.x format of the 
#                 SN2NET Header
#
#-------------------------------------------------------------------------------

sub decode_shub2_sn2net_header  {

    my $hdr_lower;
    my $hdr_upper;

    if ((! defined $_[0]) || (! defined $_[1])) {
      print " ERROR!  Upper & lower header values not provided decode_shub2_sn2net_header subroutine.\n";
      print " Returning without reading MD directory state.\n\n";
    }
    else  {
      $hdr_lower = $_[0];
      $hdr_upper = $_[1];
    }

    my $return_string = "";
    my $dir_msg = "";

    my $v_bit 		= $hdr_lower & 1;
    my $age 		= ($hdr_lower >> 1) & 0xFF; 
    my $dir 		= ($hdr_lower >> 9) & 0xF; 
    my $dst_node_10_0 	= ($hdr_lower >> 13) & 0x7FF; 
    my $cmd 		= ($hdr_lower >> 24) & 0xFF; 
    my $dst_chiplet	= ($hdr_lower >> 32) & 0x3; 
    my $dst_node_11	= ($hdr_lower >> 34) & 0x1; 
    my $addr_32_6	= ($hdr_lower >> 35) & 0x7FFFFFF; 
    my $dst_node_13_12	= ($hdr_lower >> 62) & 0x3; 

    my $suppl		= $hdr_upper & 0xFFFF;
    my $src_node_10_0	= ($hdr_upper >> 16) & 0x7FF;
    my $src_chiplet	= ($hdr_upper >> 27) & 0x3;
    my $src_node_13_11	= ($hdr_upper >> 29) & 0x7;
    my $addr_51_50	= ($hdr_upper >> 32) & 0x3;
    my $addr_5_3	= ($hdr_upper >> 35) & 0x7;
    my $addr_49_33	= ($hdr_upper >> 38) & 0x1FFFF;
    my $echo		= ($hdr_upper >> 55) & 0x1FF;

    my $final_address = ($addr_5_3 << 3) | ($addr_32_6 << 6) | ($addr_49_33 << 33) | ($addr_51_50 << 50);
    my $src_node = $src_node_10_0 | ($src_node_13_11 << 11); 
    my $dst_node = $dst_node_10_0 | ($dst_node_11 << 11) | ($dst_node_13_12 << 12); 

    if (($dir & 0x8) > 0)  {
      $dir_msg = "Poisoned Dir Field - Invalid LUT access";
    }

    my $dst_node_type = (($dst_node & 1) == 1) ? "TIO/NL4R" : "SHUB";
    my $dst_chiplet_type = (($dst_node & 1) == 1) ? "$tio_chiplet_list[$dst_chiplet]" : "$shub2_chiplet_list[$dst_chiplet]";
    my $src_node_type = (($src_node & 1) == 1) ? "TIO/NL4R" : "SHUB";
    my $src_chiplet_type = (($src_node & 1) == 1) ? "$tio_chiplet_list[$src_chiplet]" : "$shub2_chiplet_list[$src_chiplet]";


    $return_string = "\n\nThe SN2NET Header values derived from:\n";
    $return_string .= sprintf ("Header bits [127:64]:  0x%016llX   Header bits [63:0]: 0x%016llX\n\n", 
	$hdr_upper, $hdr_lower);
    $return_string .= sprintf("Source Node:\t\t0x%04X   (%s node)\n", $src_node, $src_node_type);
    $return_string .= sprintf("Source Chiplet:\t\t0x%X      (%s)\n", $src_chiplet, $src_chiplet_type);
    $return_string .= sprintf("Command:\t\t0x%02X     (%s)\n", $cmd, $sn2net_command_list[$cmd]);
    $return_string .= sprintf("Address:\t\t0x%013llX\n", $final_address);
    $return_string .= sprintf("Supplemental:\t\t0x%04X\n", $suppl);
    $return_string .= sprintf("Echo:\t\t\t0x%03X\n", $echo);
    $return_string .= sprintf("Dest Node:\t\t0x%04X   (%s node)\n", $dst_node, $dst_node_type);
    $return_string .= sprintf("Dest Chiplet:\t\t0x%X      (%s)\n", $dst_chiplet, $dst_chiplet_type);
    $return_string .= sprintf("V Bit:\t\t\t0x%0X\n", $v_bit);
    $return_string .= sprintf("Age:\t\t\t0x%0X\n", $age);
    $return_string .= sprintf("Dir:\t\t\t0x%0X   %s\n", $dir, $dir_msg);

    $return_string .= "\n\n";

    return($return_string);

}

#-------------------------------------------------------------------------------
#
#  subroutine:  decode_lb_status
#
#   @edata should now consist of:
#     [0]     SH_LB_FIRST_ERROR
#     [1]     SH_LB_ERROR_DETAIL_1
#     [2]     SH_LB_ERROR_DETAIL_2, etc
#     [5]     SH_LB_ERROR_DETAIL_5
#
#-------------------------------------------------------------------------------

sub decode_lb_status  
{

   #
   #  Check and extract the parameters
   #
  if ( $#_ != 0 )
  {  $emess = "Invalid parameters passed to subroutine decode_lb_status().";
      dsp_error_and_exit($emess);
      return;
  }
  my $aref = $_[0];
  if ( ref($aref) ne 'ARRAY' )
  {  $emess = "Invalid parameter passed to subroutine decode_lb_status().";
      $emess .= "\nFirst parameter must be an ARRAY reference.";
      dsp_error_and_exit($emess);
      return;
  }
  my @adata = @$aref;
  my $num = $#adata + 1;
  if ( $num  < 8)
  {  $emess = "Invalid number of LB error regs in decode_lb_status().";
     $emess .= "\nExpected:  7   Actual: $num";
      dsp_error_and_exit($emess);
      return;
  }

  my $reg_ptr;

  $reg_ptr = reg_create_mmr_inst("shub_01", 'SH_LB_FIRST_ERROR');
  reg_set_value($reg_ptr, $adata[0]);
  reg_write_value($reg_ptr, 'STDOUT', 'valid');

  $reg_ptr = reg_create_mmr_inst("shub_01", 'SH_LB_ERROR_SUMMARY');
  reg_set_value($reg_ptr, $adata[6]);
  reg_write_value($reg_ptr, 'STDOUT', 'valid');

  for ($lb_detail_num = 1; $lb_detail_num < 6; $lb_detail_num++)  {
    if (! defined($adata[$lb_detail_num])) { 	last; 	}
    my $lb_detail_reg = "SH_LB_ERROR_DETAIL_" . "$lb_detail_num";
    $reg_ptr = reg_create_mmr_inst("shub_01", $lb_detail_reg);
    reg_set_value($reg_ptr, $adata[$lb_detail_num]);
    reg_write_value($reg_ptr, 'STDOUT', 'all');
    if ($lb_detail_num == 2)  {
      my $reply_address = $adata[$lb_detail_num] << 3;
      printf "\n  Actual global address:  0x%012llx\n", $reply_address;
    }
  }

}


#-------------------------------------------------------------------------------
#
#  subroutine:  decode_pi_err_detail
#
#	input parameters:	register name (string)
#				register value (64 bit integer)
#				SH_PI_FIRST_ERROR value
#
#	return value:	string which describes the error detail fields
#	
#
#-------------------------------------------------------------------------------

sub decode_pi_err_detail
{
  my $dtl_reg_name = "$_[0]";
  my $dtl_reg_val  = $_[1];
  my $pi_first_err = $_[2];
  my $format_spec  = 0;
  my $rt_string    = "";

  if (($pi_first_err & 0x8fc00000) > 0) {
    $format_spec = 1;			# AFI format will be used
  }
  elsif (($pi_first_err & 0x7e7f) > 0) {
    $format_spec = 2;			# CRBP format will be used
  }
  elsif (($pi_first_err & 0x400000000) > 0) {
    $format_spec = 3;			# MSG_LEN format will be used
  }

  if (($dtl_reg_name =~ /1/) && ($format_spec == 1)) {		# SH_PI_ERROR_DETAIL_1
    my $A_addr = $dtl_reg_val & 0xfffffffffff8;
    my $REQa   = ($dtl_reg_val >> 48) & 0x3f;
    my $Aa     = ($dtl_reg_val >> 54) & 3;
    my $RS     = ($dtl_reg_val >> 56) & 7;
    my $APa    = ($dtl_reg_val >> 60) & 3;
    my $RPa    = ($dtl_reg_val >> 62) & 1;
    my $RSP    = ($dtl_reg_val >> 63) & 1;
       $A_addr = $A_addr | ($Aa << 48);
    $rt_string = sprintf("\n RSP(%X) RPa:(%X) APa:(%X) RS:(%X) REQa:(0x%02X) A_addr:(0x%013llX)\n",
	$RSP, $RPa, $APa, $RS, $REQa, $A_addr); 
  }

  elsif (($dtl_reg_name =~ /2/) && ($format_spec == 1)) {		# SH_PI_ERROR_DETAIL_2
    my $B_addr = $dtl_reg_val & 0xfffffffffff8;
    my $REQb   = ($dtl_reg_val >> 48) & 0x3f;
    my $Ab     = ($dtl_reg_val >> 54) & 3;
    my $pri_ob = ($dtl_reg_val >> 59) & 1;
    my $APb    = ($dtl_reg_val >> 60) & 3;
    my $RPb    = ($dtl_reg_val >> 62) & 1;
    my $RSP    = ($dtl_reg_val >> 63) & 1;
    $B_addr    = $B_addr | ($Ab << 48);
    my $proc_core = ($B_addr >> 23) & 1;
    my $did =      ($B_addr >> 16) & 0x3ff;
    my $exf = 	   ($B_addr >> 3) & 0x1f;
    my $b_en = 	   ($B_addr >> 8) & 0xff;
    my $attr = 	   ($B_addr >> 32) & 0xf;
    my $scnt = 	   ($B_addr >> 36) & 0xf;
    my $rcnt = 	   ($B_addr >> 40) & 0xf;
    my $attr_type = ($attr == 0) ? 'UC':
		    ($attr == 4) ? 'WC':
		    ($attr == 7) ? 'WB': '??';

    $rt_string = sprintf("\n RSP:(%X) RPb:(%X) APb:(%X) pri_owns_bus:(%X) REQb:(0x%02X) B_addr:(0x%013llX)\n",
	$RSP, $RPb, $APb, $pri_ob, $REQb, $B_addr); 
    $rt_string .= sprintf("\n RCNT:(%X) SCNT:(%X) ATTR:%s(%X) BYTE_ENABLE:(0x%02X) DID:(0x%03X) EXF:(0x%02X)\n",
	$rcnt, $scnt, $attr_type, $attr, $b_en, $did, $exf); 
    $rt_string .= sprintf(" Processor %d is indicated in the DID field.\n", $proc_core);
  }

  elsif (($dtl_reg_name =~ /1/) && ($format_spec == 2)) {		# SH_PI_ERROR_DETAIL_1
    my $addr    = $dtl_reg_val & 0x3fffffffffff8;
    my $tbl_sel = ($dtl_reg_val >> 51) & 0x1f;
    my $cmd     = ($dtl_reg_val >> 56) & 0xff;
    $rt_string = sprintf("\n Address: (0x%013llX)  Table Select: (0x%02X)  Command: (0x%02X  -  %s )\n",
   	$addr, $tbl_sel, $cmd, $sn2net_command_list[$cmd]);
  }

  elsif (($dtl_reg_name =~ /2/) && ($format_spec == 2)) {		# SH_PI_ERROR_DETAIL_2
    my $IsReal   = $dtl_reg_val & 0xf;
    my $RRB_idx  = ($dtl_reg_val >> 4) & 0x1f;
    my $WRB_idx  = ($dtl_reg_val >> 9) & 0x1f;
    my $IRB_idx  = ($dtl_reg_val >> 14) & 0x7;
    my $Ecode    = ($dtl_reg_val >> 17) & 0x1f;
    my $Echo_u   = ($dtl_reg_val >> 22) & 0x3;
    my $Echo_l   = ($dtl_reg_val >> 24) & 0xff;
    my $Source   = ($dtl_reg_val >> 32) & 0x3fff;
    my $Supl_u   = ($dtl_reg_val >> 46) & 0x3;
    my $Supl_l   = ($dtl_reg_val >> 48) & 0xfff;
    my $AXB_q    = ($dtl_reg_val >> 60) & 0xf;

    my $Echo     = $Echo_l | ($Echo_u << 8);
    my $Suppl    = $Supl_l | ($Supl_u << 12);
    my $Src_Node = $Source & 0x7ff;
    my $Src_Chip = ($Source >> 11) & 0x7;
    $rt_string = sprintf("\n Source(0x%04X - Node:0x%03X  Chiplet:%s)  Suppl(0x%04X)  Echo(0x%03X)  Ecode(0x%X)  \n IsReal GIWR(0x%X)  IRB Index(0x%X)  WRB Index(0x%X)  RRB Index(0x%X)  AXB Queue(0x%X)\n",
	$Source, $Src_Node, $sn2net_chiplet_list[$Src_Chip], $Suppl, $Echo, $Ecode, $IsReal, $IRB_idx, $WRB_idx, $RRB_idx, $AXB_q);
  
  }

  elsif (($dtl_reg_name =~ /1/) && ($format_spec == 3)) {	# MSG_LEN decode for SH_PI_ERROR_DETAIL_1
    my $Echo = $dtl_reg_val & 0x3f;
    my $ml_type = (($dtl_reg_val & 0x40) == 0x40) ? 'Obese' : 'Runt';
    my $Valid = ($dtl_reg_val >> 7) & 1;

    $rt_string = sprintf("\n Echo(0x%06X)  Type: %s    Error Detail Valid(0x%X)\n", $Echo, $ml_type, $Valid);
  }

  return ($rt_string);
}


#-------------------------------------------------------------------------------
#
#  subroutine:  decode_sh2_pi_err_detail
#
#	input parameters:	register name (string)
#				register value (64 bit integer)
#				SH2_PI_FIRST_ERROR value
#
#	return value:	string which describes the error detail fields
#	
#
#-------------------------------------------------------------------------------

sub decode_sh2_pi_err_detail
{
  my $dtl_reg_name = "$_[0]";
  my $dtl_reg_val  = $_[1];
  my $pi_first_err = $_[2];
  my $format_spec  = 0;
  my $rt_string    = "";

  my $proc_type = (defined $sh2_target_list[$sh2_cnt]->{AID_DECODE}) ? $sh2_target_list[$sh2_cnt]->{AID_DECODE} : 0;

  if (($pi_first_err & 0xffc000000) > 0) {
    $format_spec = 1;			# AFI format will be used
  }
  elsif (($pi_first_err & 0x13dff) > 0) {
    $format_spec = 2;			# CRBP format will be used
  }
  elsif (($pi_first_err & 0x200) > 0) {
    $format_spec = 3;			# Message Length Format
  }

  if (($dtl_reg_name =~ /DETAIL_1/) && ($format_spec == 1)) {		# SH2_PI_ERROR_DETAIL_1
    my $A_addr = $dtl_reg_val & 0xfffffffffff8;
    my $REQa   = ($dtl_reg_val >> 48) & 0x3f;
    my $Aa     = ($dtl_reg_val >> 54) & 3;
    my $RS     = ($dtl_reg_val >> 56) & 7;
    my $APa    = ($dtl_reg_val >> 60) & 3;
    my $RPa    = ($dtl_reg_val >> 62) & 1;
    my $RSP    = ($dtl_reg_val >> 63) & 1;
       $A_addr = $A_addr | ($Aa << 48);
    $rt_string = sprintf("\n RSP(%X) RPa:(%X) APa:(%X) RS:(%X) REQa:(0x%02X) A_addr:(0x%013llX)\n",
	$RSP, $RPa, $APa, $RS, $REQa, $A_addr); 
  }

  elsif (($dtl_reg_name =~ /DETAIL_2/) && ($format_spec == 1)) {	# SH2_PI_ERROR_DETAIL_2
    my $B_addr = $dtl_reg_val & 0xfffffffffff8;
    my $REQb   = ($dtl_reg_val >> 48) & 0x3f;
    my $Ab     = ($dtl_reg_val >> 54) & 3;
    my $pri_ob = ($dtl_reg_val >> 59) & 1;
    my $APb    = ($dtl_reg_val >> 60) & 3;
    my $RPb    = ($dtl_reg_val >> 62) & 1;
    my $RSP    = ($dtl_reg_val >> 63) & 1;
    $B_addr    = $B_addr | ($Ab << 48);
    my $proc_agent = (($proc_type & 1) == 1) ? (($B_addr >> 22) & 1) : (($B_addr >> 23) & 1);
    my $proc_core =  (($proc_type & 1) == 1) ? (($B_addr >> 23) & 1) : 0 ;
    my $proc_thread = (($proc_type & 1) == 1) ? (($B_addr >> 24) & 1) : 0;
    my $did =      ($B_addr >> 16) & 0x3ff;
    my $exf = 	   ($B_addr >> 3) & 0x1f;
    my $b_en = 	   ($B_addr >> 8) & 0xff;
    my $attr = 	   ($B_addr >> 32) & 0xf;
    my $scnt = 	   ($B_addr >> 36) & 0xf;
    my $rcnt = 	   ($B_addr >> 40) & 0xf;
    my $attr_type = ($attr == 0) ? 'UC':
		    ($attr == 4) ? 'WC':
		    ($attr == 7) ? 'WB': '??';
    $rt_string = sprintf "\n RSP:(%X) RPb:(%X) APb:(%X) pri_owns_bus:(%X) REQb:(0x%02X) B_addr:(0x%013llX)\n",
	$RSP, $RPb, $APb, $pri_ob, $REQb, $B_addr; 
    $rt_string .= sprintf "\n RCNT:(%X) SCNT:(%X) ATTR:%s(%X) BYTE_ENABLE:(0x%02X) DID:(0x%03X) EXF:(0x%02X)\n",
	$rcnt, $scnt, $attr_type, $attr, $b_en, $did, $exf; 
    $rt_string .= sprintf " PROC %d, CORE %d, THREAD %d is indicated in the DID field.\n", $proc_agent, $proc_core, $proc_thread;
  }

  elsif (($dtl_reg_name =~ /DETAIL_1/) && ($format_spec == 2)) {		# SH2_PI_ERROR_DETAIL_1
    my $addr    = $dtl_reg_val & 0x3fffffffffff8;
    my $tbl_sel = ($dtl_reg_val >> 51) & 0x1f;
    my $cmd     = ($dtl_reg_val >> 56) & 0xff;
    $rt_string = sprintf("\n Address: (0x%013llX)  Table Select: (0x%02X)  Command: (0x%02X  -  %s )\n",
   	$addr, $tbl_sel, $cmd, $sn2net_command_list[$cmd]);
  }

  elsif (($dtl_reg_name =~ /DETAIL_2/) && ($format_spec == 2)) {		# SH2_PI_ERROR_DETAIL_2
    my $IsReal   = $dtl_reg_val & 0xf;
    my $RRB_idx  = ($dtl_reg_val >> 4) & 0x1f;
    my $WRB_idx  = ($dtl_reg_val >> 9) & 0x1f;
    my $IRB_idx  = ($dtl_reg_val >> 14) & 0x7;
    my $Ecode    = ($dtl_reg_val >> 17) & 0x1f;
    my $Echo_u   = ($dtl_reg_val >> 22) & 0x3;
    my $Echo_l   = ($dtl_reg_val >> 24) & 0xff;
    my $Source   = ($dtl_reg_val >> 32) & 0xffff;
    my $Supl     = ($dtl_reg_val >> 48) & 0xfff;
    my $AXB_q    = ($dtl_reg_val >> 60) & 0xf;

    my $Echo     = $Echo_l | ($Echo_u << 8);
    my $Src_Node = $Source & 0x3fff;
    my $Src_Chip = ($Source >> 14) & 0x3;
    my $Src_Chip_Type = (($Src_Node & 1) == 1) ? "$tio_chiplet_list[$Src_Chip]" : "$shub2_chiplet_list[$Src_Chip]";
    $rt_string = sprintf("\n Source(0x%04X - Node:0x%04X  Chiplet:%s)  Suppl(0x%04X)  Echo(0x%03X)  Ecode(0x%X)  \n IsReal GIWR(0x%X)  IRB Index(0x%X)  WRB Index(0x%X)  RRB Index(0x%X)  AXB Queue(0x%X)\n",
	$Source, $Src_Node, $Src_Chip_Type, $Suppl, $Echo, $Ecode, $IsReal, $IRB_idx, $WRB_idx, $RRB_idx, $AXB_q);
  
  }

  elsif (($dtl_reg_name =~ /DETAIL_1/) && ($format_spec == 3)) {	# MSG_LEN decode for SH2_PI_ERROR_DETAIL_1
    my $addr    = $dtl_reg_val & 0x3fffffffffff8;
    my $tbl_sel = ($dtl_reg_val >> 51) & 0x1f;
    $rt_string = sprintf "\n Address: (0x%013llX)  Table Select: (0x%02X)\n", $addr, $tbl_sel;
  }

  elsif (($dtl_reg_name =~ /DETAIL_2/) && ($format_spec == 3)) {	# MSG_LEN decode for SH2_PI_ERROR_DETAIL_2
    my $IsReal   = $dtl_reg_val & 0xf;
    my $RRB_idx  = ($dtl_reg_val >> 4) & 0x1f;
    my $WRB_idx  = ($dtl_reg_val >> 9) & 0x1f;
    my $IRB_idx  = ($dtl_reg_val >> 14) & 0x7;
    my $Ecode    = ($dtl_reg_val >> 17) & 0x1f;
    my $Echo_u   = ($dtl_reg_val >> 22) & 0x3;
    my $Echo_l   = ($dtl_reg_val >> 24) & 0xff;
    my $Source   = ($dtl_reg_val >> 32) & 0xffff;
    my $Supl     = ($dtl_reg_val >> 48) & 0xfff;
    my $AXB_q    = ($dtl_reg_val >> 60) & 0xf;

    my $Echo     = $Echo_l & 0x3f;
    my $ml_crb_type = (($Supl & 1) == 1) ? "PRB" : "RRB";
    my $ml_type = (($Source & 3) == 0) ? "Runt packet detected at the Local MD Sic" 
		: (($Source & 3) == 1) ? "Obese packet detected at the Local MD Sic"
		: (($Source & 3) == 2) ? "Runt packet detected at the XN Sic Port"
		: (($Source & 3) == 3) ? "Obese packet detected at the XN Sic Port" : "Unknown Message Length Error";
    $rt_string = sprintf "\n %s Index : 0x%02X    Message Length Error Type: %s\n", $ml_crb_type, $Echo, $ml_type;
  }

  return ($rt_string);
}


#------------------------------------------------------------------------------
#
#    SUBROUTINE: shub_pktgen_get_ids()
#
#    USAGE: @vals = shub_pktgen_get_ids($op, $addr)
#           @vals = shub_pktgen_get_ids($op, $addr, $src_id)
#           @vals = shub_pktgen_get_ids($op, $addr, $src_id, $dst_id)
#           @vals = shub_pktgen_get_ids($op, $addr, $src_id, $dst_id, $dst_cid)
#
#    DESCRIPTION:  This subroutine will determine the source and
#       destination IDs required for a packet generator request header.  The
#       source node ID, source chiplet ID, destination node ID and destination
#       chiplet ID are returned.  If no parameters are given, the chip's 
#       NODE_ID MMR is read, and the NODE_ID is extracted.  If the destination
#       node ID is not provided, then the source ID is used.  If the destination
#       chiplet ID is not provided, then it is determined from the operation
#       type ('mem' or 'mmr') and address.
#
#    RETURN VALUE: Array containing source and destination IDs.
#
#------------------------------------------------------------------------------

sub shub_pktgen_get_ids
{
   my $num = $#_ + 1;

   printf "\nshub_pktgen_get_ids(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( ($num < 2) || ($num > 5) )
   {  $emess = "Invalid parameters passed to subroutine shub_pktgen_get_ids().";
      $ecode = -1;
      return;
   }
   my $op   = $_[0];
   my $addr = $_[1];
   my ($tmp, $src_id, $src_chiplet, $dst_id, $dst_chiplet);

   #--------------------------------------- 
   #  Determine the NODE IDs
   #--------------------------------------- 
   if ( $num == 2 ) # No source or destination: read local ID
   {  $tmp = mmr_read($shub_pktgen_addr[$PKTGEN_IND_ID]);
      return if ( $ecode < 0 );
      $src_id = ($tmp >> 32) & 0x7FF;
      $dst_id = $src_id;
   }
   elsif ( $num == 3 ) # Source ID provided: use for destination also
   {  $src_id = $_[2];
      $dst_id = $src_id;
   }
   else # Both source and destination ID provided
   {  $src_id = $_[2];
      $dst_id = $_[3];
   }

   #--------------------------------------- 
   #  Determine the CHIPLET IDs
   #--------------------------------------- 
   $src_chiplet = $SH_CHIPLET_LB;
   if ( $num == 5 )
   {  $dst_chiplet = $_[4];
   }
   elsif ( $op eq 'mem' )
   {  $dst_chiplet = $SH_CHIPLET_MD;
   }
   else
   {  $dst_chiplet = (($addr & 0x100000000) ? $SH_CHIPLET_LB : $SH_CHIPLET_II);
   }
   return($src_id, $src_chiplet, $dst_id, $dst_chiplet);
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: shub_reset_init()
#
#    USAGE: shub_reset_init()
#           shub_reset_init($mode)
#           shub_reset_init($mode, $top_module)
#
#    DESCRIPTION:  This subroutine will perform the reset and initialization
#       sequence on the previously selected (via mmr_select()) SHUB ASIC.
#
#    WARNING: This subroutine was used during simulator initialization only!
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub shub_reset_init
{
   my $num = $#_ + 1;
   print "\nshub_reset_init(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $module_name  = "shub_pkt_gen";
   my $mode = $INIT_MODE_NONE;
   if ( $num == 1 )
   {  $mode = $_[0];
   }
   elsif ( $num == 2 )
   {  $mode = $_[0];
      $module_name = $_[1];
   }
   elsif ( $num != 0 )
   {  $emess = "Invalid parameters passed to subroutine shub_reset_init().";
      dsp_error_and_exit($emess);
      return;
   }
   printf "reset mode = 0x%02X = %u\n", $mode, $mode  if ($debug_flag & $DEBUG_GENERAL);

   my $hard_rst_reg = "$module_name.hard_reset_reg";
   my $soft_rst_reg = "$module_name.soft_reset_reg";
   my $clk_rst_reg  = "$module_name.clk_reset_reg";
   my @tdi = ();
   my @tdo = ();

   print "\n";
   print "------------------------------------------------------------------\n";
   print "--  INITIALIZING THE SHUB USING JTAG AND SIMULATOR ACCESS\n";
   print "------------------------------------------------------------------\n";

   if ( $mode & $INIT_MODE_JTAG_RESET_HARD )
   {  print "\n->Resetting the TAP controller via TRST...\n";
      jtag_reset($JTAG_RESET_HARD, $JTAG_STATE_RTI);
   }

   if ( $mode & $INIT_MODE_JTAG_RESET_SOFT )
   {  print "\n->Resetting the TAP controller via TMS/TCK...\n";
      jtag_reset($JTAG_RESET_SOFT, $JTAG_STATE_RTI);
   }

   if ( $mode & $INIT_MODE_LOAD_TDR_DEFAULT )
   {  print "\n->Loading the TDR_DEFAULT instruction...\n";
      @tdi = ( $shub_inst_lj{TDR_DEFAULT} );  # left-justified OPCODE
      @tdo = jtag_scan_ir($SHUB_IR_LEN, \@tdi);
   }

   print "\n->Asserting the HARD RESET signal on the simulator...\n  entity=$hard_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $hard_rst_reg, "1'b0");

   print "\n->Asserting the SOFT RESET signal on the simulator...\n  entity=$soft_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $soft_rst_reg, "1'b0");

   print "\n->Asserting the CLK RESET signal on the simulator...\n  entity=$clk_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $clk_rst_reg, "1'b0");

   print "\n->Loading the CONFIG instruction...\n";
   @tdi = ( $shub_inst_lj{CONFIG} );  # left-justified OPCODE
   @tdo = jtag_scan_ir($SHUB_IR_LEN, \@tdi);
   print "\n->Loading the CONFIG data register with hardcoded value...\n";
   @tdi = ( 0x47CC0F03, 0x41000000 );
   @tdo = jtag_scan_dr($shub_dr_len{CONFIG}, \@tdi);

   print "\n->Loading the JCLK_RST instruction...\n";
   @tdi = ( $shub_inst_lj{JCLK_RST} );  # left-justified OPCODE
   @tdo = jtag_scan_ir($SHUB_IR_LEN, \@tdi);

   print "\n->Loading the IDCODE instruction (clear JCLK_RST)...\n";
   @tdi = ( $shub_inst_lj{IDCODE} );  # left-justified OPCODE
   @tdo = jtag_scan_ir($SHUB_IR_LEN, \@tdi);

   print "\n->Deasserting the CLK RESET signal on the simulator...\n  entity=$clk_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $clk_rst_reg, "1'b1");

   print "\n->Loading the IDCODE instruction (clear JCLK_RST)...\n";
   @tdi = ( $shub_inst_lj{IDCODE} );  # left-justified OPCODE
   @tdo = jtag_scan_ir($SHUB_IR_LEN, \@tdi);


   print "\n->Desserting the SOFT RESET signal on the simulator...\n  entity=$soft_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $soft_rst_reg, "1'b1");

   print "\n->Deasserting the HARD RESET signal on the simulator...\n  entity=$hard_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $hard_rst_reg, "1'b1");

   print "\n->Reading the IDCODE data register...\n";
   @tdi = ( 0x00000000 );
   @tdo = jtag_scan_dr($shub_dr_len{IDCODE}, \@tdi);

   print "------------------------------------------------------------------\n";
   print "--  RESET/INITIALIZATION SEQUENCE COMPLETE!\n";
   print "------------------------------------------------------------------\n";
}


#-------------------------------------------------------------------------------

1;

