#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: tio_01_jtdrs.pm,v 1.4 2004/06/23 20:16:50 beighley Exp $
#
#-------------------------------------------------------------------------------
#
#  FILE: tio_01_jtdrs.pm
#
#  NOTE: Definitions of the JTAG test data registers (TDRs) for TIO.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     04/11/03  Karen Beighley  Original source code submitted into tree.
#     12/14/03  Karen Beighley  Corrections to several TDR definitions.
#     12/19/03  Karen Beighley  Added support for: ISCAN_CXSSD
#     12/31/03  Karen Beighley  Corrected CONFIG TDR: swapped bits 10 & 11.
#     06/18/04  Karen Beighley  Modified CLKCNTL TDR for TIO 1.1: one of the
#                                  spare bits (2^69) is now DIV2SYNC.
#
#-------------------------------------------------------------------------------

$tio_01_tdr_defined = 0;

#-------------------------------------------------------------------------------

%tio_01_flds_00000 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%tio_01_tdr_00000 =
(  NAME   => "BYPASS",
   NOTE   => "BYPASS JTAG Register",
   OPNAME => "BYPASS",
   OPCODE => 0x7F,
   LENGTH => 1,
   RESET  => [ 0x0 ],
   FIELDS => \%tio_01_flds_00000
);

#-------------------------------------------------------------------------------

%tio_01_flds_00001 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "VERSION",          => [   0,  28,   4, 0x0000, "Version number"         ],
   "PARTNUMBER",       => [   1,  12,  16, 0x0000, "Partnumber"             ],
   "MANUFACTURER",     => [   2,   1,  11, 0x0000, "Manufacturer Code"      ],
   "LSB",              => [   3,   0,   1, 0x0000, "Mandatory LSB"          ]
);

%tio_01_tdr_00001 =
(  NAME   => "IDCODE",
   NOTE   => "JTAG Device Identification Register",
   OPNAME => "IDCODE",
   OPCODE => 0x03,
   LENGTH => 32,
   RESET  => [ 0x00000000 ],
   FIELDS => \%tio_01_flds_00001
);

#-------------------------------------------------------------------------------

%tio_01_flds_00002 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "RESERVED"              => [   0,  19,   4, 0x0000, "Spare bits" ],
   "JSEL_TP"               => [   1,  17,   2, 0x0000, "Testpoint select 4 and 5" ],
   "B_CLK_SEL"             => [   2,  16,   1, 0x0000, "B clock divider select" ],
   "WARM_FENCE_ENABLE"     => [   3,  15,   1, 0x0000, "Warm fence enable to LB" ],
   "SIM_RESET_N"           => [   4,  14,   1, 0x0000, "Simulator reset to LB" ],
   "CP1_IO_TC"             => [   5,  13,   1, 0x0000, "CP1 IO termination control bit" ],
   "CP0_IO_TC"             => [   6,  12,   1, 0x0000, "CP0 IO termination control bit" ],
   "CP1_IO_MC"             => [   7,  11,   1, 0x0000, "CP1 IO mode control bit" ],
   "CP0_IO_MC"             => [   8,  10,   1, 0x0000, "CP0 IO mode control bit" ],
   "CP1_INT_ARB_EN"        => [   9,   9,   1, 0x0000, "CP1 int arb enable bit" ],
   "CP1_PCIX_EN"           => [  10,   8,   1, 0x0000, "CP1 bus PCIX enable bit" ],
   "CP1_BUS_PCI_SPEED_SEL" => [  11,   5,   3, 0x0000, "CP1 bus PCI speed select" ],
   "CP0_INT_ARB_EN"        => [  12,   4,   1, 0x0000, "CP0 int arb enable bit" ],
   "CP0_PCIX_EN"           => [  13,   3,   1, 0x0000, "CP0 bus PCIX enable bit" ],
   "CP0_BUS_PCI_SPEED_SEL" => [  14,   0,   3, 0x0000, "CP0 bus PCI speed select" ]
);

%tio_01_tdr_00002 =
(  NAME   => "CONFIG",
   NOTE   => "TIO Pre-reset Configuration",
   OPNAME => "CONFIG",
   OPCODE => 0x7E,
   LENGTH => 23,
   RESET  => [ 0x007E10 ],
   FIELDS => \%tio_01_flds_00002
);

#-------------------------------------------------------------------------------

%tio_01_flds_00003 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "WRAP_JTAG_SCAN_ACCESS" => [   0,   4,  10, 0x0000, "Scan access bits for RAM wrappers" ],
   "CIR"                   => [   1,   3,   1, 0x0000, "Circulate TDO to TDI"              ],
   "INV"                   => [   2,   2,   1, 0x0000, "Invert circlated TDO value"        ],
   "NL3_CONFIG"            => [   3,   0,   2, 0x0000, "Config bit per channel (1=NL3)"    ]
);

%tio_01_tdr_00003 =
(  NAME   => "SCAN_CTL",
   NOTE   => "Controls gmoc_mode and scan access",
   OPNAME => "SCAN_CTL",
   OPCODE => 0x05,
   LENGTH => 14,
   RESET  => [ 0x0000 ],
   FIELDS => \%tio_01_flds_00003
);

#-------------------------------------------------------------------------------

%tio_01_flds_00004 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "NI0_CLK_I_TERM_H"    => [   0,   7,   1, 0x0000, "Term value for port ni0_i_clk_h" ],
   "NI0_CLK_I_TERM_L"    => [   1,   6,   1, 0x0000, "Term value for port ni0_i_clk_l" ],
   "NI0_DATA_F_TERM_H"   => [   2,   5,   1, 0x0000, "Term value for ports ni0_data_f_h[19:0], ni0_idr_b_of_h" ],
   "NI0_DATA_F_TERM_L"   => [   3,   4,   1, 0x0000, "Term value for ports ni0_data_f_l[19:0], ni0_idr_b_of_l" ],
   "NI1_CLK_I_TERM_H"    => [   4,   3,   1, 0x0000, "Term value for port ni1_i_clk_h" ],
   "NI1_CLK_I_TERM_L"    => [   5,   2,   1, 0x0000, "Term value for port ni1_i_clk_l" ],
   "NI1_DATA_F_TERM_H"   => [   6,   1,   1, 0x0000, "Term value for ports ni1_data_f_h[19:0], ni1_idr_b_of_h" ],
   "NI1_DATA_F_TERM_L"   => [   7,   0,   1, 0x0000, "Term value for ports ni1_data_f_l[19:0], ni1_idr_b_of_l" ]
);

%tio_01_tdr_00004 =
(  NAME   => "NL3_TERM",
   NOTE   => "NL termination value selection",
   OPNAME => "NL3_TERM",
   OPCODE => 0x09,
   LENGTH => 8,
   RESET  => [ 0xFF ],
   FIELDS => \%tio_01_flds_00004
);

#-------------------------------------------------------------------------------

%tio_01_flds_00005 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "START"               => [   0,  29,   1, 0x0000, "Start PIO operation to addressed MMR" ],
   "WRITE"               => [   1,  28,   1, 0x0000, "Operation is a write (1=write, 0=read)" ],
   "PIO_ADDRESS"         => [   2,   0,  28, 0x0000, "Physical address of MMR to access" ]
);

%tio_01_tdr_00005 =
(  NAME   => "PIO_CTRL",
   NOTE   => "Control Register for PIO",
   OPNAME => "PIO_CTRL",
   OPCODE => 0x0F,
   LENGTH => 30,
   RESET  => [ 0x0000000 ],
   FIELDS => \%tio_01_flds_00005
);

#-------------------------------------------------------------------------------

%tio_01_flds_00006 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ERROR"               => [   0,  65,   1, 0x0000, "Error occured with the PIO operation"   ],
   "VALID"               => [   1,  64,   1, 0x0000, "Read data is valid; operation complete" ],
   "DATA"                => [   2,   0,  64, 0x0000, "Read or write data from/for MMR"        ]
);

%tio_01_tdr_00006 =
(  NAME   => "PIO_DATA",
   NOTE   => "Data Register for PIO",
   OPNAME => "PIO_DATA",
   OPCODE => 0x10,
   LENGTH => 66,
   RESET  => [ 0x0, 0x00000000, 0x00000000 ],
   FIELDS => \%tio_01_flds_00006
);

#-------------------------------------------------------------------------------

%tio_01_flds_00007 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "MABIST_BIST_M3"      => [   0,  12,   1, 0x0000, "Reset active indicator (1=reset active)" ],
   "MABIST_ABDONE"       => [   1,  10,   2, 0x0000, "BIST status (1=complete)"     ],
   "MABIST_SYSPF"        => [   2,   0,  10, 0x0000, "BIST results (pass/fail)" ]
);

%tio_01_tdr_00007 =
(  NAME   => "RUNABIST",
   NOTE   => "Execute RAM BIST on SRAMs",
   OPNAME => "RUNABIST",
   OPCODE => 0x08,
   LENGTH => 13,
   RESET  => [ 0x0000 ],
   FIELDS => \%tio_01_flds_00007
);

#-------------------------------------------------------------------------------

%tio_01_flds_00008 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "CLKCNTL_SPARE"     => [   0,  70,   3, 0x0000, "" ],
   "DIV2SYNC"          => [   1,  69,   1, 0x0000, "" ],
   "OC_LOAD_GMXC_SEQ"  => [   2,  65,   4, 0x0000, "" ],
   "OC_GMXC_SEQ"       => [   3,  45,  20, 0x0000, "" ],
   "CA_LOAD_GMXC_SEQ"  => [   4,  44,   1, 0x0000, "" ],
   "CA_GMXC_SEQ"       => [   5,  24,  20, 0x0000, "" ],
   "SLOW1CLK"          => [   6,  23,   1, 0x0000, "" ],
   "SLOW2CLK"          => [   7,  22,   1, 0x0000, "" ],
   "GMXC_TDR_SEL"      => [   8,  10,  12, 0x0000, "" ],
   "TP"                => [   9,   6,   4, 0x0000, "" ],
   "SEL_CA_CLKSTOP"    => [  10,   5,   1, 0x0000, "" ],
   "SEL_IO_CLKSTOP"    => [  11,   4,   1, 0x0000, "" ],
   "SEL_CP_CLKSTOP"    => [  12,   3,   1, 0x0000, "" ],
   "CA_JTAG_CLKSTOP"   => [  13,   2,   1, 0x0000, "" ],
   "IO_JTAG_CLKSTOP"   => [  14,   1,   1, 0x0000, "" ],
   "CP_JTAG_CLKSTOP"   => [  15,   0,   1, 0x0000, "" ],
);

%tio_01_tdr_00008 =
(  NAME   => "CLKCNTL",
   NOTE   => "JTAG CLKCNTL control Register",
   OPNAME => "CLKCNTL",
   OPCODE => 0x07,
   LENGTH => 73,
   RESET  => [ 0x000, 0x00000000, 0x00000000 ],
   FIELDS => \%tio_01_flds_00008
);

#-------------------------------------------------------------------------------

%tio_01_flds_00009 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "NI0_N_REFIO_VALUE"   => [   0, 105,   9, 0x0000, "description"  ],
   "NI0_P_REFIO_VALUE"   => [   1,  96,   9, 0x0000, "description"  ],
   "NI1_N_REFIO_VALUE"   => [   2,  87,   9, 0x0000, "description"  ],
   "NI1_P_REFIO_VALUE"   => [   3,  78,   9, 0x0000, "description"  ],
   "N_AGP_REFIO_VALUE"   => [   4,  73,   5, 0x0000, "description"  ],
   "P_AGP_REFIO_VALUE"   => [   5,  68,   5, 0x0000, "description"  ],
   "AGP_NCNTDWN"         => [   6,  67,   1, 0x0000, "description"  ],
   "AGP_PCNTDWN"         => [   7,  66,   1, 0x0000, "description"  ],
   "AGP_XFER"            => [   8,  65,   1, 0x0000, "description"  ],
   "AGP_SENSE"           => [   9,  64,   1, 0x0000, "description"  ],
   "AGP_EVAL"            => [  10,  63,   1, 0x0000, "description"  ],
   "AGP_ENABLE"          => [  11,  62,   1, 0x0000, "description"  ],
   "AGP_TESTUPDT"        => [  12,  61,   1, 0x0000, "description"  ],
   "AGP_TDR_SEL"         => [  13,  60,   1, 0x0000, "description"  ],
   "AGP_PNDRIVE"         => [  14,  59,   1, 0x0000, "description"  ],
   "AGP_NOUPDT"          => [  15,  58,   1, 0x0000, "description"  ],
   "AGP_PVTP"            => [  16,  53,   5, 0x0000, "description"  ],
   "AGP_PVTN"            => [  17,  48,   5, 0x0000, "description"  ],
   "NI1_FILTER"          => [  18,  47,   1, 0x0000, "description"  ],
   "NI1_BYPASS"          => [  19,  46,   1, 0x0000, "description"  ],
   "NI1_DISABLE"         => [  20,  45,   1, 0x0000, "description"  ],
   "NI1_RESET"           => [  21,  44,   1, 0x0000, "description"  ],
   "NI1_N_TDR_VALUE"     => [  22,  36,   8, 0x0000, "description"  ],
   "NI1_P_TDR_VALUE"     => [  23,  28,   8, 0x0000, "description"  ],
   "NI1_SEL_TDR_VAL"     => [  24,  27,   1, 0x0000, "description"  ],
   "NI1_N_UPDN"          => [  25,  26,   1, 0x0000, "description"  ],
   "NI1_P_UPDN"          => [  26,  25,   1, 0x0000, "description"  ],
   "NI1_OVRD_UPDN"       => [  27,  24,   1, 0x0000, "description"  ],
   "NI0_FILTER"          => [  28,  23,   1, 0x0000, "description"  ],
   "NI0_BYPASS"          => [  29,  22,   1, 0x0000, "description"  ],
   "NI0_DISABLE"         => [  30,  21,   1, 0x0000, "description"  ],
   "NI0_RESET"           => [  31,  20,   1, 0x0000, "description"  ],
   "NI0_N_TDR_VALUE"     => [  32,  12,   8, 0x0000, "description"  ],
   "NI0_P_TDR_VALUE"     => [  33,   4,   8, 0x0000, "description"  ],
   "NI0_SEL_TDR_VAL"     => [  34,   3,   1, 0x0000, "description"  ],
   "NI0_N_UPDN"          => [  35,   2,   1, 0x0000, "description"  ],
   "NI0_P_UPDN"          => [  36,   1,   1, 0x0000, "description"  ],
   "NI0_OVRD_UPDN"       => [  37,   0,   1, 0x0000, "description"  ]
);

%tio_01_tdr_00009 =
(  NAME   => "ICNTLR",
   NOTE   => "Impedance Controller for APF and GTL I/O",
   OPNAME => "ICNTLR",
   OPCODE => 0x06,
   LENGTH => 114,
   RESET  => [ 0x00000, 0x00000000, 0x40000000, 0x00000000 ],
   FIELDS => \%tio_01_flds_00009
);

#-------------------------------------------------------------------------------

%tio_01_flds_00010 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "DAVAIL_SEL"          => [   0,  17,   2, 0x0000, "Control timing of data available signal" ],
   "FMODE_JTAG_VAL"      => [   1,  16,   1, 0x0000, "JTAG FMODE value"                        ],
   "FMODE_JTAG_EN"       => [   2,  15,   1, 0x0000, "Enable JTAG control of FMODE bit"        ],
   "UPDATE_AFTER_CNT_EN" => [   3,  14,   1, 0x0000, "" ],
   "LLP_TUNING_CMPLT_EN" => [   4,  13,   1, 0x0000, "" ],
   "RMT_UPDATE_EN"       => [   5,  12,   1, 0x0000, "Enable remote fine update (via LLP)"     ],
   "DLINE_MAN_ADJ_EN"    => [   6,  11,   1, 0x0000, "Enable manual adjust mode"               ],
   "DLINE_MAN_HOLD"      => [   7,  10,   1, 0x0000, "Hold current delay value"                ],
   "DLY_CTRL_RESET"      => [   8,   9,   1, 0x0000, "Hold delay controller in reset if set"   ],

   "PFSSD_SYNC_MODE"     => [   9,   8,   1, 0x0000, "PFSSD sync mode"   ],
   "PFSSD_DEBUG_SEL"     => [  10,   6,   2, 0x0000, "PFSSD testpoint select"   ],
   "PFSSR_DEBUG_SEL"     => [  11,   0,   6, 0x0000, "PFSSR testpoint select"   ]
);

%tio_01_tdr_00010 =
(  NAME   => "DLY_CTRL0_NI0",
   NOTE   => "DLY_CTRL0_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI0",
   OPCODE => 0x60,
   LENGTH => 19,
   RESET  => [ 0x00000 ],
   FIELDS => \%tio_01_flds_00010
);

#-------------------------------------------------------------------------------

%tio_01_flds_00011 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%tio_01_tdr_00011 =
(  NAME   => "DLY_CTRL1_NI0",
   NOTE   => "DLY_CTRL1_NI0 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI0",
   OPCODE => 0x61,
   LENGTH => 33,
   RESET  => [ 0x0, 0x00000000 ],
   FIELDS => \%tio_01_flds_00011
);

#-------------------------------------------------------------------------------

%tio_01_flds_00012 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "DAVAIL_SEL"          => [   0,  17,   2, 0x0000, "Control timing of data available signal" ],
   "FMODE_JTAG_VAL"      => [   1,  16,   1, 0x0000, "JTAG FMODE value"                        ],
   "FMODE_JTAG_EN"       => [   2,  15,   1, 0x0000, "Enable JTAG control of FMODE bit"        ],
   "UPDATE_AFTER_CNT_EN" => [   3,  14,   1, 0x0000, "" ],
   "LLP_TUNING_CMPLT_EN" => [   4,  13,   1, 0x0000, "" ],
   "RMT_UPDATE_EN"       => [   5,  12,   1, 0x0000, "Enable remote fine update (via LLP)"     ],
   "DLINE_MAN_ADJ_EN"    => [   6,  11,   1, 0x0000, "Enable manual adjust mode"               ],
   "DLINE_MAN_HOLD"      => [   7,  10,   1, 0x0000, "Hold current delay value"                ],
   "DLY_CTRL_RESET"      => [   8,   9,   1, 0x0000, "Hold delay controller in reset if set"   ],

   "PFSSD_SYNC_MODE"     => [   9,   8,   1, 0x0000, "PFSSD sync mode"   ],
   "PFSSD_DEBUG_SEL"     => [  10,   6,   2, 0x0000, "PFSSD testpoint select"   ],
   "PFSSR_DEBUG_SEL"     => [  11,   0,   6, 0x0000, "PFSSR testpoint select"   ]
);

%tio_01_tdr_00012 =
(  NAME   => "DLY_CTRL0_NI1",
   NOTE   => "DLY_CTRL0_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL0_NI1",
   OPCODE => 0x62,
   LENGTH => 19,
   RESET  => [ 0x00000 ],
   FIELDS => \%tio_01_flds_00012
);

#-------------------------------------------------------------------------------

%tio_01_flds_00013 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "FDE19"             => [   0,  32,   1, 0x0000, "description for FDE19"  ],
   "FCE19"             => [   1,  31,   1, 0x0000, "description for FCE19"  ],
   "DE19"              => [   2,  30,   1, 0x0000, "description for DE19"   ],
   "CE19"              => [   3,  29,   1, 0x0000, "description for CE19"   ],
   "DMAX"              => [   4,  22,   7, 0x0000, "description for DMAX"   ],
   "DMIN"              => [   5,  15,   7, 0x0000, "description for DMIN"   ],
   "DMCOUT"            => [   6,   8,   7, 0x0000, "description for DMCOUT" ],
   "DLINE_AND_DMC_SEL" => [   7,   2,   6, 0x0000, "DLINE_AND_DMC_SEL desc" ],
   "DLINE_SHIFT_OP"    => [   8,   0,   2, 0x0000, "DLINE_SHIFT_OP desc"    ],
);


%tio_01_tdr_00013 =
(  NAME   => "DLY_CTRL1_NI1",
   NOTE   => "DLY_CTRL1_NI1 PFSSR Control",
   OPNAME => "DLY_CTRL1_NI1",
   OPCODE => 0x63,
   LENGTH => 33,
   RESET  => [ 0x0, 0x00000000 ],
   FIELDS => \%tio_01_flds_00013
);

#-------------------------------------------------------------------------------

%tio_01_flds_00014 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "CA_AGP8X_SBASSR"   => [   0,   6,   1, 0x0000, "description" ],
   "CA_AGP8X_ADSSR"    => [   1,   5,   1, 0x0000, "description" ],
   "NI1_PFSSR"         => [   2,   4,   1, 0x0000, "description" ],
   "NI0_PFSSR"         => [   3,   3,   1, 0x0000, "description" ],
   "CA_AGP8X_SSD"      => [   4,   2,   1, 0x0000, "description" ],
   "NI1_PFSSD"         => [   5,   1,   1, 0x0000, "description" ],
   "NI0_PFSSD"         => [   6,   0,   1, 0x0000, "description" ]
);

%tio_01_tdr_00014 =
(  NAME   => "SCAN_CHAN",
   NOTE   => "Internal Scan for SSD/SSR Hardmacros",
   OPNAME => "SCAN_CHAN",
   OPCODE => 0x3C,
   LENGTH => 6,
   RESET  => [ 0x00 ],
   FIELDS => \%tio_01_flds_00014
);

#-------------------------------------------------------------------------------

%tio_01_flds_00015 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ENABLE_USER_IGNORE"  => [   0,  55,   1, 0x0000, "Enable user ignore timer" ],
   "USER_IGNORE_TIME"    => [   1,  15,  40, 0x0000, "Ignore reset interval (if enabled)" ],
   "LINK_RESET_ENABLE"   => [   2,  14,   1, 0x0000, "Enable link reset mode" ],
   "QUEUE_CNT_ENABLE"    => [   3,  13,   1, 0x0000, "Enable queue counters to adjust initial credits" ],
   "CHARGE_UNDERFLOW"    => [   4,  12,   1, 0x0000, "DC component in AC endocde data is too small" ],
   "CHARGE_OVERFLOW"     => [   5,  11,   1, 0x0000, "DC component in AC encoded data is too large" ],
   "FAKED_ALL_SN_ERRORS" => [   6,  10,   1, 0x0000, "Asserted when fake SN counter reaches zero" ],
   "SENT_ALL_SN_ERRORS"  => [   7,   9,   1, 0x0000, "Asserted when SN counter reaches zero" ],
   "SENT_ALL_CB_ERRORS"  => [   8,   8,   1, 0x0000, "Asserted when CB counter reaches zero" ],
   "FAKE_SN_ERROR"       => [   9,   7,   1, 0x0000, "SSR pretends it received a bad SN" ],
   "SEND_SN_ERROR"       => [  10,   6,   1, 0x0000, "Inc. SN counter; SN errors may be forced" ],
   "SEND_CB_ERROR"       => [  11,   5,   1, 0x0000, "Inc. CB counter; CB errors may be forced" ],
   "ENABLE_QUAD"         => [  12,   4,   1, 0x0000, "Enable sample time select" ],
   "ENABLE_RFTU"         => [  13,   3,   1, 0x0000, "Enable fine tune update" ],
   "ENABLE_TUNING"       => [  14,   2,   1, 0x0000, "Enable PFSSR tuning" ],
   "ENABLE_AC"           => [  15,   1,   1, 0x0000, "Enable AC encoding" ],
   "BIT_MODE32"          => [  16,   0,   1, 0x0000, "LLP should use 32 bit mode (0=16b)" ],
);

%tio_01_tdr_00015 =
(  NAME   => "LLP4_NI0",
   NOTE   => "NI0 LLP4 Configuration Data",
   OPNAME => "LLP4_NI0",
   OPCODE => 0x0A,
   LENGTH => 56,
   RESET  => [ 0x0000000, 0x00006011 ],
   FIELDS => \%tio_01_flds_00015
);

#-------------------------------------------------------------------------------

%tio_01_flds_00016 =
#  NAME                       IND  POS  LEN  MODE    DESCRIPTION
#  -------------------        ---  ---  ---  ------  -----------------------
(  "ENABLE_USER_IGNORE"  => [   0,  55,   1, 0x0000, "Enable user ignore timer" ],
   "USER_IGNORE_TIME"    => [   1,  15,  40, 0x0000, "Ignore reset interval (if enabled)" ],
   "LINK_RESET_ENABLE"   => [   2,  14,   1, 0x0000, "Enable link reset mode" ],
   "QUEUE_CNT_ENABLE"    => [   3,  13,   1, 0x0000, "Enable queue counters to adjust initial credits" ],
   "CHARGE_UNDERFLOW"    => [   4,  12,   1, 0x0000, "DC component in AC endocde data is too small" ],
   "CHARGE_OVERFLOW"     => [   5,  11,   1, 0x0000, "DC component in AC encoded data is too large" ],
   "FAKED_ALL_SN_ERRORS" => [   6,  10,   1, 0x0000, "Asserted when fake SN counter reaches zero" ],
   "SENT_ALL_SN_ERRORS"  => [   7,   9,   1, 0x0000, "Asserted when SN counter reaches zero" ],
   "SENT_ALL_CB_ERRORS"  => [   8,   8,   1, 0x0000, "Asserted when CB counter reaches zero" ],
   "FAKE_SN_ERROR"       => [   9,   7,   1, 0x0000, "SSR pretends it received a bad SN" ],
   "SEND_SN_ERROR"       => [  10,   6,   1, 0x0000, "Inc. SN counter; SN errors may be forced" ],
   "SEND_CB_ERROR"       => [  11,   5,   1, 0x0000, "Inc. CB counter; CB errors may be forced" ],
   "ENABLE_QUAD"         => [  12,   4,   1, 0x0000, "Enable sample time select" ],
   "ENABLE_RFTU"         => [  13,   3,   1, 0x0000, "Enable fine tune update" ],
   "ENABLE_TUNING"       => [  14,   2,   1, 0x0000, "Enable PFSSR tuning" ],
   "ENABLE_AC"           => [  15,   1,   1, 0x0000, "Enable AC encoding" ],
   "BIT_MODE32"          => [  16,   0,   1, 0x0000, "LLP should use 32 bit mode (0=16b)" ],
);

%tio_01_tdr_00016 =
(  NAME   => "LLP4_NI1",
   NOTE   => "NI1 LLP4 Configuration Data",
   OPNAME => "LLP4_NI1",
   OPCODE => 0x0B,
   LENGTH => 56,
   RESET  => [ 0x0000000, 0x00006011 ],
   FIELDS => \%tio_01_flds_00016
);

#-------------------------------------------------------------------------------

%tio_01_flds_00017 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%tio_01_tdr_00017 =
(  NAME   => "LLP4TEST_NI0",
   NOTE   => "LLP NI0 Channel BIST",
   OPNAME => "LLP4TEST_NI0",
   OPCODE => 0x1A,
   LENGTH => 242,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%tio_01_flds_00017
);

#-------------------------------------------------------------------------------

%tio_01_flds_00018 =
#  NAME                         IND  POS  LEN  MODE    DESCRIPTION
#  -------------------          ---  ---  ---  ------  -----------------------
(  "CAPTURE_RCV_SBSN"      => [   0, 226,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_CRC"       => [   1, 210,  16, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_UP"   => [   2, 146,  64, 0x0000, "description"            ],
   "CAPTURE_RCV_DATA_LO"   => [   3,  82,  64, 0x0000, "description"            ],
   "CAPTURED_CB_ERR"       => [   4,  81,   1, 0x0000, "description"            ],
   "CAPTURED"              => [   5,  80,   1, 0x0000, "description"            ],
   "WIRE_ERR_OVERFLOW"     => [   6,  79,   1, 0x0000, "description"            ],
   "WIRE_ERR_CNT"          => [   7,  55,  24, 0x0000, "description"            ],
   "ERR_CAPTURE"           => [   8,  54,   1, 0x0000, "description"            ],
   "ARM_CAPTURE"           => [   9,  53,   1, 0x0000, "description"            ],
   "NOISE_MODE"            => [  10,  51,   2, 0x0000, "description"            ],
   "LFSR_MODE"             => [  11,  49,   2, 0x0000, "description"            ],
   "WIRE_SEL"              => [  12,  43,   6, 0x0000, "description"            ],
   "SEND_TEST_MODE"        => [  13,  41,   2, 0x0000, "description"            ],
   "TEST_PATTERN"          => [  14,   1,  40, 0x0000, "description"            ],
   "TEST_MODE"             => [  15,   0,   1, 0x0000, "description"            ]
);

%tio_01_tdr_00018 =
(  NAME   => "LLP4TEST_NI1",
   NOTE   => "LLP NI1 Channel BIST",
   OPNAME => "LLP4TEST_NI1",
   OPCODE => 0x1B,
   LENGTH => 242,
   RESET  => [ 0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x0000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000 ],
   FIELDS => \%tio_01_flds_00018
);

#-------------------------------------------------------------------------------

%tio_01_flds_00019 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%tio_01_tdr_00019 =
(  NAME   => "TDR_DEFAULT",
   NOTE   => "Reset user defined TDR's",
   OPNAME => "TDR_DEFAULT",
   OPCODE => 0x7C,
   LENGTH => 1,
   RESET  => [ 0x0 ],
   FIELDS => \%tio_01_flds_00019
);

#-------------------------------------------------------------------------------

%tio_01_flds_00020 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%tio_01_tdr_00020 =
(  NAME   => "JCLK_RST",
   NOTE   => "Clock Controller Reset Instruction",
   OPNAME => "JCLK_RST",
   OPCODE => 0x7D,
   LENGTH => 1,
   RESET  => [ 0x0 ],
   FIELDS => \%tio_01_flds_00020
);

#-------------------------------------------------------------------------------

%tio_01_flds_00021 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "RSVD"              => [   0,   0,   1, 0x0000, ""                       ],
);

%tio_01_tdr_00021 =
(  NAME   => "HIGHZ",
   NOTE   => "HIGHZ instruction",
   OPNAME => "HIGHZ",
   OPCODE => 0x04,
   LENGTH => 1,
   RESET  => [ 0x0 ],
   FIELDS => \%tio_01_flds_00021
);

#-------------------------------------------------------------------------------

%tio_01_flds_00022 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "NI0_PFSSD"         => [   0,   5,   1, 0x0000, "Enable internal scan chain" ],
   "NI1_PFSSD"         => [   1,   4,   1, 0x0000, "Enable internal scan chain" ],
   "II_NGSSD20"        => [   2,   3,   1, 0x0000, "Enable internal scan chain" ],
   "NI0_PFSSR"         => [   3,   2,   1, 0x0000, "Enable internal scan chain" ],
   "NI1_PFSSR"         => [   4,   1,   1, 0x0000, "Enable internal scan chain" ],
   "II_NGSSR20"        => [   5,   0,   1, 0x0000, "Enable internal scan chain" ]
);

%tio_01_tdr_00022 =
(  NAME   => "SCAN_CHAN",
   NOTE   => "SCAN_CHAN instruction",
   OPNAME => "SCAN_CHAN",
   OPCODE => 0x3C,
   LENGTH => 6,
   RESET  => [ 0x00000000 ],
   FIELDS => \%tio_01_flds_00022
);

#-------------------------------------------------------------------------------

%tio_01_flds_00023 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%tio_01_tdr_00023 =
(  NAME   => "ASIC_L1_DATA",
   NOTE   => "TIO to L1 data register",
   OPNAME => "ASIC_L1_DATA",
   OPCODE => 0x11,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%tio_01_flds_00023
);

#-------------------------------------------------------------------------------

%tio_01_flds_00024 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA"              => [   0,   0,  64, 0x0000, ""                       ],
);

%tio_01_tdr_00024 =
(  NAME   => "L1_ASIC_DATA",
   NOTE   => "L1 to TIO data register",
   OPNAME => "L1_ASIC_DATA",
   OPCODE => 0x12,
   LENGTH => 64,
   RESET  => [ 0x00000000, 0x00000000 ],
   FIELDS => \%tio_01_flds_00024
);

#-------------------------------------------------------------------------------

%tio_01_flds_00025 =
#  NAME                     IND  POS  LEN  MODE    DESCRIPTION
#  -------------------      ---  ---  ---  ------  -----------------------
(  "DATA_256"          => [   0, 256,  18, 0x0000, ""                       ],
   "DATA_224"          => [   1, 224,  32, 0x0000, ""                       ],
   "DATA_192"          => [   2, 192,  32, 0x0000, ""                       ],
   "DATA_160"          => [   3, 160,  32, 0x0000, ""                       ],
   "DATA_128"          => [   4, 128,  32, 0x0000, ""                       ],
   "DATA_096"          => [   5,  96,  32, 0x0000, ""                       ],
   "DATA_064"          => [   6,  64,  32, 0x0000, ""                       ],
   "DATA_032"          => [   7,  32,  32, 0x0000, ""                       ],
   "DATA_000"          => [   8,   0,  32, 0x0000, ""                       ]
);

%tio_01_tdr_00025 =
(  NAME   => "ISCAN_CXSSD",
   NOTE   => "CX SSD latches",
   OPNAME => "ISCAN_CXSSD",
   OPCODE => 0x2A,
   LENGTH => 274,
   RESET  => [                                                    0x00000,
               0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
               0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
             ],
   FIELDS => \%tio_01_flds_00025
);

#-------------------------------------------------------------------------------

#-----------------------------------
#  HASH TABLE: LOOKUP TDR BY OPNAME
#-----------------------------------

%tio_01_tdrs =
(  $tio_01_tdr_00000{OPNAME} => \%tio_01_tdr_00000,
   $tio_01_tdr_00001{OPNAME} => \%tio_01_tdr_00001,
   $tio_01_tdr_00002{OPNAME} => \%tio_01_tdr_00002,
   $tio_01_tdr_00003{OPNAME} => \%tio_01_tdr_00003,
   $tio_01_tdr_00004{OPNAME} => \%tio_01_tdr_00004,
   $tio_01_tdr_00005{OPNAME} => \%tio_01_tdr_00005,
   $tio_01_tdr_00006{OPNAME} => \%tio_01_tdr_00006,
   $tio_01_tdr_00007{OPNAME} => \%tio_01_tdr_00007,
   $tio_01_tdr_00008{OPNAME} => \%tio_01_tdr_00008,
   $tio_01_tdr_00009{OPNAME} => \%tio_01_tdr_00009,
   $tio_01_tdr_00010{OPNAME} => \%tio_01_tdr_00010,
   $tio_01_tdr_00011{OPNAME} => \%tio_01_tdr_00011,
   $tio_01_tdr_00012{OPNAME} => \%tio_01_tdr_00012,
   $tio_01_tdr_00013{OPNAME} => \%tio_01_tdr_00013,
   $tio_01_tdr_00014{OPNAME} => \%tio_01_tdr_00014,
   $tio_01_tdr_00015{OPNAME} => \%tio_01_tdr_00015,
   $tio_01_tdr_00016{OPNAME} => \%tio_01_tdr_00016,
   $tio_01_tdr_00017{OPNAME} => \%tio_01_tdr_00017,
   $tio_01_tdr_00018{OPNAME} => \%tio_01_tdr_00018,
   $tio_01_tdr_00019{OPNAME} => \%tio_01_tdr_00019,
   $tio_01_tdr_00020{OPNAME} => \%tio_01_tdr_00020,
   $tio_01_tdr_00021{OPNAME} => \%tio_01_tdr_00021,
   $tio_01_tdr_00022{OPNAME} => \%tio_01_tdr_00022,
   $tio_01_tdr_00023{OPNAME} => \%tio_01_tdr_00023,
   $tio_01_tdr_00024{OPNAME} => \%tio_01_tdr_00024,
   $tio_01_tdr_00025{OPNAME} => \%tio_01_tdr_00025
);


#-----------------------------------
#  ARRAY: STORE OPNAMES IN "ORDER"
#-----------------------------------

@tio_01_opnames =
(  $tio_01_tdr_00000{OPNAME},
   $tio_01_tdr_00001{OPNAME},
   $tio_01_tdr_00002{OPNAME},
   $tio_01_tdr_00003{OPNAME},
   $tio_01_tdr_00004{OPNAME},
   $tio_01_tdr_00005{OPNAME},
   $tio_01_tdr_00006{OPNAME},
   $tio_01_tdr_00007{OPNAME},
   $tio_01_tdr_00008{OPNAME},
   $tio_01_tdr_00009{OPNAME},
   $tio_01_tdr_00010{OPNAME},
   $tio_01_tdr_00011{OPNAME},
   $tio_01_tdr_00012{OPNAME},
   $tio_01_tdr_00013{OPNAME},
   $tio_01_tdr_00014{OPNAME},
   $tio_01_tdr_00015{OPNAME},
   $tio_01_tdr_00016{OPNAME},
   $tio_01_tdr_00017{OPNAME},
   $tio_01_tdr_00018{OPNAME},
   $tio_01_tdr_00019{OPNAME},
   $tio_01_tdr_00020{OPNAME},
   $tio_01_tdr_00021{OPNAME},
   $tio_01_tdr_00022{OPNAME},
   $tio_01_tdr_00023{OPNAME},
   $tio_01_tdr_00024{OPNAME},
   $tio_01_tdr_00025{OPNAME}
);

#-------------------------------------------------------------------------------

1;
