#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: tio_subs.pm,v 1.8 2006/04/27 20:40:36 beighley Exp $
#
#-------------------------------------------------------------------------------
#
#  FILE: tio_subs.pm
#
#  DESCRIPTION:  This file contains PERL constants and subroutines that are
#     specific to the SN2 TIO ASIC.
#
#  MODIFICATION HISTORY:
#
#     WHEN      WHO             WHAT
#     --------  --------------  -----------------------------------------
#     04/29/03  Karen Beighley  Original source code submitted into tree.
#     ...
#     12/11/03  Karen Beighley  Added subroutine to enable IO permissions.
#     12/16/03  Karen Beighley  Changed default chiplet ID for ICE to 0.
#     01/09/04  Karen Beighley  Added the interrupt enable register information.
#     08/12/04  Karen Beighley  Added MMR definition: TIO_LB_HW_TIME_STAMP
#
#-------------------------------------------------------------------------------

$USE_TIO_SUBS_PM = 1;    # Define to allow test for module inclusion

use multi_chip;


#-------------------------------------------------------------------------------
#  PACKET GENERATOR SPECIFICS - The order of the address array cannot change!
#-------------------------------------------------------------------------------

$TIO_CHIPLET_LB   = 1;
$TIO_CHIPLET_ICE  = 0;  # ICE chiplet ID is programmable!
#$TIO_CHIPLET_ICE  = 3;  # ICE chiplet ID is programmable!

@tio_pktgen_addr =
( 0x0000000090060E80, # TIO_NODE_ID (NODE_ID field @ bit 32, ICE_ID @ bit 59)
  0x0000000090063000, # TIO_DIAG_MSG_CNTRL
  0x0000000090063980, # TIO_DIAG_MSG_HDR0
  0x0000000090063A00, # TIO_DIAG_MSG_HDR1
  0x0000000090063080, # TIO_DIAG_MSG_DATA0L
  0x0000000090063100, # TIO_DIAG_MSG_DATA0U
  0x0000000090063180, # TIO_DIAG_MSG_DATA1L
  0x0000000090063200, # TIO_DIAG_MSG_DATA1U
  0x0000000090063280, # TIO_DIAG_MSG_DATA2L
  0x0000000090063300, # TIO_DIAG_MSG_DATA2U
  0x0000000090063380, # TIO_DIAG_MSG_DATA3L
  0x0000000090063400, # TIO_DIAG_MSG_DATA3U
  0x0000000090063480, # TIO_DIAG_MSG_DATA4L
  0x0000000090063500, # TIO_DIAG_MSG_DATA4U
  0x0000000090063580, # TIO_DIAG_MSG_DATA5L
  0x0000000090063600, # TIO_DIAG_MSG_DATA5U
  0x0000000090063680, # TIO_DIAG_MSG_DATA6L
  0x0000000090063700, # TIO_DIAG_MSG_DATA6U
  0x0000000090063780, # TIO_DIAG_MSG_DATA7L
  0x0000000090063800, # TIO_DIAG_MSG_DATA7U
  0x0000000090063880, # TIO_DIAG_MSG_DATA8L
  0x0000000090063900  # TIO_DIAG_MSG_DATA8U
);

#-------------------------------------------------------------------------------
#  ERROR REGISTER SPECIFICS - The order of the address array cannot change!
#-------------------------------------------------------------------------------

@tio_ereg_addr =
(  0x0000000090050600, # TIO_LB_FIRST_ERROR
   0x0000000090050280, # TIO_LB_ERROR_DETAIL_1
   0x0000000090050300, # TIO_LB_ERROR_DETAIL_2
   0x0000000090050380, # TIO_LB_ERROR_DETAIL_3
   0x0000000090050400, # TIO_LB_ERROR_DETAIL_4
   0x0000000090050400, # TIO_LB_ERROR_DETAIL_5 - DOES NOT EXIST!!! COPY OF _4
   0x0000000090050580, # TIO_LB_ERROR_SUMMARY
   0x0000000090050500, # TIO_LB_ERROR_OVERFLOW
   0x0000000090050480, # TIO_LB_ERROR_MASK
   0x0000000090050588, # TIO_LB_ERROR_SUMMARY_ALIAS
   0x0000000090050508, # TIO_LB_ERROR_OVERFLOW_ALIAS
);


#-------------------------------------------------------------------------------
#  INTERRUPT ENABLE REGISTER  - The order of the address array cannot change!
#-------------------------------------------------------------------------------

@tio_int_enable_addr =
(  0x0000000090000500, # TIO_LOCAL_INT0_ENABLE
   0x0000000090000600, # TIO_LOCAL_INT1_ENABLE
   0x0000000090000700, # TIO_LOCAL_INT2_ENABLE
   0x0000000090000800, # TIO_LOCAL_INT3_ENABLE
   0x0000000090000900, # TIO_LOCAL_INT4_ENABLE
   0x0000000090000a00  # TIO_LOCAL_INT5_ENABLE
);

$TIO_LB_HW_TIME_STAMP_MMR     = 0x0000000090071100;
$TIO_EVENT_OCCURRED_MMR       = 0x0000000090010000;
$TIO_EVENT_OCCURRED_ALIAS_MMR = 0x0000000090010008;

#-------------------------------------------------------------------------------

$TIO_IR_LEN = 7;

$INIT_MODE_NONE             = 0x00;
$INIT_MODE_JTAG_RESET_HARD  = 0x01;
$INIT_MODE_JTAG_RESET_SOFT  = 0x02;
$INIT_MODE_LOAD_TDR_DEFAULT = 0x04;
$INIT_MODE_LLP_CONFIG       = 0x08;


#------------------------------------------------------------------------------
#
#    SUBROUTINE: tio_pktgen_get_ids()
#
#    USAGE: @vals = tio_pktgen_get_ids($op, $addr)
#           @vals = tio_pktgen_get_ids($op, $addr, $src_id)
#           @vals = tio_pktgen_get_ids($op, $addr, $src_id, $dst_id)
#           @vals = tio_pktgen_get_ids($op, $addr, $src_id, $dst_id, $dst_cid)
#
#    DESCRIPTION:  This subroutine will determine the source and
#       destination IDs required for a packet generator request header.  The
#       source node ID, source chiplet ID, destination node ID and destination
#       chiplet ID are returned.  If no parameters are given, the chip's
#       NODE_ID MMR is read, and the NODE_ID is extracted.  If the destination
#       node ID is not provided, then the source ID is used.  If the destination
#       chiplet ID is not provided, then it is determined from the address
#       and NODE_ID MMR (if ICE chiplet and source ID not specified).
#
#    RETURN VALUE: Array containing source and destination IDs.
#
#------------------------------------------------------------------------------

sub tio_pktgen_get_ids
{
   my $num = $#_ + 1;

   printf "\ntio_pktgen_get_ids(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   if ( ($num < 2) || ($num > 5) )
   {  $emess = "Invalid parameters passed to subroutine tio_pktgen_get_ids().";
      $ecode = -1;
      return;
   }
   my $op   = $_[0];
   my $addr = $_[1];
   my ($tmp, $src_id, $src_chiplet, $dst_id, $dst_chiplet, $ice_id);

   #--------------------------------------- 
   #  Determine the NODE IDs
   #--------------------------------------- 
   if ( $num == 2 ) # No source or destination: read local ID
   {  $tmp = mmr_read($tio_pktgen_addr[$PKTGEN_IND_ID]);
      return if ( $ecode < 0 );
      $src_id = ($tmp >> 32) & 0x3FFF;
      $dst_id = $src_id;
      $ice_id = ($tmp >> 59) & 0x3;
   }
   elsif ( $num == 3 ) # Source ID provided: use for destination also
   {  $src_id = $_[2];
      $dst_id = $src_id;
      $ice_id = $TIO_CHIPLET_ICE;
   }
   else # Both source and destination ID provided
   {  $src_id = $_[2];
      $dst_id = $_[3];
      $ice_id = $TIO_CHIPLET_ICE;
   }

   #--------------------------------------- 
   #  Determine the CHIPLET IDs
   #--------------------------------------- 
   $src_chiplet = $TIO_CHIPLET_LB;
   if ( $num == 5 )
   {  $dst_chiplet = $_[4];
   }
   else
   {  $dst_chiplet = (($addr >= 0x80000000) ? $TIO_CHIPLET_LB : $ice_id);
   }
   return($src_id, $src_chiplet, $dst_id, $dst_chiplet);
}

#-------------------------------------------------------------------------------

sub tio_pktgen_setup
{
   printf "\ntio_pktgen_setup()\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my @tio_perm_addr_list =
   (  0x00000000b00088a0, # TIO_ICE_FRZ_EXT_IO_PERM0
      0x00000000b00088a8, # TIO_ICE_FRZ_EXT_IO_PERM1
      0x00000000b00088b0, # TIO_ICE_FRZ_EXT_IO_PERM2
      0x00000000b00088b8  # TIO_ICE_FRZ_EXT_IO_PERM3
   );
   my @tio_perm_data_list =
   (  0xFFFFFFFFFFFFFFFF, # TIO_ICE_FRZ_EXT_IO_PERM0
      0xFFFFFFFFFFFFFFFF, # TIO_ICE_FRZ_EXT_IO_PERM1
      0xFFFFFFFFFFFFFFFF, # TIO_ICE_FRZ_EXT_IO_PERM2
      0xFFFFFFFFFFFFFFFF  # TIO_ICE_FRZ_EXT_IO_PERM3
   );

   mmr_write_group(\@tio_perm_addr_list, \@tio_perm_data_list);
}


#-------------------------------------------------------------------------------

sub tio_enable_permissions
{
   printf "\ntio_enable_permissions()\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my @tio_perm_addr_list =
   (  0x00000000b00088a0, # TIO_ICE_FRZ_EXT_IO_PERM0
      0x00000000b00088a8, # TIO_ICE_FRZ_EXT_IO_PERM1
      0x00000000b00088b0, # TIO_ICE_FRZ_EXT_IO_PERM2
      0x00000000b00088b8  # TIO_ICE_FRZ_EXT_IO_PERM3
   );
   my @tio_perm_data_list =
   (  0xFFFFFFFFFFFFFFFF, # TIO_ICE_FRZ_EXT_IO_PERM0
      0xFFFFFFFFFFFFFFFF, # TIO_ICE_FRZ_EXT_IO_PERM1
      0xFFFFFFFFFFFFFFFF, # TIO_ICE_FRZ_EXT_IO_PERM2
      0xFFFFFFFFFFFFFFFF  # TIO_ICE_FRZ_EXT_IO_PERM3
   );
   my $BRICKS_PER_READ = 8;

   #---------------------------------------
   #  Build list of bricks with a TIO0 chip
   #---------------------------------------
   $list_ref  = multi_chip_build_target_list("t", "MOATB", "TIO0", "logical");
   $num = $#$list_ref + 1;
   return(0) if ( $num == 0 );

   printf "   Writing TIO_ICE_FRZ_EXT_IO_PERMx MMRs on %u %s to enable PIO operations.\n", 
      $num, (($num == 1) ? "TIO" : "TIOs");

   #---------------------------------------
   #  Write all bricks with a TIO0 chip
   #---------------------------------------
   $ind = 0;
   while ( $ind < $num )
   {  multi_chip_write_mmr_list($list_ref,  $ind, $BRICKS_PER_READ, \@tio_perm_addr_list, \@tio_perm_data_list);
      $ind += $BRICKS_PER_READ;
   }

   return($num);
}


#-------------------------------------------------------------------------------
#
#    SUBROUTINE: tio_reset_init()
#
#    USAGE: tio_reset_init()
#           tio_reset_init($mode)
#           tio_reset_init($mode, $top_module)
#
#    DESCRIPTION:  This subroutine will perform the reset and initialization
#       sequence on the previously selected (via mmr_select()) TIO ASIC.
#
#    WARNING: This subroutine was used during simulator initialization only!
#
#    RETURN VALUE: none
#
#-------------------------------------------------------------------------------

sub tio_reset_init
{
   my $num = $#_ + 1;
   print "\ntio_reset_init(@_) : num=$num\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   my $module_name  = "top.tio000_001";
   my $mode = $INIT_MODE_NONE;
   if ( $num == 1 )
   {  $mode = $_[0];
   }
   elsif ( $num == 2 )
   {  $mode = $_[0];
      $module_name = $_[1];
   }
   elsif ( $num != 0 )
   {  $emess = "Invalid parameters passed to subroutine tio_reset_init().";
      dsp_error_and_exit($emess);
      return;
   }
   printf "reset mode = 0x%02X = %u\n", $mode, $mode  if ($debug_flag & $DEBUG_GENERAL);

   my $hard_rst_reg = "$module_name.hard_reset_reg";
   my $soft_rst_reg = "$module_name.soft_reset_reg";
   my $clk_rst_reg  = "$module_name.clk_reset_reg";
   my @tdi = ();
   my @tdo = ();

   print "\n";
   print "------------------------------------------------------------------\n";
   print "--  INITIALIZING THE TIO USING JTAG AND SIMULATOR ACCESS\n";
   print "------------------------------------------------------------------\n";

   if ( $mode & $INIT_MODE_JTAG_RESET_HARD )
   {  print "\n->Resetting the TAP controller via TRST...\n";
      jtag_reset($JTAG_RESET_HARD, $JTAG_STATE_RTI);
   }

   if ( $mode & $INIT_MODE_JTAG_RESET_SOFT )
   {  print "\n->Resetting the TAP controller via TMS/TCK...\n";
      jtag_reset($JTAG_RESET_SOFT, $JTAG_STATE_RTI);
   }

   print "\n->Asserting the CLK RESET signal on the simulator...\n  entity=$clk_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $clk_rst_reg, "1'b1");
   jtag_state_to_rti(1);
   print "\n->Asserting the HARD RESET signal on the simulator...\n  entity=$hard_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $hard_rst_reg, "1'b1");
   jtag_state_to_rti(1);
   print "\n->Asserting the SOFT RESET signal on the simulator...\n  entity=$soft_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $soft_rst_reg, "1'b1");
   jtag_state_to_rti(1);

   if ( $mode & $INIT_MODE_LOAD_TDR_DEFAULT )
   {  print "\n->Loading the TDR_DEFAULT instruction...\n";
      @tdi = ( 0x7C );
      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
   }

   print "\n->Loading the CONFIG instruction...\n";
   @tdi = ( 0x7E );
   @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
   print "\n->Loading the CONFIG data register with hardcoded value: assert SIMRESET\n";
   @tdi = ( 0x010231 );
   @tdo = jtag_scan_dr(23, \@tdi);

   print "\n->Loading the JCLK_RST instruction...\n";
   @tdi = ( 0x7D );
   @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);

   print "\n->Loading the BYPASS instruction (clear JCLK_RST)...\n";
   @tdi = ( 0x7F );
   @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);

#   if ( $mode & $INIT_MODE_LOAD_TDR_DEFAULT )
#   {  print "\n->Loading the TDR_DEFAULT instruction...\n";
#      @tdi = ( 0x7C );
#      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
#   }

   print "\n->Deasserting the CLK RESET signal on the simulator...\n  entity=$clk_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $clk_rst_reg, "1'b0");

   print "\n->Loading the CONFIG instruction...\n";
   @tdi = ( 0x7E );
   @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
   print "\n->Loading the CONFIG data register with hardcoded value: de-assert SIMRESET\n";
   @tdi = ( 0x014231 );
   @tdo = jtag_scan_dr(23, \@tdi);

   if ( $mode & $INIT_MODE_LLP_CONFIG )
   {  print "\n->Loading the NI0 LLP CONFIG instruction...\n";
      @tdi = ( 0x0A );
      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
      print "\n->Loading the NI0 LLP CONFIG data register with hardcoded value...\n";
      @tdi = ( 0x00000000, 0x00000009 );
      @tdo = jtag_scan_dr(56, \@tdi);

      print "\n->Loading the NI1 LLP CONFIG instruction...\n";
      @tdi = ( 0x0B );
      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
      print "\n->Loading the NI1 LLP CONFIG data register with hardcoded value...\n";
      @tdi = ( 0x00000000, 0x00000009 );
      @tdo = jtag_scan_dr(56, \@tdi);

      print "\n->Loading the NI0 DLY_CTRL0 instruction...\n";
      @tdi = ( 0x60 );
      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
      print "\n->Loading the NI0 DLY_CTRL0 data register with hardcoded value...\n";
      @tdi = ( 0x00000000 );
      @tdo = jtag_scan_dr(19, \@tdi);
      print "\n->Loading the NI0 DLY_CTRL1 instruction...\n";
      @tdi = ( 0x61 );
      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
      print "\n->Loading the NI0 DLY_CTRL1 data register with hardcoded value...\n";
      @tdi = ( 0x00000000, 0x00000000 );
      @tdo = jtag_scan_dr(33, \@tdi);

      print "\n->Loading the NI1 DLY_CTRL0 instruction...\n";
      @tdi = ( 0x62 );
      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
      print "\n->Loading the NI1 DLY_CTRL0 data register with hardcoded value...\n";
      @tdi = ( 0x00000000 );
      @tdo = jtag_scan_dr(19, \@tdi);
      print "\n->Loading the NI1 DLY_CTRL1 instruction...\n";
      @tdi = ( 0x63 );
      @tdo = jtag_scan_ir($TIO_IR_LEN, \@tdi);
      print "\n->Loading the NI1 DLY_CTRL1 data register with hardcoded value...\n";
      @tdi = ( 0x00000000, 0x00000000 );
      @tdo = jtag_scan_dr(33, \@tdi);
   }

   print "\n->Desserting the SOFT RESET signal on the simulator...\n  entity=$soft_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $soft_rst_reg, "1'b0");
   jtag_state_to_rti(3);
   print "\n->Deasserting the HARD RESET signal on the simulator...\n  entity=$hard_rst_reg\n";
   jtag_simacc_set($JTAG_SIMACC_VEC, $hard_rst_reg, "1'b0");

   jtag_state_to_rti(10);


   print "------------------------------------------------------------------\n";
   print "--  RESET/INITIALIZATION SEQUENCE COMPLETE!\n";
   print "------------------------------------------------------------------\n";
}

#------------------------------------------------------------------------------

1;
