#!/stand/sysco/tools/bin/perl
#
#==============================================================================
#
#  Unpublished copyright (c) 2006, Silicon Graphics, Inc.
#  THIS IS UNPUBLISHED CONFIDENTIAL AND PROPRIETARY SOURCE CODE OF SGI.
#                                                                      
#  The copyright notice above does  not evidence any actual or intended
#  publication  or  disclosure  of  this source  code,  which  includes
#  information that is confidential  and/or proprietary, and is a trade
#  secret, of  Silicon Graphics, Inc.   ANY REPRODUCTION, MODIFICATION,
#  DISTRIBUTION, PUBLIC  PERFORMANCE, OR  PUBLIC DISPLAY OF  OR THROUGH
#  USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS  WRITTEN CONSENT  OF
#  SILICON GRAPHICS, INC.  IS  STRICTLY PROHIBITED, AND IN VIOLATION OF
#  APPLICABLE  LAWS   AND  INTERNATIONAL  TREATIES.    THE  RECEIPT  OR
#  POSSESSION OF  THIS SOURCE CODE AND/OR RELATED  INFORMATION DOES NOT
#  CONVEY OR IMPLY ANY RIGHTS  TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS
#  CONTENTS,  OR TO  MANUFACTURE, USE,  OR  SELL ANYTHING  THAT IT  MAY
#  DESCRIBE, IN WHOLE OR IN PART.
#
#------------------------------------------------------------------------------
#
# $Id: trace_numalink_route.pm,v 1.8 2007/03/15 20:41:31 dsd Exp $
#
#------------------------------------------------------------------------------
#
#  DESCRIPTION:  NUMAlink route tracing routines.
#
#==============================================================================

use lib "/stand/sysco/lib/sscope";

use common_defs;
use shub_subs;
use shub_01_mmrs;
use shub2_subs;
use shub2_01_mmrs;
use tio_01_mmrs;
use nl4rtr_01_mmrs;
use scancfg;
use mmr_subs;
use sysco_subs;

$trace_numalink_route_ver = $1 if '$Revision: 1.8 $' =~ /(\d+\.\d+)/;

$TARGET_CHASSIS  = 0;
$TARGET_BRICK    = 1;
$TARGET_BOARD    = 2;
$TARGET_CHIP     = 3;
$TARGET_SLOT     = 4;
$TARGET_PORT     = 5;
$TARGET_NEXTDIR  = 6;
$TARGET_LINKTYPE = 7;

$CHASSIS_UNKNOWN  =  0;
$CHASSIS_191010   =  1;
$CHASSIS_1955     =  2;
$CHASSIS_1932     =  3;
$CHASSIS_OPUS_TIO =  4;
$CHASSIS_GEMINI   =  5;
$CHASSIS_VORTIX   =  6;
$CHASSIS_IA       =  7;
$CHASSIS_NL4R     =  8;
$CHASSIS_QDR      =  9;

$PORT_INVALID = 0;
$PORT_A       = 1;
$PORT_B       = 2;
$PORT_C       = 3;
$PORT_D       = 4;
$PORT_E       = 5;
$PORT_F       = 6;
$PORT_G       = 7;
$PORT_H       = 8;

$NI_0 = 0;
$NI_1 = 1;

$RTR_L = 0;
$RTR_R = 1;

@blades_191010 = (
    [ # ni0
        { board => "RTR_L", chip => "RTR0", port => $PORT_F, }, # BLADE0
        { board => "RTR_L", chip => "RTR0", port => $PORT_E, }, # BLADE1
        { board => "RTR_L", chip => "RTR0", port => $PORT_D, }, # BLADE2
        { board => "RTR_L", chip => "RTR0", port => $PORT_C, }, # BLADE3
        { board => "RTR_L", chip => "RTR0", port => $PORT_B, }, # BLADE4
        { board => "RTR_L", chip => "RTR1", port => $PORT_F, }, # BLADE5
        { board => "RTR_L", chip => "RTR1", port => $PORT_E, }, # BLADE6
        { board => "RTR_L", chip => "RTR1", port => $PORT_D, }, # BLADE7
        { board => "RTR_L", chip => "RTR1", port => $PORT_C, }, # BLADE8
        { board => "RTR_L", chip => "RTR1", port => $PORT_B, }, # BLADE9
    ],
    [ # ni1
        { board => "RTR_R", chip => "RTR1", port => $PORT_B, }, # BLADE0
        { board => "RTR_R", chip => "RTR1", port => $PORT_C, }, # BLADE1
        { board => "RTR_R", chip => "RTR1", port => $PORT_D, }, # BLADE2
        { board => "RTR_R", chip => "RTR1", port => $PORT_E, }, # BLADE3
        { board => "RTR_R", chip => "RTR1", port => $PORT_F, }, # BLADE4
        { board => "RTR_R", chip => "RTR0", port => $PORT_B, }, # BLADE5
        { board => "RTR_R", chip => "RTR0", port => $PORT_C, }, # BLADE6
        { board => "RTR_R", chip => "RTR0", port => $PORT_D, }, # BLADE7
        { board => "RTR_R", chip => "RTR0", port => $PORT_E, }, # BLADE8
        { board => "RTR_R", chip => "RTR0", port => $PORT_F, }, # BLADE9
    ],
);
%cables_191010 = (
    L0G => { board => "RTR_L", chip => "RTR0", port => $PORT_G, },
    L2G => { board => "RTR_L", chip => "RTR1", port => $PORT_G, },
    R3G => { board => "RTR_R", chip => "RTR0", port => $PORT_G, },
    R1G => { board => "RTR_R", chip => "RTR1", port => $PORT_G, },
    L1H => { board => "RTR_L", chip => "RTR0", port => $PORT_H, },
    L3H => { board => "RTR_L", chip => "RTR1", port => $PORT_H, },
    R2H => { board => "RTR_R", chip => "RTR0", port => $PORT_H, },
    R0H => { board => "RTR_R", chip => "RTR1", port => $PORT_H, },
);
@blades_1955 = (
    [ # ni0
        { board => "RTR_L", port => $PORT_F, }, # BLADE0
        { board => "RTR_L", port => $PORT_E, }, # BLADE1
        { board => "RTR_L", port => $PORT_D, }, # BLADE2
        { board => "RTR_L", port => $PORT_C, }, # BLADE3
        { board => "RTR_L", port => $PORT_B, }, # BLADE4
    ],
    [ # ni1
        { board => "RTR_R", port => $PORT_B, }, # BLADE0
        { board => "RTR_R", port => $PORT_C, }, # BLADE1
        { board => "RTR_R", port => $PORT_D, }, # BLADE2
        { board => "RTR_R", port => $PORT_E, }, # BLADE3
        { board => "RTR_R", port => $PORT_F, }, # BLADE4
    ],
);
%cables_1955 = (
    LG => { board => "RTR_L", port => $PORT_G, },
    LH => { board => "RTR_L", port => $PORT_H, },
    RG => { board => "RTR_R", port => $PORT_G, },
    RH => { board => "RTR_R", port => $PORT_H, },
);

@cables_1932 = (    # [NIx][BLADEx]
    [ "B0A", "B1A", ], ["B0B", "B1B",],
);
%cables_hash_1932 = (
    B0A => { board => "BLADE0", port => $NI_0, },
    B0B => { board => "BLADE0", port => $NI_1, },
    B1A => { board => "BLADE1", port => $NI_0, },
    B1B => { board => "BLADE1", port => $NI_1, },
);

@nodes_vortix = (
    [ # ni0
        { board => "R_TOP", port => $PORT_D, }, # NODE0
        { board => "R_TOP", port => $PORT_C, }, # NODE1
        { board => "R_BOT", port => $PORT_D, }, # NODE2
        { board => "R_BOT", port => $PORT_C, }, # NODE3
    ],
    [ # ni1
        { board => "R_BOT", port => $PORT_B, }, # NODE0
        { board => "R_BOT", port => $PORT_E, }, # NODE1
        { board => "R_TOP", port => $PORT_B, }, # NODE2
        { board => "R_TOP", port => $PORT_E, }, # NODE3
    ],
);

@cables_NL4R = (
    "invalid", "NIA", "NIB", "NIC", "NID", "NIE", "NIF", "NIG", "NIH",
);

sub trace_numalink_route
{
    $src_nasid             = $_[0];
    $dst_nasid             = $_[1];
    $src_chiplet           = $_[2];    # "pi", "md" or "lb"
    $bool_even_cache_lines = $_[3];
    $bool_request_packets  = $_[4];

    $output_route = "";
    $SHUB2        = identify_if_shub2_type();
    if (!defined $SHUB2) {
        return $output_route;
    }
    if (!$SHUB2) {
        abort("No SHUB2 ASICs found (SHUB1 is not supported)");
        return $output_route;
    }
    $SHUB_VERSION    = $SHUB2 ? "shub2_01" : "shub_01";
    $SHUB_MMR_PREFIX = $SHUB2 ? "SH2_"     : "SH_";
    $TIO_VERSION     = "tio_01";
    $RTR_VERSION     = "nl4rtr_01";
    if ($SHUB2) {
        $SHUB_ID    = reg_create_mmr_inst($SHUB_VERSION, "SH2_NODE_ID");
    } else {
        $SHUB_ID    = reg_create_mmr_inst($SHUB_VERSION, $SHUB_MMR_PREFIX."SHUB_ID");
    }
    $TIO_ID         = reg_create_mmr_inst($TIO_VERSION, "TIO_NODE_ID");
    $R4_PROT_CONFIG = reg_create_mmr_inst($RTR_VERSION, "R4_PROTECTION_CONFIG");


    if (($src_nasid & 1) == 0) {
       $src_node = find_chip_by_nasid("SHUB", $SHUB_ID, $src_nasid); 
    } else {
       $src_node = find_chip_by_nasid("TIO", $TIO_ID, $src_nasid); 
    }
    if (!defined $src_node) {
        return $output_route;
    }

    $hop_nbr = 0;
    $okay = start_packet_from_node($src_node);
    if (!defined $okay) {
        return $output_route;
    }
    $hop_start = $src_node;
    for (;;) {
        print_hop_start($hop_start, $hop_nbr);
        $hop_end = find_hop_end($hop_start);
        if (!defined $hop_end) {
            return $output_route;
        }
        print_hop_end($hop_end);
        if ((($$hop_end[$TARGET_CHIP] =~ /SHUB/) ||
             ($$hop_end[$TARGET_CHIP] =~ /TIO/ )) &&
            ($$hop_end[$TARGET_NEXTDIR] == 4    )) {
            $okay = verify_dest_nasid($hop_end);
            if (!defined $okay) {
                return $output_route;
            }
            last;
        } elsif (($$hop_end[$TARGET_CHIP] =~ /RTR/) &&
                 ($$hop_end[$TARGET_NEXTDIR] == 0 )) {
            last;
        }
        $hop_nbr   += 1;
        $hop_start  = find_hop_start($hop_end);
        if (!defined $hop_start) {
            return $output_route;
        }
    }
    return $output_route;
}

sub find_chip_by_nasid
{
    my $target_chip = $_[0];
    my $node_id_mmr = $_[1];
    my $tgt_nasid   = $_[2];
    my ($chip_name, @chip_names, $module);
    my @data = ();

    #
    #  Search each valid module/brick in the system
    #
    foreach $module ( @module_names ) {
        ($rack,$slot) = split /\D/, $module;
        $mref = $modules{$module};
        @chip_names = scan_module_chips($mref, 0xFFFF, 'logical');
        foreach $chip_name ( @chip_names ) {
            ($brick,$board,$chip) = split /:/, $chip_name;
            if ($chip =~ /$target_chip/) {
                mmr_target($SELECT, $module, $board, $chip);
                reg_mmr_read($node_id_mmr);
                $nasid = reg_get_field_value($node_id_mmr, "NODE_ID");
                if ($nasid == $tgt_nasid) {
                    $data[$TARGET_BRICK] = $module;
                    $data[$TARGET_BOARD] = $board;
                    $data[$TARGET_CHIP]  = $chip;
                    return(\@data);
                }
            }
        }
    }
    abort(sprintf("Nasid %d (0x%x) not found.", $tgt_nasid, $tgt_nasid));
    return;
}

sub identify_if_shub2_type
{
    my ($mref, @chip_names, $chip_name, $brick, $board, $chip, $atype);

    foreach $module ( @module_names ) {
        $mref = $modules{$module};
        @chip_names = scan_module_chips($mref, 0xFFFF, 'logical');
        foreach $chip_name ( @chip_names ) {
            if ($chip_name =~ /SHUB/) {
                ($brick,$board,$chip) = split /:/, $chip_name;
                mmr_target($SELECT, $brick, $board, $chip);
                $atype = mmr_target_list_get_detail(0, $DETAIL_IND_TYPE);
                return ($atype =~ /shub2/);
            }
        }
    }
    abort("No SHUBs found in target hardware.");
    return;
}

sub start_packet_from_node
{
    my $src_module = $_[0][$TARGET_BRICK];
    my $src_board  = $_[0][$TARGET_BOARD];
    my $src_chip   = $_[0][$TARGET_CHIP];
    my ($dir_mask, $ni_mask, $ni_shft);
    my ($cur_gid, $dst_gid);
    my ($lt_idx,  $gt_idx);

    mmr_target($SELECT, $src_module, $src_board, $src_chip);
    $dir_mask = 0x000f;
    $ni_mask  = 0x0020;
    $ni_shft  = 0;
    if (($src_chiplet eq "pi") || ($src_chiplet eq "md")) {
        if ($bool_even_cache_lines == $FALSE) {
            $dir_mask = 0x0f00;
            $ni_mask  = 0x2000;
            $ni_shft  = 8;
        }
        if ($bool_request_packets == $FALSE) {
            $hint_mmr = ($src_chiplet eq "pi") ? $SHUB_MMR_PREFIX."PI_RSP_PLANE_HINT" :
                                                 $SHUB_MMR_PREFIX."MD_RSP_PLANE_HINT";
            $PLANE_HINT = reg_create_mmr_inst($SHUB_VERSION, $hint_mmr);
            reg_mmr_read($PLANE_HINT);
            $invert = reg_get_field_value($PLANE_HINT, "INVERT");
            if ($invert eq "1") {
                $dir_mask ^= 0x2020;
                $ni_mask  ^= 0x0f0f;
                $ni_shft  ^= 8;
            }
        }
    }

    #
    # Ignore the over_ride_table MMRs because they aren't used once the
    # PROM has set up the routes. But if we were checking them then
    # that would be done here.
    #
    if ($SHUB2 || (($src_nasid & 1) == 1)) {
        $cur_gid = ($src_nasid & 0x3f00) >> 8;
        $dst_gid = ($dst_nasid & 0x3f00) >> 8;
        $lt_idx  =  $dst_nasid & 0x00ff;
        $gt_idx  = ($dst_nasid & 0x3fc0) >> 6;
    } else {
        $cur_gid = ($src_nasid & 0x0780) >> 7;
        $dst_gid = ($dst_nasid & 0x0780) >> 7;
        $lt_idx  =  $dst_nasid & 0x007f;
        $gt_idx  = ($dst_nasid & 0x0780) >> 7;
    }
    if ($cur_gid == $dst_gid) {
        ($lut_val, $addr) = get_node_chiplet_lut_entry($TRUE, $lt_idx);
    } else {
        ($lut_val, $addr) = get_node_chiplet_lut_entry($FALSE, $gt_idx);
    }
    if (!defined $lut_val) {
        return;
    }
    $_[0][$TARGET_PORT   ] = ($lut_val &  $ni_mask) >> ($ni_shft + 5);
    $_[0][$TARGET_NEXTDIR] = ($lut_val & $dir_mask) >>  $ni_shft;
    $okay = assert_valid_nextdir($_[0][$TARGET_NEXTDIR], $addr);
    if (!defined $okay) {
        return;
    }
    $_[0][$TARGET_CHASSIS] = get_chassis_type_code($src_module);
    if ($_[0][$TARGET_CHASSIS] == $CHASSIS_UNKNOWN) {
        return;
    }
    return $TRUE;
}

sub get_node_chiplet_lut_entry
{
    my $use_local_lut = $_[0];
    my $lut_idx       = $_[1];
    my ($LUT, $mmr_val, $domain, $prefix, $chip);

    $domain = $use_local_lut ? "LOCAL" : "GLOBAL";
    if (($src_nasid & 1) == 0) {
        $chip = $SHUB_VERSION;
        if ($src_chiplet eq "pi") {
            $prefix = "PI";
        } elsif ($src_chiplet eq "md") {
            $prefix = "MD";
        } else {
            $prefix = "IILB";
        }
        $mmr_name = $SHUB_MMR_PREFIX.$prefix."_".$domain."_TABLE";
    } else {
        $chip     = $TIO_VERSION;
        $mmr_name = "TIO_IILB_".$domain."_TABLE";
    }
    $LUT      = reg_create_mmr_inst($chip, $mmr_name);
    my $addr  = reg_addr($LUT);
    $addr    += $lut_idx * 8;
    $mmr_val  = mmr_read($addr);
    if (lut_valid($mmr_val) == $FALSE) {
        abort(sprintf("Lut entry valid not set at 0x%x; please verify destination nasid.", $addr));
        return;
    }
    return ($mmr_val, $addr);
}

sub get_chassis_type_code
{
    my $cur_module = $_[0];
    my ($rack, $slot);

    ($rack,$slot)  = split /\D/, $cur_module;
    $raw           = "r $rack s $slot brick";
    $brick_cmd_out = sysco_raw_cmd($raw);
    if (index($brick_cmd_out, "type: 191010") > -1) {
        return $CHASSIS_191010;
    } elsif (index($brick_cmd_out, "type: 1955") > -1) {
        return $CHASSIS_1955;
    } elsif (index($brick_cmd_out, "type: 1932") > -1) {
        return $CHASSIS_1932;
    } elsif (index($brick_cmd_out, "type: Opus TIO") > -1) {
        return $CHASSIS_OPUS_TIO;
    } elsif (index($brick_cmd_out, "type: Gemini") > -1) {
        return $CHASSIS_GEMINI;
    } elsif (index($brick_cmd_out, "type: C (Vortix)") > -1) {
        return $CHASSIS_VORTIX;
    } elsif (index($brick_cmd_out, "type: IA") > -1) {
        return $CHASSIS_IA;
    } elsif (index($brick_cmd_out, "type: NL4R") > -1) {
        return $CHASSIS_NL4R;
    } elsif (index($brick_cmd_out, "type: Quad Dense Router") > -1) {
        return $CHASSIS_QDR;
    }
    abort("Unknown or unsupported chassis in:\n $brick_cmd_out.");
    return $CHASSIS_UNKNOWN;
}

sub print_hop_start
{
    my $module    = $_[0][$TARGET_BRICK];
    my $board     = $_[0][$TARGET_BOARD];
    my $chip      = $_[0][$TARGET_CHIP];
    my $exit_port = $_[0][$TARGET_PORT];
    my $hop       = $_[1];
    my $port;

    if (($chip =~ /SHUB/) || ($chip =~ /TIO/)) {
        $port = sprintf("Ni%d", $exit_port);
    } else {
        $port = sprintf("Port %c(%d)", $exit_port + ord("A") - 1, $exit_port);
    }
    $output_route .= sprintf("Hop %2d: %s %-7s %-5s %9s ",
           $hop, $module, $board, $chip, $port);
}

sub print_hop_end
{
    my $module     = $_[0][$TARGET_BRICK];
    my $board      = $_[0][$TARGET_BOARD];
    my $chip       = $_[0][$TARGET_CHIP];
    my $entry_port = $_[0][$TARGET_PORT];
    my $link_type  = $_[0][$TARGET_LINKTYPE];
    my $port;

    if (($chip =~ /SHUB/) || ($chip =~ /TIO/)) {
        $port = sprintf("Ni%d", $entry_port);
    } else {
        $port = sprintf("Port %c(%d)", $entry_port + ord("A") - 1,
                        $entry_port);
    }
    $output_route .= sprintf("%s %-9s %s %-7s %-5s\n", $link_type, $port, $module, $board, $chip);
}

sub find_hop_end
{
    my $exit_chassis = $_[0][$TARGET_CHASSIS];
    my $exit_brick   = $_[0][$TARGET_BRICK  ];
    my $exit_board   = $_[0][$TARGET_BOARD  ];
    my $exit_chip    = $_[0][$TARGET_CHIP   ];
    my $exit_port    = $_[0][$TARGET_PORT   ];
    my @entrypoint   = ();

    @entrypoint[$TARGET_CHASSIS] = $exit_chassis;
    @entrypoint[$TARGET_BRICK  ] = $exit_brick;
    @entrypoint[$TARGET_PORT   ] = $exit_port;
    @entrypoint[$TARGET_BOARD  ] = $exit_board;
    @entrypoint[$TARGET_NEXTDIR] = $_[0][$TARGET_NEXTDIR];
    if ($exit_chassis == $CHASSIS_191010) {
        @entrypoint[$TARGET_LINKTYPE] = "--191010_backplane-->";
        $blade = substr($exit_board, 5, 1);
        if (($exit_chip =~ /SHUB/) || ($exit_chip =~ /TIO/)) {
            @entrypoint[$TARGET_BOARD] = $blades_191010[$exit_port][$blade]{board};
            @entrypoint[$TARGET_CHIP ] = $blades_191010[$exit_port][$blade]{chip };
            @entrypoint[$TARGET_PORT ] = $blades_191010[$exit_port][$blade]{port };
            if (route_on_abacus($exit_brick, $exit_board, $exit_chip, $exit_port, \@entrypoint)) {
                @entrypoint[$TARGET_BOARD] = $_[0][$TARGET_BOARD];
            }
        } elsif ($exit_chip =~ /RTR/) {
            if ($exit_port == $PORT_A) {
                @entrypoint[$TARGET_CHIP    ] = ($_[0][$TARGET_CHIP] eq "RTR0") ? "RTR1" : "RTR0";
                @entrypoint[$TARGET_LINKTYPE] = "------rtr_pcb------->";
            } elsif (($exit_port == $PORT_G) || ($exit_port == $PORT_H)) {
                my @ext_ports = keys %cables_191010;
                foreach $tag (@ext_ports) {
                    if (($cables_191010{$tag}{board} eq $exit_board) &&
                        ($cables_191010{$tag}{chip } eq $exit_chip ) &&
                        ($cables_191010{$tag}{port } == $exit_port )) {
                        $exit_tag = $tag;
                        last;
                    }
                }
                $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
                if (!defined $okay) {
                    return;
                }
            } else {
                FIND9: for ($entry_port = $NI_0; $entry_port <= $NI_1; $entry_port++) {
                    for ($blade = 0; $blade <= 9; $blade++) {
                        if (($blades_191010[$entry_port][$blade]{port } == $exit_port ) &&
                            ($blades_191010[$entry_port][$blade]{chip } eq $exit_chip ) &&
                            ($blades_191010[$entry_port][$blade]{board} eq $exit_board)) {
                            @entrypoint[$TARGET_BOARD] = sprintf("BLADE%d", $blade);
                            @entrypoint[$TARGET_PORT ] = $entry_port;
                            @entrypoint[$TARGET_CHIP ] = get_blade_chip(@entrypoint[$TARGET_BRICK],
                                                                        @entrypoint[$TARGET_BOARD],
                                                                        @entrypoint[$TARGET_PORT]);
                            if (!defined @entrypoint[$TARGET_CHIP]) {
                                return;
                            }
                            last FIND9;
                        }
                    }
                }
            }
        } else {
            abort("Bad hop end chip: $exit_chip.");
            return;
        }
    } elsif ($exit_chassis == $CHASSIS_1955) {
        @entrypoint[$TARGET_LINKTYPE] = "---1955_backplane--->";
        $blade = substr($exit_board, 5, 1);
        if (($exit_chip =~ /SHUB/) || ($exit_chip =~ /TIO/)) {
            @entrypoint[$TARGET_BOARD] = $blades_1955[$exit_port][$blade]{board};
            @entrypoint[$TARGET_CHIP ] = "RTR0";
            @entrypoint[$TARGET_PORT ] = $blades_1955[$exit_port][$blade]{port };
            if (route_on_abacus($exit_brick, $exit_board, $exit_chip, $exit_port, \@entrypoint)) {
                @entrypoint[$TARGET_BOARD] = $_[0][$TARGET_BOARD];
            }
        } elsif ($exit_chip =~ /RTR/) {
            $rtr = ($exit_board eq "RTR_L") ? $RTR_L : $ RTR_R;
            if (($exit_port == $PORT_G) || ($exit_port == $PORT_H)) {
                $exit_tag = substr($exit_board, 4, 1).(($exit_port == $PORT_G) ? "G" : "H");
                $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
                if (!defined $okay) {
                    return;
                }
            } else {
                FIND5: for ($entry_port = $NI_0; $entry_port <= $NI_1; $entry_port++) {
                    for ($blade = 0; $blade <= 5; $blade++) {
                        if (($blades_1955[$entry_port][$blade]{port } == $exit_port ) &&
                            ($blades_1955[$entry_port][$blade]{board} eq $exit_board)) {
                            @entrypoint[$TARGET_BOARD] = sprintf("BLADE%d", $blade);
                            @entrypoint[$TARGET_PORT ] = $entry_port;
                            @entrypoint[$TARGET_CHIP ] = get_blade_chip(@entrypoint[$TARGET_BRICK],
                                                                        @entrypoint[$TARGET_BOARD],
                                                                        @entrypoint[$TARGET_PORT ]);
                            if (!defined @entrypoint[$TARGET_CHIP]) {
                                return;
                            }
                            last FIND5;
                        }
                    }
                }
            }
        } else {
            abort("Bad hop end chip: $exit_chip.");
            return;
        }
    } elsif ($exit_chassis == $CHASSIS_1932) {
        if (route_on_abacus($exit_brick, $exit_board, $exit_chip, $exit_port, \@entrypoint) == $FALSE) {
            $blade    = substr($exit_board, 5, 1);
            $exit_tag = $cables_1932[$exit_port][$blade];
            $okay     = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
            if (!defined $okay) {
                return;
            }
        }
    } elsif ($exit_chassis == $CHASSIS_OPUS_TIO) {
        @entrypoint[$TARGET_LINKTYPE] = "------connector----->";
        if ($exit_chip =~ /SHUB/) {
            if ($exit_port == $NI_0) {
                $exit_tag = "NIA";
                $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
                if (!defined $okay) {
                    return;
                }
            } else {
                @entrypoint[$TARGET_BOARD] = "RISER";
                @entrypoint[$TARGET_CHIP ] = "TIO0";
                @entrypoint[$TARGET_PORT ] = $NI_0;
            }
        } elsif ($exit_chip =~ /TIO/) {
            if ($exit_port == $NI_1) {
                $exit_tag = "NIB";
                $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
                if (!defined $okay) {
                    return;
                }
            } else {
                @entrypoint[$TARGET_BOARD] = "NODE";
                @entrypoint[$TARGET_CHIP ] = "SHUB0";
                @entrypoint[$TARGET_PORT ] = $NI_1;
            }
        } else {
            abort("Bad hop end chip: $exit_chip.");
            return;
        }
    } elsif ($exit_chassis == $CHASSIS_GEMINI) {
        @entrypoint[$TARGET_BOARD]    = "RISER";
        @entrypoint[$TARGET_LINKTYPE] = "------connector----->";
        if ($exit_chip =~ /SHUB/) {
            if ($exit_port == $NI_0) {
                $exit_tag = "NIA";
                $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
                if (!defined $okay) {
                    return;
                }
            } else {
                @entrypoint[$TARGET_CHIP ] = "TIO0";
                @entrypoint[$TARGET_PORT ] = $NI_1;
            }
        } elsif ($exit_chip eq "TIO0") {
            if ($exit_port == $NI_0) {
                @entrypoint[$TARGET_CHIP    ] = "TIO1";
                @entrypoint[$TARGET_PORT    ] = $NI_0;
                @entrypoint[$TARGET_LINKTYPE] = "------riser_pcb----->";
            } else {
                @entrypoint[$TARGET_BOARD] = "NODE";
                @entrypoint[$TARGET_CHIP ] = "SHUB0";
                @entrypoint[$TARGET_PORT ] = $NI_1;
            }
        } elsif ($exit_chip eq "TIO1") {
            if ($exit_port == $NI_1) {
                $exit_tag = "NIB";
                $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
                if (!defined $okay) {
                    return;
                }
            } else {
                @entrypoint[$TARGET_CHIP    ] = "TIO0";
                @entrypoint[$TARGET_PORT    ] = $NI_0;
                @entrypoint[$TARGET_LINKTYPE] = "------riser_pcb----->";
            }
        } else {
            abort("Bad hop end chip: $exit_chip.");
            return;
        }
    } elsif ($exit_chassis == $CHASSIS_VORTIX) {
        @entrypoint[$TARGET_LINKTYPE] = "--vortix_backplane-->";
        if ($exit_chip eq "SHUB0") {
            $node = substr($exit_board, 4, 1);
            @entrypoint[$TARGET_BOARD] = $nodes_vortix[$exit_port][$node]{board};
            @entrypoint[$TARGET_PORT ] = $nodes_vortix[$exit_port][$node]{port };
            @entrypoint[$TARGET_CHIP ] = "RTR";
        } elsif ($exit_chip eq "RTR") {
            if (($exit_port >= $PORT_B) && ($exit_port <= $PORT_E)) {
                FINDv: for ($entry_port = $NI_0; $entry_port <= $NI_1; $entry_port++) {
                    for ($node = 0; $node <= 3; $node++) {
                        if (($nodes_vortix[$entry_port][$node]{port } == $exit_port) &&
                            ($nodes_vortix[$entry_port][$node]{board} eq $exit_board)) {
                            @entrypoint[$TARGET_BRICK] = $_[0][$TARGET_BRICK];
                            @entrypoint[$TARGET_BOARD] = sprintf("NODE%d", $node);
                            @entrypoint[$TARGET_CHIP ] = "SHUB0";
                            @entrypoint[$TARGET_PORT ] = $entry_port;
                            last FINDv;
                        }
                    }
                }
            } else {
                $exit_tag = sprintf("%s-%c",
                                    ($exit_board eq "R_TOP") ? "U" : "L",
                                    $exit_port + ord("A") - 1);
                $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
                if (!defined $okay) {
                    return;
                }
            }
        } else {
            abort("Bad hop end chip: $exit_chip.");
            return;
        }
    } elsif ($exit_chassis == $CHASSIS_IA) {
        if ($exit_port == $NI_1) {
            $exit_tag = ($exit_board eq "RISER_L") ? "NIA" : "NIB";
            $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
            if (!defined $okay) {
                return;
            }
        } else {
            @entrypoint[$TARGET_BOARD   ] = ($exit_board eq "RISER_L") ?
                                                "RISER_R" : "RISER_L";
            @entrypoint[$TARGET_CHIP    ] = "TIO0";
            @entrypoint[$TARGET_PORT    ] = $NI_0;
            @entrypoint[$TARGET_LINKTYPE] = "------connector----->";
        }
    } elsif ($exit_chassis == $CHASSIS_NL4R) {
        $exit_tag = $cables_NL4R[$exit_port];
        $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
        if (!defined $okay) {
            return;
        }
    } elsif ($exit_chassis == $CHASSIS_QDR) {
        $exit_tag = sprintf("N%s%c", substr($exit_board, 6, 1),
                            $exit_port + ord("A") - 1);
        $okay = parse_cfgp($exit_tag, $exit_brick, \@entrypoint);
        if (!defined $okay) {
            return;
        }
    } else {
        abort("Bad exit_chassis code: $exit_chassis.");
        return;
    }
    return \@entrypoint;
}

sub find_hop_start
{
    my $entry_module  = $_[0][$TARGET_BRICK  ];
    my $entry_board   = $_[0][$TARGET_BOARD  ];
    my $entry_chip    = $_[0][$TARGET_CHIP   ];
    my $entry_port    = $_[0][$TARGET_PORT   ];
    my $entry_nextdir = $_[0][$TARGET_NEXTDIR];
    my $entry_chassis = $_[0][$TARGET_CHASSIS];
    my @exitpoint     = ();
    my ($lut_idx, $mmr_name, $addr);
    
    $exitpoint[$TARGET_BRICK  ] = $entry_module;
    $exitpoint[$TARGET_BOARD  ] = $entry_board;
    $exitpoint[$TARGET_CHIP   ] = $entry_chip;
    $exitpoint[$TARGET_CHASSIS] = $entry_chassis;
    if (($entry_chip =~ /SHUB/) || ($entry_chip =~ /TIO/)) {
        if ($entry_nextdir != 1) {
            abort("Bad $entry_chip next dir: $entry_nextdir.");
            return;
        }
        $exitpoint[$TARGET_PORT] = $entry_port ^ 1;
        mmr_target($SELECT, $entry_module, $entry_board, $entry_chip);
        if ($entry_chip =~ /SHUB/) {
            reg_mmr_read($SHUB_ID);
            $node_nasid = reg_get_field_value($SHUB_ID, "NODE_ID");
            $chip       = $SHUB_VERSION;
            if (shub_global_id($node_nasid) == shub_global_id($dst_nasid)) {
                $mmr_name = sprintf("%sNI%d_LOCAL_TABLE",
                                    $SHUB_MMR_PREFIX, $entry_port);
                $lut_idx  = shub_local_dest_idx($dst_nasid);
            } else {
                $mmr_name = sprintf("%sNI%d_GLOBAL_TABLE",
                                    $SHUB_MMR_PREFIX, $entry_port);
                $lut_idx  = shub_global_dest_idx($dst_nasid);
            }
        } else {
            reg_mmr_read($TIO_ID);
            $node_nasid = reg_get_field_value($TIO_ID, "NODE_ID");
            $chip       = $TIO_VERSION;
            if (tio_global_id($node_nasid) == tio_global_id($dst_nasid)) {
                $mmr_name = sprintf("TIO_NI%d_LOCAL_TABLE", $entry_port);
                $lut_idx  = tio_local_dest_idx($dst_nasid);
            } else {
                $mmr_name = sprintf("TIO_NI%d_GLOBAL_TABLE", $entry_port);
                $lut_idx  = tio_global_dest_idx($dst_nasid);
            }
        }
        $LUT      = reg_create_mmr_inst($chip, $mmr_name);
        $addr     = reg_addr($LUT);
        $addr    += $lut_idx * 8;
        $mmr_val  = mmr_read($addr);
        if (lut_valid($mmr_val) == $FALSE) {
            abort(sprintf("Lut entry valid not set at 0x%x.", $addr));
            return;
        }
        $exitpoint[$TARGET_NEXTDIR] = $mmr_val & 0xf;
        $okay = assert_valid_nextdir($exitpoint[$TARGET_NEXTDIR], $addr);
        if (!defined $okay) {
            return;
        }
    } elsif ($entry_chip =~ /RTR/) {
        $exitpoint[$TARGET_PORT] = $entry_port + $entry_nextdir;
        if ($exitpoint[$TARGET_PORT] > $PORT_H) {
            $exitpoint[$TARGET_PORT] -= $PORT_H;
        }
        mmr_target($SELECT, $entry_module, $entry_board, $entry_chip);
        $mmr_name  = sprintf("R4_PORT%c_NEIGHBOR_TABLE", $entry_port + ord("A") - 1);
        $LUT       = reg_create_mmr_inst($RTR_VERSION, $mmr_name);
        my $addr   = reg_addr($LUT);
        $addr     += $entry_nextdir * 8;
        $mmr_val   = mmr_read($addr);
        if (lut_valid($mmr_val)) {
            if ($dst_nasid == reg_get_field_value($LUT, "NEIGHBOR_ID")) {
                $exitpoint[$TARGET_NEXTDIR] = 0;
                $okay = assert_valid_nextdir($exitpoint[$TARGET_NEXTDIR], $addr);
                if (!defined $okay) {
                    return;
                }
                return \@exitpoint;
            }
        }
        $mmr_val   = reg_mmr_read($R4_PROT_CONFIG);
        $rtr_nasid = reg_get_field_value($R4_PROT_CONFIG, "NODE_ID");
        if (r4_global_id($rtr_nasid) != r4_global_id($dst_nasid)) {
            $field_name  = sprintf("PORT%c_FORCE_LOCAL_LUT", $entry_port + ord("A") - 1);
            $force_local = reg_get_field_value($R4_PROT_CONFIG, $field_name);
            if ($force_local == 0) {
                $mmr_name  = sprintf("R4_PORT%c_GLOBAL_TABLE", $entry_port + ord("A") - 1);
                $LUT       = reg_create_mmr_inst($RTR_VERSION, $mmr_name);
                my $addr   = reg_addr($LUT);
                $addr     += r4_global_dest_idx($dst_nasid) * 8;
                $mmr_val   = mmr_read($addr);
                if (lut_valid($mmr_val) == $FALSE) {
                    abort(sprintf("Lut entry valid not set at 0x%x\n", $addr));
                    return;
                }
                $exitpoint[$TARGET_NEXTDIR] = $mmr_val & 0x1f;
                $okay = assert_valid_nextdir($exitpoint[$TARGET_NEXTDIR], $addr);
                if (!defined $okay) {
                    return;
                }
                return \@exitpoint;
            }
        }
        $mmr_name  = sprintf("R4_PORT%c_LOCAL_TABLE", $entry_port + ord("A") - 1);
        $LUT       = reg_create_mmr_inst($RTR_VERSION, $mmr_name);
        my $addr   = reg_addr($LUT);
        $addr     += r4_local_dest_idx($dst_nasid) * 8;
        $mmr_val   = mmr_read($addr);
        if (lut_valid($mmr_val) == $FALSE) {
            abort(sprintf("Lut entry valid not set at 0x%x\n", $addr));
            return;
        }
        $exitpoint[$TARGET_NEXTDIR] = $mmr_val & 0x1f;
        $okay = assert_valid_nextdir($exitpoint[$TARGET_NEXTDIR], $addr);
        if (!defined $okay) {
            return;
        }
    } else {
        abort("Bad chip: $entry_chip.");
        return;
    }
    return \@exitpoint;
}

sub lut_valid
{
    my $lut_val = $_[0];

    if (($lut_val & 0x8000000000000000) == 0) {
        return $FALSE;
    }
    return $TRUE;
}

sub r4_global_id
{
    my $nasid = $_[0];

    my $global_id = ($nasid & 0x3e00) >> 9;
    return $global_id;
}

sub r4_local_dest_idx
{
    my $nasid = $_[0];

    my $local_dest_idx = $nasid & 0x01ff;
    return $local_dest_idx;
}

sub r4_global_dest_idx
{
    my $nasid = $_[0];

    my $global_dest_idx = ($nasid & 0x3fc0) >> 6;
    return $global_dest_idx;
}

sub shub_global_id
{
    my $nasid = $_[0];

    if ($SHUB2) {
        return tio_global_id($nasid);
    }
    my $global_id = ($nasid & 0x780) >> 7;
    return $global_id;
}

sub tio_global_id
{
    my $nasid = $_[0];

    my $global_id = ($nasid & 0x3f00) >> 8;
    return $global_id;
}

sub shub_local_dest_idx
{
    my $nasid = $_[0];

    if ($SHUB2) {
        return tio_local_dest_idx($nasid);
    }
    my $local_dest_idx = $nasid & 0x007f;
    return $local_dest_idx;
}

sub tio_local_dest_idx
{
    my $nasid = $_[0];

    my $local_dest_idx = $nasid & 0x00ff;
    return $local_dest_idx;
}

sub shub_global_dest_idx
{
    my $nasid = $_[0];

    if ($SHUB2) {
        return tio_global_dest_idx($nasid);
    }
    my $global_dest_idx = ($nasid & 0x0780) >> 7;
    return $global_dest_idx;
}

sub tio_global_dest_idx
{
    my $nasid = $_[0];

    my $global_dest_idx = ($nasid & 0x3fc0) >> 6;
    return $global_dest_idx;
}

sub verify_dest_nasid
{
    my $dest_module = $_[0][$TARGET_BRICK];
    my $dest_board  = $_[0][$TARGET_BOARD];
    my $dest_chip   = $_[0][$TARGET_CHIP];

    mmr_target($SELECT, $dest_module, $dest_board, $dest_chip);
    if ($dest_chip =~ /SHUB/) {
        reg_mmr_read($SHUB_ID);
        $nasid = reg_get_field_value($SHUB_ID, "NODE_ID");
    } elsif ($dest_chip =~ /TIO/) {
        reg_mmr_read($TIO_ID);
        $nasid = reg_get_field_value($TIO_ID, "NODE_ID");
    } else {
        abort("Unknown dest chip: $dest_module $dest_board $dest_chip.");
        return;
    }
    if ($nasid != $dst_nasid) {
        $output_route .= sprintf("ERROR: Endpoint nasid %d (0x%x) was not expected %d (0x%x).\n",
                                 $nasid, $nasid, $dst_nasid, $dst_nasid);
        $output_route .= "ROUTE TRACE is probably incorrect.\n";
    }
    return $TRUE;
}

sub parse_cfgp
{
    my $exit_port_tag = $_[0];
    my $exit_brick    = $_[1];

    my ($rack,$slot) = split /\D/, $exit_brick;
    my $raw          = "r $rack s $slot cfg p";
    my $cfgp_out     = sysco_raw_cmd($raw);

    if ($cfgp_out =~ /:[0-9]+\s+$exit_port_tag\s+(\w+)\s+(\w+)/) {
        my $module         = $1;
        my $entry_port_tag = $2;
        if ($module eq "not") {
            abort("$exit_port_tag not connected in cfg p text.");
            return;
        }
        my $entry_chassis = get_chassis_type_code($module);
        if ($entry_chassis == $CHASSIS_UNKNOWN) {
            return;
        }
        $_[2][$TARGET_BRICK]    = $module;
        $_[2][$TARGET_CHASSIS]  = $entry_chassis;
        $_[2][$TARGET_LINKTYPE] = "-------cable-------->";

        if ($entry_chassis == $CHASSIS_191010) {
            $_[2][$TARGET_BOARD] = $cables_191010{$entry_port_tag}{board};
            $_[2][$TARGET_CHIP ] = $cables_191010{$entry_port_tag}{chip };
            $_[2][$TARGET_PORT ] = $cables_191010{$entry_port_tag}{port };
            if ($_[2][$TARGET_PORT ] == 0) {
                abort("Cannot match $entry_port_tag from cfg p.");
                return;
            }
        } elsif ($entry_chassis == $CHASSIS_1955) {
            $_[2][$TARGET_BOARD] = $cables_1955{$entry_port_tag}{board};
            $_[2][$TARGET_PORT ] = $cables_1955{$entry_port_tag}{port };
            $_[2][$TARGET_CHIP ] = "RTR0";
            if ($_[2][$TARGET_PORT ] == 0) {
                abort("Cannot match $entry_port_tag from cfg p.");
                return;
            }
        } elsif ($entry_chassis == $CHASSIS_1932) {
            $_[2][$TARGET_BOARD] = $cables_hash_1932{$entry_port_tag}{board};
            $_[2][$TARGET_PORT ] = $cables_hash_1932{$entry_port_tag}{port };
            $_[2][$TARGET_CHIP ] = get_blade_chip($_[2][$TARGET_BRICK],
                                                  $_[2][$TARGET_BOARD],
                                                  $_[2][$TARGET_PORT ]);
            if (!defined $_[2][$TARGET_CHIP]) {
                return;
            }
        } elsif ($entry_chassis == $CHASSIS_OPUS_TIO) {
            if ($entry_port_tag eq "NIA") {
                $_[2][$TARGET_BOARD] = "NODE";
                $_[2][$TARGET_CHIP ] = "SHUB0";
                $_[2][$TARGET_PORT ] = $NI_0;
            } elsif ($entry_port_tag eq "NIB") {
                $_[2][$TARGET_BOARD] = "RISER";
                $_[2][$TARGET_CHIP ] = "TIO0";
                $_[2][$TARGET_PORT ] = $NI_1;
            } else {
                abort("Cannot match $entry_port_tag from cfg p.");
                return;
            }
        } elsif ($entry_chassis == $CHASSIS_GEMINI) {
            if ($entry_port_tag eq "NIA") {
                $_[2][$TARGET_BOARD] = "NODE";
                $_[2][$TARGET_CHIP ] = "SHUB0";
                $_[2][$TARGET_PORT ] = $NI_0;
            } elsif ($entry_port_tag eq "NIB") {
                $_[2][$TARGET_BOARD] = "RISER";
                $_[2][$TARGET_CHIP ] = "TIO1";
                $_[2][$TARGET_PORT ] = $NI_1;
            } else {
                abort("Cannot match $entry_port_tag from cfg p.");
                return;
            }
        } elsif ($entry_chassis == $CHASSIS_VORTIX) {
            $_[2][$TARGET_BOARD] = (substr($entry_port_tag, 0, 1) eq "U") ?
                                   "R_TOP" : "R_BOT";
            $_[2][$TARGET_CHIP ] = "RTR";
            $_[2][$TARGET_PORT ] = ord(substr($entry_port_tag, 2, 1)) - ord("A") + 1;
        } elsif ($entry_chassis == $CHASSIS_IA) {
            $_[2][$TARGET_CHIP ] = "TIO0";
            $_[2][$TARGET_PORT ] = $NI_1;
            $_[2][$TARGET_BOARD] = ($entry_port_tag eq "NIA") ? "RISER_L" :
                                                                "RISER_R";
        } elsif ($entry_chassis == $CHASSIS_NL4R) {
            $_[2][$TARGET_BOARD] = "ROUTER";
            $_[2][$TARGET_CHIP ] = "RTR0";
            for (my $port_nbr = $PORT_A; $port_nbr <= $PORT_H; $port_nbr++) {
                if ($cables_NL4R[$port_nbr] eq $entry_port_tag) {
                    $_[2][$TARGET_PORT] = $port_nbr;
                    last;
                }
            }
        } elsif ($entry_chassis == $CHASSIS_QDR) {
            $_[2][$TARGET_BOARD] = "ROUTER".substr($entry_port_tag, 1, 1);
            $_[2][$TARGET_CHIP ] = "RTR0";
            $_[2][$TARGET_PORT ] = ord(substr($entry_port_tag, 2, 1)) - ord("A") + 1;
        } else {
            abort("Bad entry_chassis code: $entry_chassis.");
            return;
        }
    } else {
        abort("Cannot match $exit_port_tag in cfg p.");
        return;
    }
    return $TRUE;
}

sub get_blade_chip
{
    my $module   = $_[0];
    my $board    = $_[1];
    my $port     = $_[2];

    $mref = $modules{$module};
    ($cnum, $cpos) = scan_module_find_chip($mref, $board, "SHUB0", 'logical');
    if ($cnum >= 0) {
        return "SHUB0";
    }
    ($cnum, $cpos) = scan_module_find_chip($mref, $board, "TIO1", 'logical');
    if ($cnum >= 0) {
        if ($port == $NI_0) {
            return "TIO1";
        }
        return "TIO0";
    }
    ($cnum, $cpos) = scan_module_find_chip($mref, $board, "TIO0", 'logical');
    if ($cnum >= 0) {
        return "TIO0";
    }
    abort("Cannot determine chip on $module $board.");
    return;
}

sub abacus_blade
{
    my $brick = $_[0];
    my $board = $_[1];

    my $mref = $modules{$brick};
    my ($cnum, $cpos) = scan_module_find_chip($mref, $board, "TIO1", 'logical');
    if ($cnum >= 0) {
        return $TRUE;
    }
    return $FALSE;
}

sub route_on_abacus
{
    my $exit_brick = $_[0];
    my $exit_board = $_[1];
    my $exit_chip  = $_[2];
    my $exit_port  = $_[3];

    if (($exit_chip =~ /TIO/) &&
        abacus_blade($exit_brick, $exit_board) &&
        ((($exit_chip eq "TIO0") && ($exit_port == $NI_0)) ||
         (($exit_chip eq "TIO1") && ($exit_port == $NI_1)))) {
        if ($exit_chip eq "TIO0") {
            $_[4][$TARGET_CHIP] = "TIO1";
            $_[4][$TARGET_PORT] = $NI_1;
        } else {
            $_[4][$TARGET_CHIP] = "TIO0";
            $_[4][$TARGET_PORT] = $NI_0;
        }
        $_[4][$TARGET_LINKTYPE] = "--abacus_blade_pcb-->";
        return $TRUE;
    }
    return $FALSE;
}

sub assert_valid_nextdir
{
    my $nextdir = $_[0];
    my $addr    = $_[1];
    if ($nextdir > 7) {
        abort(sprintf("Bad next dir of $nextdir from 0x%x\n", $addr));
        return;
    }
    return $TRUE;
}

sub abort
{
    $output_route .= "ERROR:$_[0]\n";
    $output_route .= "ROUTE TRACE ABORTED\n";
}
