#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: vec_subs.pm,v 1.2 2004/04/20 20:19:05 beighley Exp $



$BITS_LONG  = 32;
$BYTES_LONG = 4;
$NIBS_LONG  = 8;

$default_vec_just = 'right'; # default data justification ('left' or 'right')

#-------------------------------------------------------------------------------

sub vec_flip_bits
{  my $num = $#_ + 1;
#   printf "\nvec_flip_bits( @_ )\n";

   $src_sbit = $_[0];  # Starting bit position in source data
   $nbits    = $_[1];  # Number of bits to flip
   $src_ref  = $_[2];  # Reference to source data (ARRAY)
   return if ( ref($src_ref) ne 'ARRAY' );
   my ($half, $pos, $new_pos, $loc0, $mask0, $loc1, $mask1, $tmp);

   $half = $src_sbit + ($nbits >> 1);

   for ($pos=$src_sbit, $new_pos=($src_sbit+$nbits-1); $pos < $half; $pos++, $new_pos--)
   {
      $loc0  = $pos / $BITS_LONG;
      $mask0 = 0x80000000 >> ( $pos % $BITS_LONG );
      $loc1  = $new_pos / $BITS_LONG;
      $mask1 = 0x80000000 >> ( $new_pos % $BITS_LONG );

      $tmp = $$src_ref[$loc0] & $mask0;

      if ( $$src_ref[$loc1] & $mask1 )
      {  $$src_ref[$loc0] |= $mask0;
      }
      else
      {  $$src_ref[$loc0] &= ~$mask0;
      }

      if ( $tmp )
      {  $$src_ref[$loc1] |= $mask1;
      }
      else
      {  $$src_ref[$loc1] &= ~$mask1;
      }
   }

}


#-------------------------------------------------------------------------------

sub vec_cpy_bits
{  my $num = $#_ + 1;
#   printf "\nvec_cpy_bits( @_ )\n";

   my $just = $default_vec_just;
   if ( $num == 6 )
   {  $just = $_[5];   # Data justification: 'left' or 'right' 
   }
   elsif ( $num != 5 )
   {  return(0);
   }
   my ($src_sbit, $src_ref, $dst_sbit, $dst_ref, $nbits);
   $src_sbit = $_[0];  # Starting bit position in source data
   $src_ref  = $_[1];  # Reference to source data (ARRAY)
   $dst_sbit = $_[2];  # Starting bit position in destination data
   $dst_ref  = $_[3];  # Reference to destination data (ARRAY)
   $nbits    = $_[4];  # Number of bits to copy

   return if ( ref($src_ref) ne 'ARRAY' );
   return if ( ref($dst_ref) ne 'ARRAY' );

   #  Adjust bit positions if right-justified
   if ( $just eq 'right' )
   {  $max   = ($#$src_ref + 1) * $BITS_LONG;
      $src_sbit = $max - $src_sbit - $nbits;
      $max   = ($#$dst_ref + 1) * $BITS_LONG;
      $dst_sbit = $max - $dst_sbit - $nbits;
   }

   my ($src_loc, $dst_loc, $src_index, $dst_index, $src_bits, $dst_bits,
       $max_bits, $bits_left, $mask, $tmp);

   #  Determine starting array entry and number of bits in the entry
   $src_index = int($src_sbit / $BITS_LONG);
   $src_loc   = $src_sbit % $BITS_LONG;
   $dst_index = int($dst_sbit / $BITS_LONG);
   $dst_loc   = $dst_sbit % $BITS_LONG;
   $bits_left = $nbits;

   #  Copy each block from the source entry to the destination entry
   while ( $bits_left > 0 )
   {  $dst_bits = $BITS_LONG - $dst_loc;
      $src_bits = $BITS_LONG - $src_loc;
      $max_bits = ( ($dst_bits <= $bits_left) ? $dst_bits : $bits_left );
      $max_bits = ( ($max_bits <= $src_bits) ? $max_bits : $src_bits );

      $mask = (0xFFFFFFFF << ($BITS_LONG - $max_bits)) & 0xFFFFFFFF;
      $tmp  = ( $mask & ($$src_ref[$src_index] << $src_loc) ) & 0xFFFFFFFF;
      $$dst_ref[$dst_index] &= ~($mask >> $dst_loc);
      $$dst_ref[$dst_index] |= ($tmp >> $dst_loc);
      $$dst_ref[$dst_index] &= 0xFFFFFFFF;

      $bits_left -= $max_bits;
      return if ( $bits_left == 0 );

      $dst_loc += $max_bits;
      if ( ($dst_loc % $BITS_LONG) == 0 )
      {  $dst_index++;
         $dst_loc = 0;
      }
      $src_loc += $max_bits;
      if ( ($src_loc % $BITS_LONG) == 0 )
      {  $src_index++;
         $src_loc = 0;
      }
   }
}


#-------------------------------------------------------------------------------

sub vec_dump_hex
{
#   printf "\nvec_dump_hex( @_ )\n";
   my $title = $_[0];  # Title/description of data
   my $aref  = $_[1];  # Reference to source data (ARRAY)
   my ($n, $i, $q, $r, $b);
   $n = $#$aref + 1;   # Number of words (32-bit value)
   $q = int($n / 8);   # Number of blocks (8 words per block)
   $r = $n % 8;        # Number of remainder words

   printf "\nVECTOR DATA = $title\n";
   for ($i = 0, $b = 0; $i < $q; $i++, $b+=8)
   {  printf "%08lX %08lX %08lX %08lX %08lX %08lX %08lX %08lX\n",
         $$aref[$b+0], $$aref[$b+1], $$aref[$b+2], $$aref[$b+3], 
         $$aref[$b+4], $$aref[$b+5], $$aref[$b+6], $$aref[$b+7] ;
   }
   return if ( $r <= 0 );
   for ($i = 0; $i < $r; $i++)
   {  printf "%08lX ", $$aref[$b+$i];
   }
   printf "\n";
}


#-------------------------------------------------------------------------------

sub vec_compute_nlong
{  return(0) if ( $#_ != 0 );
   my $len = $_[0];
   my $tmp = $len >> 5;        # number of 32-bit blocks (divide by 32)
   $tmp++ if ( $len & 0x1F );  # increment if not on a 32-bit boundary
   return($tmp);
}


#-------------------------------------------------------------------------------

sub vec_convert_num2str_hex
{  my $num = $#_ + 1;
#   print "\nvec_convert_num2str_hex( @_ )\n";
   my $str = "";        # Text string containing data
   my $just = $default_vec_just;
   if ( $num == 3 )
   {  $just = $_[2];    # Data justification: 'left' or 'right'
   }
   elsif ( $num != 2 )
   {  return("");
   }
   my $len  = $_[0];    # Bit length of data to convert
   my $dref = $_[1];    # Reference to array containing data

   my ($val, $text, $nch, $tmp, $pad);
   foreach $val ( @$dref )
   {  $text = sprintf("%08lX", $val);
      $str .= $text;
   }
   # Determine number of characters needed for length
   $nch = (($len & 0x3) ? ($len >> 2) + 1 : ($len >> 2));
   $tmp = length($str); # actual length of string
   if ( $tmp < $nch )
   {  $pad = '0' x ($nch - $tmp);
      $str = (( $just eq 'left' ) ? $str . $pad : $pad . $str );
   }
   elsif ( $tmp > $nch )
   {  $str = (( $just eq 'left' ) ? substr($str, 0, $nch) : 
                                    substr($str, ($tmp - $nch)));
   }
   $str = "0x" . $str;
   return( $str ); 
}


#-------------------------------------------------------------------------------

sub vec_convert_hex_str2num
{  my $num = $#_ + 1;
   my @data = ();       # Array containing converted data

#   print "\nvec_convert_hex_str2num( @_ )\n";

   my $just = $default_vec_just;
   if ( $num == 3 )
   {  $just = $_[2];    # Data justification: 'left' or 'right'
   }
   elsif ( $num != 2 )
   {  return(\@data);
   }
   my $len  = $_[0];    # Bit length of data to convert
   my $str  = $_[1];    # Text string containing data

   #
   #  Strip out any white space and adjust string to boundary
   #
   $str =~ s/\s+//g;
   $str =~ s/^x//g;
   $str =~ s/^0x//g;
  
   my ($nch, $tmp, $pad, $i, $val); 
   $num = vec_compute_nlong($len);
   $nch = $num * $NIBS_LONG;
   $tmp = length($str);
   if ( $tmp < $nch )
   {  $pad = '0' x ($nch - $tmp);
      $str = (( $just eq 'left' ) ? $str . $pad : $pad . $str );
   }
   elsif ( $tmp > $nch )
   {  $str = (( $just eq 'left' ) ? substr($str, 0, $nch) : 
                                    substr($str, ($tmp - $nch)));
   }

   #
   #  Process each character in the string (convert to value)
   #
   for ($i = 0, $pos = 0; $i < $num; $i++, $pos += 8)
   {  $tmp = substr($str, $pos, 8);
      $val = hex($tmp);
      $data[$i] = $val;
   }
   return(\@data);
}


#-------------------------------------------------------------------------------

sub vec_create
{  my @data = ();
   return(\@data) if ( $#_ != 0 );
   my ($num, $i);
   $num  = vec_compute_nlong($_[0]);
   for ($i = 0; $i < $num; $i++)
   {  $data[$i] = 0;
   }
   return(\@data);
}


#-------------------------------------------------------------------------------
