#
#/**************************************************************************
# *                                                                        *
# *               Copyright (C) 1989-1999 Silicon Graphics, Inc.           *
# *                                                                        *
# *  These coded instructions, statements, and computer programs  contain  *
# *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
# *  are protected by Federal copyright law.  They  may  not be disclosed  *
# *  to  third  parties  or copied or duplicated in any form, in whole or  *
# *  in part, without the prior written consent of Silicon Graphics, Inc.  *
# *                                                                        *
# **************************************************************************/
#
# @(#)$Id: wincfg.pm,v 1.9 2004/02/10 23:12:51 beighley Exp $

use Tk::FBox;

#-------------------------------------------------------------------------------
#  CONSTANTS - THESE SHOULD NOT CHANGE
#-------------------------------------------------------------------------------

$FALSE = 0;
$TRUE  = 1;

$WIN_PARM_DISPLAY = 0x01;
$WIN_PARM_SELECT  = 0x02;

$WINCFG_IND_NAME = 0;
$WINCFG_IND_FMT  = 1;
$WINCFG_IND_PARM = 2;

#-------------------------------------------------------------------------------
#  GLOBAL VARIABLES
#-------------------------------------------------------------------------------

$ssw_directory = '/stand/sysco/usr/data';

#-------------------------------------------------------------------------------

sub window_cfg_save
{
   print "\nwindow_cfg_save( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $win_top   = $_[0];  # Top-level window for file dialog
   my $win_type  = $_[1];  # Type of window (REG_ACCESS, TDR_ACCESS, etc.)
   my $win_title = $_[2];  # Title of window (user description)
   my $asic_name = $_[3];  # ASIC name: <brick>:<board>:<chip>
   my $asic_type = $_[4];  # ASIC type/decription (i.e. "shub_01")
   my ($cmt, $rad_list, $dpd_list, $i);

   #  Check the window configuration type
   if ( $win_type eq 'REG_ACCESS' )
   {  $rad_list = $_[5];   # Array of RAD references for the window
      $cmt = "Window containing MMRs";
   }
   elsif ( $win_type eq 'DEBUG_ACCESS' )
   {  $dpd_list = $_[5];   # Array of DPD references for the debug port(s)
      $cmt = "Window containing debug port configuration";
   }
   elsif ( $win_type eq 'LA_CHNCFG_WINDOW' )
   {  $chnmap = $_[5];   # Reference to array of LA channel names
      $cmt = "Window containing LA channel configuration details";
   }
   else
   {  return;
   }


   #
   #  Prompt the user for the file name
   #
#   my @types = ( ['SuperScope Window Files', '.ssw'],
#                 ['All Files',               '*' ] );
   my $out_file = $win_top->getSaveFile(
#                  -title => 'Save SuperScope Window'
#                  -filetypes => \@types,
#                  -initialdir => $ssw_directory,
                   );
   return if ( !defined($out_file) );

   #
   #  Open the output file for writing
   #
   if ( !open(WIN_OUT_FILE, ">$out_file") )
   {  $emess = "Unable to open output file for writing.";
      $emess .= "\nFile: $out_file";
      superscope_error_handler($emess);
      return; 
   }

   if ( $win_title eq "" )
   {  $win_title = "\"\"";
   }
   if ( $asic_name eq "" )
   {  $asic_name = "\"\"";
   }
   if ( $asic_type eq "" )
   {  $asic_type = "\"\"";
   }

   #  Write the file header
   printf WIN_OUT_FILE "#\n";
   printf WIN_OUT_FILE "# The file contains window configuration information for\n";
   printf WIN_OUT_FILE "#    the SuperScope debug environemnt.\n";
   printf WIN_OUT_FILE "#\n\n";

   #  Write the window details
   printf WIN_OUT_FILE "START_WINDOW\n";
   printf WIN_OUT_FILE "TYPE=$win_type\n";
   printf WIN_OUT_FILE "TITLE=$win_title\n";
   printf WIN_OUT_FILE "COMMENT=$cmt\n";
   printf WIN_OUT_FILE "TARGET=$asic_name\n";
   printf WIN_OUT_FILE "ASIC=$asic_type\n";

   #
   #  Register (MMR) access window: entry for each register in the window
   #
   if ( $win_type eq 'REG_ACCESS' )
   {  my ($rad_ref, $parm, $ewd_ptr, $fmt);

      foreach $rad_ref ( @$rad_list )  # process each register in list
      {  $parm = 0;
         $parm |= $WIN_PARM_DISPLAY if ( $$rad_ref[$RAD_VAL_DISPLAY] );
         $parm |= $WIN_PARM_SELECT if ( $$rad_ref[$RAD_VAL_SELECT] );
         $ewd_ptr = $$rad_ref[$RAD_EWD_REF]; # pointer to EWD array
         $fmt = $$ewd_ptr[$EWD_TEXT_MMR];
         $fmt =~ s/\s/_/g;
         printf WIN_OUT_FILE "REG=%-24s %-40s 0x%04lX\n", 
            reg_name($$rad_ref[$RAD_REG_REF]), $fmt, $parm;
      }
   }
   #
   #  Debug port access window: entry for each debug port and control register
   #
   elsif ( $win_type eq 'DEBUG_ACCESS' )
   {  my ($dpd_ref, $reg_ref, $val, $name, $parm, %cregs);

      foreach $dpd_ref ( @$dpd_list )  # process each debug port in list
      {  if ( $$dpd_ref[$DPD_FLAGS] & $DPD_FLAG_MMR )
         {  $reg_ref = $$dpd_ref[$DPD_DATA_REG];
            $val     = reg_get_value($reg_ref);
         }
         else
         {  $val = $$dpd_ref[$DPD_DATA_REG];
         }
         $name = $asic_name . ":" . $$dpd_ref[$DPD_NAME];
         $parm = 0;
         $parm |= $WIN_PARM_DISPLAY if ( exists $dwd_lookup{$name} );
         printf WIN_OUT_FILE "PORT=%-36s 0x%016llX 0x%04lX\n",
            $$dpd_ref[$DPD_NAME], $val, $parm;
      }

      %cregs = ();
      foreach $dpd_ref ( @$dpd_list )  # process each control register
      {  $reg_ref = $$dpd_ref[$DPD_CTRL_REG];
         $name    = reg_name($reg_ref);
         if ( !exists($cregs{$name}) )
         {  $cregs{$name} = $TRUE;
            $val  = reg_get_value($reg_ref);
            $parm = 0;
            printf WIN_OUT_FILE "CTRL=%-36s 0x%016llX 0x%04lX\n",
               $name, $val, $parm;
         }
      }
   }
   #
   #  LA channel configuration: mapping for each debug port pin
   #
   elsif ( $win_type eq 'LA_CHNCFG_WINDOW' )
   {
      for ($i = 0; $i <= $#$chnmap; $i++)
      {
         printf WIN_OUT_FILE "DEBUG_%03u=%s\n", $i, $$chnmap[$i];
      }

   }

   printf WIN_OUT_FILE "END_WINDOW\n";
   close(WIN_OUT_FILE);

   print "\nSaved window configuration: $out_file\n";
}

#-------------------------------------------------------------------------------

sub window_cfg_load
{
   print "\nwindow_cfg_load( @_ ) : directory=$ssw_directory\n" if ($debug_flag & $DEBUG_FUNCTIONS);
   my $win_top   = $_[0];  # Top-level window for file dialog

   #
   #  Prompt the user for the file name
   #
#   my @types = ( ['SuperScope Window Files', '.ssw'],
#                 ['All Files',               '*' ] );
   my $in_file = $win_top->getOpenFile(
#                  -title => 'Load SuperScope Window',
#                  -filetypes => \@types,
#                  -initialdir => $ssw_directory
                   );
   return if ( !defined($in_file) );

   #
   #  Read the window configuration file
   #
   my $win_data = window_cfg_read_file($in_file);
   return if ( !(defined $win_data) );

   wincfg_dump($win_data) if ($debug_flag & $DEBUG_DETAILS);


   #
   #  Process the window configuration data (create structures/windows)
   #
   if ( $$win_data{TYPE} eq 'REG_ACCESS' )
   {
      window_cfg_process_reg_access($win_data);
   }
   elsif ( $$win_data{TYPE} eq 'DEBUG_ACCESS' )
   {
      window_cfg_process_debug_access($win_data);
   }
   elsif ( $$win_data{TYPE} eq 'LA_CHNCFG_WINDOW' )
   {  my ($chnmap, $aref, $i);
      $chnmap = $_[1]; # Reference to array with channel names
      $aref   = $$win_data{PORTS};
      for ($i = 0; $i <= $#$aref; $i++)
      {  $$chnmap[$i] = $$aref[$i];
      }
   }

   print "\nLoaded window configuration: $in_file\n";
}


#-------------------------------------------------------------------------------

sub window_cfg_process_reg_access
{
   printf "\nwindow_cfg_process_reg_access( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return if ( $#_ != 0 );
   my $win_data = $_[0]; # Reference to window configuration data

   my ($i, $reg_cfg, $target_chip, $win_title, $full_name, $rad_ref, 
       $parm, $reg_ptr, $ewd_array, $ewd_ptr);

   #
   #  Process each register defined in the window configuration data
   #
   my $reg_list = $$win_data{REGS};
   my @reg_inds = ();
   foreach $reg_cfg ( @$reg_list )
   {
      $target_chip = $$win_data{TARGET};  # get from user!
      $win_title   = $$win_data{TITLE};
      $full_name   = $target_chip . ":" . $$reg_cfg[$WINCFG_IND_NAME];

      #  Create or retrieve the register instance structure
      $reg_ptr = reg_lookup_create_inst($full_name, $$win_data{ASIC}, 
                    $$reg_cfg[$WINCFG_IND_NAME]);
      if ( !reg_is_valid($reg_ptr) )
      {  $emess  = "Unable to find specified register in MMR modules.";
         $emess .= "\nRegister: $full_name";
         superscope_error_handler($emess);
         return;
      }

      #  Create or retrieve the entry widget data (EWD) structure
      $ewd_array = ewd_create_reg_inst($full_name, $reg_ptr, $num_base);
      $ewd_ptr = $$ewd_array[1];

      #  Overwrite the entry widget text values (if the user wants!)
      $$ewd_ptr[$EWD_TEXT_MMR] = $$reg_cfg[$WINCFG_IND_FMT];
      ra_cbf_entry_reg(0, $reg_ptr, $ewd_ptr, $TRUE, $target_chip);

      #  Add the name to the list of "selected" registers
      push(@reg_inds, reg_find_sorted_index($$win_data{ASIC}, $$reg_cfg[$WINCFG_IND_NAME]));
   }

   #
   #  Create the register access window
   #
   my @chip_parms = ( $target_chip, $$win_data{ASIC}, $win_main );
   my $win_name   = "REG_WIN_$rad_win_count";
   my $rv = create_reg_access_window(\@chip_parms, $win_title, \@reg_inds);
   if ( $rv == $FALSE )
   {  $emess  = "Unable to create register access window.";
      $emess .= "\nProgram terminating due to error condition.";
      superscope_error_handler($emess);
      return;
   }

   #
   #  Change the 'select' and 'show fields' checkboxes on new window
   #
   my $rad_list = $rad_lookup{$win_name};  # window's list of RAD references
   for ($i = 0; $i <= $#$rad_list; $i++)
   {  $rad_ref = $$rad_list[$i];   # reference to RAD structure
      $reg_cfg = $$reg_list[$i];   # reference to register details from file
      $parm    = $$reg_cfg[$WINCFG_IND_PARM];
      $$rad_ref[$RAD_VAL_SELECT]  = ( ($parm & $WIN_PARM_SELECT) ? 1 : 0 );
      $$rad_ref[$RAD_VAL_DISPLAY] = ( ($parm & $WIN_PARM_DISPLAY) ? 1 : 0 );
      ra_cbf_dsp_checkbtn($rad_ref);
   }
}


#-------------------------------------------------------------------------------

sub window_cfg_process_debug_access
{
   printf "\nwindow_cfg_process_debug_access( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return if ( $#_ != 0 );
   my $win_data = $_[0]; # Reference to window configuration data

#wincfg_dump($win_data);

   my ($chip_name, $chip_type, $dpd_list, $dpd_ref, @parms, $mref, $name,
       $text, $rv, $state, $chip_ptr, $aref, $port_cfg, $ctrl_cfg, $val, $reg_ptr);

   #
   #  Extract the chip name and type from the input data
   #
   $chip_name = $$win_data{TARGET}; # Full name of ASIC: <brick>:<board>:<chip>
   $chip_type = $$win_data{ASIC};   # ASIC type/description (i.e. "shub_01");

   #
   #  Determine if debug window already open for this chip
   # 
   $state = $cmenu_dbg->cget(-state);
   if ( ($state eq 'disabled') && # Debug window open already
        ($debug_chip_name ne $chip_name) )
   {  $text = "Close debug port access window(s) for $debug_chip_name and open for $chip_name?";
      $rv = $win_main->messageBox(-icon => 'question', -type => 'YesNo', -message => $text,
              -title => 'Load Debug Configuration File');
      if ( $rv eq 'Yes' )
      {  $debug_win->destroy();
         $state = 'normal';
      }
      else # ( $rv eq 'No' )
      {  $text = "Override debug port settings for $debug_chip_name from file?";
         $rv = $win_main->messageBox(-icon => 'question', -type => 'YesNo', -message => $text,
                 -title => 'Load Debug Configuration File');
         return if ( $rv ne 'Yes' );
         $chip_name = $debug_chip_name;
      }
   }

   #
   #  Find module reference and chip definition in configuration data
   #
   @parms = split(":", $chip_name); # Extract brick, board and chip names
   if ( !(exists $modules{$parms[0]})  )
   {  $emess  = "Specified module does not exist in the current configuration.";
      $emess .= "\nModule: $parms[0]\n";
      superscope_error_handler($emess);
      return;
   }
   $mref = $modules{$parms[0]};  # Module structure reference
   $chip_ptr = scan_module_chip_ptr($mref, $parms[1], $parms[2], 'logical');
   if ( $chip_type ne $$chip_ptr[$CMP_IND_DESC] )
   {  $emess = "Unexpected chip type from input file for $$mref{NAME}:$parms[1]:$parms[2]";
      $emess .= "\nTarget chip type: $$chip_ptr[$CMP_IND_DESC]";
      $emess .= "\nType from file: $chip_type";
      superscope_error_handler($emess);
      return;
   }

   #
   #  Create the debug port data structure(s)
   #
   $dpd_list = dpd_list_create($chip_name, $chip_type);
   if ( !defined($dpd_list) )
   {  superscope_error_handler($emess);
      return;
   }

   #
   #  Store the debug port values (from the input file) in the structure(s)
   #
   $aref = $$win_data{PORTS};
#printf "DEBUG PORTS  = ( @$aref )\n";
   foreach $port_cfg ( @$aref )
   {  
#print "\nPORT = $port_cfg = ( @$port_cfg )\n";

      $dpd_ref = dpd_list_search($chip_name, $$port_cfg[$WINCFG_IND_NAME], 'debug');
      if ( !defined($dpd_ref) )
      {  $emess  = "Unable to find specified debug port in chip definition.";
         $emess .= "\nDebug port: $$port_cfg[$WINCFG_IND_NAME]";
         superscope_error_handler($emess);
         return;
      }

      #  Update the value (where depends if MMR exists or not)
      $val = convert_text2num($$port_cfg[$WINCFG_IND_FMT], 64);
      if ( $$dpd_ref[$DPD_FLAGS] & $DPD_FLAG_MMR )
      {  $reg_ptr = $$dpd_ref[$DPD_DATA_REG];
         reg_set_value($reg_ptr, $val);
         regaccess_update_reg($chip_name, $reg_ptr);
      }
      else
      {  $$dpd_ref[$DPD_DATA_REG] = $val;
      } 
   }

   #
   #  Store the control register values (from the input file) in the structure(s)
   #
   $aref = $$win_data{CTRLS};
#printf "CONTROL REGS = ( @$aref )\n";
   foreach $ctrl_cfg ( @$aref )
   {  
#print "\nCTRL = $ctrl_cfg = ( @$ctrl_cfg )\n";

      $dpd_ref = dpd_list_search($chip_name, $$ctrl_cfg[$WINCFG_IND_NAME], 'control');
      if ( !defined($dpd_ref) )
      {  $emess  = "Unable to find specified debug control MMR in chip definition.";
         $emess .= "\nDebug control MMR: $$ctrl_cfg[$WINCFG_IND_NAME]";
         superscope_error_handler($emess);
         return;
      }

      #  Update the control value and propagate it
      $reg_ptr = $$dpd_ref[$DPD_CTRL_REG];
      $val     = convert_text2num($$ctrl_cfg[$WINCFG_IND_FMT], 64);
      reg_set_value($reg_ptr, $val);
      debug_access_update_ctrl_value($chip_name, $reg_ptr, $TRUE);
   }


   #
   #  Update existing debug port access window(s)
   #
   if ( $state eq 'disabled' )
   {  foreach $dpd_ref ( @$dpd_list )
      {  if ( $$dpd_ref[$DPD_FLAGS] & $DPD_FLAG_MMR ) # Debug port has MMR
         {  $reg_ptr = $$dpd_ref[$DPD_DATA_REG];
            $val = reg_get_value($reg_ptr);
         }
         else
         {  $val = $$dpd_ref[$DPD_DATA_REG];
         }
         $name = $chip_name . ":" . $$dpd_ref[$DPD_NAME];
         if ( exists $dwd_lookup{$name} )      # Window currently displayed?
         {  $dwd_ref   = $dwd_lookup{$name};
            debug_access_update_data_value($chip_name, $dpd_ref, $dwd_ref, $val, $TRUE);
         }
      }
   }
   #
   #  Create new debug port access window(s)
   #
   else # ( $state eq 'normal' )
   {
      cbf_create_debug_win($mref, $chip_name, 0, $cmenu_dbg);
# Open debug port windows based on flags from input file????
   }

}


#-------------------------------------------------------------------------------

sub window_cfg_read_file
{  my %win_data = ();
   print "\nwindow_cfg_read_file( @_ )\n" if ($debug_flag & $DEBUG_FUNCTIONS);

   return if ( $#_ != 0 );
   my $inp_file = $_[0]; # Input file name

   #
   #  Open the window configuration file
   #
   if ( !open(WIN_IN_FILE, "$inp_file") )
   {  $emess  = "Unable to open input file for reading.";
      $emess .= "\nFile: $inp_file";
      superscope_error_handler($emess);
      return;
   }
   my $ilnum = 0;    # Initialize the input file line counter
   my ($rv, @labels, $i, $done, $text);

   #
   #  Read the window defintion line
   #
   return if ( ($rv = wincfg_read_line(\$ilnum)) eq "" );
   if ( $rv ne "START_WINDOW" )
   {  close(WIN_IN_FILE);
      $emess  = "Invalid format; expecting \"START_WINDOW\".";
      $emess .= "\nFile: $inp_file";
      $emess .= "\nLine: $ilnum";
      superscope_error_handler($emess);
      return;
   }

   #
   #  Read the detailed definitions
   #
   @labels = ( 'TYPE', 'TITLE', 'COMMENT', 'TARGET', 'ASIC' );
   foreach $label ( @labels )
   {
      if ( ($rv = wincfg_read_line(\$ilnum)) eq "" )
      {  close(WIN_IN_FILE);
         $emess  = "Unexpected end-of-line encountered in input file.";
         $emess .= "\nExpecting data in the format: \"$label=<data>\".";
         $emess .= "\nFile: $inp_file";
         $emess .= "\nLine: $ilnum";
         superscope_error_handler($emess);
         return;
      }

      if ( $rv =~ /^$label\s*=\s*/ )
      {  $rv =~ s/^$label\s*=\s*//;
         if ( ($rv eq "") && ($label ne 'COMMENT') && ($label ne 'TITLE') )
         {  close(WIN_IN_FILE);
            $emess  = "Invalid format; expecting \"$label=<value>\".";
            $emess .= "\nFile: $inp_file";
            $emess .= "\nLine: $ilnum";
            superscope_error_handler($emess);
            return;
         }
      }
      else
      {  close(WIN_IN_FILE);
         $emess = "Invalid format; expecting \"$label=<value>\".";
         $emess .= "\nFile: $inp_file";
         $emess .= "\nLine: $ilnum";
         superscope_error_handler($emess);
         return;
      }
      $win_data{$label} = $rv;
   } 

   #
   #  Register access window: read each register entry
   #
   if ( $win_data{TYPE} eq 'REG_ACCESS' )
   {
      my @reg_data = ();
      $win_data{REGS} = \@reg_data;

      $i    = 0;
      $done = $FALSE;
      do
      {  if ( ($rv = wincfg_read_line(\$ilnum)) eq "" )
         {  close(WIN_IN_FILE);
            $emess  = "Unexpected end-of-line encountered in input file.";
            $emess .= "\nFile: $inp_file";
            $emess .= "\nLine: $ilnum";
            superscope_error_handler($emess);
            return;
         }

         if ( $rv =~ /^REG=/ )
         {  $rv =~ s/^REG=//;
            my @parms = split(" ", $rv);
            if ( $#parms != 2 )
            {  close(WIN_IN_FILE);
               $emess  = "Invalid register definition format in input line.";
               $emess .= "\nExpecting \"REG=<name> <format string> <flag>\".";
               $emess .= "\nFile: $inp_file";
               $emess .= "\nLine: $ilnum";
               superscope_error_handler($emess);
               return;
            }
            $parms[$WINCFG_IND_FMT] =~ s/_/ /g;
            $parms[$WINCFG_IND_PARM] = hex($parms[$WINCFG_IND_PARM]);
            $reg_data[$i] = \@parms;
            $i++;
         }
         elsif ( $rv =~ /^END_WINDOW/ )
         {  $done = $TRUE;
         }
         else
         {  close(WIN_IN_FILE);
            $emess  = "Invalid format; expecting \"REG=\" or \"END_WINDOW\".";
            $emess .= "\nFile: $inp_file";
            $emess .= "\nLine: $ilnum";
            superscope_error_handler($emess);
            return;
         }
      }
      while ( $done == $FALSE );

      close(WIN_IN_FILE);

      #
      #  Confirm that register were loaded
      #
      if ( $i == 0 )
      {  $emess  = "No registers were defined in the window configuration file.";
         $emess .= "\nFile: $inp_file";
         superscope_error_handler($emess);
         return;
      }
   }
   #
   #  Debug port access window: read debug port and control register entries
   #
   elsif ( $win_data{TYPE} eq 'DEBUG_ACCESS' )
   {
      my @port_data = ();
      $win_data{PORTS} = \@port_data;
      my @ctrl_data = ();
      $win_data{CTRLS} = \@ctrl_data;

      $port_i = 0;
      $ctrl_i = 0;
      $done = $FALSE;
      do
      {  if ( ($rv = wincfg_read_line(\$ilnum)) eq "" )
         {  close(WIN_IN_FILE);
            $emess  = "Unexpected end-of-line encountered in input file.";
            $emess .= "\nFile: $inp_file";
            $emess .= "\nLine: $ilnum";
            superscope_error_handler($emess);
            return;
         }

         if ( $rv =~ /^PORT=/ )
         {  $rv =~ s/^PORT=//;
            my @parms = split(" ", $rv);
            if ( $#parms != 2 )
            {  close(WIN_IN_FILE);
               $emess  = "Invalid debug port definition format in input line.";
               $emess .= "\nExpecting \"PORT=<name> <format string> <flag>\".";
               $emess .= "\nFile: $inp_file";
               $emess .= "\nLine: $ilnum";
               superscope_error_handler($emess);
               return;
            }
            $parms[$WINCFG_IND_FMT] =~ s/_/ /g;
            $parms[$WINCFG_IND_PARM] = hex($parms[$WINCFG_IND_PARM]);
            $port_data[$port_i] = \@parms;
            $port_i++;
         }
         elsif ( $rv =~ /^CTRL=/ )
         {  $rv =~ s/^CTRL=//;
            my @parms = split(" ", $rv);
            if ( $#parms != 2 )
            {  close(WIN_IN_FILE);
               $emess  = "Invalid control register definition format in input line.";
               $emess .= "\nExpecting \"CTRL=<name> <format string> <flag>\".";
               $emess .= "\nFile: $inp_file";
               $emess .= "\nLine: $ilnum";
               superscope_error_handler($emess);
               return;
            }
            $parms[$WINCFG_IND_FMT] =~ s/_/ /g;
            $parms[$WINCFG_IND_PARM] = hex($parms[$WINCFG_IND_PARM]);
            $ctrl_data[$ctrl_i] = \@parms;
            $ctrl_i++;
         }
         elsif ( $rv =~ /^END_WINDOW/ )
         {  $done = $TRUE;
         }
         else
         {  close(WIN_IN_FILE);
            $emess  = "Invalid format; expecting \"PORT=\" or \"CTRL=\" or \"END_WINDOW\".";
            $emess .= "\nFile: $inp_file";
            $emess .= "\nLine: $ilnum";
            superscope_error_handler($emess);
            return;
         }
      }
      while ( $done == $FALSE );

      close(WIN_IN_FILE);

      if ( ($port_i == 0) && ($ctrl_i == 0) )
      {  $emess  = "No registers were defined in the window configuration file.";
         $emess .= "\nFile: $inp_file";
         superscope_error_handler($emess);
         return;
      }
   }
   #
   #  LA channel configuration window: debug port mapping to LA channel probe
   #
   elsif ( $win_data{TYPE} eq 'LA_CHNCFG_WINDOW' )
   {
      my @portdata = ();
      $win_data{PORTS} = \@portdata;

      $i    = 0;
      $done = $FALSE;
      do
      {  if ( ($rv = wincfg_read_line(\$ilnum)) eq "" )
         {  close(WIN_IN_FILE);
            $emess  = "Unexpected end-of-line encountered in input file.";
            $emess .= "\nFile: $inp_file";
            $emess .= "\nLine: $ilnum";
            superscope_error_handler($emess);
            return;
         }

         $text = sprintf("DEBUG_%03u=", $i);
         if ( $rv =~ /^$text/ )
         {  $rv =~ s/^$text//;
            if ( $rv eq "" )
            {  close(WIN_IN_FILE);
               $emess  = "Invalid channel map definition format in input line.";
               $emess .= "\nExpecting \"$text<name>\".";
               $emess .= "\nFile: $inp_file";
               $emess .= "\nLine: $ilnum";
               superscope_error_handler($emess);
               return;
            }
            $portdata[$i] = $rv;
            $i++;
         }
         elsif ( $rv =~ /^END_WINDOW/ )
         {  $done = $TRUE;
         }
         else
         {  close(WIN_IN_FILE);
            $emess  = "Invalid format; expecting \"$text\" or \"END_WINDOW\".";
            $emess .= "\nFile: $inp_file";
            $emess .= "\nLine: $ilnum";
            superscope_error_handler($emess);
            return;
         }
      }
      while ( $done == $FALSE );

      close(WIN_IN_FILE);
   }

   return(\%win_data);
}


#-------------------------------------------------------------------------------

sub wincfg_dump
{  
   printf "\nwincfg_dump( @_ )\n";
   return if ( $#_ != 0 );
   my $win_ref = $_[0];
   my ($key, $aref, $rref, $i);

   print "\nWINDOW CONFIGURATION STRUCTURE CONTENTS:\n";
   foreach $key (keys %$win_ref)
   {  printf "%-12s = %s\n", $key, $$win_ref{$key};
   }

   if ( $$win_ref{TYPE} eq 'REG_ACCESS' )
   {  $aref = $$win_ref{REGS};
      printf "REGISTERS    = ( @$aref )\n";
      foreach $rref ( @$aref )
      {  print "$rref = ( @$rref )\n";
      }
   }
   elsif ( $$win_ref{TYPE} eq 'DEBUG_ACCESS' )
   {  $aref = $$win_ref{PORTS};
      printf "DEBUG PORTS  = ( @$aref )\n";
      foreach $rref ( @$aref )
      {  print "$rref = ( @$rref )\n";
      }
      $aref = $$win_ref{CTRLS};
      printf "CONTROL REGS = ( @$aref )\n";
      foreach $rref ( @$aref )
      {  print "$rref = ( @$rref )\n";
      }
   }
   elsif ( $$win_ref{TYPE} eq 'LA_CHNCFG_WINDOW' )
   {  $aref = $$win_ref{PORTS};
      printf "LA CHANNEL MAPPING TO DEBUG PORT:\n";
      for ($i = 0; $i <= $#$aref; $i++)
      {  printf "DATA[%02u] = %s\n", $i, $$aref[$i];
      }
   }
}

#-------------------------------------------------------------------------------

sub wincfg_read_line
{
#   print "\nwincfg_read_line( @_ )\n";

   my $lnum = $_[0];  # Line number in file (update)
   my $line = "";

   while (<WIN_IN_FILE>)
   {
      $$lnum = $$lnum + 1;

      if ( !/^#/ )
      {
         if ( length() > 1 )
         {
            chomp();
#printf "LINE %5u = \"$_\"\n", $$lnum;
            return( $_ );
         }
      }
   }

   return("");
}


#-------------------------------------------------------------------------------

1;
